@@ -278,7 +278,6 @@ void cla_layconmodifyarcinst(ARCINST *ai, INTBIG oldx0, INTBIG oldy0, INTBIG old
 	INTBIG oldy1, INTBIG oldwid, INTBIG oldlen) {}
 void cla_layconmodifyportproto(PORTPROTO *pp, NODEINST *oni, PORTPROTO *opp) {}
 void cla_layconmodifynodeproto(NODEPROTO *np) {}
-void cla_layconmodifydescript(VARIABLE *var, INTBIG olddes) {}
 void cla_layconnewlib(LIBRARY *lib) {}
 void cla_layconkilllib(LIBRARY *lib) {}
 void cla_layconnewvariable(INTBIG addr, INTBIG type, INTBIG skey, INTBIG stype) {}
@@ -290,6 +289,7 @@ void cla_layconinsertvariable(INTBIG addr, INTBIG type, INTBIG key, INTBIG ainde
 void cla_laycondeletevariable(INTBIG addr, INTBIG type, INTBIG key, INTBIG aindex,
 	INTBIG oldval) {}
 void cla_layconsetvariable(void) {}
+void cla_layconmodifydescript(INTBIG addr, INTBIG type, VARIABLE *var, INTBIG olddes) {}
 
 /******************** NODE MODIFICATION CODE *************************/
 
@@ -744,7 +744,9 @@ INTSML cla_modflex(NODEINST *ni, INTSML dangle, INTSML dtrans)
 
 		/* see if other nodeinst has changed */
 		mangle = 1;
+#if 0
 		if (ono->changed >= cla_changeclock) mangle = 0;
+#endif
 		if ((ai->userbits&FIXANG) == 0) mangle = 0;
 		if (ono->proto->primindex == 0)
 		{
@@ -779,6 +781,10 @@ INTSML cla_modflex(NODEINST *ni, INTSML dangle, INTSML dtrans)
 					if (dx != odx && (ai->userbits&CANTSLIDE) == 0 &&
 						db_stillinport(ai, thatend, ax[thatend], ay[thatend]) != 0)
 							dx = odx = 0;
+
+					/* if other node already moved, don't move it any more */
+					if (ono->changed >= cla_changeclock) dx = odx = 0;
+
 #ifdef	DEBUG
 					if (cla_conlaydebug)
 						ttyputmsg("  Moving other end of arc to (%s,%s), other node by (%s,0)",
@@ -791,11 +797,6 @@ INTSML cla_modflex(NODEINST *ni, INTSML dangle, INTSML dtrans)
 					cla_domovearcinst(ai, ax[0],ay[0], ax[1],ay[1], 1);
 					if (dx != odx)
 						if (cla_modnodearcs(ono, 0, 0)) examinefacet++;
-#ifdef	DEBUG
-					if (cla_conlaydebug)
-						ttyputmsg("  Moved other end of arc to (%s,%s), other node by (%s,0)",
-							latoa(ax[thatend]), latoa(ay[thatend]), latoa(dx-odx));
-#endif
 					continue;
 				}
 			}
@@ -811,6 +812,10 @@ INTSML cla_modflex(NODEINST *ni, INTSML dangle, INTSML dtrans)
 				if (dy != ody && (ai->userbits&CANTSLIDE) == 0 &&
 					db_stillinport(ai, thatend, ax[thatend], ay[thatend]) != 0)
 						dy = ody = 0;
+
+				/* if other node already moved, don't move it any more */
+				if (ono->changed >= cla_changeclock) dx = odx = 0;
+
 #ifdef	DEBUG
 				if (cla_conlaydebug)
 					ttyputmsg("  Moving other end of arc to (%s,%s), other node by (0,%s)",
@@ -822,11 +827,6 @@ INTSML cla_modflex(NODEINST *ni, INTSML dangle, INTSML dtrans)
 				}
 				cla_domovearcinst(ai, ax[0],ay[0], ax[1],ay[1], 1);
 				if (dy != ody) if (cla_modnodearcs(ono, 0, 0)) examinefacet++;
-#ifdef	DEBUG
-				if (cla_conlaydebug)
-					ttyputmsg("  Moved other end of arc to (%s,%s), other node by (0,%s)",
-						latoa(ax[thatend]), latoa(ay[thatend]), latoa(dy-ody));
-#endif
 				continue;
 			}
 
@@ -840,6 +840,9 @@ INTSML cla_modflex(NODEINST *ni, INTSML dangle, INTSML dtrans)
 			/* change the arc */
 			cla_updatearc(ai, ax[0],ay[0], ax[1],ay[1], 1);
 
+			/* if other node already moved, don't move it any more */
+			if (ono->changed >= cla_changeclock) dx = dy = 0;
+
 			if (dx != 0 || dy != 0)
 			{
 				if (cla_modifynodeinst(ono, dx, dy, dx, dy, 0, 0, 1)) examinefacet++;
@@ -1334,6 +1337,7 @@ void cla_computefacet(NODEPROTO *facet, INTSML forcedlook)
 	REGISTER INTBIG dlx, dly, dhx, dhy, flx, fhx, fly, fhy;
 	REGISTER INTSML mixed;
 	REGISTER CHANGE *c;
+	REGISTER LIBRARY *lib;
 
 #ifdef	DEBUG
 	if (cla_conlaydebug)
@@ -1412,8 +1416,9 @@ void cla_computefacet(NODEPROTO *facet, INTSML forcedlook)
 	if (cla_conlaydebug)
 		ttyputmsg("   Performing complex tree examination");
 #endif
-	for(np = facet->cell->lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
-		np->userbits &= ~(FACETMOD|FACETNOMOD);
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+			np->userbits &= ~(FACETMOD|FACETNOMOD);
 	facet->userbits |= FACETMOD;
 	for(np = facet->cell->lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 	{
