@@ -32,7 +32,7 @@
 #include "conlin.h"
 
 /* command completion table for this constraint solver */
-static COMCOMP cli_linconsp = {NOKEYWORD, us_topoffacets, us_nextparse, NOPARAMS,
+static COMCOMP cli_linconsp = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
 	NOBACKUP, INPUTOPT, " \t", "facet to solve for linear inequalities", 0};
 static KEYWORD linconiopt[] =
 {
@@ -802,7 +802,7 @@ void cli_linconmodifyportproto(PORTPROTO *pp, NODEINST *oni, PORTPROTO *opp)
 }
 
 void cli_linconmodifynodeproto(NODEPROTO *np) {}
-void cli_linconmodifydescript(VARIABLE *var, INTBIG descript) {}
+void cli_linconmodifydescript(INTBIG addr, INTBIG type, VARIABLE *var, INTBIG descript) {}
 void cli_linconnewlib(LIBRARY *lib) {}
 void cli_linconkilllib(LIBRARY *lib) {}
 
