@@ -1486,14 +1486,16 @@ void cli_getfactors(NODEINST *ni, NODEPROTO *np, COMPONENT *compo, INTBIG *lx,
  */
 void cli_killthenode(NODEINST *ni)
 {
-	REGISTER PORTARCINST *pi;
+	REGISTER PORTARCINST *pi, *nextpi;
 	REGISTER ARCINST *ai;
 	REGISTER INTSML i;
 	REGISTER NODEINST *oni;
 
-	for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
+	for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = nextpi)
 	{
+		nextpi = pi->nextportarcinst;
 		ai = pi->conarcinst;
+		if ((ai->userbits&DEADA) != 0) continue;
 		cli_deleteequivcon(ai);
 		for(i=0; i<2; i++)
 		{
