@@ -137,6 +137,7 @@ CHANGE *db_change(INTBIG obj, INTSML change, INTBIG a1, INTBIG a2, INTBIG a3, IN
 	REGISTER ARCINST *ai;
 	REGISTER CHANGE *c;
 	REGISTER LIBRARY *lib;
+	REGISTER VARIABLE *var;
 	static char *broadcasttype[] = {"nodeinstnew", "nodeinstkill", "nodeinstmod",
 		"arcinstnew", "arcinstkill", "arcinstmod", "portprotonew",
 		"portprotokill", "portprotomod", "nodeprotonew", "nodeprotokill",
@@ -212,12 +213,20 @@ CHANGE *db_change(INTBIG obj, INTSML change, INTBIG a1, INTBIG a2, INTBIG a3, IN
 				lib = db_whichlibrary(obj, a1);
 			}
 			break;
+		case DESCRIPTMOD:
+			var = (VARIABLE *)a2;
+			if ((var->type&VDONTSAVE) != 0) { lib = NOLIBRARY;   np = NONODEPROTO; } else
+			{
+				np = db_whichnodeproto(obj, a1);
+				lib = db_whichlibrary(obj, a1);
+			}
+			break;
 	}
 
 	/* set "changed" and "dirty" bits */
 	if (np != NONODEPROTO)
 	{
-		np->revisiondate = getcurrenttime();
+		if (major != 0) np->revisiondate = getcurrenttime();
 		np->adirty = 0;
 		for(i=0; i<el_maxaid; i++)
 			if ((el_aids[i].aidstate & AIDON) == 0) np->adirty |= 1 << i;
@@ -399,7 +408,7 @@ void db_broadcastchange(CHANGE *c, INTSML firstchange, INTSML undoredo)
 		case DESCRIPTMOD:
 			for(i=0; i<el_maxaid; i++)
 				if ((el_aids[i].aidstate & AIDON) != 0 && el_aids[i].modifydescript != 0)
-					(*el_aids[i].modifydescript)((VARIABLE *)c->entryaddr, c->p1);
+					(*el_aids[i].modifydescript)(c->entryaddr, c->p1, (VARIABLE *)c->p2, c->p3);
 			break;
 	}
 	db_broadcasting = oldbroadcasting;
@@ -542,7 +551,7 @@ void db_erasebatch(CHANGEBATCH *batch)
 					{
 						e = w->editor;
 						if ((w->state&WINDOWTYPE) == POPTEXTWINDOW)
-							(void)us_restorebox(e->savedbox, -1);
+							(void)screenrestorebox(e->savedbox, -1);
 						us_freeeditor(e);
 					}
 					freewindowpart(w);
@@ -1008,6 +1017,7 @@ INTSML db_reversechange(CHANGE *c)
 				if (c->p1 == VPORTPROTO)
 				{
 					if (strcmp(attname, "textdescript") == 0 ||
+						strcmp(attname, "userbits") == 0 ||
 						strcmp(attname, "protoname") == 0) return(1);
 				}
 				break;
@@ -1120,9 +1130,9 @@ INTSML db_reversechange(CHANGE *c)
 			if (c->p1 == VNODEINST || c->p1 == VARCINST) return(1);
 			break;
 		case DESCRIPTMOD:
-			var = (VARIABLE *)c->entryaddr;
+			var = (VARIABLE *)c->p2;
 			if (var == NOVARIABLE) break;
-			oldval = var->textdescript;   var->textdescript = c->p1;   c->p1 = oldval;
+			oldval = var->textdescript;   var->textdescript = c->p3;   c->p3 = oldval;
 			return(1);
 	}
 	return(0);
@@ -1278,8 +1288,8 @@ void showhistorylist(INTSML which)
 					variable++;
 					break;
 				case DESCRIPTMOD:
-					if ((VARIABLE *)c->entryaddr != NOVARIABLE) variable++; else
-						if ((PORTPROTO *)c->p1 != NOPORTPROTO) portproto++; else
+					if ((VARIABLE *)c->p1 != NOVARIABLE) variable++; else
+						if ((PORTPROTO *)c->p3 != NOPORTPROTO) portproto++; else
 							node++;
 					break;
 		}
@@ -1561,7 +1571,7 @@ char *db_describechange(INTSML changetype, INTBIG entryaddr, INTBIG p1, INTBIG p
 			db_namevariable(entryaddr, p1);
 			return(returninfstr());
 		case DESCRIPTMOD:
-			var = (VARIABLE *)entryaddr;
+			var = (VARIABLE *)p2;
 			(void)initinfstr();
 			if (var != NOVARIABLE)
 			{
@@ -1572,25 +1582,6 @@ char *db_describechange(INTSML changetype, INTBIG entryaddr, INTBIG p1, INTBIG p
 				(void)addstringtoinfstr(line);
 				return(returninfstr());
 			}
-			pp = (PORTPROTO *)p1;
-			if (pp != NOPORTPROTO)
-			{
-				(void)addstringtoinfstr(" Text descriptor changed on port ");
-				(void)addstringtoinfstr(pp->protoname);
-				(void)addstringtoinfstr(" in facet ");
-				(void)addstringtoinfstr(describenodeproto(pp->parent));
-				sprintf(line, " [was 0%lo]", p3);
-				(void)addstringtoinfstr(line);
-				return(returninfstr());
-			}
-			ni = (NODEINST *)p2;
-			(void)addstringtoinfstr(" Text descriptor changed on node ");
-			(void)addstringtoinfstr(describenodeinst(ni));
-			(void)addstringtoinfstr(" in facet ");
-			(void)addstringtoinfstr(describenodeproto(ni->parent));
-			sprintf(line, " [was 0%lo]", p3);
-			(void)addstringtoinfstr(line);
-			return(returninfstr());
 	}
 	return("?");
 }
