@@ -33,7 +33,7 @@
 #include "tech.h"
 #include "edialogs.h"
 
-char *el_version = "6.00";		/* the current Electric version */
+char *el_version = "6.01";		/* the current Electric version */
 
 /* miscellaneous defaults */
 #define	DEFTECH     "nmos"        /* default technology */
@@ -45,7 +45,7 @@ void db_freeallmemory(void);
 /*
  * the primary initialization of Electric
  */
-void el_primaryinit(void)
+void osprimaryosinit(void)
 {
 	REGISTER INTSML i;
 	REGISTER TECHNOLOGY *tech, *lasttech, *realtech;
@@ -175,7 +175,7 @@ void el_primaryinit(void)
 /*
  * the secondary initialization of Electric
  */
-void el_secondaryinit(INTBIG argc, char *argv[])
+void ossecondaryinit(INTBIG argc, char *argv[])
 {
 	REGISTER INTSML i;
 	REGISTER TECHNOLOGY *tech;
@@ -220,7 +220,7 @@ void el_secondaryinit(INTBIG argc, char *argv[])
 }
 
 /* the body of the main loop of Electric */
-void el_slice(void)
+void tooltimeslice(void)
 {
 	REGISTER INTSML s;
 	REGISTER AIDENTRY *aid;
@@ -267,6 +267,7 @@ void aidturnon(AIDENTRY *aid, INTSML nocatchup)
 {
 	REGISTER INTBIG bit;
 	REGISTER NODEPROTO *np;
+	REGISTER LIBRARY *lib;
 
 	/* turn on the aid */
 	(void)setval((INTBIG)aid, VAID, "aidstate", aid->aidstate|AIDON, VINTEGER);
@@ -279,7 +280,8 @@ void aidturnon(AIDENTRY *aid, INTSML nocatchup)
 
 	/* look through all facets and update those that need it */
 	bit = 1 << aid->aidindex;
-	for(np=el_curlib->firstnodeproto; np!=NONODEPROTO; np=np->nextnodeproto)
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 	{
 		if (np->adirty & bit)
 		{
