@@ -864,7 +864,7 @@ void boundobj(GEOM *geom, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 	REGISTER INTBIG end0extend, end1extend, radius, angle,
 		xs, ys, xe, ye, xc, yc, xv, yv;
 	INTBIG x1, y1, x2, y2, num, xp, yp, plx, phx, ply, phy;
-	INTSML j, pts;
+	INTBIG j, pts;
 	double startoffset, endangle, ctx, cty, a, b, p;
 	XARRAY trans;
 	REGISTER ARCINST *ai;
@@ -953,55 +953,69 @@ void boundobj(GEOM *geom, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 
 		/* get the bounding box of the polygon as the arc extent */
 		getbbox(poly, lx, hx, ly, hy);
+
+#if 0
+		/* include any text on the arc */
+		pts = tech_displayableavars(ai);
+		for(j=0; j<pts; j++)
+		{
+			var = tech_filldisplayableavar(ai, poly);
+			getbbox(poly, &plx, &phx, &ply, &phy);
+			if (plx < *lx) *lx = plx;
+			if (phx > *hx) *hx = phx;
+			if (ply < *ly) *ly = ply;
+			if (phy > *hy) *hy = phy;
+		}
+#endif
 		return;
 	}
 	if (geom->entrytype == OBJNODEINST)
 	{
 		ni = geom->entryaddr.ni;
 
-		/* special case for arcs of circles: compute precise bounding box */
-		if (ni->proto == art_circleprim)
-		{
-			/* see if there this circle is only a partial one */
-			getarcdegrees(ni, &startoffset, &endangle);
-			if (startoffset != 0.0 || endangle != 0.0)
-			{
-				/* get center of ellipse */
-				ctx = (ni->lowx + ni->highx) / 2;
-				cty = (ni->lowy + ni->highy) / 2;
-
-				/* compute the length of the semi-major and semi-minor axes */
-				a = (ni->highx - ni->lowx) / 2;
-				b = (ni->highy - ni->lowy) / 2;
-
-				pts = (INTSML)(endangle * ELLIPSEPOINTS / (EPI * 2.0));
-				makerot(ni, trans);
-				for(j=0; j<pts; j++)
-				{
-					p = startoffset + j * endangle / (pts-1);
-					xv = rounddouble(ctx + a * cos(p));
-					yv = rounddouble(cty + b * sin(p));
-					xform(xv, yv, &xp, &yp, trans);
-					if (j == 0)
-					{
-						*lx = *hx = xp;
-						*ly = *hy = yp;
-					} else
-					{
-						if (xp < *lx) *lx = xp;
-						if (xp > *hx) *hx = xp;
-						if (yp < *ly) *ly = yp;
-						if (yp > *hy) *hy = yp;
-					}
-				}
-				return;
-			}
-		}
-
-		/* special cases for special primitives */
+		/* handle special cases primitives */
 		if (ni->proto->primindex != 0)
 		{
-			/* special case for wire pins: reduce geometry if not a steiner point */
+			/* special case for arcs of circles: compute precise bounding box */
+			if (ni->proto == art_circleprim)
+			{
+				/* see if there this circle is only a partial one */
+				getarcdegrees(ni, &startoffset, &endangle);
+				if (startoffset != 0.0 || endangle != 0.0)
+				{
+					/* get center of ellipse */
+					ctx = (ni->lowx + ni->highx) / 2;
+					cty = (ni->lowy + ni->highy) / 2;
+
+					/* compute the length of the semi-major and semi-minor axes */
+					a = (ni->highx - ni->lowx) / 2;
+					b = (ni->highy - ni->lowy) / 2;
+
+					pts = (INTSML)(endangle * ELLIPSEPOINTS / (EPI * 2.0));
+					makerot(ni, trans);
+					for(j=0; j<pts; j++)
+					{
+						p = startoffset + j * endangle / (pts-1);
+						xv = rounddouble(ctx + a * cos(p));
+						yv = rounddouble(cty + b * sin(p));
+						xform(xv, yv, &xp, &yp, trans);
+						if (j == 0)
+						{
+							*lx = *hx = xp;
+							*ly = *hy = yp;
+						} else
+						{
+							if (xp < *lx) *lx = xp;
+							if (xp > *hx) *hx = xp;
+							if (yp < *ly) *ly = yp;
+							if (yp > *hy) *hy = yp;
+						}
+					}
+					return;
+				}
+			}
+
+			/* special case for pins that become steiner points */
 			if ((ni->proto->userbits&WIPEON1OR2) != 0)
 			{
 				if (tech_pinusecount(ni, NOARCPROTO) != 0)
@@ -1022,7 +1036,7 @@ void boundobj(GEOM *geom, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 					num = nodepolys(ni);
 					for(j=0; j<num; j++)
 					{
-						shapenodepoly(ni, j, poly);
+						shapenodepoly(ni, (INTSML)j, poly);
 						xformpoly(poly, trans);
 						getbbox(poly, &plx, &phx, &ply, &phy);
 						if (j == 0)
@@ -1054,6 +1068,20 @@ void boundobj(GEOM *geom, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 			*ly = ni->lowy;   *hy = ni->highy;
 			xformbox(lx, hx, ly, hy, trans);
 		}
+
+#if 0
+		/* include any text on the node */
+		pts = tech_displayablenvars(ni);
+		for(j=0; j<pts; j++)
+		{
+			var = tech_filldisplayablenvar(ni, poly);
+			getbbox(poly, &plx, &phx, &ply, &phy);
+			if (plx < *lx) *lx = plx;
+			if (phx > *hx) *hx = phx;
+			if (ply < *ly) *ly = ply;
+			if (phy > *hy) *hy = phy;
+		}
+#endif
 	}
 }
 
@@ -1066,7 +1094,8 @@ void db_boundfacet(NODEPROTO *facet, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG
 {
 	REGISTER NODEINST *ni;
 	REGISTER ARCINST *ai;
-	REGISTER INTSML first;
+	REGISTER VARIABLE *var;
+	REGISTER INTSML first, i;
 
 	/* cannot compute bounds if facet has nothing in it */
 	*lx = *hx = *ly = *hy = 0;
@@ -1079,6 +1108,18 @@ void db_boundfacet(NODEPROTO *facet, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG
 		/* special case: do not include "facet center" primitive from Generic */
 		if (ni->proto == gen_facetcenterprim) continue;
 
+		/* special case for invisible pins: do not include if interior-only */
+		if (ni->proto == gen_invispinprim)
+		{
+			for(i=0; i<ni->numvar; i++)
+			{
+				var = &ni->firstvar[i];
+				if ((var->type&VDISPLAY) != 0 &&
+					(var->textdescript&VTINTERIOR) != 0) break;
+			}
+			if (i < ni->numvar) continue;
+		}
+
 		if (first != 0)
 		{
 			*lx = ni->geom->lowx;
@@ -1097,6 +1138,15 @@ void db_boundfacet(NODEPROTO *facet, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG
 	/* include all arcs in the facet */
 	for(ai = facet->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
 	{
+		if (first != 0)
+		{
+			*lx = ai->geom->lowx;
+			*hx = ai->geom->highx;
+			*ly = ai->geom->lowy;
+			*hy = ai->geom->highy;
+			first = 0;
+			continue;
+		}
 		*lx = mmini(*lx, ai->geom->lowx);
 		*hx = mmaxi(*hx, ai->geom->highx);
 		*ly = mmini(*ly, ai->geom->lowy);
