@@ -108,6 +108,10 @@ INTSML doquerry(char *code, INTBIG language, INTBIG type, INTBIG *retval)
 #if LANGLISP
 	ELKObject obj;
 #endif
+#if LANGJAVA
+	char *str;
+	INTBIG methodreturntype;
+#endif
 #if	LANGMM
 	char *str;
 	static INTBIG retarray[1];
@@ -149,6 +153,33 @@ INTSML doquerry(char *code, INTBIG language, INTBIG type, INTBIG *retval)
 			return(0);
 #endif
 
+#if LANGJAVA
+		case VJAVA:
+			/* evaluate the string */
+			java_init();
+			str = java_query(code, &methodreturntype);
+			switch (type&VTYPE)
+			{
+				case VSTRING:
+					*retval = (INTBIG)str;
+					break;
+
+				case VFLOAT:
+				case VDOUBLE:
+					*retval = castint((float)atof(str));
+					break;
+
+				case VFRACT:
+					*retval = myatoi(str) * WHOLE;
+					break;
+
+				default:
+					*retval = myatoi(str);
+					break;
+			}
+			return(0);
+#endif
+
 #if LANGMM
 		case VMATHEMATICA:
 			/* make sure Mathematica is initialized */
@@ -267,7 +298,7 @@ INTSML languageconverse(INTBIG language)
 	static INTBIG curlang;
 #if LANGTCL
 	static INTSML gotPartial;
-	char *cmd, *str, *promptCmd;
+	char *cmd, *tstr, *promptCmd;
 	REGISTER INTSML code;
 	static Tcl_DString command;
 	Tcl_Obj *objv[3];
@@ -276,7 +307,10 @@ INTSML languageconverse(INTBIG language)
 	ELKObject pred;
 #endif
 #if LANGMM
-	char *str;
+	char *mstr;
+#endif
+#if LANGJAVA
+	char *jstr;
 #endif
 
 	/* on the first call, initialize the interpreter */
@@ -285,19 +319,35 @@ INTSML languageconverse(INTBIG language)
 		curlang = language;
 		switch (curlang)
 		{
-#if LANGLISP
 			case VLISP:
+#if LANGLISP
 				ttyputmsg("ELK Lisp 3.0, type %s to quit", getmessageseofkey());
 				lsp_init();
-				break;
+#else
+				ttyputerr("LISP Interpreter is not installed");
+				return(1);
 #endif
+				break;
 
-#if LANGTCL
 			case VTCL:
+#if LANGTCL
 				ttyputmsg("TCL Interpreter, type %s to quit", getmessageseofkey());
 				Tcl_DStringInit(&command);
-				break;
+#else
+				ttyputerr("TCL Interpreter is not installed");
+				return(1);
 #endif
+				break;
+
+			case VJAVA:
+#if LANGJAVA
+				ttyputmsg("JAVA Interpreter, type %s to quit", getmessageseofkey());
+				java_init();
+#else
+				ttyputerr("JAVA Interpreter is not installed");
+				return(1);
+#endif
+				break;
 
 #if LANGMM
 			case VMATHEMATICA:
@@ -339,13 +389,13 @@ INTSML languageconverse(INTBIG language)
 
 			/* get input using queued output as prompt */
 			*tcl_outputloc = 0;
-			str = ttygetlinemessages(tcl_outputbuffer);
+			tstr = ttygetlinemessages(tcl_outputbuffer);
 			tcl_outputloc = tcl_outputbuffer;
-			if (str == 0) return(1);
+			if (tstr == 0) return(1);
 
 			/* evaluate and print result */
-			cmd = Tcl_DStringAppend(&command, str, -1);
-			if ((str[0] != 0) && !Tcl_CommandComplete(cmd))
+			cmd = Tcl_DStringAppend(&command, tstr, -1);
+			if ((tstr[0] != 0) && !Tcl_CommandComplete(cmd))
 			{
 				gotPartial = 1;
 				return(0);
@@ -363,14 +413,24 @@ INTSML languageconverse(INTBIG language)
 			return(0);
 #endif
 
+#if LANGJAVA
+		case VJAVA:
+			jstr = ttygetlinemessages("> ");
+			if (jstr == 0) return(1);
+
+			/* send the string to JAVA */
+			java_evaluate(jstr);
+			return(0);
+#endif
+
 #if LANGMM
 		case VMATHEMATICA:
-			str = ttygetlinemessages("> ");
-			if (str == 0) return(1);
+			mstr = ttygetlinemessages("> ");
+			if (mstr == 0) return(1);
 
 			/* send the string to Mathematica */
 			MLPutFunction(db_mathematicalink, "ToExpression", 1);
-				MLPutString(db_mathematicalink, str);
+				MLPutString(db_mathematicalink, mstr);
 			MLEndPacket(db_mathematicalink);
 
 			if (MLError(db_mathematicalink) != MLEOK)
@@ -400,4 +460,7 @@ void db_termlanguage(void)
 		MLClose(db_mathematicalink);
 	}
 #endif
+#if LANGJAVA
+	java_freememory();
+#endif
 }
