@@ -1580,16 +1580,16 @@ ELKObject lsp_getpinproto(ELKObject oap)
 	return(Make_EElectric((INTBIG)np, (INTSML)T_ENodeProto));
 }
 
-ELKObject lsp_getnetwork(ELKObject onp, ELKObject oname)
+ELKObject lsp_getnetwork(ELKObject oname, ELKObject onp)
 {
 	char *name;
 	REGISTER NETWORK *net;
 	REGISTER NODEPROTO *np;
 
 	/* get inputs from LISP */
-	Ensure_Type(onp, T_ENodeProto);   np = ENODEPROTO(onp)->handle;
 	name = lsp_getstringobject(oname);
 	if (name == 0) return(Null);
+	Ensure_Type(onp, T_ENodeProto);   np = ENODEPROTO(onp)->handle;
 
 	net = getnetwork(name, np);
 	if (net == NONETWORK) return(Null);
