@@ -330,6 +330,7 @@ void changelambda(INTBIG oldlam, INTBIG lam, TECHNOLOGY *tech)
 {
 	REGISTER INTSML i;
 	REGISTER NODEPROTO *np;
+	REGISTER LIBRARY *lib;
 
 	if (tech != NOTECHNOLOGY)
 	{
@@ -352,13 +353,17 @@ void changelambda(INTBIG oldlam, INTBIG lam, TECHNOLOGY *tech)
 	if (lam == oldlam) return;
 
 	/* mark all facets in the library for scaling */
-	for(np=el_curlib->firstnodeproto; np!=NONODEPROTO; np=np->nextnodeproto)
-		np->temp1 = 0;
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+			np->temp1 = 0;
 
-	/* scale all the facets in this library */
-	for(np=el_curlib->firstnodeproto; np!=NONODEPROTO; np=np->nextnodeproto)
-		db_scalefacet(np, lam, oldlam);
-	el_curlib->userbits |= LIBCHANGEDMAJOR;
+	/* scale all the facets */
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+	{
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+			db_scalefacet(np, lam, oldlam);
+		lib->userbits |= LIBCHANGEDMAJOR;
+	}
 }
 
 /*
