@@ -46,13 +46,12 @@
  * At end of facet, call:
  *    mrgdonefacet(writepolygon)
  * where "writepolygon" is a routine that will be called for every polygon with
- * six parameters:
+ * five parameters:
  *   (1) the layer number (INTSML)
  *   (2) the technology
  *   (3) the X coordinate array (INTBIG array)
  *   (4) the Y coordinate array (INTBIG array)
  *   (5) the number of coordinates in each array (INTSML)
- *   (6) the area of the polygon (float)
  *
  * When done with merging, call:
  *    mrgterm()
@@ -86,7 +85,7 @@ void db_mrg_remove_inclusion(BOXLISTHEAD*);
 void db_mrg_remove_overlap(BOXLISTHEAD*);
 BOXPOLY *db_mrg_breakbox(BOXPOLY*, BOXPOLY*, INTSML*);
 BOXPOLY *db_mrg_do_break(BOXPOLY*, BOXPOLY*, INTSML*);
-void db_mrg_write_lists(void(*)(INTSML, TECHNOLOGY*, INTBIG*, INTBIG*, INTSML, float));
+void db_mrg_write_lists(void(*)(INTSML, TECHNOLOGY*, INTBIG*, INTBIG*, INTSML));
 POLYCOORD *db_mrg_getpoly(void);
 INTSML db_mrg_mergeboxwithpolygon(BOXPOLY*);
 INTSML db_mrg_check_intersect(BOXPOLY*, POLYCOORD*, INTSML*);
@@ -95,7 +94,7 @@ void db_mrg_insert1(INTSML, BOXPOLY*);
 void db_mrg_insert2(INTSML, BOXPOLY*);
 void db_mrg_insert3(INTSML, BOXPOLY*);
 void db_mrg_insert4(INTSML, BOXPOLY*);
-void db_mrg_write_polygon(void(*)(INTSML, TECHNOLOGY*, INTBIG*, INTBIG*, INTSML, float), INTSML, TECHNOLOGY*, float);
+void db_mrg_write_polygon(void(*)(INTSML, TECHNOLOGY*, INTBIG*, INTBIG*, INTSML), INTSML, TECHNOLOGY*);
 INTSML db_mrg_pred(INTSML);
 INTSML db_mrg_succ(INTSML);
 INTSML db_mrg_nxtside(INTSML);
@@ -138,7 +137,7 @@ void mrgterm(void)
 	db_mrg_dump_free_boxhead_list();
 }
 
-void mrgdonefacet(void (*writepolygon)(INTSML, TECHNOLOGY*, INTBIG*, INTBIG*, INTSML, float))
+void mrgdonefacet(void (*writepolygon)(INTSML, TECHNOLOGY*, INTBIG*, INTBIG*, INTSML))
 {
 	db_mrg_write_lists(writepolygon);
 	db_mrg_remove_boxhead_list();
@@ -634,12 +633,11 @@ BOXPOLY *db_mrg_do_break(BOXPOLY *me, BOXPOLY *him, INTSML *rtc)
 	return(NULL);
 }
 
-void db_mrg_write_lists(void (*writepolygon)(INTSML, TECHNOLOGY*, INTBIG*, INTBIG*, INTSML, float))
+void db_mrg_write_lists(void (*writepolygon)(INTSML, TECHNOLOGY*, INTBIG*, INTBIG*, INTSML))
 {
 	BOXLISTHEAD *lay_box;
 	BOXPOLY *boxscanptr, *garbage, *oldboxscan;
 	INTSML lay_not_done, merged;
-	float area;         /* SRP920622 */
 
 	for(lay_box=db_mrg_curr_facet; lay_box!=NULL; lay_box=lay_box->nextlayer) /* for each layer... */
 	{
@@ -650,7 +648,6 @@ void db_mrg_write_lists(void (*writepolygon)(INTSML, TECHNOLOGY*, INTBIG*, INTBI
 /*		(*setlayer)(lay_box->layer, lay_box->tech); */
 		db_mrg_polycoordlist.firstpoly = NULL;
 		db_mrg_polycoordlist.numedge = 0;
-		area = 0.0;     /* was inside the while()   (SRP) */
 		while (lay_not_done)
 		{
 			db_mrg_polycoordlist.modified = 0;
@@ -664,10 +661,6 @@ void db_mrg_write_lists(void (*writepolygon)(INTSML, TECHNOLOGY*, INTBIG*, INTBI
 					db_mrg_polycoordlist.modified = 1;
 					merged = 0; /* reset indicator flag */
 
-					/* update area in (float) SRP920622 */
-					area += (float)(boxscanptr->top - boxscanptr->bot) *
-						(float)(boxscanptr->right - boxscanptr->left);
-
 					/* if box is first on list */
 					if (boxscanptr == lay_box->box)
 					{
@@ -710,11 +703,10 @@ void db_mrg_write_lists(void (*writepolygon)(INTSML, TECHNOLOGY*, INTBIG*, INTBI
 			if ((!db_mrg_polycoordlist.modified) || (!lay_not_done) ||
 				(db_mrg_polycoordlist.numedge > maxnumpts))
 			{
-				db_mrg_write_polygon(writepolygon, lay_box->layer, lay_box->tech, area);
+				db_mrg_write_polygon(writepolygon, lay_box->layer, lay_box->tech);
 				db_mrg_remove_poly_list(); /* dispose of polygon records */
 
 				/* re-initialize for next pass */
-				area = 0.0;     /* since the caller is accumulating areas (SRP) */
 				db_mrg_polycoordlist.firstpoly = NULL;
 				db_mrg_polycoordlist.numedge = 0;
 				db_mrg_polycoordlist.modified = 0;
@@ -1526,8 +1518,8 @@ void db_mrg_insert4(INTSML firstcon, BOXPOLY *boxptr)
 }
 
 /* write polygon */
-void db_mrg_write_polygon(void (*writepolygon)(INTSML, TECHNOLOGY*, INTBIG*, INTBIG*, INTSML, float),
-	INTSML layer, TECHNOLOGY *tech, float area)
+void db_mrg_write_polygon(void (*writepolygon)(INTSML, TECHNOLOGY*, INTBIG*, INTBIG*, INTSML),
+	INTSML layer, TECHNOLOGY *tech)
 {
 	POLYCOORD *polyptr;
 	INTSML i;
@@ -1552,7 +1544,7 @@ void db_mrg_write_polygon(void (*writepolygon)(INTSML, TECHNOLOGY*, INTBIG*, INT
 		db_mrgybuf[i] = polyptr->ed_start[1];
 		polyptr = polyptr->nextpoly; /* increment pointer */
 	}
-	(*writepolygon)(layer, tech, db_mrgxbuf, db_mrgybuf, db_mrg_polycoordlist.numedge, area);
+	(*writepolygon)(layer, tech, db_mrgxbuf, db_mrgybuf, db_mrg_polycoordlist.numedge);
 }
 
 /* returns successor polygon edge # */
@@ -1854,25 +1846,38 @@ typedef struct Impoly
 	struct Impoly *nextmpoly;			/* next polygon in linked list */
 } MPOLY;
 
-MPOLY *db_mpolylist = NOMPOLY;			/* the list of active MPOLYs */
-MPOLY *db_mpolyfree = NOMPOLY;			/* the list of unused MPOLYs */
-MPOLY *db_mergenewmp = NOMPOLY;			/* temporary MPOLY for merging */
-MPOLY *db_mergeholemp = NOMPOLY;		/* temporary MPOLY for finding holes */
+MPOLY  *db_mpolylist = NOMPOLY;			/* the list of active MPOLYs */
+MPOLY  *db_mpolyfree = NOMPOLY;			/* the list of unused MPOLYs */
+MPOLY  *db_mergenewmp = NOMPOLY;		/* temporary MPOLY for merging */
+MPOLY  *db_mergeholemp = NOMPOLY;		/* temporary MPOLY for finding holes */
 
-INTSML db_addholetompoly(MPOLY *holemp, MPOLY *newmp);
-INTSML db_addintersectionpoints(MPOLY *mp, MPOLY *omp);
-MPOLY *db_allocmpoly(void);
-INTSML db_copympoly(MPOLY *smp, MPOLY *dmp);
-INTSML db_ensurespace(MPOLY *mp, INTBIG count);
-void   db_freempoly(MPOLY *mp);
-INTSML db_insertmpolypoint(MPOLY *newmp, INTBIG *insert, INTBIG x, INTBIG y);
-INTSML db_insidempoly(INTBIG x, INTBIG y, MPOLY *mp);
-INTSML db_linesintersect(INTBIG fx, INTBIG fy, INTBIG tx, INTBIG ty,
+/* for figuring out what to do at an intersection */
+INTBIG *db_mergeintpol;					/* polygon with intersection */
+INTBIG *db_mergeintpt;					/* point with intersection */
+INTBIG *db_mergeintleaveangle;			/* angle of intersection */
+INTBIG  db_mergeintcount;				/* number of intersection */
+INTBIG  db_mergeinttotal = 0;			/* size of intersection arrays */
+
+INTSML db_mergeaddholetopoly(MPOLY *holemp, MPOLY *newmp);
+void   db_mergeaddintersectionpoint(INTBIG polygon, INTBIG point);
+INTSML db_mergeaddintersectionpoints(MPOLY *mp, MPOLY *omp);
+MPOLY *db_mergeallocpoly(void);
+INTSML db_mergecopypoly(MPOLY *smp, MPOLY *dmp);
+INTSML db_mergeensurespace(MPOLY *mp, INTBIG count);
+INTBIG db_mergefindstartingpoint(MPOLY *mp, MPOLY *omp);
+void   db_mergefreepoly(MPOLY *mp);
+INTSML db_mergeinsertpoint(MPOLY *newmp, INTBIG *insert, INTBIG x, INTBIG y);
+INTSML db_mergeisinside(INTBIG x, INTBIG y, MPOLY *mp);
+INTSML db_mergelinesintersect(INTBIG fx, INTBIG fy, INTBIG tx, INTBIG ty,
 		INTBIG ofx, INTBIG ofy, INTBIG otx, INTBIG oty, INTBIG *ix, INTBIG *iy);
-INTSML db_mergempoly(MPOLY *mp, MPOLY *omp);
-INTBIG db_mpolyarea(MPOLY *mp);
-void   db_simplifyandbbox(MPOLY *mp);
-INTSML db_walkedge(MPOLY *mp, INTBIG i, MPOLY *omp, MPOLY *newmp);
+INTSML db_mergepoly(MPOLY *mp, MPOLY *omp);
+void   db_mergesimplifyandbbox(MPOLY *mp);
+INTSML db_mergewalkedge(MPOLY *mp, INTBIG i, MPOLY *omp, MPOLY *newmp);
+
+/* #define DEBUGWALK 1 */			/* uncomment to debug polygon point traversal */
+/* #define SHOWRESULTS  1 */		/* uncomment to show results of each merge */
+/* #define SHOWTICKS    1 */		/* uncomment to show coordinate values with results */
+/* #define DESIREDLAYER 6 */		/* uncomment to select a specific layer */
 
 /*
  * Routine to free all memory associated with this module.
@@ -1889,13 +1894,13 @@ void db_freemergememory(void)
 	}
 
 	/* free memory used by polygonal merger */
-	if (db_mergenewmp != NOMPOLY) db_freempoly(db_mergenewmp);
-	if (db_mergeholemp != NOMPOLY) db_freempoly(db_mergeholemp);
+	if (db_mergenewmp != NOMPOLY) db_mergefreepoly(db_mergenewmp);
+	if (db_mergeholemp != NOMPOLY) db_mergefreepoly(db_mergeholemp);
 	while (db_mpolylist != NOMPOLY)
 	{
 		mp = db_mpolylist;
 		db_mpolylist = mp->nextmpoly;
-		db_freempoly(mp);
+		db_mergefreepoly(mp);
 	}
 	while (db_mpolyfree != NOMPOLY)
 	{
@@ -1910,6 +1915,13 @@ void db_freemergememory(void)
 		}
 		efree((char *)mp);
 	}
+
+	if (db_mergeinttotal > 0)
+	{
+		efree((char *)db_mergeintpol);
+		efree((char *)db_mergeintpt);
+		efree((char *)db_mergeintleaveangle);
+	}
 }
 
 /*
@@ -1926,7 +1938,7 @@ void mergeinit(void)
 	{
 		mp = db_mpolylist;
 		db_mpolylist = mp->nextmpoly;
-		db_freempoly(mp);
+		db_mergefreepoly(mp);
 	}
 }
 
@@ -1941,14 +1953,14 @@ void mergestorepolygon(INTBIG layer, TECHNOLOGY *tech, POLYGON *poly)
 	INTBIG lx, hx, ly, hy;
 
 	/* copy the polygon into a local structure */
-	mp = db_allocmpoly();
-	if (db_ensurespace(mp, poly->count) != 0) return;
+	mp = db_mergeallocpoly();
+	if (db_mergeensurespace(mp, poly->count) != 0) return;
 	if (poly->style == FILLEDRECT || poly->style == CLOSEDRECT)
 	{
 		getbbox(poly, &lx, &hx, &ly, &hy);
 		makerectpoly(lx, hx, ly, hy, poly);
 	}
-	if (areapoly(poly) < 0)
+	if (areapoints(poly->count, poly->xv, poly->yv) < 0)
 	{
 		/* reverse direction of points while copying */
 		for(i=0; i<poly->count; i++)
@@ -1967,12 +1979,32 @@ void mergestorepolygon(INTBIG layer, TECHNOLOGY *tech, POLYGON *poly)
 	mp->count = poly->count;
 	mp->layer = layer;
 	mp->tech = tech;
-	db_simplifyandbbox(mp);
+	db_mergesimplifyandbbox(mp);
+
+#if 0
+	{
+		char line[20];
+		(void)initinfstr();
+		addstringtoinfstr("Adding polygon on layer ");
+		sprintf(line, "%ld", layer);
+		addstringtoinfstr(line);
+		addstringtoinfstr(" is");
+		for(i=0; i<mp->count; i++)
+		{
+			addstringtoinfstr(" (");
+			addstringtoinfstr(latoa(mp->x[i]));
+			addstringtoinfstr(",");
+			addstringtoinfstr(latoa(mp->y[i]));
+			addstringtoinfstr(")");
+		}
+		ttyputmsg("%s", returninfstr());
+	}
+#endif
 
 	/* now look for merges */
 	for(omp = db_mpolylist; omp != NOMPOLY; omp = omp->nextmpoly)
 	{
-		if (db_mergempoly(mp, omp) != 0) break;
+		if (db_mergepoly(mp, omp) != 0) break;
 	}
 	if (omp == NOMPOLY)
 	{
@@ -1982,7 +2014,7 @@ void mergestorepolygon(INTBIG layer, TECHNOLOGY *tech, POLYGON *poly)
 	} else
 	{
 		/* merged: delete this */
-		db_freempoly(mp);
+		db_mergefreepoly(mp);
 
 		/* see if any other merging can be done */
 		done = 0;
@@ -1994,11 +2026,11 @@ void mergestorepolygon(INTBIG layer, TECHNOLOGY *tech, POLYGON *poly)
 			{
 				if (mp != omp)
 				{
-					if (db_mergempoly(mp, omp) != 0)
+					if (db_mergepoly(mp, omp) != 0)
 					{
 						if (lastmp == NOMPOLY) db_mpolylist = mp->nextmpoly; else
 							lastmp->nextmpoly = mp->nextmpoly;
-						db_freempoly(mp);
+						db_mergefreepoly(mp);
 						done = 0;
 						break;
 					}
@@ -2007,19 +2039,28 @@ void mergestorepolygon(INTBIG layer, TECHNOLOGY *tech, POLYGON *poly)
 			}
 		}
 	}
-#if 0
+
+#ifdef SHOWRESULTS
+
+#ifdef DESIREDLAYER
+	if (layer == DESIREDLAYER)
+#endif
 	{
 		REGISTER NODEPROTO *facet, *prim;
 		REGISTER NODEINST *ni;
 		REGISTER INTBIG lx, hx, ly, hy, cx, cy, *newlist;
-		static INTBIG xoff = 0;
+		static INTBIG xoff = 0, yoff = 0;
+		char line[20];
 
-		xoff += el_curtech->deflambda * 20;
+		xoff += el_curtech->deflambda * 150;
 		facet = getcurfacet();
 		if (facet == NONODEPROTO) return;
 		prim = getnodeproto("metal-1-node");
 		for(mp = db_mpolylist; mp != NOMPOLY; mp = mp->nextmpoly)
 		{
+#ifdef DESIREDLAYER
+			if (mp->layer != DESIREDLAYER) continue;
+#endif
 			lx = hx = mp->x[0];
 			ly = hy = mp->y[0];
 			for(i=1; i<mp->count; i++)
@@ -2030,7 +2071,7 @@ void mergestorepolygon(INTBIG layer, TECHNOLOGY *tech, POLYGON *poly)
 				if (mp->y[i] > hy) hy = mp->y[i];
 			}
 			cx = (lx+hx) / 2;   cy = (ly+hy) / 2;
-			ni = newnodeinst(prim, lx+xoff, hx+xoff, ly, hy, 0, 0, facet);
+			ni = newnodeinst(prim, lx+xoff, hx+xoff, ly+yoff, hy+yoff, 0, 0, facet);
 			newlist = (INTBIG *)emalloc(mp->count * 2 * SIZEOFINTBIG, el_tempcluster);
 			if (newlist == 0) return;
 			for(i=0; i<mp->count; i++)
@@ -2041,7 +2082,101 @@ void mergestorepolygon(INTBIG layer, TECHNOLOGY *tech, POLYGON *poly)
 			(void)setvalkey((INTBIG)ni, VNODEINST, el_trace, (INTBIG)newlist,
 				VINTEGER|VISARRAY|((mp->count*2)<<VLENGTHSH));
 			endobjectchange((INTBIG)ni, VNODEINST);
+
+			(void)initinfstr();
+			addstringtoinfstr("Resulting poly on layer ");
+			sprintf(line, "%ld", layer);
+			addstringtoinfstr(line);
+			addstringtoinfstr(" is");
+			for(i=0; i<mp->count; i++)
+			{
+				addstringtoinfstr(" (");
+				addstringtoinfstr(latoa(mp->x[i]));
+				addstringtoinfstr(",");
+				addstringtoinfstr(latoa(mp->y[i]));
+				addstringtoinfstr(")");
+			}
+			ttyputmsg("%s", returninfstr());
 		}
+#ifdef SHOWTICKS
+		for(mp = db_mpolylist; mp != NOMPOLY; mp = mp->nextmpoly)
+		{
+			if (mp == db_mpolylist)
+			{
+				lx = hx = mp->x[0];
+				ly = hy = mp->y[0];
+			}
+			for(i=0; i<mp->count; i++)
+			{
+				if (mp->x[i] < lx) lx = mp->x[i];
+				if (mp->x[i] > hx) hx = mp->x[i];
+				if (mp->y[i] < ly) ly = mp->y[i];
+				if (mp->y[i] > hy) hy = mp->y[i];
+			}
+		}
+		for(mp = db_mpolylist; mp != NOMPOLY; mp = mp->nextmpoly)
+		{
+			REGISTER INTBIG j;
+			extern NODEPROTO *gen_invispinprim;
+
+			for(i=0; i<mp->count; i++)
+			{
+				/* see if the X value has already been done */
+				for(j=0; j<i; j++) if (mp->x[j] == mp->x[i]) break;
+				if (j < i) continue;
+				for(omp = db_mpolylist; omp != mp; omp = omp->nextmpoly)
+				{
+					for(j=0; j<omp->count; j++)
+						if (omp->x[j] == mp->x[i]) break;
+					if (j < omp->count) break;
+				}
+				if (omp != mp) continue;
+
+				/* show the X coordinate */
+				ni = newnodeinst(gen_invispinprim, mp->x[i]+xoff, mp->x[i]+xoff,
+					ly-el_curtech->deflambda*2+yoff, ly-el_curtech->deflambda*2+yoff,
+						0, 0, facet);
+				if (ni != NONODEINST)
+				{
+					REGISTER VARIABLE *var;
+					var = setvalkey((INTBIG)ni, VNODEINST, el_node_name, (INTBIG)latoa(mp->x[i]),
+						VSTRING|VDISPLAY);
+					if (var != NOVARIABLE)
+						var->textdescript = (var->textdescript & ~(VTSIZE|VTPOSITION)) |
+							(VTPOSDOWN | (1<<VTSIZESH));
+					endobjectchange((INTBIG)ni, VNODEINST);
+				}
+			}
+
+			for(i=0; i<mp->count; i++)
+			{
+				/* see if the Y value has already been done */
+				for(j=0; j<i; j++) if (mp->y[j] == mp->y[i]) break;
+				if (j < i) continue;
+				for(omp = db_mpolylist; omp != mp; omp = omp->nextmpoly)
+				{
+					for(j=0; j<omp->count; j++)
+						if (omp->y[j] == mp->y[i]) break;
+					if (j < omp->count) break;
+				}
+				if (omp != mp) continue;
+
+				/* show the Y coordinate */
+				ni = newnodeinst(gen_invispinprim, lx-el_curtech->deflambda*2+xoff,
+					lx-el_curtech->deflambda*2+xoff, mp->y[i]+yoff, mp->y[i]+yoff, 0, 0, facet);
+				if (ni != NONODEINST)
+				{
+					REGISTER VARIABLE *var;
+					var = setvalkey((INTBIG)ni, VNODEINST, el_node_name, (INTBIG)latoa(mp->y[i]),
+						VSTRING|VDISPLAY);
+					if (var != NOVARIABLE)
+						var->textdescript = (var->textdescript & ~(VTSIZE|VTPOSITION)) |
+							(VTPOSLEFT | (1<<VTSIZESH));
+					endobjectchange((INTBIG)ni, VNODEINST);
+				}
+			}
+		}
+#endif
 	}
 #endif
 }
@@ -2071,7 +2206,7 @@ void mergedone(void (*writepolygon)(INTSML, TECHNOLOGY*, INTBIG*, INTBIG*, INTSM
  * second polygon).  If they merge then the routine returns nonzero and places the
  * merged polygon in "omp".
  */
-INTSML db_mergempoly(MPOLY *mp, MPOLY *omp)
+INTSML db_mergepoly(MPOLY *mp, MPOLY *omp)
 {
 	REGISTER INTBIG i, oi, startpol, startpt, intpoints;
 	MPOLY *mps[2];
@@ -2088,9 +2223,9 @@ INTSML db_mergempoly(MPOLY *mp, MPOLY *omp)
 	for(oi=0; oi<omp->count; oi++) omp->intersect[oi] = 0;
 
 	/* create additional points on each polygon at the intersections */
-	if (db_addintersectionpoints(mp, omp) != 0) return(0);
-	if (db_addintersectionpoints(mp, mp) != 0) return(0);
-	if (db_addintersectionpoints(omp, omp) != 0) return(0);
+	if (db_mergeaddintersectionpoints(mp, omp) != 0) return(0);
+	if (db_mergeaddintersectionpoints(mp, mp) != 0) return(0);
+	if (db_mergeaddintersectionpoints(omp, omp) != 0) return(0);
 
 	/* if there are no intersection points, see if one polygon is inside the other */
 	intpoints = 0;
@@ -2099,50 +2234,46 @@ INTSML db_mergempoly(MPOLY *mp, MPOLY *omp)
 	if (intpoints == 0)
 	{
 		/* figure out which polygon is smaller */
-		if (db_mpolyarea(mp) < db_mpolyarea(omp))
+		if (areapoints(mp->count, mp->x, mp->y) <
+			areapoints(omp->count, omp->x, omp->y))
 		{
 			/* if polygons don't intersect, stop */
-			if (db_insidempoly(mp->x[0], mp->y[0], omp) == 0) return(0);
+			if (db_mergeisinside(mp->x[0], mp->y[0], omp) == 0) return(0);
 
 			/* first polygon is smaller and covered by second */
 			return(1);
 		} else
 		{
 			/* if polygons don't intersect, stop */
-			if (db_insidempoly(omp->x[0], omp->y[0], mp) == 0) return(0);
+			if (db_mergeisinside(omp->x[0], omp->y[0], mp) == 0) return(0);
 
 			/* second polygon is smaller, copy first to second */
-			if (db_copympoly(mp, omp) != 0) return(0);
-			db_simplifyandbbox(omp);
+			if (db_mergecopypoly(mp, omp) != 0) return(0);
+			db_mergesimplifyandbbox(omp);
 			return(1);
 		}
 	}
 
 	/* find a starting point (one with no intersections and not inside the other) */
-	for(i=0; i<mp->count; i++)
-		if (mp->intersect[i] == 0 && db_insidempoly(mp->x[i], mp->y[i], omp) == 0) break;
-	if (i < mp->count)
+	i = db_mergefindstartingpoint(mp, omp);
+	if (i >= 0)
 	{
 		startpol = 0;
 		startpt = i;
 	} else
 	{
-		for(oi=0; oi<omp->count; oi++)
-			if (omp->intersect[oi] == 0 && db_insidempoly(omp->x[i], omp->y[i], mp) == 0) break;
-		if (oi >= omp->count)
+		oi = db_mergefindstartingpoint(omp, mp);
+		if (oi < 0)
 		{
 			/* all points are intersections: return larger polygon */
-			if (db_mpolyarea(mp) < db_mpolyarea(omp))
-			{
-				/* ignore first polygon */
-				return(1);
-			} else
+			if (areapoints(mp->count, mp->x, mp->y) >
+				areapoints(omp->count, omp->x, omp->y))
 			{
 				/* copy first to second */
-				if (db_copympoly(mp, omp) != 0) return(0);
-				db_simplifyandbbox(omp);
-				return(1);
+				if (db_mergecopypoly(mp, omp) != 0) return(0);
 			}
+			db_mergesimplifyandbbox(omp);
+			return(1);
 		}
 		startpol = 1;
 		startpt = oi;
@@ -2152,44 +2283,85 @@ INTSML db_mergempoly(MPOLY *mp, MPOLY *omp)
 	for(i=0; i<mp->count; i++) mp->seen[i] = 0;
 	for(oi=0; oi<omp->count; oi++) omp->seen[oi] = 0;
 	mps[0] = mp;   mps[1] = omp;
-	if (db_mergenewmp == NOMPOLY) db_mergenewmp = db_allocmpoly();
+	if (db_mergenewmp == NOMPOLY) db_mergenewmp = db_mergeallocpoly();
 	db_mergenewmp->count = 0;
-	if (db_walkedge(mps[startpol], startpt, mps[1-startpol], db_mergenewmp) != 0)
+	if (db_mergewalkedge(mps[startpol], startpt, mps[1-startpol], db_mergenewmp) != 0)
 		return(0);
 
 	/* see if there are any points that didn't get seen and are not in the other polygon */
-	if (db_mergeholemp == NOMPOLY) db_mergeholemp = db_allocmpoly();
+	if (db_mergeholemp == NOMPOLY) db_mergeholemp = db_mergeallocpoly();
 	for(i=0; i<mp->count; i++)
 	{
 		if (mp->seen[i] != 0) continue;
 		if (mp->intersect[i] != 0) continue;
-		if (db_insidempoly(mp->x[i], mp->y[i], omp) != 0) continue;
+		if (db_mergeisinside(mp->x[i], mp->y[i], omp) != 0) continue;
 		db_mergeholemp->count = 0;
-		if (db_walkedge(mp, i, omp, db_mergeholemp) != 0) return(0);
-		if (db_addholetompoly(db_mergeholemp, db_mergenewmp) != 0) return(0);
+		if (db_mergewalkedge(mp, i, omp, db_mergeholemp) != 0) return(0);
+		if (db_mergeaddholetopoly(db_mergeholemp, db_mergenewmp) != 0) return(0);
 	}
 	for(oi=0; oi<omp->count; oi++)
 	{
 		if (omp->seen[oi] != 0) continue;
 		if (omp->intersect[oi] != 0) continue;
-		if (db_insidempoly(omp->x[oi], omp->y[oi], mp) != 0) continue;
+		if (db_mergeisinside(omp->x[oi], omp->y[oi], mp) != 0) continue;
 		db_mergeholemp->count = 0;
-		if (db_walkedge(omp, oi, mp, db_mergeholemp) != 0) return(0);
-		if (db_addholetompoly(db_mergeholemp, db_mergenewmp) != 0) return(0);
+		if (db_mergewalkedge(omp, oi, mp, db_mergeholemp) != 0) return(0);
+		if (db_mergeaddholetopoly(db_mergeholemp, db_mergenewmp) != 0) return(0);
 	}
 
 	/* copy result back to second polygon */
-	if (db_copympoly(db_mergenewmp, omp) != 0) return(0);
-	db_simplifyandbbox(omp);
+	if (db_mergecopypoly(db_mergenewmp, omp) != 0) return(0);
+	db_mergesimplifyandbbox(omp);
 	return(1);
 }
 
+/*
+ * Routine to find a good starting point on polygon "mp" (the other one is "omp").
+ * Looks for points with no intersection marks, preferably on the outside edge,
+ * and not inside of the other polygon
+ */
+INTBIG db_mergefindstartingpoint(MPOLY *mp, MPOLY *omp)
+{
+	REGISTER INTBIG i, lx, hx, ly, hy;
+
+	lx = hx = mp->x[0];
+	ly = hy = mp->y[0];
+	for(i=1; i<mp->count; i++)
+	{
+		if (mp->x[i] < lx) lx = mp->x[i];
+		if (mp->x[i] > hx) hx = mp->x[i];
+		if (mp->y[i] < ly) ly = mp->y[i];
+		if (mp->y[i] > hy) hy = mp->y[i];
+	}
+
+	/* look for points with no intersection, on edge, and not inside other */
+	for(i=0; i<mp->count; i++)
+	{
+		if (mp->intersect[i] != 0) continue;
+		if (mp->x[i] != lx && mp->x[i] != hx &&
+			mp->y[i] != ly && mp->y[i] != hy) continue;
+		if (db_mergeisinside(mp->x[i], mp->y[i], omp) != 0) continue;
+		return(i);
+	}
+
+	/* accept points that aren't on the edge */
+	for(i=0; i<mp->count; i++)
+	{
+		if (mp->intersect[i] != 0) continue;
+		if (db_mergeisinside(mp->x[i], mp->y[i], omp) != 0) continue;
+		return(i);
+	}
+
+	/* nothing found */
+	return(-1);
+}
+
 /*
  * Helper routine to add hole polygon "holemp" to the main polygon "newmp".
  * Finds the closest points on the two and adds the points.
  * Returns nonzero on error.
  */
-INTSML db_addholetompoly(MPOLY *holemp, MPOLY *newmp)
+INTSML db_mergeaddholetopoly(MPOLY *holemp, MPOLY *newmp)
 {
 	REGISTER INTBIG j, k, dist, bestdist, hint, nint, nx, ny;
 	INTBIG bestpos;
@@ -2211,19 +2383,19 @@ INTSML db_addholetompoly(MPOLY *holemp, MPOLY *newmp)
 	nx = newmp->x[nint];   ny = newmp->y[nint];
 	if (nx != holemp->x[hint] || ny != holemp->y[hint])
 	{
-		if (db_insertmpolypoint(newmp, &bestpos, holemp->x[hint], holemp->y[hint]) != 0) return(1);
+		if (db_mergeinsertpoint(newmp, &bestpos, holemp->x[hint], holemp->y[hint]) != 0) return(1);
 	}
 	j = hint;
 	for(;;)
 	{
 		j++;
 		if (j >= holemp->count) j = 0;
-		if (db_insertmpolypoint(newmp, &bestpos, holemp->x[j], holemp->y[j]) != 0) return(1);
+		if (db_mergeinsertpoint(newmp, &bestpos, holemp->x[j], holemp->y[j]) != 0) return(1);
 		if (j == hint) break;
 	}
 	if (nx != holemp->x[hint] || ny != holemp->y[hint])
 	{
-		if (db_insertmpolypoint(newmp, &bestpos, nx, ny) != 0) return(1);
+		if (db_mergeinsertpoint(newmp, &bestpos, nx, ny) != 0) return(1);
 	}
 	return(0);
 }
@@ -2234,7 +2406,7 @@ INTSML db_addholetompoly(MPOLY *holemp, MPOLY *newmp)
  * points are added to the polygon and marked (1 for self-intersection, 2 for intersection
  * with the other polygon).  Returns nonzero on error.
  */
-INTSML db_addintersectionpoints(MPOLY *mp, MPOLY *omp)
+INTSML db_mergeaddintersectionpoints(MPOLY *mp, MPOLY *omp)
 {
 	REGISTER INTBIG i, j, oi, done, fx, fy, tx, ty, ofx, ofy, otx, oty, last, olast,
 		intvalue;
@@ -2261,7 +2433,7 @@ INTSML db_addintersectionpoints(MPOLY *mp, MPOLY *omp)
 				if (mp == omp && (i == oi || last == oi || olast == i)) continue;
 
 				/* see if they intersect */
-				if (db_linesintersect(fx,fy, tx,ty, ofx,ofy, otx,oty, &ix,&iy) == 0) continue;
+				if (db_mergelinesintersect(fx,fy, tx,ty, ofx,ofy, otx,oty, &ix,&iy) == 0) continue;
 
 				/* mark points that intersect */
 				if (ix == fx && iy == fy) mp->intersect[last] = intvalue;
@@ -2278,7 +2450,7 @@ INTSML db_addintersectionpoints(MPOLY *mp, MPOLY *omp)
 				/* add intersection point to the first polygon */
 				if ((ix != fx || iy != fy) && (ix != tx || iy != ty))
 				{
-					if (db_ensurespace(mp, mp->count+1) != 0) return(1);
+					if (db_mergeensurespace(mp, mp->count+1) != 0) return(1);
 					for(j = mp->count; j > i; j--)
 					{
 						mp->x[j] = mp->x[j-1];
@@ -2294,7 +2466,7 @@ INTSML db_addintersectionpoints(MPOLY *mp, MPOLY *omp)
 				/* add intersection point to other polygon */
 				if ((ix != ofx || iy != ofy) && (ix != otx || iy != oty))
 				{
-					if (db_ensurespace(omp, omp->count+1) != 0) return(1);
+					if (db_mergeensurespace(omp, omp->count+1) != 0) return(1);
 					for(j = omp->count; j > oi; j--)
 					{
 						omp->x[j] = omp->x[j-1];
@@ -2320,15 +2492,20 @@ INTSML db_addintersectionpoints(MPOLY *mp, MPOLY *omp)
  * Helper routine to walk around the edge of polygon "mp", starting at point "i".  If it hits
  * polygon "omp", merge it in.  Resulting polygon is in "newmp".  Returns nonzero on error.
  */
-INTSML db_walkedge(MPOLY *mp, INTBIG i, MPOLY *omp, MPOLY *newmp)
+INTSML db_mergewalkedge(MPOLY *mp, INTBIG i, MPOLY *omp, MPOLY *newmp)
 {
 	MPOLY *mps[2];
 	REGISTER MPOLY *mp1, *mp2;
-	REGISTER INTSML enterangle, leaveangle, oleaveangle, ooleaveangle;
-	REGISTER INTBIG startpol, startpt, last, next, pol, pt, seenpt, unseenpt, seenthis, unseenthis;
+	REGISTER INTSML enterangle, leaveangle;
+	REGISTER INTBIG startpol, startpt, last, next, pol, pt, unseenpt,
+		unseenthis, smallestthisangle, smallestotherangle;
 	INTBIG insert;
 	static INTBIG seen = 0;
 
+#ifdef DEBUGWALK
+	ttyputmsg("Walking around edge (polygon 1 has %d points, polygon 2 has %d points):",
+		mp->count, omp->count);
+#endif
 	seen++;
 	if (seen == 0) seen++;
 	mps[0] = mp;   mps[1] = omp;
@@ -2341,38 +2518,38 @@ INTSML db_walkedge(MPOLY *mp, INTBIG i, MPOLY *omp, MPOLY *newmp)
 		mp1 = mps[pol];
 		mp2 = mps[1-pol];
 		mp1->seen[pt] = seen;
-		if (db_insertmpolypoint(newmp, &insert, mps[pol]->x[pt], mps[pol]->y[pt]) != 0)
+#ifdef DEBUGWALK
+		ttyputmsg("  Add (%s,%s) from polygon %d", latoa(mps[pol]->x[pt]), latoa(mps[pol]->y[pt]), pol);
+#endif
+		if (db_mergeinsertpoint(newmp, &insert, mps[pol]->x[pt], mps[pol]->y[pt]) != 0)
 			return(1);
 		if (newmp->count == 1000)
 		{
-			ttyputerr("Warning: merged polygon with 1000 points");
+			ttyputerr("WARNING: loop detected while merging polygons on layer %s",
+				layername(mp->tech, (INTSML)mp->layer));
 			return(1);
 		}
 
-		/* see if this point is elsewhere on this polygon */
-		seenthis = unseenthis = -1;
+		/* see if there are intersection points on this or the other polygon */
+		db_mergeintcount = 0;
 		for(i=0; i<mp1->count; i++)
 		{
 			if (i == pt) continue;
 			if (mp1->x[i] != mp1->x[pt] || mp1->y[i] != mp1->y[pt]) continue;
-			if (mp1->seen[i] == seen) seenthis = i; else
-				unseenthis = i;
+			db_mergeaddintersectionpoint(pol, i);
 		}
-		if (unseenthis < 0) unseenthis = seenthis;
-
-		/* see if this point is on the other polygon */
-		seenpt = unseenpt = -1;
 		for(i=0; i<mp2->count; i++)
 		{
 			if (mp2->x[i] != mp1->x[pt] || mp2->y[i] != mp1->y[pt]) continue;
-			if (mp2->seen[i] == seen) seenpt = i; else
-				unseenpt = i;
+			db_mergeaddintersectionpoint(1-pol, i);
 		}
-		if (unseenpt < 0) unseenpt = seenpt;
 
 		/* if point not on other polygon or elsewhere on this, keep walking around edge */
-		if (unseenthis < 0 && unseenpt < 0)
+		if (db_mergeintcount == 0)
 		{
+#ifdef DEBUGWALK
+			ttyputmsg("      Not an intersection point");
+#endif
 			pt++;
 			if (pt >= mp1->count) pt = 0;
 			if (pol == startpol && pt == startpt) break;
@@ -2388,26 +2565,49 @@ INTSML db_walkedge(MPOLY *mp, INTBIG i, MPOLY *omp, MPOLY *newmp)
 		if (leaveangle > enterangle) leaveangle -= 3600;
 		leaveangle = enterangle - leaveangle;
 
-		if (unseenpt >= 0)
+#ifdef DEBUGWALK
+		ttyputmsg("      Intersection point!  enter at %d, leave at %d", enterangle/10, leaveangle/10);
+#endif
+		for(i=0; i<db_mergeintcount; i++)
 		{
-			if (unseenpt == mp2->count-1) next = 0; else next = unseenpt+1;
-			oleaveangle = figureangle(mp1->x[pt], mp1->y[pt], mp2->x[next], mp2->y[next]);
-			if (oleaveangle > enterangle) oleaveangle -= 3600;
-			oleaveangle = enterangle - oleaveangle;
-			if (oleaveangle == 0) oleaveangle = 7200;
-		} else oleaveangle = 7200;
+			if (db_mergeintpt[i] == mps[db_mergeintpol[i]]->count-1) next = 0; else next = db_mergeintpt[i]+1;
+			db_mergeintleaveangle[i] = figureangle(mp1->x[pt], mp1->y[pt],
+				mps[db_mergeintpol[i]]->x[next], mps[db_mergeintpol[i]]->y[next]);
+			if (db_mergeintleaveangle[i] > enterangle) db_mergeintleaveangle[i] -= 3600;
+			db_mergeintleaveangle[i] = enterangle - db_mergeintleaveangle[i];
+			if (db_mergeintleaveangle[i] == 0) db_mergeintleaveangle[i] = 7200;
+#ifdef DEBUGWALK
+			ttyputmsg("        Point on %s polygon goes to (%s,%s), angle=%d",
+				(db_mergeintpol[i] == pol ? "this" : "other"), latoa(mps[db_mergeintpol[i]]->x[next]),
+				latoa(mps[db_mergeintpol[i]]->y[next]), db_mergeintleaveangle[i]/10);
+#endif
+		}
 
-		if (unseenthis >= 0)
+		smallestthisangle = 7200;
+		unseenthis = -1;
+		for(i=0; i<db_mergeintcount; i++)
 		{
-			if (unseenthis == mp1->count-1) next = 0; else next = unseenthis+1;
-			ooleaveangle = figureangle(mp1->x[pt], mp1->y[pt], mp1->x[next], mp1->y[next]);
-			if (ooleaveangle > enterangle) ooleaveangle -= 3600;
-			ooleaveangle = enterangle - ooleaveangle;
-			if (ooleaveangle == 0) ooleaveangle = 7200;
-		} else ooleaveangle = 7200;
-		if (ooleaveangle < leaveangle && ooleaveangle < oleaveangle)
+			if (db_mergeintpol[i] != pol) continue;
+			if (db_mergeintleaveangle[i] > smallestthisangle) continue;
+			smallestthisangle = db_mergeintleaveangle[i];
+			unseenthis = db_mergeintpt[i];
+		}
+
+		smallestotherangle = 7200;
+		unseenpt = -1;
+		for(i=0; i<db_mergeintcount; i++)
+		{
+			if (db_mergeintpol[i] == pol) continue;
+			if (db_mergeintleaveangle[i] > smallestotherangle) continue;
+			smallestotherangle = db_mergeintleaveangle[i];
+			unseenpt = db_mergeintpt[i];
+		}
+		if (smallestthisangle < leaveangle && smallestthisangle < smallestotherangle)
 		{
 			/* staying on the same polygon but jumping to a new point */
+#ifdef DEBUGWALK
+			ttyputmsg("        Stay on same polygon, jump to new point");
+#endif
 			pt = unseenthis;
 			if (pol == startpol && pt == startpt) break;
 			mp1->seen[pt] = seen;
@@ -2418,14 +2618,20 @@ INTSML db_walkedge(MPOLY *mp, INTBIG i, MPOLY *omp, MPOLY *newmp)
 			if (pol == startpol && pt == startpt) break;
 			continue;
 		}
-		if (leaveangle < oleaveangle)
+		if (leaveangle <= smallestthisangle && leaveangle <= smallestotherangle)
 		{
 			/* stay on the current polygon */
+#ifdef DEBUGWALK
+			ttyputmsg("        Stay on same polygon");
+#endif
 			pt++;
 			if (pt >= mp1->count) pt = 0;
 		} else
 		{
 			/* switch to the other polygon */
+#ifdef DEBUGWALK
+			ttyputmsg("        Switch to other polygon");
+#endif
 			pt = unseenpt;
 			pol = 1 - pol;
 			if (pol == startpol && pt == startpt) break;
@@ -2444,11 +2650,11 @@ INTSML db_walkedge(MPOLY *mp, INTBIG i, MPOLY *omp, MPOLY *newmp)
  * Helper routine to insert the point (x, y) at "insert" in polygon "mp".
  * Returns nonzero on error.
  */
-INTSML db_insertmpolypoint(MPOLY *mp, INTBIG *insert, INTBIG x, INTBIG y)
+INTSML db_mergeinsertpoint(MPOLY *mp, INTBIG *insert, INTBIG x, INTBIG y)
 {
 	REGISTER INTBIG i;
 
-	if (db_ensurespace(mp, mp->count+1) != 0) return(1);
+	if (db_mergeensurespace(mp, mp->count+1) != 0) return(1);
 	if (*insert <= 0 || mp->x[*insert-1] != x || mp->y[*insert-1] != y)
 	{
 		for(i=mp->count; i > *insert; i--)
@@ -2464,12 +2670,52 @@ INTSML db_insertmpolypoint(MPOLY *mp, INTBIG *insert, INTBIG x, INTBIG y)
 	return(0);
 }
 
+/*
+ * Routine to add polygon "polygon" and point "point" to the global arrays of 
+ * intersection points.
+ */
+void db_mergeaddintersectionpoint(INTBIG polygon, INTBIG point)
+{
+	REGISTER INTBIG i, newtotal, *newpol, *newpt, *newangle;
+
+	if (db_mergeintcount >= db_mergeinttotal)
+	{
+		newtotal = db_mergeinttotal * 2;
+		if (newtotal <= 0) newtotal = 10;
+		newpol = (INTBIG *)emalloc(newtotal * SIZEOFINTBIG, db_cluster);
+		if (newpol == 0) return;
+		newpt = (INTBIG *)emalloc(newtotal * SIZEOFINTBIG, db_cluster);
+		if (newpt == 0) return;
+		newangle = (INTBIG *)emalloc(newtotal * SIZEOFINTBIG, db_cluster);
+		if (newangle == 0) return;
+		for(i=0; i<db_mergeintcount; i++)
+		{
+			newpol[i] = db_mergeintpol[i];
+			newpt[i] = db_mergeintpt[i];
+			newangle[i] = db_mergeintleaveangle[i];
+		}
+		if (db_mergeinttotal > 0)
+		{
+			efree((char *)db_mergeintpol);
+			efree((char *)db_mergeintpt);
+			efree((char *)db_mergeintleaveangle);
+		}
+		db_mergeintpol = newpol;
+		db_mergeintpt = newpt;
+		db_mergeintleaveangle = newangle;
+		db_mergeinttotal = newtotal;
+	}
+	db_mergeintpol[db_mergeintcount] = polygon;
+	db_mergeintpt[db_mergeintcount] = point;
+	db_mergeintcount++;
+}
+
 /*
  * Helper routine to determine whether the line segement (fx,fy)-(tx,ty)
  * intersects the line segment (ofx,ofy)-(otx,oty).  If it does, the intersection
  * point is placed in (ix,iy) and the routine returns nonzero.
  */
-INTSML db_linesintersect(INTBIG fx, INTBIG fy, INTBIG tx, INTBIG ty,
+INTSML db_mergelinesintersect(INTBIG fx, INTBIG fy, INTBIG tx, INTBIG ty,
 	INTBIG ofx, INTBIG ofy, INTBIG otx, INTBIG oty, INTBIG *ix, INTBIG *iy)
 {
 	double fa1, fb1, fc1, fa2, fb2, fc2, fswap, fiy, ang, oang, hang, ohang;
@@ -2521,7 +2767,7 @@ INTSML db_linesintersect(INTBIG fx, INTBIG fy, INTBIG tx, INTBIG ty,
  * Helper routine to simplify polygon "mp", removing redundant and colinear points,
  * and recomputing its bounding box.
  */
-void db_simplifyandbbox(MPOLY *mp)
+void db_mergesimplifyandbbox(MPOLY *mp)
 {
 	REGISTER INTBIG i, j, prev, next;
 
@@ -2574,37 +2820,10 @@ void db_simplifyandbbox(MPOLY *mp)
 	}
 }
 
-#define DIV2(x)     ((x) >= 0 ? (x)>>1 : (-((-(x)) >> 1)))
-
-/*
- * Helper routine to return the area of polygon "mp".
- * The area will be negative if the points are going counterclockwise.
- */
-INTBIG db_mpolyarea(MPOLY *mp)
-{
-	REGISTER INTBIG i, x0, y0, x1, y1, area;
-
-	x0 = mp->x[0];
-	y0 = mp->y[0];
-	area = 0;
-	for (i=1; i<mp->count; i++)
-	{
-		x1 = mp->x[i];
-		y1 = mp->y[i];
-
-		/* triangulate around the polygon */
-		area += DIV2((x1 - x0) * (y0 + y1));
-		x0 = x1;
-		y0 = y1;
-	}
-	area += DIV2((mp->x[0] - x0) * (mp->y[0] + y0));
-	return(abs(area));
-}
-
 /*
  * Helper routine to return nonzero if poing (x,y) is inside of polygon "mp".
  */
-INTSML db_insidempoly(INTBIG x, INTBIG y, MPOLY *mp)
+INTSML db_mergeisinside(INTBIG x, INTBIG y, MPOLY *mp)
 {
 	REGISTER INTSML i, ang, lastp, tang, thisp;
 
@@ -2630,11 +2849,11 @@ INTSML db_insidempoly(INTBIG x, INTBIG y, MPOLY *mp)
  * Helper routine to copy polygon "smp" to polygon "dmp".
  * Returns nonzero on error.
  */
-INTSML db_copympoly(MPOLY *smp, MPOLY *dmp)
+INTSML db_mergecopypoly(MPOLY *smp, MPOLY *dmp)
 {
 	REGISTER INTBIG i;
 
-	if (db_ensurespace(dmp, smp->count) != 0) return(1);
+	if (db_mergeensurespace(dmp, smp->count) != 0) return(1);
 	for(i=0; i<smp->count; i++)
 	{
 		dmp->x[i] = smp->x[i];
@@ -2648,7 +2867,7 @@ INTSML db_copympoly(MPOLY *smp, MPOLY *dmp)
  * Helper routine to make sure that polygon "mp" has room for "count" points.
  * Returns nonzero on error.
  */
-INTSML db_ensurespace(MPOLY *mp, INTBIG count)
+INTSML db_mergeensurespace(MPOLY *mp, INTBIG count)
 {
 	REGISTER INTBIG i, *newx, *newy, *newseen, *newintersect, newlimit;
 
@@ -2689,7 +2908,7 @@ INTSML db_ensurespace(MPOLY *mp, INTBIG count)
  * Helper routine to allocate a new polygon.
  * Returns NOMPOLY on error.
  */
-MPOLY *db_allocmpoly(void)
+MPOLY *db_mergeallocpoly(void)
 {
 	REGISTER MPOLY *mp;
 
@@ -2710,7 +2929,7 @@ MPOLY *db_allocmpoly(void)
 /*
  * Helper routine to free polygon "mp".
  */
-void db_freempoly(MPOLY *mp)
+void db_mergefreepoly(MPOLY *mp)
 {
 	mp->nextmpoly = db_mpolyfree;
 	db_mpolyfree = mp;
