@@ -983,10 +983,16 @@ static GRAPHICS tech_frame = {LAYERO, MENTXT, SOLIDC, SOLIDC,
 	{0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF}, NOVARIABLE, 0};
 
 #define FRAMESCALE  700.0f
-#define ASCHXSIZE   (10.5f  / FRAMESCALE)
-#define ASCHYSIZE   ( 8.0f  / FRAMESCALE)
-#define DSCHXSIZE   (34.0f  / FRAMESCALE)
-#define DSCHYSIZE   (21.5f  / FRAMESCALE)
+#define ASCHXSIZE   (11.0f  / FRAMESCALE)
+#define ASCHYSIZE   ( 8.5f  / FRAMESCALE)
+#define BSCHXSIZE   (17.0f  / FRAMESCALE)
+#define BSCHYSIZE   (11.0f  / FRAMESCALE)
+#define CSCHXSIZE   (24.0f  / FRAMESCALE)
+#define CSCHYSIZE   (17.0f  / FRAMESCALE)
+#define DSCHXSIZE   (36.0f  / FRAMESCALE)
+#define DSCHYSIZE   (24.0f  / FRAMESCALE)
+#define ESCHXSIZE   (48.0f  / FRAMESCALE)
+#define ESCHYSIZE   (36.0f  / FRAMESCALE)
 #define FRAMEWID    ( 0.15f / FRAMESCALE)
 #define XLOGOBOX    ( 2.0f  / FRAMESCALE)
 #define YLOGOBOX    ( 1.0f  / FRAMESCALE)
@@ -999,34 +1005,27 @@ static GRAPHICS tech_frame = {LAYERO, MENTXT, SOLIDC, SOLIDC,
 INTSML framesize(INTBIG *x, INTBIG *y, NODEPROTO *np)
 {
 	char *size;
+	float wid, hei;
 
 	size = db_getframedesc(np);
 	if (*size == 0) return(1);
 	switch (size[0])
 	{
-		case 'a':
-			if (size[1] == 'v')
-			{
-				*x = scalefromdispunit(ASCHYSIZE - FRAMEWID, DISPUNITINCH);
-				*y = scalefromdispunit(ASCHXSIZE - FRAMEWID, DISPUNITINCH);
-			} else
-			{
-				*x = scalefromdispunit(ASCHXSIZE - FRAMEWID, DISPUNITINCH);
-				*y = scalefromdispunit(ASCHYSIZE - FRAMEWID, DISPUNITINCH);
-			}
-			break;
-		default:
-			/* default to D horizontal size pages */
-			if (size[1] == 'v')
-			{
-				*x = scalefromdispunit(DSCHYSIZE - FRAMEWID, DISPUNITINCH);
-				*y = scalefromdispunit(DSCHXSIZE - FRAMEWID, DISPUNITINCH);
-			} else
-			{
-				*x = scalefromdispunit(DSCHXSIZE - FRAMEWID, DISPUNITINCH);
-				*y = scalefromdispunit(DSCHYSIZE - FRAMEWID, DISPUNITINCH);
-			}
-			break;
+		case 'a': wid = ASCHXSIZE;   hei = ASCHYSIZE;   break;
+		case 'b': wid = BSCHXSIZE;   hei = BSCHYSIZE;   break;
+		case 'c': wid = CSCHXSIZE;   hei = CSCHYSIZE;   break;
+		case 'd': wid = DSCHXSIZE;   hei = DSCHYSIZE;   break;
+		case 'e': wid = ESCHXSIZE;   hei = ESCHYSIZE;   break;
+		default:  wid = DSCHXSIZE;   hei = DSCHYSIZE;   break;
+	}
+	if (size[1] == 'v')
+	{
+		*x = scalefromdispunit(hei - FRAMEWID, DISPUNITINCH);
+		*y = scalefromdispunit(wid - FRAMEWID, DISPUNITINCH);
+	} else
+	{
+		*x = scalefromdispunit(wid - FRAMEWID, DISPUNITINCH);
+		*y = scalefromdispunit(hei - FRAMEWID, DISPUNITINCH);
 	}
 	return(0);
 }
@@ -1038,23 +1037,14 @@ INTSML framepolys(NODEPROTO *np)
 
 	size = db_getframedesc(np);
 	if (*size == 0) return(0);
-	switch (size[0])
+	if (size[1] == 'v')
 	{
-		case 'a':
-			xsections = ysections = 4;
-			break;
-		default:
-			/* default to D horizontal size pages */
-			if (size[1] == 'v')
-			{
-				xsections = 4;
-				ysections = 8;
-			} else
-			{
-				xsections = 8;
-				ysections = 4;
-			}
-			break;
+		xsections = 4;
+		ysections = 8;
+	} else
+	{
+		xsections = 8;
+		ysections = 4;
 	}
 
 	/* compute number of polygons */
@@ -1065,52 +1055,39 @@ INTSML framepolys(NODEPROTO *np)
 void framepoly(INTSML boxnum, POLYGON *poly, NODEPROTO *np)
 {
 	REGISTER INTSML pindex, xsections, ysections;
-	REGISTER INTBIG xsecsize, ysecsize, schxsize, schysize, aschxsize, aschysize,
-		dschxsize, dschysize, framewid, xlogobox, ylogobox;
+	REGISTER INTBIG xsecsize, ysecsize, schxsize, schysize, framewid, xlogobox, ylogobox;
+	float wid, hei;
 	REGISTER VARIABLE *var;
 	static char line[10];
 	static char ndstring[320];
 	char *size;
 
 	/* get true sizes */
-	aschxsize = scalefromdispunit(ASCHXSIZE, DISPUNITINCH);
-	aschysize = scalefromdispunit(ASCHYSIZE, DISPUNITINCH);
-	dschxsize = scalefromdispunit(DSCHXSIZE, DISPUNITINCH);
-	dschysize = scalefromdispunit(DSCHYSIZE, DISPUNITINCH);
-	framewid = scalefromdispunit(FRAMEWID, DISPUNITINCH);
-	xlogobox = scalefromdispunit(XLOGOBOX, DISPUNITINCH);
-	ylogobox = scalefromdispunit(YLOGOBOX, DISPUNITINCH);
 	size = db_getframedesc(np);
 	switch (size[0])
 	{
-		case 'a':
-			xsections = ysections = 4;
-			if (size[1] == 'v')
-			{
-				schxsize = aschysize - framewid;
-				schysize = aschxsize - framewid;
-			} else
-			{
-				schxsize = aschxsize - framewid;
-				schysize = aschysize - framewid;
-			}
-			break;
-		default:
-			/* default to D horizontal size pages */
-			if (size[1] == 'v')
-			{
-				xsections = 4;
-				ysections = 8;
-				schxsize = dschysize - framewid;
-				schysize = dschxsize - framewid;
-			} else
-			{
-				xsections = 8;
-				ysections = 4;
-				schxsize = dschxsize - framewid;
-				schysize = dschysize - framewid;
-			}
-			break;
+		case 'a': wid = ASCHXSIZE;   hei = ASCHYSIZE;   break;
+		case 'b': wid = BSCHXSIZE;   hei = BSCHYSIZE;   break;
+		case 'c': wid = CSCHXSIZE;   hei = CSCHYSIZE;   break;
+		case 'd': wid = DSCHXSIZE;   hei = DSCHYSIZE;   break;
+		case 'e': wid = ESCHXSIZE;   hei = ESCHYSIZE;   break;
+		default:  wid = DSCHXSIZE;   hei = DSCHYSIZE;   break;
+	}
+	framewid = scalefromdispunit(FRAMEWID, DISPUNITINCH);
+	xlogobox = scalefromdispunit(XLOGOBOX, DISPUNITINCH);
+	ylogobox = scalefromdispunit(YLOGOBOX, DISPUNITINCH);
+	if (size[1] == 'v')
+	{
+		xsections = 4;
+		ysections = 8;
+		schxsize = scalefromdispunit(hei, DISPUNITINCH) - framewid;
+		schysize = scalefromdispunit(wid, DISPUNITINCH) - framewid;
+	} else
+	{
+		xsections = 8;
+		ysections = 4;
+		schxsize = scalefromdispunit(wid, DISPUNITINCH) - framewid;
+		schysize = scalefromdispunit(hei, DISPUNITINCH) - framewid;
 	}
 
 	if (poly->limit < 4) (void)extendpolygon(poly, 4);
@@ -1351,7 +1328,6 @@ INTSML db_schematic_pages(NODEPROTO *np)
 
 char *db_getframedesc(NODEPROTO *np)
 {
-	static INTBIG pagesize = 0;
 	static char line[10];
 	REGISTER VARIABLE *var;
 	REGISTER char *pt;
@@ -1359,17 +1335,11 @@ char *db_getframedesc(NODEPROTO *np)
 	(void)strcpy(line, "");
 	if (np != NONODEPROTO)
 	{
-		if (pagesize == 0) pagesize = makekey("FACET_schematic_page_size");
-		var = getvalkey((INTBIG)np, VNODEPROTO, VSTRING, pagesize);
+		var = getvalkey((INTBIG)np, VNODEPROTO, VSTRING, el_schematic_page_size);
 		if (var != NOVARIABLE)
 		{
 			(void)strcpy(line, (char *)var->addr);
 			for(pt = line; *pt != 0; pt++) if (isupper(*pt)) *pt = tolower(*pt);
-		} else
-		{
-			/* on multipage schematic views, presume "d" default frame size */
-			if ((np->cellview->viewstate&MULTIPAGEVIEW) != 0)
-				(void)strcpy(line, "d");
 		}
 	}
 	return(line);
@@ -1417,6 +1387,9 @@ NODEPROTO *copynodeproto(NODEPROTO *fromnp, LIBRARY *tolib, char *toname)
 	{
 		if (ni->proto->primindex != 0) continue;
 
+		/* keep cross-library references */
+		if (ni->proto->cell->lib != fromnp->cell->lib) continue;
+
 		/* search for facet with same name and view in new library */
 		for(lnt = tolib->firstnodeproto; lnt != NONODEPROTO; lnt = lnt->nextnodeproto)
 			if (namesame(lnt->cell->cellname, ni->proto->cell->cellname) == 0 &&
@@ -1427,7 +1400,7 @@ NODEPROTO *copynodeproto(NODEPROTO *fromnp, LIBRARY *tolib, char *toname)
 			return(NONODEPROTO);
 		}
 
-		/* make sure there all used ports can be found on the uncopied facet */
+		/* make sure all used ports can be found on the uncopied facet */
 		for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
 		{
 			pp = pi->proto;
@@ -1476,13 +1449,17 @@ NODEPROTO *copynodeproto(NODEPROTO *fromnp, LIBRARY *tolib, char *toname)
 	for(ni = fromnp->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 	{
 		/* figure out nodeproto */
-		if (ni->proto->primindex != 0 || destlib == NOLIBRARY)
+		if (ni->proto->primindex != 0 || destlib == NOLIBRARY ||
+			ni->proto->cell->lib != fromnp->cell->lib)
 		{
-			/* primitive, or copying within a library: use same prototype */
+			/*
+			 * primitive, copying within a library, or already a cross-lib reference:
+			 * use same prototype
+			 */
 			lnt = ni->proto;
 		} else
 		{
-			/* crossing libraries: find equivalent in other library */
+			/* using facet from destination library: find it */
 
 			/* first find the cell by name */
 			for(c = tolib->firstcell; c != NOCELL; c = c->nextcell)
@@ -1778,7 +1755,7 @@ NODEINST *replacenodeinst(NODEINST *ni, NODEPROTO *np)
 	REGISTER VARIABLE *varold, *varnew;
 	NODEINST *endnodeinst[2];
 	PORTPROTO *endportproto[2];
-	REGISTER PORTARCINST *pi;
+	REGISTER PORTARCINST *pi, *nextpi;
 	REGISTER PORTEXPINST *pe;
 	REGISTER ARCINST *ai, *newai, *newai2, *aiswap;
 	REGISTER NODEPROTO *pinnp;
@@ -1911,9 +1888,11 @@ NODEINST *replacenodeinst(NODEINST *ni, NODEPROTO *np)
 	}
 
 	/* replace the arcs */
-	for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
+	for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = nextpi)
 	{
+		nextpi = pi->nextportarcinst;
 		ai = pi->conarcinst;
+		if ((ai->userbits&DEADA) != 0) continue;
 		for(i=0; i<2; i++)
 		{
 			if (ai->end[i].nodeinst == ni)
@@ -2280,7 +2259,11 @@ void changeallports(PORTPROTO *pp)
 			if (pe->proto != pp) continue;
 
 			/* change this port and recurse up the hierarchy */
-			pe->exportproto->userbits = pp->userbits;
+			if (pe->exportproto->userbits != pp->userbits)
+			{
+				setval((INTBIG)pe->exportproto, VPORTPROTO, "userbits",
+					pp->userbits, VINTEGER);
+			}
 			pe->exportproto->connects = pp->connects;
 			changeallports(pe->exportproto);
 		}
