@@ -38,17 +38,23 @@
 #include "efunction.h"
 #include <math.h>
 
-INTSML tech_realpolys;	/* polygon count without displayable variables */
+       INTSML      tech_realpolys;					/* polygon count without displayable variables */
 
 /* cached variables */
-static INTBIG *tech_node_widoff = 0;			/* cache for "nodesizeoffset" */
-static INTBIG *tech_layer_function = 0;			/* cache for "layerfunction" */
-static INTBIG *tech_layer_names = 0;			/* cache for "layername" */
-static INTBIG *tech_arc_widoff = 0;				/* cache for "arcwidthoffset" */
-static INTBIG *tech_drcmaxdistances = 0;		/* cache for "maxdrcsurround" */
-static INTBIG *tech_drcconndistance = 0;		/* cache for "drcmindistance" */
-static INTBIG *tech_drcuncondistance = 0;		/* cache for "drcmindistance" */
-static INTBIG *tech_drcminwidth = 0;			/* cache for "drcminwidth" */
+static INTBIG     *tech_node_widoff = 0;			/* cache for "nodesizeoffset" */
+static INTBIG     *tech_layer_function = 0;			/* cache for "layerfunction" */
+static INTBIG     *tech_layer_names = 0;			/* cache for "layername" */
+static INTBIG     *tech_arc_widoff = 0;				/* cache for "arcwidthoffset" */
+static INTBIG     *tech_drcmaxdistances = 0;		/* cache for "maxdrcsurround" */
+static INTBIG     *tech_drcconndistance = 0;		/* cache for "drcmindistance" */
+static INTBIG     *tech_drcuncondistance = 0;		/* cache for "drcmindistance" */
+static INTBIG     *tech_drcminwidth = 0;			/* cache for "drcminwidth" */
+static INTBIG      tech_techlayer3dheightkey;		/* cache for "TECH_layer_3dheight" */
+static INTBIG      tech_techlayer3dthicknesskey;	/* cache for "TECH_layer_3dthickness" */
+static TECHNOLOGY *tech_3dcurtech = NOTECHNOLOGY;
+static VARIABLE   *tech_3dcurthicknessvar;
+static VARIABLE   *tech_3dcurheightvar;
+
 
 #define NOROUTINELIST ((ROUTINELIST *)-1)
 
@@ -66,6 +72,7 @@ void tech_initlayerfunction(void);
 void tech_initlayername(void);
 void tech_initarcwidthoffset(void);
 INTBIG tech_getdrcmindistance(TECHNOLOGY*, INTSML, INTSML, INTSML);
+void tech_3ddefaultlayerheight(TECHNOLOGY *tech);
 
 /*
  * Routine to free all memory associated with this module.
@@ -1873,6 +1880,84 @@ INTBIG layerfunctionheight(INTBIG funct)
 	return(35);
 }
 
+INTSML get3dfactors(TECHNOLOGY *tech, INTBIG layer, INTBIG *height, INTBIG *thickness)
+{
+	static INTSML heightsetup = 0;
+	REGISTER TECHNOLOGY *itech;
+	REGISTER VARIABLE *varh, *vart;
+
+	/* make sure that every technology has layer height information */
+	if (heightsetup == 0)
+	{
+		heightsetup = 1;
+		tech_techlayer3dheightkey = makekey("TECH_layer_3dheight");
+		tech_techlayer3dthicknesskey = makekey("TECH_layer_3dthickness");
+		for(itech = el_technologies; itech != NOTECHNOLOGY; itech = itech->nexttechnology)
+		{
+			varh = getvalkey((INTBIG)itech, VTECHNOLOGY, VINTEGER|VISARRAY, tech_techlayer3dheightkey);
+			vart = getvalkey((INTBIG)itech, VTECHNOLOGY, VINTEGER|VISARRAY, tech_techlayer3dthicknesskey);
+			if (varh == NOVARIABLE || vart == NOVARIABLE)
+				tech_3ddefaultlayerheight(itech);
+		}
+	}
+
+	if (tech != tech_3dcurtech)
+	{
+		tech_3dcurtech = tech;
+		tech_3dcurheightvar = getvalkey((INTBIG)tech_3dcurtech, VTECHNOLOGY, VINTEGER|VISARRAY,
+			tech_techlayer3dheightkey);
+		tech_3dcurthicknessvar = getvalkey((INTBIG)tech_3dcurtech, VTECHNOLOGY, VINTEGER|VISARRAY,
+			tech_techlayer3dthicknesskey);
+		if (tech_3dcurheightvar == NOVARIABLE || tech_3dcurthicknessvar == NOVARIABLE)
+		{
+			tech_3dcurtech = NOTECHNOLOGY;
+			return(1);
+		}
+	}
+	if (layer < 0 || layer >= tech->layercount) return(1);
+
+	*thickness = ((INTBIG *)tech_3dcurthicknessvar->addr)[layer];
+	*height = ((INTBIG *)tech_3dcurheightvar->addr)[layer];
+	return(0);
+}
+
+void set3dheight(TECHNOLOGY *tech, INTBIG *depth)
+{
+	setvalkey((INTBIG)tech, VTECHNOLOGY, tech_techlayer3dheightkey, (INTBIG)depth,
+		VINTEGER|VISARRAY|(tech->layercount<<VLENGTHSH)|VDONTSAVE);
+	tech_3dcurtech = NOTECHNOLOGY;
+}
+
+void set3dthickness(TECHNOLOGY *tech, INTBIG *thickness)
+{
+	setvalkey((INTBIG)tech, VTECHNOLOGY, tech_techlayer3dthicknesskey, (INTBIG)thickness,
+		VINTEGER|VISARRAY|(tech->layercount<<VLENGTHSH)|VDONTSAVE);
+	tech_3dcurtech = NOTECHNOLOGY;
+}
+
+/*
+ * Routine to establish default layer height and thickness for
+ * technology "tech".
+ */
+void tech_3ddefaultlayerheight(TECHNOLOGY *tech)
+{
+	REGISTER INTBIG *layerheight, *layerthickness, i, funct;
+
+	layerheight = (INTBIG *)emalloc(tech->layercount * SIZEOFINTBIG, el_tempcluster);
+	layerthickness = (INTBIG *)emalloc(tech->layercount * SIZEOFINTBIG, el_tempcluster);
+	for(i=0; i<tech->layercount; i++)
+	{
+		funct = layerfunction(tech, (INTSML)i) & LFTYPE;
+		layerheight[i] = layerfunctionheight(funct);
+		layerthickness[i] = 0;
+		if (layeriscontact(funct) != 0) layerthickness[i] = 2;
+	}
+	set3dheight(tech, layerheight);
+	set3dthickness(tech, layerthickness);
+	efree((char *)layerheight);
+	efree((char *)layerthickness);
+}
+
 /*
  * routine to tell the minimum distance between layers "layer1" and "layer2" in
  * technology "tech".  If "connected" is zero, the two layers are not connected,
