@@ -61,10 +61,16 @@ static GRAPHICS tech_arrow = {LAYERO, FACETOUT, SOLIDC, SOLIDC,
 static GRAPHICS tech_vartxt = {LAYERO, FACETTXT, SOLIDC, SOLIDC,
 	{0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF}, NOVARIABLE, 0};
 
+static INTSML    tech_numvar;
+static VARIABLE *tech_firstvar;
+
 /* prototypes for local routines */
 void db_adjusttree(NODEPROTO*);
 void tech_shortenmostrans(NODEINST*, POLYGON*, TECH_POLYGON*, INTBIG, INTSML, TECH_PORTS*);
 INTSML db_stillinport(ARCINST*, INTSML, INTBIG, INTBIG);
+INTSML tech_displayableanyvars(void);
+VARIABLE *tech_filldisplayableanyvar(POLYGON *poly, INTBIG lx, INTBIG hx,
+	INTBIG ly, INTBIG hy, INTBIG addr, INTBIG type, TECHNOLOGY *tech);
 
 /******************** GENERAL ********************/
 
@@ -1148,18 +1154,9 @@ void tech_filltransport(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans
  */
 INTSML tech_displayablenvars(NODEINST *ni)
 {
-	REGISTER INTSML i, total;
-	REGISTER VARIABLE *var;
-
-	tech_ndisplayindex = tech_ndisplaysubindex = total = 0;
-	for(i = 0; i < ni->numvar; i++)
-	{
-		var = &ni->firstvar[i];
-		if ((var->type&VDISPLAY) == 0) continue;
-		if ((var->type&VISARRAY) == 0) total++; else
-			total += (INTSML)getlength(var);
-	}
-	return(total);
+	tech_numvar = ni->numvar;
+	tech_firstvar = ni->firstvar;
+	return(tech_displayableanyvars());
 }
 
 /*
@@ -1168,77 +1165,32 @@ INTSML tech_displayablenvars(NODEINST *ni)
  */
 VARIABLE *tech_filldisplayablenvar(NODEINST *ni, POLYGON *poly)
 {
-	REGISTER INTBIG descript, len, language;
-	REGISTER INTSML whichindex;
-	REGISTER VARIABLE *var;
-	VARIABLE retvar;
+	tech_numvar = ni->numvar;
+	tech_firstvar = ni->firstvar;
+	return(tech_filldisplayableanyvar(poly, ni->lowx, ni->highx,
+		ni->lowy, ni->highy, (INTBIG)ni, VNODEINST, whattech(ni->proto)));
+}
 
-	/* get next displayable variable */
-	poly->count = 1;
-	poly->xv[0] = (ni->lowx + ni->highx) / 2;
-	poly->yv[0] = (ni->lowy + ni->highy) / 2;
-	poly->style = CROSS;
-	for(;;)
-	{
-		if (tech_ndisplayindex >= ni->numvar)
-		{
-			descript = 0;
-			poly->string = "ERROR";
-			var = NOVARIABLE;
-			break;
-		}
-		var = &ni->firstvar[tech_ndisplayindex];
-		if ((var->type&VDISPLAY) != 0)
-		{
-			language = var->type & (VCODE1|VCODE2);
-			if (language != 0)
-			{
-				retvar.key = var->key;
-				retvar.type = var->type & ~(VCODE1|VCODE2);
-				retvar.textdescript = var->textdescript;
-				if (doquerry((char *)var->addr, language, retvar.type, &retvar.addr) == 0)
-					var = &retvar;
-			}
-			if ((var->type&VISARRAY) == 0)
-			{
-				tech_ndisplayindex++;
-				tech_ndisplaysubindex = 0;
-				if ((var->textdescript&VTPOSITION) == VTPOSBOXED)
-				{
-					makerectpoly(ni->lowx, ni->highx, ni->lowy, ni->highy, poly);
-					poly->style = CLOSED;
-				}
-				descript = var->textdescript;
-				poly->string = describevariable(var, -1, -1);
-				break;
-			}
-			len = getlength(var);
-			if (tech_ndisplaysubindex < len)
-			{
-				whichindex = tech_ndisplaysubindex++;
-				poly->string = describevariable(var, whichindex, -1);
-				descript = var->textdescript;
-				if ((var->textdescript&VTPOSITION) == VTPOSBOXED)
-				{
-					makerectpoly(ni->lowx, ni->highx, ni->lowy, ni->highy, poly);
-					poly->style = CLOSED;
-				} else
-				{
-					getdisparrayvarlinepos(ni->geom, el_curwindowpart, var,
-						whichindex, &poly->xv[0], &poly->yv[0], 0);
-				}
-				break;
-			}
-		}
-		tech_ndisplayindex++;
-		tech_ndisplaysubindex = 0;
-	}
+/*
+ * routine to compute the number of displayable polygons on nodeproto "np"
+ */
+INTSML tech_displayablefacetvars(NODEPROTO *np)
+{
+	tech_numvar = np->numvar;
+	tech_firstvar = np->firstvar;
+	return(tech_displayableanyvars());
+}
 
-	adjustdisoffset(ni->geom, poly, descript);
-	poly->desc = &tech_vartxt;
-	tech_vartxt.col = el_colfacettxt;
-	poly->layer = -1;
-	return(var);
+/*
+ * routine to fill polygon "poly" with the next displayable variable
+ * on nodeproto "np".  Returns the address of the variable just filled.
+ */
+VARIABLE *tech_filldisplayablefacetvar(NODEPROTO *np, POLYGON *poly)
+{
+	tech_numvar = np->numvar;
+	tech_firstvar = np->firstvar;
+	return(tech_filldisplayableanyvar(poly, np->lowx, np->highx,
+		np->lowy, np->highy, (INTBIG)np, VNODEPROTO, whattech(np)));
 }
 
 /******************** PORTINST DESCRIPTION ********************/
@@ -1324,6 +1276,31 @@ INTSML db_stillinport(ARCINST *ai, INTSML e, INTBIG x, INTBIG y)
 	return(0);
 }
 
+/*
+ * routine to compute the number of displayable polygons on portproto "pp"
+ */
+INTSML tech_displayableportvars(PORTPROTO *pp)
+{
+	tech_numvar = pp->numvar;
+	tech_firstvar = pp->firstvar;
+	return(tech_displayableanyvars());
+}
+
+/*
+ * routine to fill polygon "poly" with the next displayable variable
+ * on nodeproto "np".  Returns the address of the variable just filled.
+ */
+VARIABLE *tech_filldisplayableportvar(PORTPROTO *pp, POLYGON *poly)
+{
+	INTBIG x, y;
+
+	portposition(pp->subnodeinst, pp->subportproto, &x, &y);
+	tech_numvar = pp->numvar;
+	tech_firstvar = pp->firstvar;
+	return(tech_filldisplayableanyvar(poly, x, x,
+		y, y, (INTBIG)pp, VPORTPROTO, whattech(pp->parent)));
+}
+
 /******************** ARCINST DESCRIPTION ********************/
 
 /*
@@ -1657,12 +1634,9 @@ void tech_resetnegated(ARCINST *ai)
  */
 INTSML tech_displayableavars(ARCINST *ai)
 {
-	REGISTER INTSML i, total;
-
-	tech_adisplayindex = total = 0;
-	for(i = 0; i < ai->numvar; i++)
-		if ((ai->firstvar[i].type&VDISPLAY) != 0) total++;
-	return(total);
+	tech_numvar = ai->numvar;
+	tech_firstvar = ai->firstvar;
+	return(tech_displayableanyvars());
 }
 
 /*
@@ -1671,47 +1645,114 @@ INTSML tech_displayableavars(ARCINST *ai)
  */
 VARIABLE *tech_filldisplayableavar(ARCINST *ai, POLYGON *poly)
 {
-	REGISTER INTBIG descript, language;
+	tech_numvar = ai->numvar;
+	tech_firstvar = ai->firstvar;
+	return(tech_filldisplayableanyvar(poly, ai->geom->lowx, ai->geom->highx,
+		ai->geom->lowy, ai->geom->highy, (INTBIG)ai, VARCINST, ai->proto->tech));
+}
+
+/******************** SUPPORT ********************/
+
+/*
+ * routine to compute the number of displayable polygons in the pair "tech_numvar/
+ * tech_firstvar"
+ */
+INTSML tech_displayableanyvars(void)
+{
+	REGISTER INTSML i, total;
 	REGISTER VARIABLE *var;
-	VARIABLE retvar;
+
+	tech_ndisplayindex = tech_ndisplaysubindex = total = 0;
+	for(i = 0; i < tech_numvar; i++)
+	{
+		var = &tech_firstvar[i];
+		if ((var->type&VDISPLAY) == 0) continue;
+		if ((var->type&VISARRAY) == 0) total++; else
+			total += (INTSML)getlength(var);
+	}
+	return(total);
+}
+
+/*
+ * routine to fill polygon "poly" with the next displayable variable
+ * on nodeinst "ni".  Returns the address of the variable just filled.
+ */
+VARIABLE *tech_filldisplayableanyvar(POLYGON *poly, INTBIG lx, INTBIG hx,
+	INTBIG ly, INTBIG hy, INTBIG addr, INTBIG type, TECHNOLOGY *tech)
+{
+	REGISTER INTBIG descript, len, language;
+	REGISTER INTSML whichindex;
+	REGISTER VARIABLE *var;
+	REGISTER NODEINST *ni;
+	static VARIABLE retvar;
 
 	/* get next displayable variable */
+	poly->count = 1;
+	poly->xv[0] = (lx+hx)/2;
+	poly->yv[0] = (ly+hy)/2;
+	poly->style = CROSS;
 	for(;;)
 	{
-		if (tech_adisplayindex >= ai->numvar) { var = NOVARIABLE;   break; }
-		var = &ai->firstvar[tech_adisplayindex++];
-		if ((var->type&VDISPLAY) != 0) break;
-	}
-	if (var == NOVARIABLE)
-	{
-		poly->string = "ERROR";
-		descript = 0;
-	} else
-	{
-		language = var->type & (VCODE1|VCODE2);
-		if (language != 0)
+		if (tech_ndisplayindex >= tech_numvar)
 		{
-			retvar.key = var->key;
-			retvar.type = var->type & ~(VCODE1|VCODE2);
-			retvar.textdescript = var->textdescript;
-			if (doquerry((char *)var->addr, language, retvar.type, &retvar.addr) == 0)
-				var = &retvar;
+			descript = 0;
+			poly->string = "ERROR";
+			var = NOVARIABLE;
+			break;
 		}
-		poly->string = describevariable(var, -1, -1);
-		descript = var->textdescript;
+		var = &tech_firstvar[tech_ndisplayindex];
+		if ((var->type&VDISPLAY) != 0)
+		{
+			language = var->type & (VCODE1|VCODE2);
+			if (language != 0)
+			{
+				retvar.key = var->key;
+				retvar.type = var->type & ~(VCODE1|VCODE2);
+				retvar.textdescript = var->textdescript;
+				if (doquerry((char *)var->addr, language, retvar.type, &retvar.addr) == 0)
+					var = &retvar;
+			}
+			if ((var->type&VISARRAY) == 0)
+			{
+				tech_ndisplayindex++;
+				tech_ndisplaysubindex = 0;
+				if ((var->textdescript&VTPOSITION) == VTPOSBOXED)
+				{
+					makerectpoly(lx, hx, ly, hy, poly);
+					poly->style = CLOSED;
+				}
+				descript = var->textdescript;
+				poly->string = describevariable(var, -1, -1);
+				break;
+			}
+			len = getlength(var);
+			if (tech_ndisplaysubindex < len)
+			{
+				whichindex = tech_ndisplaysubindex++;
+				poly->string = describevariable(var, whichindex, -1);
+				descript = var->textdescript;
+				if ((var->textdescript&VTPOSITION) == VTPOSBOXED)
+				{
+					makerectpoly(lx, hx, ly, hy, poly);
+					poly->style = CLOSED;
+				} else
+				{
+					if (type == VNODEINST)
+					{
+						/* multiline variables only work on nodes! */
+						ni = (NODEINST *)addr;
+						getdisparrayvarlinepos((INTBIG)ni, VNODEINST, tech, el_curwindowpart, var,
+							whichindex, &poly->xv[0], &poly->yv[0], 0);
+					}
+				}
+				break;
+			}
+		}
+		tech_ndisplayindex++;
+		tech_ndisplaysubindex = 0;
 	}
 
-	/* position the variable */
-	if ((descript&VTPOSITION) == VTPOSBOXED)
-	{
-		makerectpoly(ai->geom->lowx, ai->geom->highx, ai->geom->lowy, ai->geom->highy, poly);
-	} else
-	{
-		poly->xv[0] = (ai->end[0].xpos + ai->end[1].xpos) / 2;
-		poly->yv[0] = (ai->end[0].ypos + ai->end[1].ypos) / 2;
-		poly->count = 1;
-	}
-	adjustdisoffset(ai->geom, poly, descript);
+	adjustdisoffset(addr, type, tech, poly, descript);
 	poly->desc = &tech_vartxt;
 	tech_vartxt.col = el_colfacettxt;
 	poly->layer = -1;
