@@ -792,6 +792,86 @@ char *nextaids(void)
 	return(el_aids[db_poscomcomp++].aidname);
 }
 
+/*
+ * routines to do command completion on facet names
+ */
+static NODEPROTO *db_posnodeprotos;
+INTSML topoffacets(char **c)
+{
+	REGISTER char *pt;
+	REGISTER LIBRARY *lib;
+
+	/* by default, assume the current library */
+	db_posnodeprotos = el_curlib->firstnodeproto;
+
+	/* see if a library specification was given */
+	for(pt = *c; *pt != 0; pt++) if (*pt == ':') break;
+	if (*pt == ':')
+	{
+		*pt = 0;
+		lib = getlibrary(*c);
+		*pt++ = ':';
+		if (lib != NOLIBRARY)
+		{
+			*c = pt;
+			db_posnodeprotos = lib->firstnodeproto;
+		}
+	}
+	return(1);
+}
+char *nextfacets(void)
+{
+	REGISTER char *ret;
+
+	if (db_posnodeprotos != NONODEPROTO)
+	{
+		ret = describenodeproto(db_posnodeprotos);
+		db_posnodeprotos = db_posnodeprotos->nextnodeproto;
+		return(ret);
+	}
+	return(0);
+}
+
+/*
+ * routines to do command completion on arc names
+ */
+static ARCPROTO *db_posarcs;
+INTSML topofarcs(char **c)
+{
+	REGISTER char *pt;
+	REGISTER TECHNOLOGY *t;
+
+	/* by default, assume the current technology */
+	db_posarcs = el_curtech->firstarcproto;
+
+	/* see if a technology specification was given */
+	for(pt = *c; *pt != 0; pt++) if (*pt == ':') break;
+	if (*pt == ':')
+	{
+		*pt = 0;
+		t = gettechnology(*c);
+		*pt++ = ':';
+		if (t != NOTECHNOLOGY)
+		{
+			*c = pt;
+			db_posarcs = t->firstarcproto;
+		}
+	}
+	return(1);
+}
+char *nextarcs(void)
+{
+	REGISTER char *ret;
+
+	if (db_posarcs != NOARCPROTO)
+	{
+		ret = describearcproto(db_posarcs);
+		db_posarcs = db_posarcs->nextarcproto;
+		return(ret);
+	}
+	return(0);
+}
+
 /*
  * routines to do command completion on network names
  */
@@ -1454,6 +1534,28 @@ char *describevariable(VARIABLE *var, INTSML aindex, INTSML purpose)
 	}
 
 	err = initinfstr();
+
+	/* add variable name if requested */
+	if ((var->type&VDISPLAY) != 0 &&
+		(var->textdescript&VTDISPLAYPART) == VTDISPLAYNAMEVALUE &&
+		(var->type&VISARRAY) == 0)
+	{
+		char *name;
+
+		name = makename(var->key);
+		if (strncmp(name, "ATTR_", 5) == 0)
+		{
+			name += 5;
+		} else if (strncmp(name, "ATTRP_", 6) == 0)
+		{
+			len = strlen(name);
+			for(i=len-1; i>=0; i--) if (name[i] == '_') break;
+			name += i;
+		}
+		(void)addstringtoinfstr(name);
+		(void)addtoinfstr('=');
+	}
+
 	if ((var->type & (VCODE1|VCODE2)) != 0)
 	{
 		/* special case for code: it is a string, the type applies to the result */
@@ -2226,7 +2328,8 @@ char *timetostring(UINTBIG curtime)
  * Routine to parse the time in "curtime" and convert it to the year (since 1900),
  * month (0 = January), and day of month.
  */
-void parsetime(UINTBIG curtime, INTSML *year, INTSML *month, INTSML *mday)
+void parsetime(UINTBIG curtime, INTSML *year, INTSML *month, INTSML *mday,
+	INTSML *hour, INTSML *minute, INTSML *second)
 {
 	struct tm *tm;
 
@@ -2235,6 +2338,9 @@ void parsetime(UINTBIG curtime, INTSML *year, INTSML *month, INTSML *mday)
 	*month = tm->tm_mon;
 	*mday = tm->tm_mday;
 	*year = tm->tm_year;
+	*hour = tm->tm_hour;
+	*minute = tm->tm_min;
+	*second = tm->tm_sec;
 }
 
 /******************** SUBROUTINES FOR FILE I/O ****************/
@@ -2247,52 +2353,55 @@ struct
 	char *longname;
 } db_filetypeinfo[] =
 {
-	{"tab",  "*.tab",		"actel",     "QuickPartTable"},		/*  01: FILETYPEACTELTAB */
-	{"arr",  "*.arr",		"arrfile",   "Array"},				/*  02: FILETYPEARRAY */
-	{"elib", "*.elib",      "blib",      "Binary library"},		/*  03: FILETYPEBLIB */
-	{"",     "*.*",			"cadrc",     "Startup"},			/*  04: FILETYPECADRC */
-	{"cif",  "*.cif",		"cif",       "CIF"},				/*  05: FILETYPECIF */
-	{"map",  "*.map",		"color",     "Color map"},			/*  06: FILETYPECOLORMAP */
-	{"def",  "*.def",		"def",       "DEF"},				/*  07: FILETYPEDEF */
-	{"rul",  "*.map",		"drac",      "Dracula"},			/* 010: FILETYPEDRAC */
-	{"dxf",  "*.dxf",		"dxf",       "AutoCAD DXF"},		/* 011: FILETYPEDXF */
-	{"edif", "*.edif;*.ed?","edif",      "EDIF"},				/* 012: FILETYPEEDIF */
-	{"sim",  "*.sim",		"esim",      "ESIM netlist"},		/* 013: FILETYPEESIM */
-	{"fpga", "*.fpga",		"fpga",      "FPGA Architecture"},	/* 014: FILETYPEFPGA */
-	{"gds",  "*.gds",		"gds",       "GDS II"},				/* 015: FILETYPEGDS */
-	{"help", "*.help",		"help",      "Help"},				/* 016: FILETYPEHELP */
-	{"hpgl", "*.hpgl",		"hpgl",      "HPGL"},				/* 017: FILETYPEHPGL */
-	{"tr0",  "*.tr0",		"hspiceout", "HSPICE output"},		/* 020: FILETYPEHSPICEOUT */
-	{"l",    "*.l",			"l",         "L"},					/* 021: FILETYPEL */
-	{"lef",  "*.lef",		"lef",       "LEF"},				/* 022: FILETYPELEF */
-	{"log",  "*.log",		"log",       "Log"},				/* 023: FILETYPELOG */
-	{"mac",  "*.mac",		"macro",     "Macro package"},		/* 024: FILETYPEMACRO */
-	{"ntk",  "*.ntk",		"mossim",    "MOSSIM netlist"},		/* 025: FILETYPEMOSSIM */
-	{"net",  "*.net",		"netlisp",   "NetLisp netlist"},	/* 026: FILETYPENETLISP */
-	{"news", "*.news",		"news",      "News"},				/* 027: FILETYPENEWS */
-	{"pal",  "*.pal",		"pal",       "Abel PAL"},			/* 030: FILETYPEPAL */
-	{"",     "*.*",			"platab",    "PLA table"},			/* 031: FILETYPEPLATAB */
-	{"",     "*.*",			"proj",      "Project"},			/* 032: FILETYPEPROJ */
-	{"",     "*.*",			"projusers", "Project Users"},		/* 033: FILETYPEPROJUSERS */
-	{"ps",   "*.ps;*.eps",	"ps",        "PostScript"},			/* 034: FILETYPEPS */
-	{"net",  "*.net",		"als",       "ALS netlist"},		/* 035: FILETYPEALS */
-	{"vec",  "*.vec",		"alsvec",    "ALS vector"},			/* 036: FILETYPEALSVEC */
-	{"sci",  "*.sci",		"quisc",     "QUISC netlist"},		/* 037: FILETYPEQUISC */
-	{"sim",  "*.sim",		"rsim",      "RSIM netlist"},		/* 040: FILETYPERSIM */
-	{"",     "*.*",			"sctab",     "QUISC table"},		/* 041: FILETYPESCTAB */
-	{"",     "*.*",			"scsim",     "QUISC simulation"},	/* 042: FILETYPESCSIM */
-	{"sdf",  "*.sdf",		"sdf",       "SDF"},				/* 043: FILETYPESDF */
-	{"sil",  "*.sil",		"silos",     "SILOS netlist"},		/* 044: FILETYPESILOS */
-	{"il",   "*.il",		"skill",     "SKILL commands"},		/* 045: FILETYPESKILL */
-	{"spi",  "*.spi;*.spo",	"spice",     "SPICE netlist"},		/* 046: FILETYPESPICE */
-	{"spo",  "*.spo",		"spiceout",  "SPICE output"},		/* 047: FILETYPESPICEOUT */
-	{"sue",  "*.sue",		"sue",       "SUE"},				/* 050: FILETYPESUE */
-	{"tdl",  "*.tdl",		"tegas",     "TEGAS netlist"},		/* 051: FILETYPETEGAS */
-	{"",     "*.*",			"tegastab",  "TEGAS table"},		/* 052: FILETYPETEGASTAB */
-	{"",     "*.*",			"text",      "Text"},				/* 053: FILETYPETEXT */
-	{"txt",  "*.txt",		"tlib",      "Text library"},		/* 054: FILETYPETLIB */
-	{"v",    "*.v;*.ver",	"verilog",   "Verilog"},			/* 055: FILETYPEVERILOG */
-	{"vhdl", "*.vhdl;*.vhd","vhdl",      "VHDL"},				/* 056: FILETYPEVHDL */
+	{"tab",  "*.tab",		"actel",     "QuickPartTable"},		/* FILETYPEACTELTAB */
+	{"arr",  "*.arr",		"arrfile",   "Array"},				/* FILETYPEARRAY */
+	{"elib", "*.elib",      "blib",      "Binary library"},		/* FILETYPEBLIB */
+	{"",     "*.*",			"cadrc",     "Startup"},			/* FILETYPECADRC */
+	{"cif",  "*.cif",		"cif",       "CIF"},				/* FILETYPECIF */
+	{"map",  "*.map",		"color",     "Color map"},			/* FILETYPECOLORMAP */
+	{"def",  "*.def",		"def",       "DEF"},				/* FILETYPEDEF */
+	{"rul",  "*.map",		"drac",      "Dracula"},			/* FILETYPEDRAC */
+	{"dxf",  "*.dxf",		"dxf",       "AutoCAD DXF"},		/* FILETYPEDXF */
+	{"txt",  "*.txt",		"eagle",     "Eagle netlist"},		/* FILETYPEEAGLE */
+	{"edif", "*.edif;*.ed?","edif",      "EDIF"},				/* FILETYPEEDIF */
+	{"sim",  "*.sim",		"esim",      "ESIM netlist"},		/* FILETYPEESIM */
+	{"inp",  "*.inp",		"fasthenry", "FastHenry netlist"},	/* FILETYPEFASTHENRY */
+	{"fpga", "*.fpga",		"fpga",      "FPGA Architecture"},	/* FILETYPEFPGA */
+	{"gds",  "*.gds",		"gds",       "GDS II"},				/* FILETYPEGDS */
+	{"help", "*.help",		"help",      "Help"},				/* FILETYPEHELP */
+	{"hpgl", "*.hpgl",		"hpgl",      "HPGL"},				/* FILETYPEHPGL */
+	{"tr0",  "*.tr0",		"hspiceout", "HSPICE output"},		/* FILETYPEHSPICEOUT */
+	{"l",    "*.l",			"l",         "L"},					/* FILETYPEL */
+	{"lef",  "*.lef",		"lef",       "LEF"},				/* FILETYPELEF */
+	{"log",  "*.log",		"log",       "Log"},				/* FILETYPELOG */
+	{"mac",  "*.mac",		"macro",     "Macro package"},		/* FILETYPEMACRO */
+	{"ntk",  "*.ntk",		"mossim",    "MOSSIM netlist"},		/* FILETYPEMOSSIM */
+	{"net",  "*.net",		"netlisp",   "NetLisp netlist"},	/* FILETYPENETLISP */
+	{"news", "*.news",		"news",      "News"},				/* FILETYPENEWS */
+	{"asc",  "*.asc",		"pad",       "PADS netlist"},		/* FILETYPEPADS */
+	{"pal",  "*.pal",		"pal",       "Abel PAL"},			/* FILETYPEPAL */
+	{"",     "*.*",			"platab",    "PLA table"},			/* FILETYPEPLATAB */
+	{"",     "*.*",			"proj",      "Project"},			/* FILETYPEPROJ */
+	{"",     "*.*",			"projusers", "Project Users"},		/* FILETYPEPROJUSERS */
+	{"ps",   "*.ps;*.eps",	"ps",        "PostScript"},			/* FILETYPEPS */
+	{"net",  "*.net",		"als",       "ALS netlist"},		/* FILETYPEALS */
+	{"vec",  "*.vec",		"alsvec",    "ALS vector"},			/* FILETYPEALSVEC */
+	{"sci",  "*.sci",		"quisc",     "QUISC netlist"},		/* FILETYPEQUISC */
+	{"sim",  "*.sim",		"rsim",      "RSIM netlist"},		/* FILETYPERSIM */
+	{"",     "*.*",			"sctab",     "QUISC table"},		/* FILETYPESCTAB */
+	{"",     "*.*",			"scsim",     "QUISC simulation"},	/* FILETYPESCSIM */
+	{"sdf",  "*.sdf",		"sdf",       "SDF"},				/* FILETYPESDF */
+	{"sil",  "*.sil",		"silos",     "SILOS netlist"},		/* FILETYPESILOS */
+	{"il",   "*.il",		"skill",     "SKILL commands"},		/* FILETYPESKILL */
+	{"spi",  "*.spi;*.spo",	"spice",     "SPICE netlist"},		/* FILETYPESPICE */
+	{"spo",  "*.spo",		"spiceout",  "SPICE output"},		/* FILETYPESPICEOUT */
+	{"sue",  "*.sue",		"sue",       "SUE"},				/* FILETYPESUE */
+	{"tdl",  "*.tdl",		"tegas",     "TEGAS netlist"},		/* FILETYPETEGAS */
+	{"",     "*.*",			"tegastab",  "TEGAS table"},		/* FILETYPETEGASTAB */
+	{"",     "*.*",			"text",      "Text"},				/* FILETYPETEXT */
+	{"txt",  "*.txt",		"tlib",      "Text library"},		/* FILETYPETLIB */
+	{"v",    "*.v;*.ver",	"verilog",   "Verilog"},			/* FILETYPEVERILOG */
+	{"vhdl", "*.vhdl;*.vhd","vhdl",      "VHDL"},				/* FILETYPEVHDL */
 	{0, 0, 0}
 };
 
