@@ -40,7 +40,7 @@
 #include "usr.h"
 
 /* the DRC aid table */
-static COMCOMP drccp = {NOKEYWORD, us_topoffacets, us_nextparse, NOPARAMS,
+static COMCOMP drccp = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
 	NOBACKUP, INPUTOPT, " \t", "facet to be rechecked", "check current facet"};
 static KEYWORD drcnotopt[] =
 {
@@ -50,9 +50,9 @@ static KEYWORD drcnotopt[] =
 };
 static COMCOMP drcnotp = {drcnotopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
 	NOBACKUP, INPUTOPT, " \t", "Design Rule Checker negative action", 0};
-static COMCOMP drcfip = {NOKEYWORD, us_topoffacets, us_nextparse, NOPARAMS,
+static COMCOMP drcfip = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
 	NOBACKUP, INPUTOPT, " \t", "facet to ignored for short check DRC", 0};
-static COMCOMP drcfup = {NOKEYWORD, us_topoffacets, us_nextparse, NOPARAMS,
+static COMCOMP drcfup = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
 	NOBACKUP, INPUTOPT, " \t", "facet to un-ignored for short check DRC", 0};
 static KEYWORD drcfopt[] =
 {
@@ -65,13 +65,16 @@ static COMCOMP drcfp = {drcfopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
 	NOBACKUP, INPUTOPT, " \t", "Short Detecting Design Rule Checker action", 0};
 static KEYWORD drcnopt[] =
 {
-	{"run",             0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"use-dialog",      0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"no-dialog",       0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"verbose",         0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"current-error",   0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"next-error",      0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"previous-error",  0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"run",               0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"use-dialog",        0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"no-dialog",         0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"verbose",           0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"current-error",     0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"next-error",        0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"previous-error",    0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"reset-check-dates", 0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"just-1-error",      0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"all-errors",        0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	TERMKEY
 };
 static COMCOMP drcbatchp = {drcnopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
@@ -94,12 +97,12 @@ COMCOMP dr_drcp = {drcopt, NOTOPLIST, NONEXTLIST, NOPARAMS, NOBACKUP,
 /* dcheck modules */
 #define	NODCHECK ((DCHECK *)-1)
 
-#define	NODENEW    1			/* this nodeinst was created */
-#define	ARCNEW     2			/* this arcinst was created */
-#define	NODEMOD    3			/* this nodeinst was modified */
-#define	ARCMOD     4			/* this arcinst was modified */
-#define	NODEKILL   5			/* this nodeinst was deleted */
-#define	ARCKILL    6			/* this arcinst was deleted */
+#define	NODENEW    1					/* this nodeinst was created */
+#define	ARCNEW     2					/* this arcinst was created */
+#define	NODEMOD    3					/* this nodeinst was modified */
+#define	ARCMOD     4					/* this arcinst was modified */
+#define	NODEKILL   5					/* this nodeinst was deleted */
+#define	ARCKILL    6					/* this arcinst was deleted */
 
 typedef struct Idcheck
 {
@@ -109,74 +112,70 @@ typedef struct Idcheck
 		NODEINST *ni;
 		ARCINST  *ai;
 		void     *blind;
-	} entryaddr;					/* object being checked */
-	struct Idcheck *nextdcheck;		/* next in list */
+	} entryaddr;						/* object being checked */
+	struct Idcheck *nextdcheck;			/* next in list */
 } DCHECK;
-static DCHECK *dr_firstdcheck;
-static DCHECK *dr_dcheckfree;
+static DCHECK   *dr_firstdcheck;
+static DCHECK   *dr_dcheckfree;
 
 /*
  * The meaning of "aid:drc.DRC_pointout" is as follows:
- * 0: don't identify errors
- * 1: identify errors
- */
-INTBIG dr_pointout;					/* key for aid:drc.DRC_pointout */
-
-/* */
-INTBIG dr_incrementalon;			/* key for aid:drc.DRC_incrementalon */
-
-/*
- * The meaning of "dr_pointedout" is as follows:
  *  0: not identifying errors
  *  1: errors being identified
  * >1: some errors have already been identified
  * -2: do not identify any more errors in this session
  */
-INTBIG dr_pointedout;
-static INTBIG dr_verbose;			/* key for aid:drc.DRC_verbose */
-       INTBIG dr_ignore_list;		/* key for aid:drc.DRC_ignore_list */
+static INTBIG    dr_pointoutkey;		/* key for "DRC_pointout" */
+static INTBIG    dr_pointout;			/* cached value of "DRC_pointout" */
+static INTBIG    dr_verbosekey;			/* key for "DRC_verbose" */
+       INTBIG    dr_ignore_listkey;		/* key for "DRC_ignore_list" */
+       INTBIG    dr_lastgooddrckey;		/* key for "DRC_last_good_drc" */
+       INTBIG    dr_incrementalonkey;	/* key for "DRC_incrementalon" */
+static INTBIG    dr_optionskey;			/* key for "DRC_options" */
+       INTBIG    dr_options;			/* cached value of "DRC_options" */
+static INTBIG    dr_pseudonet = 1;
+static NODEINST *dr_tinynodeinst;
+static GEOM     *dr_tinyobj;
+       AIDENTRY *dr_aid;				/* the DRC aid object */
 
 struct plist
 {
-	POLYGON **polygons;				/* polygons to be checked on node or arc */
+	POLYGON **polygons;					/* polygons to be checked on node or arc */
 	INTSML polylistsize;				/* size of above list */
 } dr_polylist, dr_subpolylist;
 
-static INTBIG dr_pseudonet = 1;
-static NODEINST *dr_tinynodeinst;
-static GEOM *dr_tinyobj;
-AIDENTRY *dr_aid;					/* the DRC aid object */
-
 /* DRC Options */
 DIALOGITEM dr_optionsdialogitems[] =
 {
- /*  1 */ {0, {272,116,296,180}, BUTTON, "OK"},
- /*  2 */ {0, {272,16,296,80}, BUTTON, "Cancel"},
- /*  3 */ {0, {296,360,312,415}, EDITTEXT, ""},
- /*  4 */ {0, {68,32,84,172}, CHECK, "Highlights errors"},
+ /*  1 */ {0, {276,116,300,180}, BUTTON, "OK"},
+ /*  2 */ {0, {276,16,300,80}, BUTTON, "Cancel"},
+ /*  3 */ {0, {296,354,312,403}, EDITTEXT, ""},
+ /*  4 */ {0, {56,32,72,192}, CHECK, "Highlights errors"},
  /*  5 */ {0, {32,312,48,506}, POPUP, "from layer"},
  /*  6 */ {0, {32,220,48,307}, MESSAGE, "For Layer:"},
  /*  7 */ {0, {104,220,287,486}, SCROLL, ""},
  /*  8 */ {0, {56,244,72,360}, MESSAGE, "Minimum Width:"},
- /*  9 */ {0, {296,220,312,354}, MESSAGE, "Minimum Distance:"},
+ /*  9 */ {0, {296,220,312,350}, MESSAGE, "Minimum Distance:"},
  /* 10 */ {0, {84,360,100,498}, RADIO, "Connected To:"},
  /* 11 */ {0, {84,220,100,355}, RADIO, "Unconnected To:"},
- /* 12 */ {0, {216,32,232,160}, BUTTON, "Edit Rules Deck"},
- /* 13 */ {0, {36,32,52,117}, CHECK, "On"},
+ /* 12 */ {0, {232,32,248,160}, BUTTON, "Edit Rules Deck"},
+ /* 13 */ {0, {32,32,48,117}, CHECK, "On"},
  /* 14 */ {0, {0,208,312,209}, DIVIDELINE, "item"},
  /* 15 */ {0, {56,368,72,423}, EDITTEXT, ""},
- /* 16 */ {0, {296,424,312,512}, BUTTON, "Factory Reset"},
+ /* 16 */ {0, {296,412,312,512}, BUTTON, "Factory Reset"},
  /* 17 */ {0, {8,332,24,490}, MESSAGE|INACTIVE, ""},
  /* 18 */ {0, {8,240,24,327}, MESSAGE, "Technology:"},
- /* 19 */ {0, {140,32,156,172}, CHECK, "Use dialog"},
+ /* 19 */ {0, {120,32,136,192}, CHECK, "Use dialog"},
  /* 20 */ {0, {8,8,24,169}, MESSAGE, "Incremental DRC:"},
- /* 21 */ {0, {96,4,97,208}, DIVIDELINE, ""},
- /* 22 */ {0, {108,8,124,169}, MESSAGE, "Hierarchical DRC:"},
- /* 23 */ {0, {172,4,173,208}, DIVIDELINE, ""},
- /* 24 */ {0, {184,8,200,169}, MESSAGE, "Dracula DRC Interface:"},
- /* 25 */ {0, {244,4,245,208}, DIVIDELINE, ""}
+ /* 21 */ {0, {84,4,85,208}, DIVIDELINE, ""},
+ /* 22 */ {0, {96,8,112,169}, MESSAGE, "Hierarchical DRC:"},
+ /* 23 */ {0, {196,4,197,208}, DIVIDELINE, ""},
+ /* 24 */ {0, {208,8,224,169}, MESSAGE, "Dracula DRC Interface:"},
+ /* 25 */ {0, {260,4,261,208}, DIVIDELINE, ""},
+ /* 26 */ {0, {168,32,184,192}, BUTTON, "Clear valid DRC dates"},
+ /* 27 */ {0, {144,32,160,192}, CHECK, "Just 1 error per facet"}
 };
-DIALOG dr_optionsdialog = {{50,75,371,596}, "DRC Options", 25, dr_optionsdialogitems};
+DIALOG dr_optionsdialog = {{50,75,371,596}, "DRC Options", 27, dr_optionsdialogitems};
 
 /* Hierarchical DRC */
 DIALOGITEM dr_hierdialoglist[] =
@@ -304,6 +303,8 @@ void dr_unqueuecheck(LIBRARY *lib)
 
 void dr_init(INTBIG *argc, char *argv[], AIDENTRY *thisaid)
 {
+	REGISTER VARIABLE *var;
+
 	/* only initialize during pass 1 */
 	if (thisaid == NOAID || thisaid == 0) return;
 
@@ -313,13 +314,19 @@ void dr_init(INTBIG *argc, char *argv[], AIDENTRY *thisaid)
 	dr_polylist.polylistsize = dr_subpolylist.polylistsize = 0;
 	dr_dcheckfree = NODCHECK;
 	dr_firstdcheck = NODCHECK;
-	dr_ignore_list = makekey("DRC_ignore_list");
-	dr_pointout = makekey("DRC_pointout");
-	dr_incrementalon = makekey("DRC_incrementalon");
-	dr_verbose = makekey("DRC_verbose");
+
+	dr_pointoutkey = makekey("DRC_pointout");
+	dr_verbosekey = makekey("DRC_verbose");
+	dr_ignore_listkey = makekey("DRC_ignore_list");
+	dr_lastgooddrckey = makekey("DRC_last_good_drc");
+	dr_incrementalonkey = makekey("DRC_incrementalon");
+	dr_optionskey = makekey("DRC_options");
+	var = getvalkey((INTBIG)dr_aid, VAID, VINTEGER, dr_optionskey);
+	if (var == NOVARIABLE) dr_options = 0; else
+		dr_options = var->addr;
 	nextvarchangequiet();
-	(void)setvalkey((INTBIG)dr_aid, VAID, dr_pointout, 0, VINTEGER|VDONTSAVE);
-	dr_pointedout = 0;
+	(void)setvalkey((INTBIG)dr_aid, VAID, dr_pointoutkey, 0, VINTEGER|VDONTSAVE);
+	dr_pointout = 0;
 	DiaDeclareHook("drcopt", &dr_drcp, dr_optionsdlog);
 }
 
@@ -345,7 +352,7 @@ void dr_done(void)
 INTSML dr_set(INTSML count, char *par[])
 {
 	REGISTER INTSML l, negate;
-	REGISTER INTBIG i, options, errs;
+	REGISTER INTBIG i, errs;
 	REGISTER char *pp;
 	REGISTER FILE *f;
 	REGISTER NODEPROTO *np;
@@ -357,11 +364,11 @@ INTSML dr_set(INTSML count, char *par[])
 
 	if (count == 0)
 	{
-		var = getvalkey((INTBIG)dr_aid, VAID, VINTEGER, dr_verbose);
+		var = getvalkey((INTBIG)dr_aid, VAID, VINTEGER, dr_verbosekey);
 		if (var == NOVARIABLE) i = 0; else i = var->addr;
 		if (i == 0) ttyputmsg("Design rule checker is brief"); else
 			ttyputmsg("Design rule checker is verbose");
-		var = getvalkey((INTBIG)dr_aid, VAID, VINTEGER, dr_pointout);
+		var = getvalkey((INTBIG)dr_aid, VAID, VINTEGER, dr_pointoutkey);
 		if (var == NOVARIABLE) i = 0; else i = var->addr;
 		if (i == 0) ttyputmsg("Errors will not be highlighted"); else
 			ttyputmsg("Errors will be highlighted");
@@ -391,9 +398,9 @@ INTSML dr_set(INTSML count, char *par[])
 		if (np == NONODEPROTO) return(1);
 
 		/* erase all ignore lists on all objects */
-		var = getvalkey((INTBIG)np, VNODEPROTO, VGEOM|VISARRAY, dr_ignore_list);
+		var = getvalkey((INTBIG)np, VNODEPROTO, VGEOM|VISARRAY, dr_ignore_listkey);
 		if (var != NOVARIABLE)
-			(void)delvalkey((INTBIG)np, VNODEPROTO, dr_ignore_list);
+			(void)delvalkey((INTBIG)np, VNODEPROTO, dr_ignore_listkey);
 
 		ttyputmsg("Ignored violations turned back on.");
 		return(0);
@@ -503,9 +510,7 @@ INTSML dr_set(INTSML count, char *par[])
 			}
 
 			/* see if a dialog should be displayed */
-			var = getval((INTBIG)dr_aid, VAID, VINTEGER, "DRC_options");
-			if (var == NOVARIABLE) options = 0; else options = var->addr;
-			if ((options&DRCHIERDIALOG) != 0)
+			if ((dr_options&DRCHIERDIALOG) != 0)
 			{
 				dr_batchdlog(np);
 			} else
@@ -533,22 +538,32 @@ INTSML dr_set(INTSML count, char *par[])
 		}
 		if (namesamen(pp, "use-dialog", l) == 0)
 		{
-			var = getval((INTBIG)dr_aid, VAID, VINTEGER, "DRC_options");
-			if (var == NOVARIABLE) options = 0; else options = var->addr;
-			(void)setval((INTBIG)dr_aid, VAID, "DRC_options",
-				options | DRCHIERDIALOG, VINTEGER);
+			(void)setvalkey((INTBIG)dr_aid, VAID, dr_optionskey,
+				dr_options | DRCHIERDIALOG, VINTEGER);
 			ttyputmsg("Hierarchical DRC will use a dialog interface");
 			return(0);
 		}
 		if (namesamen(pp, "no-dialog", l) == 0)
 		{
-			var = getval((INTBIG)dr_aid, VAID, VINTEGER, "DRC_options");
-			if (var == NOVARIABLE) options = 0; else options = var->addr;
-			(void)setval((INTBIG)dr_aid, VAID, "DRC_options",
-				options & ~DRCHIERDIALOG, VINTEGER);
+			(void)setvalkey((INTBIG)dr_aid, VAID, dr_optionskey,
+				dr_options & ~DRCHIERDIALOG, VINTEGER);
 			ttyputmsg("Hierarchical DRC will not use a dialog interface");
 			return(0);
 		}
+		if (namesamen(pp, "just-1-error", l) == 0)
+		{
+			(void)setvalkey((INTBIG)dr_aid, VAID, dr_optionskey,
+				dr_options | DRCFIRSTERROR, VINTEGER);
+			ttyputmsg("Hierarchical DRC will stop after first error in a facet");
+			return(0);
+		}
+		if (namesamen(pp, "all-errors", l) == 0)
+		{
+			(void)setvalkey((INTBIG)dr_aid, VAID, dr_optionskey,
+				dr_options & ~DRCFIRSTERROR, VINTEGER);
+			ttyputmsg("Hierarchical DRC will find all errors in a facet");
+			return(0);
+		}
 		if (namesamen(pp, "verbose", l) == 0)
 			return(drcb_debug());
 		if (namesamen(pp, "next-error", l) == 0)
@@ -566,6 +581,12 @@ INTSML dr_set(INTSML count, char *par[])
 			ttyputmsg("%s", drcb_curr_error(1, 0, 0));
 			return(0);
 		}
+		if (namesamen(pp, "reset-check-dates", l) == 0)
+		{
+			drcb_reset_dates();
+			ttyputmsg("All date information about valid DRC is reset");
+			return(0);
+		}
 		ttyputerr("Bad TELLAID DRC BATCH option: %s", pp);
 		return(1);
 	}
@@ -588,7 +609,7 @@ INTSML dr_set(INTSML count, char *par[])
 				ttyputerr("...now reissue the DRC command");
 				return(1);
 			}
-			return(drc_flatwrite(getcurfacet()));
+			return(dr_flatwrite(getcurfacet()));
 		}
 		if (namesamen(pp, "ignore", l) == 0)
 		{
@@ -597,7 +618,7 @@ INTSML dr_set(INTSML count, char *par[])
 				ttyputerr("Usage: tellaid drc shortcheck ignore FACET");
 				return(1);
 			}
-			return(drc_flatignore(par[2]));
+			return(dr_flatignore(par[2]));
 		}
 		if (namesamen(pp, "unignore", l) == 0)
 		{
@@ -606,7 +627,7 @@ INTSML dr_set(INTSML count, char *par[])
 				ttyputerr("Usage: tellaid drc shortcheck unignore FACET");
 				return(1);
 			}
-			return(drc_flatunignore(par[2]));
+			return(dr_flatunignore(par[2]));
 		}
 		ttyputerr("Invalid shortcheck DRC option: %s", pp);
 		return(1);
@@ -616,12 +637,12 @@ INTSML dr_set(INTSML count, char *par[])
 	{
 		if (negate == 0)
 		{
-			(void)setvalkey((INTBIG)dr_aid, VAID, dr_verbose, 1, VINTEGER|VDONTSAVE);
+			(void)setvalkey((INTBIG)dr_aid, VAID, dr_verbosekey, 1, VINTEGER|VDONTSAVE);
 			ttyputverbose("Design rule checker verbose");
 		} else
 		{
-			if (getvalkey((INTBIG)dr_aid, VAID, VINTEGER, dr_verbose) != NOVARIABLE)
-				(void)delvalkey((INTBIG)dr_aid, VAID, dr_verbose);
+			if (getvalkey((INTBIG)dr_aid, VAID, VINTEGER, dr_verbosekey) != NOVARIABLE)
+				(void)delvalkey((INTBIG)dr_aid, VAID, dr_verbosekey);
 			ttyputverbose("Design rule checker brief");
 		}
 		return(0);
@@ -631,11 +652,11 @@ INTSML dr_set(INTSML count, char *par[])
 	{
 		if (negate == 0)
 		{
-			(void)setvalkey((INTBIG)dr_aid, VAID, dr_pointout, 1, VINTEGER);
+			(void)setvalkey((INTBIG)dr_aid, VAID, dr_pointoutkey, 1, VINTEGER);
 			ttyputverbose("Errors will be identified");
 		} else
 		{
-			(void)setvalkey((INTBIG)dr_aid, VAID, dr_pointout, 0, VINTEGER);
+			(void)setvalkey((INTBIG)dr_aid, VAID, dr_pointoutkey, 0, VINTEGER);
 			ttyputverbose("Errors not identified");
 		}
 		return(0);
@@ -657,9 +678,9 @@ void dr_examinenodeproto(NODEPROTO *np)
 	if ((net_aid->aidstate&AIDON) == 0) return;
 
 	/* setup for identifying errors */
-	var = getvalkey((INTBIG)dr_aid, VAID, VINTEGER, dr_pointout);
-	if (var != NOVARIABLE) dr_pointedout = var->addr; else
-		dr_pointedout = 0;
+	var = getvalkey((INTBIG)dr_aid, VAID, VINTEGER, dr_pointoutkey);
+	if (var != NOVARIABLE) dr_pointout = var->addr; else
+		dr_pointout = 0;
 
 #if 0
 	/* now check every object */
@@ -714,9 +735,9 @@ void dr_slice(void)
 	}
 
 	/* setup for identifying errors */
-	var = getvalkey((INTBIG)dr_aid, VAID, VINTEGER, dr_pointout);
-	if (var != NOVARIABLE) dr_pointedout = var->addr; else
-		dr_pointedout = 0;
+	var = getvalkey((INTBIG)dr_aid, VAID, VINTEGER, dr_pointoutkey);
+	if (var != NOVARIABLE) dr_pointout = var->addr; else
+		dr_pointout = 0;
 
 	/* now check the objects */
 	nomore = 0;
@@ -741,7 +762,7 @@ void dr_slice(void)
 	(void)stopping("DRC");
 
 	/* restore highlighting features */
-	i = abs(dr_pointedout);
+	i = abs(dr_pointout);
 	if (i > 1) ttyputverbose("DRC done");
 #else
 	REGISTER DCHECK *d, *nextd;
@@ -751,7 +772,7 @@ void dr_slice(void)
 	REGISTER VARIABLE *var;
 
 	/* see if the tool should be off */
-	var = getvalkey((INTBIG)dr_aid, VAID, VINTEGER, dr_incrementalon);
+	var = getvalkey((INTBIG)dr_aid, VAID, VINTEGER, dr_incrementalonkey);
 	if (var != NOVARIABLE && var->addr == 0)
 	{
 		aidturnoff(dr_aid, 0);
@@ -835,14 +856,14 @@ void dr_reporterrors(void)
 	GEOM *geom1, *geom2;
 
 	/* setup for identifying errors */
-	var = getvalkey((INTBIG)dr_aid, VAID, VINTEGER, dr_pointout);
-	if (var != NOVARIABLE) dr_pointedout = var->addr; else
-		dr_pointedout = 0;
+	var = getvalkey((INTBIG)dr_aid, VAID, VINTEGER, dr_pointoutkey);
+	if (var != NOVARIABLE) dr_pointout = var->addr; else
+		dr_pointout = 0;
 
 	errs = drcb_errorcount();
 	for(i=0; i<errs; i++)
 	{
-		if (dr_pointedout <= 0)
+		if (dr_pointout <= 0)
 		{
 			pt = drcb_next_error(0, 0, 0);
 			ttyputerr("%s", pt);
@@ -850,20 +871,20 @@ void dr_reporterrors(void)
 		{
 			/* save highlight features if this is the first error */
 			pt = drcb_next_error(1, &geom1, &geom2);
-			dr_pointedout++;
+			dr_pointout++;
 			answer = dr_errordlog(pt);
 			if (answer == 0) continue;
 			len = strlen(answer);
 			if (namesamen(answer, "quit-checking", len) == 0) break;
 			if (namesamen(answer, "continue-silently", len) == 0)
 			{
-				dr_pointedout = -2;
+				dr_pointout = -2;
 				continue;
 			}
 			if (namesamen(answer, "highlight-off", len) == 0)
 			{
-				dr_pointedout = 0;
-				setvalkey((INTBIG)dr_aid, VAID, dr_pointout, 0, VINTEGER|VDONTSAVE);
+				dr_pointout = 0;
+				setvalkey((INTBIG)dr_aid, VAID, dr_pointoutkey, 0, VINTEGER|VDONTSAVE);
 				continue;
 			}
 			if (namesamen(answer, "drc-off", len) == 0)
@@ -883,7 +904,7 @@ void dr_reporterrors(void)
 	}
 
 	/* restore highlighting features */
-	i = abs(dr_pointedout);
+	i = abs(dr_pointout);
 	if (i > 1) ttyputverbose("DRC done");
 }
 
@@ -922,6 +943,26 @@ void dr_eraselibrary(LIBRARY *lib)
 	dr_unqueuecheck(lib);
 }
 
+void dr_newvariable(INTBIG addr, INTBIG type, INTBIG key, INTBIG newtype)
+{
+	REGISTER VARIABLE *var;
+
+	if (key == dr_optionskey)
+	{
+		var = getvalkey((INTBIG)dr_aid, VAID, VINTEGER, dr_optionskey);
+		if (var != NOVARIABLE) dr_options = var->addr;
+	}
+}
+
+void dr_readlibrary(LIBRARY *lib)
+{
+	REGISTER VARIABLE *var;
+
+	/* recache DRC variables */
+	var = getvalkey((INTBIG)dr_aid, VAID, VINTEGER, dr_optionskey);
+	if (var != NOVARIABLE) dr_options = var->addr;
+}
+
 /******************** SUPPORT ROUTINES ********************/
 
 /*
@@ -1257,7 +1298,7 @@ INTSML dr_check(INTSML layer1, NETWORK *net1, GEOM *pos1, POLYGON *poly1, INTSML
 			tech2 = pos2->entryaddr.ni->proto->tech;
 
 	/* add in verbosity if requested */
-	var = getvalkey((INTBIG)dr_aid, VAID, VINTEGER, dr_verbose);
+	var = getvalkey((INTBIG)dr_aid, VAID, VINTEGER, dr_verbosekey);
 	if (var != NOVARIABLE && var->addr != 0)
 	{
 		if (net1 == NONETWORK || net2 == NONETWORK || net1 == net2)
@@ -1303,7 +1344,7 @@ INTSML dr_check(INTSML layer1, NETWORK *net1, GEOM *pos1, POLYGON *poly1, INTSML
 		}
 	}
 
-	if (dr_pointedout <= 0)
+	if (dr_pointout <= 0)
 	{
 		ttyputerr("%s", returninfstr());
 	} else
@@ -1314,20 +1355,20 @@ INTSML dr_check(INTSML layer1, NETWORK *net1, GEOM *pos1, POLYGON *poly1, INTSML
 		(void)askaid(us_aid, "show-area", lx1,hx1,ly1,hy1,np);
 		(void)askaid(us_aid, "show-area", lx2,hx2,ly2,hy2,np);
 		flushscreen();
-		dr_pointedout++;
+		dr_pointout++;
 		answer = dr_errordlog(returninfstr());
 		if (answer == 0) return(1);
 		len = strlen(answer);
 		if (namesamen(answer, "quit-checking", len) == 0) return(-1);
 		if (namesamen(answer, "continue-silently", len) == 0)
 		{
-			dr_pointedout = -2;
+			dr_pointout = -2;
 			return(1);
 		}
 		if (namesamen(answer, "highlight-off", len) == 0)
 		{
-			dr_pointedout = 0;
-			setvalkey((INTBIG)dr_aid, VAID, dr_pointout, 0, VINTEGER|VDONTSAVE);
+			dr_pointout = 0;
+			setvalkey((INTBIG)dr_aid, VAID, dr_pointoutkey, 0, VINTEGER|VDONTSAVE);
 			return(1);
 		}
 		if (namesamen(answer, "drc-off", len) == 0)
@@ -1421,13 +1462,13 @@ void dr_setignore(GEOM *pos1, GEOM *pos2)
 	REGISTER NODEPROTO *np;
 
 	np = geomparent(pos1);
-	var = getvalkey((INTBIG)np, VNODEPROTO, VGEOM|VISARRAY, dr_ignore_list);
+	var = getvalkey((INTBIG)np, VNODEPROTO, VGEOM|VISARRAY, dr_ignore_listkey);
 
 	/* if the list is empty: create it with one entry */
 	if (var == NOVARIABLE)
 	{
 		dummylist[0] = pos1;   dummylist[1] = pos2;
-		(void)setvalkey((INTBIG)np, VNODEPROTO, dr_ignore_list, (INTBIG)dummylist,
+		(void)setvalkey((INTBIG)np, VNODEPROTO, dr_ignore_listkey, (INTBIG)dummylist,
 			VGEOM|VISARRAY|(2<<VLENGTHSH)|VDONTSAVE);
 		return;
 	}
@@ -1453,7 +1494,7 @@ void dr_setignore(GEOM *pos1, GEOM *pos2)
 	/* update the list */
 	for(i=0; i<len; i++) ignorelist[i] = ((GEOM **)var->addr)[i];
 	ignorelist[len] = pos1;   ignorelist[len+1] = pos2;
-	(void)setvalkey((INTBIG)np, VNODEPROTO, dr_ignore_list, (INTBIG)ignorelist,
+	(void)setvalkey((INTBIG)np, VNODEPROTO, dr_ignore_listkey, (INTBIG)ignorelist,
 		(INTBIG)(VGEOM|VISARRAY|((len+2)<<VLENGTHSH)|VDONTSAVE));
 	efree((char *)ignorelist);
 }
@@ -1469,7 +1510,7 @@ void dr_unignore(GEOM *pos)
 	REGISTER NODEPROTO *np;
 
 	np = geomparent(pos);
-	var = getvalkey((INTBIG)np, VNODEPROTO, VGEOM|VISARRAY, dr_ignore_list);
+	var = getvalkey((INTBIG)np, VNODEPROTO, VGEOM|VISARRAY, dr_ignore_listkey);
 
 	/* if the list is empty there is nothing to do */
 	if (var == NOVARIABLE) return;
@@ -1498,9 +1539,9 @@ void dr_unignore(GEOM *pos)
 	}
 
 	/* set the list back in place */
-	if (pt > 0) (void)setvalkey((INTBIG)np, VNODEPROTO, dr_ignore_list,
+	if (pt > 0) (void)setvalkey((INTBIG)np, VNODEPROTO, dr_ignore_listkey,
 		(INTBIG)ignorelist, (INTBIG)(VGEOM|VISARRAY|(pt<<VLENGTHSH)|VDONTSAVE)); else
-			(void)delvalkey((INTBIG)np, VNODEPROTO, dr_ignore_list);
+			(void)delvalkey((INTBIG)np, VNODEPROTO, dr_ignore_listkey);
 	efree((char *)ignorelist);
 }
 
@@ -1626,7 +1667,7 @@ char *dr_describeerror(INTSML layer1, GEOM *pos1, INTSML layer2, GEOM *pos2)
 
 	/* first see if this is to be ignored */
 	np = geomparent(pos1);
-	var = getvalkey((INTBIG)np, VNODEPROTO, VGEOM|VISARRAY, dr_ignore_list);
+	var = getvalkey((INTBIG)np, VNODEPROTO, VGEOM|VISARRAY, dr_ignore_listkey);
 	if (var != NOVARIABLE)
 	{
 		len = getlength(var);
@@ -1888,6 +1929,8 @@ void dr_batchdlog(NODEPROTO *facet)
  * Factor Reset            = 16 (button)
  * Technology name         = 17 (stat text)
  * Hierarchical dialog     = 19 (check)
+ * Clear valid DRC dates   = 26 (button)
+ * Just 1 error per facet  = 27 (check)
  */
 void dr_optionsdlog(void)
 {
@@ -1895,7 +1938,7 @@ void dr_optionsdlog(void)
 		higherrors, i, l, *conlist, *unconlist, numentries, dist,
 		layer1, layer2, temp, dindex, *minwidth, *list;
 	REGISTER char **layernames, *pt, *qual;
-	INTSML dummy;
+	INTSML dummy, clearvaliddrcdates;
 	char header[200], *dummyfile[1];
 	REGISTER VARIABLE *var;
 	REGISTER WINDOWPART *w;
@@ -1954,29 +1997,35 @@ void dr_optionsdlog(void)
 
 	DiaSetText(17, el_curtech->techname);
 	DiaSetControl(11, 1);
-	var = getvalkey((INTBIG)dr_aid, VAID, VINTEGER, dr_pointout);
+	var = getvalkey((INTBIG)dr_aid, VAID, VINTEGER, dr_pointoutkey);
 	if (var == NOVARIABLE) higherrors = 0; else higherrors = var->addr;
 	if (higherrors != 0) DiaSetControl(4, 1);
 
-	var = getval((INTBIG)dr_aid, VAID, VINTEGER, "DRC_options");
-	if (var == NOVARIABLE) options = 0; else options = var->addr;
-	if ((options&DRCHIERDIALOG) != 0) DiaSetControl(19, 1);
+	if ((dr_options&DRCHIERDIALOG) != 0) DiaSetControl(19, 1);
+	if ((dr_options&DRCFIRSTERROR) != 0) DiaSetControl(27, 1);
 
 	if ((dr_aid->aidstate & AIDON) != 0) DiaSetControl(13, 1);
 	dr_loaddrcdialog(unconlist, conlist, minwidth);
 	dr_loaddrcdistance(unconlist, conlist);
 	conchanged = unconchanged = minwidchanged = 0;
+	clearvaliddrcdates = 0;
 
 	/* loop until done */
 	for(;;)
 	{
 		itemHit = DiaNextHit();
 		if (itemHit == OK || itemHit == CANCEL || itemHit == 12) break;
-		if (itemHit == 4 || itemHit == 13 || itemHit == 19)
+		if (itemHit == 4 || itemHit == 13 || itemHit == 19 || itemHit == 27)
 		{
 			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
 			continue;
 		}
+		if (itemHit == 26)
+		{
+			clearvaliddrcdates = 1;
+			DiaDimItem(26);
+			continue;
+		}
 		if (itemHit == 3)
 		{
 			/* typed into distance field */
@@ -2081,12 +2130,12 @@ void dr_optionsdlog(void)
 	{
 		if (DiaGetControl(4) != 0) higherrors = 1; else
 			higherrors = 0;
-		(void)setvalkey((INTBIG)dr_aid, VAID, dr_pointout,
+		(void)setvalkey((INTBIG)dr_aid, VAID, dr_pointoutkey,
 			higherrors, VINTEGER);
 		options = 0;
 		if (DiaGetControl(19) != 0) options |= DRCHIERDIALOG;
-		(void)setval((INTBIG)dr_aid, VAID, "DRC_options",
-			options, VINTEGER);
+		if (DiaGetControl(27) != 0) options |= DRCFIRSTERROR;
+		(void)setvalkey((INTBIG)dr_aid, VAID, dr_optionskey, options, VINTEGER);
 		if (unconchanged != 0)
 		{
 			setval((INTBIG)el_curtech, VTECHNOLOGY, "DRC_min_unconnected_distances",
@@ -2110,12 +2159,15 @@ void dr_optionsdlog(void)
 		if (DiaGetControl(13) != 0)
 		{
 			if ((dr_aid->aidstate & AIDON) == 0) aidturnon(dr_aid, 0);
-			setvalkey((INTBIG)dr_aid, VAID, dr_incrementalon, 1, VINTEGER);
+			setvalkey((INTBIG)dr_aid, VAID, dr_incrementalonkey, 1, VINTEGER);
 		} else
 		{
 			if ((dr_aid->aidstate & AIDON) != 0) aidturnoff(dr_aid, 0);
-			setvalkey((INTBIG)dr_aid, VAID, dr_incrementalon, 0, VINTEGER);
+			setvalkey((INTBIG)dr_aid, VAID, dr_incrementalonkey, 0, VINTEGER);
 		}
+
+		/* clear valid DRC dates if requested */
+		if (clearvaliddrcdates != 0) drcb_reset_dates();
 	}
 	DiaDoneDialog();
 	efree((char *)unconlist);
