@@ -36,31 +36,33 @@
 #include "efunction.h"
 #include "egraphics.h"
 
-#define MAX_DRC_FLATIGNORE 100			/* max facets to ignore */
+#define MAX_DRC_FLATIGNORE 100				/* max facets to ignore */
 #define	FLATDRCNAME        "ffindshort"		/* fast short program */
 #define	SFLATDRCNAME       "findshort"		/* slow short program */
-#define	SORTNAME           "sort"		/* sort program */
-#define MAXSHORTLAYER	   9			/* Max. layer index  */
+#define	SORTNAME           "sort"			/* sort program */
+#define MAXSHORTLAYER	   9				/* Max. layer index  */
 
-static NODEPROTO *drc_flatignored[MAX_DRC_FLATIGNORE];
-static INTBIG drc_index;
-static INTSML drc_nflat_ignored = 0;
-static INTBIG drc_flat_boxcount;
-static FILE *drc_file;
-static TECHNOLOGY *drc_maintech;
-char flatdrcfile[132], yminfile[132], ymaxfile[132];
+static NODEPROTO  *dr_flatignored[MAX_DRC_FLATIGNORE];
+static INTBIG      dr_index;
+static INTSML      dr_nflat_ignored = 0;
+static INTBIG      dr_flat_boxcount;
+static FILE       *dr_file;
+static TECHNOLOGY *dr_maintech;
+static char        dr_flatdrcfile[132];
+static char        dr_yminfile[132];
+static char        dr_ymaxfile[132];
 
 /* prototypes for local routines */
-INTSML drc_readshorterrors(INTBIG);
-void drc_flatprint(NODEPROTO*, XARRAY);
-void drc_transistor_hack(POLYGON*, NODEINST*);
-void drc_flatdesc_poly(POLYGON*, INTBIG, XARRAY, TECHNOLOGY*);
-void drc_flatprop(ARCINST*, INTBIG);
+INTSML dr_readshorterrors(INTBIG);
+void   dr_flatprint(NODEPROTO*, XARRAY);
+void   dr_transistor_hack(POLYGON*, NODEINST*);
+void   dr_flatdesc_poly(POLYGON*, INTBIG, XARRAY, TECHNOLOGY*);
+void   dr_flatprop(ARCINST*, INTBIG);
 
 /*
  * routine to flag facet "facet" as ignorable for flat DRC
  */
-INTSML drc_flatignore(char *facet)
+INTSML dr_flatignore(char *facet)
 {
 	INTSML i;
 	REGISTER NODEPROTO *np;
@@ -76,14 +78,14 @@ INTSML drc_flatignore(char *facet)
 		ttyputerr("Can only ignore facets, not primitives");
 		return(1);
 	}
-	for(i=0; i<drc_nflat_ignored; i++) if (drc_flatignored[i] == np)
+	for(i=0; i<dr_nflat_ignored; i++) if (dr_flatignored[i] == np)
 		return(0);
-	if (drc_nflat_ignored == MAX_DRC_FLATIGNORE)
+	if (dr_nflat_ignored == MAX_DRC_FLATIGNORE)
 	{
 		ttyputerr("Too many facets");
 		return(1);
 	}
-	drc_flatignored[drc_nflat_ignored++] = np;
+	dr_flatignored[dr_nflat_ignored++] = np;
 	ttyputmsg("Facet %s will be ignored from flat DRC", describenodeproto(np));
 	return(0);
 }
@@ -91,7 +93,7 @@ INTSML drc_flatignore(char *facet)
 /*
  * routine to flag facet "facet" as not ignorable for flat DRC
  */
-INTSML drc_flatunignore(char *facet)
+INTSML dr_flatunignore(char *facet)
 {
 	INTSML i;
 	REGISTER NODEPROTO *np;
@@ -107,28 +109,28 @@ INTSML drc_flatunignore(char *facet)
 		ttyputerr("Can only unignore facets, not primitives");
 		return(1);
 	}
-	for(i=0; i<drc_nflat_ignored && drc_flatignored[i] != np; i++) ;
-	if (i == drc_nflat_ignored)
+	for(i=0; i<dr_nflat_ignored && dr_flatignored[i] != np; i++) ;
+	if (i == dr_nflat_ignored)
 	{
 		ttyputerr("Not being ignored");
 		return(1);
 	}
-	for(; i<drc_nflat_ignored; i++)
-		drc_flatignored[i-1] = drc_flatignored[i];
-	drc_nflat_ignored--;
+	for(; i<dr_nflat_ignored; i++)
+		dr_flatignored[i-1] = dr_flatignored[i];
+	dr_nflat_ignored--;
 	ttyputmsg("Facet %s will be included in flat DRC", describenodeproto(np));
 	return(0);
 }
 
-INTSML drc_flatwrite(NODEPROTO *simnp)
+INTSML dr_flatwrite(NODEPROTO *simnp)
 {
-	REGISTER INTBIG drc_process, save1, save2;
+	REGISTER INTBIG dr_process, save1, save2;
 	REGISTER INTSML ret=1;
 	REGISTER PORTPROTO *pp, *spp;
 	REGISTER TECHNOLOGY *tech;
 	REGISTER VARIABLE *var;
 	char p1[20], p2[20];
-	int drc_fromshorts[2];
+	int dr_fromshorts[2];
 	char *flatdrcloc, *sflatdrcloc, *execpars[10];
 
 	if (simnp == NONODEPROTO)
@@ -136,30 +138,30 @@ INTSML drc_flatwrite(NODEPROTO *simnp)
 		ttyputerr("No facet to flatten for DRC");
 		return(1);
 	}
-	drc_maintech = whattech(simnp);
+	dr_maintech = whattech(simnp);
 
 	/* create output file names */
-	(void)sprintf(flatdrcfile, "%s", simnp->cell->cellname);
-	(void)strcat(flatdrcfile, ".drc");
+	(void)sprintf(dr_flatdrcfile, "%s", simnp->cell->cellname);
+	(void)strcat(dr_flatdrcfile, ".drc");
 
-	(void)sprintf(yminfile, "%s", simnp->cell->cellname);
-	(void)strcat(yminfile, "_miny.drc");
+	(void)sprintf(dr_yminfile, "%s", simnp->cell->cellname);
+	(void)strcat(dr_yminfile, "_miny.drc");
 
-	(void)sprintf(ymaxfile, "%s", simnp->cell->cellname);
-	(void)strcat(ymaxfile, "_maxy.drc");
+	(void)sprintf(dr_ymaxfile, "%s", simnp->cell->cellname);
+	(void)strcat(dr_ymaxfile, "_maxy.drc");
 
 	/* create the file */
-	drc_file = xcreate(flatdrcfile, FILETYPETEXT, 0, 0);
-	if (drc_file == NULL)
+	dr_file = xcreate(dr_flatdrcfile, FILETYPETEXT, 0, 0);
+	if (dr_file == NULL)
 	{
-		ttyputerr("Cannot write %s", flatdrcfile);
+		ttyputerr("Cannot write %s", dr_flatdrcfile);
 		return(1);
 	}
 	ttyputmsg("Writing flattened %s circuit into '%s'...",
-		drc_maintech->techname, flatdrcfile);
+		dr_maintech->techname, dr_flatdrcfile);
 
 	/* re-assign net-list values, starting at each port of the top facet */
-	drc_index = 1;
+	dr_index = 1;
 	for(pp=simnp->firstportproto; pp!=NOPORTPROTO; pp=pp->nextportproto)
 	{
 		for(spp=simnp->firstportproto; spp!=pp; spp=spp->nextportproto)
@@ -172,7 +174,7 @@ INTSML drc_flatwrite(NODEPROTO *simnp)
 		}
 
 		/* assign a new net number if the loop terminated normally */
-		if (spp == pp) pp->temp2 = drc_index++;
+		if (spp == pp) pp->temp2 = dr_index++;
 	}
 
 	/* initialize cache of CIF layer information */
@@ -183,17 +185,17 @@ INTSML drc_flatwrite(NODEPROTO *simnp)
 	}
 
 	/* write the flattened geometry */
-	drc_flat_boxcount = 0;
+	dr_flat_boxcount = 0;
 	begintraversehierarchy();
-	drc_flatprint(simnp, el_matid);
+	dr_flatprint(simnp, el_matid);
 
 	/* clean up the file */
-	xclose(drc_file);
+	xclose(dr_file);
 
 	if (stopping("DRC") != 0) return(0);
 
 	/* no point continuing if nothing to do */
-	if (drc_flat_boxcount == 0)
+	if (dr_flat_boxcount == 0)
 	{
 		ttyputmsg("Zero boxes written");
 		ttyputmsg("DRC short detection complete");
@@ -201,24 +203,24 @@ INTSML drc_flatwrite(NODEPROTO *simnp)
 	}
 
 	/* setup communication with fast short finding program */
-	ttyputmsg("%ld boxes written, now attempting fast short detection...", drc_flat_boxcount);
-	(void)epipe(drc_fromshorts);
-	drc_process = efork();
-	if (drc_process == 1)
+	ttyputmsg("%ld boxes written, now attempting fast short detection...", dr_flat_boxcount);
+	(void)epipe(dr_fromshorts);
+	dr_process = efork();
+	if (dr_process == 1)
 	{
 		ttyputerr("Shortcheck cannot run on this machine");
 		return(0);
 	}
-	if (drc_process == 0)
+	if (dr_process == 0)
 	{
-		save1 = channelreplacewithchannel(1, drc_fromshorts[1]);
-		save2 = channelreplacewithchannel(2, drc_fromshorts[1]);
+		save1 = channelreplacewithchannel(1, dr_fromshorts[1]);
+		save2 = channelreplacewithchannel(2, dr_fromshorts[1]);
 		(void)sprintf(p1, "%ld", el_curtech->deflambda);
-		(void)sprintf(p2, "%ld", drc_flat_boxcount);
+		(void)sprintf(p2, "%ld", dr_flat_boxcount);
 		if ((flatdrcloc = egetenv("FLATDRCLOC")) == NULL)
 			flatdrcloc = FLATDRCLOC;
 		execpars[0] = FLATDRCNAME;
-		execpars[1] = flatdrcfile;
+		execpars[1] = dr_flatdrcfile;
 		execpars[2] = p1;
 		execpars[3] = p2;
 		execpars[4] = 0;
@@ -230,13 +232,13 @@ INTSML drc_flatwrite(NODEPROTO *simnp)
 	}
 
 	/* close the file descriptor so the pipe will terminate when done */
-	eclose(drc_fromshorts[1]);
+	eclose(dr_fromshorts[1]);
 
 	/* run the fast short detection program */
-	ret = drc_readshorterrors((INTBIG)drc_fromshorts[0]);
-	eclose(drc_fromshorts[0]);
-	ekill(drc_process);
-	ewait(drc_process);
+	ret = dr_readshorterrors((INTBIG)dr_fromshorts[0]);
+	eclose(dr_fromshorts[0]);
+	ekill(dr_process);
+	ewait(dr_process);
 
 	/* if this program ran correctly, stop now */
 	if (ret == 0)
@@ -247,49 +249,49 @@ INTSML drc_flatwrite(NODEPROTO *simnp)
 
 	/* setup for slow short detection */
 	ttyputmsg("Too many boxes: disk version must be run");
-	ttyputmsg("  Interim step: sorting by minimum Y into '%s'...", yminfile);
-	if ((drc_process = efork()) == 0)
+	ttyputmsg("  Interim step: sorting by minimum Y into '%s'...", dr_yminfile);
+	if ((dr_process = efork()) == 0)
 	{
 		execpars[0] = SORTNAME;
 		execpars[1] = "+0n";
 		execpars[2] = "-1";
 		execpars[3] = "-o";
-		execpars[4] = yminfile;
-		execpars[5] = flatdrcfile;
+		execpars[4] = dr_yminfile;
+		execpars[5] = dr_flatdrcfile;
 		execpars[6] = 0;
 		eexec(SORTLOC, execpars);
 		ttyputerr("Cannot run sort");
 		exit(1);
 	}
-	ewait(drc_process);
-	ttyputmsg("  Interim step: sorting by maximum Y into '%s'...", ymaxfile);
-	if ((drc_process = efork()) == 0)
+	ewait(dr_process);
+	ttyputmsg("  Interim step: sorting by maximum Y into '%s'...", dr_ymaxfile);
+	if ((dr_process = efork()) == 0)
 	{
 		execpars[0] = SORTNAME;
 		execpars[1] = "+1n";
 		execpars[2] = "-2";
 		execpars[3] = "-o";
-		execpars[4] = ymaxfile;
-		execpars[5] = flatdrcfile;
+		execpars[4] = dr_ymaxfile;
+		execpars[5] = dr_flatdrcfile;
 		execpars[6] = 0;
 		eexec(SORTLOC, execpars);
 		ttyputerr("Cannot run sort");
 		exit(1);
 	}
-	ewait(drc_process);
+	ewait(dr_process);
 
 	ttyputmsg("Now invoking slow short detection...");
-	(void)epipe(drc_fromshorts);
-	if ((drc_process = efork()) == 0)
+	(void)epipe(dr_fromshorts);
+	if ((dr_process = efork()) == 0)
 	{
-		save1 = channelreplacewithchannel(1, drc_fromshorts[1]);
-		save2 = channelreplacewithchannel(2, drc_fromshorts[1]);
+		save1 = channelreplacewithchannel(1, dr_fromshorts[1]);
+		save2 = channelreplacewithchannel(2, dr_fromshorts[1]);
 		(void)sprintf(p1, "%ld", el_curtech->deflambda);
 		if ((sflatdrcloc = egetenv("SFLATDRCLOC")) == NULL)
 			sflatdrcloc = SFLATDRCLOC;
 		execpars[0] = SFLATDRCNAME;
-		execpars[1] = yminfile;
-		execpars[2] = ymaxfile;
+		execpars[1] = dr_yminfile;
+		execpars[2] = dr_ymaxfile;
 		execpars[3] = p1;
 		execpars[4] = 0;
 		eexec(sflatdrcloc, execpars);
@@ -300,19 +302,19 @@ INTSML drc_flatwrite(NODEPROTO *simnp)
 	}
 
 	/* close the file descriptor so the pipe will terminate when done */
-	eclose(drc_fromshorts[1]);
+	eclose(dr_fromshorts[1]);
 
 	/* run the short detection program */
-	(void)drc_readshorterrors((INTBIG)drc_fromshorts[0]);
+	(void)dr_readshorterrors((INTBIG)dr_fromshorts[0]);
 
-	eclose(drc_fromshorts[0]);
-	ekill(drc_process);
-	ewait(drc_process);
+	eclose(dr_fromshorts[0]);
+	ekill(dr_process);
+	ewait(dr_process);
 	ttyputmsg("DRC short detection complete");
 	return(0);
 }
 
-INTSML drc_readshorterrors(INTBIG where)
+INTSML dr_readshorterrors(INTBIG where)
 {
 	REGISTER char *ptr, *layname;
 	REGISTER INTBIG i, errors, lx, hx, ly, hy;
@@ -348,7 +350,7 @@ INTSML drc_readshorterrors(INTBIG where)
 				(void)sscanf(line, "L %s B %ld %ld %ld %ld", layer, &sx, &sy, &cx, &cy);
 				lx = cx-sx/2;   hx = cx+sx/2;
 				ly = cy-sy/2;   hy = cy+sy/2;
-				layname = layername(drc_maintech, (INTSML)atoi(layer));
+				layname = layername(dr_maintech, (INTSML)atoi(layer));
 				ttyputmsg("Error on %s layer from X(%s...%s) Y(%s...%s)",
 					layname, latoa(lx), latoa(hx), latoa(ly), latoa(hy));
 				(void)askaid(us_aid, "show-area", lx, hx, ly, hy, el_curwindowpart->curnodeproto);
@@ -372,7 +374,7 @@ INTSML drc_readshorterrors(INTBIG where)
  * routine to flatten hierarchy from facet "facet" down, printing boxes and
  * net numbers.  The transformation into the facet is "mytrans".
  */
-void drc_flatprint(NODEPROTO *facet, XARRAY mytrans)
+void dr_flatprint(NODEPROTO *facet, XARRAY mytrans)
 {
 	REGISTER INTSML i;
 	REGISTER ARCINST *ai;
@@ -387,7 +389,7 @@ void drc_flatprint(NODEPROTO *facet, XARRAY mytrans)
 	if (stopping("DRC") != 0) return;
 
 	/* see if this facet is to be ignored */
-	for(i = 0; i < drc_nflat_ignored; i++) if (facet == drc_flatignored[i]) return;
+	for(i = 0; i < dr_nflat_ignored; i++) if (facet == dr_flatignored[i]) return;
 
 	/* make sure there is a polygon */
 	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, dr_aid->cluster);
@@ -405,24 +407,24 @@ void drc_flatprint(NODEPROTO *facet, XARRAY mytrans)
 			ai = pi->conarcinst;
 			if (ai->network != pp->network) continue;
 			if (ai->temp2 != 0) continue;
-			drc_flatprop(ai, pp->temp2);
+			dr_flatprop(ai, pp->temp2);
 			break;
 		}
 	}
 
 	/* set node numbers on arcs that are local */
 	for(ai = facet->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
-		if (ai->temp2 == 0) drc_flatprop(ai, drc_index++);
+		if (ai->temp2 == 0) dr_flatprop(ai, dr_index++);
 
 	/* write every arc in the facet */
 	for(ai = facet->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
 	{
-		if (ai->proto->tech != drc_maintech) continue;
+		if (ai->proto->tech != dr_maintech) continue;
 		n = arcpolys(ai);
 		for(i = 0; i < n; i++)
 		{
 			shapearcpoly(ai, i, poly);
-			drc_flatdesc_poly(poly, ai->temp2, mytrans, ai->proto->tech);
+			dr_flatdesc_poly(poly, ai->temp2, mytrans, ai->proto->tech);
 		}
 	}
 
@@ -458,14 +460,14 @@ void drc_flatprint(NODEPROTO *facet, XARRAY mytrans)
 				pp->temp2 = spp->temp2;
 				break;
 			}
-			if (pp->temp2 == 0) pp->temp2 = drc_index++;
+			if (pp->temp2 == 0) pp->temp2 = dr_index++;
 		}
 
 		/* if it is primitive, write the information */
 		downhierarchy(ni);
 		if (ni->proto->primindex != 0)
 		{
-			if (ni->proto->tech == drc_maintech)
+			if (ni->proto->tech == dr_maintech)
 			{
 				makerot(ni, subtrans);
 				n = nodeEpolys(ni);
@@ -473,9 +475,9 @@ void drc_flatprint(NODEPROTO *facet, XARRAY mytrans)
 				{
 					shapeEnodepoly(ni, i, poly);
 					if (poly->portproto == NOPORTPROTO) continue;
-					drc_transistor_hack(poly, ni);
+					dr_transistor_hack(poly, ni);
 					xformpoly(poly, subtrans);
-					drc_flatdesc_poly(poly, poly->portproto->temp2, mytrans, ni->proto->tech);
+					dr_flatdesc_poly(poly, poly->portproto->temp2, mytrans, ni->proto->tech);
 				}
 			}
 		} else
@@ -485,7 +487,7 @@ void drc_flatprint(NODEPROTO *facet, XARRAY mytrans)
 			transmult(temptrans1, mytrans, temptrans2);
 			maketrans(ni, temptrans1);
 			transmult(temptrans1, temptrans2, subtrans);
-			drc_flatprint(ni->proto, subtrans);
+			dr_flatprint(ni->proto, subtrans);
 		}
 		uphierarchy();
 	}
@@ -513,7 +515,7 @@ void drc_flatprint(NODEPROTO *facet, XARRAY mytrans)
  * and if so, shortens the distance.  It works great on regular transistors, but only
  * marginally well on the serpentine transistors.  Written by Ken Stevens.
  */
-void drc_transistor_hack(POLYGON *poly, NODEINST *ni)
+void dr_transistor_hack(POLYGON *poly, NODEINST *ni)
 {
 	REGISTER PORTPROTO *pp, *spp;
 	REGISTER PORTARCINST *pi, *spi;
@@ -614,7 +616,7 @@ void drc_transistor_hack(POLYGON *poly, NODEINST *ni)
  * number is "net".  The polygon is transformed by "trans" and is from
  * technology "tech".
  */
-void drc_flatdesc_poly(POLYGON *poly, INTBIG net, XARRAY trans, TECHNOLOGY *tech)
+void dr_flatdesc_poly(POLYGON *poly, INTBIG net, XARRAY trans, TECHNOLOGY *tech)
 {
 	INTBIG lx, hx, ly, hy, lxb, hxb, lyb, hyb;
 	REGISTER INTBIG xv, yv;
@@ -647,8 +649,8 @@ void drc_flatdesc_poly(POLYGON *poly, INTBIG net, XARRAY trans, TECHNOLOGY *tech
 	/* ignore zero or negative sizes */
 	if ((hy - ly) > 0 && (hx - lx) > 0)
 	{
-		xprintf(drc_file, "%ld %ld %ld %ld %d %ld\n", ly, hy, lx, hx, poly->layer, net);
-		drc_flat_boxcount++;
+		xprintf(dr_file, "%ld %ld %ld %ld %d %ld\n", ly, hy, lx, hx, poly->layer, net);
+		dr_flat_boxcount++;
 	}
 }
 
@@ -656,7 +658,7 @@ void drc_flatdesc_poly(POLYGON *poly, INTBIG net, XARRAY trans, TECHNOLOGY *tech
  * routine to propagate the node number "nindex" to the "temp2" of arcs
  * connected to arc "ai".
  */
-void drc_flatprop(ARCINST *ai, INTBIG nindex)
+void dr_flatprop(ARCINST *ai, INTBIG nindex)
 {
 	REGISTER ARCINST *oai;
 	REGISTER NODEINST *ni;
@@ -685,7 +687,7 @@ void drc_flatprop(ARCINST *ai, INTBIG nindex)
 				continue;
 
 			/* recurse on the nodes of this arc */
-			drc_flatprop(oai, nindex);
+			dr_flatprop(oai, nindex);
 		}
 	}
 }
