@@ -114,6 +114,7 @@ BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWnd)
 	ON_COMMAND(ID_ACCELERATOR_Z, OnAcceleratorZ)
 	ON_WM_SIZE()
 	ON_WM_CLOSE()
+	ON_WM_TIMER()
 	//}}AFX_MSG_MAP
 END_MESSAGE_MAP()
 
@@ -146,6 +147,9 @@ int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
 		return -1;      // fail to create
 	}
 
+	/* count off tenths of a second */
+	SetTimer(1, 100, 0);
+
 	return 0;
 }
 
@@ -177,6 +181,7 @@ short gWhichMenu;
 void gra_nativemenudoone(short item, short menuindex);
 void gra_resizemain(int cx, int cy);
 int gra_closeworld(void);
+void gra_timerticked(void);
 void gra_keyaction(UINT nChar, UINT nFlags, UINT nRepCnt);
 
 void CMainFrame::OnElectricMenu01() { gra_nativemenudoone( 1, gWhichMenu); }
@@ -279,3 +284,11 @@ void CMainFrame::OnClose()
 
 	CMDIFrameWnd::OnClose();
 }
+
+void CMainFrame::OnTimer(UINT nIDEvent) 
+{
+	// TODO: Add your message handler code here and/or call default
+	gra_timerticked();
+	
+	CMDIFrameWnd::OnTimer(nIDEvent);
+}
