@@ -56,23 +56,16 @@
 #  define DIRSEP           ':'						/* directory separator */
 #  define NONFILECH        '|'						/* character that cannot be in file name */
 #  define CADRCFILENAME    "cadrc"					/* CAD startup file */
-#  define ELECTRICLOG      "electric.log"			/* log file during run */
-#  define ELECTRICLOGSAVE  "electriclast.log"		/* log file after standard exit */
-#  define OPTIONSFILE      "electricoptions.elib"	/* where options are saved */
 #  define LIBDIR           ":lib:"					/* location of library files */
-#  define SORTLOC          ""
 #  define ESIMLOC          ""
 #  define RSIMLOC          ""
 #  define PRESIMLOC        ""
 #  define RNLLOC           ""
 #  define SPICELOC         ""
+#  define FASTHENRYLOC     ""
+#  define SORTLOC          ""
 #  define FLATDRCLOC       ""
 #  define SFLATDRCLOC      ""
-#  define HUGEINT          0x7FFFFFFF				/* largest possible integer */
-   typedef          long   INTBIG;					/* at least 32 bits, can hold address */
-   typedef          short  INTSML;					/* at least 16 bits */
-   typedef unsigned long  UINTBIG;					/* at least 32 bits, can hold address */
-   typedef unsigned short UINTSML;					/* at least 16 bits */
 #endif
 
 #ifdef WIN32						/***** Windows *****/
@@ -81,24 +74,17 @@
 #  define DIRSEP           '\\'						/* directory separator */
 #  define NONFILECH        '|'						/* character that cannot be in file name */
 #  define CADRCFILENAME    "cadrc"					/* CAD startup file */
-#  define ELECTRICLOG      "electric.log"			/* log file during run */
-#  define ELECTRICLOGSAVE  "electriclast.log"		/* log file after standard exit */
-#  define OPTIONSFILE      "electricoptions.elib"	/* where options are saved */
 #  define LIBDIR           "lib\\"					/* location of library files */
-#  define SORTLOC          ""
 #  define ESIMLOC          ""
 #  define RSIMLOC          ""
 #  define PRESIMLOC        ""
 #  define RNLLOC           ""
 #  define SPICELOC         ""
+#  define FASTHENRYLOC     ""
+#  define SORTLOC          ""
 #  define FLATDRCLOC       ""
 #  define SFLATDRCLOC      ""
 #  undef TECHNOLOGY
-#  define HUGEINT          0x7FFFFFFF				/* largest possible integer */
-   typedef LONG            INTBIG;					/* at least 32 bits, can hold address */
-   typedef SHORT           INTSML;					/* at least 16 bits */
-   typedef ULONG          UINTBIG;					/* at least 32 bits, can hold address */
-   typedef USHORT         UINTSML;					/* at least 16 bits */
 #endif
 
 #ifdef ONUNIX						/***** UNIX *****/
@@ -107,95 +93,66 @@
 #  define DIRSEP           '/'						/* directory separator */
 #  define NONFILECH        '?'						/* character that cannot be in file name */
 #  define CADRCFILENAME    ".cadrc"					/* CAD startup file */
-#  define ELECTRICLOG      ".electric.log"			/* log file during run */
-#  define ELECTRICLOGSAVE  ".electriclast.log"		/* log file after standard exit */
-#  define OPTIONSFILE      ".electricoptions.elib"	/* where options are saved */
 #  define LIBDIR           "lib/"					/* location of library files */
-#  define SORTLOC          "/usr/bin/sort"
 #  define ESIMLOC          "/usr/local/bin/esim"
 #  define RSIMLOC          "/usr/local/bin/rsim"
 #  define PRESIMLOC        "/usr/local/bin/presim"
 #  define RNLLOC           "/usr/local/bin/rnl"
 #  define SPICELOC         "/usr/local/bin/spice"
+#  define FASTHENRYLOC     "/usr/local/bin/fasthenry"
+#  define SORTLOC          "/usr/bin/sort"
 #  define FLATDRCLOC       "/usr/local/bin/ffindshort"
 #  define SFLATDRCLOC      "/usr/local/bin/findshort"
-#  define HUGEINT          0x7FFFFFFF				/* largest possible integer */
-   typedef          long   INTBIG;					/* at least 32 bits, can hold address */
-   typedef          short  INTSML;					/* at least 16 bits */
-   typedef unsigned long  UINTBIG;					/* at least 32 bits, can hold address */
-   typedef unsigned short UINTSML;					/* at least 16 bits */
-#endif
-
-/********************* DETERMINE CONFIGURATION *********************/
-
-/*
- * There are two different source configurations available,
- * all defined externally when compiling Electric:
- *
- * EGNU     GNU release
- * EVE      Full VLSI system
- */
-#ifdef EVE
-#  undef  EVE
-#  define EVE 1
-#else
-#  define EVE 0
-#endif
-
-/* if not a full release, then it is for GNU */
-#if EVE == 0
-#  define EGNU 1
-#else
-#  define EGNU 0
 #endif
 
 /********************* SOURCE MODULE CONFIGURATION *********************/
 
 /* The tools (User and I/O are always enabled) */
-#define COMAID             ( EGNU | EVE )
-#define COMPENAID          ( EGNU | EVE )
-#define DRCAID             ( EGNU | EVE )
-#define ERCAID             ( EGNU | EVE )
-#define LOGEFFAID          ( EGNU | EVE )
-#define MAPPERAID          (   0  |  0  )
-#define PLAAID             ( EGNU | EVE )
-#define PROJECTAID         ( EGNU | EVE )
-#define ROUTAID            ( EGNU | EVE )
-#define SCAID              ( EGNU | EVE )
-#define SIMAID             ( EGNU | EVE )
-#define VHDLAID            ( EGNU | EVE )
+#define COMAID             1
+#define COMPENAID          1
+#define DRCAID             1
+#define ERCAID             1
+#define LOGEFFAID          1
+#define MAPPERAID          0
+#define PLAAID             1
+#define PROJECTAID         1
+#define ROUTAID            1
+#define SCAID              1
+#define SIMAID             1
+#define VHDLAID            1
 
 /* The I/O modules (Binary, Text, and printer formats are always enabled) */
-#define IOCIF              ( EGNU | EVE )
-#define IODEF              (   0  | EVE | FORCECADENCE)
-#define IODXF              ( EGNU | EVE )
-#define IOEDIF             ( EGNU | EVE )
-#define IOGDS              ( EGNU | EVE )
-#define IOL                ( EGNU | EVE )
-#define IOLEF              (   0  | EVE | FORCECADENCE)
-#define IOSDF              ( EGNU | EVE )
-#define IOSKILL            (   0  | EVE | FORCECADENCE)
-#define IOSUE              ( EGNU | EVE )
+#define IOCIF              1
+#define IODEF              1
+#define IODXF              1
+#define IOEAGLE            1
+#define IOEDIF             1
+#define IOGDS              1
+#define IOL                1
+#define IOLEF              1
+#define IOPADS             1
+#define IOSDF              1
+#define IOSKILL            FORCECADENCE
+#define IOSUE              1
 
 /* The technologies (Artwork, Generic and Schematic are always enabled) */
-#define TECBICMOS          ( EGNU | EVE )
-#define TECBIPOLAR         ( EGNU | EVE )
-#define TECCMOS            ( EGNU | EVE )
-#define TECCMOSDODN        ( EGNU | EVE )
-#define TECEFIDO           ( EGNU | EVE )
-#define TECFPGA            ( EGNU | EVE )
-#define TECGEM             ( EGNU | EVE )
-#define TECMOCMOS          ( EGNU | EVE )
-#define TECMOCMOSSUB       ( EGNU | EVE )
-#define TECMOCMOS2         ( EGNU | EVE )
-#define TECNCMOS3          (   0  | EVE )
-#define TECNMOS            ( EGNU | EVE )
-#define TECPCB             ( EGNU | EVE )
-#define TECRCMOS           ( EGNU | EVE )
+#define TECBICMOS          1
+#define TECBIPOLAR         1
+#define TECCMOS            1
+#define TECCMOSDODN        1
+#define TECEFIDO           1
+#define TECFPGA            1
+#define TECGEM             1
+#define TECMOCMOS          1
+#define TECMOCMOSSUB       1
+#define TECMOCMOS2         1
+#define TECNMOS            1
+#define TECPCB             1
+#define TECRCMOS           1
 
 /* The language interfaces */
-#define LANGJAVA           (   0  |  0  | FORCEJAVA)
-#define LANGLISP           (   0  | EVE | FORCELISP)
-#define LANGTCL            (   0  | EVE | FORCETCL)
+#define LANGJAVA           FORCEJAVA
+#define LANGLISP           FORCELISP
+#define LANGTCL            FORCETCL
 
 #endif  /* _CONFIG_H_ - at top */
