@@ -195,7 +195,7 @@ void cli_linconmodifynodeinst(NODEINST*, INTBIG, INTBIG, INTBIG, INTBIG, INTSML,
 void cli_linconmodifyarcinst(ARCINST*, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
 void cli_linconmodifyportproto(PORTPROTO*, NODEINST*, PORTPROTO*);
 void cli_linconmodifynodeproto(NODEPROTO*);
-void cli_linconmodifydescript(VARIABLE*, INTBIG);
+void cli_linconmodifydescript(INTBIG, INTBIG, VARIABLE*, INTBIG);
 void cli_linconnewlib(LIBRARY*);
 void cli_linconkilllib(LIBRARY*);
 void cli_linconnewvariable(INTBIG, INTBIG, INTBIG, INTBIG);
