@@ -35,35 +35,41 @@ extern "C"
 
 /* the meaning of "DRC_options" */
 #define DRCHIERDIALOG   1				/* set to force hierarchical DRC to use dialog */
+#define DRCFIRSTERROR   2				/* set to stop after first error in a facet */
 
 extern AIDENTRY *dr_aid;				/* the DRC aid object */
-extern INTBIG    dr_ignore_list;		/* key for aid:drc.DRC_ignore_list */
+extern INTBIG    dr_ignore_listkey;		/* key for aid:drc.DRC_ignore_list */
+extern INTBIG    dr_lastgooddrckey;		/* key for "DRC_last_good_drc" */
+extern INTBIG    dr_options;			/* cached options */
 
-void dr_init(INTBIG*, char*[], AIDENTRY*);
-void dr_done(void);
-INTSML dr_set(INTSML, char*[]);
-void dr_examinenodeproto(NODEPROTO*);
-void dr_slice(void);
-void dr_modifynodeinst(NODEINST*,INTBIG,INTBIG,INTBIG,INTBIG,INTSML,INTSML);
-void dr_modifyarcinst(ARCINST*,INTBIG,INTBIG,INTBIG,INTBIG,INTBIG,INTBIG);
-void dr_newobject(INTBIG, INTBIG);
-void dr_killobject(INTBIG, INTBIG);
-void dr_eraselibrary(LIBRARY*);
+void     dr_init(INTBIG*, char*[], AIDENTRY*);
+void     dr_done(void);
+INTSML   dr_set(INTSML, char*[]);
+void     dr_examinenodeproto(NODEPROTO*);
+void     dr_slice(void);
+void     dr_modifynodeinst(NODEINST*,INTBIG,INTBIG,INTBIG,INTBIG,INTSML,INTSML);
+void     dr_modifyarcinst(ARCINST*,INTBIG,INTBIG,INTBIG,INTBIG,INTBIG,INTBIG);
+void     dr_newobject(INTBIG, INTBIG);
+void     dr_killobject(INTBIG, INTBIG);
+void     dr_newvariable(INTBIG, INTBIG, INTBIG, INTBIG);
+void     dr_readlibrary(LIBRARY*);
+void     dr_eraselibrary(LIBRARY*);
 
 /* intertool prototypes */
-INTBIG drcb_check(NODEPROTO*);
-INTSML drcb_debug(void);
-char *drcb_next_error(INTSML highlight, GEOM **geom1, GEOM **geom2);
-char *drcb_prev_error(void);
-char *drcb_curr_error(INTSML highlight, GEOM **geom1, GEOM **geom2);
-INTSML drc_flatwrite(NODEPROTO*);
-INTSML drc_flatignore(char*);
-INTSML drc_flatunignore(char*);
-INTBIG drcb_errorcount(void);
+INTSML   dr_flatwrite(NODEPROTO*);
+INTSML   dr_flatignore(char*);
+INTSML   dr_flatunignore(char*);
 NETWORK *dr_network(NODEINST*, PORTPROTO*);
-INTSML drcb_initincrementalcheck(NODEPROTO *facet);
-INTSML drcb_checkincremental(GEOM *geom, INTSML partial);
-void drcb_term(void);
+INTBIG   drcb_check(NODEPROTO*);
+INTSML   drcb_debug(void);
+char    *drcb_next_error(INTSML highlight, GEOM **geom1, GEOM **geom2);
+char    *drcb_prev_error(void);
+char    *drcb_curr_error(INTSML highlight, GEOM **geom1, GEOM **geom2);
+void     drcb_reset_dates(void);
+INTSML   drcb_initincrementalcheck(NODEPROTO *facet);
+INTSML   drcb_checkincremental(GEOM *geom, INTSML partial);
+void     drcb_term(void);
+INTBIG   drcb_errorcount(void);
 
 #if defined(__cplusplus) && !defined(ALLCPLUSPLUS)
 }
