@@ -60,6 +60,8 @@
 #define	FLEF                    14		/* LEF (Library Exchange Format) */
 #define	FDEF                    15		/* DEF (Design Exchange Format) */
 #define	FSKILL                  16		/* SKILL (CADENCE command language) */
+#define	FEAGLE                  17		/* EAGLE (CadSoft) netlist */
+#define	FPADS                   18		/* PADS netlist */
 
 /* aid:inout.IO_state: */
 #define	CIFINSQUARE             01		/* bit set for CIF input to square wires */
@@ -77,7 +79,7 @@
 #define EDIFSCHEMATIC       010000		/* bit set for EDIF write schematic instead of netlist */
 #define DXFALLLAYERS        020000		/* bit set for DXF to read all layers instead of technology list */
 #define DXFFLATTENINPUT     040000		/* bit set for DXF to flatten input */
-
+#define DEFNOPHYSICAL      0100000		/* bit set to ignore DEF physical data */
 #define GDSINTEXT          0200000		/* bit set for GDS input of text */
 #define GDSINEXPAND        0400000		/* bit set for GDS expansion of cells */
 #define GDSINARRAYS	      01000000		/* bit set for GDS array instantiation */
@@ -86,6 +88,7 @@
 #define PSROTATE         010000000		/* bit set for PostScript rotation (by 90 degrees) */
 #define	CIFOUTNOTOPCALL  020000000		/* bit set for CIF output to not call top cell */
 #define	GDSINIGNOREUKN   040000000		/* bit set for GDS input to ignore unknown layers */
+#define DEFNOLOGICAL    0100000000		/* bit set to ignore DEF logical data */
 
 #define	DEFAULTPSWIDTH         638		/* default PostScript page width (8.5" at 75dpi) */
 #define	DEFAULTPSHEIGHT        825		/* default PostScript page height (11.0" at 75dpi) */
@@ -98,11 +101,6 @@ extern "C"
 
 /* miscellaneous */
 extern FILE        *io_fileout;			/* channel for output */
-extern FILE        *io_filein;			/* channel for input */
-extern char        *io_tempstring;		/* for reading temporary strings */
-extern INTSML       io_tempstringlength;/* length of temporary string */
-extern char        *io_namesave;		/* for saving the global namespace */
-extern INTBIG      *io_newnames;		/* for adding new names to global namespace */
 extern jmp_buf      io_filerror;		/* nonlocal jump when I/O fails */
 extern INTSML       io_cifbase;			/* index used when writing CIF */
 extern INTBIG       io_state;			/* key for "IO_state" */
@@ -110,19 +108,6 @@ extern INTBIG       io_postscriptfilename;/* key for "IO_postscript_filename" */
 extern INTBIG       io_postscriptfiledate;/* key for "IO_postscript_filedate" */
 extern INTBIG       io_postscriptepsscale;/* key for "IO_postscript_EPS_scale" */
 extern INTSML       io_verbose;			/* 0: silent  1:verbose  -1:display facets during input */
-extern NODEINST   **io_nodelist;		/* list of nodes for readin */
-extern INTBIG      *io_nodecount;		/* number of nodeinsts in each facet */
-extern ARCINST    **io_arclist;			/* list of arcs for readin */
-extern INTBIG      *io_arccount;		/* number of arcinsts in each facet */
-extern NODEPROTO  **io_nodeprotolist;	/* list of facets for readin */
-extern CELL       **io_celllist;		/* list of cells for readin */
-extern TECHNOLOGY **io_techlist;		/* list of technologies for readin */
-extern INTBIG      *io_aidlist;			/* list of aids for readin */
-extern PORTPROTO  **io_portprotolist;	/* list of portprotos for readin */
-extern INTBIG      *io_portcount;		/* number of portprotos in each facet */
-extern PORTPROTO  **io_portpprotolist;	/* list of primitive portprotos for readin */
-extern NODEPROTO  **io_nodepprotolist;	/* list of primitive nodeprotos for readin */
-extern ARCPROTO   **io_arcprotolist;	/* list of arcprotos for readin */
 extern AIDENTRY    *io_aid;				/* the I/O aid object */
 
 /* prototypes for tool interface */
@@ -134,45 +119,57 @@ void io_slice(void);
 
 /* prototypes for intratool interface */
 PORTPROTO *io_convertoldportname(char *portname, NODEPROTO *np);
-void       io_fixnewlib(LIBRARY*);
-void       io_setoutputbloat(char*, INTBIG);
-INTSML     io_setuptechorder(TECHNOLOGY*);
-INTSML     io_nextplotlayer(INTBIG);
-INTBIG     io_getoutputbloat(char*);
-INTSML     io_readbinlibrary(LIBRARY*);
-INTSML     io_readciflibrary(LIBRARY*);
-INTSML     io_readdeflibrary(LIBRARY*);
-INTSML     io_readdxflibrary(LIBRARY*);
-INTSML     io_readediflibrary(LIBRARY*);
-INTSML     io_readgdslibrary(LIBRARY*,INTSML);
-INTSML     io_readleflibrary(LIBRARY*);
-INTSML     io_readsdflibrary(LIBRARY*);
-INTSML     io_readsuelibrary(LIBRARY*);
-INTSML     io_readtextlibrary(LIBRARY*);
-INTSML     io_readvhdllibrary(LIBRARY*);
-INTSML     io_writebinlibrary(LIBRARY*);
-INTSML     io_writeciflibrary(LIBRARY*);
-INTSML     io_writedxflibrary(LIBRARY*);
-INTSML     io_writeediflibrary(LIBRARY*);
-INTSML     io_writegdslibrary(LIBRARY*);
-INTSML     io_writehpgllibrary(LIBRARY*);
-INTSML     io_writellibrary(LIBRARY*);
-INTSML     io_writeleflibrary(LIBRARY*);
-INTSML     io_writepostscriptlibrary(LIBRARY*,INTBIG);
-INTSML     io_writequickdrawlibrary(LIBRARY*);
-INTSML     io_writeskilllibrary(LIBRARY*);
-INTSML     io_writetextlibrary(LIBRARY*);
+void       io_fixnewlib(LIBRARY *lib, INTBIG statusitem);
+void       io_setoutputbloat(char *layer, INTBIG amount);
+INTSML     io_setuptechorder(TECHNOLOGY *tech);
+INTSML     io_nextplotlayer(INTBIG i);
+INTBIG     io_getoutputbloat(char *layer);
+INTSML     io_doreadbinlibrary(LIBRARY *lib, INTSML newprogress);
+INTSML     io_doreadtextlibrary(LIBRARY *lib, INTSML newprogress);
+INTSML     io_readbinlibrary(LIBRARY *lib);
+INTSML     io_readciflibrary(LIBRARY *lib);
+INTSML     io_readdeflibrary(LIBRARY *lib);
+INTSML     io_readdxflibrary(LIBRARY *lib);
+INTSML     io_readediflibrary(LIBRARY *lib);
+INTSML     io_readgdslibrary(LIBRARY *lib, INTSML position);
+INTSML     io_readleflibrary(LIBRARY *lib);
+INTSML     io_readsdflibrary(LIBRARY *lib);
+INTSML     io_readsuelibrary(LIBRARY *lib);
+INTSML     io_readtextlibrary(LIBRARY *lib);
+INTSML     io_readvhdllibrary(LIBRARY *lib);
+INTSML     io_writebinlibrary(LIBRARY *lib);
+INTSML     io_writeciflibrary(LIBRARY *lib);
+INTSML     io_writedxflibrary(LIBRARY *lib);
+INTSML     io_writeeaglelibrary(LIBRARY *lib);
+INTSML     io_writeediflibrary(LIBRARY *lib);
+INTSML     io_writegdslibrary(LIBRARY *lib);
+INTSML     io_writehpgllibrary(LIBRARY *lib);
+INTSML     io_writellibrary(LIBRARY *lib);
+INTSML     io_writeleflibrary(LIBRARY *lib);
+INTSML     io_writepadslibrary(LIBRARY *lib);
+INTSML     io_writepostscriptlibrary(LIBRARY *lib, INTBIG printit);
+INTSML     io_writequickdrawlibrary(LIBRARY *lib);
+INTSML     io_writeskilllibrary(LIBRARY *lib);
+INTSML     io_writetextlibrary(LIBRARY *lib);
+void       io_freebininmemory(void);
 void       io_freecifinmemory(void);
 void       io_freecifparsmemory(void);
+void       io_freecifoutmemory(void);
 void       io_freedxfmemory(void);
 void       io_freegdsoutmemory(void);
 void       io_freepostscriptmemory(void);
 void       io_freesdfimemory(void);
 void       io_freesuememory(void);
-void       io_freetextmemory(void);
+void       io_freetextinmemory(void);
 void       io_freedefimemory(void);
 void       io_freelefimemory(void);
 void       io_freeedifinmemory(void);
+void       io_initskill(void);
+void       io_initdef(void);
+void       io_initdxf(void);
+void       io_initcif(void);
+void       io_initedif(void);
+void       io_initgds(void);
 
 #if defined(__cplusplus) && !defined(ALLCPLUSPLUS)
 }
