@@ -51,9 +51,15 @@
 
 /* #define DEBUGMEMORY 1 */					/* uncomment to debug memory */
 
-typedef INTBIG XARRAY[3][3];
+typedef          long   INTBIG;				/* at least 32 bits, can hold address */
+typedef unsigned long  UINTBIG;
+typedef          short  INTSML;				/* at least 16 bits */
+typedef unsigned short UINTSML;
+typedef INTBIG         XARRAY[3][3];		/* 3x3 transformation matrix */
+
 #define	SIZEOFINTBIG (sizeof (INTBIG))		/* bytes per long integer */
 #define	SIZEOFINTSML (sizeof (INTSML))		/* bytes per short integer */
+#define HUGEINT      0x7FFFFFFF				/* largest possible integer */
 #define REGISTER     register
 
 typedef struct
@@ -76,6 +82,11 @@ struct Irtnode;
 struct Ilibrary;
 struct Itechnology;
 
+#if defined(__cplusplus) && !defined(ALLCPLUSPLUS)
+extern "C"
+{
+#endif
+
 /*************************** VARIABLES ****************************/
 
 #define	NOVARIABLE  ((VARIABLE *)-1)
@@ -143,12 +154,14 @@ struct Itechnology;
 #define	VTPOSDOWNLEFT              7		/* text centered to lower-left of point */
 #define	VTPOSDOWNRIGHT             8		/* text centered to lower-right of point */
 #define	VTPOSBOXED                 9		/* text centered and limited to object size */
-#define	VTROTATION               060		/* rotation of text (90 degree increments) */
-#define	VTROTATIONSH               4		/* right shift of VTROTATION */
+#define	VTDISPLAYPART            060		/* bits telling what to display */
+#define	VTDISPLAYVALUE             0		/* just display the value */
+#define	VTDISPLAYNAME            020		/* just display the name */
+#define	VTDISPLAYNAMEVALUE       040		/* display the name and value */
 #define	VTSIZE                 01700		/* size of text (in points) */
 #define	VTSIZESH                   6		/* right shift of VTSIZE */
-#define	VTFONT                 06000		/* text font (2 bits only) */
-#define	VTFONTSH                  10		/* right shift of VTFONT */
+#define	VTINTERIOR             02000		/* text only appears inside facet */
+#define	VTINHERIT              04000		/* set to inherit value from proto to inst */
 #define	VTXOFF              07770000		/* X offset of text */
 #define	VTXOFFSH                  12		/* right shift of VTXOFF */
 #define	VTXOFFNEG          010000000		/* set if X offset is negative */
@@ -173,6 +186,7 @@ extern INTBIG  el_arc_name;					/* variable "ARC_name" */
 extern INTBIG  el_arc_radius;				/* variable "ARC_radius" */
 extern INTBIG  el_trace;					/* variable "trace" */
 extern INTBIG  el_facet_message;			/* variable "FACET_message" */
+extern INTBIG  el_schematic_page_size;		/* variable "FACET_schematic_page_size" */
 extern INTBIG  el_transistor_width;			/* variable "transistor_width" */
 extern INTBIG  el_prototype_center;			/* variable "prototype_center" */
 extern INTBIG  el_node_default_size;		/* variable "NODE_default_size" */
@@ -272,7 +286,7 @@ typedef struct Iaidentry
 	void   (*modifyarcinst)(struct Iarcinst*,INTBIG,INTBIG,INTBIG,INTBIG,INTBIG,INTBIG);	/* broadcast modified arcinst */
 	void   (*modifyportproto)(struct Iportproto*, struct Inodeinst*, struct Iportproto*);	/* broadcast modified portproto */
 	void   (*modifynodeproto)(struct Inodeproto*);			/* broadcast modified nodeproto */
-	void   (*modifydescript)(VARIABLE*, INTBIG);			/* broadcast modified descriptor */
+	void   (*modifydescript)(INTBIG, INTBIG, VARIABLE*, INTBIG);	/* broadcast modified descriptor */
 
 	void   (*newobject)(INTBIG, INTBIG);					/* broadcast new object */
 	void   (*killobject)(INTBIG, INTBIG);					/* broadcast deleted object */
@@ -864,7 +878,7 @@ typedef struct IWindowFrame
 	void /*Tk_Window*/  *tkwin;				/* the TK window placeholder */
 #endif
 #ifdef WIN32
-	void /*CChildFrame*/ *wndframe;			/* address of Windows subframe */
+	void /*CChildFrame*/*wndframe;			/* address of Windows subframe */
 	HWND                 realwindow;		/* the real window */
 	char                *data;				/* raster data for window */
 	void /*CDC*/        *hDC;				/* device context for real window */
@@ -1193,7 +1207,7 @@ typedef struct Iconstraint
 	void    (*modifyarcinst)(ARCINST*, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);	/* constrain a modified arc instance */
 	void    (*modifyportproto)(PORTPROTO*, NODEINST*, PORTPROTO*);					/* constrain a modified port prototype */
 	void    (*modifynodeproto)(NODEPROTO*);											/* constrain a modified node prototype */
-	void    (*modifydescript)(VARIABLE*, INTBIG);									/* constrain a modified descriptor */
+	void    (*modifydescript)(INTBIG, INTBIG, VARIABLE*, INTBIG);					/* constrain a modified descriptor */
 
 	void    (*newlib)(LIBRARY*);			/* constrain a newly created library */
 	void    (*killlib)(LIBRARY*);			/* constrain a deleted library */
@@ -1243,43 +1257,46 @@ extern CONSTRAINT  el_constraints[];		/* list of constraint solvers */
 #define FILETYPEDEF               07		/* DEF */
 #define FILETYPEDRAC             010		/* Dracula */
 #define FILETYPEDXF              011		/* DXF */
-#define FILETYPEEDIF             012		/* EDIF */
-#define FILETYPEESIM             013		/* ESIM netlist */
-#define FILETYPEFPGA             014		/* FPGA architecture */
-#define FILETYPEGDS              015		/* GDS */
-#define FILETYPEHELP             016		/* Help */
-#define FILETYPEHPGL             017		/* HPGL */
-#define FILETYPEHSPICEOUT        020		/* HSPICE output */
-#define FILETYPEL                021		/* L */
-#define FILETYPELEF              022		/* LEF */
-#define FILETYPELOG              023		/* Log */
-#define FILETYPEMACRO            024		/* Macro */
-#define FILETYPEMOSSIM           025		/* MOSSIM netlist */
-#define FILETYPENETLISP          026		/* Netlisp netlist */
-#define FILETYPENEWS             027		/* News */
-#define FILETYPEPAL              030		/* PAL netlist */
-#define FILETYPEPLATAB           031		/* PLA table */
-#define FILETYPEPROJ             032		/* Project */
-#define FILETYPEPROJUSERS        033		/* Project users */
-#define FILETYPEPS               034		/* PostScript */
-#define FILETYPEALS              035		/* ALS netlist */
-#define FILETYPEALSVEC           036		/* ALS vectors */
-#define FILETYPEQUISC            037		/* QUISC netlist */
-#define FILETYPERSIM             040		/* RSIM netlist */
-#define FILETYPESCTAB            041		/* Silicon compiler table */
-#define FILETYPESCSIM            042		/* Silicon compiler simulation */
-#define FILETYPESDF              043		/* SDF */
-#define FILETYPESILOS            044		/* Silos netlist */
-#define FILETYPESKILL            045		/* SKILL commands */
-#define FILETYPESPICE            046		/* SPICE input */
-#define FILETYPESPICEOUT         047		/* SPICE output */
-#define FILETYPESUE              050		/* SUE */
-#define FILETYPETEGAS            051		/* Tegas netlist */
-#define FILETYPETEGASTAB         052		/* Tegas table */
-#define FILETYPETEXT             053		/* Plain text */
-#define FILETYPETLIB             054		/* Text library */
-#define FILETYPEVERILOG          055		/* Verilog */
-#define FILETYPEVHDL             056		/* VHDL */
+#define FILETYPEEAGLE            012		/* EAGLE netlist */
+#define FILETYPEEDIF             013		/* EDIF */
+#define FILETYPEESIM             014		/* ESIM netlist */
+#define FILETYPEFASTHENRY        015		/* FastHenry netlist */
+#define FILETYPEFPGA             016		/* FPGA architecture */
+#define FILETYPEGDS              017		/* GDS */
+#define FILETYPEHELP             020		/* Help */
+#define FILETYPEHPGL             021		/* HPGL */
+#define FILETYPEHSPICEOUT        022		/* HSPICE output */
+#define FILETYPEL                023		/* L */
+#define FILETYPELEF              024		/* LEF */
+#define FILETYPELOG              025		/* Log */
+#define FILETYPEMACRO            026		/* Macro */
+#define FILETYPEMOSSIM           027		/* MOSSIM netlist */
+#define FILETYPENETLISP          030		/* Netlisp netlist */
+#define FILETYPENEWS             031		/* News */
+#define FILETYPEPADS             032		/* PADS netlist */
+#define FILETYPEPAL              033		/* PAL netlist */
+#define FILETYPEPLATAB           034		/* PLA table */
+#define FILETYPEPROJ             035		/* Project */
+#define FILETYPEPROJUSERS        036		/* Project users */
+#define FILETYPEPS               037		/* PostScript */
+#define FILETYPEALS              040		/* ALS netlist */
+#define FILETYPEALSVEC           041		/* ALS vectors */
+#define FILETYPEQUISC            042		/* QUISC netlist */
+#define FILETYPERSIM             043		/* RSIM netlist */
+#define FILETYPESCTAB            044		/* Silicon compiler table */
+#define FILETYPESCSIM            045		/* Silicon compiler simulation */
+#define FILETYPESDF              046		/* SDF */
+#define FILETYPESILOS            047		/* Silos netlist */
+#define FILETYPESKILL            050		/* SKILL commands */
+#define FILETYPESPICE            051		/* SPICE input */
+#define FILETYPESPICEOUT         052		/* SPICE output */
+#define FILETYPESUE              053		/* SUE */
+#define FILETYPETEGAS            054		/* Tegas netlist */
+#define FILETYPETEGASTAB         055		/* Tegas table */
+#define FILETYPETEXT             056		/* Plain text */
+#define FILETYPETLIB             057		/* Text library */
+#define FILETYPEVERILOG          060		/* Verilog */
+#define FILETYPEVHDL             061		/* VHDL */
 #define FILETYPEWRITE           0100		/* bit set to write file (instead of read) */
 #define FILETYPEAPPEND          0200		/* bit set to append to file (instead of read) */
 
@@ -1298,11 +1315,6 @@ extern char  *el_version;					/* current version string */
 
 /*************************** DECLARATIONS ****************************/
 
-#if defined(__cplusplus) && !defined(ALLCPLUSPLUS)
-extern "C"
-{
-#endif
-
 /* node routines */
 NODEINST    *newnodeinst(NODEPROTO *proto, INTBIG lowx, INTBIG highx, INTBIG lowy, INTBIG highy,
 				INTSML transposition, INTSML rotation, NODEPROTO *parent);
@@ -1423,9 +1435,9 @@ VARIABLE    *getval(INTBIG addr, INTBIG  type, INTBIG want, char *name);
 VARIABLE    *getvalkey(INTBIG addr, INTBIG type, INTBIG want, INTBIG key);
 #ifdef DEBUGMEMORY
 #  define    setval(a, t, n, na, nt) _setval((a), (t), (n), (na), (nt), (char *)__FILE__, (INTBIG)__LINE__)
-  VARIABLE  *_setval(INTBIG addr, INTBIG type, char *name, INTBIG newaddr, INTBIG newtype, char*, INTBIG);
+  VARIABLE *_setval(INTBIG addr, INTBIG type, char *name, INTBIG newaddr, INTBIG newtype, char*, INTBIG);
 #  define    setvalkey(a, t, k, na, nt) _setvalkey((a), (t), (k), (na), (nt), (char *)__FILE__, (INTBIG)__LINE__)
-  VARIABLE  *_setvalkey(INTBIG addr, INTBIG type, INTBIG key, INTBIG newaddr, INTBIG newtype, char*, INTBIG);
+  VARIABLE *_setvalkey(INTBIG addr, INTBIG type, INTBIG key, INTBIG newaddr, INTBIG newtype, char*, INTBIG);
 #else
   VARIABLE  *setval(INTBIG addr, INTBIG type, char *name, INTBIG newaddr, INTBIG newtype);
   VARIABLE  *setvalkey(INTBIG addr, INTBIG type, INTBIG key, INTBIG newaddr, INTBIG newtype);
@@ -1447,7 +1459,7 @@ void         nextvarchangequiet(void);
 INTBIG       initobjlist(INTBIG addr, INTBIG type, INTSML restrictdirect);
 char        *nextobjectlist(VARIABLE **var, INTBIG search);
 INTSML       copyvars(INTBIG fromaddr, INTBIG fromtype, INTBIG toaddr, INTBIG totype);
-void         modifydescript(VARIABLE *var, INTBIG newdescript);
+void         modifydescript(INTBIG addr, INTBIG type, VARIABLE *var, INTBIG newdescript);
 INTBIG       getlength(VARIABLE *var);
 char        *describevariable(VARIABLE *var, INTSML aindex, INTSML purpose);
 void         makeoptionstemporary(LIBRARY *lib);
@@ -1482,6 +1494,9 @@ INTBIG       drcmindistance(TECHNOLOGY *tech, INTSML layer1, INTSML layer2, INTS
 INTBIG       drcminwidth(TECHNOLOGY *tech, INTSML layer);
 INTSML       getecolor(char *colorname);
 INTSML       ecolorname(INTSML color, char **colorname, char **colorsymbol);
+INTSML       get3dfactors(TECHNOLOGY *tech, INTBIG layer, INTBIG *height, INTBIG *thickness);
+void         set3dheight(TECHNOLOGY *tech, INTBIG *depth);
+void         set3dthickness(TECHNOLOGY *tech, INTBIG *thickness);
 
 /* geometric routines */
 INTBIG       initsearch(INTBIG lowx, INTBIG highx, INTBIG lowy, INTBIG highy, NODEPROTO *facet);
@@ -1548,6 +1563,7 @@ INTBIG       computedistance(INTBIG x1, INTBIG y1, INTBIG x2, INTBIG y2);
 INTSML       figureangle(INTBIG x1, INTBIG y1, INTBIG x2, INTBIG y2);
 INTSML       intersect(INTBIG x1, INTBIG y1, INTBIG angle1, INTBIG x2, INTBIG y2, INTBIG angle2,
 				INTBIG *x, INTBIG *y);
+INTSML       isonline(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
 INTBIG       closestpointtosegment(INTBIG x1, INTBIG y1, INTBIG x2, INTBIG y2, INTBIG *x, INTBIG *y);
 INTSML       findcenters(INTBIG r, INTBIG x1, INTBIG y1, INTBIG x2, INTBIG y2, INTBIG d,
 				INTBIG *ix1, INTBIG *iy1, INTBIG *ix2, INTBIG *iy2);
@@ -1573,7 +1589,7 @@ INTSML       cropbox(INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy, INTBIG bx,
 void         arctopoly(INTBIG cx, INTBIG cy, INTBIG sx, INTBIG sy, INTBIG ex, INTBIG ey, POLYGON *poly,
 				INTBIG arcres, INTBIG arcsag);
 void         circletopoly(INTBIG cx, INTBIG cy, INTBIG radius, POLYGON *poly, INTBIG arcres, INTBIG arcsag);
-void         adjustdisoffset(GEOM *geom, POLYGON *poly, INTBIG textdescription);
+void         adjustdisoffset(INTBIG addr, INTBIG type, TECHNOLOGY *tech, POLYGON *poly, INTBIG textdescription);
 void         vectoradd3d(float *a, float *b, float *sum);
 void         vectorsubtract3d(float *a, float *b, float *diff);
 void         vectormultiply3d(float *a, float b, float *res);
@@ -1592,14 +1608,14 @@ void         mrginit(void);
 void         mrgterm(void);
 void         mrgstorebox(INTSML layer, TECHNOLOGY *tech, INTBIG length, INTBIG width, INTBIG xc, INTBIG yc);
 void         mrgdonefacet(void (*writepolygon)(INTSML layernum, TECHNOLOGY *tech,
-				INTBIG *x, INTBIG *y, INTSML count, float area));
+				INTBIG *x, INTBIG *y, INTSML count));
 void         mergeinit(void);
 void         mergestorepolygon(INTBIG layer, TECHNOLOGY *tech, POLYGON *poly);
 void         mergedone(void (*writepolygon)(INTSML, TECHNOLOGY*, INTBIG*, INTBIG*, INTSML));
 
 /* polygon routines */
 #ifdef DEBUGMEMORY
-  POLYGON   *_allocpolygon(INTSML points, CLUSTER *cluster, char*, INTBIG);
+  POLYGON  *_allocpolygon(INTSML points, CLUSTER *cluster, char*, INTBIG);
 #  define    allocpolygon(p, c) _allocpolygon((p), (c), (char *)__FILE__, (INTBIG)__LINE__)
 #else
   POLYGON   *allocpolygon(INTSML points, CLUSTER *cluster);
@@ -1610,7 +1626,8 @@ INTSML       extendpolygon(POLYGON *poly, INTSML newcount);
 INTSML       polysame(POLYGON *poly1, POLYGON *poly2);
 void         polycopy(POLYGON *dpoly, POLYGON *spoly);
 INTSML       isinside(INTBIG x, INTBIG y, POLYGON *poly);
-INTBIG       areapoly(POLYGON *poly);
+float        areapoly(POLYGON *poly);
+float        areapoints(INTBIG count, INTBIG *x, INTBIG *y);
 void         xformpoly(POLYGON *poly, XARRAY trans);
 INTSML       isbox(POLYGON *poly, INTBIG *xl, INTBIG *xh, INTBIG *yl, INTBIG *yh);
 void         maketruerect(POLYGON *poly);
@@ -1620,8 +1637,8 @@ INTSML       polyintersect(POLYGON*, POLYGON*);
 void         makerectpoly(INTBIG lx, INTBIG ux, INTBIG ly, INTBIG uy, POLYGON *poly);
 void         maketruerectpoly(INTBIG lx, INTBIG ux, INTBIG ly, INTBIG uy, POLYGON *poly);
 void         makedisparrayvarpoly(GEOM *geom, WINDOWPART *win, VARIABLE *var, POLYGON *poly);
-void         getdisparrayvarlinepos(GEOM *geom, WINDOWPART *win, VARIABLE *var,
-				INTBIG line, INTBIG *x, INTBIG *y, INTSML absolute);
+void         getdisparrayvarlinepos(INTBIG addr, INTBIG type, TECHNOLOGY *tech,
+				WINDOWPART *win, VARIABLE *var, INTBIG line, INTBIG *x, INTBIG *y, INTSML absolute);
 void         getcenter(POLYGON *poly, INTBIG *x, INTBIG *y);
 void         getbbox(POLYGON *poly, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy);
 INTSML       polysplithoriz(INTBIG yl, POLYGON *which, POLYGON **abovep, POLYGON **belowp);
@@ -1665,11 +1682,11 @@ INTSML       loadcode(char *program, INTBIG language);
 
 /* memory allocation routines */
 #ifdef DEBUGMEMORY
-  INTBIG    *_emalloc(INTBIG size, CLUSTER *cluster, char*, INTBIG);
+  INTBIG   *_emalloc(INTBIG size, CLUSTER *cluster, char*, INTBIG);
 # define     emalloc(x, c) _emalloc((x), (c), (char *)__FILE__, (INTBIG)__LINE__)
-  INTSML     _allocstring(char **addr, char *str, CLUSTER *cluster, char*, INTBIG);
+  INTSML    _allocstring(char **addr, char *str, CLUSTER *cluster, char*, INTBIG);
 # define     allocstring(a, s, c) _allocstring((a), (s), (c), (char *)__FILE__, (INTBIG)__LINE__)
-  INTSML     _reallocstring(char **addr, char *str, CLUSTER *cluster, char*, INTBIG);
+  INTSML    _reallocstring(char **addr, char *str, CLUSTER *cluster, char*, INTBIG);
 # define     reallocstring(a, s, c) _reallocstring((a), (s), (c), (char *)__FILE__, (INTBIG)__LINE__)
 #else
   INTBIG    *emalloc(INTBIG size, CLUSTER *cluster);
@@ -1709,7 +1726,6 @@ void         freewindowpart(WINDOWPART *win);
 void         ttynewcommand(void);
 void         ttyputerr(char *format, ...);
 void         ttyputmsg(char *format, ...);
-void         ttyputinfo(char *format, ...);
 void         ttyputverbose(char *format, ...);
 void         error(char *format, ...);
 INTSML       ttyquiet(INTSML flag);
@@ -1747,7 +1763,8 @@ void         starttimer(void);
 float        endtimer(void);
 UINTBIG      getcurrenttime(void);
 char        *timetostring(UINTBIG time);
-void         parsetime(UINTBIG time, INTSML *year, INTSML *month, INTSML *mday);
+void         parsetime(UINTBIG time, INTSML *year, INTSML *month, INTSML *mday,
+				INTSML *hour, INTSML *minute, INTSML *second);
 
 /* disk file routines */
 char        *truepath(char *path);
@@ -1814,7 +1831,53 @@ void         ewait(INTBIG process);
 void         flushscreen(void);
 void         exitprogram(void);
 
-/* low-level control of the messages window (not documented in Internals Manual) */
+/* graphics and control (not documented in Internals Manual) */
+void         osprimaryosinit(void);
+void         ossecondaryinit(INTBIG, char*[]);
+void         tooltimeslice(void);
+void         graphicsoptions(char *name, INTBIG *argc, char **argv);
+void         setupenvironment(void);
+void         setlibdir(char *libdir);
+INTSML       initgraphics(INTSML messages);
+void         termgraphics(void);
+WINDOWFRAME *newwindowframe(INTSML floating, RECTAREA *r);
+void         killwindowframe(WINDOWFRAME*);
+void         getpaletteparameters(INTSML *wid, INTSML *hei, INTSML *palettewidth);
+void         sizewindowframe(WINDOWFRAME *frame, INTSML wid, INTSML hei);
+void         movewindowframe(WINDOWFRAME *frame, INTSML left, INTSML top);
+void         bringwindowtofront(WINDOWFRAME *frame);
+WINDOWFRAME *getwindowframe(INTSML canfloat);
+void         getwindowframesize(WINDOWFRAME *frame, INTSML *wid, INTSML *hei);
+INTSML       graphicshas(INTSML want);
+void         setdefaultcursortype(INTSML state);
+void         setnormalcursor(INTSML curs);
+void         colormapload(INTBIG *red, INTBIG *green, INTBIG *blue, INTSML low, INTSML high);
+void         screendrawline(WINDOWPART *win, INTSML x1, INTSML y1, INTSML x2, INTSML y2, GRAPHICS *desc,
+				INTSML texture);
+void         screeninvertline(WINDOWPART *win, INTSML x1, INTSML y1, INTSML x2, INTSML y2);
+void         screendrawpolygon(WINDOWPART *win, INTBIG *x, INTBIG *y, INTSML count, GRAPHICS *desc);
+void         screendrawbox(WINDOWPART *win, INTSML lowx, INTSML highx, INTSML lowy, INTSML highy, GRAPHICS *desc);
+void         screeninvertbox(WINDOWPART *win, INTSML lowx, INTSML highx, INTSML lowy, INTSML highy);
+void         screenmovebox(WINDOWPART *win, INTSML sx, INTSML sy, INTSML wid, INTSML hei, INTSML dx, INTSML dy);
+INTBIG       screensavebox(WINDOWPART *win, INTSML lx, INTSML hx, INTSML ly, INTSML hy);
+void         screenmovesavedbox(INTBIG code, INTSML dx, INTSML dy);
+INTSML       screenrestorebox(INTBIG code, INTSML destroy);
+void         screensettextsize(WINDOWPART *win, INTSML fnt);
+void         screengettextsize(WINDOWPART *win, char *str, INTSML *x, INTSML *y);
+void         screendrawtext(WINDOWPART *win, INTSML atx, INTSML aty, char *s, GRAPHICS *desc);
+INTSML       gettextbits(WINDOWPART *win, char *msg, INTSML *wid, INTSML *hei, char ***rowstart);
+void         screendrawcircle(WINDOWPART *win, INTBIG atx, INTBIG aty, INTBIG radius, GRAPHICS *desc);
+void         screendrawdisc(WINDOWPART *win, INTBIG atx, INTBIG aty, INTBIG radius, GRAPHICS *desc);
+void         screendrawcirclearc(WINDOWPART *win, INTBIG centerx, INTBIG centery, INTBIG p1x, INTBIG p1y,
+				INTBIG p2x, INTBIG p2y, GRAPHICS *desc);
+void         screendrawgrid(WINDOWPART *win, POLYGON *obj);
+INTSML       getnxtchar(void);
+UINTBIG      machinetimeoffset(void);
+INTSML       logplayback(char *file);
+void         logstartrecord(void);
+void         logfinishrecord(void);
+void         getacceleratorstrings(char **acceleratorstring, char **acceleratorprefix);
+char        *getinterruptkey(void);
 char        *getmessagesstring(char *prompt);
 void         putmessagesstring(char *str, INTSML important);
 char        *getmessageseofkey(void);
@@ -1825,68 +1888,19 @@ INTSML       copyfrommessages(void);
 INTSML       pastetomessages(void);
 char        *getcutbuffer(void);
 void         setcutbuffer(char *msg);
+char        *optionsfilepath(void);
 
-/* graphics and control (not documented in Internals Manual) */
-void         el_primaryinit(void);
-void         el_secondaryinit(INTBIG, char*[]);
-void         el_slice(void);
-void         us_setdisplay(char *name, INTBIG *argc, char **argv);
-void         us_unsetdisplay(void);
-void         setupenvironment(void);
-void         setlibdir(char *libdir);
-INTSML       us_initgraphics(INTSML messages);
-void         us_termgraphics(void);
-WINDOWFRAME *us_newwindowframe(INTSML floating, RECTAREA *r);
-void         us_killwindowframe(WINDOWFRAME*);
-void         us_getpaletteparameters(INTSML *wid, INTSML *hei, INTSML *palettewidth);
-void         us_sizewindowframe(WINDOWFRAME *frame, INTSML wid, INTSML hei);
-void         us_movewindowframe(WINDOWFRAME *frame, INTSML left, INTSML top);
-void         us_bringwindowtofront(WINDOWFRAME *frame);
-WINDOWFRAME *us_getwindowframe(INTSML canfloat);
-void         us_getwindowsize(WINDOWFRAME *frame, INTSML *wid, INTSML *hei);
-void         us_startbatching(void);
-void         us_endbatching(void);
-INTSML       us_graphicshas(INTSML want);
-void         setdefaultcursortype(INTSML state);
-void         us_setnormalcursor(INTSML curs);
-void         us_loadmap(INTBIG *red, INTBIG *green, INTBIG *blue, INTSML low, INTSML high);
-void         us_drawline(WINDOWPART *win, INTSML x1, INTSML y1, INTSML x2, INTSML y2, GRAPHICS *desc,
-				INTSML texture);
-void         us_invertline(WINDOWPART *win, INTSML x1, INTSML y1, INTSML x2, INTSML y2);
-void         us_drawpolygon(WINDOWPART *win, INTBIG *x, INTBIG *y, INTSML count, GRAPHICS *desc);
-void         us_drawbox(WINDOWPART *win, INTSML lowx, INTSML highx, INTSML lowy, INTSML highy, GRAPHICS *desc);
-void         us_invertbox(WINDOWPART *win, INTSML lowx, INTSML highx, INTSML lowy, INTSML highy);
-void         us_movebox(WINDOWPART *win, INTSML sx, INTSML sy, INTSML wid, INTSML hei, INTSML dx, INTSML dy);
-INTBIG       us_savebox(WINDOWPART *win, INTSML lx, INTSML hx, INTSML ly, INTSML hy);
-void         us_movesavedbox(INTBIG code, INTSML dx, INTSML dy);
-INTSML       us_restorebox(INTBIG code, INTSML destroy);
-void         us_settextsize(WINDOWPART *win, INTSML fnt);
-void         us_textsize(WINDOWPART *win, char *str, INTSML *x, INTSML *y);
-void         us_puttext(WINDOWPART *win, INTSML atx, INTSML aty, char *s, GRAPHICS *desc);
-INTSML       us_gettextbits(WINDOWPART *win, char *msg, INTSML *wid, INTSML *hei, char ***rowstart);
-void         us_drawcircle(WINDOWPART *win, INTBIG atx, INTBIG aty, INTBIG radius, GRAPHICS *desc);
-void         us_drawdisc(WINDOWPART *win, INTBIG atx, INTBIG aty, INTBIG radius, GRAPHICS *desc);
-void         us_drawcirclearc(WINDOWPART *win, INTBIG centerx, INTBIG centery, INTBIG p1x, INTBIG p1y,
-				INTBIG p2x, INTBIG p2y, GRAPHICS *desc);
-void         us_drawgrid(WINDOWPART *win, POLYGON *obj);
-INTSML       us_getnxtchar(void);
-UINTBIG      machinetimeoffset(void);
-INTSML       us_logplayback(char *file);
-void         us_logstartrecord(void);
-void         us_logfinishrecord(void);
-void         getacceleratorstrings(char **acceleratorstring, char **acceleratorprefix);
-char        *getinterruptkey(void);
-
-/* terminal I/O (not documented in Internals Manual) */
+/* command completion coroutines (not documented in Internals Manual) */
+void         requiredextension(char *extension);
 char        *nextaids(void);
+char        *nextarcs(void);
 char        *nextcells(void);
+char        *nextfacets(void);
 char        *nextfile(void);
 char        *nextlibs(void);
 char        *nextnets(void);
 char        *nexttechs(void);
 char        *nextviews(void);
-char        *us_nextparse(void);
-void         requiredextension(char *extension);
 INTSML       topofaids(char **chrpos);
 INTSML       topofcells(char **chrpos);
 INTSML       topoffile(char **chrpos);
@@ -1895,8 +1909,8 @@ INTSML       topoflibs(char **chrpos);
 INTSML       topofnets(char **chrpos);
 INTSML       topoftechs(char **chrpos);
 INTSML       topofviews(char **chrpos);
-INTSML       us_topoffacets(char **chrpos);
-INTSML       us_topofarcs(char **chrpos);
+INTSML       topoffacets(char **chrpos);
+INTSML       topofarcs(char **chrpos);
 INTSML       us_patoplist(char **chrpos);
 INTSML       us_paparams(char *word, COMCOMP *arr[], char breakc);
 void         us_pabackupto(INTSML a, char *b[]);
