@@ -30,17 +30,18 @@
 
 /********************* for all simulators *********************/
 
-#define	ESIM     0						/* ESIM from MIT (SIM format) */
-#define	RSIM     1						/* RSIM from MIT (SIM format) */
-#define	RNL      2						/* RNL from MIT (SIM format) */
-#define COSMOS	 3						/* COSMOS from CMU (SIM format) */
-#define	SPICE    4						/* SPICE from Berkeley (SPICE format) */
-#define MOSSIM   5						/* MOSSIM from CalTech (NTK format) */
-#define TEXSIM   6						/* TEGAS from Calma (TDL format) */
-#define ABEL     7						/* ABEL from Data I/O (PAL programmer) */
-#define VERILOG  8						/* VERILOG from Gateway (VER format) */
-#define SILOS	 9						/* SILOS from Simucad */
-#define ALS 	10						/* ALS (NET format) */
+#define	ESIM       0					/* ESIM from MIT (SIM format) */
+#define	RSIM       1					/* RSIM from MIT (SIM format) */
+#define	RNL        2					/* RNL from MIT (SIM format) */
+#define COSMOS     3					/* COSMOS from CMU (SIM format) */
+#define	SPICE      4					/* SPICE from Berkeley (SPICE format) */
+#define MOSSIM     5					/* MOSSIM from CalTech (NTK format) */
+#define TEXSIM     6					/* TEGAS from Calma (TDL format) */
+#define ABEL       7					/* ABEL from Data I/O (PAL programmer) */
+#define VERILOG    8					/* VERILOG from Gateway (VER format) */
+#define SILOS      9					/* SILOS from Simucad */
+#define ALS       10					/* ALS (NET format) */
+#define FASTHENRY 11					/* FastHenry (INP format) */
 
 /* the meaning of "SIM_dontrun" */
 #define SIMRUNYES       -1				/* run simulation, show output */
@@ -89,6 +90,10 @@ INTSML sim_window_isactive(NODEPROTO **np);
 void   sim_window_redraw(void);
 void   sim_window_setlines(INTSML);
 INTSML sim_window_getlines(void);
+void   sim_window_setvislines(INTSML count);
+INTSML sim_window_getvislines(void);
+void   sim_window_settopline(INTSML count);
+INTSML sim_window_gettopline(void);
 void   sim_window_savegraph(void);
 void   sim_window_displaymode(INTSML full);
 void   sim_window_displaycolor(INTSML strength, INTSML color);
@@ -172,6 +177,31 @@ extern INTBIG sim_spice_state;			/* key for "SIM_spice_state" */
 extern INTSML sim_spice_debug;			/* SPICE debugging */
 extern INTBIG sim_listingfile;			/* key for "SIM_listingfile" */
 
+/********************* for FASTHENRY *********************/
+
+/* the meaning of "aid:sim.SIM_fasthenry_state" */
+#define FHUSESINGLEFREQ       01			/* use single frequency */
+#define FHMAKEMULTIPOLECKT    02			/* make multipole circuit */
+#define FHMAKEPOSTSCRIPTVIEW  04			/* make PostScript view */
+#define FHMAKESPICESUBCKT    010			/* make SPICE subcircuit */
+#define FHEXECUTETYPE       0160			/* what to do after writing the deck */
+#define FHEXECUTENONE          0			/*    no simulator run */
+#define FHEXECUTERUNFH       020			/*    run fasthenry */
+#define FHEXECUTERUNFHMUL    040			/*    run fasthenry for every group */
+
+extern INTBIG sim_fasthenrystatekey;		/* cached key for "SIM_fasthenry_state" */
+extern INTBIG sim_fasthenryfreqstartkey;	/* cached key for "SIM_fasthenry_freqstart" */
+extern INTBIG sim_fasthenryfreqendkey;		/* cached key for "SIM_fasthenry_freqend" */
+extern INTBIG sim_fasthenryrunsperdecadekey;/* cached key for "SIM_fasthenry_runsperdecade" */
+extern INTBIG sim_fasthenrynumpoleskey;		/* cached key for "SIM_fasthenry_numpoles" */
+extern INTBIG sim_fasthenryseglimitkey;		/* cached key for "SIM_fasthenry_seglimit" */
+extern INTBIG sim_fasthenrythicknesskey;	/* cached key for "SIM_fasthenry_thickness" */
+extern INTBIG sim_fasthenrywidthsubdivkey;	/* cached key for "SIM_fasthenry_width_subdivs" */
+extern INTBIG sim_fasthenryheightsubdivkey;	/* cached key for "SIM_fasthenry_height_subdivs" */
+extern INTBIG sim_fasthenryzheadkey;		/* cached key for "SIM_fasthenry_z_head" */
+extern INTBIG sim_fasthenryztailkey;		/* cached key for "SIM_fasthenry_z_tail" */
+extern INTBIG sim_fasthenrygroupnamekey;	/* cached key for "SIM_fasthenry_group_name" */
+
 #if defined(__cplusplus) && !defined(ALLCPLUSPLUS)
 extern "C"
 {
@@ -201,6 +231,8 @@ NETWORK *sim_spice_networkfromname(char *name);
 char    *sim_spice_signalname(NETWORK *net);
 INTSML   sim_simpointout(char*, INTSML);
 void     sim_spice_execute(char*, char*);
+void     sim_writefasthenrynetlist(NODEPROTO*);
+void     sim_fasthenryinit(void);
 void     sim_writesim(NODEPROTO*, INTSML);
 void     sim_writepalnetlist(NODEPROTO*);
 void     sim_writemossim(NODEPROTO*);
