@@ -35,7 +35,7 @@ typedef enum {DELAY_MIN=0, DELAY_TYP, DELAY_MAX} DELAY_TYPES;
  * More events can be accommodated in the trace buffer by
  * increasing its size.
  */
-#define TRACE_SIZE	10000
+#define DEFAULT_TRACE_SIZE	10000
 
 typedef struct model_ds
 {
@@ -214,6 +214,7 @@ extern INTSML      simals_seed_flag;
 extern float       simals_time_abs, simals_delta_def,
 				   simals_linear_def, simals_exp_def, simals_random_def, simals_abs_def;
 extern NODEPROTO  *simals_mainproto;
+extern INTBIG      simals_trace_size;
 
 /* prototypes for intratool interface */
 LINKPTR   simals_alloc_link_mem(void);
