@@ -453,6 +453,10 @@ INTSML    tech_inittrans(INTSML, NODEINST*);
 INTSML    tech_pinusecount(NODEINST*, ARCPROTO*);
 INTSML    tech_displayablenvars(NODEINST*);
 VARIABLE *tech_filldisplayablenvar(NODEINST*, POLYGON*);
+INTSML    tech_displayablefacetvars(NODEPROTO *np);
+VARIABLE *tech_filldisplayablefacetvar(NODEPROTO *np, POLYGON *poly);
+INTSML    tech_displayableportvars(PORTPROTO *pp);
+VARIABLE *tech_filldisplayableportvar(PORTPROTO *pp, POLYGON *poly);
 void      tech_filltrans(POLYGON*, TECH_POLYGON**, TECH_SERPENT*, NODEINST*, INTBIG,
 			INTSML, TECH_PORTS*);
 void      tech_moscutpoly(NODEINST*, INTSML, INTBIG[]);
