@@ -96,6 +96,24 @@
 #define	FLIPINTERACTIVECON    040000		/* set to flip the sense of the interactive constraint */
 #define	CENTEREDPRIMITIVES   0100000		/* set to place primitives by centers */
 
+/* special keys */
+#define	LEFTARROWKEY             034		/* the Left Arrow key */
+#define	RIGHTARROWKEY            035		/* the Right Arrow key */
+#define	UPARROWKEY               036		/* the Up Arrow key */
+#define	DOWNARROWKEY             037		/* the Down Arrow key */
+#define FUNCTIONF1              0216
+#define FUNCTIONF2              0217
+#define FUNCTIONF3              0220
+#define FUNCTIONF4              0221
+#define FUNCTIONF5              0222
+#define FUNCTIONF6              0223
+#define FUNCTIONF7              0224
+#define FUNCTIONF8              0225
+#define FUNCTIONF9              0226
+#define FUNCTIONF10             0227
+#define FUNCTIONF11             0230
+#define FUNCTIONF12             0231
+
 /* the O/S capabilities */
 #define CANLOGINPUT               01		/* can log input for playback */
 #define CANUSEFRAMES              02		/* can display multiple window frames */
@@ -104,6 +122,31 @@
 #define CANHAVENOWINDOWS         020		/* can exist with no edit windows */
 #define CANSTATUSPERFRAME        040		/* can put separate status info in each frame */
 
+/* the icon styles (in aid:user.USER_icon_style) */
+#define ICONSTYLESIDEIN            03			/* side number for input ports     (         */
+#define ICONSTYLESIDEINSH           0			/* shift for ICONSTYLESIDEIN */
+#define ICONSTYLESIDEOUT          014			/* side number for input ports      0=left   */
+#define ICONSTYLESIDEOUTSH          2			/* shift for ICONSTYLESIDEOUT */
+#define ICONSTYLESIDEBIDIR        060			/* side number for input ports      1=right  */
+#define ICONSTYLESIDEBIDIRSH        4			/* shift for ICONSTYLESIDEBIDIR */
+#define ICONSTYLESIDEPOWER       0300			/* side number for input ports      2=top    */
+#define ICONSTYLESIDEPOWERSH        6			/* shift for ICONSTYLESIDEPOWER */
+#define ICONSTYLESIDEGROUND     01400			/* side number for input ports      3=bottom */
+#define ICONSTYLESIDEGROUNDSH       8			/* shift for ICONSTYLESIDEGROUND */
+#define ICONSTYLESIDECLOCK      06000			/* side number for input ports     )         */
+#define ICONSTYLESIDECLOCKSH       10			/* shift for ICONSTYLESIDECLOCK */
+#define ICONSTYLEPORTLOC      0140000			/* location of ports */
+#define ICONSTYLEPORTLOCSH         14			/* shift for ICONSTYLEPORTLOC */
+#define ICONSTYLEPORTSTYLE    0600000			/* style of port text */
+#define ICONSTYLEPORTSTYLESH       16			/* shift for ICONSTYLEPORTSTYLE */
+#define ICONSTYLEDRAWNOLEADS 01000000			/* nonzero to remove leads */
+#define ICONSTYLEDRAWNOBODY  02000000			/* nonzero to remove body */
+
+#define ICONSTYLEEFAULT (0<<ICONSTYLESIDEINSH) | (1<<ICONSTYLESIDEOUTSH) | \
+						(2<<ICONSTYLESIDEBIDIRSH) | (3<<ICONSTYLESIDEPOWERSH) | \
+						(3<<ICONSTYLESIDEGROUNDSH) | (0<<ICONSTYLESIDECLOCKSH) | \
+						(1<<ICONSTYLEPORTLOCSH) | (0<<ICONSTYLEPORTSTYLESH)
+
 /* the "style" parameter to "us_getcolormap()" */
 #define	COLORSEXISTING             0		/* reload existing colors */
 #define COLORSDEFAULT              1		/* load default colors (gray background) */
@@ -194,9 +237,9 @@ extern INTBIG       us_quickkeys;			/* cached key for "USER_quick_keys" */
 extern INTBIG       us_interactiveangle;	/* cached key for "USER_interactive_angle" */
 extern INTBIG       us_tinylambdaperpixel;	/* cached key for "USER_tiny_lambda_per_pixel" */
 extern INTBIG       us_techstate;			/* cached key for "TECH_state" */
-extern INTBIG       us_techlayer3dheightkey;/* cached key for "TECH_layer_3dheight" */
-extern INTBIG       us_techlayer3dthicknesskey;	/* cached key for "TECH_layer_3dthickness" */
 extern INTBIG       us_ignoreoptionchanges;	/* cached key for "USER_ignore_option_changes" */
+extern INTBIG       us_displayunits;		/* cached key for "USER_display_units" */
+
 extern INTBIG       us_useroptions;			/* mirror for "USER_optionflags" */
 extern INTBIG       us_tinyratio;			/* mirror for "USER_tiny_lambda_per_pixel" */
 extern INTBIG       us_separatechar;		/* separating character in port names */
@@ -234,6 +277,25 @@ extern char       **us_quickkeylist;		/* quick keys in initial setup */
 
 /******************** HIGHLIGHT MODULES ********************/
 
+/*
+ * Highlight modules contain the following:
+ *
+ *  status   fromgeom  fromport   frompoint  fromvar     facet        what is selected
+ * ========  ========  =========  =========  ========  =========  ============================
+ * HIGHFROM  nodeinst  ---------  ---------  --------  nodeproto  node
+ * HIGHFROM  arcinst   ---------  ---------  --------  nodeproto  arc
+ * HIGHFROM  nodeinst  portproto  ---------  --------  nodeproto  node and port
+ * HIGHFROM  nodeinst  ---------  integer    --------  nodeproto  node and outline point
+ * HIGHTEXT  nodeinst  ---------  ---------  variable  nodeproto  displayable variable on node
+ * HIGHTEXT  arcinst   ---------  ---------  variable  nodeproto  displayable variable on arc
+ * HIGHTEXT  nodeinst  portproto  ---------  variable  nodeproto  displayable variable on port
+ * HIGHTEXT  --------  ---------  ---------  variable  nodeproto  displayable variable on facet
+ * HIGHTEXT  --------  portproto  ---------  --------  nodeproto  port name
+ * HIGHTEXT  nodeinst  ---------  ---------  --------  nodeproto  node name
+ * HIGHBBOX  --------  ---------  ---------  --------  nodeproto  bounding box
+ * HIGHLINE  --------  ---------  ---------  --------  nodeproto  line
+ */
+
 #define	NOHIGHLIGHT ((HIGHLIGHT *)-1)
 #define	HIGHFROM                  01		/* set to display "from" object */
 #define	HIGHBBOX                  04		/* set to display bounding box */
@@ -553,7 +615,6 @@ void           us_searchtext(WINDOWPART*, char*, INTSML);
 
 /* prototypes for intratool interface */
 void           us_3dbuttonhandler(WINDOWPART *w, INTSML but, INTSML x, INTSML y);
-void           us_3ddefaultlayerheight(TECHNOLOGY *tech);
 void           us_3denddrawing(void);
 void           us_3dfillview(WINDOWPART *win);
 void           us_3dpanview(WINDOWPART *win, INTBIG x, INTBIG y);
@@ -580,14 +641,14 @@ void           us_beginchanges(void);
 INTSML         us_bottomrecurse(NODEINST*, PORTPROTO*, INTSML);
 USERCOM       *us_buildcommand(INTSML, char*[]);
 void           us_buildquickkeylist(void);
-void           us_buildtexthighpoly(GEOM*, INTBIG, INTBIG, INTBIG, INTBIG, INTSML, POLYGON*);
+void           us_buildtexthighpoly(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTSML, POLYGON*);
 void           us_buttonhandler(WINDOWPART*, INTSML, INTSML, INTSML);
 INTSML         us_canedit(NODEPROTO*, NODEPROTO*, INTSML);
 INTSML         us_charhandler(WINDOWPART*, INTSML);
 void           us_chatportproto(NODEINST*, PORTPROTO*);
 void           us_check_facet_date(NODEPROTO*, UINTBIG);
 void           us_checkdatabase(INTSML);
-void           us_cleanupfacet(NODEPROTO*);
+void           us_cleanupfacet(NODEPROTO*, INTSML);
 void           us_clearhighlightcount(void);
 void           us_clearwindow(WINDOWPART*);
 char          *us_commandvarname(INTSML);
@@ -597,6 +658,8 @@ void           us_copyobjects(WINDOWPART*);
 NODEPROTO     *us_copyrecursively(NODEPROTO*, char*, LIBRARY*, VIEW*, INTSML, char*);
 INTSML         us_copyvars(INTBIG fromaddr, INTBIG fromtype, INTBIG toaddr, INTBIG totype);
 void           us_coverimplant(void);
+void           us_createexplorerstruct(void);
+NODEPROTO     *us_currentexplorernode(void);
 INTSML         us_cursoroverhigh(HIGHLIGHT*, INTBIG, INTBIG, WINDOWPART*);
 void           us_cutobjects(WINDOWPART*);
 void           us_delfacetmessage(NODEPROTO*);
@@ -612,15 +675,22 @@ char          *us_describetextdescript(INTBIG);
 void           us_docommands(FILE*, INTSML, char*);
 void           us_doexpand(NODEINST*, INTSML, INTSML);
 void           us_dofillet(void);
+void           us_dokillfacet(NODEPROTO *np);
 void           us_dopeek(INTBIG, INTBIG, INTBIG, INTBIG, NODEPROTO*, WINDOWPART*);
 void           us_doubchanges(void);
 void           us_dounexpand(NODEINST*);
 INTSML         us_drawarcinst(ARCINST*, INTSML, XARRAY, INTSML, WINDOWPART*);
 void           us_drawdispwindowsliders(WINDOWPART *w);
 INTSML         us_drawfacet(NODEINST*, INTSML, XARRAY, INTSML, WINDOWPART*);
+void           us_drawfacetvariable(VARIABLE *var, NODEPROTO *np);
+void           us_drawhorizontalslider(WINDOWPART*, INTSML, INTSML, INTSML);
+void           us_drawhorizontalsliderthumb(WINDOWPART*, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
 void           us_drawmenu(INTSML, WINDOWFRAME*);
 void           us_drawmenuentry(INTBIG, INTBIG, char*);
 INTSML         us_drawnodeinst(NODEINST*, INTSML, XARRAY, INTSML, WINDOWPART*);
+void           us_drawslidercorner(WINDOWPART *win, INTBIG lx, INTBIG hx, INTBIG ly, INTBIG hy);
+void           us_drawverticalslider(WINDOWPART*, INTSML, INTSML, INTSML, INTSML);
+void           us_drawverticalsliderthumb(WINDOWPART*, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
 void           us_drawwindow(WINDOWPART*, INTSML);
 void           us_editvariabletext(VARIABLE*, INTBIG, INTBIG, char*);
 void           us_endchanges(WINDOWPART*);
@@ -634,6 +704,9 @@ INTSML         us_evaluatevariable(char*, INTBIG*, INTBIG*, char**);
 void           us_execute(USERCOM*, INTBIG, INTSML, INTSML);
 INTSML         us_expandaddrtypearray(INTBIG*, INTBIG**, INTBIG**, INTBIG);
 void           us_expandttybuffers(INTSML);
+void           us_explorebuttonhandler(WINDOWPART*, INTSML, INTSML, INTSML);
+INTSML         us_explorecharhandler(WINDOWPART*, INTSML);
+void           us_exploreredisphandler(WINDOWPART*);
 INTSML         us_facetedithandler(WINDOWPART*, INTSML);
 INTBIG         us_figuregrabpoint(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
 void           us_figuretechopaque(TECHNOLOGY*);
@@ -662,12 +735,17 @@ GEOM          *us_getclosest(INTBIG, INTBIG, NODEPROTO*);
 void           us_getcolormap(TECHNOLOGY*, INTSML, INTSML);
 INTSML         us_getcolormapentry(char*, INTSML);
 COMCOMP       *us_getcomcompfromkeyword(char *keyword);
+void           us_gethighaddrtype(HIGHLIGHT *high, INTBIG *addr, INTBIG *type);
+INTBIG         us_gethighdescript(HIGHLIGHT *high);
 GEOM         **us_gethighlighted(INTSML);
+NODEINST      *us_gethighnodeinst(HIGHLIGHT *high);
+char          *us_gethighstring(HIGHLIGHT *high);
 void           us_gethightextcenter(HIGHLIGHT*, INTBIG*, INTBIG*, INTSML*);
 void           us_getlowleft(NODEINST*, INTBIG*, INTBIG*);
 VARIABLE      *us_getmacro(char*);
 MACROPACK     *us_getmacropack(char*);
 UINTBIG        us_getobject(INTSML, INTSML);
+TECHNOLOGY    *us_getobjectinfo(INTBIG addr, INTBIG type, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy);
 HIGHLIGHT     *us_getonehighlight(void);
 INTSML         us_getonesnappoint(INTBIG *x, INTBIG *y);
 POPUPMENU     *us_getpopupmenu(char*);
@@ -676,6 +754,7 @@ void           us_getslide(INTSML, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG*, INTB
 void           us_getsnappoint(HIGHLIGHT*, INTBIG*, INTBIG*);
 INTBIG         us_gettextposition(char*);
 INTBIG         us_gettextsize(char*, INTBIG);
+void           us_gettruewindowbounds(WINDOWPART *w, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy);
 INTSML         us_gettwoobjects(GEOM**, PORTPROTO**, GEOM**, PORTPROTO**);
 INTSML         us_getvar(char*, INTBIG*, INTBIG*, char**, INTSML*, INTSML*);
 void           us_graphfacets(void);
@@ -684,9 +763,11 @@ INTSML         us_hasotherwindowpart(INTBIG x, INTBIG y, WINDOWPART *ow);
 void           us_highlighteverywhere(GEOM*, PORTPROTO*, INTSML, INTSML, INTSML, INTSML);
 void           us_highlightmenu(INTBIG, INTBIG, INTSML);
 void           us_highlightwindow(WINDOWPART*, INTSML);
+TECHNOLOGY    *us_hightech(HIGHLIGHT *high);
 void           us_hsvtorgb(float, float, float, INTBIG*, INTBIG*, INTBIG*);
 void           us_identifyports(NODEINST*, NODEPROTO*, INTSML);
 void           us_illustratecommandset(void);
+void           us_inheritattributes(NODEINST *ni);
 INTSML         us_initialbinding(void);
 void           us_initlayerletters(void);
 void           us_initpulldownmenu(POPUPMENU*);
@@ -705,7 +786,7 @@ char          *us_makehighlightstring(HIGHLIGHT*);
 NODEPROTO     *us_makeiconfacet(PORTPROTO*, char*, char*, TECHNOLOGY*, LIBRARY*);
 void           us_makenewportproto(NODEPROTO*, NODEINST*, PORTPROTO*, char*, INTBIG, INTBIG, INTBIG);
 INTSML         us_makescreen(INTBIG*, INTBIG*, INTBIG*, INTBIG*, WINDOWPART*);
-void           us_maketextpoly(char*, WINDOWPART*, INTBIG, INTBIG, GEOM*, INTBIG, POLYGON*);
+void           us_maketextpoly(char*, WINDOWPART*, INTBIG, INTBIG, NODEINST*, TECHNOLOGY*, INTBIG, POLYGON*);
 void           us_manhattantravel(NODEINST*, INTSML);
 void           us_manymove(GEOM**, NODEINST**, INTSML, INTBIG, INTBIG);
 INTSML         us_modarcbits(INTSML, INTSML, char*, GEOM**);
@@ -715,7 +796,6 @@ NODEPROTO     *us_needfacet(void);
 INTSML         us_needwindow(void);
 void           us_nettravel(NODEINST*, INTBIG);
 MACROPACK     *us_newmacropack(char*);
-char          *us_nextparse(void);
 char          *us_nextvars(void);
 INTSML         us_nulldisplayroutine(POLYGON*, WINDOWPART*);
 INTSML         us_pagetword(INTSML(*)(COMCOMP**, INTSML*), void(*)(char*), void(*)(void), void(*)(void), INTSML);
@@ -740,6 +820,7 @@ void           us_queueopaque(GEOM*, INTSML);
 void           us_queueredraw(GEOM*, INTSML);
 void           us_redisplay(WINDOWPART*);
 void           us_redisplaynow(WINDOWPART*, INTSML);
+void           us_redoexplorerwindow(void);
 void           us_redostatus(WINDOWFRAME*);
 INTSML         us_reexportport(PORTPROTO*, NODEINST*);
 void           us_regridfacet(void);
@@ -752,6 +833,7 @@ void           us_rgbtohsv(INTSML, INTSML, INTSML, float*, float*, float*);
 INTBIG         us_rotatedescript(GEOM*, INTBIG);
 INTBIG         us_rotatedescriptI(GEOM*, INTBIG);
 ARCINST       *us_runarcinst(NODEINST*, PORTPROTO*, INTBIG, INTBIG, NODEINST*, PORTPROTO*, INTBIG, INTBIG, ARCPROTO*, INTBIG);
+INTSML         us_samecontents(NODEPROTO *np1, NODEPROTO *np2, INTBIG explain);
 void           us_scaletowindow(INTBIG*, INTBIG*, WINDOWPART*);
 void           us_scaletraceinfo(NODEINST*, INTBIG, INTBIG, INTBIG, INTBIG);
 void           us_scanforkeyequiv(POPUPMENU*);
@@ -808,7 +890,6 @@ void           us_termpulldownmenu(POPUPMENU*);
 void           us_textfacetchanges(WINDOWPART*, INTSML, char*, char*, INTBIG);
 INTSML         us_topofallthings(char**);
 INTSML         us_topofarcnodes(char**);
-INTSML         us_topofarcs(char**);
 INTSML         us_topofcellfacets(char**);
 INTSML         us_topofcommands(char**);
 INTSML         us_topofconstraints(char**);
@@ -817,7 +898,6 @@ INTSML         us_topofedtecarc(char**);
 INTSML         us_topofedteclay(char**);
 INTSML         us_topofedtecnode(char**);
 INTSML         us_topofexpports(char**);
-INTSML         us_topoffacets(char**);
 INTSML         us_topofhighlight(char**);
 INTSML         us_topoflayers(char**);
 INTSML         us_topofmacros(char**);
@@ -830,6 +910,7 @@ INTSML         us_topofvars(char**);
 INTSML         us_topofwindows(char**);
 void           us_undisplayobject(GEOM*);
 void           us_undoportproto(NODEINST*, PORTPROTO*);
+void           us_undrawfacetvariable(VARIABLE *var, NODEPROTO *np);
 char          *us_uniqueportname(char*, NODEPROTO*);
 void           us_unknowncommand(void);
 void           us_unqueueredraw(LIBRARY *lib);
