@@ -41,6 +41,7 @@ INTSML us_arcsizedlog(char*[]);
 INTSML us_areadlog(char*, char*[], char*);
 INTSML us_arraydlog(char*[]);
 INTSML us_artlookdlog(char*[]);
+INTSML us_attributesdlog(char*[]);
 INTSML us_capacitancedlog(char*[], char*);
 INTSML us_colormixdlog(char *overlayernames[]);
 INTSML us_copyfacetdlog(char*, char*[]);
@@ -56,8 +57,10 @@ INTSML us_filedlog(COMCOMP*, char*[], char*, INTSML);
 INTSML us_frameoptionsdlog(char *[]);
 INTSML us_gaindlog(char*, char*[], char*);
 INTSML us_griddlog(char*[]);
+void   us_givemessagedialog(char*);
 INTSML us_helpdlog(char*, char*[]);
 INTSML us_highlayerlog(char*[]);
+INTSML us_iconstyledlog(char*[]);
 INTSML us_inductancedlog(char*[], char*);
 INTSML us_lambdadlog(char*[]);
 INTSML us_librarypathdlog(char*[]);
@@ -65,6 +68,7 @@ INTSML us_menudlog(char*[]);
 INTSML us_newfacetdlog(char*[]);
 INTSML us_newviewdlog(char*[]);
 INTSML us_nodesizedlog(char*[]);
+INTSML us_noyesalwaysdlog(char*, char*[]);
 INTSML us_noyesdlog(char*, char*[]);
 INTSML us_oldlibrarydlog(char*[]);
 INTSML us_optionsavingdlog(char*[]);
