@@ -40,6 +40,7 @@
 #include <math.h>
 
 /* the command parsing table */
+#if IOCIF
 static KEYWORD iocifoopt[] =
 {
 	{"fully-instantiated",    0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
@@ -55,7 +56,7 @@ static KEYWORD iocifoopt[] =
 	{"dont-instantiate-top",  0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	TERMKEY
 };
-static COMCOMP iocifop = {iocifoopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
+COMCOMP io_cifop = {iocifoopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
 	INPUTOPT, " \t", "style of CIF output", 0};
 static KEYWORD iocifiopt[] =
 {
@@ -68,11 +69,85 @@ static COMCOMP iocifip = {iocifiopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
 static KEYWORD iocifopt[] =
 {
 	{"input",     1,{&iocifip,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"output",    1,{&iocifop,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"output",    1,{&io_cifop,NOKEY,NOKEY,NOKEY,NOKEY}},
 	TERMKEY
 };
-static COMCOMP iocifp = {iocifopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
+COMCOMP io_cifp = {iocifopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
 	INPUTOPT, " \t", "control of CIF", 0};
+#endif
+
+#if IODXF
+static KEYWORD iodxfaopt[] =
+{
+	{"all",                  0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"restrict",             0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	TERMKEY
+};
+COMCOMP iodxfap = {iodxfaopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
+	INPUTOPT, " \t", "DXF layer restriction options", 0};
+static KEYWORD iodxfopt[] =
+{
+	{"acceptable-layers",   1,{&iodxfap,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"flatten-input",       0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"not-flatten-input",   0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	TERMKEY
+};
+COMCOMP io_dxfp = {iodxfopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
+	INPUTOPT, " \t", "DXF options", 0};
+#endif
+
+#if IOGDS
+static COMCOMP iogdsoarp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
+	INPUTOPT, " \t", "maximum number of degrees per arc segment", 0};
+static COMCOMP iogdsoasp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
+	INPUTOPT, " \t", "maximum sag distance for an arc segment", 0};
+static KEYWORD iogdsoopt[] =
+{
+	{"individual-boxes",         0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"merge-boxes",              0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"include-cloak-layer",      0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"ignore-cloak-layer",       0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"arc-resolution",           1,{&iogdsoarp,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"arc-sag",                  1,{&iogdsoasp,NOKEY,NOKEY,NOKEY,NOKEY}},
+	TERMKEY
+};
+static COMCOMP iogdsop = {iogdsoopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
+	INPUTOPT, " \t", "style of GDS output", 0};
+static KEYWORD iogdsiopt[] =
+{
+	{"text",                     0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"expand",                   0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"arrays",                   0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"unknown-layers",           0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"no-text",                  0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"no-expand",                0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"no-arrays",                0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"no-unknown-layers",        0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	TERMKEY
+};
+static COMCOMP iogdsiop = {iogdsiopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
+	INPUTOPT, " \t", "style of GDS input", 0};
+static KEYWORD iogdsopt[] =
+{
+	{"output",   1,{&iogdsop,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"input",    1,{&iogdsiop,NOKEY,NOKEY,NOKEY,NOKEY}},
+	TERMKEY
+};
+COMCOMP io_gdsp = {iogdsopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
+	INPUTOPT, " \t", "control of GDS", 0};
+#endif
+
+#if IOEDIF
+static KEYWORD ioedifopt[] =
+{
+	{"schematic",         0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"netlist",           0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	TERMKEY
+};
+COMCOMP io_edifp = {ioedifopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
+	INPUTOPT, " \t", "EDIF output option", 0};
+#endif
+
 static KEYWORD ioplotnopt[] =
 {
 	{"focus",       0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
@@ -91,44 +166,16 @@ static KEYWORD ioplotopt[] =
 static COMCOMP ioplotp = {ioplotopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
 	INPUTOPT, " \t", "control of plot output", 0};
 
-static COMCOMP iogdsoarp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	INPUTOPT, " \t", "maximum number of degrees per arc segment", 0};
-static COMCOMP iogdsoasp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	INPUTOPT, " \t", "maximum sag distance for an arc segment", 0};
-static KEYWORD iogdsoopt[] =
+static KEYWORD ioveropt[] =
 {
-	{"individual-boxes",         0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"merge-boxes",              0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"include-cloak-layer",      0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"ignore-cloak-layer",       0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"arc-resolution",           1,{&iogdsoarp,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"arc-sag",                  1,{&iogdsoasp,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"off",              0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"on",               0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"graphical",        0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	TERMKEY
 };
-static KEYWORD iogdsiopt[] =
-{
-	{"text",                     0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"expand",                   0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"arrays",                   0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"unknown-layers",           0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"no-text",                  0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"no-expand",                0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"no-arrays",                0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"no-unknown-layers",        0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	TERMKEY
-};
-static COMCOMP iogdsop = {iogdsoopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	INPUTOPT, " \t", "style of GDS output", 0};
-static COMCOMP iogdsiop = {iogdsiopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	INPUTOPT, " \t", "style of GDS input", 0};
-static KEYWORD iogdsopt[] =
-{
-	{"output",   1,{&iogdsop,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"input",    1,{&iogdsiop,NOKEY,NOKEY,NOKEY,NOKEY}},
-	TERMKEY
-};
-static COMCOMP iogdsp = {iogdsopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	INPUTOPT, " \t", "control of GDS", 0};
+static COMCOMP ioverp = {ioveropt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
+	INPUTOPT, " \t", "verbose option", 0};
+
 static COMCOMP iopostsynch = {NOKEYWORD, topoffile, nextfile, NOPARAMS, NOBACKUP,
 	NOFILL|INPUTOPT, " \t", "PostScript synchronization file name", 0};
 static COMCOMP iopostplotwid = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
@@ -156,15 +203,7 @@ static KEYWORD iopostopt[] =
 };
 static COMCOMP iopostp = {iopostopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
 	INPUTOPT, " \t", "control of PostScript format", 0};
-static KEYWORD ioveropt[] =
-{
-	{"off",              0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"on",               0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"graphical",        0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	TERMKEY
-};
-static COMCOMP ioverp = {ioveropt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	INPUTOPT, " \t", "verbose option", 0};
+
 static COMCOMP iohpgl2sp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
 	INPUTOPT, " \t", "internal units per pixel", 0};
 static KEYWORD iohpgl2opt[] =
@@ -182,45 +221,30 @@ static KEYWORD iohpglopt[] =
 };
 static COMCOMP iohpglp = {iohpglopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
 	INPUTOPT, " \t", "HPGL version", 0};
-static KEYWORD ioedifopt[] =
-{
-	{"schematic",         0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"netlist",           0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	TERMKEY
-};
-static COMCOMP ioedifp = {ioedifopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	INPUTOPT, " \t", "EDIF output option", 0};
-static KEYWORD iodxfaopt[] =
-{
-	{"all",                  0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"restrict",             0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	TERMKEY
-};
-COMCOMP iodxfap = {iodxfaopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	INPUTOPT, " \t", "DXF layer restriction options", 0};
-static KEYWORD iodxfopt[] =
-{
-	{"acceptable-layers",   1,{&iodxfap,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"flatten-input",       0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"not-flatten-input",   0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
-	TERMKEY
-};
-static COMCOMP iodxfp = {iodxfopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	INPUTOPT, " \t", "DXF options", 0};
+
 static COMCOMP iobloatlp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
 	INPUTOPT, " \t", "Layer name to bloat", 0};
 static COMCOMP iobloatap = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
 	0, " \t", "amount to bloat layer (in internal units)", 0};
+
 static KEYWORD ioopt[] =
 {
-	{"cif",            1,{&iocifp,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"dxf",            1,{&iodxfp,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"gds",            1,{&iogdsp,NOKEY,NOKEY,NOKEY,NOKEY}},
+#if IOCIF
+	{"cif",            1,{&io_cifp,NOKEY,NOKEY,NOKEY,NOKEY}},
+#endif
+#if IODXF
+	{"dxf",            1,{&io_dxfp,NOKEY,NOKEY,NOKEY,NOKEY}},
+#endif
+#if IOGDS
+	{"gds",            1,{&io_gdsp,NOKEY,NOKEY,NOKEY,NOKEY}},
+#endif
+#if IOEDIF
+	{"edif",           1,{&io_edifp,NOKEY,NOKEY,NOKEY,NOKEY}},
+#endif
 	{"plot",           1,{&ioplotp,NOKEY,NOKEY,NOKEY,NOKEY}},
 	{"verbose",        1,{&ioverp,NOKEY,NOKEY,NOKEY,NOKEY}},
 	{"postscript",     1,{&iopostp,NOKEY,NOKEY,NOKEY,NOKEY}},
 	{"hpgl",           1,{&iohpglp,NOKEY,NOKEY,NOKEY,NOKEY}},
-	{"edif",           1,{&ioedifp,NOKEY,NOKEY,NOKEY,NOKEY}},
 	{"bloat-output",   2,{&iobloatlp,&iobloatap,NOKEY,NOKEY,NOKEY}},
 	TERMKEY
 };
@@ -243,8 +267,11 @@ struct
 #if IODXF
 	{"dxf",                2, FDXF},
 #endif
+#if IOEAGLE
+	{"eagle",              2, FEAGLE},
+#endif
 #if IOEDIF
-	{"edif",               1, FEDIF},
+	{"edif",               2, FEDIF},
 #endif
 #if IOGDS
 	{"gds",                1, FGDS},
@@ -255,6 +282,9 @@ struct
 #endif
 #if IOLEF
 	{"lef",                2, FLEF},
+#endif
+#if IOPADS
+	{"pads",               2, FPADS},
 #endif
 	{"postscript",         2, FPOSTSCRIPT},
 	{"printed-postscript", 2, FPRINTEDPOSTSCRIPT},
@@ -274,7 +304,7 @@ struct
 #if VHDLAID
 	{"vhdl",               1, FVHDL},
 #endif
-	{NULL, 0, 0}  /* 0 */
+	{NULL, 0, 0}
 };
 
 #define	NOBLOAT ((BLOAT *)-1)
@@ -295,9 +325,6 @@ static INTSML     *io_overlaporder;
 
 /* miscellaneous */
 FILE        *io_fileout;		/* channel for output */
-FILE        *io_filein;			/* channel for input */
-char        *io_namesave;		/* for saving the global namespace */
-INTBIG      *io_newnames;		/* for adding new names to global namespace */
 jmp_buf      io_filerror;		/* nonlocal jump when I/O fails */
 INTSML       io_cifbase;		/* index used when writing CIF */
 INTBIG       io_state;			/* key for "IO_state" */
@@ -305,102 +332,14 @@ INTBIG       io_postscriptfilename;/* key for "IO_postscript_filename" */
 INTBIG       io_postscriptfiledate;/* key for "IO_postscript_filedate" */
 INTBIG       io_postscriptepsscale;/* key for "IO_postscript_EPS_scale" */
 INTSML       io_verbose;		/* 0: silent  1:chattier  -1:graphical */
-NODEINST   **io_nodelist;		/* list of nodes for readin */
-INTBIG      *io_nodecount;		/* number of nodeinsts in each facet */
-ARCINST    **io_arclist;		/* list of arcs for readin */
-INTBIG      *io_arccount;		/* number of arcinsts in each facet */
-NODEPROTO  **io_nodeprotolist;	/* list of facets for readin */
-CELL       **io_celllist;		/* list of cells for readin */
-TECHNOLOGY **io_techlist;		/* list of technologies for readin */
-INTBIG      *io_aidlist;		/* list of aids for readin */
-PORTPROTO  **io_portprotolist;	/* list of portprotos for readin */
-INTBIG      *io_portcount;		/* number of portprotos in each facet */
-PORTPROTO  **io_portpprotolist;	/* list of primitive portprotos for readin */
-NODEPROTO  **io_nodepprotolist;	/* list of primitive nodeprotos for readin */
-ARCPROTO   **io_arcprotolist;	/* list of arcprotos for readin */
 AIDENTRY    *io_aid;			/* the I/O aid object */
 
-/* EDIF Options */
-DIALOGITEM io_edifoptionsdialogitems[] =
-{
- /*  1 */ {0, {32,120,56,192}, BUTTON, "OK"},
- /*  2 */ {0, {32,16,56,88}, BUTTON, "Cancel"},
- /*  3 */ {0, {8,8,24,200}, CHECK, "Use Schematic View"}
-};
-DIALOG io_edifoptionsdialog = {{50,75,115,284}, "EDIF Options", 3, io_edifoptionsdialogitems};
-
-/* DXF Options */
-DIALOGITEM io_dxfoptionsdialogitems[] =
-{
- /*  1 */ {0, {144,344,168,416}, BUTTON, "OK"},
- /*  2 */ {0, {144,248,168,320}, BUTTON, "Cancel"},
- /*  3 */ {0, {8,16,168,231}, SCROLL, ""},
- /*  4 */ {0, {8,240,24,320}, MESSAGE, "DXF Layer:"},
- /*  5 */ {0, {8,324,24,430}, EDITTEXT, ""},
- /*  6 */ {0, {32,240,48,430}, CHECK, "Input flattens hierarchy"}
-};
-DIALOG io_dxfoptionsdialog = {{50,75,228,514}, "DXF Options", 6, io_dxfoptionsdialogitems};
-
-/* GDS Options */
-DIALOGITEM io_gdsoptionsdialogitems[] =
-{
- /*  1 */ {0, {224,344,248,416}, BUTTON, "OK"},
- /*  2 */ {0, {224,248,248,320}, BUTTON, "Cancel"},
- /*  3 */ {0, {8,16,248,231}, SCROLL, ""},
- /*  4 */ {0, {8,240,24,320}, MESSAGE, "GDS Layer:"},
- /*  5 */ {0, {8,324,24,430}, EDITTEXT, ""},
- /*  6 */ {0, {32,240,48,464}, CHECK, "Input Includes Text"},
- /*  7 */ {0, {56,240,72,464}, CHECK, "Input Expands Facets"},
- /*  8 */ {0, {80,240,96,464}, CHECK, "Input Instantiates Arrays"},
- /*  9 */ {0, {128,240,144,464}, CHECK, "Output Merges Boxes"},
- /* 10 */ {0, {152,240,168,430}, MESSAGE, "Output Arc Conversion:"},
- /* 11 */ {0, {176,250,192,394}, MESSAGE, "Maximum arc angle:"},
- /* 12 */ {0, {176,396,192,464}, EDITTEXT, ""},
- /* 13 */ {0, {200,250,216,394}, MESSAGE, "Maximum arc sag:"},
- /* 14 */ {0, {200,396,216,464}, EDITTEXT, ""},
- /* 15 */ {0, {104,240,120,464}, CHECK, "Input Ignores Unknown Layers"}
-};
-DIALOG io_gdsoptionsdialog = {{50,75,307,548}, "GDS Options", 15, io_gdsoptionsdialogitems};
-
-/* CIF Options */
-DIALOGITEM io_cifoptionsdialogitems[] =
-{
- /*  1 */ {0, {144,388,168,460}, BUTTON, "OK"},
- /*  2 */ {0, {144,248,168,320}, BUTTON, "Cancel"},
- /*  3 */ {0, {8,16,168,231}, SCROLL, ""},
- /*  4 */ {0, {8,240,24,320}, MESSAGE, "CIF Layer:"},
- /*  5 */ {0, {8,324,24,458}, EDITTEXT, ""},
- /*  6 */ {0, {32,240,48,462}, CHECK, "Output Mimics Display"},
- /*  7 */ {0, {56,240,72,462}, CHECK, "Output Merges Boxes"},
- /*  8 */ {0, {104,240,120,462}, CHECK, "Input Squares Wires"},
- /*  9 */ {0, {80,240,96,462}, CHECK, "Output Instantiates Top Level"}
-};
-DIALOG io_cifoptionsdialog = {{50,75,228,546}, "CIF Options", 9, io_cifoptionsdialogitems};
-
-/* SKILL Options */
-DIALOGITEM io_skilloptionsdialogitems[] =
-{
- /*  1 */ {0, {204,156,228,228}, BUTTON, "OK"},
- /*  2 */ {0, {204,12,228,84}, BUTTON, "Cancel"},
- /*  3 */ {0, {8,8,168,231}, SCROLL, ""},
- /*  4 */ {0, {176,12,192,108}, MESSAGE, "SKILL Layer:"},
- /*  5 */ {0, {176,116,192,222}, EDITTEXT, ""}
-};
-DIALOG io_skilloptionsdialog = {{50,75,289,323}, "SKILL Options", 5, io_skilloptionsdialogitems};
-
 /* prototypes for local routines */
-void io_fixrtree(RTNODE*);
 double io_calc_angle(double r, double dx, double dy);
-void io_compute_center(INTBIG xc, INTBIG yc, INTBIG x1, INTBIG y1,
-	INTBIG x2, INTBIG y2, INTBIG *cx, INTBIG *cy);
-void io_fixtechlayers(void);
-
-/* prototypes for dialog routines */
-void io_cifoptionsdlog(void);
-void io_dxfoptionsdlog(void);
-void io_edifoptionsdlog(void);
-void io_gdsoptionsdlog(void);
-void io_skilloptionsdlog(void);
+void   io_compute_center(INTBIG xc, INTBIG yc, INTBIG x1, INTBIG y1,
+		INTBIG x2, INTBIG y2, INTBIG *cx, INTBIG *cy);
+void   io_fixrtree(RTNODE*);
+void   io_fixtechlayers(void);
 
 void io_init(INTBIG  *argc, char *argv[], AIDENTRY *thisaid)
 {
@@ -417,23 +356,36 @@ void io_init(INTBIG  *argc, char *argv[], AIDENTRY *thisaid)
 	(void)setvalkey((INTBIG)io_aid, VAID, io_state, DXFFLATTENINPUT|GDSINARRAYS, VINTEGER);
 	io_verbose = 0;
 
-	/* register options dialog */
-	DiaDeclareHook("cifopt", &iocifp, io_cifoptionsdlog);
-	DiaDeclareHook("skillopt", &iocifop, io_skilloptionsdlog);
-	DiaDeclareHook("dxfopt", &iodxfp, io_dxfoptionsdlog);
-	DiaDeclareHook("edifopt", &ioedifp, io_edifoptionsdlog);
-	DiaDeclareHook("gdsopt", &iogdsp, io_gdsoptionsdlog);
+#if IOCIF
+	io_initcif();
+#endif
+#if IODEF
+	io_initdef();
+#endif
+#if IODXF
+	io_initdxf();
+#endif
+#if IOEDIF
+	io_initedif();
+#endif
+#if IOGDS
+	io_initgds();
+#endif
+#if IOSKILL
+	io_initskill();
+#endif
 }
 
 void io_done(void)
 {
 	if (io_mostlayers > 0) efree((char *)io_overlaporder);
-	if (io_tempstringlength != 0) efree(io_tempstring);
+	io_freebininmemory();
 	io_freepostscriptmemory();
-	io_freetextmemory();
+	io_freetextinmemory();
 #if IOCIF
 	io_freecifinmemory();
 	io_freecifparsmemory();
+	io_freecifoutmemory();
 #endif
 #if IODEF
 	io_freedefimemory();
@@ -1244,6 +1196,12 @@ INTBIG io_request(char *command, va_list ap)
 				break;
 #endif
 
+#if IOEAGLE
+			case FEAGLE:
+				l = io_writeeaglelibrary(lib);
+				break;
+#endif
+
 #if IOEDIF
 			case FEDIF:
 				l = io_writeediflibrary(lib);
@@ -1272,6 +1230,12 @@ INTBIG io_request(char *command, va_list ap)
 				break;
 #endif
 
+#if IOPADS
+			case FPADS:
+				l = io_writepadslibrary(lib);
+				break;
+#endif
+
 			case FPOSTSCRIPT:
 			case FPRINTEDPOSTSCRIPT:
 				l = io_writepostscriptlibrary(lib, (format == FPOSTSCRIPT ? 0 : 1));
@@ -1465,7 +1429,7 @@ INTBIG io_getoutputbloat(char *layer)
  * routine to complete formation of library "lib" that has just been read from
  * disk.  Some data is not read, but computed from other data.
  */
-void io_fixnewlib(LIBRARY *lib)
+void io_fixnewlib(LIBRARY *lib, INTBIG statusitem)
 {
 	REGISTER NODEPROTO *np, *pnt;
 	REGISTER NODEINST *ni;
@@ -1544,6 +1508,7 @@ void io_fixnewlib(LIBRARY *lib)
 	}
 
 	/* create network information */
+	if (statusitem > 0) DiaSetText(statusitem, "Building network data...");
 	(void)askaid(net_aid, "total-re-number", (INTBIG)lib);
 
 	/* check for incorrect layer counts on stored variables */
@@ -1939,10 +1904,11 @@ INTSML io_nextplotlayer(INTBIG i)
 	return(io_overlaporder[i]);
 }
 
-/******************** GDS MATH HELPERS ********************/
-
-#if IOGDS
+/******************** MATH HELPERS ********************/
 
+/*
+ * This routine is used by "ioedifo.c" and "routmaze.c".
+ */
 void io_compute_center(INTBIG xc, INTBIG yc, INTBIG x1, INTBIG y1,
 	INTBIG x2, INTBIG y2, INTBIG *cx, INTBIG *cy)
 {
@@ -1978,399 +1944,3 @@ double io_calc_angle(double r, double dx, double dy)
 	if (a2 < 0.0) return(3600.0 - a1);
 	return(a1);
 }
-/****************************** FOREIGN FILE OPTIONS DIALOGS ******************************/
-
-/*
- * special case for the "EDIF Options" dialog
- * Use Schematic View   = 3 (check)
- */
-void io_edifoptionsdlog(void)
-{
-	INTBIG itemHit;
-	REGISTER INTBIG curstate;
-	REGISTER VARIABLE *var;
-
-	if (DiaInitDialog(&io_edifoptionsdialog) != 0) return;
-	var = getvalkey((INTBIG)io_aid, VAID, VINTEGER, io_state);
-	if (var == NOVARIABLE) curstate = 0; else curstate = var->addr;
-	if ((curstate&EDIFSCHEMATIC) != 0) DiaSetControl(3, 1);
-
-	/* loop until done */
-	for(;;)
-	{
-		itemHit = DiaNextHit();
-		if (itemHit == OK || itemHit == CANCEL) break;
-		if (itemHit == 3)
-		{
-			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
-			continue;
-		}
-	}
-
-	if (itemHit != CANCEL)
-	{
-		if (DiaGetControl(3) != 0) curstate |= EDIFSCHEMATIC; else
-			curstate &= ~EDIFSCHEMATIC;
-		(void)setvalkey((INTBIG)io_aid, VAID, io_state,
-			curstate, VINTEGER);
-	}
-	DiaDoneDialog();
-}
-
-/*
- * special case for the "DXF Options" dialog
- * Layer list               = 3 (scroll list)
- * New layer                = 5 (edit text)
- * Input flattens hierarchy = 6 (check)
- */
-void io_dxfoptionsdlog(void)
-{
-	INTBIG i, itemHit;
-	REGISTER INTBIG curstate;
-	REGISTER char **layernames, *pt;
-	REGISTER VARIABLE *dxfvar, *var;
-
-	if (DiaInitDialog(&io_dxfoptionsdialog) != 0) return;
-	DiaInitTextDialog(3, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, 0,
-		SCSELMOUSE|SCREPORT);
-	layernames = (char **)emalloc(el_curtech->layercount * (sizeof (char *)), el_tempcluster);
-	dxfvar = getval((INTBIG)el_curtech, VTECHNOLOGY, VSTRING|VISARRAY,
-		"IO_dxf_layer_names");
-	for(i=0; i<el_curtech->layercount; i++)
-	{
-		if (dxfvar == NOVARIABLE) pt = ""; else
-			pt = ((char **)dxfvar->addr)[i];
-		(void)allocstring(&layernames[i], pt, el_tempcluster);
-		(void)initinfstr();
-		(void)addstringtoinfstr(layername(el_curtech, (INTSML)i));
-		(void)addstringtoinfstr(" (");
-		(void)addstringtoinfstr(pt);
-		(void)addstringtoinfstr(")");
-		DiaStuffLine(3, returninfstr());
-	}
-	DiaSelectLine(3, 0);
-	DiaSetText(5, layernames[0]);
-	var = getvalkey((INTBIG)io_aid, VAID, VINTEGER, io_state);
-	if (var == NOVARIABLE) curstate = 0; else curstate = var->addr;
-	if ((curstate&DXFFLATTENINPUT) != 0) DiaSetControl(6, 1);
-
-	/* loop until done */
-	for(;;)
-	{
-		itemHit = DiaNextHit();
-		if (itemHit == OK || itemHit == CANCEL) break;
-		if (itemHit == 3)
-		{
-			i = DiaGetCurLine(3);
-			DiaSetText(5, layernames[i]);
-			continue;
-		}
-		if (itemHit == 5)
-		{
-			i = DiaGetCurLine(3);
-			pt = DiaGetText(5);
-			(void)reallocstring(&layernames[i], pt, el_tempcluster);
-			(void)initinfstr();
-			(void)addstringtoinfstr(layername(el_curtech, (INTSML)i));
-			(void)addstringtoinfstr(" (");
-			(void)addstringtoinfstr(layernames[i]);
-			(void)addstringtoinfstr(")");
-			DiaSetScrollLine(3, i, returninfstr());
-			continue;
-		}
-		if (itemHit == 6)
-		{
-			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
-			continue;
-		}
-	}
-
-	if (itemHit != CANCEL)
-	{
-		setval((INTBIG)el_curtech, VTECHNOLOGY, "IO_dxf_layer_names",
-			(INTBIG)layernames, VSTRING|VISARRAY|
-				(el_curtech->layercount<<VLENGTHSH));
-		if (DiaGetControl(6) != 0) curstate |= DXFFLATTENINPUT; else
-			curstate &= ~DXFFLATTENINPUT;
-		(void)setvalkey((INTBIG)io_aid, VAID, io_state,
-			curstate, VINTEGER);
-	}
-	for(i=0; i<el_curtech->layercount; i++) efree(layernames[i]);
-	efree((char *)layernames);
-	DiaDoneDialog();
-}
-
-/*
- * special case for the "GDS Options" dialog
- * Layer list                   = 3 (scroll list)
- * New layer                    = 5 (edit text)
- * Input Includes Text          = 6 (check)
- * Input Expands Facets         = 7 (check)
- * Input Instantiates Arrays    = 8 (check)
- * Input Ignores unknown layers = 15 (check)
- * Output Merges Boxes          = 9 (check)
- * Output Arc Angle             = 12 (edit text)
- * Output Arc Sag               = 14 (edit text)
- */
-void io_gdsoptionsdlog(void)
-{
-	INTBIG i, itemHit;
-	REGISTER INTBIG *layernumbers, curstate, val;
-	INTBIG arcres, arcsag;
-	char buf[50];
-	REGISTER VARIABLE *var, *gdsvar;
-
-	if (DiaInitDialog(&io_gdsoptionsdialog) != 0) return;
-	DiaInitTextDialog(3, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, 0,
-		SCSELMOUSE|SCREPORT);
-	layernumbers = (INTBIG *)emalloc(el_curtech->layercount * SIZEOFINTBIG, el_tempcluster);
-	gdsvar = getval((INTBIG)el_curtech, VTECHNOLOGY, VINTEGER|VISARRAY,
-		"IO_gds_layer_numbers");
-	for(i=0; i<el_curtech->layercount; i++)
-	{
-		if (gdsvar == NOVARIABLE) val = -1; else
-			val = ((INTBIG *)gdsvar->addr)[i];
-		layernumbers[i] = val;
-		(void)initinfstr();
-		(void)addstringtoinfstr(layername(el_curtech, (INTSML)i));
-		sprintf(buf, " (%ld)", val);
-		(void)addstringtoinfstr(buf);
-		DiaStuffLine(3, returninfstr());
-	}
-	DiaSelectLine(3, 0);
-	sprintf(buf, "%ld", layernumbers[0]);
-	DiaSetText(5, buf);
-	var = getvalkey((INTBIG)io_aid, VAID, VINTEGER, io_state);
-	if (var == NOVARIABLE) curstate = 0; else curstate = var->addr;
-	if ((curstate&GDSINTEXT) != 0) DiaSetControl(6, 1);
-	if ((curstate&GDSINEXPAND) != 0) DiaSetControl(7, 1);
-	if ((curstate&GDSINARRAYS) != 0) DiaSetControl(8, 1);
-	if ((curstate&GDSOUTMERGE) != 0) DiaSetControl(9, 1);
-	if ((curstate&GDSINIGNOREUKN) != 0) DiaSetControl(15, 1);
-	getcontoursegmentparameters(&arcres, &arcsag);
-	DiaSetText(12, frtoa(arcres*WHOLE/10));
-	DiaSetText(14, latoa(arcsag));
-
-	/* loop until done */
-	for(;;)
-	{
-		itemHit = DiaNextHit();
-		if (itemHit == OK || itemHit == CANCEL) break;
-		if ((itemHit >= 6 && itemHit <= 9) || itemHit == 15)
-		{
-			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
-			continue;
-		}
-		if (itemHit == 3)
-		{
-			i = DiaGetCurLine(3);
-			sprintf(buf, "%ld", layernumbers[i]);
-			DiaSetText(5, buf);
-			continue;
-		}
-		if (itemHit == 5)
-		{
-			i = DiaGetCurLine(3);
-			val = atoi(DiaGetText(5));
-			layernumbers[i] = val;
-			(void)initinfstr();
-			(void)addstringtoinfstr(layername(el_curtech, (INTSML)i));
-			sprintf(buf, " (%ld)", val);
-			(void)addstringtoinfstr(buf);
-			DiaSetScrollLine(3, i, returninfstr());
-			continue;
-		}
-	}
-
-	if (itemHit != CANCEL)
-	{
-		if (DiaGetControl(6) != 0) curstate |= GDSINTEXT; else
-			curstate &= ~GDSINTEXT;
-		if (DiaGetControl(7) != 0) curstate |= GDSINEXPAND; else
-			curstate &= ~GDSINEXPAND;
-		if (DiaGetControl(8) != 0) curstate |= GDSINARRAYS; else
-			curstate &= ~GDSINARRAYS;
-		if (DiaGetControl(9) != 0) curstate |= GDSOUTMERGE; else
-			curstate &= ~GDSOUTMERGE;
-		if (DiaGetControl(15) != 0) curstate |= GDSINIGNOREUKN; else
-			curstate &= ~GDSINIGNOREUKN;
-		(void)setvalkey((INTBIG)io_aid, VAID, io_state,
-			curstate, VINTEGER);
-		setval((INTBIG)el_curtech, VTECHNOLOGY, "IO_gds_layer_numbers",
-			(INTBIG)layernumbers, VINTEGER|VISARRAY|
-				(el_curtech->layercount<<VLENGTHSH));
-		arcres = atofr(DiaGetText(12)) * 10 / WHOLE;
-		arcsag = atola(DiaGetText(14));
-		setcontoursegmentparameters(arcres, arcsag);
-	}
-	efree((char *)layernumbers);
-	DiaDoneDialog();
-}
-
-/*
- * special case for the "CIF Options" dialog
- * Output Mimics Display   = 6 (check)
- * Output Merges Boxes     = 7 (check)
- * Output Instantiates top = 9 (check)
- * Input Squares Wires     = 8 (check)
- * Layer list              = 3 (scroll list)
- * New layer               = 5 (edit text)
- */
-void io_cifoptionsdlog(void)
-{
-	INTBIG i, itemHit;
-	REGISTER INTBIG curstate;
-	REGISTER char **layernames, *pt;
-	REGISTER VARIABLE *var, *cifvar;
-
-	if (DiaInitDialog(&io_cifoptionsdialog) != 0) return;
-	DiaInitTextDialog(3, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, 0,
-		SCSELMOUSE|SCREPORT);
-	layernames = (char **)emalloc(el_curtech->layercount * (sizeof (char *)), el_tempcluster);
-	cifvar = getval((INTBIG)el_curtech, VTECHNOLOGY, VSTRING|VISARRAY,
-		"IO_cif_layer_names");
-	for(i=0; i<el_curtech->layercount; i++)
-	{
-		if (cifvar == NOVARIABLE || i >= getlength(cifvar)) pt = ""; else
-			pt = ((char **)cifvar->addr)[i];
-		(void)allocstring(&layernames[i], pt, el_tempcluster);
-		(void)initinfstr();
-		(void)addstringtoinfstr(layername(el_curtech, (INTSML)i));
-		(void)addstringtoinfstr(" (");
-		(void)addstringtoinfstr(pt);
-		(void)addstringtoinfstr(")");
-		DiaStuffLine(3, returninfstr());
-	}
-	DiaSelectLine(3, 0);
-	DiaSetText(5, layernames[0]);
-	var = getvalkey((INTBIG)io_aid, VAID, VINTEGER, io_state);
-	if (var == NOVARIABLE) curstate = 0; else curstate = var->addr;
-	if ((curstate&CIFOUTEXACT) != 0) DiaSetControl(6, 1);
-	if ((curstate&CIFOUTMERGE) != 0) DiaSetControl(7, 1);
-	if ((curstate&CIFINSQUARE) != 0) DiaSetControl(8, 1);
-	if ((curstate&CIFOUTNOTOPCALL) == 0) DiaSetControl(9, 1);
-
-	/* loop until done */
-	for(;;)
-	{
-		itemHit = DiaNextHit();
-		if (itemHit == OK || itemHit == CANCEL) break;
-		if (itemHit >= 6 && itemHit <= 9)
-		{
-			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
-			continue;
-		}
-		if (itemHit == 3)
-		{
-			i = DiaGetCurLine(3);
-			DiaSetText(5, layernames[i]);
-			continue;
-		}
-		if (itemHit == 5)
-		{
-			i = DiaGetCurLine(3);
-			pt = DiaGetText(5);
-			(void)reallocstring(&layernames[i], pt, el_tempcluster);
-			(void)initinfstr();
-			(void)addstringtoinfstr(layername(el_curtech, (INTSML)i));
-			(void)addstringtoinfstr(" (");
-			(void)addstringtoinfstr(layernames[i]);
-			(void)addstringtoinfstr(")");
-			DiaSetScrollLine(3, i, returninfstr());
-			continue;
-		}
-	}
-
-	if (itemHit != CANCEL)
-	{
-		if (DiaGetControl(6) != 0) curstate |= CIFOUTEXACT; else
-			curstate &= ~CIFOUTEXACT;
-		if (DiaGetControl(7) != 0) curstate |= CIFOUTMERGE; else
-			curstate &= ~CIFOUTMERGE;
-		if (DiaGetControl(8) != 0) curstate |= CIFINSQUARE; else
-			curstate &= ~CIFINSQUARE;
-		if (DiaGetControl(9) == 0) curstate |= CIFOUTNOTOPCALL; else
-			curstate &= ~CIFOUTNOTOPCALL;
-		(void)setvalkey((INTBIG)io_aid, VAID, io_state,
-			curstate, VINTEGER);
-		setval((INTBIG)el_curtech, VTECHNOLOGY, "IO_cif_layer_names",
-			(INTBIG)layernames, VSTRING|VISARRAY|
-				(el_curtech->layercount<<VLENGTHSH));
-	}
-	for(i=0; i<el_curtech->layercount; i++) efree(layernames[i]);
-	efree((char *)layernames);
-	DiaDoneDialog();
-}
-
-/*
- * special case for the "SKILL Options" dialog
- * Layer list            = 3 (scroll list)
- * New layer             = 5 (edit text)
- */
-void io_skilloptionsdlog(void)
-{
-	INTBIG i, itemHit;
-	REGISTER INTBIG len;
-	REGISTER char **layernames, *pt;
-	REGISTER VARIABLE *skillvar;
-
-	if (DiaInitDialog(&io_skilloptionsdialog) != 0) return;
-	DiaInitTextDialog(3, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, 0,
-		SCSELMOUSE|SCREPORT);
-	layernames = (char **)emalloc(el_curtech->layercount * (sizeof (char *)), el_tempcluster);
-	skillvar = getval((INTBIG)el_curtech, VTECHNOLOGY, VSTRING|VISARRAY,
-		"IO_skill_layer_names");
-	if (skillvar != NOVARIABLE) len = getlength(skillvar);
-	for(i=0; i<el_curtech->layercount; i++)
-	{
-		if (skillvar == NOVARIABLE || i >= len) pt = ""; else
-			pt = ((char **)skillvar->addr)[i];
-		(void)allocstring(&layernames[i], pt, el_tempcluster);
-		(void)initinfstr();
-		(void)addstringtoinfstr(layername(el_curtech, (INTSML)i));
-		(void)addstringtoinfstr(" (");
-		(void)addstringtoinfstr(pt);
-		(void)addstringtoinfstr(")");
-		DiaStuffLine(3, returninfstr());
-	}
-	DiaSelectLine(3, 0);
-	DiaSetText(5, layernames[0]);
-
-	/* loop until done */
-	for(;;)
-	{
-		itemHit = DiaNextHit();
-		if (itemHit == OK || itemHit == CANCEL) break;
-		if (itemHit == 3)
-		{
-			i = DiaGetCurLine(3);
-			DiaSetText(5, layernames[i]);
-			continue;
-		}
-		if (itemHit == 5)
-		{
-			i = DiaGetCurLine(3);
-			pt = DiaGetText(5);
-			(void)reallocstring(&layernames[i], pt, el_tempcluster);
-			(void)initinfstr();
-			(void)addstringtoinfstr(layername(el_curtech, (INTSML)i));
-			(void)addstringtoinfstr(" (");
-			(void)addstringtoinfstr(layernames[i]);
-			(void)addstringtoinfstr(")");
-			DiaSetScrollLine(3, i, returninfstr());
-			continue;
-		}
-	}
-
-	if (itemHit != CANCEL)
-	{
-		setval((INTBIG)el_curtech, VTECHNOLOGY, "IO_skill_layer_names",
-			(INTBIG)layernames, VSTRING|VISARRAY|
-				(el_curtech->layercount<<VLENGTHSH));
-	}
-	for(i=0; i<el_curtech->layercount; i++) efree(layernames[i]);
-	efree((char *)layernames);
-	DiaDoneDialog();
-}
-
-#endif
