@@ -33,10 +33,11 @@
 #include "usr.h"
 
 static INTSML  io_outputstage;		/* progress of database (aborting) */
+static char   *io_namesave;			/* for saving the global namespace */
 
 /* prototypes for local routines */
 void io_cleanup(void);
-void io_writenodeproto(NODEPROTO*);
+void io_writenodeproto(NODEPROTO*, INTSML);
 void io_writenodeinst(NODEINST*);
 void io_writearcinst(ARCINST*);
 void io_writenamespace(void);
@@ -52,11 +53,13 @@ INTSML io_writebinlibrary(LIBRARY *lib)
 	INTBIG i, a, n, p, magic, aacount, techcount, nodepprotoindex,
 		portpprotoindex, arcprotoindex, nodeprotoindex, nodeindex,
 		portprotoindex, arcindex, geomindex, curnodeproto, cellindex;
+	INTSML save;
 	REGISTER NODEPROTO *np;
 	REGISTER ARCPROTO *ap;
 	REGISTER PORTPROTO *pp;
 	REGISTER NODEINST *ni;
 	REGISTER ARCINST *ai;
+	REGISTER LIBRARY *olib;
 	REGISTER TECHNOLOGY *tech;
 	REGISTER char *rename, *name, *istrue, *dotaddr, *msg;
 	char *truename, *arg[3];
@@ -141,24 +144,59 @@ INTSML io_writebinlibrary(LIBRARY *lib)
 	techcount = el_maxtech;
 	io_putoutbig(&techcount);
 
-	/* transform database from pointer base to index base */
+	/* count the number of objects in the database */
 	nodeindex = portprotoindex = nodeprotoindex = arcindex = nodepprotoindex =
 		portpprotoindex = arcprotoindex = cellindex = 0;
 
-	/* count and number the objects in the library */
+	/* prepare to locate references to facets in other libraries */
+	for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+	{
+		for(np = olib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+			np->temp2 = 0;
+		for(c = olib->firstcell; c != NOCELL; c = c->nextcell)
+			c->temp2 = 0;
+	}
+	for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+	{
+		for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
+		{
+			if (ni->proto->primindex != 0) continue;
+			ni->proto->temp2 = 1;
+			ni->proto->cell->temp2 = 1;
+		}
+	}
+
+	/* count and number the cells, facets, nodes, arcs, and ports in this library */
 	for(c = lib->firstcell; c != NOCELL; c = c->nextcell)
 		c->temp1 = cellindex++;
 	for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 	{
 		np->temp1 = nodeprotoindex++;
-		for(pp=np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
+		for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 			pp->temp1 = portprotoindex++;
-		for(ai=np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
+		for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
 			ai->temp1 = arcindex++;
-		for(ni=np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
+		for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 			ni->temp1 = nodeindex++;
 	}
 
+	/* count and number the facets and cells in other libraries */
+	for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+	{
+		if (olib == lib) continue;
+		for(np = olib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+		{
+			if (np->temp2 == 0) continue;
+			np->temp1 = nodeprotoindex++;
+			for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
+				pp->temp1 = portprotoindex++;
+		}
+		for(c = olib->firstcell; c != NOCELL; c = c->nextcell)
+		{
+			if (c->temp2 != 0) c->temp1 = cellindex++;
+		}
+	}
+
 	/* count and number the primitive node and port prototypes */
 	for(tech = el_technologies; tech != NOTECHNOLOGY; tech = tech->nexttechnology)
 	{
@@ -232,16 +270,31 @@ INTSML io_writebinlibrary(LIBRARY *lib)
 	/* write total number of arcinsts, nodeinsts, and ports in each facet */
 	for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 	{
-		for(a=0, ai=np->firstarcinst; ai!=NOARCINST; ai=ai->nextarcinst) a++;
+		for(a = 0, ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst) a++;
 		io_putoutbig(&a);
-		for(n=0, ni=np->firstnodeinst; ni!=NONODEINST; ni=ni->nextnodeinst) n++;
+		for(n = 0, ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst) n++;
 		io_putoutbig(&n);
-		for(p=0, pp=np->firstportproto; pp!=NOPORTPROTO; pp=pp->nextportproto) p++;
+		for(p = 0, pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto) p++;
 		io_putoutbig(&p);
 		if (io_verbose > 0) ttyputmsg("Facet %s has %ld arcs, %ld nodes, %ld ports",
 			describenodeproto(np), a, n, p);
 	}
 
+	/* write dummy numbers of arcinsts and nodeinst; count ports for external facets */
+	for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+	{
+		if (olib == lib) continue;
+		for(np = olib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+		{
+			if (np->temp2 == 0) continue;
+			a = -1;
+			io_putoutbig(&a);
+			io_putoutbig(&a);
+			for(p = 0, pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto) p++;
+			io_putoutbig(&p);
+		}
+	}
+
 	/* write the names of technologies and primitive prototypes */
 	for(tech = el_technologies; tech != NOTECHNOLOGY; tech = tech->nexttechnology)
 	{
@@ -322,7 +375,7 @@ INTSML io_writebinlibrary(LIBRARY *lib)
 		io_writevariables(v->firstvar, v->numvar);
 	}
 
-	/* write the rest of the database */
+	/* write all of the cell names in this library */
 	for(c = lib->firstcell; c != NOCELL; c = c->nextcell)
 	{
 		if (stopping("Binary output") != 0) longjmp(io_filerror, 1);
@@ -335,11 +388,44 @@ INTSML io_writebinlibrary(LIBRARY *lib)
 
 		if (io_verbose > 0) ttyputmsg("Wrote cell %s", c->cellname);
 	}
+
+	/* write all of the cell names in external libraries */
+	for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+	{
+		if (olib == lib) continue;
+		for(c = olib->firstcell; c != NOCELL; c = c->nextcell)
+		{
+			if (c->temp2 == 0) continue;
+
+			/* write the cell name */
+			io_putstring(c->cellname);
+
+			/* write dummy variable information */
+			save = c->numvar;   c->numvar = 0;
+			io_writevariables(c->firstvar, c->numvar);
+			c->numvar = save;
+		}
+	}
+
+	/* write all of the facets in this library */
 	for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 	{
 		if (stopping("Binary output") != 0) longjmp(io_filerror, 1);
-		io_writenodeproto(np);
+		io_writenodeproto(np, 1);
 	}
+
+	/* write all of the facets in external libraries */
+	for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+	{
+		if (olib == lib) continue;
+		for(np = olib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+		{
+			if (np->temp2 == 0) continue;
+			io_writenodeproto(np, 0);
+		}
+	}
+
+	/* write all of the arcs and nodes in this library */
 	for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 	{
 		if (stopping("Binary output") != 0) longjmp(io_filerror, 1);
@@ -392,10 +478,11 @@ void io_cleanup(void)
 
 /******************** COMPONENT OUTPUT ********************/
 
-void io_writenodeproto(NODEPROTO *np)
+void io_writenodeproto(NODEPROTO *np, INTSML thislib)
 {
 	INTBIG i;
 	REGISTER PORTPROTO *pp;
+	REGISTER LIBRARY *instlib;
 
 	/* write facet information */
 	io_putoutbig(&np->cell->temp1);
@@ -410,41 +497,58 @@ void io_writenodeproto(NODEPROTO *np)
 	io_putoutbig(&np->lowy);
 	io_putoutbig(&np->highy);
 
+	if (thislib == 0)
+	{
+		instlib = np->cell->lib;
+		io_putstring(instlib->libfile);
+	}
+
 	/* write the number of portprotos on this nodeproto */
 	i = 0;
-	for(pp=np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto) i++;
+	for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto) i++;
 	io_putoutbig(&i);
 
 	/* write the portprotos on this nodeproto */
-	for(pp=np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
+	for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 	{
-		/* write the connecting subnodeinst for this portproto */
-		if (pp->subnodeinst == NONODEINST) i = -1; else
-			i = pp->subnodeinst->temp1;
-		io_putoutbig(&i);
+		if (thislib != 0)
+		{
+			/* write the connecting subnodeinst for this portproto */
+			if (pp->subnodeinst == NONODEINST) i = -1; else
+				i = pp->subnodeinst->temp1;
+			io_putoutbig(&i);
 
-		/* write the portproto index in the subnodeinst */
-		if (pp->subnodeinst == NONODEINST || pp->subportproto == NOPORTPROTO) i = -1; else
-			i = pp->subportproto->temp1;
-		io_putoutbig(&i);
+			/* write the portproto index in the subnodeinst */
+			if (pp->subnodeinst == NONODEINST || pp->subportproto == NOPORTPROTO) i = -1; else
+				i = pp->subportproto->temp1;
+			io_putoutbig(&i);
+		}
 
-		/* write the portproto name and text descriptor */
+		/* write the portproto name */
 		io_putstring(pp->protoname);
-		io_putoutbig(&pp->textdescript);
 
-		/* write the portproto aid information */
-		io_putoutbig(&pp->userbits);
+		if (thislib != 0)
+		{
+			/* write the text descriptor */
+			io_putoutbig(&pp->textdescript);
 
-		/* write variable information */
-		io_writevariables(pp->firstvar, pp->numvar);
+			/* write the portproto aid information */
+			io_putoutbig(&pp->userbits);
+
+			/* write variable information */
+			io_writevariables(pp->firstvar, pp->numvar);
+		}
 	}
 
-	/* write aid information */
-	io_putoutbig(&np->adirty);
-	io_putoutbig(&np->userbits);
+	if (thislib != 0)
+	{
+		/* write aid information */
+		io_putoutbig(&np->adirty);
+		io_putoutbig(&np->userbits);
 
-	/* write variable information */
-	io_writevariables(np->firstvar, np->numvar);
+		/* write variable information */
+		io_writevariables(np->firstvar, np->numvar);
+	}
 
 	if (io_verbose > 0) ttyputmsg("Wrote facet %s", describenodeproto(np));
 }
@@ -470,11 +574,11 @@ void io_writenodeinst(NODEINST *ni)
 
 	/* count the arc ports */
 	i = 0;
-	for(pi=ni->firstportarcinst; pi!=NOPORTARCINST; pi=pi->nextportarcinst) i++;
+	for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst) i++;
 	io_putoutbig(&i);
 
 	/* write the arc ports */
-	for(pi=ni->firstportarcinst; pi!=NOPORTARCINST; pi=pi->nextportarcinst)
+	for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
 	{
 		/* write the arcinst index (and the particular end on that arc) */
 		ai = pi->conarcinst;
@@ -491,11 +595,11 @@ void io_writenodeinst(NODEINST *ni)
 
 	/* count the exported ports */
 	i = 0;
-	for(pe=ni->firstportexpinst; pe!=NOPORTEXPINST; pe=pe->nextportexpinst) i++;
+	for(pe = ni->firstportexpinst; pe != NOPORTEXPINST; pe = pe->nextportexpinst) i++;
 	io_putoutbig(&i);
 
 	/* write the exported ports */
-	for(pe=ni->firstportexpinst; pe!=NOPORTEXPINST; pe=pe->nextportexpinst)
+	for(pe = ni->firstportexpinst; pe != NOPORTEXPINST; pe = pe->nextportexpinst)
 	{
 		io_putoutbig(&pe->exportproto->temp1);
 
