@@ -1026,7 +1026,7 @@ static void io_findbb(stentry *sym)		/* find bb for sym */
 	if (ob == NIL)			/* empty symbol */
 	{
 		char mess[100];
-		(void)sprintf(mess, "symbol %lu is empty, a null bb at 0,0 is assumed",
+		(void)sprintf(mess, "symbol %lu has no geometry in it",
 			sym->st_symnumber);
 		io_report(mess, ADVISORY);
 		sym->st_bb.l = 0;   sym->st_bb.r = 0;
@@ -2337,8 +2337,6 @@ INTSML io_fatalerrors(void)
 
 void io_report(char *mess, INTBIG kind)
 {
-	extern jmp_buf io_filerror;
-
 	io_identify();
 	io_cifecounts[kind]++;
 
