@@ -102,6 +102,7 @@ DXFLAYER  *io_dxfgetlayer(char *name);
 INTSML     io_dxfgetnextline(FILE *io, char *text, INTSML canBeBlank);
 INTSML     io_dxfgetnextpair(FILE *io, INTSML *groupID, char **text);
 INTSML     io_dxfignoreentity(FILE *io);
+void       io_dxfoptionsdlog(void);
 void       io_dxfpushpair(INTSML groupID, char *text);
 INTSML     io_dxfreadcircleentity(FILE *io, INTBIG *x, INTBIG *y, INTBIG *z, INTBIG *rad, DXFLAYER **layer);
 INTSML     io_dxfreadentities(FILE *io, LIBRARY *lib);
@@ -156,12 +157,23 @@ void io_freedxfmemory(void)
 #endif
 }
 
+/*
+ * Routine to initialize DXF I/O.
+ */
+void io_initdxf(void)
+{
+	extern COMCOMP io_dxfp;
+
+	DiaDeclareHook("dxfopt", &io_dxfp, io_dxfoptionsdlog);
+}
+
 INTSML io_writedxflibrary(LIBRARY *lib)
 {
 	char file[100], *truename;
 	REGISTER char *name, *pt;
 	REGISTER INTBIG i, j, len, code;
 	REGISTER NODEINST *ni;
+	REGISTER LIBRARY *olib;
 	REGISTER VARIABLE *varheadertext, *varheaderid;
 	REGISTER NODEPROTO *np;
 	static char *ignorefromheader[] = {"$DWGCODEPAGE", "$HANDSEED", "$SAVEIMAGES", 0};
@@ -226,8 +238,9 @@ INTSML io_writedxflibrary(LIBRARY *lib)
 	io_dxfentityhandle = 0x100;
 	fprintf(io_fileout, "  0\nSECTION\n");
 	fprintf(io_fileout, "  2\nBLOCKS\n");
-	for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
-		np->temp1 = 0;
+	for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+		for(np = olib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+			np->temp1 = 0;
 	for(ni = lib->curnodeproto->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 	{
 		np = ni->proto;
@@ -1675,7 +1688,7 @@ INTSML io_dxfreadtextentity(FILE *io, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG
 			*ly = *hy = y;
 		} else
 		{
-			us_textsize(el_curwindowpart, saveMsg, &px, &py);
+			screengettextsize(el_curwindowpart, saveMsg, &px, &py);
 			*lx = x;	*hx = x + height*px/py;
 			*ly = y;	*hy = y + height;
 		}
@@ -2172,4 +2185,99 @@ char *io_dxfblockname(char *name)
 	return(returninfstr());
 }
 
+/* DXF Options */
+DIALOGITEM io_dxfoptionsdialogitems[] =
+{
+ /*  1 */ {0, {144,344,168,416}, BUTTON, "OK"},
+ /*  2 */ {0, {144,248,168,320}, BUTTON, "Cancel"},
+ /*  3 */ {0, {8,16,168,231}, SCROLL, ""},
+ /*  4 */ {0, {8,240,24,320}, MESSAGE, "DXF Layer:"},
+ /*  5 */ {0, {8,324,24,430}, EDITTEXT, ""},
+ /*  6 */ {0, {32,240,48,430}, CHECK, "Input flattens hierarchy"}
+};
+DIALOG io_dxfoptionsdialog = {{50,75,228,514}, "DXF Options", 6, io_dxfoptionsdialogitems};
+
+/*
+ * special case for the "DXF Options" dialog
+ * Layer list               = 3 (scroll list)
+ * New layer                = 5 (edit text)
+ * Input flattens hierarchy = 6 (check)
+ */
+void io_dxfoptionsdlog(void)
+{
+	INTBIG i, itemHit;
+	REGISTER INTBIG curstate;
+	REGISTER char **layernames, *pt;
+	REGISTER VARIABLE *dxfvar, *var;
+
+	if (DiaInitDialog(&io_dxfoptionsdialog) != 0) return;
+	DiaInitTextDialog(3, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, 0,
+		SCSELMOUSE|SCREPORT);
+	layernames = (char **)emalloc(el_curtech->layercount * (sizeof (char *)), el_tempcluster);
+	dxfvar = getval((INTBIG)el_curtech, VTECHNOLOGY, VSTRING|VISARRAY,
+		"IO_dxf_layer_names");
+	for(i=0; i<el_curtech->layercount; i++)
+	{
+		if (dxfvar == NOVARIABLE) pt = ""; else
+			pt = ((char **)dxfvar->addr)[i];
+		(void)allocstring(&layernames[i], pt, el_tempcluster);
+		(void)initinfstr();
+		(void)addstringtoinfstr(layername(el_curtech, (INTSML)i));
+		(void)addstringtoinfstr(" (");
+		(void)addstringtoinfstr(pt);
+		(void)addstringtoinfstr(")");
+		DiaStuffLine(3, returninfstr());
+	}
+	DiaSelectLine(3, 0);
+	DiaSetText(5, layernames[0]);
+	var = getvalkey((INTBIG)io_aid, VAID, VINTEGER, io_state);
+	if (var == NOVARIABLE) curstate = 0; else curstate = var->addr;
+	if ((curstate&DXFFLATTENINPUT) != 0) DiaSetControl(6, 1);
+
+	/* loop until done */
+	for(;;)
+	{
+		itemHit = DiaNextHit();
+		if (itemHit == OK || itemHit == CANCEL) break;
+		if (itemHit == 3)
+		{
+			i = DiaGetCurLine(3);
+			DiaSetText(5, layernames[i]);
+			continue;
+		}
+		if (itemHit == 5)
+		{
+			i = DiaGetCurLine(3);
+			pt = DiaGetText(5);
+			(void)reallocstring(&layernames[i], pt, el_tempcluster);
+			(void)initinfstr();
+			(void)addstringtoinfstr(layername(el_curtech, (INTSML)i));
+			(void)addstringtoinfstr(" (");
+			(void)addstringtoinfstr(layernames[i]);
+			(void)addstringtoinfstr(")");
+			DiaSetScrollLine(3, i, returninfstr());
+			continue;
+		}
+		if (itemHit == 6)
+		{
+			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
+			continue;
+		}
+	}
+
+	if (itemHit != CANCEL)
+	{
+		setval((INTBIG)el_curtech, VTECHNOLOGY, "IO_dxf_layer_names",
+			(INTBIG)layernames, VSTRING|VISARRAY|
+				(el_curtech->layercount<<VLENGTHSH));
+		if (DiaGetControl(6) != 0) curstate |= DXFFLATTENINPUT; else
+			curstate &= ~DXFFLATTENINPUT;
+		(void)setvalkey((INTBIG)io_aid, VAID, io_state,
+			curstate, VINTEGER);
+	}
+	for(i=0; i<el_curtech->layercount; i++) efree(layernames[i]);
+	efree((char *)layernames);
+	DiaDoneDialog();
+}
+
 #endif  /* IODXF - at top */
