@@ -39,6 +39,7 @@
 #include "tech.h"
 #include "tecgen.h"
 #include "network.h"
+#include "edialogs.h"
 #include <math.h>
 
 #define WORD	   256
@@ -134,6 +135,17 @@ INTBIG EO_free_stream(EO_STREAM_PTR);
 char *EO_get_timestamp(char *);
 char *EO_make_string(char *);
 char *EO_get_exp(double);
+void io_edifoptionsdlog(void);
+
+/*
+ * Routine to initialize EDIF I/O.
+ */
+void io_initedif(void)
+{
+	extern COMCOMP io_edifp;
+
+	DiaDeclareHook("edifopt", &io_edifp, io_edifoptionsdlog);
+}
 
 /*
  * routine to write a ".edif" file (or a ".foot" file for layouts)
@@ -145,6 +157,7 @@ INTSML io_writeediflibrary(LIBRARY *lib)
 	REGISTER NODEPROTO *lnp, *np;
 	REGISTER INTSML i, backannotate;
 	extern AIDENTRY *net_aid;
+	REGISTER LIBRARY *olib;
 	REGISTER TECHNOLOGY *tech;
 	INTBIG fun, rgrid;
 	VARIABLE *var;
@@ -214,8 +227,9 @@ INTSML io_writeediflibrary(LIBRARY *lib)
 		for (tech = el_technologies; tech != NOTECHNOLOGY; tech = tech->nexttechnology)
 			for (lnp = tech->firstnodeproto; lnp != NONODEPROTO; lnp = lnp->nextnodeproto)
 				lnp->temp1 = 0;
-		for (lnp = lib->firstnodeproto; lnp != NONODEPROTO; lnp = lnp->nextnodeproto)
-			lnp->temp1 = 0;
+		for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+			for (lnp = olib->firstnodeproto; lnp != NONODEPROTO; lnp = lnp->nextnodeproto)
+				lnp->temp1 = 0;
 
 		/* search recursively for all primitives used */
 		if (io_edifsearch(np) != 0)
@@ -257,7 +271,8 @@ INTSML io_writeediflibrary(LIBRARY *lib)
 
 	/* search recursively for all external libraries required */
 	backannotate = 0;
-	for (lnp = lib->firstnodeproto; lnp != NONODEPROTO; lnp = lnp->nextnodeproto)
+	for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+		for (lnp = olib->firstnodeproto; lnp != NONODEPROTO; lnp = lnp->nextnodeproto)
 	{
 		lnp->temp1 = 0;
 		lnp->temp2 = 0;
@@ -266,8 +281,9 @@ INTSML io_writeediflibrary(LIBRARY *lib)
 	io_edifextsearch(np);
 
 	/* mark all node prototypes for final netlisting */
-	for (lnp = lib->firstnodeproto; lnp != NONODEPROTO; lnp = lnp->nextnodeproto)
-		lnp->temp1 = 0;
+	for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+		for (lnp = olib->firstnodeproto; lnp != NONODEPROTO; lnp = lnp->nextnodeproto)
+			lnp->temp1 = 0;
 
 	/* write out all external references in the library */
 	if (io_edifoextlibcount > 0)
@@ -2731,5 +2747,51 @@ char *EO_get_exp(double val)
 	return(result);
 }
 
+/* EDIF Options */
+DIALOGITEM io_edifoptionsdialogitems[] =
+{
+ /*  1 */ {0, {32,120,56,192}, BUTTON, "OK"},
+ /*  2 */ {0, {32,16,56,88}, BUTTON, "Cancel"},
+ /*  3 */ {0, {8,8,24,200}, CHECK, "Use Schematic View"}
+};
+DIALOG io_edifoptionsdialog = {{50,75,115,284}, "EDIF Options", 3, io_edifoptionsdialogitems};
+
+/*
+ * special case for the "EDIF Options" dialog
+ * Use Schematic View   = 3 (check)
+ */
+void io_edifoptionsdlog(void)
+{
+	INTBIG itemHit;
+	REGISTER INTBIG curstate;
+	REGISTER VARIABLE *var;
+
+	if (DiaInitDialog(&io_edifoptionsdialog) != 0) return;
+	var = getvalkey((INTBIG)io_aid, VAID, VINTEGER, io_state);
+	if (var == NOVARIABLE) curstate = 0; else curstate = var->addr;
+	if ((curstate&EDIFSCHEMATIC) != 0) DiaSetControl(3, 1);
+
+	/* loop until done */
+	for(;;)
+	{
+		itemHit = DiaNextHit();
+		if (itemHit == OK || itemHit == CANCEL) break;
+		if (itemHit == 3)
+		{
+			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
+			continue;
+		}
+	}
+
+	if (itemHit != CANCEL)
+	{
+		if (DiaGetControl(3) != 0) curstate |= EDIFSCHEMATIC; else
+			curstate &= ~EDIFSCHEMATIC;
+		(void)setvalkey((INTBIG)io_aid, VAID, io_state,
+			curstate, VINTEGER);
+	}
+	DiaDoneDialog();
+}
+
 #endif  /* IOEDIF - at top */
 
