@@ -38,6 +38,7 @@
 #include "dbcontour.h"
 #include "tecgen.h"
 #include "tecart.h"
+#include "edialogs.h"
 
 #define OLDGDS 1
 
@@ -125,9 +126,10 @@ void io_gds_int2_arr(INTSML*, INTSML);
 void io_gds_int4_arr(INTBIG*, INTSML);
 void io_gds_string(char*, INTSML);
 UINTBIG io_gds_floatint(INTBIG);
-void io_gds_write_polygon(INTSML, TECHNOLOGY*, INTBIG*, INTBIG*, INTSML, float);
+void io_gds_write_polygon(INTSML, TECHNOLOGY*, INTBIG*, INTBIG*, INTSML);
 char *io_gds_makename(char*);
 void io_gds_cvfloat8(double*, UINTBIG*, UINTBIG*);
+void io_gdsoptionsdlog(void);
 
 /*
  * Routine to free all memory associated with this module.
@@ -138,12 +140,23 @@ void io_freegdsoutmemory(void)
 	if (io_empty != 0) efree((char *)io_empty);
 }
 
+/*
+ * Routine to initialize GDS I/O.
+ */
+void io_initgds(void)
+{
+	extern COMCOMP io_gdsp;
+
+	DiaDeclareHook("gdsopt", &io_gdsp, io_gdsoptionsdlog);
+}
+
 INTSML io_writegdslibrary(LIBRARY *lib)
 {
 	char *name, *truename;
 	REGISTER NODEPROTO *np;
 	REGISTER VARIABLE *var;
 	REGISTER TECHNOLOGY *tech;
+	REGISTER LIBRARY *olib;
 
 	/* create the proper disk file for the GDS II */
 	if (lib->curnodeproto == NONODEPROTO)
@@ -204,8 +217,9 @@ INTSML io_writegdslibrary(LIBRARY *lib)
 	io_gds_bgnlib(lib->curnodeproto);
 
 	/* write the GDS-II */
-	for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
-		np->temp1 = 0;
+	for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+		for(np = olib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+			np->temp1 = 0;
 	io_gdswritecell(lib->curnodeproto);
 
 	/* clean up */
@@ -982,7 +996,7 @@ UINTBIG io_gds_floatint(INTBIG val)       /* Convert integer to gds float */
  * We assume that it does not exceed 200 points.
  */
 void io_gds_write_polygon(INTSML layer, TECHNOLOGY *tech, INTBIG *xbuf, INTBIG *ybuf,
-	INTSML count, float area)
+	INTSML count)
 {
 	static POLYGON *gds_poly = NOPOLYGON;
 	INTSML i;
@@ -1080,4 +1094,132 @@ void io_gds_cvfloat8(double *a, UINTBIG *m, UINTBIG *n)
 	*n = (UINTBIG)frac;   *m = highmantissa | (exponent<<24);
 }
 
+/* GDS Options */
+DIALOGITEM io_gdsoptionsdialogitems[] =
+{
+ /*  1 */ {0, {224,344,248,416}, BUTTON, "OK"},
+ /*  2 */ {0, {224,248,248,320}, BUTTON, "Cancel"},
+ /*  3 */ {0, {8,16,248,231}, SCROLL, ""},
+ /*  4 */ {0, {8,240,24,320}, MESSAGE, "GDS Layer:"},
+ /*  5 */ {0, {8,324,24,430}, EDITTEXT, ""},
+ /*  6 */ {0, {32,240,48,464}, CHECK, "Input Includes Text"},
+ /*  7 */ {0, {56,240,72,464}, CHECK, "Input Expands Facets"},
+ /*  8 */ {0, {80,240,96,464}, CHECK, "Input Instantiates Arrays"},
+ /*  9 */ {0, {128,240,144,464}, CHECK, "Output Merges Boxes"},
+ /* 10 */ {0, {152,240,168,430}, MESSAGE, "Output Arc Conversion:"},
+ /* 11 */ {0, {176,250,192,394}, MESSAGE, "Maximum arc angle:"},
+ /* 12 */ {0, {176,396,192,464}, EDITTEXT, ""},
+ /* 13 */ {0, {200,250,216,394}, MESSAGE, "Maximum arc sag:"},
+ /* 14 */ {0, {200,396,216,464}, EDITTEXT, ""},
+ /* 15 */ {0, {104,240,120,464}, CHECK, "Input Ignores Unknown Layers"}
+};
+DIALOG io_gdsoptionsdialog = {{50,75,307,548}, "GDS Options", 15, io_gdsoptionsdialogitems};
+
+/*
+ * special case for the "GDS Options" dialog
+ * Layer list                   = 3 (scroll list)
+ * New layer                    = 5 (edit text)
+ * Input Includes Text          = 6 (check)
+ * Input Expands Facets         = 7 (check)
+ * Input Instantiates Arrays    = 8 (check)
+ * Input Ignores unknown layers = 15 (check)
+ * Output Merges Boxes          = 9 (check)
+ * Output Arc Angle             = 12 (edit text)
+ * Output Arc Sag               = 14 (edit text)
+ */
+void io_gdsoptionsdlog(void)
+{
+	INTBIG i, itemHit;
+	REGISTER INTBIG *layernumbers, curstate, val;
+	INTBIG arcres, arcsag;
+	char buf[50];
+	REGISTER VARIABLE *var, *gdsvar;
+
+	if (DiaInitDialog(&io_gdsoptionsdialog) != 0) return;
+	DiaInitTextDialog(3, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, 0,
+		SCSELMOUSE|SCREPORT);
+	layernumbers = (INTBIG *)emalloc(el_curtech->layercount * SIZEOFINTBIG, el_tempcluster);
+	gdsvar = getval((INTBIG)el_curtech, VTECHNOLOGY, VINTEGER|VISARRAY,
+		"IO_gds_layer_numbers");
+	for(i=0; i<el_curtech->layercount; i++)
+	{
+		if (gdsvar == NOVARIABLE) val = -1; else
+			val = ((INTBIG *)gdsvar->addr)[i];
+		layernumbers[i] = val;
+		(void)initinfstr();
+		(void)addstringtoinfstr(layername(el_curtech, (INTSML)i));
+		sprintf(buf, " (%ld)", val);
+		(void)addstringtoinfstr(buf);
+		DiaStuffLine(3, returninfstr());
+	}
+	DiaSelectLine(3, 0);
+	sprintf(buf, "%ld", layernumbers[0]);
+	DiaSetText(5, buf);
+	var = getvalkey((INTBIG)io_aid, VAID, VINTEGER, io_state);
+	if (var == NOVARIABLE) curstate = 0; else curstate = var->addr;
+	if ((curstate&GDSINTEXT) != 0) DiaSetControl(6, 1);
+	if ((curstate&GDSINEXPAND) != 0) DiaSetControl(7, 1);
+	if ((curstate&GDSINARRAYS) != 0) DiaSetControl(8, 1);
+	if ((curstate&GDSOUTMERGE) != 0) DiaSetControl(9, 1);
+	if ((curstate&GDSINIGNOREUKN) != 0) DiaSetControl(15, 1);
+	getcontoursegmentparameters(&arcres, &arcsag);
+	DiaSetText(12, frtoa(arcres*WHOLE/10));
+	DiaSetText(14, latoa(arcsag));
+
+	/* loop until done */
+	for(;;)
+	{
+		itemHit = DiaNextHit();
+		if (itemHit == OK || itemHit == CANCEL) break;
+		if ((itemHit >= 6 && itemHit <= 9) || itemHit == 15)
+		{
+			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
+			continue;
+		}
+		if (itemHit == 3)
+		{
+			i = DiaGetCurLine(3);
+			sprintf(buf, "%ld", layernumbers[i]);
+			DiaSetText(5, buf);
+			continue;
+		}
+		if (itemHit == 5)
+		{
+			i = DiaGetCurLine(3);
+			val = atoi(DiaGetText(5));
+			layernumbers[i] = val;
+			(void)initinfstr();
+			(void)addstringtoinfstr(layername(el_curtech, (INTSML)i));
+			sprintf(buf, " (%ld)", val);
+			(void)addstringtoinfstr(buf);
+			DiaSetScrollLine(3, i, returninfstr());
+			continue;
+		}
+	}
+
+	if (itemHit != CANCEL)
+	{
+		if (DiaGetControl(6) != 0) curstate |= GDSINTEXT; else
+			curstate &= ~GDSINTEXT;
+		if (DiaGetControl(7) != 0) curstate |= GDSINEXPAND; else
+			curstate &= ~GDSINEXPAND;
+		if (DiaGetControl(8) != 0) curstate |= GDSINARRAYS; else
+			curstate &= ~GDSINARRAYS;
+		if (DiaGetControl(9) != 0) curstate |= GDSOUTMERGE; else
+			curstate &= ~GDSOUTMERGE;
+		if (DiaGetControl(15) != 0) curstate |= GDSINIGNOREUKN; else
+			curstate &= ~GDSINIGNOREUKN;
+		(void)setvalkey((INTBIG)io_aid, VAID, io_state,
+			curstate, VINTEGER);
+		setval((INTBIG)el_curtech, VTECHNOLOGY, "IO_gds_layer_numbers",
+			(INTBIG)layernumbers, VINTEGER|VISARRAY|
+				(el_curtech->layercount<<VLENGTHSH));
+		arcres = atofr(DiaGetText(12)) * 10 / WHOLE;
+		arcsag = atola(DiaGetText(14));
+		setcontoursegmentparameters(arcres, arcsag);
+	}
+	efree((char *)layernumbers);
+	DiaDoneDialog();
+}
+
 #endif  /* IOGDS - at top */
