@@ -69,7 +69,7 @@ static float io_hpfontheight[] =
 };
 
 /* prototypes for local routines */
-void io_hpwritepolygon(INTSML, TECHNOLOGY*, INTBIG*, INTBIG*, INTSML, float);
+void io_hpwritepolygon(INTSML, TECHNOLOGY*, INTBIG*, INTBIG*, INTSML);
 INTSML io_hppoly(POLYGON*, WINDOWPART*);
 void io_hpline(INTBIG, INTBIG, INTBIG, INTBIG, INTSML);
 void io_hparc(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTSML);
@@ -84,7 +84,7 @@ void io_hpmove(INTBIG, INTBIG);
 void io_hpdraw(INTBIG, INTBIG);
 void io_hpcmd(char[]);
 
-void io_hpwritepolygon(INTSML layer, TECHNOLOGY *tech, INTBIG *x, INTBIG *y, INTSML count, float area)
+void io_hpwritepolygon(INTSML layer, TECHNOLOGY *tech, INTBIG *x, INTBIG *y, INTSML count)
 {
 	io_hpfilledpolygon(x, y, count, layer);
 }
