@@ -48,6 +48,7 @@
 #include "egraphics.h"
 #include "eio.h"
 #include "usr.h"
+#include "edialogs.h"
 
 /* #define CLIP 1 */  /* set if the bounding box is used as the clip path, for EPS only */
 
@@ -75,6 +76,9 @@ static INTSML     io_pspolygonput;			/* PostScript for polygon put out */
 static INTSML     io_psfilledpolygonput;	/* PostScript for filled polygon put out */
 static INTSML     io_psstringput;			/* PostScript for strings put out */
 static INTBIG    *io_redmap, *io_greenmap, *io_bluemap;
+static INTBIG     io_lastred, io_lastgreen, io_lastblue;
+static INTBIG     io_pspolygoncount;		/* number of polygons in job so far */
+static INTBIG     io_pspolygontotal;		/* number of polygons in job */
 
 GRAPHICS io_psblack = {LAYERO, BLACK, SOLIDC, SOLIDC,
 	{0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF}, NOVARIABLE, 0};
@@ -89,6 +93,7 @@ void   io_psdot(INTBIG, INTBIG);
 void   io_psline(INTBIG, INTBIG, INTBIG, INTBIG, INTSML);
 INTSML io_pspattern(GRAPHICS*);
 INTSML io_pspoly(POLYGON*, WINDOWPART*);
+INTSML io_pspolycount(POLYGON*, WINDOWPART*);
 void   io_pspolygon(INTBIG*, INTBIG*, INTSML, GRAPHICS*);
 void   io_pstext(INTSML, INTBIG, INTBIG, INTBIG, INTBIG, INTSML, char*);
 void   io_pswrite(char *s, ...);
@@ -113,15 +118,17 @@ INTSML io_writepostscriptlibrary(LIBRARY *lib, INTBIG printit)
 	REGISTER INTBIG i, numsyncs;
 	REGISTER NODEPROTO *np;
 	REGISTER INTSML synchother;
+	REGISTER LIBRARY *olib;
 	REGISTER VARIABLE *var, *vardate;
 	REGISTER UINTBIG mdate;
 	extern COMCOMP us_yesnop;
 
-	/* see if there are synchronization links in this library */
+	/* see if there are synchronization links */
 	synchother = 0;
 	if (printit == 0)
 	{
-		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+		for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+			for(np = olib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 		{
 			var = getvalkey((INTBIG)np, VNODEPROTO, VSTRING, io_postscriptfilename);
 			if (var != NOVARIABLE && np != lib->curnodeproto) synchother++;
@@ -146,9 +153,10 @@ INTSML io_writepostscriptlibrary(LIBRARY *lib, INTBIG printit)
 	if (synchother == 0)
 		return(io_writepostscriptfacet(np, 0, printit));
 
-	/* synchronize all facets in the library */
+	/* synchronize all facets */
 	numsyncs = 0;
-	for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+	for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+		for(np = olib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 	{
 		var = getvalkey((INTBIG)np, VNODEPROTO, VSTRING, io_postscriptfilename);
 		if (var == NOVARIABLE) continue;
@@ -192,13 +200,14 @@ INTSML io_writepostscriptfacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 	char file[100], *facetname, *truename, *execpars[5];
 	INTBIG lx, hx, ly, hy, hlx, hhx, hly, hhy, gridlx, gridly, gridx, gridy,
 		cx, cy, i, j, psulx, psuhx, psuly, psuhy, sulx, suhx,
-		suly, suhy, sslx, sshx, ssly, sshy;
+		suly, suhy, sslx, sshx, ssly, sshy, oldoptions;
 	INTBIG bblx, bbhx, bbly, bbhy, t1, t2, unitsx, unitsy, printprocess;
 	UINTBIG curdate;
 	REGISTER NODEPROTO *onp;
 	VARIABLE *var;
 	REGISTER VARIABLE *varstate, *varred, *vargreen, *varblue;
 	static POLYGON *poly = NOPOLYGON;
+	extern DIALOG us_progressdialog;
 
 	/* clear flags that tell whether headers have been included */
 	io_psdotput = 0;
@@ -303,6 +312,13 @@ INTSML io_writepostscriptfacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 		}
 	}
 
+	if (io_verbose < 0)
+	{
+		if (DiaInitDialog(&us_progressdialog) != 0) return(1);
+		DiaPercent(1, 0);
+		DiaSetText(2, "Preparing PostScript Output...");
+	}
+
 	/* build pseudowindow for text scaling */
 	psulx = psuly = 0;
 	psuhx = psuhy = 1000;
@@ -530,7 +546,22 @@ INTSML io_writepostscriptfacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 	/* initialize list of GRAPHICS modules that have been put out */
 	io_psgraphicsseencount = 0;
 
+	/* disable "tiny facet" removal */
+	oldoptions = us_useroptions;
+	us_useroptions |= DRAWTINYFACETS;
+
+	/* count the number of polygons in the job */
+	io_pspolygoncount = 0;
+	if (io_verbose < 0)
+	{
+		(void)askaid(us_aid, "display-to-routine", io_pspolycount);
+		io_pspolygontotal = io_pspolygoncount;
+		if (io_pspolygontotal == 0) io_pspolygontotal = 1;
+		DiaSetText(2, "Writing PostScript...");
+	}
+
 	/* plot everything */
+	io_pspolygoncount = 0;
 	sulx = el_curwindowpart->uselx;      suhx = el_curwindowpart->usehx;
 	suly = el_curwindowpart->usely;      suhy = el_curwindowpart->usehy;
 	sslx = el_curwindowpart->screenlx;   sshx = el_curwindowpart->screenhx;
@@ -542,10 +573,12 @@ INTSML io_writepostscriptfacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 	el_curwindowpart->screenly = io_pswindow.screenly;
 	el_curwindowpart->screenhy = io_pswindow.screenhy;
 	computewindowscale(el_curwindowpart);
+	io_lastred = io_lastgreen = io_lastblue = -1;
 	if (io_psusecolor != 0)
 	{
 		/* color: plot layers in proper order */
 		io_maxpslayer = io_setuptechorder(el_curtech);
+		io_pspolygontotal *= (io_maxpslayer+1);
 		for(i=0; i<io_maxpslayer; i++)
 		{
 			io_whichlayer = io_nextplotlayer(i) + 1;
@@ -565,6 +598,9 @@ INTSML io_writepostscriptfacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 	el_curwindowpart->screenly = ssly;   el_curwindowpart->screenhy = sshy;
 	computewindowscale(el_curwindowpart);
 
+	/* restore "tiny facet" removal option */
+	us_useroptions = oldoptions;
+
 	/* put out dates if requested */
 	if (varstate != NOVARIABLE && (varstate->addr&PLOTDATES) != 0)
 	{
@@ -605,6 +641,8 @@ INTSML io_writepostscriptfacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 #endif
 	io_pswrite("%%%%Trailer\n");
 	xclose(io_psout);
+	if (io_verbose < 0)
+		DiaDoneDialog();
 	if (printit != 0)
 	{
 		printprocess = efork();
@@ -615,9 +653,19 @@ INTSML io_writepostscriptfacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 		}
 		if (printprocess == 0)
 		{
-			execpars[0] = "lpr";
-			execpars[1] = file;
-			execpars[2] = 0;
+			INTBIG ac;
+			char *printer = egetenv("PRINTER");
+			ac = 0;
+			execpars[ac++] = "lpr";
+			if (printer != 0 && *printer != 0)
+			{
+				(void)initinfstr();
+				(void)addstringtoinfstr("-P");
+				(void)addstringtoinfstr(printer);
+				execpars[ac++] = returninfstr();
+			}
+			execpars[ac++] = file;
+			execpars[ac] = 0;
 			eexec("lpr", execpars);
 			eunlink(file);
 			ttyputmsg("Could not run 'lpr' to print");
@@ -630,17 +678,34 @@ INTSML io_writepostscriptfacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 }
 
 /*
- * routine to plot the polygon "poly"
+ * coroutine to count the number of polygons that will be plotted (for progress info).
+ */
+INTSML io_pspolycount(POLYGON *poly, WINDOWPART *win)
+{
+	io_pspolygoncount++;
+	return(0);
+}
+
+/*
+ * coroutine to plot the polygon "poly"
  */
 INTSML io_pspoly(POLYGON *poly, WINDOWPART *win)
 {
 	REGISTER INTSML k, type, font;
+	REGISTER INTBIG red, green, blue;
 	REGISTER float r, g, b;
 	INTBIG xl, xh, yl, yh, x, y, listx[4], listy[4];
 	static WINDOWPART *cachedwindow = NOWINDOWPART;
 	static NODEPROTO *cachedfacet = NONODEPROTO;
 	static TECHNOLOGY *cachedtech = NOTECHNOLOGY;
 
+	io_pspolygoncount++;
+	if ((io_pspolygoncount % 100) == 99)
+	{
+		if (io_verbose < 0)
+			DiaPercent(1, io_pspolygoncount*100/io_pspolygontotal);
+	}
+
 	/* ignore null layers */
 	if (poly->desc->bits == LAYERN || poly->desc->col == ALLOFF) return(1);
 
@@ -663,10 +728,19 @@ INTSML io_pspoly(POLYGON *poly, WINDOWPART *win)
 	/* set color if requested */
 	if (io_psusecolor != 0)
 	{
-		r = (float)io_redmap[poly->desc->col] / 255.0f;
-		g = (float)io_greenmap[poly->desc->col] / 255.0f;
-		b = (float)io_bluemap[poly->desc->col] / 255.0f;
-		io_pswrite("%f %f %f setrgbcolor\n", r, g, b);
+		red = io_redmap[poly->desc->col];
+		green = io_greenmap[poly->desc->col];
+		blue = io_bluemap[poly->desc->col];
+		if (red != io_lastred || green != io_lastgreen || blue != io_lastblue)
+		{
+			io_lastred = red;
+			io_lastgreen = green;
+			io_lastblue = blue;
+			r = (float)red / 255.0f;
+			g = (float)green / 255.0f;
+			b = (float)blue / 255.0f;
+			io_pswrite("%f %f %f setrgbcolor\n", r, g, b);
+		}
 	}
 
 	switch (poly->style)
@@ -889,8 +963,9 @@ void io_psdisc(INTBIG atx, INTBIG aty, INTBIG ex, INTBIG ey, GRAPHICS *desc)
 
 void io_pspolygon(INTBIG *x, INTBIG *y, INTSML count, GRAPHICS *desc)
 {
-	INTSML i;
-	INTBIG lx, hx, ly, hy, psx, psy;
+	REGISTER INTSML i, solidfill;
+	REGISTER INTBIG lx, hx, ly, hy;
+	INTBIG psx, psy;
 	static char *polygon[] =
 	{
 		"/Polygon {",			/* put array into path */
@@ -963,10 +1038,17 @@ void io_pspolygon(INTBIG *x, INTBIG *y, INTSML count, GRAPHICS *desc)
 		for(i=0; polygon[i] != 0; i++) io_pswrite("%s\n", polygon[i]);
 	}
 
-	/* use solid black if solid pattern, high-resolution mode, or no pattern */
-	for(i=0; i<8; i++)
-		if ((desc->raster[i]&0xFFFF) != 0xFFFF) break;
-	if (i >= 8 || io_psscale > 4 || desc->bwstyle != PATTERNED)
+	/* use solid black if solid pattern, color PS, high-resolution mode, or no pattern */
+	if (io_psusecolor != 0 || io_psscale > 4 || desc->bwstyle != PATTERNED)
+		solidfill = 1; else
+			solidfill = 0;
+	if (solidfill == 0)
+	{
+		for(i=0; i<8; i++)
+			if ((desc->raster[i]&0xFFFF) != 0xFFFF) break;
+		if (i >= 8) solidfill = 1;
+	}
+	if (solidfill != 0)
 	{
 		/* solid fill: simply blacken the area */
 		io_pswrite("[");
