@@ -84,14 +84,16 @@ INTSML io_writequickdrawlibrary(LIBRARY *lib)
 	REGISTER NODEPROTO *np, *onp;
 	INTSML err, ix, swid, shei;
 	INTBIG hlx, hhx, hly, hhy, gridlx, gridly, gridx, gridy, cx, cy, prod1, prod2, i,
-		revy, slx, shx, sly, shy, ulx, uhx, uly, uhy, sxs, sys, dxs, dys;
+		revy, slx, shx, sly, shy, ulx, uhx, uly, uhy, sxs, sys, dxs, dys, saver, saveg,
+		saveb, white;
 	VARIABLE *var;
-	REGISTER VARIABLE *varstate;
+	REGISTER VARIABLE *varstate, *varred, *vargreen, *varblue;
 	static POLYGON *poly = NOPOLYGON;
 	static THPrint hPrint = 0;
 	TPPrPort plotwin;
 	TPrStatus prstat;
 	Rect sr, dr;
+	CGrafPtr window, gra_getoffscreen(WINDOWPART *);
 
 	np = lib->curnodeproto;
 	if (np == NONODEPROTO)
@@ -174,6 +176,17 @@ INTSML io_writequickdrawlibrary(LIBRARY *lib)
 	}
 	SetPort((WindowPtr)plotwin);
 
+	/* save background color and make it white */
+	varred = getvalkey((INTBIG)us_aid, VAID, VINTEGER|VISARRAY, us_colormap_red);
+	vargreen = getvalkey((INTBIG)us_aid, VAID, VINTEGER|VISARRAY, us_colormap_green);
+	varblue = getvalkey((INTBIG)us_aid, VAID, VINTEGER|VISARRAY, us_colormap_blue);
+	if (varred == NOVARIABLE || vargreen == NOVARIABLE || varblue == NOVARIABLE) return(1);
+	saver = ((INTBIG *)varred->addr)[0];
+	saveg = ((INTBIG *)vargreen->addr)[0];
+	saveb = ((INTBIG *)varblue->addr)[0];
+	white = 0xFF;
+	colormapload(&white, &white, &white, 0, 0);
+
 	/* see if the printer is color */
 	if ((plotwin->gPort.portBits.rowBytes & 0xC000) == 0xC000)
 	{
@@ -183,10 +196,11 @@ INTSML io_writequickdrawlibrary(LIBRARY *lib)
 		(void)us_makescreen(&hlx, &hly, &hhx, &hhy, el_curwindowpart);
 		sr.left = hlx;   sr.right = hhx;
 
-		us_getwindowsize(el_curwindowpart->frame, &swid, &shei);
+		getwindowframesize(el_curwindowpart->frame, &swid, &shei);
 		revy = shei - 1;
 
 		sr.top = revy - hhy;    sr.bottom = revy - hly;
+		sr.bottom++;
 		dr = plotwin->gPort.portRect;
 
 		/* ensure that the destination has the same aspect ratio as the source */
@@ -205,14 +219,11 @@ INTSML io_writequickdrawlibrary(LIBRARY *lib)
 		}
 
 		/* now copy the offscreen bits at "sr" to the print port at "dr" */
-{
-	CGrafPtr window, gra_getoffscreen(WINDOWPART *);
 		window = gra_getoffscreen(el_curwindowpart);
 		(void)LockPixels(window->portPixMap);
 		CopyBits((BitMap *)*(window->portPixMap), &plotwin->gPort.portBits,
 			&sr, &dr, srcCopy, 0L);
 		UnlockPixels(window->portPixMap);
-}
 
 		PrClosePage(plotwin);
 		PrCloseDoc(plotwin);
@@ -225,6 +236,9 @@ INTSML io_writequickdrawlibrary(LIBRARY *lib)
 				ttyputerr("Error queuing print page");
 		}
 		PrClose();
+		
+		/* restore background color */
+		colormapload(&saver, &saveg, &saveb, 0, 0);
 		return(0);
 	}
 
@@ -366,6 +380,9 @@ INTSML io_writequickdrawlibrary(LIBRARY *lib)
 		PrPicFile(hPrint, 0, 0, 0, &prstat);
 	}
 	PrClose();
+		
+	/* restore background color */
+	colormapload(&saver, &saveg, &saveb, 0, 0);
 	return(0);
 }
 
@@ -906,7 +923,7 @@ void io_maccopyhighlighted(void)
 		INTSML revy, swid, shei;
 
 		window = gra_getoffscreen(el_curwindowpart);
-		us_getwindowsize(el_curwindowpart->frame, &swid, &shei);
+		getwindowframesize(el_curwindowpart->frame, &swid, &shei);
 		revy = shei - 1;
 
 		sr.left = ulx;   sr.right = uhx;
