@@ -176,6 +176,7 @@ char   io_sueorigline[MAXLINE];
 char   io_suecurline[MAXLINE];
 char **io_suedirectories;
 INTBIG io_suenumdirectories = 0;
+FILE  *io_suefilein;
 
 INTSML     io_sueadddirectory(char *directory);
 INTBIG     io_suearraylength(char *name);
@@ -235,8 +236,8 @@ INTSML io_readsuelibrary(LIBRARY *lib)
 		*filename, **filelist;
 
 	/* open the file */
-	io_filein = xopen(lib->libfile, FILETYPESUE, "", &filename);
-	if (io_filein == 0)
+	io_suefilein = xopen(lib->libfile, FILETYPESUE, "", &filename);
+	if (io_suefilein == 0)
 	{
 		ttyputerr("File %s not found", lib->libfile);
 		return(1);
@@ -301,10 +302,10 @@ INTSML io_readsuelibrary(LIBRARY *lib)
 	}
 
 	/* prepare for input */
-	io_suefilesize = filesize(io_filein);
+	io_suefilesize = filesize(io_suefilein);
 	if (DiaInitDialog(&us_progressdialog) != 0)
 	{
-		xclose(io_filein);
+		xclose(io_suefilein);
 		return(1);
 	}
 	io_suelineno = 0;
@@ -318,7 +319,7 @@ INTSML io_readsuelibrary(LIBRARY *lib)
 
 	/* clean up */
 	DiaDoneDialog();
-	xclose(io_filein);
+	xclose(io_suefilein);
 	ttyputmsg("%s read", truesuefile);
 
 	return(0);
@@ -1692,8 +1693,8 @@ NODEPROTO *io_suereadfromdisk(LIBRARY *lib, char *name)
 		f = xopen(subfilename, FILETYPESUE, "", &truename);
 		if (f != 0)
 		{
-			savefp = io_filein;
-			io_filein = f;
+			savefp = io_suefilein;
+			io_suefilein = f;
 			for(i=0; i<MAXLINE; i++) savecurline[i] = io_suecurline[i];
 			strcpy(saveorigline, io_sueorigline);
 			strcpy(savesuelastline, io_suelastline);
@@ -1707,17 +1708,17 @@ NODEPROTO *io_suereadfromdisk(LIBRARY *lib, char *name)
 			strcpy(subfilename, name);
 			savefilesize = io_suefilesize;
 			savelineno = io_suelineno;
-			io_suefilesize = filesize(io_filein);
+			io_suefilesize = filesize(io_suefilein);
 			io_suelineno = 0;
 			(void)io_suereadfile(lib, subfilename);
-			xclose(io_filein);
-			io_filein = savefp;
+			xclose(io_suefilein);
+			io_suefilein = savefp;
 			io_suefilesize = savefilesize;
 			io_suelineno = savelineno;
 			strcpy(io_suelastline, savesuelastline);
 			strcpy(io_sueorigline, saveorigline);
 			for(i=0; i<MAXLINE; i++) io_suecurline[i] = savecurline[i];
-			filepos = xtell(io_filein);
+			filepos = xtell(io_suefilein);
 			DiaSetText(2, lastprogressmsg);
 			DiaPercent(1, filepos*100/io_suefilesize);
 
@@ -1730,7 +1731,7 @@ NODEPROTO *io_suereadfromdisk(LIBRARY *lib, char *name)
 }
 
 /*
- * Routine to read the next line from file "io_filein" and break
+ * Routine to read the next line from file "io_suefilein" and break
  * it up into space-separated keywords.  Returns the number
  * of keywords (-1 on EOF)
  */
@@ -1743,11 +1744,11 @@ INTSML io_suegetnextline(char **keywords, INTSML curlydepth)
 	{
 		if (io_suelastline[0] == 0)
 		{
-			if (xfgets(io_suelastline, MAXLINE, io_filein) != 0) return(-1);
+			if (xfgets(io_suelastline, MAXLINE, io_suefilein) != 0) return(-1);
 			io_suelineno++;
 			if ((io_suelineno%50) == 0)
 			{
-				filepos = xtell(io_filein);
+				filepos = xtell(io_suefilein);
 				DiaPercent(1, filepos*100/io_suefilesize);
 			}
 		}
