@@ -146,7 +146,7 @@ INTSML com_examineonefacet(NODEPROTO*, INTSML, INTSML);
 void comp_optionsdlog(void);
 
 /* the COMPACTOR aid table */
-static COMCOMP comcp = {NOKEYWORD, us_topoffacets, us_nextparse, NOPARAMS,
+static COMCOMP comcp = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
 	NOBACKUP, 0, " \t", "facet to be rechecked", "default is current facet"};
 static KEYWORD comnotopt[] =
 {
