@@ -52,18 +52,18 @@ COMCOMP erc_tablep = {ercopt, NOTOPLIST, NONEXTLIST, NOPARAMS, NOBACKUP,
 /* ERC: Options Dialog */
 DIALOGITEM erc_optionsdialogitems[] =
 {
- /*  1 */ {0, {108,380,132,444}, BUTTON, "OK"},
- /*  2 */ {0, {108,280,132,344}, BUTTON, "Cancel"},
- /*  3 */ {0, {32,8,48,208}, RADIO, "Must have contact in every area"},
- /*  4 */ {0, {56,8,72,208}, RADIO, "Must have at least 1 contact"},
- /*  5 */ {0, {80,8,96,208}, RADIO, "Do not check for contacts"},
+ /*  1 */ {0, {108,396,132,460}, BUTTON, "OK"},
+ /*  2 */ {0, {108,296,132,360}, BUTTON, "Cancel"},
+ /*  3 */ {0, {32,8,48,228}, RADIO, "Must have contact in every area"},
+ /*  4 */ {0, {56,8,72,228}, RADIO, "Must have at least 1 contact"},
+ /*  5 */ {0, {80,8,96,228}, RADIO, "Do not check for contacts"},
  /*  6 */ {0, {8,44,24,144}, MESSAGE, "For well:"},
- /*  7 */ {0, {32,232,48,444}, RADIO, "Must have contact in every area"},
- /*  8 */ {0, {56,232,72,444}, RADIO, "Must have at least 1 contact"},
- /*  9 */ {0, {80,232,96,444}, RADIO, "Do not check for contacts"},
- /* 10 */ {0, {8,268,24,368}, MESSAGE, "For substrate:"}
+ /*  7 */ {0, {32,248,48,480}, RADIO, "Must have contact in every area"},
+ /*  8 */ {0, {56,248,72,480}, RADIO, "Must have at least 1 contact"},
+ /*  9 */ {0, {80,248,96,480}, RADIO, "Do not check for contacts"},
+ /* 10 */ {0, {8,284,24,384}, MESSAGE, "For substrate:"}
 };
-DIALOG erc_optionsdialog = {{50,75,191,528}, "ERC Options", 10, erc_optionsdialogitems};
+DIALOG erc_optionsdialog = {{50,75,191,564}, "ERC Options", 10, erc_optionsdialogitems};
 
 /* substrate/well contacts */
 #define NOSUBWELLCON ((SUBWELLCON *)-1)
@@ -148,11 +148,13 @@ ERCERROR   *erc_prevercerror = NOERCERROR;
 
 AIDENTRY   *erc_aid;						/* this tool */
 NODEPROTO  *erc_facet;
-INTBIG      erc_nonmanhattan;				/* number of nonmanhattan polygons examined */
 INTBIG      erc_globalnetnumber;			/* global network number */
 INTBIG      erc_attachednetnumber;			/* attached network number (above this floats) */
 INTBIG      erc_optionskey;					/* key for "ERC_options" */
 INTBIG      erc_options;					/* value of "ERC_options" */
+INTBIG      erc_jobsize;					/* number of polygons to analyze */
+INTBIG      erc_polyscrunched;				/* number of polygons analyzed */
+INTBIG      erc_realrun;					/* nonzero if this is the real analysis */
 
 extern AIDENTRY *us_aid;
 
@@ -168,7 +170,7 @@ void         erc_freeercerror(ERCERROR *ee);
 void         erc_freesubwellarea(SUBWELLAREA *swa);
 void         erc_freesubwellcon(SUBWELLCON *swc);
 void         erc_gathersubandwell(NODEPROTO *np, XARRAY trans);
-void         erc_getpolygon(INTSML layer, TECHNOLOGY *tech, INTBIG *x, INTBIG *y, INTSML count, float area);
+void         erc_getpolygon(INTSML layer, TECHNOLOGY *tech, INTBIG *x, INTBIG *y, INTSML count);
 void         erc_identifypoint(INTBIG x, INTBIG y, NODEPROTO *facet);
 void         erc_optionsdlog(void);
 void         erc_reporterror(void);
@@ -279,6 +281,7 @@ void erc_analyzefacet(NODEPROTO *facet)
 	REGISTER VARIABLE *var;
 	REGISTER PORTPROTO *pp, *opp;
 	REGISTER SUBWELLAREA *swa, *oswa;
+	extern DIALOG us_progressdialog;
 
 	/* make sure network tool is on */
 	if ((net_aid->aidstate&AIDON) == 0)
@@ -288,7 +291,7 @@ void erc_analyzefacet(NODEPROTO *facet)
 		return;
 	}
 
-	/* initialize to analyze facet */
+	/* announce start of analysis */
 	(void)initinfstr();
 	(void)addstringtoinfstr("Checking electrical rules...");
 	abortseq = getinterruptkey();
@@ -299,10 +302,14 @@ void erc_analyzefacet(NODEPROTO *facet)
 		(void)addstringtoinfstr(" to abort)");
 	}
 	ttyputmsg(returninfstr());
+	if (DiaInitDialog(&us_progressdialog) != 0) return;
+	DiaPercent(1, 0);
+	DiaSetText(2, "Preparing to analyze...");
+
+	/* initialize to analyze facet */
 	erc_freeallerrors();
 	erc_freeallsubwellcons();
 	erc_freeallsubwellareas();
-	erc_nonmanhattan = 0;
 	erc_worstwelldist = erc_worstsubdist = 0;
 	erc_facet = facet;
 	var = getvalkey((INTBIG)erc_aid, VAID, VINTEGER, erc_optionskey);
@@ -330,11 +337,31 @@ void erc_analyzefacet(NODEPROTO *facet)
 		for(np = tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 			np->temp1 = 0;
 
-	/* analyze facet contents */
-	mrginit();
+	/* run through the work and count the number of polygons */
+	erc_polyscrunched = 0;
+	erc_realrun = 0;
 	erc_gathersubandwell(facet, el_matid);
-	mrgdonefacet(erc_getpolygon);
-	mrgterm();
+	erc_jobsize = erc_polyscrunched;
+
+	/* now gather all of the geometry into the polygon merging system */
+	DiaSetText(2, "Gathering geometry...");
+	mergeinit();
+	erc_realrun = 1;
+	erc_polyscrunched = 0;
+	erc_gathersubandwell(facet, el_matid);
+
+	/* determine the number of polygon points in the analysis */
+	DiaSetText(2, "Analyzing...");
+	erc_polyscrunched = 0;
+	erc_realrun = 0;
+	mergedone(erc_getpolygon);
+	erc_jobsize = erc_polyscrunched;
+
+	/* do the analysis */
+	erc_realrun = 1;
+	erc_polyscrunched = 0;
+	mergedone(erc_getpolygon);
+	DiaPercent(1, 100);
 
 	/* make sure all of the contacts are on the same net */
 	for(swc = erc_firstsubwellcon; swc != NOSUBWELLCON; swc = swc->nextsubwellcon)
@@ -408,10 +435,6 @@ void erc_analyzefacet(NODEPROTO *facet)
 		}
 	}
 
-	/* warn if nonmanhattan polygons were encountered */
-	if (erc_nonmanhattan > 0)
-		ttyputerr("Warning: %ld nonmanhattan polygons were ignored", erc_nonmanhattan);
-
 	/* show the farthest distance from a well/substrate contact */
 	if (erc_worstwelldist > 0 || erc_worstsubdist > 0)
 	{
@@ -459,6 +482,7 @@ void erc_analyzefacet(NODEPROTO *facet)
 	}
 
 	/* report the number of networks */
+	DiaDoneDialog();
 	ttyputmsg("%ld networks found", erc_globalnetnumber);
 	
 	/* report the number of errors found */
@@ -487,8 +511,6 @@ void erc_gathersubandwell(NODEPROTO *np, XARRAY trans)
 	REGISTER PORTEXPINST *pe;
 	REGISTER PORTPROTO *pp, *opp;
 	REGISTER NETWORK *net;
-	REGISTER INTBIG xc, yc, xs, ys;
-	INTBIG lx, hx, ly, hy;
 	REGISTER INTSML i, tot, nfun, layer;
 	char *extra;
 	static INTBIG checkstop = 0;
@@ -499,121 +521,105 @@ void erc_gathersubandwell(NODEPROTO *np, XARRAY trans)
 	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, erc_aid->cluster);
 
 	/* grab global network information from the ports */
-	for(net = np->firstnetwork; net != NONETWORK; net = net->nextnetwork)
-		net->temp1 = 0;
-	for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
-		pp->network->temp1 = pp->temp1;
-	for(net = np->firstnetwork; net != NONETWORK; net = net->nextnetwork)
+	if (erc_realrun != 0)
 	{
-		if (net->temp1 == 0) net->temp1 = erc_globalnetnumber++;
+		for(net = np->firstnetwork; net != NONETWORK; net = net->nextnetwork)
+			net->temp1 = 0;
+		for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
+			pp->network->temp1 = pp->temp1;
+		for(net = np->firstnetwork; net != NONETWORK; net = net->nextnetwork)
+		{
+			if (net->temp1 == 0) net->temp1 = erc_globalnetnumber++;
+		}
+		if (erc_attachednetnumber == 0) erc_attachednetnumber = erc_globalnetnumber;
+		DiaPercent(1, erc_polyscrunched * 100 / erc_jobsize);
 	}
-	if (erc_attachednetnumber == 0) erc_attachednetnumber = erc_globalnetnumber;
 
 	for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 	{
-		makerot(ni, localrot);
-		transmult(localrot, trans, final);
 		subnp = ni->proto;
+		if (erc_realrun != 0)
+		{
+			makerot(ni, localrot);
+			transmult(localrot, trans, final);
+		}
 		if (subnp->primindex == 0)
 		{
-			if (stopping("ERC") != 0) return;
-
-			/* propagate global network numbers into this instance */
-			for(pp = subnp->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
-				pp->temp1 = 0;
-			for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
-				pi->proto->temp1 = pi->conarcinst->network->temp1;
-			for(pe = ni->firstportexpinst; pe != NOPORTEXPINST; pe = pe->nextportexpinst)
-				pe->proto->temp1 = pe->exportproto->network->temp1;
-			for(pp = subnp->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
+			if (erc_realrun != 0)
 			{
-				if (pp->temp1 != 0) continue;
-				for(opp = subnp->firstportproto; opp != NOPORTPROTO; opp = opp->nextportproto)
-					if (opp != pp && opp->temp1 != 0 && opp->network == pp->network) break;
-				if (opp != NOPORTPROTO) pp->temp1 = opp->temp1; else
-					pp->temp1 = erc_globalnetnumber++;
-			}
+				if (stopping("ERC") != 0) return;
 
-			maketrans(ni, localtran);
-			transmult(localtran, final, subrot);
+				/* propagate global network numbers into this instance */
+				for(pp = subnp->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
+					pp->temp1 = 0;
+				for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
+					pi->proto->temp1 = pi->conarcinst->network->temp1;
+				for(pe = ni->firstportexpinst; pe != NOPORTEXPINST; pe = pe->nextportexpinst)
+					pe->proto->temp1 = pe->exportproto->network->temp1;
+				for(pp = subnp->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
+				{
+					if (pp->temp1 != 0) continue;
+					for(opp = subnp->firstportproto; opp != NOPORTPROTO; opp = opp->nextportproto)
+						if (opp != pp && opp->temp1 != 0 && opp->network == pp->network) break;
+					if (opp != NOPORTPROTO) pp->temp1 = opp->temp1; else
+						pp->temp1 = erc_globalnetnumber++;
+				}
+
+				maketrans(ni, localtran);
+				transmult(localtran, final, subrot);
+			}
 			erc_gathersubandwell(subnp, subrot);
 		} else
 		{
-			checkstop++;
-			if ((checkstop%10) == 0)
+			if (erc_realrun != 0)
 			{
-				if (stopping("ERC") != 0) return;
-			}
-			nfun = nodefunction(ni, &extra);
-			ni->proto->temp1++;
-#if 0
-			if (nfun != NPUNKNOWN && nfun != NPPIN && nfun != NPCONTACT &&
-				nfun != NPNODE && nfun != NPCONNECT && nfun != NPSOURCET &&
-				nfun != NPSOURCEDC && nfun != NPSOURCEAC && nfun != NPSOURCEN &&
-				nfun != NPSOURCEX && nfun != NPSOURCES && nfun != NPART &&
-				nfun != NPALIGN)
-			{
-				/* make sure all wires into this node are connected */
-				for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
+				checkstop++;
+				if ((checkstop%10) == 0)
 				{
-					ai = pi->conarcinst;
-					if (ai->network->temp1 >= erc_attachednetnumber)
-					{
-						REGISTER POLYGON *copypoly;
-						REGISTER ERCERROR *ee;
-
-						tot = arcpolys(ai);
-						shapearcpoly(ai, 0, poly);
-						xformpoly(poly, trans);
-						copypoly = allocpolygon(poly->count, erc_aid->cluster);
-						polycopy(copypoly, poly);
-						ee = erc_allocercerror(FLOATINGARC);
-						ee->poly1 = copypoly;
-					}
+					if (stopping("ERC") != 0) return;
 				}
-			}
-#endif
-			if (nfun == NPWELL || nfun == NPSUBSTRATE)
-			{
-				swc = erc_allocsubwellcon();
-				if (swc == NOSUBWELLCON) return;
-				xform((ni->lowx + ni->highx) / 2, (ni->lowy + ni->highy) / 2,
-					&swc->x, &swc->y, final);
-				swc->np = subnp;
-				swc->fun = nfun;
-				if (ni->firstportarcinst != NOPORTARCINST)
+				nfun = nodefunction(ni, &extra);
+				ni->proto->temp1++;
+
+				if (nfun == NPWELL || nfun == NPSUBSTRATE)
 				{
-					swc->netnum = ni->firstportarcinst->conarcinst->network->temp1;
-				} else if (ni->firstportexpinst != NOPORTEXPINST)
-				{
-					swc->netnum = ni->firstportexpinst->exportproto->network->temp1;
-				} else swc->netnum = 0;
-				swc->nextsubwellcon = erc_firstsubwellcon;
-				erc_firstsubwellcon = swc;
+					swc = erc_allocsubwellcon();
+					if (swc == NOSUBWELLCON) return;
+					xform((ni->lowx + ni->highx) / 2, (ni->lowy + ni->highy) / 2,
+						&swc->x, &swc->y, final);
+					swc->np = subnp;
+					swc->fun = nfun;
+					if (ni->firstportarcinst != NOPORTARCINST)
+					{
+						swc->netnum = ni->firstportarcinst->conarcinst->network->temp1;
+					} else if (ni->firstportexpinst != NOPORTEXPINST)
+					{
+						swc->netnum = ni->firstportexpinst->exportproto->network->temp1;
+					} else swc->netnum = 0;
+					swc->nextsubwellcon = erc_firstsubwellcon;
+					erc_firstsubwellcon = swc;
+				}
 			}
 
 			tot = nodepolys(ni);
-			for(i=0; i<tot; i++)
+			erc_polyscrunched += tot;
+			if (erc_realrun != 0)
 			{
-				shapenodepoly(ni, i, poly);
-				if (nfun == NPWELL)
+				for(i=0; i<tot; i++)
 				{
-					layer = erc_truewelllayer(poly->tech, poly->layer);
-				} else if (nfun == NPSUBSTRATE)
-				{
-					layer = erc_truesubstratelayer(poly->tech, poly->layer);
-				} else layer = poly->layer;
-				if (erc_welllayer(poly->tech, layer) == 0 &&
-					erc_substratelayer(poly->tech, layer) == 0) continue;
-				xformpoly(poly, final);
-				if (isbox(poly, &lx, &hx, &ly, &hy) == 0)
-				{
-					erc_nonmanhattan++;
-					continue;
+					shapenodepoly(ni, i, poly);
+					if (nfun == NPWELL)
+					{
+						layer = erc_truewelllayer(poly->tech, poly->layer);
+					} else if (nfun == NPSUBSTRATE)
+					{
+						layer = erc_truesubstratelayer(poly->tech, poly->layer);
+					} else layer = poly->layer;
+					if (erc_welllayer(poly->tech, layer) == 0 &&
+						erc_substratelayer(poly->tech, layer) == 0) continue;
+					xformpoly(poly, final);
+					mergestorepolygon(layer, poly->tech, poly);
 				}
-				xs = hx - lx;   ys = hy - ly;
-				xc = (lx + hx) / 2;   yc = (ly + hy) / 2;
-				mrgstorebox(layer, poly->tech, xs, ys, xc, yc);
 			}
 		}
 	}
@@ -626,20 +632,17 @@ void erc_gathersubandwell(NODEPROTO *np, XARRAY trans)
 			if (stopping("ERC") != 0) return;
 		}
 		tot = arcpolys(ai);
-		for(i=0; i<tot; i++)
+		erc_polyscrunched += tot;
+		if (erc_realrun != 0)
 		{
-			shapearcpoly(ai, i, poly);
-			if (erc_welllayer(poly->tech, poly->layer) == 0 &&
-				erc_substratelayer(poly->tech, poly->layer) == 0) continue;
-			xformpoly(poly, trans);
-			if (isbox(poly, &lx, &hx, &ly, &hy) == 0)
+			for(i=0; i<tot; i++)
 			{
-				erc_nonmanhattan++;
-				continue;
+				shapearcpoly(ai, i, poly);
+				if (erc_welllayer(poly->tech, poly->layer) == 0 &&
+					erc_substratelayer(poly->tech, poly->layer) == 0) continue;
+				xformpoly(poly, trans);
+				mergestorepolygon(poly->layer, poly->tech, poly);
 			}
-			xs = hx - lx;   ys = hy - ly;
-			xc = (lx + hx) / 2;   yc = (ly + hy) / 2;
-			mrgstorebox(poly->layer, poly->tech, xs, ys, xc, yc);
 		}
 	}
 }
@@ -648,7 +651,7 @@ void erc_gathersubandwell(NODEPROTO *np, XARRAY trans)
  * Coroutine of the polygon merging package that is given a merged polygon with
  * "count" points in (x,y) with area "area".
  */
-void erc_getpolygon(INTSML layer, TECHNOLOGY *tech, INTBIG *x, INTBIG *y, INTSML count, float area)
+void erc_getpolygon(INTSML layer, TECHNOLOGY *tech, INTBIG *x, INTBIG *y, INTSML count)
 {
 	REGISTER INTSML i, first, welltype;
 	REGISTER INTBIG dist, bestdist, desiredcontact, perareabit, nocontacterror, prev, xp, yp;
@@ -657,6 +660,9 @@ void erc_getpolygon(INTSML layer, TECHNOLOGY *tech, INTBIG *x, INTBIG *y, INTSML
 	REGISTER SUBWELLAREA *swa;
 	static INTBIG checkstop = 0;
 
+	erc_polyscrunched += count;
+	if (erc_realrun == 0) return;
+
 	checkstop++;
 	if ((checkstop%10) == 0)
 	{
@@ -665,6 +671,7 @@ void erc_getpolygon(INTSML layer, TECHNOLOGY *tech, INTBIG *x, INTBIG *y, INTSML
 	{
 		if (el_pleasestop != 0) return;
 	}
+	DiaPercent(1, erc_polyscrunched * 100 / erc_jobsize);
 
 	/* save the polygon */
 	swa = erc_allocsubwellarea();
