@@ -40,16 +40,16 @@
 
 /***** command parsing *****/
 
-static COMCOMP projcop = {NOKEYWORD, us_topoffacets, us_nextparse, NOPARAMS,
+static COMCOMP projcop = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
 	NOBACKUP, INPUTOPT, " \t", "facet to be checked-out", "check-out current facet"};
-static COMCOMP projcip = {NOKEYWORD, us_topoffacets, us_nextparse, NOPARAMS,
+static COMCOMP projcip = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
 	NOBACKUP, INPUTOPT, " \t", "facet to be checked-in", "check-in current facet"};
-static COMCOMP projoldp = {NOKEYWORD, us_topoffacets, us_nextparse, NOPARAMS,
+static COMCOMP projoldp = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
 	NOBACKUP, INPUTOPT, " \t", "facet from which old version should be retrieved",
 		"use current facet"};
-static COMCOMP projap = {NOKEYWORD, us_topoffacets, us_nextparse, NOPARAMS,
+static COMCOMP projap = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
 	NOBACKUP, INPUTOPT, " \t", "facet to be added", "add current facet"};
-static COMCOMP projdp = {NOKEYWORD, us_topoffacets, us_nextparse, NOPARAMS,
+static COMCOMP projdp = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
 	NOBACKUP, INPUTOPT, " \t", "facet to be deleted", "delete current facet"};
 static KEYWORD projopt[] =
 {
@@ -519,7 +519,7 @@ void proj_checkin(char *facetname)
 	REGISTER NODEPROTO *np, *onp;
 	REGISTER NODEINST *ni;
 	REGISTER INTSML total, propagated;
-	REGISTER LIBRARY *lib;
+	REGISTER LIBRARY *olib;
 	REGISTER PROJECTFACET *pf;
 
 	/* find out which facet is being checked in */
@@ -531,19 +531,22 @@ void proj_checkin(char *facetname)
 	}
 
 	/* mark the facet to be checked-in */
-	lib = np->cell->lib;
-	for(onp = lib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
-		onp->temp1 = 0;
+	for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+		for(onp = olib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
+			onp->temp1 = 0;
 	np->temp1 = 1;
 
 	/* look for facets above this one that must also be checked in */
-	for(onp = lib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto) onp->temp2 = 0;
+	for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+		for(onp = olib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
+			onp->temp2 = 0;
 	np->temp2 = 1;
 	propagated = 1;
 	while (propagated != 0)
 	{
 		propagated = 0;
-		for(onp = lib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
+		for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+			for(onp = olib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
 		{
 			if (onp->temp2 == 1)
 			{
@@ -558,7 +561,8 @@ void proj_checkin(char *facetname)
 	}
 	np->temp2 = 0;
 	total = 0;
-	for(onp = lib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
+	for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+		for(onp = olib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
 	{
 		if (onp->temp2 == 0) continue;
 		pf = proj_findfacet(onp);
@@ -571,13 +575,16 @@ void proj_checkin(char *facetname)
 	}
 
 	/* look for facets below this one that must also be checked in */
-	for(onp = lib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto) onp->temp2 = 0;
+	for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+		for(onp = olib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
+			onp->temp2 = 0;
 	np->temp2 = 1;
 	propagated = 1;
 	while (propagated != 0)
 	{
 		propagated = 0;
-		for(onp = lib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
+		for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+			for(onp = olib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
 		{
 			if (onp->temp2 == 1)
 			{
@@ -592,7 +599,8 @@ void proj_checkin(char *facetname)
 		}
 	}
 	np->temp2 = 0;
-	for(onp = lib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
+	for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+		for(onp = olib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
 	{
 		if (onp->temp2 == 0) continue;
 		pf = proj_findfacet(onp);
@@ -609,7 +617,8 @@ void proj_checkin(char *facetname)
 	{
 		total = 0;
 		(void)initinfstr();
-		for(onp = lib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
+		for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+			for(onp = olib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
 		{
 			if (onp == np || onp->temp1 == 0) continue;
 			if (total > 0) (void)addstringtoinfstr(", ");
@@ -620,7 +629,7 @@ void proj_checkin(char *facetname)
 	}
 
 	/* check it in */
-	proj_checkinmany(lib);
+	proj_checkinmany(np->cell->lib);
 }
 
 void proj_checkout(char *facetname, INTSML showfacet)
@@ -628,7 +637,7 @@ void proj_checkout(char *facetname, INTSML showfacet)
 	REGISTER NODEPROTO *np, *newvers, *oldvers;
 	REGISTER NODEINST *ni;
 	REGISTER INTSML worked, propagated, total;
-	REGISTER LIBRARY *lib;
+	REGISTER LIBRARY *lib, *olib;
 	char projectpath[256], projectfile[256], *argv[3];
 	PROJECTFACET *pf;
 
@@ -740,13 +749,16 @@ void proj_checkout(char *facetname, INTSML showfacet)
 		ttyputmsg("Facet %s checked out for your use", describenodeproto(newvers));
 
 		/* advise of possible problems with other checkouts higher up in the hierarchy */
-		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto) np->temp1 = 0;
+		for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+			for(np = olib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+				np->temp1 = 0;
 		newvers->temp1 = 1;
 		propagated = 1;
-		while(propagated != 0)
+		while (propagated != 0)
 		{
 			propagated = 0;
-			for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+			for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+				for(np = olib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 			{
 				if (np->temp1 == 1)
 				{
@@ -759,7 +771,8 @@ void proj_checkout(char *facetname, INTSML showfacet)
 		}
 		newvers->temp1 = 0;
 		total = 0;
-		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+		for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+			for(np = olib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 		{
 			if (np->temp1 == 0) continue;
 			pf = proj_findfacet(np);
@@ -775,7 +788,8 @@ void proj_checkout(char *facetname, INTSML showfacet)
 		{
 			ttyputmsg("*** Warning: the following facets are above this in the hierarchy");
 			ttyputmsg("*** and are checked out to others.  This may cause problems");
-			for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+			for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+				for(np = olib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 			{
 				if (np->temp1 != 3) continue;
 				pf = (PROJECTFACET *)np->temp2;
@@ -784,13 +798,16 @@ void proj_checkout(char *facetname, INTSML showfacet)
 		}
 
 		/* advise of possible problems with other checkouts lower down in the hierarchy */
-		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto) np->temp1 = 0;
+		for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+			for(np = olib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+				np->temp1 = 0;
 		newvers->temp1 = 1;
 		propagated = 1;
 		while(propagated != 0)
 		{
 			propagated = 0;
-			for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+			for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+				for(np = olib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 			{
 				if (np->temp1 == 1)
 				{
@@ -806,7 +823,8 @@ void proj_checkout(char *facetname, INTSML showfacet)
 		}
 		newvers->temp1 = 0;
 		total = 0;
-		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+		for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+			for(np = olib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 		{
 			if (np->temp1 == 0) continue;
 			pf = proj_findfacet(np);
@@ -822,7 +840,8 @@ void proj_checkout(char *facetname, INTSML showfacet)
 		{
 			ttyputmsg("*** Warning: the following facets are below this in the hierarchy");
 			ttyputmsg("*** and are checked out to others.  This may cause problems");
-			for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+			for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+				for(np = olib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 			{
 				if (np->temp1 != 3) continue;
 				pf = (PROJECTFACET *)np->temp2;
@@ -1238,7 +1257,7 @@ void proj_deletefacet(char *facetname)
 {
 	REGISTER NODEPROTO *np, *onp;
 	REGISTER NODEINST *ni;
-	REGISTER LIBRARY *lib;
+	REGISTER LIBRARY *lib, *olib;
 	char projectpath[256], projectfile[256], *pt;
 	PROJECTFACET *pf, *lastpf;
 
@@ -1254,14 +1273,16 @@ void proj_deletefacet(char *facetname)
 	/* make sure the facet is not being used */
 	if (np->firstinst != NONODEINST)
 	{
-		for(onp = lib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
-			onp->temp1 = 0;
+		for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+			for(onp = olib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
+				onp->temp1 = 0;
 		for(ni = np->firstinst; ni != NONODEINST; ni = ni->nextinst)
 			ni->parent->temp1++;
 		ttyputerr("Cannot delete facet %s because it is still being used by:", facetname);
-		for(onp = lib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
-			if (onp->temp1 != 0)
-				ttyputmsg("   Facet %s has %ld instance(s)", describenodeproto(onp), onp->temp1);
+		for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+			for(onp = olib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
+				if (onp->temp1 != 0)
+					ttyputmsg("   Facet %s has %ld instance(s)", describenodeproto(onp), onp->temp1);
 		return;
 	}
 
@@ -1645,6 +1666,7 @@ void proj_checkinmany(LIBRARY *lib)
 	REGISTER NODEPROTO *np;
 	char projectpath[256], projectfile[256];
 	PROJECTFACET *pf;
+	REGISTER LIBRARY *olib;
 
 	/* make sure there is a valid user name */
 	if (proj_getusername(0, lib) != 0)
@@ -1671,7 +1693,8 @@ void proj_checkinmany(LIBRARY *lib)
 	if (proj_readprojectfile(projectpath, projectfile) != 0)
 		ttyputerr("Cannot read project file"); else
 	{
-		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+		for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+			for(np = olib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 		{
 			if (np->temp1 == 0) continue;
 			if (stopping("Checkin") != 0) break;
