@@ -75,16 +75,8 @@ static INTSML  net_bucketnumber = 1;
 
 /*********************** EQUIV MODULES ***********************/
 
-#define	NOEQUIV ((EQUIV *)-1)
-
-typedef struct Iequiv
-{
-	NODEINST      *first;					/* component */
-	NODEINST      *second;					/* component */
-	struct Iequiv *nextequiv;				/* next in list */
-} EQUIV;
-static EQUIV *net_equivfree = NOEQUIV;
-static EQUIV *net_firstequiv = NOEQUIV;
+EQUIV *net_equivfree = NOEQUIV;
+EQUIV *net_firstequiv = NOEQUIV;
 
 /* prototypes for local routines */
 BUCKET *net_allocbucket(INTSML, INTSML);
@@ -267,7 +259,7 @@ INTSML net_setsecond(void)
 	e = net_firstequiv;
 	if (e == NOEQUIV || e->second != NONODEINST)
 	{
-		ttyputerr("Must do '-tellaid network equate-set-first' first");
+		ttyputerr("Must select the a 'First' equate before the second");
 		return(1);
 	}
 
@@ -334,8 +326,6 @@ INTSML net_setequiv(void)
 			e->nextequiv = net_firstequiv;
 			net_firstequiv = e;
 			e->first = ni1;
-			e->second = NONODEINST;
-			e = net_firstequiv;
 			e->second = ni2;
 			ttyputmsg("Associating component %s", (char *)var1->addr);
 		}
@@ -582,10 +572,10 @@ INTSML net_compare(void)
 
 	/* build network of pseudocomponents */
 	net_pcomp1 = net_makepseudo(net_facet1, &comp1, &net1, &power1, &ground1,
-		&net_nodelist1, (INTSML)net_compare_hierarchy, (INTSML)net_unify_port_names);
+		&net_nodelist1, (INTSML)net_compare_hierarchy, (INTSML)net_unify_port_names, 1);
 	if (net_pcomp1 == NOPCOMP) return(1);
 	net_pcomp2 = net_makepseudo(net_facet2, &comp2, &net2, &power2, &ground2,
-		&net_nodelist2, (INTSML)net_compare_hierarchy, (INTSML)net_unify_port_names);
+		&net_nodelist2, (INTSML)net_compare_hierarchy, (INTSML)net_unify_port_names, 1);
 	if (net_pcomp2 == NOPCOMP) return(1);
 
 	ttyputmsg("Comparing %d components and %d nets in facet %s", comp1, net1,
@@ -735,16 +725,12 @@ INTSML net_compare(void)
 			/* ignore unwired components */
 			if (b->wirecount == 0) continue;
 
-#if 0
-			/* ignore buckets that are singletons */
-			if (b->net1count <= 1 && b->net2count <= 1) continue;
-#else
 			/* ignore buckets that are already associated */
 			if (b->net1count == 1 && b->net2count == 1) continue;
 
 			/* ignore buckets that don't have two components */
 			if (b->net1count + b->net2count < 2) continue;
-#endif
+
 			/* examine each component in list 1 for possible splitting */
 			ret = net_scancomplist(b, 1);
 			if (ret < 0) return(1);
@@ -776,14 +762,14 @@ INTSML net_compare(void)
 		if (b->net1count == 0 && b->net2count != 0)
 		{
 			for(p = b->net2list; p != NOPCOMP; p = p->nextinbucket)
-				if (p->actual->parent != net_facet1 && p->actual->parent != net_facet2) insub++; else
+				if (p->topactual->parent != net_facet1 && p->topactual->parent != net_facet2) insub++; else
 			{
 				if (found != 0) (void)addtoinfstr('/');
 				found++;
 				(void)addstringtoinfstr("FACET=");
-				(void)addstringtoinfstr(describenodeproto(geomparent(p->actual->geom)));
+				(void)addstringtoinfstr(describenodeproto(geomparent(p->topactual->geom)));
 				(void)addstringtoinfstr(" FROM=");
-				(void)sprintf(line, "0%lo", (INTBIG)p->actual->geom);
+				(void)sprintf(line, "0%lo", (INTBIG)p->topactual->geom);
 				(void)addstringtoinfstr(line);
 				(void)addstringtoinfstr(";-1;0");
 			}
@@ -792,14 +778,14 @@ INTSML net_compare(void)
 		if (b->net2count == 0 && b->net1count != 0)
 		{
 			for(p = b->net1list; p != NOPCOMP; p = p->nextinbucket)
-				if (p->actual->parent != net_facet1 && p->actual->parent != net_facet2) insub++; else
+				if (p->topactual->parent != net_facet1 && p->topactual->parent != net_facet2) insub++; else
 			{
 				if (found != 0) (void)addtoinfstr('/');
 				found++;
 				(void)addstringtoinfstr("FACET=");
-				(void)addstringtoinfstr(describenodeproto(geomparent(p->actual->geom)));
+				(void)addstringtoinfstr(describenodeproto(geomparent(p->topactual->geom)));
 				(void)addstringtoinfstr(" FROM=");
-				(void)sprintf(line, "0%lo", (INTBIG)p->actual->geom);
+				(void)sprintf(line, "0%lo", (INTBIG)p->topactual->geom);
 				(void)addstringtoinfstr(line);
 				(void)addstringtoinfstr(";-1;0");
 			}
@@ -1261,11 +1247,6 @@ INTSML net_diffwires(PCONN *n1, INTSML c1, INTSML f1, PNET *net1,
 	/* simple test: number of wire lists must be equal */
 	if (c1 != c2) return(1);
 
-#if 0
-	/* simple test: component factor (negated, exported) must be equal */
-	if (f1 != f2) return(1);
-
-#else
 	/* simple test: negated factor must be equal */
 	if ((f1&NEGATEDPORT) != (f2&NEGATEDPORT)) return(1);
 
@@ -1280,7 +1261,6 @@ INTSML net_diffwires(PCONN *n1, INTSML c1, INTSML f1, PNET *net1,
 				namesame(pp1->protoname, pp2->protoname) != 0) return(1);
 		}
 	}
-#endif
 
 	/* make sure there is memory for flags */
 	if (c1 > net_difflist1size)
@@ -1412,21 +1392,26 @@ void net_freeallbuckets(void)
  */
 INTSML net_getfacets(NODEPROTO **facet1, NODEPROTO **facet2)
 {
-	REGISTER WINDOWPART *w1, *w2;
+	REGISTER WINDOWPART *w;
 
 	/* get two windows */
-	w1 = el_topwindowpart;
-	if (w1 == NOWINDOWPART) return(1);
-	w2 = w1->nextwindowpart;
-	if (w2 == NOWINDOWPART) return(1);
-	if (w2->nextwindowpart != NOWINDOWPART) return(1);
-
-	/* get facets in the windows */
-	*facet1 = w1->curnodeproto;
-	*facet2 = w2->curnodeproto;
-	if (*facet1 == NONODEPROTO) return(1);
+	*facet1 = *facet2 = NONODEPROTO;
+	for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
+	{
+		if ((w->state&WINDOWTYPE) != DISPWINDOW &&
+			(w->state&WINDOWTYPE) != DISP3DWINDOW) continue;
+		if (*facet1 == NONODEPROTO) *facet1 = w->curnodeproto; else
+		{
+			if (*facet2 == NONODEPROTO)
+			{
+				if (w->curnodeproto != *facet1) *facet2 = w->curnodeproto;
+			} else
+			{
+				if (w->curnodeproto != NONODEPROTO) return(1);
+			}
+		}
+	}
 	if (*facet2 == NONODEPROTO) return(1);
-	if (*facet1 == *facet2) return(1);
 	return(0);
 }
 
