@@ -35,7 +35,7 @@
  *   PNET *pnet;
  *   INTSML components, nets, powernets, groundnets;
  *   pcomp = net_makepseudo(facet, &components, &nets, &powernets, &groundnets,
- *	&pnet, hierarchical, sortports);
+ *	&pnet, hierarchical, sortports, useequiv);
  *   .....
  *        do something with the network in "pcomp" and "pnet"
  *   .....
@@ -57,10 +57,11 @@
 #include "network.h"
 #include "efunction.h"
 
-static INTSML net_pseudonode;		/* net number for pseudonetworks */
+static INTSML    net_pseudonode;	/* net number for pseudonetworks */
+static NODEINST *net_toplevelinst;	/* actual top-instance associated with this node */
 
 /* prototypes for local routines */
-PCOMP *net_buildpseudo(NODEPROTO*, PCOMP*, PNET**, PNET**, INTSML*, PNET**, INTSML, INTSML);
+PCOMP *net_buildpseudo(NODEPROTO*, PCOMP*, PNET**, PNET**, INTSML*, PNET**, INTSML, INTSML, INTSML);
 PNET *net_newpnet(PNET**);
 PCOMP *net_allocpcomp(void);
 void net_freepcomp(PCOMP*);
@@ -106,11 +107,11 @@ void net_freeflatmemory(void)
  * the number of pseudo-nets in "nets", the number of power nets in "powernets",
  * and the number of ground nets in "groundnets".  Extraction is done
  * hierarchically if "comphier" is nonzero.  Ports are sorted alphabetically
- * in each component if "sortports" is nonzero.  Returns NOPCOMP if there is
- * an error
+ * in each component if "sortports" is nonzero.  Equivalences are used if
+ * "useequivs" is nonzero.  Returns NOPCOMP if there is an error
  */
 PCOMP *net_makepseudo(NODEPROTO *facet, INTSML *components, INTSML *nets, INTSML *powernets,
-	INTSML *groundnets, PNET **pnetlist, INTSML comphier, INTSML sortports)
+	INTSML *groundnets, PNET **pnetlist, INTSML comphier, INTSML sortports, INTSML useequivs)
 {
 	REGISTER PCONN *w;
 	REGISTER PCOMP *pcomp, *pcomplist, *p;
@@ -147,13 +148,11 @@ PCOMP *net_makepseudo(NODEPROTO *facet, INTSML *components, INTSML *nets, INTSML
 	}
 
 	/* create a list of pseudocomponents in this facet */
-#if 0
-	power = ground = NOPNET;
-#endif
 	*components = 0;
 	begintraversehierarchy();
+	net_toplevelinst = NONODEINST;
 	pcomplist = net_buildpseudo(facet, NOPCOMP, &power, &ground, components,
-		pnetlist, comphier, sortports);
+		pnetlist, comphier, sortports, useequivs);
 	if (pcomplist == 0) return(NOPCOMP);
 	if (*components == 0)
 	{
@@ -211,7 +210,7 @@ PCOMP *net_makepseudo(NODEPROTO *facet, INTSML *components, INTSML *nets, INTSML
  */
 PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, PNET **power,
 	PNET **ground, INTSML *components, PNET **pnetlist, INTSML compare_hierarchically,
-	INTSML sortports)
+	INTSML sortports, INTSML useequivs)
 {
 	REGISTER PCOMP *pcomp;
 	REGISTER PCONN *pswap;
@@ -220,9 +219,13 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, PNET **power,
 	REGISTER PORTEXPINST *pe;
 	REGISTER ARCINST *ai, *oai;
 	REGISTER NODEINST *ni;
-	REGISTER INTSML fun, i, toldshort, swap, still;
+	REGISTER INTSML fun, i, toldshort, swap, still, toplevel, flattenit;
 	REGISTER NODEPROTO *realnp;
 	REGISTER PNET *net;
+	REGISTER EQUIV *e;
+
+	if (net_toplevelinst == NONODEINST) toplevel = 1; else
+		toplevel = 0;
 
 	/* place pseudo-netnumbers on all arcs, taken from ports */
 	for(ai = facet->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
@@ -233,7 +236,6 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, PNET **power,
 			if (ai->network == pp->network) ai->temp1 = pp->temp1;
 	}
 
-#if 1
 	/* spread power and ground information from appropriate nodes */
 	for(ni = facet->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 	{
@@ -278,7 +280,6 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, PNET **power,
 				if (oai->network == ai->network) oai->temp1 = ai->temp1;
 		}
 	}
-#endif
 
 	/* generate new pseudo-netnumbers for arcs not connected to ports */
 	for(ai = facet->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
@@ -296,8 +297,23 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, PNET **power,
 	toldshort = 0;
 	for(ni = facet->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 	{
+		if (toplevel != 0) net_toplevelinst = ni;
+
 		/* if flattening the circuit, explore contents of facet instances */
-		if (ni->proto->primindex == 0 && compare_hierarchically != 0)
+		flattenit = 0;
+		if (ni->proto->primindex == 0 && compare_hierarchically != 0) flattenit = 1;
+
+		if (flattenit != 0 && useequivs != 0)
+		{
+			/* see if this is equivalenced */
+			for(e = net_firstequiv; e != NOEQUIV; e = e->nextequiv)
+			{
+				/* check the validity of the equivalence */
+				if (e->first == ni || e->second == ni) flattenit = 0;
+			}
+		}
+
+		if (flattenit != 0)		
 		{
 			/* if there is an alternate contents facet, use it */
 			realnp = contentsview(ni->proto);
@@ -351,7 +367,7 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, PNET **power,
 			/* recurse into the facet */
 			downhierarchy(ni);
 			initiallist = net_buildpseudo(realnp, initiallist, power, ground,
-				components, pnetlist, compare_hierarchically, sortports);
+				components, pnetlist, compare_hierarchically, sortports, useequivs);
 			uphierarchy();
 			if (initiallist == 0) return(0);
 			continue;
@@ -376,6 +392,7 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, PNET **power,
 		pcomp->id = -1;
 		pcomp->function = fun;
 		pcomp->actual = ni;
+		pcomp->topactual = net_toplevelinst;
 		(*components)++;
 
 		/* count the number of electrically distinct nets on the component */
