@@ -39,9 +39,9 @@
 #include "edialogs.h"
 
 /* the ROUTER aid table */
-static COMCOMP routerfacetp = {NOKEYWORD, us_topoffacets, us_nextparse, NOPARAMS,
+static COMCOMP routerfacetp = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
 	NOBACKUP, INPUTOPT, " \t", "Facet to stitch", 0};
-COMCOMP routerarcp = {NOKEYWORD, us_topofarcs, us_nextparse, NOPARAMS,
+COMCOMP routerarcp = {NOKEYWORD, topofarcs, nextarcs, NOPARAMS,
 	NOBACKUP, INPUTOPT, " \t", "Arc prototype to use in stitching (* to reset)", 0};
 static KEYWORD routerautoopt[] =
 {
@@ -63,7 +63,7 @@ static KEYWORD routermimicopt[] =
 };
 static COMCOMP routermimicp = {routermimicopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
 	NOBACKUP, INPUTOPT, " \t", "Mimic-stitching action", 0};
-static COMCOMP routerriverp = {NOKEYWORD, us_topoffacets, us_nextparse, NOPARAMS,
+static COMCOMP routerriverp = {NOKEYWORD, topoffacets, nextfacets, NOPARAMS,
 	NOBACKUP, INPUTOPT, " \t", "Facet to river-route", 0};
 static COMCOMP routermazesetboundaryp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
 	0, " \t", "routing boundary", 0};
@@ -410,7 +410,7 @@ INTSML ro_set(INTSML count, char *par[])
 		l = strlen(pp = par[1]);
 		if (namesamen(pp, "do-now", l) == 0)
 		{
-			ro_mimicstitch();
+			ro_mimicstitch(1);
 			return(0);
 		}
 		if (namesamen(pp, "enable", l) == 0)
@@ -619,7 +619,7 @@ void ro_slice(void)
 			setactivity("Facets stitched");
 			break;
 		case MIMICSTITCH:
-			ro_mimicstitch();
+			ro_mimicstitch(0);
 			setactivity("Facets stitched");
 			break;
 	}
