@@ -35,6 +35,7 @@
 #include "efunction.h"
 #include "rout.h"
 #include "usr.h"
+#include <math.h>
 
 ARCPROTO *ro_preferedarc;	/* the prefered arc */
 
@@ -58,6 +59,7 @@ void ro_autostitch(void)
 	REGISTER PORTPROTO *pp, *rpp;
 	REGISTER NODEPROTO *np;
 	REGISTER ARCPROTO *ap;
+	REGISTER LIBRARY *lib;
 	REGISTER TECHNOLOGY *tech;
 	extern AIDENTRY *net_aid;
 	REGISTER VARIABLE *var;
@@ -74,8 +76,9 @@ void ro_autostitch(void)
 	if (ro_firstrcheck == NORCHECK) return;
 
 	/* set temp1 flag on all facets that will be checked */
-	for(np = el_curlib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
-		np->temp1 = 0;
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+			np->temp1 = 0;
 
 	/* next pre-compute bounds on all nodes in facets to be changed */
 	count = 0;
@@ -157,8 +160,9 @@ void ro_autostitch(void)
 	ro_firstrcheck = NORCHECK;
 
 	/* free any memory associated with this operation */
-	for(np = el_curlib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
-		if (np->temp1 != 0)
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+			if (np->temp1 != 0)
 	{
 		for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 			if (ni->temp2 != -1 && ni->temp2 != 0)
@@ -659,8 +663,8 @@ INTSML ro_canconnect(ARCPROTO *ap, PORTPROTO *pp)
 void ro_findsmallestlayer(ARCPROTO *ap)
 {
 	REGISTER ARCINST *ai;
-	REGISTER INTBIG area, bestarea;
-	REGISTER INTSML i, j;
+	REGISTER float area, bestarea;
+	REGISTER INTSML i, j, bestfound;
 	static POLYGON *poly = NOPOLYGON;
 
 	/* quit if the value has already been computed */
@@ -678,14 +682,15 @@ void ro_findsmallestlayer(ARCPROTO *ap)
 	ai->length = 10000;
 
 	/* find the smallest layer */
-	bestarea = HUGEINT;
+	bestfound = 0;
 	j = arcpolys(ai);
 	for(i=0; i<j; i++)
 	{
 		shapearcpoly(ai, i, poly);
-		area = abs(areapoly(poly));
-		if (area >= bestarea) continue;
+		area = (float)fabs(areapoly(poly));
+		if (bestfound != 0 && area >= bestarea) continue;
 		bestarea = area;
+		bestfound++;
 		ap->temp1 = poly->layer;
 	}
 }
