@@ -396,6 +396,127 @@ static KEYWORD sim_veropt[] =
 COMCOMP sim_verp = {sim_veropt, NOTOPLIST, NONEXTLIST,
 	NOPARAMS, NOBACKUP, INPUTOPT, " \t", "Verilog simulation option", 0};
 
+/************************** FASTHENRY COMMANDS **************************/
+
+#if 0
+tellaid simulation fasthenry arc add GROUPNAME
+tellaid simulation fasthenry arc remove
+tellaid simulation fasthenry arc thickness [THICKNESS]
+tellaid simulation fasthenry arc width-subdivisions [WIDTHSUBS]
+tellaid simulation fasthenry arc height-subdivisions [HEIGHTSUBS]
+tellaid simulation fasthenry arc z-head [ZHEAD]
+tellaid simulation fasthenry arc z-tail [ZTAIL]
+
+tellaid simulation fasthenry pole [single | multiple NUMPOLES]
+tellaid simulation fasthenry frequency [single | multiple START END RUNSPERDECADE]
+tellaid simulation fasthenry thickness [DEFTHICKNESS]
+tellaid simulation fasthenry width-subdivisions [DEFWIDTHSUBS]
+tellaid simulation fasthenry height-subdivisions [DEFHEIGHTSUBS]
+tellaid simulation fasthenry max-segment-length [MAXSEGLENGTH]
+tellaid simulation fasthenry execute [none | run | multiple-run]
+tellaid simulation fasthenry postscript [on | off]
+tellaid simulation fasthenry spice [on | off]
+#endif
+
+static COMCOMP simfhaaddp = {NOKEYWORD, NOTOPLIST, NONEXTLIST,
+	NOPARAMS, NOBACKUP, INPUTOPT, " \t", "FastHenry arc group name", 0};
+static COMCOMP simfhathickp = {NOKEYWORD, NOTOPLIST, NONEXTLIST,
+	NOPARAMS, NOBACKUP, INPUTOPT, " \t", "FastHenry arc thickness", 0};
+static COMCOMP simfhawidsubp = {NOKEYWORD, NOTOPLIST, NONEXTLIST,
+	NOPARAMS, NOBACKUP, INPUTOPT, " \t", "FastHenry arc width subdivisions", 0};
+static COMCOMP simfhaheisubp = {NOKEYWORD, NOTOPLIST, NONEXTLIST,
+	NOPARAMS, NOBACKUP, INPUTOPT, " \t", "FastHenry arc height subdivisions", 0};
+static COMCOMP simfhazheadp = {NOKEYWORD, NOTOPLIST, NONEXTLIST,
+	NOPARAMS, NOBACKUP, INPUTOPT, " \t", "FastHenry arc head Z", 0};
+static COMCOMP simfhaztailp = {NOKEYWORD, NOTOPLIST, NONEXTLIST,
+	NOPARAMS, NOBACKUP, INPUTOPT, " \t", "FastHenry arc tail Z", 0};
+static KEYWORD sim_fhaopt[] =
+{
+	{"add",                 1,{&simfhaaddp,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"remove",              0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"thickness",           1,{&simfhathickp,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"width-subdivisions",  1,{&simfhawidsubp,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"height-subdivisions", 1,{&simfhaheisubp,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"z-head",              1,{&simfhazheadp,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"z-tail",              1,{&simfhaztailp,NOKEY,NOKEY,NOKEY,NOKEY}},
+	TERMKEY
+};
+COMCOMP sim_fhap = {sim_fhaopt, NOTOPLIST, NONEXTLIST,
+	NOPARAMS, NOBACKUP, INPUTOPT, " \t", "FastHenry arc option", 0};
+static COMCOMP simfhpolemp = {NOKEYWORD, NOTOPLIST, NONEXTLIST,
+	NOPARAMS, NOBACKUP, INPUTOPT, " \t", "Number of poles", 0};
+static KEYWORD simfhpoleopt[] =
+{
+	{"single",           0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"multiple",         1,{&simfhpolemp,NOKEY,NOKEY,NOKEY,NOKEY}},
+	TERMKEY
+};
+static COMCOMP simfhpolep = {simfhpoleopt, NOTOPLIST, NONEXTLIST,
+	NOPARAMS, NOBACKUP, INPUTOPT, " \t", "FastHenry multipole option", 0};
+static COMCOMP simfhfreqsp = {NOKEYWORD, NOTOPLIST, NONEXTLIST,
+	NOPARAMS, NOBACKUP, INPUTOPT, " \t", "Starting frequency", 0};
+static COMCOMP simfhfreqep = {NOKEYWORD, NOTOPLIST, NONEXTLIST,
+	NOPARAMS, NOBACKUP, INPUTOPT, " \t", "Ending frequency", 0};
+static COMCOMP simfhfreqrp = {NOKEYWORD, NOTOPLIST, NONEXTLIST,
+	NOPARAMS, NOBACKUP, INPUTOPT, " \t", "Runs per decade", 0};
+static KEYWORD simfhfreqopt[] =
+{
+	{"single",           0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"multiple",         3,{&simfhfreqsp,&simfhfreqep,&simfhfreqrp,NOKEY,NOKEY}},
+	TERMKEY
+};
+static COMCOMP simfhfreqp = {simfhfreqopt, NOTOPLIST, NONEXTLIST,
+	NOPARAMS, NOBACKUP, INPUTOPT, " \t", "FastHenry frequency option", 0};
+static COMCOMP simfhthickp = {NOKEYWORD, NOTOPLIST, NONEXTLIST,
+	NOPARAMS, NOBACKUP, INPUTOPT, " \t", "FastHenry default thickness", 0};
+static COMCOMP simfhwidthp = {NOKEYWORD, NOTOPLIST, NONEXTLIST,
+	NOPARAMS, NOBACKUP, INPUTOPT, " \t", "FastHenry default width subdivisions", 0};
+static COMCOMP simfhheightp = {NOKEYWORD, NOTOPLIST, NONEXTLIST,
+	NOPARAMS, NOBACKUP, INPUTOPT, " \t", "FastHenry default height subdivisions", 0};
+static COMCOMP simfhseglenp = {NOKEYWORD, NOTOPLIST, NONEXTLIST,
+	NOPARAMS, NOBACKUP, INPUTOPT, " \t", "FastHenry maximum segment length", 0};
+static KEYWORD simfhexeopt[] =
+{
+	{"none",           0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"run",            0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"multiple-run",   0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	TERMKEY
+};
+static COMCOMP simfhexep = {simfhexeopt, NOTOPLIST, NONEXTLIST,
+	NOPARAMS, NOBACKUP, INPUTOPT, " \t", "FastHenry execution option", 0};
+static KEYWORD simfhpsopt[] =
+{
+	{"on",           0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"off",          0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	TERMKEY
+};
+static COMCOMP simfhpsp = {simfhpsopt, NOTOPLIST, NONEXTLIST,
+	NOPARAMS, NOBACKUP, INPUTOPT, " \t", "FastHenry PostScript option", 0};
+static KEYWORD simfhspiceopt[] =
+{
+	{"on",           0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"off",          0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	TERMKEY
+};
+static COMCOMP simfhspicep = {simfhspiceopt, NOTOPLIST, NONEXTLIST,
+	NOPARAMS, NOBACKUP, INPUTOPT, " \t", "FastHenry Spice option", 0};
+static KEYWORD sim_fhopt[] =
+{
+	{"arc",                 1,{&sim_fhap,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"pole",                1,{&simfhpolep,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"frequency",           1,{&simfhfreqp,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"thickness",           1,{&simfhthickp,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"width-subdivisions",  1,{&simfhwidthp,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"height-subdivisions", 1,{&simfhheightp,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"max-segment-length",  1,{&simfhseglenp,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"execute",             1,{&simfhexep,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"postscript",          1,{&simfhpsp,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"spice",               1,{&simfhspicep,NOKEY,NOKEY,NOKEY,NOKEY}},
+	TERMKEY
+};
+COMCOMP sim_fhp = {sim_fhopt, NOTOPLIST, NONEXTLIST,
+	NOPARAMS, NOBACKUP, INPUTOPT, " \t", "FastHenry simulation option", 0};
+
 /************************** SIMULATOR COMMANDS **************************/
 
 static KEYWORD simulatorsimulateopt[] =
@@ -404,6 +525,7 @@ static KEYWORD simulatorsimulateopt[] =
 	{"rsim",             0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	{"rnl",              0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	{"cosmos",           0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"fasthenry",        0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	{"spice",            0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	{"mossim",           0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	{"texsim",           0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
@@ -423,6 +545,7 @@ static KEYWORD simulatoropt[] =
 	{"pointout",                 1,{&simulatornetp,NOKEY,NOKEY,NOKEY,NOKEY}},
 	{"als",                      1,{&sim_alsp,NOKEY,NOKEY,NOKEY,NOKEY}},
 	{"verilog",                  1,{&sim_verp,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"fasthenry",                1,{&sim_fhp,NOKEY,NOKEY,NOKEY,NOKEY}},
 	{"no-execute",               0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	{"execute-only",             0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	{"execute-quietly",          0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
@@ -443,17 +566,18 @@ struct
 	INTSML format;
 } sim_simtable[] =
 {
-	{"abel-pal", 1, ABEL},
-	{"cosmos",	 1, COSMOS},
-	{"esim",     1, ESIM},
-	{"rnl",      2, RNL},
-	{"rsim",     2, RSIM},
-	{"mossim",   1, MOSSIM},
-	{"als",      1, ALS},
-	{"texsim",   1, TEXSIM},
-	{"silos",    2, SILOS},
-	{"spice",    2, SPICE},
-	{"verilog",  1, VERILOG},
+	{"abel-pal",  1, ABEL},
+	{"cosmos",	  1, COSMOS},
+	{"esim",      1, ESIM},
+	{"fasthenry", 1, FASTHENRY},
+	{"rnl",       2, RNL},
+	{"rsim",      2, RSIM},
+	{"mossim",    1, MOSSIM},
+	{"als",       1, ALS},
+	{"texsim",    1, TEXSIM},
+	{"silos",     2, SILOS},
+	{"spice",     2, SPICE},
+	{"verilog",   1, VERILOG},
 	{NULL, 0, 0}  /* 0 */
 };
 
@@ -479,9 +603,11 @@ DIALOGITEM sim_optionsdialogitems[] =
  /*  3 */ {0, {8,8,24,205}, CHECK, "Resimulate each change"},
  /*  4 */ {0, {32,8,48,205}, CHECK, "Auto advance time"},
  /*  5 */ {0, {56,8,72,205}, CHECK, "Multistate display"},
- /*  6 */ {0, {80,8,96,205}, CHECK, "Show waveform window"}
+ /*  6 */ {0, {80,8,96,205}, CHECK, "Show waveform window"},
+ /*  7 */ {0, {104,8,120,133}, MESSAGE, "Maximum events:"},
+ /*  8 */ {0, {104,136,120,217}, EDITTEXT, ""}
 };
-DIALOG sim_optionsdialog = {{50,75,158,372}, "Simulation Options", 6, sim_optionsdialogitems};
+DIALOG sim_optionsdialog = {{50,75,179,364}, "Simulation Options", 8, sim_optionsdialogitems};
 
 /* ALS Clock */
 DIALOGITEM sim_alsclockdialogitems[] =
@@ -613,9 +739,26 @@ void sim_init(INTBIG *argc, char *argv[], AIDENTRY *thisaid)
 	(void)setvalkey((INTBIG)sim_aid, VAID, sim_spice_state, SPICERESISTANCE|SPICE3, VINTEGER);
 	nextvarchangequiet();
 	(void)setvalkey((INTBIG)sim_aid, VAID, sim_spice_level, 1, VINTEGER);
+	DiaDeclareHook("spice", &sim_spicep, sim_spicedlog);
 
 	/* VERILOG initialization */
 	sim_verilog_state = makekey("SIM_verilog_state");
+	DiaDeclareHook("verilog", &sim_verp, sim_verilogdlog);
+
+	/* FastHenry initialization */
+	sim_fasthenrystatekey = makekey("SIM_fasthenry_state");
+	sim_fasthenryfreqstartkey = makekey("SIM_fasthenry_freqstart");
+	sim_fasthenryfreqendkey = makekey("SIM_fasthenry_freqend");
+	sim_fasthenryrunsperdecadekey = makekey("SIM_fasthenry_runsperdecade");
+	sim_fasthenrynumpoleskey = makekey("SIM_fasthenry_numpoles");
+	sim_fasthenryseglimitkey = makekey("SIM_fasthenry_seglimit");
+	sim_fasthenrythicknesskey = makekey("SIM_fasthenry_thickness");
+	sim_fasthenrywidthsubdivkey = makekey("SIM_fasthenry_width_subdivs");
+	sim_fasthenryheightsubdivkey = makekey("SIM_fasthenry_height_subdivs");
+	sim_fasthenryzheadkey = makekey("SIM_fasthenry_z_head");
+	sim_fasthenryztailkey = makekey("SIM_fasthenry_z_tail");
+	sim_fasthenrygroupnamekey = makekey("SIM_fasthenry_group_name");
+	sim_fasthenryinit();
 
 	/* initialize the simulation window system */
 	sim_window_init();
@@ -625,11 +768,7 @@ void sim_init(INTBIG *argc, char *argv[], AIDENTRY *thisaid)
 	sim_simnt = NONODEPROTO;
 	sim_circuitchanged = 0;
 	sim_undoredochange = 0;
-
-	/* register dialogs */
 	DiaDeclareHook("simopt", &sim_simulatorp, sim_optionsdlog);
-	DiaDeclareHook("spice", &sim_spicep, sim_spicedlog);
-	DiaDeclareHook("verilog", &sim_verp, sim_verilogdlog);
 }
 
 void sim_done(void)
@@ -654,10 +793,12 @@ void sim_done(void)
 
 INTSML sim_set(INTSML count, char *par[])
 {
-	REGISTER INTSML i, l, newformat, amount, signals;
-	REGISTER INTBIG spicestate, verilogstate, windowstate;
+	REGISTER INTSML i, l, newformat, amount, signals, curtop, vis;
+	REGISTER INTBIG spicestate, verilogstate, windowstate, options, value;
+	float fvalue;
 	REGISTER char *pp;
 	REGISTER VARIABLE *var;
+	REGISTER ARCINST *ai;
 	NODEPROTO *np;
 	NODEPROTO *simnp;
 	REGISTER INTSML active;
@@ -756,6 +897,9 @@ INTSML sim_set(INTSML count, char *par[])
 			case ABEL:
 				sim_writepalnetlist(np);
 				break;
+			case FASTHENRY:
+				sim_writefasthenrynetlist(np);
+				break;
 			case COSMOS:
 			case ESIM:
 			case RNL:
@@ -1081,8 +1225,10 @@ INTSML sim_set(INTSML count, char *par[])
 			}
 			if (namesamen(pp, "up", l) == 0 || namesamen(pp, "down", l) == 0)
 			{
+				curtop = sim_window_gettopline();
+				vis = sim_window_getvislines();
 				signals = sim_window_getlines();
-				if (count < 4) amount = signals / 2; else
+				if (count < 4) amount = vis-1; else
 				{
 					amount = (INTSML)myatoi(par[3]);
 					if (amount <= 0)
@@ -1091,8 +1237,16 @@ INTSML sim_set(INTSML count, char *par[])
 						return(1);
 					}
 				}
-
-				/* this is wrong: simals_set_current_level(); */
+				if (namesamen(pp, "down", l) == 0)
+				{
+					curtop -= amount;
+					if (curtop < 0) curtop = 0;
+				} else
+				{
+					curtop += amount;
+					if (curtop + vis > signals) curtop = signals - vis;
+				}
+				sim_window_settopline(curtop);
 				sim_window_redraw();
 				return(0);
 			}
@@ -1432,6 +1586,7 @@ INTSML sim_set(INTSML count, char *par[])
 
 	if (namesamen(pp, "verilog", l) == 0 && l >= 1)
 	{
+		l = strlen(pp = par[1]);
 		if (namesamen(pp, "use-assign", l) == 0)
 		{
 			var = getvalkey((INTBIG)sim_aid, VAID, VINTEGER, sim_verilog_state);
@@ -1467,6 +1622,345 @@ INTSML sim_set(INTSML count, char *par[])
 		return(1);
 	}
 
+	if (namesamen(pp, "fasthenry", l) == 0 && l >= 1)
+	{
+		l = strlen(pp = par[1]);
+		if (namesamen(pp, "pole", l) == 0 && l >= 3)
+		{
+			if (count <= 2)
+			{
+				ttyputerr("Usage: tellaid simulation fasthenry pole (single | multiple NUMPOLES)");
+				return(1);
+			}
+			l = strlen(pp = par[2]);
+			if (namesamen(pp, "single", l) == 0)
+			{
+				var = getvalkey((INTBIG)sim_aid, VAID, VINTEGER, sim_fasthenrystatekey);
+				if (var != NOVARIABLE) options = var->addr; else options = 0;
+				(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenrystatekey, options & ~FHMAKEMULTIPOLECKT, VINTEGER);
+				return(0);
+			}
+			if (namesamen(pp, "multiple", l) == 0)
+			{
+				if (count <= 3)
+				{
+					ttyputerr("Usage: tellaid simulation fasthenry pole multiple NUMPOLES");
+					return(1);
+				}
+				var = getvalkey((INTBIG)sim_aid, VAID, VINTEGER, sim_fasthenrystatekey);
+				if (var != NOVARIABLE) options = var->addr; else options = 0;
+				(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenrystatekey, options | FHMAKEMULTIPOLECKT, VINTEGER);
+				value = atoi(par[3]);
+				(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenrynumpoleskey, value, VINTEGER);
+				return(0);
+			}
+			ttyputerr("Bad FASTHENRY POLE option: %s", pp);
+			return(1);
+		}
+		if (namesamen(pp, "frequency", l) == 0)
+		{
+			if (count <= 2)
+			{
+				ttyputerr("Usage: tellaid simulation fasthenry frequency (single | multiple START END RUNSPERDECADE)");
+				return(1);
+			}
+			l = strlen(pp = par[2]);
+			if (namesamen(pp, "single", l) == 0)
+			{
+				var = getvalkey((INTBIG)sim_aid, VAID, VINTEGER, sim_fasthenrystatekey);
+				if (var != NOVARIABLE) options = var->addr; else options = 0;
+				options |= FHUSESINGLEFREQ;
+				(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenrystatekey, options, VINTEGER);
+				return(0);
+			}
+			if (namesamen(pp, "multiple", l) == 0)
+			{
+				if (count <= 5)
+				{
+					ttyputerr("Usage: tellaid simulation fasthenry frequency multiple START END RUNSPERDECADE");
+					return(1);
+				}
+				var = getvalkey((INTBIG)sim_aid, VAID, VINTEGER, sim_fasthenrystatekey);
+				if (var != NOVARIABLE) options = var->addr; else options = 0;
+				options &= ~FHUSESINGLEFREQ;
+				(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenrystatekey, options, VINTEGER);
+				fvalue = (float)atof(par[3]);
+				(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenryfreqstartkey, castint(fvalue), VFLOAT);
+				fvalue = (float)atof(par[4]);
+				(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenryfreqendkey, castint(fvalue), VFLOAT);
+				value = atoi(par[5]);
+				(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenryrunsperdecadekey, value, VINTEGER);
+				return(0);
+			}
+			ttyputerr("Bad FASTHENRY FREQUENCY option: %s", pp);
+			return(1);
+		}
+		if (namesamen(pp, "thickness", l) == 0)
+		{
+			if (count <= 2)
+			{
+				ttyputerr("Usage: tellaid simulation fasthenry thickness DEFTHICKNESS");
+				return(1);
+			}
+			value = atola(par[2]);
+			(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenrythicknesskey, value, VINTEGER);
+			return(0);
+		}
+		if (namesamen(pp, "width-subdivisions", l) == 0)
+		{
+			if (count <= 2)
+			{
+				ttyputerr("Usage: tellaid simulation fasthenry width-subdivisions DEFWIDTHSUBS");
+				return(1);
+			}
+			value = atoi(par[2]);
+			(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenrywidthsubdivkey, value, VINTEGER);
+			return(0);
+		}
+		if (namesamen(pp, "height-subdivisions", l) == 0)
+		{
+			if (count <= 2)
+			{
+				ttyputerr("Usage: tellaid simulation fasthenry height-subdivisions DEFHEIGHTSUBS");
+				return(1);
+			}
+			value = atoi(par[2]);
+			(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenryheightsubdivkey, value, VINTEGER);
+			return(0);
+		}
+		if (namesamen(pp, "max-segment-length", l) == 0)
+		{
+			if (count <= 2)
+			{
+				ttyputerr("Usage: tellaid simulation fasthenry max-segment-length MAXSEGLENGTH");
+				return(1);
+			}
+			value = atola(par[2]);
+			(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenryseglimitkey, value, VINTEGER);
+			return(0);
+		}
+		if (namesamen(pp, "execute", l) == 0)
+		{
+			if (count <= 2)
+			{
+				ttyputerr("Usage: tellaid simulation fasthenry execute (none | run | multiple-run)");
+				return(1);
+			}
+			l = strlen(pp = par[2]);
+			if (namesamen(pp, "none", l) == 0)
+			{
+				var = getvalkey((INTBIG)sim_aid, VAID, VINTEGER, sim_fasthenrystatekey);
+				if (var != NOVARIABLE) options = var->addr; else options = 0;
+				(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenrystatekey,
+					(options & ~FHEXECUTETYPE) | FHEXECUTENONE, VINTEGER);
+				return(0);
+			}
+			if (namesamen(pp, "run", l) == 0)
+			{
+				var = getvalkey((INTBIG)sim_aid, VAID, VINTEGER, sim_fasthenrystatekey);
+				if (var != NOVARIABLE) options = var->addr; else options = 0;
+				(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenrystatekey,
+					(options & ~FHEXECUTETYPE) | FHEXECUTERUNFH, VINTEGER);
+				return(0);
+			}
+			if (namesamen(pp, "multiple-run", l) == 0)
+			{
+				var = getvalkey((INTBIG)sim_aid, VAID, VINTEGER, sim_fasthenrystatekey);
+				if (var != NOVARIABLE) options = var->addr; else options = 0;
+				(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenrystatekey,
+					(options & ~FHEXECUTETYPE) | FHEXECUTERUNFHMUL, VINTEGER);
+				return(0);
+			}
+			ttyputerr("Bad FASTHENRY EXECUTE option: %s", pp);
+			return(1);
+		}
+		if (namesamen(pp, "postscript", l) == 0 && l >= 3)
+		{
+			if (count <= 2)
+			{
+				ttyputerr("Usage: tellaid simulation fasthenry postscript (on | off)");
+				return(1);
+			}
+			l = strlen(pp = par[2]);
+			if (namesamen(pp, "on", l) == 0 && l >= 2)
+			{
+				var = getvalkey((INTBIG)sim_aid, VAID, VINTEGER, sim_fasthenrystatekey);
+				if (var != NOVARIABLE) options = var->addr; else options = 0;
+				(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenrystatekey,
+					options | FHMAKEPOSTSCRIPTVIEW, VINTEGER);
+				return(0);
+			}
+			if (namesamen(pp, "off", l) == 0 && l >= 2)
+			{
+				var = getvalkey((INTBIG)sim_aid, VAID, VINTEGER, sim_fasthenrystatekey);
+				if (var != NOVARIABLE) options = var->addr; else options = 0;
+				(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenrystatekey,
+					options & ~FHMAKEPOSTSCRIPTVIEW, VINTEGER);
+				return(0);
+			}
+			ttyputerr("Bad FASTHENRY POSTSCRIPT option: %s", pp);
+			return(1);
+		}
+		if (namesamen(pp, "spice", l) == 0)
+		{
+			if (count <= 2)
+			{
+				ttyputerr("Usage: tellaid simulation fasthenry spice (on | off)");
+				return(1);
+			}
+			l = strlen(pp = par[2]);
+			if (namesamen(pp, "on", l) == 0 && l >= 2)
+			{
+				var = getvalkey((INTBIG)sim_aid, VAID, VINTEGER, sim_fasthenrystatekey);
+				if (var != NOVARIABLE) options = var->addr; else options = 0;
+				(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenrystatekey,
+					options | FHMAKESPICESUBCKT, VINTEGER);
+				return(0);
+			}
+			if (namesamen(pp, "off", l) == 0 && l >= 2)
+			{
+				var = getvalkey((INTBIG)sim_aid, VAID, VINTEGER, sim_fasthenrystatekey);
+				if (var != NOVARIABLE) options = var->addr; else options = 0;
+				(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenrystatekey,
+					options & ~FHMAKESPICESUBCKT, VINTEGER);
+				return(0);
+			}
+			ttyputerr("Bad FASTHENRY SPICE option: %s", pp);
+			return(1);
+		}
+		if (namesamen(pp, "arc", l) == 0)
+		{
+			ai = (ARCINST *)askaid(us_aid, "get-arc");
+			if (ai == NOARCINST)
+			{
+				ttyputerr("Select an arc first");
+				return(1);
+			}
+			if (count <= 2)
+			{
+				ttyputerr("Usage: tellaid simulation fasthenry arc OPTIONS");
+				return(1);
+			}
+			l = strlen(pp = par[2]);
+			if (namesamen(pp, "add", l) == 0)
+			{
+				if (count <= 3)
+				{
+					ttyputerr("Usage: tellaid simulation fasthenry arc add GROUPNAME");
+					return(1);
+				}
+				startobjectchange((INTBIG)ai, VARCINST);
+				setvalkey((INTBIG)ai, VARCINST, sim_fasthenrygroupnamekey, (INTBIG)par[3], VSTRING|VDISPLAY);
+				endobjectchange((INTBIG)ai, VARCINST);
+				return(0);
+			}
+			if (namesamen(pp, "remove", l) == 0)
+			{
+				if (getvalkey((INTBIG)ai, VARCINST, VSTRING, sim_fasthenrygroupnamekey) != NOVARIABLE)
+				{
+					startobjectchange((INTBIG)ai, VARCINST);
+					delvalkey((INTBIG)ai, VARCINST, sim_fasthenrygroupnamekey);
+					endobjectchange((INTBIG)ai, VARCINST);
+				}
+				return(0);
+			}
+			if (namesamen(pp, "thickness", l) == 0)
+			{
+				if (count <= 3)
+				{
+					if (getvalkey((INTBIG)ai, VARCINST, VINTEGER, sim_fasthenrythicknesskey) != NOVARIABLE)
+					{
+						startobjectchange((INTBIG)ai, VARCINST);
+						delvalkey((INTBIG)ai, VARCINST, sim_fasthenrythicknesskey);
+						endobjectchange((INTBIG)ai, VARCINST);
+					}
+				} else
+				{
+					startobjectchange((INTBIG)ai, VARCINST);
+					setvalkey((INTBIG)ai, VARCINST, sim_fasthenrythicknesskey, atola(par[3]), VINTEGER);
+					endobjectchange((INTBIG)ai, VARCINST);
+				}
+				return(0);
+			}
+			if (namesamen(pp, "width-subdivisions", l) == 0)
+			{
+				if (count <= 3)
+				{
+					if (getvalkey((INTBIG)ai, VARCINST, VINTEGER, sim_fasthenrywidthsubdivkey) != NOVARIABLE)
+					{
+						startobjectchange((INTBIG)ai, VARCINST);
+						delvalkey((INTBIG)ai, VARCINST, sim_fasthenrywidthsubdivkey);
+						endobjectchange((INTBIG)ai, VARCINST);
+					}
+				} else
+				{
+					startobjectchange((INTBIG)ai, VARCINST);
+					setvalkey((INTBIG)ai, VARCINST, sim_fasthenrywidthsubdivkey, atoi(par[3]), VINTEGER);
+					endobjectchange((INTBIG)ai, VARCINST);
+				}
+				return(0);
+			}
+			if (namesamen(pp, "height-subdivisions", l) == 0)
+			{
+				if (count <= 3)
+				{
+					if (getvalkey((INTBIG)ai, VARCINST, VINTEGER, sim_fasthenryheightsubdivkey) != NOVARIABLE)
+					{
+						startobjectchange((INTBIG)ai, VARCINST);
+						delvalkey((INTBIG)ai, VARCINST, sim_fasthenryheightsubdivkey);
+						endobjectchange((INTBIG)ai, VARCINST);
+					}
+				} else
+				{
+					startobjectchange((INTBIG)ai, VARCINST);
+					setvalkey((INTBIG)ai, VARCINST, sim_fasthenryheightsubdivkey, atoi(par[3]), VINTEGER);
+					endobjectchange((INTBIG)ai, VARCINST);
+				}
+				return(0);
+			}
+			if (namesamen(pp, "z-head", l) == 0 && l >= 3)
+			{
+				if (count <= 3)
+				{
+					if (getvalkey((INTBIG)ai, VARCINST, VINTEGER, sim_fasthenryzheadkey) != NOVARIABLE)
+					{
+						startobjectchange((INTBIG)ai, VARCINST);
+						delvalkey((INTBIG)ai, VARCINST, sim_fasthenryzheadkey);
+						endobjectchange((INTBIG)ai, VARCINST);
+					}
+				} else
+				{
+					startobjectchange((INTBIG)ai, VARCINST);
+					setvalkey((INTBIG)ai, VARCINST, sim_fasthenryzheadkey, atola(par[3]), VINTEGER);
+					endobjectchange((INTBIG)ai, VARCINST);
+				}
+				return(0);
+			}
+			if (namesamen(pp, "z-tail", l) == 0 && l >= 3)
+			{
+				if (count <= 3)
+				{
+					if (getvalkey((INTBIG)ai, VARCINST, VINTEGER, sim_fasthenryztailkey) != NOVARIABLE)
+					{
+						startobjectchange((INTBIG)ai, VARCINST);
+						delvalkey((INTBIG)ai, VARCINST, sim_fasthenryztailkey);
+						endobjectchange((INTBIG)ai, VARCINST);
+					}
+				} else
+				{
+					startobjectchange((INTBIG)ai, VARCINST);
+					setvalkey((INTBIG)ai, VARCINST, sim_fasthenryztailkey, atola(par[3]), VINTEGER);
+					endobjectchange((INTBIG)ai, VARCINST);
+				}
+				return(0);
+			}
+			ttyputerr("Bad FASTHENRY ARC option: %s", pp);
+			return(1);
+		}
+		ttyputerr("Bad FASTHENRY option: %s", pp);
+		return(1);
+	}
+
 	/* unknown command */
 	ttyputerr("Unknown simulation option: %s", pp);
 	return(1);
@@ -1581,12 +2075,15 @@ void sim_killvariable(INTBIG addr, INTBIG type, INTBIG key, INTBIG oldaddr, INTB
  * Auto advance time      = 4 (check)
  * Multistate display     = 5 (check)
  * Show waveform window   = 6 (check)
+ * Maximum events         = 8 (edit text)
  */
 void sim_optionsdlog(void)
 {
-	INTBIG itemHit;
+	INTBIG itemHit, newtracesize;
 	REGISTER VARIABLE *var;
+	char line[20];
 	REGISTER INTBIG noupdate, windowstate;
+	REGISTER TRAKPTR newtrakroot;
 
 	if (DiaInitDialog(&sim_optionsdialog) != 0) return;
 	var = getval((INTBIG)sim_aid, VAID, VINTEGER, "SIM_als_no_update");
@@ -1597,6 +2094,14 @@ void sim_optionsdlog(void)
 	if ((windowstate&ADVANCETIME) != 0) DiaSetControl(4, 1);
 	if ((windowstate&FULLSTATE) != 0) DiaSetControl(5, 1);
 	if ((windowstate&SHOWWAVEFORM) != 0) DiaSetControl(6, 1);
+	if (simals_trace_size == 0)
+	{
+		var = getval((INTBIG)sim_aid, VAID, VINTEGER, "SIM_als_num_events");
+		if (var == NOVARIABLE) simals_trace_size = DEFAULT_TRACE_SIZE; else
+			simals_trace_size = var->addr;
+	}
+	sprintf(line, "%ld", simals_trace_size);
+	DiaSetText(8, line);
 
 	/* loop until done */
 	for(;;)
@@ -1624,6 +2129,19 @@ void sim_optionsdlog(void)
 			noupdate, VINTEGER);
 		(void)setval((INTBIG)sim_aid, VAID, "SIM_window_state",
 			windowstate, VINTEGER);
+		newtracesize = atoi(DiaGetText(8));
+		if (newtracesize != simals_trace_size)
+		{
+			newtrakroot = (TRAKPTR)simals_alloc_mem((INTBIG)(newtracesize * sizeof(TRAK)));
+			if (newtrakroot != 0)
+			{
+				if (simals_trakroot != 0) efree((char *)simals_trakroot);
+				simals_trakroot = newtrakroot;
+				simals_trace_size = newtracesize;
+				(void)setval((INTBIG)sim_aid, VAID, "SIM_als_num_events",
+					newtracesize, VINTEGER);
+			}
+		}
 	}
 	DiaDoneDialog();
 }
@@ -1821,7 +2339,7 @@ void sim_spicedlog(void)
 	DiaSetPopup(7, 3, levelnames);
 
 	/* state of the execute flag */
-	canexecute = us_graphicshas(CANRUNPROCESS);
+	canexecute = graphicshas(CANRUNPROCESS);
 	if (canexecute != 0)
 	{
 		var = getvalkey((INTBIG)sim_aid, VAID, VINTEGER, sim_dontrun);
@@ -1888,7 +2406,7 @@ void sim_spicedlog(void)
 	}
 
 	/* list of facets for model description files */
-	DiaInitTextDialog(23, us_topoffacets, us_nextparse, DiaNullDlogDone,
+	DiaInitTextDialog(23, topoffacets, nextfacets, DiaNullDlogDone,
 		0, SCSELMOUSE|SCSELKEY|SCREPORT);
 	for(np = el_curlib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 	{
@@ -2286,7 +2804,7 @@ void sim_verilogdlog(void)
 	if ((verilog_state&VERILOGUSEASSIGN) != 0) DiaSetControl(7, 1);
 
 	/* list of facets for model description files */
-	DiaInitTextDialog(8, us_topoffacets, us_nextparse, DiaNullDlogDone,
+	DiaInitTextDialog(8, topoffacets, nextfacets, DiaNullDlogDone,
 		0, SCSELMOUSE|SCSELKEY|SCREPORT);
 	for(np = el_curlib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 	{
