@@ -55,6 +55,7 @@ INTSML      simals_seed_flag = 1;
 float       simals_time_abs, simals_delta_def, simals_linear_def,
 			simals_exp_def, simals_random_def, simals_abs_def;
 NODEPROTO  *simals_mainproto = NONODEPROTO;
+INTBIG      simals_trace_size = 0;
 
 /********************************* LOCALS *********************************/
 
@@ -84,6 +85,8 @@ INTSML simals_parse_delay(void);
 
 void simals_init(void)
 {
+	REGISTER VARIABLE *var;
+
 	/* allocate memory */
 	if (simals_instbuf == 0)
 	{
@@ -93,9 +96,15 @@ void simals_init(void)
 	{
 		if (simals_alloc_ptr_buffer(&simals_instptr, &simals_iptrsize, 100) != 0) return;
 	}
+	if (simals_trace_size == 0)
+	{
+		var = getval((INTBIG)sim_aid, VAID, VINTEGER, "SIM_als_num_events");
+		if (var == NOVARIABLE) simals_trace_size = DEFAULT_TRACE_SIZE; else
+			simals_trace_size = var->addr;
+	}
 	if (simals_trakroot == 0)
 	{
-		simals_trakroot = (TRAKPTR)simals_alloc_mem((INTBIG)(TRACE_SIZE * sizeof(TRAK)));
+		simals_trakroot = (TRAKPTR)simals_alloc_mem((INTBIG)(simals_trace_size * sizeof(TRAK)));
 		if (simals_trakroot == 0) return;
 	}
 	simals_no_update = makekey("SIM_als_no_update");
@@ -1815,7 +1824,7 @@ char *simals_nextinstance(void)
  * Calling Argument:
  *	sp = pointer to char string containing path name to level in xref table
  */
-CONPTR  simals_find_level(char *sp)
+CONPTR simals_find_level(char *sp)
 {
 	INTSML    i, j;
 	char    s1[80];
