@@ -77,7 +77,7 @@ static COMCOMP sim_window_pickdtrace = {NOKEYWORD, sim_window_topofdsignals,
 INTSML simals_charhandlerwave(WINDOWPART *w, INTSML chr)
 {
 	REGISTER INTSML i, j, traces, pos, thispos, lines, strength;
-	REGISTER INTBIG tr, trl, state, highsig, windowstate;
+	REGISTER INTBIG tr, trl, nexttr, prevtr, state, highsig, windowstate;
 	REGISTER VARIABLE *var;
 	char *par[30], *name;
 	NODEPTR node, nodehead;
@@ -85,6 +85,8 @@ INTSML simals_charhandlerwave(WINDOWPART *w, INTSML chr)
 	float endtime;
 	LINKPTR sethead;
 
+	ttynewcommand();
+
 	/* can always preserve snapshot */
 	if (chr == 'p')
 	{
@@ -238,12 +240,18 @@ INTSML simals_charhandlerwave(WINDOWPART *w, INTSML chr)
 		/* delete it */
 		thispos = sim_window_gettraceline(highsig);
 		sim_window_inittraceloop();
+		nexttr = prevtr = 0;
 		for(;;)
 		{
 			trl = sim_window_nexttraceloop();
 			if (trl == 0) break;
 			pos = sim_window_gettraceline(trl);
-			if (pos > thispos) sim_window_settraceline(trl, (INTSML)(pos-1));
+			if (pos > thispos)
+			{
+				pos--;
+				if (pos == thispos) nexttr = trl;
+				sim_window_settraceline(trl, (INTSML)pos);
+			} else if (pos == thispos-1) prevtr = trl;
 		}
 		lines = sim_window_getlines();
 		if (lines > 1) sim_window_setlines((INTSML)(lines-1));
@@ -262,6 +270,8 @@ INTSML simals_charhandlerwave(WINDOWPART *w, INTSML chr)
 
 		/* kill trace, redraw */
 		sim_window_killtrace(highsig);
+		if (nexttr != 0) sim_window_sethighlighttrace(nexttr); else
+			if (prevtr != 0) sim_window_sethighlighttrace(prevtr);
 		sim_window_redraw();
 		return(0);
 	}
@@ -301,6 +311,7 @@ INTSML simals_charhandlerschem(WINDOWPART *w, INTSML chr)
 	NODEPTR nodehead;
 	LINKPTR sethead;
 
+	ttynewcommand();
 	switch (chr)
 	{
 		case '?':
@@ -580,7 +591,7 @@ void simals_fill_display_arrays(void)
 
 	sim_window_gettimerange(&min, &max);
 	if (simals_trakfull == 0) i = 0; else i = (INTSML)simals_trakptr;
-	for( ; i < simals_trakptr && i < TRACE_SIZE; i++)
+	for( ; i < simals_trakptr && i < simals_trace_size; i++)
 	{
 		trakhead = &(simals_trakroot[i]);
 		if (trakhead->time > max) break;
@@ -610,7 +621,7 @@ void simals_fill_display_arrays(void)
 		statearrays[i][0] = (LOGIC_LOW << 8) | 0;
 	}
 	if (simals_trakfull == 0) i = 0; else i = (INTSML)simals_trakptr;
-	for( ; i < simals_trakptr && i < TRACE_SIZE; i++)
+	for( ; i < simals_trakptr && i < simals_trace_size; i++)
 	{
 		trakhead = &(simals_trakroot[i]);
 		if (trakhead->time > max) break;
