@@ -296,7 +296,7 @@ INTSML simals_fire_event(void)
 			{
 				simals_calculate_clock_time(linkhead, rowhead);
 				return(0);
-		}
+			}
 	}
 
 	simals_free_link_mem(linkhead);
@@ -356,7 +356,7 @@ void simals_create_check_list(NODEPTR nodehead)
 		trakhead->state = state;
 		trakhead->strength = strength;
 		trakhead->time = simals_time_abs;
-		simals_trakptr = (simals_trakptr + 1) % TRACE_SIZE;
+		simals_trakptr = (simals_trakptr + 1) % simals_trace_size;
 		if (! simals_trakptr)
 		{
 			simals_trakfull = 1;
