@@ -64,6 +64,7 @@ void sim_writemossim(NODEPROTO *np)
 	char name[100], *truename;
 	REGISTER NODEPROTO *lnp;
 	extern AIDENTRY *net_aid;
+	REGISTER LIBRARY *lib;
 
 	/* make sure network tool is on */
 	if ((net_aid->aidstate&AIDON) == 0)
@@ -88,8 +89,9 @@ void sim_writemossim(NODEPROTO *np)
 		sim_mossim_strength = makekey("SIM_mossim_strength");
 
 	/* reset flags for facets that have been written */
-	for(lnp = np->cell->lib->firstnodeproto; lnp != NONODEPROTO; lnp = lnp->nextnodeproto)
-		lnp->temp1 = 0;
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(lnp = lib->firstnodeproto; lnp != NONODEPROTO; lnp = lnp->nextnodeproto)
+			lnp->temp1 = 0;
 	if (sim_writemossimfacet(np, 1) != 0)
 		ttyputmsg("Back-annotation information has been added (library must be saved)");
 
