@@ -98,6 +98,7 @@ void sim_writesilnetlist(NODEPROTO *np)
 	char name[100], numberstring[100], *truename;
 	REGISTER NODEPROTO *lnp;
 	extern AIDENTRY *net_aid;
+	REGISTER LIBRARY *lib;
 	UINTBIG cdate, rdate;
 	INTBIG rnum;
 
@@ -161,8 +162,9 @@ void sim_writesilnetlist(NODEPROTO *np)
 	if (sim_silos_model_key == 0) sim_silos_model_key = makekey("SC_silos");
 
 	/* reset flags for facets that have been written */
-	for(lnp = np->cell->lib->firstnodeproto; lnp != NONODEPROTO; lnp = lnp->nextnodeproto)
-		lnp->temp1 = 0;
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(lnp = lib->firstnodeproto; lnp != NONODEPROTO; lnp = lnp->nextnodeproto)
+			lnp->temp1 = 0;
 	if (sim_writesilfacet(np, 1) != 0)
 		ttyputmsg("Back-annotation information has been added (library must be saved)");
 	sim_sendtosil("\n");
