@@ -328,7 +328,7 @@ char       *sim_spice_dc;							/* DC analysis message */
 char       *sim_spice_tran;							/* Transient analysis message */
 POLYGON    *sim_polygonfree = NOPOLYGON;			/* list of free simulation polygons */
 float       sim_spice_mask_scale = 1.0;				/* Mask shrink factor (default =1) */
-INTBIG     *sim_spice_extra_area = NULL;			/* Duplicated area on each layer */
+float      *sim_spice_extra_area = 0;				/* Duplicated area on each layer */
 INTSML      sim_spice_diffusion_index[DIFFTYPES];	/* diffusion layers indices */
 INTSML      sim_spice_layer_count;
 INTBIG      sim_spice_level;						/* key for "SIM_spice_level" */
@@ -383,7 +383,7 @@ char *sim_spice_pp_name(PORTPROTO*);
 void sim_spice_xputs(char*, FILE*);
 char *sim_spice_describesource(NODEINST*);
 void sim_spice_store_box(TECHNOLOGY*, INTSML, INTBIG, INTBIG, INTBIG, INTBIG);
-void sim_spice_evalpolygon(INTSML, TECHNOLOGY*, INTBIG*, INTBIG*, INTSML, float);
+void sim_spice_evalpolygon(INTSML, TECHNOLOGY*, INTBIG*, INTBIG*, INTSML);
 INTSML sim_spice_layerisdiff(TECHNOLOGY*, INTSML);
 INTSML sim_spice_arcisdiff(ARCINST*);
 float sim_spice_resistance(TECHNOLOGY*, INTSML);
@@ -495,6 +495,7 @@ void sim_writespice(NODEPROTO *np)
 	REGISTER INTSML retval;
 	char deckfile[256], *pt;
 	REGISTER VARIABLE *var;
+	REGISTER LIBRARY *lib;
 	REGISTER NODEPROTO *onp;
 	REGISTER TECHNOLOGY *tech, *t;
 	extern AIDENTRY *net_aid;
@@ -536,8 +537,8 @@ void sim_writespice(NODEPROTO *np)
 	var = getval((INTBIG)tech, VTECHNOLOGY, VFLOAT, "SIM_spice_mask_scale");
 	if (var != NOVARIABLE) sim_spice_mask_scale = castfloat(var->addr); else
 		sim_spice_mask_scale = 1.0;
-	sim_spice_extra_area =
-		  emalloc((sizeof(INTBIG)*sim_spice_layer_count),sim_aid->cluster);
+	sim_spice_extra_area = (float *)emalloc((sizeof(float)*sim_spice_layer_count),
+		sim_aid->cluster);
 
 	/* get the layer resistance and capacitance arrays for each technology */
 	for(t = el_technologies; t != NOTECHNOLOGY; t = t->nexttechnology)
@@ -563,7 +564,8 @@ void sim_writespice(NODEPROTO *np)
 	(void)sim_spice_writeheader(np->cell, spfile, tech);
 
 	/* reset bits on all facets */
-	for(onp = np->cell->lib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(onp = lib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
 	{
 		onp->temp1 = 0;
 		onp->cell->temp1 = 0;
@@ -819,7 +821,7 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 		net->netnumber = netindex++;
 
 		/* reset */
-		for (j = 0; j < sim_spice_layer_count; j++) sim_spice_extra_area[j] = 0;
+		for (j = 0; j < sim_spice_layer_count; j++) sim_spice_extra_area[j] = 0.0;
 		for (j = 0; j < DIFFTYPES; j++) sim_spice_diffusion_index[j] = -1;
 		net->nextnet = sim_spice_firstnet;
 		sim_spice_firstnet = net;
@@ -864,7 +866,7 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 		ni = pp->subnodeinst;
 
 		/* reset */
-		for (j = 0; j < sim_spice_layer_count; j++) sim_spice_extra_area[j] = 0;
+		for (j = 0; j < sim_spice_layer_count; j++) sim_spice_extra_area[j] = 0.0;
 		for (j = 0; j < DIFFTYPES; j++) sim_spice_diffusion_index[j] = -1;
 		sim_spice_nodearea(net, ni, pp->subportproto);
 
@@ -1979,11 +1981,14 @@ void sim_spice_nodearea(NET *net, NODEINST *ni, PORTPROTO *pp)
 					{
 						dominant = -1;      /* Flag for diffusion on this port */
 						break;
-					} else if ((cap = abs(areapoly(poly)) *
-						sim_spice_capacitance(tech, poly->layer)) > worst)
+					} else
 					{
-						worst = cap;
-						dominant = poly->layer;
+						cap = (float)fabs(areapoly(poly));
+						if (cap * sim_spice_capacitance(tech, poly->layer) > worst)
+						{
+							worst = cap;
+							dominant = poly->layer;
+						}
 					}
 				}
 			}
@@ -2007,7 +2012,7 @@ void sim_spice_nodearea(NET *net, NODEINST *ni, PORTPROTO *pp)
 					layername(tech,poly->layer));
 			if (sim_spice_layerisdiff(tech, poly->layer) == ISNONE &&
 				poly->layer != dominant)
-					sim_spice_extra_area[poly->layer] += abs(areapoly(poly));
+					sim_spice_extra_area[poly->layer] += (float)fabs(areapoly(poly));
 		}
 
 		/* free the polygons */
@@ -2296,9 +2301,10 @@ void sim_spice_store_box(TECHNOLOGY *tech, INTSML layer, INTBIG xs, INTBIG ys, I
 /*
  * routine to obtain a polygon from the box merging system
  */
-void sim_spice_evalpolygon(INTSML layer, TECHNOLOGY *tech, INTBIG *xbuf, INTBIG *ybuf, INTSML count, float area)
+void sim_spice_evalpolygon(INTSML layer, TECHNOLOGY *tech, INTBIG *xbuf, INTBIG *ybuf, INTSML count)
 {
 	REGISTER INTBIG perim;
+	float area;
 	REGISTER INTSML i, j;
 
 	if (sim_spice_debug != 0)
@@ -2315,10 +2321,12 @@ void sim_spice_evalpolygon(INTSML layer, TECHNOLOGY *tech, INTBIG *xbuf, INTBIG
 		perim += computedistance(xbuf[j], ybuf[j], xbuf[i], ybuf[i]);
 	}
 
-	if (sim_spice_extra_area[layer] != 0)/* adjust area */
+	/* get area */
+	area = areapoints(count, xbuf, ybuf);
+	if (sim_spice_extra_area[layer] != 0.0)
 	{
 		area -= sim_spice_extra_area[layer];
-			sim_spice_extra_area[layer] = 0; /* but only once */
+		sim_spice_extra_area[layer] = 0.0; /* but only once */
 	}
 
 	i = sim_spice_layerisdiff(tech, layer);
