@@ -77,10 +77,11 @@ INTBIG sim_strcspn(char *str, char *set);
 void sim_writetexnetlist(NODEPROTO *np)
 {
 	NODEPROTO *lnp;
+	REGISTER LIBRARY *lib;
 	INTBIG length;
 	char str1[30], filename[200], *libname, *resfilename, *truename;
 
-	/*  WRITE THE "TDL" FILE */
+	/* write the "TDL" file */
 	(void)strcpy(filename, np->cell->cellname);
 	(void)strcat(filename, ".tdl");
 	sim_texfile = xcreate(filename, FILETYPETEGAS, "TEGAS File", &truename);
@@ -90,13 +91,13 @@ void sim_writetexnetlist(NODEPROTO *np)
 		return;
 	}
 
-	/* READ LIST OF RESERVED WORDS INTO AN INFINITE STRING, EACH SEPARATED BY "\n" */
+	/* read list of reserved words into an infinite string, each separated by "\n" */
 	reservefile = xopen(RESFILE, FILETYPETEGASTAB, "", &resfilename);
 
 	sim_texstrprint("/* GENERATED BY THE ELECTRIC VLSI DESIGN SYSTEM */\n\n");
 	sim_texstrprint("COMPILE  ;\n\n");
 
-	/* GET LIBRARY NAME, CHECK VALIDITY */
+	/* get library name, check validity */
 	strcpy(str1, "DIRECTORY:  ");
 	libname = sim_texlibname(np);
 
@@ -107,7 +108,7 @@ void sim_writetexnetlist(NODEPROTO *np)
 		ttyputmsg("TDL directory will be truncated to :- %s", sim_texname(libname));
 	}
 
-	/* CHECK LIBRARY NAME */
+	/* check library name */
 	if (sim_texcheckname(sim_texname(libname)))
 		strcat(str1, sim_texname(libname)); else
 	{
@@ -115,22 +116,23 @@ void sim_writetexnetlist(NODEPROTO *np)
 		return;
 	}
 
-	/* WRITE "DIRECTORY LINE" TO FILE */
+	/* write "directory line" to file */
 	sim_texstrprint(str1);
 	xprintf(sim_texfile, " ;\n\n");
 
 	sim_texstrprint("OPTIONS:   REPLACE  ;\n\n");
 
-	/* RESET FLAGS FOR CELLS THAT HAVE BEEN WRITTEN */
-	for(lnp = np->cell->lib->firstnodeproto; lnp != NONODEPROTO; lnp = lnp->nextnodeproto)
-		lnp->temp1 = lnp->temp2 = 0;
+	/* reset flags for cells that have been written */
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(lnp = lib->firstnodeproto; lnp != NONODEPROTO; lnp = lnp->nextnodeproto)
+			lnp->temp1 = lnp->temp2 = 0;
 
-	/* WRITE TDL DESCRIPTIONS INTO FILE */
+	/* write TDL descriptions into file */
 	sim_writetexcell(np, 1);
 
 	sim_texstrprint(" END COMPILE;\n\n ");
 
-	/* CLOSE  FILES */
+	/* close files */
 	if (reservefile != NULL) xclose(reservefile);
 	xclose(sim_texfile);
 
@@ -147,7 +149,7 @@ void sim_writetexcell(NODEPROTO *np, INTBIG top)
 		count, length, tempa, tempb, module_err=0;
 	INTBIG pwr=0, gnd=0;
 
-	/* RECURSE ON SUB-CELS FIRST */
+	/* recurse on sub-cels first */
 	for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 	{
 		if (ni->proto->primindex != 0) continue;
@@ -336,7 +338,7 @@ void sim_writetexcell(NODEPROTO *np, INTBIG top)
 	/*************************  DEFINE  ***********************************/
 	sim_texstrprint("DEFINE:\n");
 
-	/* COUNT NO. OF INVERTERS (NEGATED ARCS NOT ATTACHED TO LOGIC PRIMITIVES */
+	/* count no. of inverters (negated arcs not attached to logic primitives */
 	count = 1;
 	for (ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
 	{
@@ -367,7 +369,7 @@ void sim_writetexcell(NODEPROTO *np, INTBIG top)
 		ni->temp1 = 0;
 		f = nodefunction(ni,&extra);
 
-		/* HANDLE NODEINST DESCRIPTIONS */
+		/* handle nodeinst descriptions */
 		switch(f)
 		{
 			case NPPIN:
@@ -402,8 +404,8 @@ void sim_writetexcell(NODEPROTO *np, INTBIG top)
 		}
 	}
 
-	/* ANY PORTS ON THE SAME NETWORK IN THE CELL HAVE TO BE MADE EQUIVALENT POINTS */
-	/* IN THE TDL DESCRIPTION */
+	/* any ports on the same network in the cell have to be made equivalent points */
+	/* in the tdl description */
 	for(pp=np->firstportproto; pp!= NOPORTPROTO; pp=pp->nextportproto)
 	{
 		if (pp->temp2 != 0) continue;
@@ -420,10 +422,10 @@ void sim_writetexcell(NODEPROTO *np, INTBIG top)
 		pp->temp2++;
 	}
 
-	/* END MODULE */
+	/* end module */
 	sim_texstrprint(" END MODULE;\n\n");
 
-	/* IF COMPILED CORRECTLY THEN RESET VARIABLE */
+	/* if compiled correctly then reset variable */
 	if (module_err != 0) ttyputerr("*** Errors in module %s",
 		sim_texname(describenodeproto(np)));
 }
@@ -456,7 +458,7 @@ char *sim_textdlpower(NODEINST *ni, INTBIG f, INTBIG *pwr, INTBIG *gnd, INTBIG *
 	return("");
 }
 
-/* ROUTINE TO COUNT THE WIRES ON A PRIMITIVE WITH ONLY ONE INPORT */
+/* routine to count the wires on a primitive with only one inport */
 INTSML sim_texcountwires(NODEINST *ni)
 {
 	INTSML inputs;
@@ -473,8 +475,8 @@ INTSML sim_texcountwires(NODEINST *ni)
 }
 
 /*
- * A ROUTINE TO RETURN THE NAME OF THE NETWORK WHICH MAY BE NET<x> OR THE
- * NAME OF AN EXPORTED PORT ON THE NET
+ * a routine to return the name of the network which may be net<x> or the
+ * name of an exported port on the net
  */
 char *sim_texsrcname(NODEINST *ni, PORTARCINST *pai)
 {
@@ -497,7 +499,7 @@ char *sim_texsrcname(NODEINST *ni, PORTARCINST *pai)
 	return(str);
 }
 
-/* A ROUTINE TO WRITE THE SOURCES OF SIGNALS ON THE INPUT PORTS OF A NODEINST */
+/* a routine to write the sources of signals on the input ports of a nodeinst */
 char *sim_texwritesignals(NODEINST *ni, INTBIG f, INTBIG *module_err)
 {
 	PORTPROTO *pp;
@@ -565,8 +567,8 @@ char *sim_texwritesignals(NODEINST *ni, INTBIG f, INTBIG *module_err)
 }
 
 /*
- * ROUTINE TO WRITE THE SIGNALS IN THE CORRECT PIN ORDER FOR TDL FOR FLIP
- * FLOPS N.B THIS IS ESSENTIALLY THE SAME AS sim_texwritesignals
+ * routine to write the signals in the correct pin order for TDL for flip
+ * flops n.b this is essentially the same as sim_texwritesignals
  */
 char *sim_texwriteffsignals(NODEINST *ni, INTBIG *module_err)
 {
@@ -628,8 +630,8 @@ char *sim_texwriteffsignals(NODEINST *ni, INTBIG *module_err)
 }
 
 /*
- * ROUTINE TO TRACE BACK TO THE OUTPUT SOURCE SUPPLYING A SIGNAL TO THE
- * PORTARCINST OF A NODEINST
+ * routine to trace back to the output source supplying a signal to the
+ * portarcinst of a nodeinst
  */
 char *sim_textracesignal(PORTARCINST *pai, NODEINST *ni)
 {
@@ -648,7 +650,7 @@ char *sim_textracesignal(PORTARCINST *pai, NODEINST *ni)
 	return(sim_texsrcname(ni, pai));
 }
 
-/* WRITE OUT AN INVERTER DESCRIPTION FOR A NEGATED ARC */
+/* write out an inverter description for a negated arc */
 void sim_texwriteinv(PORTARCINST *pai, char *str1, char *str2)
 {
 	static char invstring[MAXNAMECHARS * 4];
@@ -663,9 +665,9 @@ void sim_texwriteinv(PORTARCINST *pai, char *str1, char *str2)
 }
 
 /*
- * ROUTINE TO RETURN A STRING WITH THE TDL NAMES OF THE OUTPUT SOURCES FROM
- * THIS NODEINST. IF NECCESSARY ARCS THAT ARE NEGATED WILL HAVE INVERTER
- * DESCRIPTIONS WRITTEN.
+ * routine to return a string with the TDL names of the output sources from
+ * this nodeinst. if neccessary arcs that are negated will have inverter
+ * descriptions written.
  */
 char *sim_textdlinst(NODEINST *ni, INTBIG f)
 {
@@ -767,7 +769,7 @@ char *sim_textdlinst(NODEINST *ni, INTBIG f)
 	return(netstr);
 }
 
-/* TO HANDLE ALL ERROR CONDITIONS */
+/* To handle all error conditions */
 void sim_texerror(INTBIG error, INTBIG *lasterror, char *str)
 {
 	char *msg;
@@ -826,10 +828,10 @@ void sim_texerror(INTBIG error, INTBIG *lasterror, char *str)
 }
 
 /*
- * THIS ROUTINE OUTPUTS TDL TO THE .TDL FILE. THE MAXIMUM LINE LENGTH IN A
- * TDL FILE IS 80 CHARS. THIS ROUTINE ENSURES THAT NO LINES EXCEED THIS
- * IT SEARCHES FOR THE LAST SPACE CHARACTER AND INSERTS A NEW LINE CHARACTER
- * AT THAT POINT
+ * this routine outputs TDL to the .TDL file. the maximum line length in a
+ * TDL file is 80 chars. this routine ensures that no lines exceed this
+ * it searches for the last space character and inserts a new line character
+ * at that point
  */
 void sim_texstrprint(char *str)
 {
@@ -875,7 +877,7 @@ void sim_texstrprint(char *str)
 	}
 }
 
-/* RETURNS A STRING OF MAX 12 CHARS AND IN UPPER CASE FOR TDL */
+/* returns a string of max 12 chars and in upper case for TDL */
 char *sim_texname(char *str)
 {
 	static char buffer[MAXNAMECHARS +1], letter;
@@ -894,8 +896,8 @@ char *sim_texname(char *str)
 }
 
 /*
- * ROUTINE TO RETURN THE TDL NAME OF A NODEINST
- * THIS ROUTINE WILL STRIP OUT A TDL DIRECTORY NAME IF PRESENT
+ * routine to return the TDL name of a nodeinst
+ * this routine will strip out a TDL directory name if present
  */
 char *sim_texproto(NODEINST *ni, INTBIG f, INTBIG *module_err)
 {
@@ -947,7 +949,7 @@ char *sim_texproto(NODEINST *ni, INTBIG f, INTBIG *module_err)
 	}
 }
 
-/* ROUTINE TO RETURN THE TDL PRIMITIVE NAME OF A FLIP-FLOP IN ELECTRIC */
+/* routine to return the TDL primitive name of a flip-flop in electric */
 char *sim_texffname(NODEINST *ni, INTBIG *module_err)
 {
 	VARIABLE *var;
@@ -1003,8 +1005,8 @@ char *sim_texffname(NODEINST *ni, INTBIG *module_err)
 }
 
 /*
- * THIS ROUTINE RETURNS A STRING CONTAINING THE TDL DELAY. IF NO DELAY VARIABLE
- * PRESENT THE DEFAULT "/1,1/" IS RETURNED
+ * this routine returns a string containing the TDL delay. if no delay variable
+ * present the default "/1,1/" is returned
  */
 char *sim_texgetdelay(NODEINST *ni, INTBIG f)
 {
@@ -1015,8 +1017,8 @@ char *sim_texgetdelay(NODEINST *ni, INTBIG f)
 	/* user defined modules are not allowed delays */
 	if (f == NPUNKNOWN) return("");
 
-	/* INITIALISE VARIABLE FOR SIGNAL SOURCE NAMES TO BE HELD ON PORTARCINSTS */
-	/* AND TEX RISE AND FALL TIME DELAYS */
+	/* initialise variable for signal source names to be held on portarcinsts */
+	/* and TEX rise and fall time delays */
 	if (SIM_rise_delay == 0) SIM_rise_delay = makekey("SIM_rise_delay");
 	if (SIM_fall_delay == 0) SIM_fall_delay = makekey("SIM_fall_delay");
 
@@ -1038,7 +1040,7 @@ char *sim_texgetdelay(NODEINST *ni, INTBIG f)
 	return(str1);
 }
 
-/* DETERMINES WHETHER A NODEINST HAS AN OUTPUT ARC THAT IS NEGATED */
+/* determines whether a nodeinst has an output arc that is negated */
 INTSML sim_texnegatedoutput(NODEINST *ni)
 {
 	PORTARCINST *pai;
@@ -1051,7 +1053,7 @@ INTSML sim_texnegatedoutput(NODEINST *ni)
 	return(0);
 }
 
-/* ROUTINE TO REMOVE THE PATH SPECIFICATION FROM THE ELECTRIC LIBRARY NAME */
+/* routine to remove the path specification from the electric library name */
 char *sim_texlibname(NODEPROTO *np)
 {
 	INTBIG separator;
@@ -1064,8 +1066,8 @@ char *sim_texlibname(NODEPROTO *np)
 }
 
 /*
- * ROUTINE TO RETURN THE PORTARCINST AT THE OPPOSITE END OF THE ARC WHICH
- * CONNECTS TO pai
+ * routine to return the portarcinst at the opposite end of the arc which
+ * connects tO pai
  */
 PORTARCINST *sim_oppendpai(PORTARCINST *pai)
 {
@@ -1075,8 +1077,8 @@ PORTARCINST *sim_oppendpai(PORTARCINST *pai)
 }
 
 /*
- * ROUTINE TO RETURN THE NODEINST AT THE OPPOSITE END OF THE ARC WHICH
- * CONNECTS TO pai
+ * routine to return the nodeinst at the opposite end of the arc which
+ * connects to pai
  */
 NODEINST *sim_oppendni(PORTARCINST *pai)
 {
@@ -1086,8 +1088,8 @@ NODEINST *sim_oppendni(PORTARCINST *pai)
 }
 
 /*
- * ROUTINE THAT TAKES A STRING OF MAX. LENGTH 12 AND CHECKS TO SEE IF IT IS A
- * RESERVED WORD
+ * routine that takes a string of max. length 12 and checks to see if it is a
+ * reserved word
  */
 INTSML sim_texcheckname(char *str)
 {
