@@ -52,6 +52,10 @@
  * sim_window_redraw()                          Redraws simulation window
  * sim_window_setlines(count)                   Sets number of lines in simulation window
  * INTSML sim_window_getlines()                 Returns number of lines in sim window
+ * sim_window_setvislines(count)                Sets number of visible lines in simulation window
+ * INTSML sim_window_gettopline()               Returns top line in simulation window
+ * sim_window_settopline(count)                 Sets top visible line in simulation window
+ * INTSML sim_window_getvislines()              Returns number of visible lines in sim window
  * sim_window_savegraph()                       Preserves plot in database
  * sim_window_displaycolor(s, c)                Set color of strength/state "s" to "c"
  *
@@ -129,6 +133,8 @@ static INTSML sim_window_anacolor[] = {RED, GREEN, BLUE, CYAN, MAGENTA, YELLOW,
 static INTSML        sim_window_curx, sim_window_cury;
 static INTSML        sim_window_textsize;
 static INTSML        sim_window_lines;
+static INTSML        sim_window_vislines;
+static INTSML        sim_window_topline;
 static INTSML        sim_window_txthei;
 static INTSML        sim_window_offx, sim_window_offy;
 static INTSML        sim_window_basex, sim_window_basey;
@@ -149,9 +155,13 @@ static INTSML        sim_colorstrengthpower;	/* color of power-strength */
 static INTSML        sim_colorlevellow;			/* color of low-levels */
 static INTSML        sim_colorlevelhigh;		/* color of high-levels */
 static INTSML        sim_colorlevelundef;		/* color of power-levels */
+static WINDOWPART   *sim_waveformwindow;		/* current window when arrows are clicked */
+static INTBIG        sim_waveformsliderpart;	/* 0:down arrow 1:below thumb 2:thumb 3:above thumb 4:up arrow */
 
 /* prototypes for local routines */
 void   sim_window_addsegment(WINDOWPART*, INTBIG*, INTBIG*, INTSML, INTSML, INTSML, INTSML);
+INTSML sim_window_verticalslider(INTBIG x, INTBIG y);
+INTSML sim_window_horizontalslider(INTBIG x, INTBIG y);
 void   sim_window_buttonhandler(WINDOWPART*, INTSML, INTSML, INTSML);
 void   sim_window_drawbox(WINDOWPART*, INTSML, INTSML, INTSML, INTSML);
 void   sim_window_drawcstring(WINDOWPART*, char*);
@@ -191,6 +201,8 @@ void sim_window_init(void)
 	sim_window_maincursor = 0.0000002f;
 	sim_window_extensioncursor = 0.0000003f;
 	sim_window_lines = 0;
+	sim_window_vislines = 0;
+	sim_window_topline = 0;
 	sim_window_plot = NODISCHANNEL;
 	sim_window_plotfree = NODISCHANNEL;
 	sim_window_highlightedtrace = NODISCHANNEL;
@@ -255,8 +267,13 @@ INTSML sim_window_create(INTSML linecount, NODEPROTO *np, INTSML (*charhandlerwa
 		sim_window_lines = linecount;
 		if (sim_window_lines < 1) sim_window_lines = 1;
 		sim_window_killalltraces();
+		sim_window_topline = 0;
 	}
 
+	/* determine the number of visible signals */
+	sim_window_vislines = sim_window_lines;
+	if (sim_window_lines > 20) sim_window_vislines = 20;
+
 	/* create waveform window if requested */
 	if (charhandlerwave != 0)
 	{
@@ -371,15 +388,7 @@ void sim_window_redraw(void)
 	/* get the waveform window */
 	wavewin = sim_findwaveformwindow();
 	if (wavewin == NOWINDOWPART) return;
-
-	sim_window_setcolor(ALLOFF);
-	sim_window_setmask(LAYERA);
-	sim_window_drawbox(wavewin, 0, 0, 767, 600);
-	sim_window_drawtimescale(wavewin);
-	sim_window_drawgraph(wavewin);
-	sim_window_drawcursors(wavewin);
-	if (sim_window_highlightedtrace != NODISCHANNEL)
-		sim_window_highlighttrace(wavewin, sim_window_highlightedtrace, HIGHLIT);
+	sim_window_redisphandler(wavewin);
 }
 
 /*
@@ -399,6 +408,40 @@ INTSML sim_window_getlines(void)
 	return(sim_window_lines);
 }
 
+/*
+ * routine that changes the number of visible trace lines in the simulation window to "count"
+ */
+void sim_window_setvislines(INTSML count)
+{
+	if (count <= 0) count = 1;
+	sim_window_vislines = count;
+}
+
+/*
+ * routine that returns the number of visible signals in the simulation window
+ */
+INTSML sim_window_getvislines(void)
+{
+	return(sim_window_vislines);
+}
+
+/*
+ * routine that changes the top visible trace line in the simulation window to "count"
+ */
+void sim_window_settopline(INTSML top)
+{
+	if (top < 0) top = 0;
+	sim_window_topline = top;
+}
+
+/*
+ * routine that returns the top visible signal in the simulation window.
+ */
+INTSML sim_window_gettopline(void)
+{
+	return(sim_window_topline);
+}
+
 /********************************* CONTROL OF TRACES *********************************/
 
 /*
@@ -1042,9 +1085,10 @@ void sim_window_drawtracedrag(WINDOWPART *wavewin, INTSML on)
 	INTSML y_pos, temp_trace_spacing, line, lx, hx, ly, hy;
 
 	/* compute positions */
-	temp_trace_spacing = 540 / sim_window_lines;
-	line = sim_window_tracedragtr->position;
-	y_pos = (INTSML)((sim_window_lines - 1 - line) * temp_trace_spacing +
+	temp_trace_spacing = 540 / sim_window_vislines;
+	line = sim_window_tracedragtr->position - sim_window_topline;
+	if (line < 0 || line >= sim_window_vislines) return;
+	y_pos = (INTSML)((sim_window_vislines - 1 - line) * temp_trace_spacing +
 		31 + sim_window_tracedragyoff);
 	ly = y_pos + sim_window_tracedragtr->nameoff - 1;
 	hy = ly + sim_window_txthei;
@@ -1079,14 +1123,228 @@ INTSML sim_window_eachdown(INTBIG x, INTBIG y)
 	return(0);
 }
 
+/*
+ * Routine called when vertical slider parts are clicked.
+ */
+INTSML sim_window_verticalslider(INTBIG x, INTBIG y)
+{
+	INTSML newtopline;
+
+	if (x >= DISPLAYSLIDERSIZE) return(0);
+	switch (sim_waveformsliderpart)
+	{
+		case 0: /* down arrow clicked */
+			if (y > sim_waveformwindow->usely + DISPLAYSLIDERSIZE) return(0);
+			if (y < sim_waveformwindow->usely) return(0);
+			if (sim_window_topline + sim_window_vislines < sim_window_lines)
+			{
+				sim_window_topline++;
+				sim_window_redisphandler(sim_waveformwindow);
+			}
+			break;
+		case 1: /* below thumb */
+			if (y > sim_waveformwindow->thumbly) return(0);
+			newtopline = sim_window_topline + sim_window_vislines - 1;
+			if (newtopline + sim_window_vislines > sim_window_lines)
+				newtopline = sim_window_lines - sim_window_vislines;
+			if (sim_window_topline == newtopline) break;
+			sim_window_topline = newtopline;
+			sim_window_redisphandler(sim_waveformwindow);
+			break;
+		case 2: /* on thumb (not handled here) */
+			break;
+		case 3: /* above thumb */
+			if (y < sim_waveformwindow->thumbhy) return(0);
+			if (sim_window_topline == 0) return(0);
+			sim_window_topline -= sim_window_vislines - 1;
+			if (sim_window_topline < 0) sim_window_topline = 0;
+			sim_window_redisphandler(sim_waveformwindow);
+			break;
+		case 4: /* up arrow clicked */
+			if (y < sim_waveformwindow->usehy - DISPLAYSLIDERSIZE) return(0);
+			if (sim_window_topline > 0)
+			{
+				sim_window_topline--;
+				sim_window_redisphandler(sim_waveformwindow);
+			}
+			break;
+	}
+	return(0);
+}
+
+/*
+ * Routine called when horizontal slider parts are clicked.
+ */
+INTSML sim_window_horizontalslider(INTBIG x, INTBIG y)
+{
+	float range;
+
+	if (y >= DISPLAYSLIDERSIZE) return(0);
+	switch (sim_waveformsliderpart)
+	{
+		case 0: /* left arrow clicked */
+			if (x > sim_waveformwindow->uselx + DISPLAYSLIDERSIZE) return(0);
+			if (sim_window_mintime <= 0.0) break;
+			range = sim_window_maxtime - sim_window_mintime;
+			sim_window_mintime -= range/10.0f;
+			sim_window_maxtime -= range/10.0f;
+			if (sim_window_mintime < 0.0)
+			{
+				sim_window_maxtime -= sim_window_mintime;
+				sim_window_mintime = 0.0;
+			}
+			sim_window_redisphandler(sim_waveformwindow);
+			break;
+		case 1: /* left of thumb */
+			if (x > sim_waveformwindow->thumblx) return(0);
+			if (sim_window_mintime <= 0.0) break;
+			range = sim_window_maxtime - sim_window_mintime;
+			sim_window_mintime -= range/2.0f;
+			sim_window_maxtime -= range/2.0f;
+			if (sim_window_mintime < 0.0)
+			{
+				sim_window_maxtime -= sim_window_mintime;
+				sim_window_mintime = 0.0;
+			}
+			sim_window_redisphandler(sim_waveformwindow);
+			break;
+		case 2: /* on thumb (not handled here) */
+			break;
+		case 3: /* right of thumb */
+			if (x < sim_waveformwindow->thumbhx) return(0);
+			range = sim_window_maxtime - sim_window_mintime;
+			sim_window_mintime += range/2.0f;
+			sim_window_maxtime += range/2.0f;
+			sim_window_redisphandler(sim_waveformwindow);
+			break;
+		case 4: /* right arrow clicked */
+			if (x < sim_waveformwindow->usehx - DISPLAYSLIDERSIZE) return(0);
+			range = sim_window_maxtime - sim_window_mintime;
+			sim_window_mintime += range/10.0f;
+			sim_window_maxtime += range/10.0f;
+			sim_window_redisphandler(sim_waveformwindow);
+			break;
+	}
+	return(0);
+}
+
 void sim_window_buttonhandler(WINDOWPART *w, INTSML button, INTSML inx, INTSML iny)
 {
-	INTBIG x, y, xpos, ypos;
+	INTBIG x, y, xpos, ypos, newtop;
+	float newpos;
 	INTSML ind, temp_trace_spacing, tracelinesmoved;
 	char *par[2];
 	extern AIDENTRY *net_aid;
 	REGISTER DISCHANNEL *tr, *otr, *trprev, *otrprev;
 
+	ttynewcommand();
+
+	/* hit in slider on left */
+	if (inx < DISPLAYSLIDERSIZE)
+	{
+		if (iny <= w->usely + DISPLAYSLIDERSIZE)
+		{
+			/* hit in bottom arrow: go down 1 (may repeat) */
+			sim_waveformwindow = w;
+			sim_waveformsliderpart = 0;
+			trackcursor(0, us_nullup, us_nullvoid, sim_window_verticalslider, us_nullchar,
+				us_nullvoid, TRACKNORMAL);
+			return;
+		}
+		if (iny < w->thumbly)
+		{
+			/* hit below thumb: go down lots (may repeat) */
+			sim_waveformwindow = w;
+			sim_waveformsliderpart = 1;
+			trackcursor(0, us_nullup, us_nullvoid, sim_window_verticalslider, us_nullchar,
+				us_nullvoid, TRACKNORMAL);
+			return;
+		}
+		if (iny <= w->thumbhy)
+		{
+			/* hit on thumb: track thumb motion */
+			us_vthumbbegin(iny, w, w->uselx-DISPLAYSLIDERSIZE, w->usely, w->usehy,
+				sim_window_lines, 1);
+			trackcursor(0, us_nullup, us_nullvoid, us_vthumbdown, us_nullchar,
+				us_svthumbdone, TRACKNORMAL); 
+			newtop = us_svthumbend();
+			if (newtop >= 0)
+			{
+				sim_window_topline = (INTSML)newtop;
+				sim_window_redisphandler(w);
+			}
+			return;
+		}
+		if (iny < w->usehy - DISPLAYSLIDERSIZE)
+		{
+			/* hit above thumb: go up lots (may repeat) */
+			sim_waveformwindow = w;
+			sim_waveformsliderpart = 3;
+			trackcursor(0, us_nullup, us_nullvoid, sim_window_verticalslider, us_nullchar,
+				us_nullvoid, TRACKNORMAL);
+			return;
+		}
+
+		/* hit in up arrow: go up 1 (may repeat) */
+		sim_waveformwindow = w;
+		sim_waveformsliderpart = 4;
+		trackcursor(0, us_nullup, us_nullvoid, sim_window_verticalslider, us_nullchar,
+			us_nullvoid, TRACKNORMAL);
+		return;
+	}
+
+	/* hit in slider on bottom */
+	if (iny < DISPLAYSLIDERSIZE)
+	{
+		if (inx <= w->uselx + DISPLAYSLIDERSIZE)
+		{
+			/* hit in left arrow: small shift down in time (may repeat) */
+			sim_waveformwindow = w;
+			sim_waveformsliderpart = 0;
+			trackcursor(0, us_nullup, us_nullvoid, sim_window_horizontalslider, us_nullchar,
+				us_nullvoid, TRACKNORMAL);
+			return;
+		}
+		if (inx < w->thumblx)
+		{
+			/* hit to left of thumb: large shift down in time (may repeat) */
+			sim_waveformwindow = w;
+			sim_waveformsliderpart = 1;
+			trackcursor(0, us_nullup, us_nullvoid, sim_window_horizontalslider, us_nullchar,
+				us_nullvoid, TRACKNORMAL);
+			return;
+		}
+		if (inx <= w->thumbhx)
+		{
+			/* hit on thumb: track thumb motion */
+			us_hthumbbegin(inx, w, w->usely, w->uselx, w->usehx);
+			trackcursor(0, us_nullup, us_nullvoid, us_hthumbdown, us_nullchar,
+				us_shthumbdone, TRACKNORMAL); 
+			newpos = us_shthumbend();
+			if (newpos == sim_window_mintime) return;
+			sim_window_maxtime += newpos - sim_window_mintime;
+			sim_window_mintime = newpos;
+			sim_window_redisphandler(w);
+			return;
+		}
+		if (inx < w->usehx - DISPLAYSLIDERSIZE)
+		{
+			/* hit to right of thumb: large shift up in time (may repeat) */
+			sim_waveformwindow = w;
+			sim_waveformsliderpart = 3;
+			trackcursor(0, us_nullup, us_nullvoid, sim_window_horizontalslider, us_nullchar,
+				us_nullvoid, TRACKNORMAL);
+			return;
+		}
+
+		/* hit in right arrow: small shift up in time (may repeat) */
+		sim_waveformwindow = w;
+		sim_waveformsliderpart = 4;
+		trackcursor(0, us_nullup, us_nullvoid, sim_window_horizontalslider, us_nullchar,
+			us_nullvoid, TRACKNORMAL);
+		return;
+	}
+
 	sim_window_setviewport(w);
 	x = inx;   y = iny;
 	sim_window_scaletowindow(w, &x, &y);
@@ -1095,14 +1353,14 @@ void sim_window_buttonhandler(WINDOWPART *w, INTSML button, INTSML inx, INTSML i
 	if (x < 114)
 	{
 		sim_window_sethighlighttrace(0);
-		temp_trace_spacing = 540 / sim_window_lines;
-		ind = sim_window_lines - 1 - (y - 31) / temp_trace_spacing;
+		temp_trace_spacing = 540 / sim_window_vislines;
+		ind = sim_window_vislines - 1 - (y - 31) / temp_trace_spacing;
 		trprev = NODISCHANNEL;
 		for(tr = sim_window_plot; tr != NODISCHANNEL; tr = tr->nextdischannel)
 		{
-			if (tr->position == ind)
+			if (tr->position == ind + sim_window_topline)
 			{
-				ypos = (sim_window_lines - 1 - ind) * temp_trace_spacing + 31 + tr->nameoff;
+				ypos = (sim_window_vislines - 1 - ind) * temp_trace_spacing + 31 + tr->nameoff;
 				if (y >= ypos && y <= ypos + sim_window_txthei) break;
 			}
 			trprev = tr;
@@ -1197,37 +1455,89 @@ void sim_window_buttonhandler(WINDOWPART *w, INTSML button, INTSML inx, INTSML i
 void sim_window_redisphandler(WINDOWPART *w)
 {
 	INTSML lx, hx, ly, hy;
+	float range;
+	REGISTER INTBIG thumbtop, thumbarea, thumbsize;
 
 	sim_window_setviewport(w);
 
 	/* clear the window */
 	sim_window_desc.bits = LAYERA;
 	sim_window_desc.col = ALLOFF;
+	w->uselx -= DISPLAYSLIDERSIZE;
+	w->usely -= DISPLAYSLIDERSIZE;
 	lx = w->uselx;   hx = w->usehx;
 	ly = w->usely;   hy = w->usehy;
-	us_drawbox(w, lx, hx, ly, hy, &sim_window_desc);
+	screendrawbox(w, lx, hx, ly, hy, &sim_window_desc);
 
 	/* draw red outline */
 	if ((w->state&WINDOWSIMULATING) != 0)
 	{
 		sim_window_desc.col = RED;
-		us_drawbox(w, (INTSML)(lx-SIMULATINGBORDERSIZE), (INTSML)(hx+SIMULATINGBORDERSIZE), (INTSML)(ly-SIMULATINGBORDERSIZE), ly,
-			&sim_window_desc);
-		us_drawbox(w, (INTSML)(lx-SIMULATINGBORDERSIZE), (INTSML)(hx+SIMULATINGBORDERSIZE), hy, (INTSML)(hy+SIMULATINGBORDERSIZE),
-			&sim_window_desc);
-		us_drawbox(w, (INTSML)(lx-SIMULATINGBORDERSIZE), lx, ly, hy, &sim_window_desc);
-		us_drawbox(w, hx, (INTSML)(hx+SIMULATINGBORDERSIZE), ly, hy, &sim_window_desc);
+		screendrawbox(w, (INTSML)(lx-SIMULATINGBORDERSIZE), (INTSML)(hx+SIMULATINGBORDERSIZE),
+			(INTSML)(ly-SIMULATINGBORDERSIZE), (INTSML)(ly-1), &sim_window_desc);
+		screendrawbox(w, (INTSML)(lx-SIMULATINGBORDERSIZE), (INTSML)(hx+SIMULATINGBORDERSIZE),
+			(INTSML)(hy+1), (INTSML)(hy+SIMULATINGBORDERSIZE), &sim_window_desc);
+		screendrawbox(w, (INTSML)(lx-SIMULATINGBORDERSIZE), (INTSML)(lx-1), ly, hy, &sim_window_desc);
+		screendrawbox(w, (INTSML)(hx+1), (INTSML)(hx+SIMULATINGBORDERSIZE), ly, hy, &sim_window_desc);
 	}
 
+	/* draw vertical slider on the left */
+	us_drawverticalslider(w, (INTSML)(lx-2), (INTSML)(ly+DISPLAYSLIDERSIZE), hy, 1);
+	thumbsize = sim_window_vislines * 100 / sim_window_lines;
+	if (thumbsize >= 100) thumbsize = 100;
+	if (thumbsize == 100 && sim_window_topline == 0)
+	{
+		w->thumbly = 0;   w->thumbhy = -1;
+	} else
+	{
+		thumbtop = sim_window_topline * 100 / sim_window_lines;
+		thumbarea = w->usehy - w->usely - DISPLAYSLIDERSIZE*2;
+
+		w->thumbhy = w->usehy - DISPLAYSLIDERSIZE - thumbarea * thumbtop / 100;
+		w->thumbly = w->thumbhy - thumbarea * thumbsize / 100;
+		if (w->thumbhy > w->usehy-DISPLAYSLIDERSIZE-2) w->thumbhy = w->usehy-DISPLAYSLIDERSIZE-2;
+		if (w->thumbly < w->usely+DISPLAYSLIDERSIZE*2+2) w->thumbly = w->usely+DISPLAYSLIDERSIZE*2+2;
+	}
+	us_drawverticalsliderthumb(w, lx-2, ly+DISPLAYSLIDERSIZE, hy, w->thumbly, w->thumbhy);
+
+	/* draw horizontal slider on the bottom */
+	us_drawhorizontalslider(w, (INTSML)(ly+DISPLAYSLIDERSIZE), (INTSML)(lx+DISPLAYSLIDERSIZE), hx);
+	thumbarea = w->usehx - w->uselx - DISPLAYSLIDERSIZE*2;
+	if (sim_window_mintime <= 0.0)
+	{
+		w->thumblx = w->uselx+DISPLAYSLIDERSIZE*2+2;
+		w->thumbhx = w->thumblx + thumbarea/2;
+	} else
+	{
+		range = sim_window_maxtime * 2.0f;
+		w->thumblx = (INTSML)(w->uselx+DISPLAYSLIDERSIZE*2+2 + (INTBIG)(sim_window_mintime/range*thumbarea));
+		w->thumbhx = (INTSML)(w->uselx+DISPLAYSLIDERSIZE*2+2 + (INTBIG)(sim_window_maxtime/range*thumbarea));
+	}
+	if (w->thumbhx > w->usehx-DISPLAYSLIDERSIZE-2) w->thumbhx = w->usehx-DISPLAYSLIDERSIZE-2;
+	if (w->thumblx < w->uselx+DISPLAYSLIDERSIZE*2+2) w->thumblx = w->uselx+DISPLAYSLIDERSIZE*2+2;
+	us_drawhorizontalsliderthumb(w, ly+DISPLAYSLIDERSIZE, lx+DISPLAYSLIDERSIZE, hx, w->thumblx, w->thumbhx);
+
+	us_drawslidercorner(w, lx, lx+DISPLAYSLIDERSIZE-1, ly, ly+DISPLAYSLIDERSIZE-1);
+
+	/* reset bounds */
+	w->uselx += DISPLAYSLIDERSIZE;
+	w->usely += DISPLAYSLIDERSIZE;
+
 	/* write banner at top of window */
-	us_settextsize(w, TXT10P);
+	screensettextsize(w, TXT10P);
 	sim_window_desc.col = el_colmentxt;
 	(void)initinfstr();
 	if ((w->state&WINDOWSIMULATING) == 0) (void)addstringtoinfstr("Inactive ");
 	(void)addstringtoinfstr("Simulation Window, ? for help");
-	us_puttext(w, (INTSML)(lx+5), (INTSML)(hy-18), returninfstr(), &sim_window_desc);
-	us_invertbox(w, lx, hx, (INTSML)(hy-20), hy);
-	sim_window_redraw();
+	lx = w->uselx;
+	screendrawtext(w, (INTSML)(lx+5), (INTSML)(hy-18), returninfstr(), &sim_window_desc);
+	screeninvertbox(w, lx, hx, (INTSML)(hy-20), hy);
+
+	sim_window_drawtimescale(w);
+	sim_window_drawgraph(w);
+	sim_window_drawcursors(w);
+	if (sim_window_highlightedtrace != NODISCHANNEL)
+		sim_window_highlighttrace(w, sim_window_highlightedtrace, HIGHLIT);
 }
 
 /****************************** SUPPORT FUNCTIONS ******************************/
@@ -1281,9 +1591,10 @@ void sim_window_writetracename(WINDOWPART *wavewin, DISCHANNEL *tr)
 	INTBIG maxheight, maxwidth, len, save;
 
 	/* compute positions */
-	temp_trace_spacing = 540 / sim_window_lines;
-	line = tr->position;
-	y_pos = (sim_window_lines - 1 - line) * temp_trace_spacing + 31;
+	temp_trace_spacing = 540 / sim_window_vislines;
+	line = tr->position - sim_window_topline;
+	if (line < 0 || line >= sim_window_vislines) return;
+	y_pos = (sim_window_vislines - 1 - line) * temp_trace_spacing + 31;
 	ly = y_pos + tr->nameoff-1;   hy = ly + temp_trace_spacing;
 	lx = 0;                       hx = 112;
 
@@ -1304,8 +1615,8 @@ void sim_window_writetracename(WINDOWPART *wavewin, DISCHANNEL *tr)
 	len = strlen(tr->name);
 	for(size = sim_window_textsize; size >= TXT4P; size--)
 	{
-		us_settextsize(wavewin, size);
-		us_textsize(wavewin, tr->name, &wid, &hei);
+		screensettextsize(wavewin, size);
+		screengettextsize(wavewin, tr->name, &wid, &hei);
 		if (hei > maxheight && size > TXT4P) continue;
 		sim_window_txthei = hei * (wavewin->screenhy - wavewin->screenly) /
 			(wavewin->usehy - wavewin->usely - 20);
@@ -1313,11 +1624,11 @@ void sim_window_writetracename(WINDOWPART *wavewin, DISCHANNEL *tr)
 		{
 			len--;
 			save = tr->name[len];   tr->name[len] = 0;
-			us_textsize(wavewin, tr->name, &wid, &hei);
+			screengettextsize(wavewin, tr->name, &wid, &hei);
 			tr->name[len] = (char)save;
 		}
 		save = tr->name[len];   tr->name[len] = 0;
-		us_puttext(wavewin, px, py, tr->name, &sim_window_desc);
+		screendrawtext(wavewin, px, py, tr->name, &sim_window_desc);
 		tr->name[len] = (char)save;
 		break;
 	}
@@ -1331,9 +1642,10 @@ void sim_window_highlighttrace(WINDOWPART *wavewin, DISCHANNEL *tr, INTSML color
 	float time, position;
 
 	/* compute positions */
-	temp_trace_spacing = 540 / sim_window_lines;
-	line = tr->position;
-	y_pos = (sim_window_lines - 1 - line) * temp_trace_spacing + 31;
+	temp_trace_spacing = 540 / sim_window_vislines;
+	line = tr->position - sim_window_topline;
+	if (line < 0 || line >= sim_window_vislines) return;
+	y_pos = (sim_window_vislines - 1 - line) * temp_trace_spacing + 31;
 	ly = y_pos + tr->nameoff-1;   hy = ly + sim_window_txthei;
 	lx = 0;                       hx = 112;
 
@@ -1345,8 +1657,8 @@ void sim_window_highlighttrace(WINDOWPART *wavewin, DISCHANNEL *tr, INTSML color
 	sim_window_drawto(wavewin, lx, hy);
 	sim_window_drawto(wavewin, lx, ly);
 
-	slot = tr->position;
-	y_min = (sim_window_lines - 1 - slot) * temp_trace_spacing + 31;
+	slot = tr->position - sim_window_topline;
+	y_min = (sim_window_vislines - 1 - slot) * temp_trace_spacing + 31;
 	y_max = y_min + temp_trace_spacing - 10;
 
 	first = 1;
@@ -1504,14 +1816,14 @@ void sim_window_drawgraph(WINDOWPART *wavewin)
 	float time, position;
 
 	/* determine height of text in the window */
-	us_settextsize(wavewin, sim_window_textsize);
-	us_textsize(wavewin, "X", &x, &y);
+	screensettextsize(wavewin, sim_window_textsize);
+	screengettextsize(wavewin, "X", &x, &y);
 	sim_window_txthei = y * (wavewin->screenhy - wavewin->screenly) /
 		(wavewin->usehy - wavewin->usely - 20);
 
 	/* set name offsets */
 	for(tr = sim_window_plot; tr != NODISCHANNEL; tr = tr->nextdischannel) tr->nameoff = -1;
-	temp_trace_spacing = 540 / sim_window_lines;
+	temp_trace_spacing = 540 / sim_window_vislines;
 	for(tr = sim_window_plot; tr != NODISCHANNEL; tr = tr->nextdischannel)
 	{
 		/* determine position of trace label on the line */
@@ -1547,7 +1859,7 @@ void sim_window_drawgraph(WINDOWPART *wavewin)
 			/* if there are analog traces on this line, show height values */
 			if (hasana != 0)
 			{
-				y_min = (sim_window_lines - 1 - tr->position) * temp_trace_spacing + 31;
+				y_min = (sim_window_vislines - 1 - (tr->position - sim_window_topline)) * temp_trace_spacing + 31;
 				y_max = y_min + temp_trace_spacing - 10;
 				sim_window_setmask(LAYERA);
 				sim_window_setcolor(MENTXT);
@@ -1584,9 +1896,10 @@ void sim_window_drawgraph(WINDOWPART *wavewin)
 	{
 		if (stopping("Simulation Redraw") != 0) break;
 
-		slot = tr->position;
+		slot = tr->position - sim_window_topline;
+		if (slot < 0 || slot >= sim_window_vislines) continue;
 		sim_window_writetracename(wavewin, tr);
-		y_min = (sim_window_lines - 1 - slot) * temp_trace_spacing + 31;
+		y_min = (sim_window_vislines - 1 - slot) * temp_trace_spacing + 31;
 		y_max = y_min + temp_trace_spacing - 10;
 
 		first = 1;
@@ -1721,8 +2034,8 @@ void sim_window_savegraph(void)
 	if (wavewin == NOWINDOWPART) return;
 
 	/* determine height of text in the window */
-	us_settextsize(wavewin, sim_window_textsize);
-	us_textsize(wavewin, "X", &x, &y);
+	screensettextsize(wavewin, sim_window_textsize);
+	screengettextsize(wavewin, "X", &x, &y);
 	sim_window_txthei = y * (wavewin->screenhy - wavewin->screenly) /
 		(wavewin->usehy - wavewin->usely - 20);
 
@@ -1763,7 +2076,7 @@ void sim_window_savegraph(void)
 
 	/* set name offsets */
 	for(tr = sim_window_plot; tr != NODISCHANNEL; tr = tr->nextdischannel) tr->nameoff = -1;
-	temp_trace_spacing = 540 / sim_window_lines;
+	temp_trace_spacing = 540 / sim_window_vislines;
 	for(tr = sim_window_plot; tr != NODISCHANNEL; tr = tr->nextdischannel)
 	{
 		/* determine position of trace label on the line */
@@ -1799,7 +2112,7 @@ void sim_window_savegraph(void)
 			/* if there are analog traces on this line, show height values */
 			if (hasana != 0)
 			{
-				y_min = (sim_window_lines - 1 - tr->position) * temp_trace_spacing + 31;
+				y_min = (sim_window_vislines - 1 - (tr->position - sim_window_topline)) * temp_trace_spacing + 31;
 				y_max = y_min + temp_trace_spacing - 10;
 
 				tx1 = 101;   ty1 = y_min;   sim_window_mapcoord(wavewin, &tx1, &ty1);
@@ -1869,8 +2182,9 @@ void sim_window_savegraph(void)
 		if (stopping("Simulation Redraw") != 0) break;
 
 		/* compute positions */
-		slot = tr->position;
-		y_min = (sim_window_lines - 1 - slot) * temp_trace_spacing + 31;
+		slot = tr->position - sim_window_topline;
+		if (slot < 0 || slot >= sim_window_vislines) continue;
+		y_min = (sim_window_vislines - 1 - slot) * temp_trace_spacing + 31;
 
 		/* draw the vertical line between the label and the trace */
 		tx1 = 113;   ty1 = y_min;   sim_window_mapcoord(wavewin, &tx1, &ty1);
@@ -1904,7 +2218,7 @@ void sim_window_savegraph(void)
 			(void)setvalkey((INTBIG)ni, VNODEINST, art_colorkey, tr->status, VINTEGER);
 		}
 
-		y_min = (sim_window_lines - 1 - slot) * temp_trace_spacing + 31;
+		y_min = (sim_window_vislines - 1 - slot) * temp_trace_spacing + 31;
 		y_max = y_min + temp_trace_spacing - 10;
 
 		first = 1;
@@ -2227,7 +2541,7 @@ void sim_window_drawto(WINDOWPART *w, INTSML x, INTSML y)
 	{
 		fx = (INTSML)cfx;   fy = (INTSML)cfy;
 		tx = (INTSML)ctx;   ty = (INTSML)cty;
-		us_drawline(w, fx, fy, tx, ty, &sim_window_desc, 0);
+		screendrawline(w, fx, fy, tx, ty, &sim_window_desc, 0);
 	}
 	sim_window_curx = x;
 	sim_window_cury = y;
@@ -2243,7 +2557,7 @@ void sim_window_drawbox(WINDOWPART *w, INTSML xll, INTSML yll, INTSML xur, INTSM
 	sim_window_mapcoord(w, &hx, &hy);
 	if (lx > hx) { swap = lx;   lx = hx;   hx = swap; }
 	if (ly > hy) { swap = ly;   ly = hy;   hy = swap; }
-	us_drawbox(w, lx, hx, ly, hy, &sim_window_desc);
+	screendrawbox(w, lx, hx, ly, hy, &sim_window_desc);
 }
 
 void sim_window_drawcstring(WINDOWPART *w, char *s)
@@ -2252,53 +2566,30 @@ void sim_window_drawcstring(WINDOWPART *w, char *s)
 
 	px = sim_window_curx;   py = sim_window_cury;
 	sim_window_mapcoord(w, &px, &py);
-	us_settextsize(w, sim_window_textsize);
-	us_textsize(w, s, &x, &y);
-	us_puttext(w, (INTSML)(px-x/2), py, s, &sim_window_desc);
+	screensettextsize(w, sim_window_textsize);
+	screengettextsize(w, s, &x, &y);
+	screendrawtext(w, (INTSML)(px-x/2), py, s, &sim_window_desc);
 }
 
 void sim_window_drawlstring(WINDOWPART *w, char *s, INTSML maxx)
 {
-#if 0
-	INTSML x, y, save, hch, ux, uy, px, py;
-
-	ux = maxx;
-	uy = sim_window_cury;
-	sim_window_mapcoord(w, &ux, &uy);
-	px = sim_window_curx;   py = sim_window_cury;
-	sim_window_mapcoord(w, &px, &py);
-
-	us_settextsize(w, sim_window_textsize);
-	for(hch = strlen(s); hch > 0; hch--)
-	{
-		save = s[hch];
-		s[hch] = 0;
-		us_textsize(w, s, &x, &y);
-		if (x <= ux - px) break;
-		s[hch] = (char)save;
-	}
-
-	us_puttext(w, px, py, s, &sim_window_desc);
-	s[hch] = (char)save;
-#else
 	INTSML px, py;
 
 	px = sim_window_curx;   py = sim_window_cury;
 	sim_window_mapcoord(w, &px, &py);
-	us_settextsize(w, sim_window_textsize);
-	us_puttext(w, px, py, s, &sim_window_desc);
-#endif
+	screensettextsize(w, sim_window_textsize);
+	screendrawtext(w, px, py, s, &sim_window_desc);
 }
 
 void sim_window_drawrstring(WINDOWPART *w, char *s)
 {
 	INTSML x, y, px, py;
 
-	us_settextsize(w, sim_window_textsize);
-	us_textsize(w, s, &x, &y);
+	screensettextsize(w, sim_window_textsize);
+	screengettextsize(w, s, &x, &y);
 	px = sim_window_curx;   py = sim_window_cury;
 	sim_window_mapcoord(w, &px, &py);
-	us_puttext(w, (INTSML)(px-x), py, s, &sim_window_desc);
+	screendrawtext(w, (INTSML)(px-x), py, s, &sim_window_desc);
 }
 
 /*
