@@ -117,14 +117,6 @@ extern TECH_VARIABLES gen_variables[];
   INTSML mocmos2_initprocess(TECHNOLOGY*, INTSML);
 #endif
 
-#if TECNCMOS3
-# include "tecncmos3.h"
-  extern GRAPHICS *ncmos3_layers[];
-  extern TECH_ARCS *ncmos3_arcprotos[];
-  extern TECH_NODES *ncmos3_nodeprotos[];
-  extern TECH_VARIABLES ncmos3_variables[];
-#endif
-
 #if TECNMOS
   extern GRAPHICS *nmos_layers[];
   extern TECH_ARCS *nmos_arcprotos[];
@@ -264,18 +256,6 @@ TECHNOLOGY el_technologylist[] =
 	NOTECHNOLOGY, NONSTANDARD|NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},				/* miscellaneous */
 #endif
 
-#if TECNCMOS3
-	/* Northern Telecom CMOS */
-	{"ncmos3", 0, 2000, NONODEPROTO,NOARCPROTO,NOVARIABLE,0,NOCOMCOMP,NOCLUSTER,	/* info */
-	"Complementary MOS (from Northern Telecom, P-Well, 3 micron, double metal)",	/* description */
-	0, ncmos3_layers, 0, ncmos3_arcprotos, 0, ncmos3_nodeprotos, ncmos3_variables,	/* tables */
-	ncmos3_initprocess, 0, 0, 0, 0,													/* control routines */
-	ncmos3_nodepolys, ncmos3_nodeEpolys, ncmos3_shapenodepoly, ncmos3_shapeEnodepoly, 0,	/* node routines */
-	0,																				/* port routine */
-	0, 0, 0,																		/* arc routines */
-	NOTECHNOLOGY, NONSTANDARD|NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},				/* miscellaneous */
-#endif
-
 #if TECCMOSDODN
 	/* DOD CMOS (from Gary Lambert) */
 	{"cmosdodn", 0, 2000, NONODEPROTO,NOARCPROTO,NOVARIABLE,0,NOCOMCOMP,NOCLUSTER,	/* info */
