@@ -81,9 +81,8 @@ INTBIG       us_quickkeys;				/* cached key for "USER_quick_keys" */
 INTBIG       us_interactiveangle;		/* cached key for "USER_interactive_angle" */
 INTBIG       us_tinylambdaperpixel;		/* cached key for "USER_tiny_lambda_per_pixel" */
 INTBIG       us_techstate;				/* cached key for "TECH_state" */
-INTBIG       us_techlayer3dheightkey;	/* cached key for "TECH_layer_3dheight" */
-INTBIG       us_techlayer3dthicknesskey;/* cached key for "TECH_layer_3dthickness" */
 INTBIG       us_ignoreoptionchanges;	/* cached key for "USER_ignore_option_changes" */
+INTBIG       us_displayunits;			/* cached key for "USER_display_units" */
 INTBIG       us_useroptions;			/* mirror for "USER_optionflags" */
 INTBIG       us_tinyratio;				/* mirror for "USER_tiny_lambda_per_pixel" */
 INTBIG       us_separatechar;			/* separating character in port names */
@@ -280,9 +279,8 @@ INTSML us_do2init(INTBIG *argc, char *argv[])
 	us_tinylambdaperpixel = makekey("USER_tiny_lambda_per_pixel");
 	us_techstate = makekey("TECH_state");
 	us_optionflags = makekey("USER_optionflags");
-	us_techlayer3dheightkey = makekey("TECH_layer_3dheight");
-	us_techlayer3dthicknesskey = makekey("TECH_layer_3dthickness");
 	us_ignoreoptionchanges = makekey("USER_ignore_option_changes");
+	us_displayunits = makekey("USER_display_units");
 
 	/* initially ignore all graphics */
 	us_displayroutine = us_nulldisplayroutine;
@@ -302,16 +300,16 @@ INTSML us_do2init(INTBIG *argc, char *argv[])
 	us_firstlibrary = 0;
 	us_desiredlibrary[0] = 0;
 	us_firstmacrofile = 0;
-	us_setdisplay("display", argc, argv);
+	graphicsoptions("display", argc, argv);
 	if (us_options(argc, argv) != 0) return(1);
 
 	/* initialize status output and graphics */
-	if (us_initgraphics(1)) exit(1);
+	if (initgraphics(1)) exit(1);
 	us_getcolormap(el_curtech, COLORSDEFAULT, 0);
 	varred = getvalkey((INTBIG)us_aid, VAID, VINTEGER|VISARRAY, us_colormap_red);
 	vargreen = getvalkey((INTBIG)us_aid, VAID, VINTEGER|VISARRAY, us_colormap_green);
 	varblue = getvalkey((INTBIG)us_aid, VAID, VINTEGER|VISARRAY, us_colormap_blue);
-	us_loadmap((INTBIG *)varred->addr, (INTBIG *)vargreen->addr,
+	colormapload((INTBIG *)varred->addr, (INTBIG *)vargreen->addr,
 		(INTBIG *)varblue->addr, 0, 255);
 
 	/* establish the colors to use for peripheral graphics */
@@ -351,11 +349,11 @@ INTSML us_do2init(INTBIG *argc, char *argv[])
 	us_initstatus();
 
 	/* create the window frame that has the palette */
-	us_menuframe = us_newwindowframe(1, 0);
+	us_menuframe = newwindowframe(1, 0);
 
 	/* create the initial window structure */
-	wf = us_newwindowframe(0, 0);
-	if (wf == NOWINDOWFRAME) wf = us_getwindowframe(0);
+	wf = newwindowframe(0, 0);
+	if (wf == NOWINDOWFRAME) wf = getwindowframe(0);
 	el_curwindowpart = newwindowpart("entire", NOWINDOWPART);
 	if (el_curwindowpart == NOWINDOWPART) return(1);
 	el_curwindowpart->frame = wf;
@@ -376,20 +374,23 @@ INTSML us_do2init(INTBIG *argc, char *argv[])
 	nextvarchangequiet();
 	(void)setvalkey((INTBIG)us_aid, VAID, us_alignment_edge,
 		us_edgealignment, VINTEGER|VDONTSAVE);
+	nextvarchangequiet();
+	(void)setvalkey((INTBIG)us_aid, VAID, us_displayunits,
+		el_units&DISPLAYUNITS, VINTEGER|VDONTSAVE);
 
 	/* if the screen is seriously wide, put menus on the left */
-	us_getwindowsize(el_curwindowpart->frame, &swid, &shei);
+	getwindowframesize(el_curwindowpart->frame, &swid, &shei);
 	if (swid*2 > shei*3) us_menupos = 2;
 
 	/* display initial message */
-	us_settextsize(el_curwindowpart, TXT20P);
+	screensettextsize(el_curwindowpart, TXT20P);
 	us_box.col = BLACK;
 	sprintf(msg, "Electric Version %s", el_version);
-	us_textsize(el_curwindowpart, msg, &twid, &thei);
-	us_puttext(el_curwindowpart, (INTSML)((swid-twid)/2), (INTSML)(shei/2+thei), msg, &us_box);
+	screengettextsize(el_curwindowpart, msg, &twid, &thei);
+	screendrawtext(el_curwindowpart, (INTSML)((swid-twid)/2), (INTSML)(shei/2+thei), msg, &us_box);
 	strcpy(msg, "Loading...");
-	us_textsize(el_curwindowpart, msg, &twid, &thei);
-	us_puttext(el_curwindowpart, (INTSML)((swid-twid)/2), (INTSML)(shei/2-thei), msg, &us_box);
+	screengettextsize(el_curwindowpart, msg, &twid, &thei);
+	screendrawtext(el_curwindowpart, (INTSML)((swid-twid)/2), (INTSML)(shei/2-thei), msg, &us_box);
 	flushscreen();
 	return(0);
 }
@@ -400,7 +401,7 @@ INTSML us_do3init(void)
 	REGISTER char *libname, *libfile;
 	REGISTER INTBIG oldverbose, len;
 	REGISTER VARIABLE *var, *varred, *vargreen, *varblue;
-	char *newpar[3], *hd, clipboardname[20];
+	char *newpar[3], clipboardname[20];
 	REGISTER LIBRARY *lib;
 	extern AIDENTRY *io_aid;
 
@@ -411,34 +412,26 @@ INTSML us_do3init(void)
 	us_redostatus(NOWINDOWFRAME);
 
 	/* set application environment */
-#if EGNU
+#if LANGLISP
 	nextvarchangequiet();
-	(void)setvalkey((INTBIG)us_aid, VAID, makekey("USER_application"),
-		(INTBIG)"EGNU", VSTRING|VDONTSAVE);
+	(void)setvalkey((INTBIG)us_aid, VAID, makekey("USER_have_lisp"), 1,
+		VINTEGER|VDONTSAVE);
 #endif
-#if EVE
+#if LANGTCL
 	nextvarchangequiet();
-	(void)setvalkey((INTBIG)us_aid, VAID, makekey("USER_application"),
-		(INTBIG)"EVE", VSTRING|VDONTSAVE);
+	(void)setvalkey((INTBIG)us_aid, VAID, makekey("USER_have_tcl"), 1,
+		VINTEGER|VDONTSAVE);
 #endif
-#if ELM
+#if LANGJAVA
 	nextvarchangequiet();
-	(void)setvalkey((INTBIG)us_aid, VAID, makekey("USER_application"),
-		(INTBIG)"ELM", VSTRING|VDONTSAVE);
-#endif
-#if ELF
-	nextvarchangequiet();
-	(void)setvalkey((INTBIG)us_aid, VAID, makekey("USER_application"),
-		(INTBIG)"ELF", VSTRING|VDONTSAVE);
+	(void)setvalkey((INTBIG)us_aid, VAID, makekey("USER_have_java"), 1,
+		VINTEGER|VDONTSAVE);
 #endif
 
-#if LANGLISP | LANGTCL
+#if IOSKILL
 	nextvarchangequiet();
-	(void)setvalkey((INTBIG)us_aid, VAID, makekey("USER_have_languages"), 1, VINTEGER|VDONTSAVE);
-#endif
-#if IODEF | IOLEF | IOSKILL
-	nextvarchangequiet();
-	(void)setvalkey((INTBIG)us_aid, VAID, makekey("USER_have_cadence"), 1, VINTEGER|VDONTSAVE);
+	(void)setvalkey((INTBIG)us_aid, VAID, makekey("USER_have_cadence"), 1,
+		VINTEGER|VDONTSAVE);
 #endif
 
 	/* read "cadrc" files */
@@ -460,12 +453,7 @@ INTSML us_do3init(void)
 
 	/* read the options library */
 	libname = us_tempoptionslibraryname();
-	(void)initinfstr();
-	hd = hashomedir();
-	if (hd == 0) (void)addstringtoinfstr(el_libdir); else
-		(void)addstringtoinfstr(hd);
-	(void)addstringtoinfstr(OPTIONSFILE);
-	libfile = returninfstr();
+	libfile = optionsfilepath();
 	if (fileexistence(truepath(libfile)) == 1)
 	{
 		lib = newlibrary(libname, libfile);
@@ -516,7 +504,7 @@ INTSML us_do3init(void)
 	varred = getvalkey((INTBIG)us_aid, VAID, VINTEGER|VISARRAY, us_colormap_red);
 	vargreen = getvalkey((INTBIG)us_aid, VAID, VINTEGER|VISARRAY, us_colormap_green);
 	varblue = getvalkey((INTBIG)us_aid, VAID, VINTEGER|VISARRAY, us_colormap_blue);
-	us_loadmap((INTBIG *)varred->addr, (INTBIG *)vargreen->addr,
+	colormapload((INTBIG *)varred->addr, (INTBIG *)vargreen->addr,
 		(INTBIG *)varblue->addr, 0, 255);
 
 	/* setup current window */
@@ -547,14 +535,14 @@ INTSML us_do3init(void)
 	/* now draw everything */
 	us_drawmenu(1, NOWINDOWFRAME);	/* draw basic screen layout */
 
-#if !defined(MACOS)
+#ifndef MACOS
 	if ((us_aid->aidstate&USEDIALOGS) != 0)
 	{
 		(void)ttydataready();
 		us_aboutdlog("Protected by License");
 		(void)ttydataready();
 	} else
-		ttyputmsg("Electric is Licensed from Static Free Software");
+		ttyputmsg("Electric, from Static Free Software");
 #endif
 
 	/* if there is a current facet in the library, edit it */
@@ -573,7 +561,7 @@ INTSML us_do3init(void)
 	us_optionschanged = 0;				/* options are unchanged */
 
 	/* start session logging if not turned off */
-	if (us_logging != 0) us_logstartrecord();
+	if (us_logging != 0) logstartrecord();
 	return(0);
 }
 
@@ -584,7 +572,7 @@ void us_done(void)
 
 	/* delete all windows */
 	if (us_menuframe != NOWINDOWFRAME)
-		us_killwindowframe(us_menuframe);
+		killwindowframe(us_menuframe);
 	while (el_topwindowpart != NOWINDOWPART)
 	{
 		w = el_topwindowpart;
@@ -593,8 +581,8 @@ void us_done(void)
 	}
 
 	us_ttyclose();
-	us_logfinishrecord();
-	us_termgraphics();
+	logfinishrecord();
+	termgraphics();
 
 #ifdef DEBUGMEMORY
 	/* free popups */
@@ -1128,12 +1116,12 @@ void us_slice(void)
 
 		setactivity("Language Interpreter");
 
-		/* set all facet center changes detected during the last broadcast */
+		/* set all facet message changes detected during the last broadcast */
 		us_doubchanges();
 		return;
 	}
 
-	/* set all facet center changes detected during the last broadcast */
+	/* set all facet message changes detected during the last broadcast */
 	us_doubchanges();
 
 	/* remove menu highlighting if not temporary */
@@ -1153,9 +1141,7 @@ void us_slice(void)
 	if (el_pleasestop != 0) return;
 
 	/* get a button */
-	el_pleasestop = 0;
 	waitforbutton(&x, &y, &but);
-
 	if (el_pleasestop != 0) return;
 	if (but < 0) return;
 
@@ -1172,6 +1158,7 @@ void us_ontablet(INTSML x, INTSML y, INTSML but)
 	REGISTER char *str;
 	POPUPMENU *popup, *cpopup;
 	INTSML butstate, swid, shei;
+	INTBIG lx, hx, ly, hy;
 	REGISTER INTSML e, mindex;
 	REGISTER VARIABLE *var;
 	REGISTER NODEPROTO *np;
@@ -1190,20 +1177,8 @@ void us_ontablet(INTSML x, INTSML y, INTSML but)
 	/* first see if this is a slide to a window partition divider */
 	for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
 	{
-		REGISTER INTBIG lx, hx, ly, hy;
 		/* see if the cursor is over a window partition separator */
-		lx = w->uselx;   hx = w->usehx;
-		ly = w->usely;   hy = w->usehy;
-		if ((w->state&WINDOWSIMULATING) != 0)
-		{
-			lx -= SIMULATINGBORDERSIZE;   hx += SIMULATINGBORDERSIZE;
-			ly -= SIMULATINGBORDERSIZE;   hy += SIMULATINGBORDERSIZE;
-		}
-		if ((w->state&WINDOWTYPE) == DISPWINDOW)
-		{
-			hx += DISPLAYSLIDERSIZE;
-			ly -= DISPLAYSLIDERSIZE;
-		}
+		us_gettruewindowbounds(w, &lx, &hx, &ly, &hy);
 		if (x >= lx-1 && x <= lx+1 && y > ly+1 && y < hy-1 &&
 			us_hasotherwindowpart(lx-10, y, w) != 0)
 		{
@@ -1236,7 +1211,7 @@ void us_ontablet(INTSML x, INTSML y, INTSML but)
 	}
 
 	/* first switch windows to the current frame */
-	curframe = us_getwindowframe(1);
+	curframe = getwindowframe(1);
 	for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
 	{
 		if (w->frame != curframe) continue;
@@ -1255,7 +1230,7 @@ void us_ontablet(INTSML x, INTSML y, INTSML but)
 	}
 
 	/* see if it is a pulldown menu hit */
-	us_getwindowsize(curframe, &swid, &shei);
+	getwindowframesize(curframe, &swid, &shei);
 	if (us_menubarsize != 0 && y > shei-us_menubarsize)
 	{
 		for(;;)
@@ -1270,7 +1245,7 @@ void us_ontablet(INTSML x, INTSML y, INTSML but)
 			popup = us_pulldowns[e];
 
 			/* highlight the pulldown title */
-			us_invertbox(el_curwindowpart, mindex, us_pulldownmenupos[e], (INTSML)(shei-us_menubarsize+1), shei);
+			screeninvertbox(el_curwindowpart, mindex, us_pulldownmenupos[e], (INTSML)(shei-us_menubarsize+1), shei);
 
 			/* initialize the popup menu for pulldown use */
 			us_initpulldownmenu(popup);
@@ -1284,7 +1259,7 @@ void us_ontablet(INTSML x, INTSML y, INTSML but)
 			us_termpulldownmenu(popup);
 
 			/* unhighlight the pulldown title */
-			us_invertbox(el_curwindowpart, mindex, us_pulldownmenupos[e], (INTSML)(shei-us_menubarsize+1), shei);
+			screeninvertbox(el_curwindowpart, mindex, us_pulldownmenupos[e], (INTSML)(shei-us_menubarsize+1), shei);
 
 			/* if another menu is desired, do it */
 			if (butstate != 0)
@@ -1341,103 +1316,147 @@ void us_ontablet(INTSML x, INTSML y, INTSML but)
 		if (us_setxy(x, y))
 		{
 			/* see if it is a slider hit in a display window */
-			wf = us_getwindowframe(0);
+			wf = getwindowframe(0);
 			for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
 			{
 				if (w->frame != wf) continue;
-				if ((w->state&WINDOWTYPE) != DISPWINDOW) continue;
-				if (x > w->usehx && x <= w->usehx+DISPLAYSLIDERSIZE &&
-					y > w->usely && y < w->usehy)
+				if ((w->state&WINDOWTYPE) == WAVEFORMWINDOW)
 				{
-					/* check for vertical slider hit */
-					np = w->curnodeproto;
-					if (w != el_curwindowpart)
+					if (x >= w->uselx-DISPLAYSLIDERSIZE && x < w->usehx &&
+						y >= w->usely-DISPLAYSLIDERSIZE && y < w->usehy)
 					{
-						(void)setvalkey((INTBIG)us_aid, VAID, us_current_window, (INTBIG)w,
-							VWINDOWPART|VDONTSAVE);
-						(void)setval((INTBIG)el_curlib, VLIBRARY, "curnodeproto",
-							(INTBIG)np, VNODEPROTO);
-					}
-					if (np == NONODEPROTO) return;
-					if (np->highy == np->lowy) return;
-					if (y <= w->usely+DISPLAYSLIDERSIZE)
-					{
-						/* small shift up */
-						us_slideup((w->screenhy - w->screenly) / 10);
-						return;
-					}
-					if (y <= w->thumbly)
-					{
-						/* large shift up */
-						us_slideup((w->screenhy - w->screenly) / 2);
-						return;
-					}
-					if (y <= w->thumbhy)
-					{
-						/* on thumb */
-						us_vthumbbegin(y, w);
-						trackcursor(0, us_nullup, us_nullvoid, us_vthumbdown, us_nullchar,
-							us_vthumbdone, TRACKNORMAL);
-						return;
-					}
-					if (y <= w->usehy-DISPLAYSLIDERSIZE)
-					{
-						/* large shift down */
-						us_slideup(-(w->screenhy - w->screenly) / 2);
-						return;
-					}
-					if (y <= w->usehy)
-					{
-						/* small shift down */
-						us_slideup(-(w->screenhy - w->screenly) / 10);
+						if (w->buttonhandler != 0)
+							(*w->buttonhandler)(w, but, x, y);
 						return;
 					}
 				}
-				if (y < w->usely && y >= w->usely-DISPLAYSLIDERSIZE &&
-					x > w->uselx && x < w->usehx)
+				if ((w->state&WINDOWTYPE) == DISPWINDOW)
 				{
-					/* check for horizontal slider hit */
-					np = w->curnodeproto;
-					if (w != el_curwindowpart)
+					if (x > w->usehx && x <= w->usehx+DISPLAYSLIDERSIZE &&
+						y > w->usely && y < w->usehy)
 					{
-						(void)setvalkey((INTBIG)us_aid, VAID, us_current_window, (INTBIG)w,
-							VWINDOWPART|VDONTSAVE);
-						(void)setval((INTBIG)el_curlib, VLIBRARY, "curnodeproto",
-							(INTBIG)np, VNODEPROTO);
+						/* check for vertical slider hit */
+						np = w->curnodeproto;
+						if (w != el_curwindowpart)
+						{
+							(void)setvalkey((INTBIG)us_aid, VAID, us_current_window, (INTBIG)w,
+								VWINDOWPART|VDONTSAVE);
+							(void)setval((INTBIG)el_curlib, VLIBRARY, "curnodeproto",
+								(INTBIG)np, VNODEPROTO);
+						}
+						if (np == NONODEPROTO) return;
+						if (np->highy == np->lowy) return;
+						if (y <= w->usely+DISPLAYSLIDERSIZE)
+						{
+							/* small shift up (may repeat) */
+							us_arrowclickbegin(w, w->usehx, w->usehx+DISPLAYSLIDERSIZE,
+								w->usely, w->usely+DISPLAYSLIDERSIZE,
+								(w->screenhy - w->screenly) / 10);
+							trackcursor(0, us_nullup, us_nullvoid, us_varrowdown, us_nullchar,
+								us_nullvoid, TRACKNORMAL);
+							return;
+						}
+						if (y <= w->thumbly)
+						{
+							/* large shift up (may repeat) */
+							us_arrowclickbegin(w, w->usehx, w->usehx+DISPLAYSLIDERSIZE,
+								w->usely+DISPLAYSLIDERSIZE, w->thumbhy,
+								(w->screenhy - w->screenly) / 2);
+							trackcursor(0, us_nullup, us_nullvoid, us_varrowdown, us_nullchar,
+								us_nullvoid, TRACKNORMAL);
+							return;
+						}
+						if (y <= w->thumbhy)
+						{
+							/* on thumb (track thumb motion) */
+							us_vthumbbegin(y, w, w->usehx, w->usely, w->usehy, 0, 0);
+							trackcursor(0, us_nullup, us_nullvoid, us_vthumbdown, us_nullchar,
+								us_vthumbdone, TRACKNORMAL);
+							return;
+						}
+						if (y <= w->usehy-DISPLAYSLIDERSIZE)
+						{
+							/* large shift down (may repeat) */
+							us_arrowclickbegin(w, w->usehx, w->usehx+DISPLAYSLIDERSIZE,
+								w->thumbhy, w->usehy-DISPLAYSLIDERSIZE,
+								-(w->screenhy - w->screenly) / 2);
+							trackcursor(0, us_nullup, us_nullvoid, us_varrowdown, us_nullchar,
+								us_nullvoid, TRACKNORMAL);
+							return;
+						}
+						if (y <= w->usehy)
+						{
+							/* small shift down (may repeat) */
+							us_arrowclickbegin(w, w->usehx, w->usehx+DISPLAYSLIDERSIZE,
+								w->usehy-DISPLAYSLIDERSIZE, w->usehy,
+								-(w->screenhy - w->screenly) / 10);
+							trackcursor(0, us_nullup, us_nullvoid, us_varrowdown, us_nullchar,
+								us_nullvoid, TRACKNORMAL);
+							return;
+						}
 					}
-					if (np == NONODEPROTO) return;
-					if (np->highx == np->lowx) return;
-					if (x <= w->uselx+DISPLAYSLIDERSIZE)
+					if (y < w->usely && y >= w->usely-DISPLAYSLIDERSIZE &&
+						x > w->uselx && x < w->usehx)
 					{
-						/* small shift left */
-						us_slideleft(-(w->screenhx - w->screenlx) / 10);
-						return;
-					}
-					if (x <= w->thumblx)
-					{
-						/* large shift left */
-						us_slideleft(-(w->screenhx - w->screenlx) / 2);
-						return;
-					}
-					if (x <= w->thumbhx)
-					{
-						/* on thumb */
-						us_hthumbbegin(x, w);
-						trackcursor(0, us_nullup, us_nullvoid, us_hthumbdown, us_nullchar,
-							us_hthumbdone, TRACKNORMAL);
-						return;
-					}
-					if (x <= w->usehx-DISPLAYSLIDERSIZE)
-					{
-						/* large shift right */
-						us_slideleft((w->screenhx - w->screenlx) / 2);
-						return;
-					}
-					if (x <= w->usehx)
-					{
-						/* small shift right */
-						us_slideleft((w->screenhx - w->screenlx) / 10);
-						return;
+						/* check for horizontal slider hit */
+						np = w->curnodeproto;
+						if (w != el_curwindowpart)
+						{
+							(void)setvalkey((INTBIG)us_aid, VAID, us_current_window, (INTBIG)w,
+								VWINDOWPART|VDONTSAVE);
+							(void)setval((INTBIG)el_curlib, VLIBRARY, "curnodeproto",
+								(INTBIG)np, VNODEPROTO);
+						}
+						if (np == NONODEPROTO) return;
+						if (np->highx == np->lowx) return;
+						if (x <= w->uselx+DISPLAYSLIDERSIZE)
+						{
+							/* small shift left (may repeat) */
+							us_arrowclickbegin(w, w->uselx, w->uselx+DISPLAYSLIDERSIZE,
+								w->usely-DISPLAYSLIDERSIZE, w->usely,
+								-(w->screenhx - w->screenlx) / 10);
+							trackcursor(0, us_nullup, us_nullvoid, us_harrowdown, us_nullchar,
+								us_nullvoid, TRACKNORMAL);
+							return;
+						}
+						if (x <= w->thumblx)
+						{
+							/* large shift left (may repeat) */
+							us_arrowclickbegin(w, w->uselx+DISPLAYSLIDERSIZE, w->thumblx,
+								w->usely-DISPLAYSLIDERSIZE, w->usely,
+								-(w->screenhx - w->screenlx) / 2);
+							trackcursor(0, us_nullup, us_nullvoid, us_harrowdown, us_nullchar,
+								us_nullvoid, TRACKNORMAL);
+							return;
+						}
+						if (x <= w->thumbhx)
+						{
+							/* on thumb (track thumb motion) */
+							us_hthumbbegin(x, w, w->usely, w->uselx, w->usehx);
+							trackcursor(0, us_nullup, us_nullvoid, us_hthumbdown, us_nullchar,
+								us_hthumbdone, TRACKNORMAL);
+							return;
+						}
+						if (x <= w->usehx-DISPLAYSLIDERSIZE)
+						{
+							/* large shift right (may repeat) */
+							us_arrowclickbegin(w, w->thumbhx, w->usehx-DISPLAYSLIDERSIZE,
+								w->usely-DISPLAYSLIDERSIZE, w->usely,
+								(w->screenhx - w->screenlx) / 2);
+							trackcursor(0, us_nullup, us_nullvoid, us_harrowdown, us_nullchar,
+								us_nullvoid, TRACKNORMAL);
+							return;
+						}
+						if (x <= w->usehx)
+						{
+							/* small shift right (may repeat) */
+							us_arrowclickbegin(w, w->usehx-DISPLAYSLIDERSIZE, w->usehx,
+								w->usely-DISPLAYSLIDERSIZE, w->usely,
+								(w->screenhx - w->screenlx) / 10);
+							trackcursor(0, us_nullup, us_nullvoid, us_harrowdown, us_nullchar,
+								us_nullvoid, TRACKNORMAL);
+							return;
+						}
 					}
 				}
 			}
@@ -1456,24 +1475,13 @@ void us_ontablet(INTSML x, INTSML y, INTSML but)
 INTSML us_hasotherwindowpart(INTBIG x, INTBIG y, WINDOWPART *ow)
 {
 	REGISTER WINDOWPART *w;
-	REGISTER INTBIG lx, hx, ly, hy;
+	INTBIG lx, hx, ly, hy;
 
 	for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
 	{
 		if (w == ow) continue;
 		if (w->frame != ow->frame) continue;
-		lx = w->uselx;   hx = w->usehx;
-		ly = w->usely;   hy = w->usehy;
-		if ((w->state&WINDOWSIMULATING) != 0)
-		{
-			lx -= SIMULATINGBORDERSIZE;   hx += SIMULATINGBORDERSIZE;
-			ly -= SIMULATINGBORDERSIZE;   hy += SIMULATINGBORDERSIZE;
-		}
-		if ((w->state&WINDOWTYPE) == DISPWINDOW)
-		{
-			hx += DISPLAYSLIDERSIZE;
-			ly -= DISPLAYSLIDERSIZE;
-		}
+		us_gettruewindowbounds(w, &lx, &hx, &ly, &hy);
 		if (x > lx && x < hx && y > ly && y < hy) return(1);
 	}
 	return(0);
@@ -1640,18 +1648,22 @@ void us_forceeditchanges(void)
 static WINDOWPART us_oldwindow, *us_firstnewwindow, *us_secondnewwindow, *us_killedwindow;
 static INTSML us_maplow, us_maphigh, us_menuchanged, us_newwindowcount, us_gridfactorschanged;
 static INTBIG us_oldstate;
+static INTSML us_facetstructurechanged;
 
 void us_startbatch(AIDENTRY *source, INTSML undoredo)
 {
 	us_state &= ~HIGHLIGHTSET;
 	us_beginchanges();
 	us_menuchanged = 0;
+	us_facetstructurechanged = 0;
 }
 
 void us_endbatch(void)
 {
 	us_endchanges(NOWINDOWPART);
 	us_showallhighlight();
+	if (us_facetstructurechanged != 0)
+		us_redoexplorerwindow();
 }
 
 /*
@@ -1733,12 +1745,12 @@ void us_endobjectchange(INTBIG addr, INTBIG type)
 				us_menuchanged = 0;
 				if (us_menuframe != NOWINDOWFRAME && (us_aid->aidstate&MENUON) == 0)
 				{
-					us_killwindowframe(us_menuframe);
+					killwindowframe(us_menuframe);
 					us_menuframe = NOWINDOWFRAME;
 					return;
 				}
 				if (us_menuframe == NOWINDOWFRAME && (us_aid->aidstate&MENUON) != 0)
-					us_menuframe = us_newwindowframe(1, 0);
+					us_menuframe = newwindowframe(1, 0);
 				us_drawmenu(1, us_menuframe);
 				return;
 			}
@@ -1750,7 +1762,7 @@ void us_endobjectchange(INTBIG addr, INTBIG type)
 				vargreen = getvalkey((INTBIG)us_aid, VAID, VINTEGER|VISARRAY, us_colormap_green);
 				varblue = getvalkey((INTBIG)us_aid, VAID, VINTEGER|VISARRAY, us_colormap_blue);
 				if (varred != NOVARIABLE && vargreen != NOVARIABLE && varblue != NOVARIABLE)
-					us_loadmap(&((INTBIG *)varred->addr)[us_maplow],
+					colormapload(&((INTBIG *)varred->addr)[us_maplow],
 						&((INTBIG *)vargreen->addr)[us_maplow], &((INTBIG *)varblue->addr)[us_maplow],
 							us_maplow, us_maphigh);
 			}
@@ -1785,12 +1797,12 @@ void us_endobjectchange(INTBIG addr, INTBIG type)
 						{
 							w = w2;   w2 = w3;   w3 = w;
 						}
-						us_movebox(w2, (INTSML)((w1->usehx+w1->uselx - w3->usehx+w3->uselx) / 2),
+						screenmovebox(w2, (INTSML)((w1->usehx+w1->uselx - w3->usehx+w3->uselx) / 2),
 							(INTSML)((w1->usehy+w1->usely - w3->usehy+w3->usely) / 2),
 								(INTSML)(w3->usehx-w3->uselx+1), (INTSML)(w3->usehy-w3->usely+1),
 									w3->uselx, w3->usely);
 						if (w2->curnodeproto != NONODEPROTO && w2->curnodeproto == w3->curnodeproto)
-							us_movebox(w2, w3->uselx, w3->usely, (INTSML)(w2->usehx-w2->uselx+1),
+							screenmovebox(w2, w3->uselx, w3->usely, (INTSML)(w2->usehx-w2->uselx+1),
 								(INTSML)(w2->usehy-w2->usely+1), w2->uselx, w2->usely);
 						us_drawwindow(w2, el_colwinbor);
 						us_drawwindow(w3, el_colwinbor);
@@ -1828,7 +1840,7 @@ void us_endobjectchange(INTBIG addr, INTBIG type)
 			w = (WINDOWPART *)addr;
 			ow = &us_oldwindow;
 
-			/* adjust window if it changed from display window to something else */
+			/* adjust window if changing in or out of being a display window */
 			if ((w->state&WINDOWTYPE) == DISPWINDOW && (ow->state&WINDOWTYPE) != DISPWINDOW)
 			{
 				/* became a display window: shrink the bottom and right edge */
@@ -1844,6 +1856,22 @@ void us_endobjectchange(INTBIG addr, INTBIG type)
 				computewindowscale(w);
 			}
 
+			/* adjust window if changing in or out of being a waveform window */
+			if ((w->state&WINDOWTYPE) == WAVEFORMWINDOW && (ow->state&WINDOWTYPE) != WAVEFORMWINDOW)
+			{
+				/* became a waveform window: shrink the left edge */
+				w->uselx += DISPLAYSLIDERSIZE;
+				w->usely += DISPLAYSLIDERSIZE;
+				computewindowscale(w);
+			}
+			if ((w->state&WINDOWTYPE) != WAVEFORMWINDOW && (ow->state&WINDOWTYPE) == WAVEFORMWINDOW)
+			{
+				/* no longer a waveform window: shrink the bottom and right edge */
+				w->uselx -= DISPLAYSLIDERSIZE;
+				w->usely -= DISPLAYSLIDERSIZE;
+				computewindowscale(w);
+			}
+
 			/* switch between 2-D and 3-D drawing */
 			if ((ow->state&WINDOWTYPE) == DISPWINDOW && (w->state&WINDOWTYPE) == DISP3DWINDOW)
 			{
@@ -2042,6 +2070,9 @@ void us_newobject(INTBIG addr, INTBIG type)
 			/* look for the "facet-center" node in the "generic" technology */
 			if (ni->proto == gen_facetcenterprim)
 				us_setnodeprotocenter(ni->lowx, ni->lowy, ni->parent);
+
+			/* if a new instance was created, then the facet structure has changed */
+			if (ni->proto->primindex == 0) us_facetstructurechanged = 1;
 			break;
 
 		case VARCINST:
@@ -2078,6 +2109,9 @@ void us_newobject(INTBIG addr, INTBIG type)
 				us_setfacetname(w);
 			}
 
+			/* the facet structure has changed */
+			us_facetstructurechanged = 1;
+
 			onp = us_curnodeproto;
 			if (onp == NONODEPROTO) return;
 			if (onp->primindex != 0) return;
@@ -2096,6 +2130,11 @@ void us_newobject(INTBIG addr, INTBIG type)
 					us_queueredraw(ni->geom, 0);
 			break;
 
+		case VLIBRARY:
+			/* the facet structure has changed */
+			us_facetstructurechanged = 1;
+			break;
+
 		case VWINDOWPART:
 			w = (WINDOWPART *)addr;
 			us_newwindowcount++;
@@ -2125,6 +2164,9 @@ void us_killobject(INTBIG addr, INTBIG type)
 			/* stop tracking duplicated node if it is deleted */
 			if (ni == us_dupnode) us_dupnode = NONODEINST;
 
+			/* if an instance was deleted, then the facet structure has changed */
+			if (ni->proto->primindex == 0) us_facetstructurechanged = 1;
+
 			/* look for the "facet-center" node in the "generic" technology */
 			if (ni->proto == gen_facetcenterprim)
 				us_delnodeprotocenter(ni->parent);
@@ -2170,6 +2212,9 @@ void us_killobject(INTBIG addr, INTBIG type)
 				}
 			}
 			us_removeubchange(np);
+
+			/* the facet structure has changed */
+			us_facetstructurechanged = 1;
 			break;
 
 		case VPORTPROTO:
@@ -2184,6 +2229,11 @@ void us_killobject(INTBIG addr, INTBIG type)
 			}
 			break;
 
+		case VLIBRARY:
+			/* the facet structure has changed */
+			us_facetstructurechanged = 1;
+			break;
+
 		case VWINDOWPART:
 			w = (WINDOWPART *)addr;
 			copywindowpart(&us_oldwindow, w);
@@ -2217,6 +2267,7 @@ void us_newvariable(INTBIG addr, INTBIG type, INTBIG key, INTBIG newtype)
 			for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
 				if (w->curnodeproto != NONODEPROTO && w->curnodeproto->cell == c)
 					us_setfacetname(w);
+			us_facetstructurechanged = 1;
 			return;
 		}
 
@@ -2246,6 +2297,7 @@ void us_newvariable(INTBIG addr, INTBIG type, INTBIG key, INTBIG newtype)
 				if (w->curnodeproto == NONODEPROTO) continue;
 				if (w->curnodeproto->cell->lib == lib) us_setfacetname(w);
 			}
+			if (namesame(name, "libname") == 0) us_facetstructurechanged = 1;
 			return;
 		}
 		return;
@@ -2283,6 +2335,14 @@ void us_newvariable(INTBIG addr, INTBIG type, INTBIG key, INTBIG newtype)
 			}
 		}
 
+		if (key == us_displayunits)
+		{
+			var = getvalkey(addr, type, VINTEGER, key);
+			if (var != NOVARIABLE)
+				el_units = (el_units & ~DISPLAYUNITS) | (var->addr & DISPLAYUNITS);
+			return;
+		}
+
 		/* see if default text editor changed */
 		if (key == us_text_editor)
 		{
@@ -2869,11 +2929,11 @@ void us_readlibrary(LIBRARY *lib)
 		/* only want permanent user-interface variables (ones just read in) */
 		var = &us_aid->firstvar[i];
 		if ((var->type&VDONTSAVE) != 0) continue;
-		var->type |= VDONTSAVE;
 
 		/* handle popup menus */
 		if (namesamen(makename(var->key), "USER_binding_popup_", 19) == 0)
 		{
+			var->type |= VDONTSAVE;
 			name = &makename(var->key)[19];
 			pm = us_getpopupmenu(name);
 			mi = pm->list;
@@ -2901,6 +2961,10 @@ void us_readlibrary(LIBRARY *lib)
 	var = getvalkey((INTBIG)us_aid, VAID, VSTRING|VISARRAY, us_quickkeys);
 	if (var != NOVARIABLE) us_adjustquickkeys(var);
 
+	var = getvalkey((INTBIG)us_aid, VAID, VINTEGER, us_displayunits);
+	if (var != NOVARIABLE)
+		el_units = (el_units & ~DISPLAYUNITS) | (var->addr & DISPLAYUNITS);
+
 	/* recache technology variables */
 	for(tech = el_technologies; tech != NOTECHNOLOGY; tech = tech->nexttechnology)
 	{
@@ -2925,6 +2989,9 @@ void us_readlibrary(LIBRARY *lib)
 		}
 	}
 
+	/* redisplay the explorer window */
+	us_facetstructurechanged = 1;
+
 	/*
 	 * see if the library is old enough to require MOCMOS conversion
 	 * or serpentine port conversion
