@@ -650,7 +650,7 @@ ARCINST *us_directarcinst(NODEINST *fromnodeinst, PORTPROTO *fromppt,
 	{
 		ai = us_runarcinst(fromnodeinst, fromportproto, bestfx, bestfy, 
 			tonodeinst, toportproto, besttx, bestty, typ, wid);
-		if (ai != NOARCINST) ttyputinfo("Created 1 %s arc", typ->protoname);
+		if (ai != NOARCINST) ttyputmsg("Created 1 %s arc", typ->protoname);
 		return(ai);
 	}
 
@@ -839,7 +839,7 @@ ARCINST *us_onebend(NODEINST *fromnodeinst, PORTPROTO *fromppt, NODEINST *tonode
 		ai2 = us_runarcinst(ni, fpt, bestxi, bestyi, tonodeinst, toportproto,
 			besttx, bestty, typ, wid);
 		if (ai2 == NOARCINST) return(NOARCINST);
-		ttyputinfo("Created 2 %s arcs", typ->protoname);
+		ttyputmsg("Created 2 %s arcs", typ->protoname);
 		if (abs(bestfx-bestxi)+abs(bestfy-bestyi) > abs(bestxi-besttx)+abs(bestyi-bestty))
 		{
 			*alt = ai2;
@@ -997,7 +997,7 @@ ARCINST *us_twobend(NODEINST *fromnodeinst, PORTPROTO *fromppt, NODEINST *tonode
 	if (ai == NOARCINST) return(NOARCINST);
 	*alt2 = us_runarcinst(ni2,fpt, xi2,yi2, tonodeinst, toportproto, besttx, bestty, typ, wid);
 	if (*alt2 == NOARCINST) return(NOARCINST);
-	ttyputinfo("Created 3 %s arcs", typ->protoname);
+	ttyputmsg("Created 3 %s arcs", typ->protoname);
 	return(ai);
 }
 
