@@ -177,6 +177,23 @@ void us_checkdatabase(INTSML verbose)
 			warnings++;
 			ttyputmsg("    View %s", v->viewname);
 		}
+		if ((v->viewstate&MULTIPAGEVIEW) != 0)
+		{
+			if (namesamen(v->viewname, "schematic-page-", 15) != 0)
+			{
+				warnings++;
+				ttyputmsg("    View %s should not be multipage", v->viewname);
+				v->viewstate &= ~MULTIPAGEVIEW;
+			}
+		} else
+		{
+			if (namesamen(v->viewname, "schematic-page-", 15) == 0)
+			{
+				warnings++;
+				ttyputmsg("    View %s should be multipage", v->viewname);
+				v->viewstate |= MULTIPAGEVIEW;
+			}
+		}
 	}
 
 	savelib = el_curlib;
@@ -225,7 +242,7 @@ void us_checklibrary(LIBRARY *curlib, INTSML verbose, INTSML *warnings, INTSML *
 	REGISTER CELL *c, *oc;
 	REGISTER NETWORK *net;
 	REGISTER VIEW *v;
-	REGISTER INTSML found, foundtot, foundtrue, facetcenters, neterrors, len;
+	REGISTER INTSML found, foundtot, foundtrue, facetcenters, neterrors, len, accumulatesize;
 	REGISTER INTBIG totlx, tothx, totly, tothy, truelx, truehx, truely, truehy, i, j, *arr;
 	INTBIG lx, hx, ly, hy, position[2];
 	REGISTER char *ch;
@@ -457,10 +474,8 @@ void us_checklibrary(LIBRARY *curlib, INTSML verbose, INTSML *warnings, INTSML *
 					if (namesame(pp->protoname, opp->protoname) != 0) continue;
 					ttyputmsg("Facet %s, port %s: duplicate name",
 						describenodeproto(np), pp->protoname);
-					(void)initinfstr();
-					(void)addstringtoinfstr(pp->protoname);
-					(void)addstringtoinfstr(".0");
-					(void)reallocstring(&pp->protoname, returninfstr(), np->cell->cluster);
+					(void)reallocstring(&pp->protoname, us_uniqueportname(pp->protoname, np),
+						np->cell->cluster);
 					(*warnings)++;
 					break;
 				}
@@ -722,14 +737,6 @@ void us_checklibrary(LIBRARY *curlib, INTSML verbose, INTSML *warnings, INTSML *
 				if (ni->rotation < 0) ni->rotation += 3600;
 			}
 
-			/* check the proto validity */
-			if (ni->proto->primindex == 0 && ni->proto->cell->lib != curlib)
-			{
-				ttyputmsg("Facet %s, node %s (E): wrong library", describenodeproto(np),
-					describenodeinst(ni));
-				(*errors)++;
-			}
-
 			/* check the arc connections on this nodeinst */
 			for(lpi = NOPORTARCINST, pi = ni->firstportarcinst; pi != NOPORTARCINST;
 				pi = pi->nextportarcinst)
@@ -944,7 +951,22 @@ void us_checklibrary(LIBRARY *curlib, INTSML verbose, INTSML *warnings, INTSML *
 			}
 
 			/* accumulate facet size information if not a "facet-center" primitive */
-			if (ni->proto != gen_facetcenterprim)
+			accumulatesize = 1;
+			if (ni->proto == gen_facetcenterprim) accumulatesize = 0;
+			if (ni->proto == gen_invispinprim)
+			{
+				for(i=0; i<ni->numvar; i++)
+				{
+					var = &ni->firstvar[i];
+					if ((var->type&VDISPLAY) != 0 &&
+						(var->textdescript&VTINTERIOR) != 0)
+					{
+						accumulatesize = 0;
+						break;
+					}
+				}
+			}
+			if (accumulatesize != 0)
 			{
 				if (foundtot == 0)
 				{
@@ -1128,42 +1150,6 @@ void us_checklibrary(LIBRARY *curlib, INTSML verbose, INTSML *warnings, INTSML *
 					}
 					(*warnings)++;
 				}
-#if 0
-if (ai->end[i].nodeinst->proto->primindex != 0)
-{
-	NODEINST *ni;
-	PORTPROTO *pp;
-	INTBIG pindex, lambda, lx, hx, ly, hy, rx, ry, total, size, px, py;
-	TECH_PORTS *portdata;
-	TECH_NODES *nodedata;
-	XARRAY trans;
-	extern INTSML db_multstyle;
-
-	ttyputmsg("Facet %s, arc %s, end %ld:", describenodeproto(np), describearcinst(ai), i);
-	ni = ai->end[i].nodeinst;
-	pp = ai->end[i].portarcinst->proto;
-	pindex = ni->proto->primindex;
-	nodedata = ni->proto->tech->nodeprotos[pindex-1];
-	portdata = &nodedata->portlist[0];
-	lambda = ni->proto->tech->deflambda;
-	total = ni->lowx + ni->highx;   size = ni->highx - ni->lowx;
-	lx = total/2 + size*portdata->lowxmul/WHOLE + portdata->lowxsum*lambda/WHOLE;
-	ttyputmsg("  lx = %d/2 + %d x %d / WHOLE + %d x %d / WHOLE = %d", total, size,
-		portdata->lowxmul, portdata->lowxsum, lambda, lx);
-	total = ni->lowy + ni->highy;   size = ni->highy - ni->lowy;
-	ly = total/2 + size*portdata->lowymul/WHOLE + portdata->lowysum*lambda/WHOLE;
-	ttyputmsg("  ly = %d/2 + %d x %d / WHOLE + %d x %d / WHOLE = %d", total, size,
-		portdata->lowymul, portdata->lowysum, lambda, ly);
-	makerot(ni, trans);
-	px = trans[2][0];   py = trans[2][1];
-	rx = mult(trans[0][0],lx) + mult(trans[1][0],ly) + px;
-	ttyputmsg("  lx' = %d x %d + %d x %d + %d = %d", trans[0][0], lx, trans[1][0], ly, px, rx);
-	ry = mult(trans[0][1],lx) + mult(trans[1][1],ly) + py;
-	ttyputmsg("  ly' = %d x %d + %d x %d + %d = %d", trans[0][1], lx, trans[1][1], ly, py, ry);
-	ttyputmsg("  multstyle = %d, end is at (%ld,%ld)", db_multstyle, rx, ry);
-	ttyputmsg("  end %ld is at (%ld,%ld), port at (%d,%d)",i,ai->end[i].xpos,ai->end[i].ypos,poly->xv[0],poly->yv[0]);
-}
-#endif
 
 				/* make sure arcinst connects in portinst area */
 				shapeportpoly(ai->end[i].nodeinst, ai->end[i].portarcinst->proto, poly, 0);
