@@ -364,7 +364,7 @@ void us_array(INTSML count, char *par[])
 	REGISTER INTSML xsize, ysize, flipx, flipy, staggerx, staggery, ro, tr, noname, l,
 		diagonal, first;
 	REGISTER INTBIG x, y, xpos, ypos, nodecount, arccount, i, lx, hx, ly, hy, cx, cy,
-		cx0, cy0, cx1, cy1, xoff0, xoff1, yoff0, yoff1, xoff, yoff;
+		cx0, cy0, cx1, cy1, xoff0, xoff1, yoff0, yoff1, xoff, yoff, keepbits;
 	REGISTER GEOM **list, *geom;
 	INTBIG xoverlap, yoverlap, plx, ply, phx, phy;
 	REGISTER char *pt1, *pt2, *portname;
@@ -641,7 +641,8 @@ void us_array(INTSML count, char *par[])
 				(void)us_pophighlight(0);
 				return;
 			}
-			newno->userbits = ni->userbits;
+			keepbits = NEXPAND | HARDSELECTN;
+			newno->userbits = (newno->userbits & ~keepbits) | (ni->userbits & keepbits);
 			(void)copyvars((INTBIG)ni, VNODEINST, (INTBIG)newno, VNODEINST);
 			if (noname == 0)
 			{
