@@ -879,7 +879,7 @@ void us_create(INTSML count, char *par[])
 	NODEINST *ani[2];
 	REGISTER INTSML ang, l, remainhighlighted, ishor, isver, getcontents,
 		pangle, join, waitfordown, angledcreate;
-	REGISTER INTBIG bestdist, bestdist1, bestdist2, wid, truewid, i, k,
+	REGISTER INTBIG bestdist, bestdist1, bestdist2, wid, truewid, i, j, k,
 		bits1, bits2, dist;
 	INTBIG px, py, nx, ny, cx, cy, x, y, xp[2], yp[2], xcur, ycur, otheralign, pxs, pys;
 	PORTPROTO *app[2];
@@ -906,9 +906,6 @@ void us_create(INTSML count, char *par[])
 	curfacet = us_needfacet();
 	if (curfacet == NONODEPROTO) return;
 
-	/* disallow creation if lock is on */
-	if (us_canedit(curfacet, NONODEPROTO, 1) != 0) return;
-
 	remainhighlighted = getcontents = waitfordown = angledcreate = join = 0;
 	while (count > 0)
 	{
@@ -966,6 +963,8 @@ void us_create(INTSML count, char *par[])
 				return;
 			}
 
+			if (us_canedit(curfacet, NONODEPROTO, 1) != 0) return;
+
 			/* run an arc from "fromgeom" to "togeom" */
 			if (remainhighlighted != 0) us_pushhighlight();
 			us_clearhighlightcount();
@@ -1065,6 +1064,9 @@ void us_create(INTSML count, char *par[])
 			}
 			if ((np->userbits&WANTNEXPAND) != 0) newno->userbits |= NEXPAND;
 			if (np == gen_facetcenterprim) newno->userbits |= HARDSELECTN;
+
+			/* copy "inheritable" attributes */
+			us_inheritattributes(newno);
 			endobjectchange((INTBIG)newno, VNODEINST);
 
 			newhigh.status = HIGHFROM;
@@ -1090,6 +1092,7 @@ void us_create(INTSML count, char *par[])
 			us_abortcommand("Cursor must be in the same facet as the highlighted node");
 			return;
 		}
+		if (us_canedit(curfacet, NONODEPROTO, 1) != 0) return;
 
 		/* find the closest port to the cursor */
 		var = getvalkey((INTBIG)us_aid, VAID, VSTRING|VISARRAY, us_highlighted);
@@ -1174,6 +1177,7 @@ void us_create(INTSML count, char *par[])
 				}
 			}
 		} else ppt = fpt;
+		newhigh.fromport = ppt;
 
 		/* turn off highlighting */
 		us_clearhighlightcount();
@@ -1278,6 +1282,24 @@ void us_create(INTSML count, char *par[])
 						}
 					}
 				}
+				if (fpt != ppt)
+				{
+					/* the closest port cannot connect, but maybe it can with a different arc */
+					for(i=0; fpt->connects[i] != NOARCPROTO; i++)
+					{
+						if (fpt->connects[i]->tech == gen_tech) continue;
+						for(j=0; newhigh.fromport->connects[j] != NOARCPROTO; j++)
+						{
+							if (fpt->connects[i] == newhigh.fromport->connects[j]) break;
+						}
+						if (newhigh.fromport->connects[j] != NOARCPROTO) break;
+					}
+					if (fpt->connects[i] != NOARCPROTO)
+					{
+						us_setarcproto(fpt->connects[i], 0);
+						ppt = fpt;
+					}
+				}
 				if (ppt != NOPORTPROTO) fpt = ppt;
 			}
 			(void)aconnect(newhigh.fromgeom, newhigh.fromport, foundgeom,
@@ -1359,7 +1381,7 @@ void us_create(INTSML count, char *par[])
 		{
 			ai = us_runarcinst(ni, ppt, x, y, newno,
 				newno->proto->firstportproto, nx, ny, us_curarcproto, truewid);
-			if (ai != NOARCINST) ttyputinfo("Created 1 %s arc", us_curarcproto->protoname);
+			if (ai != NOARCINST) ttyputmsg("Created 1 %s arc", us_curarcproto->protoname);
 		} else
 		{
 			k = us_bottomrecurse(ni, ppt, 0) % 3600;
@@ -1633,6 +1655,8 @@ void us_create(INTSML count, char *par[])
 		}
 		ppt = np->firstportproto;
 
+		if (us_canedit(curfacet, np, 1) != 0) return;
+
 		/* turn off highlighting */
 		us_clearhighlightcount();
 		if ((us_state&NONPERSISTENTCURNODE) != 0) us_setnodeproto(np);
@@ -1813,7 +1837,7 @@ void us_debug(INTSML count, char *par[])
 	REGISTER LIBRARY *lib;
 	REGISTER VARIABLE *var;
 	REGISTER INTSML i, j, l, let1, let2, let3;
-	char line[20], *pt, *libname, *libfile, *hd;
+	char line[200], *pt, *libname, *libfile;
 	extern INTSML db_printerrors, db_minrtnodesize, db_maxrtnodesize;
 	extern AIDENTRY *io_aid;
 	HIGHLIGHT newhigh;
@@ -1831,16 +1855,17 @@ void us_debug(INTSML count, char *par[])
 			{
 				for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 				{
-					j = 0;
 					for(pt = pp->protoname; *pt != 0; pt++)
+						if (*pt == '-') break;
+					if (*pt != 0)
 					{
-						if (*pt == '-')
-						{
-							*pt = '_';
-							j = 1;
-						}
+						i++;
+						strcpy(line, pp->protoname);
+						for(pt = line; *pt != 0; pt++)
+							if (*pt == '-') *pt = '_';
+						pt = us_uniqueportname(line, np);
+						(void)reallocstring(&pp->protoname, pt, us_aid->cluster);
 					}
-					if (j != 0) i++;
 				}
 			}
 		}
@@ -2040,30 +2065,6 @@ void us_debug(INTSML count, char *par[])
 		return;
 	}
 
-	if (namesamen(par[0], "highlight-nonmanhattan", l) == 0 && l >= 1)
-	{
-		np = us_needfacet();
-		if (np == NONODEPROTO) return;
-		i = 0;
-		for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
-		{
-			var = getvalkey((INTBIG)ai, VARCINST, VINTEGER, el_arc_radius);
-			if (var == NOVARIABLE && (ai->end[0].xpos == ai->end[1].xpos ||
-				ai->end[0].ypos == ai->end[1].ypos)) continue;
-			if (i == 0) us_clearhighlightcount();
-			newhigh.status = HIGHFROM;
-			newhigh.facet = np;
-			newhigh.fromgeom = ai->geom;
-			newhigh.fromport = NOPORTPROTO;
-			newhigh.frompoint = 0;
-			(void)us_addhighlight(&newhigh);
-			i++;
-		}
-		if (i == 0) ttyputmsg("No nonmanhattan arcs in this facet"); else
-			ttyputmsg("%d arcs are not manhattan in this facet", i);
-		return;
-	}
-
 	if (namesamen(par[0], "label-facet", l) == 0 && l >= 1)
 	{
 		np = us_needfacet();
@@ -2169,12 +2170,7 @@ void us_debug(INTSML count, char *par[])
 	if (namesamen(par[0], "examine-options", l) == 0 && l >= 2)
 	{
 		libname = us_tempoptionslibraryname();
-		(void)initinfstr();
-		hd = hashomedir();
-		if (hd == 0) (void)addstringtoinfstr(el_libdir); else
-			(void)addstringtoinfstr(hd);
-		(void)addstringtoinfstr(OPTIONSFILE);
-		libfile = returninfstr();
+		libfile = optionsfilepath();
 		if (fileexistence(truepath(libfile)) == 1)
 		{
 			lib = newlibrary(libname, libfile);
