@@ -33,6 +33,7 @@
 #include "usrtrack.h"
 #include "usrdiacom.h"
 #include "efunction.h"
+#include "tecart.h"
 #include "tecgen.h"
 #ifdef HAVE_UNISTD_H
 #  include <unistd.h>
@@ -85,7 +86,7 @@ void us_editfacet(INTSML count, char *par[])
 	INTBIG lx, hx, ly, hy;
 	REGISTER NODEPROTO *np, *onp, *np1, *curfacet;
 	REGISTER NODEINST *ni, *stacknodeinst;
-	REGISTER LIBRARY *lib;
+	REGISTER LIBRARY *lib, *olib;
 	REGISTER WINDOWPART *win;
 	REGISTER char *pt;
 	NODEINST *hini;
@@ -197,8 +198,9 @@ void us_editfacet(INTSML count, char *par[])
 	/* check dates of subfacets */
 	if ((us_useroptions&CHECKDATE) != 0)
 	{
-		for(np1 = el_curlib->firstnodeproto; np1 != NONODEPROTO; np1 = np1->nextnodeproto)
-			np1->temp1 = 0;
+		for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+			for(np1 = olib->firstnodeproto; np1 != NONODEPROTO; np1 = np1->nextnodeproto)
+				np1->temp1 = 0;
 		us_check_facet_date(np, np->revisiondate);
 	}
 
@@ -210,7 +212,7 @@ void us_editfacet(INTSML count, char *par[])
 		if (win != NOWINDOWPART)
 		{
 			/* switch to window "win" */
-			us_bringwindowtofront(win->frame);
+			bringwindowtofront(win->frame);
 			us_highlightwindow(win, 0);
 			return;
 		}
@@ -260,32 +262,45 @@ void us_editfacet(INTSML count, char *par[])
 
 void us_erase(INTSML count, char *par[])
 {
-	REGISTER INTSML i, j, otherend;
+	REGISTER INTSML i, j, l, otherend, foundnode;
 	REGISTER NODEINST *ni, *nextni, *ni1, *ni2;
 	REGISTER ARCINST *ai;
 	REGISTER PORTARCINST *pi;
 	REGISTER NODEPROTO *np;
 	HIGHLIGHT high;
+	REGISTER char *pp;
 	REGISTER GEOM **list;
 	REGISTER VARIABLE *highvar;
 
+	if (count > 0)
+	{
+		l = strlen(pp = par[0]);
+		if (namesamen(pp, "clean-up-all", l) == 0 && l > 8)
+		{
+			us_clearhighlightcount();
+			for(np = el_curlib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+				us_cleanupfacet(np, 0);
+			return;
+		}
+		if (namesamen(pp, "clean-up", l) == 0)
+		{
+			np = us_needfacet();
+			if (np == NONODEPROTO) return;
+			us_clearhighlightcount();
+			us_cleanupfacet(np, 1);
+			return;
+		}
+		if (namesamen(pp, "geometry", l) == 0)
+		{
+			np = us_needfacet();
+			if (np == NONODEPROTO) return;
+			us_erasegeometry(np);
+			return;
+		}
+	}
+
 	np = us_needfacet();
 	if (np == NONODEPROTO) return;
-
-	/* disallow erasing if lock is on */
-	if (us_canedit(np, NONODEPROTO, 1) != 0) return;
-
-	if (count > 0 && namesamen(par[0], "clean-up", strlen(par[0])) == 0)
-	{
-		us_cleanupfacet(np);
-		return;
-	}
-	if (count > 0 && namesamen(par[0], "geometry", strlen(par[0])) == 0)
-	{
-		us_erasegeometry(np);
-		return;
-	}
-
 	highvar = getvalkey((INTBIG)us_aid, VAID, VSTRING|VISARRAY, us_highlighted);
 	if (highvar == NOVARIABLE)
 	{
@@ -297,6 +312,9 @@ void us_erase(INTSML count, char *par[])
 	/* special case if a text object is highlighted */
 	if (getlength(highvar) == 1 && (high.status&HIGHTYPE) == HIGHTEXT)
 	{
+		/* disallow erasing if lock is on */
+		if (us_canedit(np, NONODEPROTO, 1) != 0) return;
+
 		/* deleting variable on object */
 		if (high.fromvar != NOVARIABLE)
 		{
@@ -346,13 +364,14 @@ void us_erase(INTSML count, char *par[])
 	}
 
 	/* remove from list if a node is locked */
-	j = 0;
+	j = foundnode = 0;
 	for(i=0; list[i] != NOGEOM; i++)
 	{
 		if (list[i]->entrytype == OBJNODEINST)
 		{
 			ni = list[i]->entryaddr.ni;
-			if (us_canedit(np, ni->proto, 0)) continue;
+			if (us_canedit(np, ni->proto, 1)) continue;
+			foundnode = 1;
 		}
 		list[j++] = list[i];
 	}
@@ -363,6 +382,12 @@ void us_erase(INTSML count, char *par[])
 		return;
 	}
 
+	if (foundnode == 0)
+	{
+		/* disallow erasing if lock is on */
+		if (us_canedit(np, NONODEPROTO, 1) != 0) return;
+	}
+
 	/* look for option to re-connect arcs into an erased node */
 	if (count > 0 && namesamen(par[0], "pass-through", strlen(par[0])) == 0)
 	{
@@ -483,15 +508,16 @@ void us_erase(INTSML count, char *par[])
 
 void us_find(INTSML count, char *par[])
 {
-	REGISTER INTSML findport, i, l, findpoint, findexclusively, findmore, findangle,
+	REGISTER INTSML findport, l, findpoint, findexclusively, findmore, findangle,
 		findwithin, findstill, extrainfo, findnobox, findspecial, size, findconstrained,
 		findeasy, findhard;
-	REGISTER INTBIG total, type, addr;
+	REGISTER INTBIG i, total, type, addr;
 	INTBIG xcur, ycur;
 	XARRAY trans;
 	REGISTER INTBIG areasizex, areasizey, x, y, *newlist;
 	REGISTER NODEINST *ni;
-	REGISTER NODEPROTO *np;
+	REGISTER NODEPROTO *np, *onp;
+	REGISTER LIBRARY *lib;
 	REGISTER GEOM *geom, **glist;
 	REGISTER ARCINST *ai;
 	REGISTER PORTPROTO *pp;
@@ -761,6 +787,7 @@ void us_find(INTSML count, char *par[])
 			}
 			newhigh.status = HIGHFROM;
 			newhigh.fromgeom = ai->geom;
+			newhigh.facet = np;
 			us_setfind(&newhigh, 0, extrainfo, findmore, findnobox);
 			return;
 		}
@@ -963,6 +990,7 @@ void us_find(INTSML count, char *par[])
 			newhigh.status = HIGHFROM;
 			newhigh.fromgeom = pp->subnodeinst->geom;
 			newhigh.fromport = pp->subportproto;
+			newhigh.facet = np;
 			us_setfind(&newhigh, 0, extrainfo, findmore, findnobox);
 			return;
 		}
@@ -1093,10 +1121,99 @@ void us_find(INTSML count, char *par[])
 			newhigh.status = HIGHFROM;
 			newhigh.fromgeom = ni->geom;
 			newhigh.fromport = NOPORTPROTO;
+			newhigh.facet = np;
 			us_setfind(&newhigh, findpoint, extrainfo, findmore, findnobox);
 			return;
 		}
 
+		if (namesamen(pt, "nonmanhattan", l) == 0 && l >= 3)
+		{
+			i = 0;
+			for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+				for(onp = lib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
+					onp->temp1 = 0;
+			for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+			{
+				for(onp = lib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
+				{
+					for(ai = onp->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
+					{
+						if (ai->proto->tech == gen_tech || ai->proto->tech == art_tech)
+							continue;
+						var = getvalkey((INTBIG)ai, VARCINST, VINTEGER, el_arc_radius);
+						if (var != NOVARIABLE || (ai->end[0].xpos != ai->end[1].xpos &&
+							ai->end[0].ypos != ai->end[1].ypos)) onp->temp1++;
+					}
+					for(ni = onp->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
+					{
+						if ((ni->rotation % 900) != 0) onp->temp1++;
+					}
+				}
+			}
+			for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
+			{
+				if (ai->proto->tech == gen_tech || ai->proto->tech == art_tech)
+					continue;
+				var = getvalkey((INTBIG)ai, VARCINST, VINTEGER, el_arc_radius);
+				if (var == NOVARIABLE && (ai->end[0].xpos == ai->end[1].xpos ||
+					ai->end[0].ypos == ai->end[1].ypos)) continue;
+				if (i == 0) us_clearhighlightcount();
+				newhigh.status = HIGHFROM;
+				newhigh.facet = np;
+				newhigh.fromgeom = ai->geom;
+				newhigh.fromport = NOPORTPROTO;
+				newhigh.frompoint = 0;
+				(void)us_addhighlight(&newhigh);
+				i++;
+			}
+			for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
+			{
+				if ((ni->rotation % 900) == 0) continue;
+				if (i == 0) us_clearhighlightcount();
+				newhigh.status = HIGHFROM;
+				newhigh.facet = np;
+				newhigh.fromgeom = ni->geom;
+				newhigh.fromport = NOPORTPROTO;
+				newhigh.frompoint = 0;
+				(void)us_addhighlight(&newhigh);
+				i++;
+			}
+			if (i == 0) ttyputmsg("No nonmanhattan objects in this facet"); else
+				ttyputmsg("%ld objects are not manhattan in this facet", i);
+			for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+			{
+				if ((lib->userbits&HIDDENLIBRARY) != 0) continue;
+				i = 0;
+				for(onp = lib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
+					if (onp != np) i += onp->temp1;
+				if (i == 0) continue;
+				if (lib == el_curlib)
+				{
+					l = 0;
+					(void)initinfstr();
+					for(onp = lib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
+					{
+						if (onp == np || onp->temp1 == 0) continue;
+						(void)addtoinfstr(' ');
+						(void)addstringtoinfstr(describenodeproto(onp));
+						l++;
+					}
+					if (l == 1)
+					{
+						ttyputmsg("Found nonmanhattan geometry in facet%s", returninfstr());
+					} else
+					{
+						ttyputmsg("Found nonmanhattan geometry in these facets:%s",
+							returninfstr());
+					}
+				} else
+				{
+					ttyputmsg("Found nonmanhattan geometry in library %s", lib->libname);
+				}
+			}
+			return;
+		}
+
 		if (namesamen(pt, "object", l) == 0)
 		{
 			if (count < 2)
@@ -1158,6 +1275,7 @@ void us_find(INTSML count, char *par[])
 					newhigh.status = HIGHFROM;
 					newhigh.fromgeom = ni->geom;
 					newhigh.fromport = NOPORTPROTO;
+					newhigh.facet = np;
 					us_setfind(&newhigh, findpoint, extrainfo, findmore, findnobox);
 					break;
 				case VARCINST:
@@ -1171,6 +1289,7 @@ void us_find(INTSML count, char *par[])
 					newhigh.status = HIGHFROM;
 					newhigh.fromgeom = ai->geom;
 					newhigh.fromport = NOPORTPROTO;
+					newhigh.facet = np;
 					us_setfind(&newhigh, findpoint, extrainfo, findmore, findnobox);
 					break;
 				case VNETWORK:
@@ -1187,6 +1306,7 @@ void us_find(INTSML count, char *par[])
 						newhigh.status = HIGHFROM;
 						newhigh.fromgeom = ai->geom;
 						newhigh.fromport = NOPORTPROTO;
+						newhigh.facet = np;
 						us_setfind(&newhigh, findpoint, extrainfo, findmore, findnobox);
 					}
 					for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
@@ -1195,6 +1315,7 @@ void us_find(INTSML count, char *par[])
 						newhigh.status = HIGHFROM;
 						newhigh.fromgeom = pp->subnodeinst->geom;
 						newhigh.fromport = pp->subportproto;
+						newhigh.facet = np;
 						us_setfind(&newhigh, 0, extrainfo, findmore, findnobox);
 					}
 					break;
@@ -1209,6 +1330,7 @@ void us_find(INTSML count, char *par[])
 					newhigh.status = HIGHFROM;
 					newhigh.fromgeom = pp->subnodeinst->geom;
 					newhigh.fromport = pp->subportproto;
+					newhigh.facet = np;
 					us_setfind(&newhigh, 0, extrainfo, findmore, findnobox);
 					break;
 				default:
@@ -1392,8 +1514,9 @@ void us_find(INTSML count, char *par[])
 	/* find the closest object to the cursor */
 	if (findwithin == 0)
 		us_findobject(xcur, ycur, el_curwindowpart, &newhigh, findexclusively, 0, findport, 0, findspecial);
-	if (newhigh.fromgeom == NOGEOM) return;
+	if (newhigh.status == 0) return;
 	if (findport == 0) newhigh.fromport = NOPORTPROTO;
+	newhigh.facet = np;
 	us_setfind(&newhigh, findpoint, extrainfo, findmore, findnobox);
 }
 
@@ -1429,11 +1552,6 @@ void us_getproto(INTSML count, char *par[])
 			us_abortcommand("Cannot find node '%s'", par[1]);
 			return;
 		}
-		if (np->primindex == 0 && np->cell->lib != el_curlib)
-		{
-			us_abortcommand("Use 'copyfacet' for references to other libraries");
-			return;
-		}
 		us_setnodeproto(np);
 		return;
 	}
@@ -1535,11 +1653,6 @@ void us_getproto(INTSML count, char *par[])
 			us_abortcommand("Cannot find node '%s'", pp);
 		return;
 	}
-	if (np->primindex == 0 && np->cell->lib != el_curlib)
-	{
-		us_abortcommand("Use 'copyfacet' for references to other libraries");
-		return;
-	}
 	us_setnodeproto(np);
 }
 
@@ -1658,7 +1771,7 @@ void us_help(INTSML count, char *par[])
 	char *paramstart[MAXPARS];
 	REGISTER char *pp;
 	REGISTER INTSML lastquiet, nday, nmonth, nyear, on;
-	INTSML day, month, year;
+	INTSML day, month, year, hour, minute, second;
 	FILE *in;
 	char line[256], sep[2], *filename, *hd;
 	UINTBIG clock;
@@ -1702,7 +1815,7 @@ void us_help(INTSML count, char *par[])
 		{
 			fclose(in);
 			clock = filedate(pp);
-			parsetime(clock, &year, &month, &day);
+			parsetime(clock, &year, &month, &day, &hour, &minute, &second);
 			month++;
 		}
 
@@ -2091,13 +2204,10 @@ void us_iterate(INTSML count, char *par[])
 
 void us_killfacet(INTSML count, char *par[])
 {
-	REGISTER NODEPROTO *np, *onp, *curfacet;
+	REGISTER NODEPROTO *np, *onp;
 	REGISTER NODEINST *ni;
-	REGISTER VIEW *oldview;
-	REGISTER CELL *oldcell;
-	REGISTER INTSML oldversion, iscurrent;
+	REGISTER LIBRARY *lib;
 	extern COMCOMP us_showdp;
-	REGISTER WINDOWPART *w, *nextw, *neww;
 
 	if (count == 0)
 	{
@@ -2120,11 +2230,6 @@ void us_killfacet(INTSML count, char *par[])
 		us_abortcommand("Can only kill facets");
 		return;
 	}
-	if (np->cell->lib != el_curlib)
-	{
-		us_abortcommand("Can only delete facets in the current library");
-		return;
-	}
 
 	/* disallow killing if lock is on */
 	if (us_canedit(np, NONODEPROTO, 1) != 0) return;
@@ -2132,101 +2237,20 @@ void us_killfacet(INTSML count, char *par[])
 	/* if there are still instances of the facet, mention them */
 	if (np->firstinst != NONODEINST)
 	{
-		for(onp = el_curlib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
-			onp->temp1 = 0;
+		for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+			for(onp = lib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
+				onp->temp1 = 0;
 		for(ni = np->firstinst; ni != NONODEINST; ni = ni->nextinst)
 			ni->parent->temp1++;
 		ttyputerr("Erase all of the following instances of this facet first:");
-		for(onp = el_curlib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
-			if (onp->temp1 != 0)
-				ttyputmsg("  %ld instance(s) in facet %s", onp->temp1, describenodeproto(onp));
+		for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+			for(onp = lib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
+				if (onp->temp1 != 0)
+					ttyputmsg("  %ld instance(s) in facet %s", onp->temp1, describenodeproto(onp));
 		return;
 	}
 
-	/* delete random references to this facet */
-	curfacet = getcurfacet();
-	if (np == curfacet)
-	{
-		(void)setval((INTBIG)el_curlib, VLIBRARY, "curnodeproto", (INTBIG)NONODEPROTO, VNODEPROTO);
-		us_clearhighlightcount();
-	}
-
-	/* close windows that reference this facet */
-	for(w = el_topwindowpart; w != NOWINDOWPART; w = nextw)
-	{
-		nextw = w->nextwindowpart;
-		if (w->curnodeproto != np) continue;
-		if (w == el_curwindowpart) iscurrent = 1; else iscurrent = 0;
-		if (iscurrent != 0)
-			(void)setvalkey((INTBIG)us_aid, VAID, us_current_window, (INTBIG)NOWINDOWPART,
-				VWINDOWPART|VDONTSAVE);
-		startobjectchange((INTBIG)us_aid, VAID);
-		neww = newwindowpart(w->location, w);
-		if (neww == NOWINDOWPART) return;
-
-		/* adjust the new window to account for borders in the old one */
-		if ((w->state&WINDOWTYPE) != DISPWINDOW)
-		{
-			neww->usehx -= DISPLAYSLIDERSIZE;
-			neww->usely += DISPLAYSLIDERSIZE;
-		}
-		if ((w->state&WINDOWSIMULATING) != 0)
-		{
-			neww->uselx -= SIMULATINGBORDERSIZE;   neww->usehx += SIMULATINGBORDERSIZE;
-			neww->usely -= SIMULATINGBORDERSIZE;   neww->usehy += SIMULATINGBORDERSIZE;
-		}
-
-		neww->curnodeproto = NONODEPROTO;
-		neww->buttonhandler = DEFAULTBUTTONHANDLER;
-		neww->charhandler = DEFAULTCHARHANDLER;
-		neww->changehandler = DEFAULTCHANGEHANDLER;
-		neww->termhandler = DEFAULTTERMHANDLER;
-		neww->redisphandler = DEFAULTREDISPHANDLER;
-		neww->state = (neww->state & ~(WINDOWTYPE|WINDOWSIMULATING)) | DISPWINDOW;
-		killwindowpart(w);
-		endobjectchange((INTBIG)us_aid, VAID);
-		if (iscurrent != 0)
-			(void)setvalkey((INTBIG)us_aid, VAID, us_current_window, (INTBIG)neww,
-				VWINDOWPART|VDONTSAVE);
-	}
-
-	oldcell = np->cell;
-	oldview = np->cellview;
-	oldversion = np->version;
-	if (killnodeproto(np))
-	{
-		ttyputerr("Error killing facet");
-		return;
-	}
-
-	/* see if this was the latest version of a cell */
-	for(onp = oldcell->firstincell; onp != NONODEPROTO; onp = onp->nextincell)
-		if (onp->cellview == oldview && onp->version < oldversion) break;
-	if (onp != NONODEPROTO)
-	{
-		/* newest version was deleted: rename next older version */
-		for(ni = onp->firstinst; ni != NONODEINST; ni = ni->nextinst)
-		{
-			if ((ni->userbits&NEXPAND) != 0) continue;
-			startobjectchange((INTBIG)ni, VNODEINST);
-			endobjectchange((INTBIG)ni, VNODEINST);
-		}
-	}
-
-	/* update status display if necessary */
-	if (us_curnodeproto != NONODEPROTO && us_curnodeproto->primindex == 0)
-	{
-		if (np == us_curnodeproto)
-		{
-			if ((us_state&NONPERSISTENTCURNODE) != 0) us_setnodeproto(NONODEPROTO); else
-				us_setnodeproto(el_curtech->firstnodeproto);
-		} else if (np->cell == us_curnodeproto->cell)
-		{
-			onp = us_curnodeproto;
-			us_curnodeproto = NONODEPROTO;
-			us_setnodeproto(onp);
-		}
-	}
-
+	/* kill the facet */
+	us_dokillfacet(np);
 	ttyputmsg("Facet %s deleted", par[0]);
 }
