@@ -44,10 +44,11 @@
 void us_lambda(INTSML count, char *par[])
 {
 	REGISTER char *pp;
-	REGISTER INTBIG lam, oldlam;
+	REGISTER INTBIG lam, oldlam, newunit;
 	REGISTER INTSML l;
 	REGISTER WINDOWPART *w;
 	REGISTER NODEPROTO *np;
+	REGISTER LIBRARY *lib;
 	REGISTER TECHNOLOGY *tech;
 	extern COMCOMP us_lambdachp, us_noyesp;
 
@@ -61,35 +62,19 @@ void us_lambda(INTSML count, char *par[])
 			if (count >= 2)
 			{
 				l = strlen(pp = par[1]);
-				if (namesamen(pp, "lambda", l) == 0)
-				{
-					el_units = (el_units & ~DISPLAYUNITS) | DISPUNITLAMBDA;
-				} else if (namesamen(pp, "inch", l) == 0)
-				{
-					el_units = (el_units & ~DISPLAYUNITS) | DISPUNITINCH;
-				} else if (namesamen(pp, "centimeter", l) == 0 && l >= 7)
-				{
-					el_units = (el_units & ~DISPLAYUNITS) | DISPUNITCM;
-				} else if (namesamen(pp, "millimeter", l) == 0 && l >= 7)
-				{
-					el_units = (el_units & ~DISPLAYUNITS) | DISPUNITMM;
-				} else if (namesamen(pp, "mil", l) == 0 && l >= 3)
-				{
-					el_units = (el_units & ~DISPLAYUNITS) | DISPUNITMIL;
-				} else if (namesamen(pp, "micron", l) == 0 && l >= 3)
-				{
-					el_units = (el_units & ~DISPLAYUNITS) | DISPUNITMIC;
-				} else if (namesamen(pp, "centimicron", l) == 0 && l >= 7)
-				{
-					el_units = (el_units & ~DISPLAYUNITS) | DISPUNITCMIC;
-				} else if (namesamen(pp, "millimicron", l) == 0 && l >= 7)
-				{
-					el_units = (el_units & ~DISPLAYUNITS) | DISPUNITMMIC;
-				} else
+				if (namesamen(pp, "lambda", l) == 0) newunit = DISPUNITLAMBDA; else
+				if (namesamen(pp, "inch", l) == 0) newunit = DISPUNITINCH; else
+				if (namesamen(pp, "centimeter", l) == 0 && l >= 7) newunit = DISPUNITCM; else
+				if (namesamen(pp, "millimeter", l) == 0 && l >= 7) newunit = DISPUNITMM; else
+				if (namesamen(pp, "mil", l) == 0 && l >= 3) newunit = DISPUNITMIL; else
+				if (namesamen(pp, "micron", l) == 0 && l >= 3) newunit = DISPUNITMIC; else
+				if (namesamen(pp, "centimicron", l) == 0 && l >= 7) newunit = DISPUNITCMIC; else
+				if (namesamen(pp, "millimicron", l) == 0 && l >= 7) newunit = DISPUNITMMIC; else
 				{
 					us_abortcommand("Unknown display unit: %s", pp);
 					return;
 				}
+				setvalkey((INTBIG)us_aid, VAID, us_displayunits, newunit, VINTEGER);
 			}
 			switch (el_units&DISPLAYUNITS)
 			{
@@ -212,7 +197,9 @@ void us_lambda(INTSML count, char *par[])
 
 	/* determine which technologys to report */
 	for(tech = el_technologies; tech != NOTECHNOLOGY; tech = tech->nexttechnology) tech->temp1 = 0;
-	for(np=el_curlib->firstnodeproto; np!=NONODEPROTO; np=np->nextnodeproto) whattech(np)->temp1++;
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+			whattech(np)->temp1++;
 	el_curtech->temp1++;
 	for(tech = el_technologies; tech != NOTECHNOLOGY; tech = tech->nexttechnology)
 	{
@@ -223,18 +210,19 @@ void us_lambda(INTSML count, char *par[])
 
 void us_library(INTSML count, char *par[])
 {
-	REGISTER LIBRARY *lib, *ol, *lastlib, *mergelib;
+	REGISTER LIBRARY *lib, *olib, *lastlib, *mergelib;
 	REGISTER INTSML i, l, makecurrent, multifile, found, total, stripopts;
 	REGISTER INTBIG retval, arg;
 	REGISTER char *pp, *pt, *style, *libf, *oldlibfile, save;
 	REGISTER NODEPROTO *np;
+	REGISTER NODEINST *ni;
 	REGISTER VARIABLE *var;
 	REGISTER EDITOR *ed;
-	REGISTER WINDOWPART *w;
+	REGISTER WINDOWPART *w, *curw, *ow;
 	INTBIG lx, hx, ly, hy;
 	INTSML dummy;
-	char *newpar[2], *libn, *extn;
-	extern COMCOMP us_libraryp;
+	char *newpar[5], *libn, *extn;
+	extern COMCOMP us_libraryp, us_yesnop;
 	extern AIDENTRY *io_aid;
 
 	if (count == 0)
@@ -272,8 +260,8 @@ void us_library(INTSML count, char *par[])
 			{
 				if (np->newestversion == np) continue;
 				if (np->firstinst != NONODEINST) continue;
-				newpar[0] = describenodeproto(np);
-				us_killfacet(1, newpar);
+				ttyputmsg("Deleting facet %s", describenodeproto(np));
+				us_dokillfacet(np);
 				found++;
 				total++;
 				break;
@@ -323,6 +311,14 @@ void us_library(INTSML count, char *par[])
 
 		/* kill the library */
 		killlibrary(lib);
+
+		/*
+		 * the facet structure has changed
+		 *
+		 * this wouldn't be necessary if the "killlibrary()" routine did a
+		 * broadcast.  Then it would be detected properly in "us_killobject"
+		 */
+		us_redoexplorerwindow();
 		return;
 	}
 
@@ -444,7 +440,8 @@ void us_library(INTSML count, char *par[])
 					/* erase the old library */
 					eraselibrary(lib);
 					if (reallocstring(&lib->libfile, libf, lib->cluster)) ttyputerr("No memory!");
-					for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart) w->curnodeproto = NONODEPROTO;
+					for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
+						w->curnodeproto = NONODEPROTO;
 				}
 
 				/* remove any GETPROTOs to facets in this library */
@@ -499,21 +496,26 @@ void us_library(INTSML count, char *par[])
 					us_shadownodeproto(NOWINDOWFRAME, el_curtech->firstnodeproto);
 				}
 				np = el_curlib->curnodeproto;
-				if (np != NONODEPROTO && el_curwindowpart != NOWINDOWPART)
+				curw = el_curwindowpart;
+				if (curw != NOWINDOWPART && (curw->state&WINDOWTYPE) == EXPLORERWINDOW)
 				{
-#if 1
-					if (el_curwindowpart->termhandler != 0)
-						(*el_curwindowpart->termhandler)(el_curwindowpart);
-					if ((el_curwindowpart->state&WINDOWTYPE) == POPTEXTWINDOW)
-						(void)us_restorebox(el_curwindowpart->editor->savedbox, -1);
-					if ((el_curwindowpart->state&WINDOWTYPE) == TEXTWINDOW ||
-						(el_curwindowpart->state&WINDOWTYPE) == POPTEXTWINDOW)
-							us_freeeditor(el_curwindowpart->editor);
-#endif
+					for(ow = el_topwindowpart; ow != NOWINDOWPART; ow = ow->nextwindowpart)
+						if (ow != curw && ow->frame == curw->frame) break;
+					if (ow != NOWINDOWPART) curw = ow;
+				}
+				if (np != NONODEPROTO && curw != NOWINDOWPART)
+				{
+					if (curw->termhandler != 0)
+						(*curw->termhandler)(curw);
+					if ((curw->state&WINDOWTYPE) == POPTEXTWINDOW)
+						(void)screenrestorebox(curw->editor->savedbox, -1);
+					if ((curw->state&WINDOWTYPE) == TEXTWINDOW ||
+						(curw->state&WINDOWTYPE) == POPTEXTWINDOW)
+							us_freeeditor(curw->editor);
 					if ((np->cellview->viewstate&TEXTVIEW) != 0)
 					{
 						/* text window: make an editor */
-						if (us_makeeditor(el_curwindowpart, describenodeproto(np), &dummy, &dummy) == NOWINDOWPART)
+						if (us_makeeditor(curw, describenodeproto(np), &dummy, &dummy) == NOWINDOWPART)
 						{
 							efree(libn);
 							return;
@@ -534,61 +536,70 @@ void us_library(INTSML count, char *par[])
 						}
 
 						/* setup for editing */
-						el_curwindowpart->curnodeproto = np;
-						ed = el_curwindowpart->editor;
+						curw->curnodeproto = np;
+						ed = curw->editor;
 						ed->editobjaddr = (char *)np;
 						ed->editobjtype = VNODEPROTO;
 						ed->editobjqual = "FACET_message";
 						ed->editobjvar = var;
 
 						/* load the text into the window */
-						us_suspendgraphics(el_curwindowpart);
+						us_suspendgraphics(curw);
 						l = (INTSML)getlength(var);
 						for(i=0; i<l; i++)
 						{
 							pp = ((char **)var->addr)[i];
 							if (i == l-1 && *pp == 0) continue;
-							us_addline(el_curwindowpart, i, pp);
+							us_addline(curw, i, pp);
 						}
-						us_resumegraphics(el_curwindowpart);
-						el_curwindowpart->changehandler = us_textfacetchanges;
+						us_resumegraphics(curw);
+						curw->changehandler = us_textfacetchanges;
 					} else
 					{
 						/* adjust the new window to account for borders in the old one */
-						if ((el_curwindowpart->state&WINDOWTYPE) != DISPWINDOW)
+/*						us_gettruewindowbounds(curw, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy); */
+						if ((curw->state&WINDOWTYPE) != DISPWINDOW)
 						{
-							el_curwindowpart->usehx -= DISPLAYSLIDERSIZE;
-							el_curwindowpart->usely += DISPLAYSLIDERSIZE;
+							curw->usehx -= DISPLAYSLIDERSIZE;
+							curw->usely += DISPLAYSLIDERSIZE;
 						}
-						if ((el_curwindowpart->state&WINDOWSIMULATING) != 0)
+						if ((curw->state&WINDOWTYPE) == WAVEFORMWINDOW)
 						{
-							el_curwindowpart->uselx -= SIMULATINGBORDERSIZE;
-							el_curwindowpart->usehx += SIMULATINGBORDERSIZE;
-							el_curwindowpart->usely -= SIMULATINGBORDERSIZE;
-							el_curwindowpart->usehy += SIMULATINGBORDERSIZE;
+							curw->uselx -= DISPLAYSLIDERSIZE;
+							curw->usely -= DISPLAYSLIDERSIZE;
 						}
-						el_curwindowpart->curnodeproto = np;
-						el_curwindowpart->state = (el_curwindowpart->state & ~(WINDOWTYPE|WINDOWSIMULATING)) | DISPWINDOW;
-						el_curwindowpart->editor = NOEDITOR;
-						el_curwindowpart->buttonhandler = DEFAULTBUTTONHANDLER;
-						el_curwindowpart->charhandler = DEFAULTCHARHANDLER;
-						el_curwindowpart->changehandler = DEFAULTCHANGEHANDLER;
-						el_curwindowpart->termhandler = DEFAULTTERMHANDLER;
-						el_curwindowpart->redisphandler = DEFAULTREDISPHANDLER;
+						if ((curw->state&WINDOWSIMULATING) != 0)
+						{
+							curw->uselx -= SIMULATINGBORDERSIZE;
+							curw->usehx += SIMULATINGBORDERSIZE;
+							curw->usely -= SIMULATINGBORDERSIZE;
+							curw->usehy += SIMULATINGBORDERSIZE;
+						}
+						curw->curnodeproto = np;
+						curw->state = (curw->state & ~(WINDOWTYPE|WINDOWSIMULATING)) | DISPWINDOW;
+						curw->editor = NOEDITOR;
+						curw->buttonhandler = DEFAULTBUTTONHANDLER;
+						curw->charhandler = DEFAULTCHARHANDLER;
+						curw->changehandler = DEFAULTCHANGEHANDLER;
+						curw->termhandler = DEFAULTTERMHANDLER;
+						curw->redisphandler = DEFAULTREDISPHANDLER;
 						us_fullview(np, &lx, &hx, &ly, &hy);
-						us_squarescreen(el_curwindowpart, NOWINDOWPART, 0, &lx, &hx, &ly, &hy);
-						el_curwindowpart->screenlx = lx;
-						el_curwindowpart->screenhx = hx;
-						el_curwindowpart->screenly = ly;
-						el_curwindowpart->screenhy = hy;
-						computewindowscale(el_curwindowpart);
-						us_setfacetname(el_curwindowpart);
-						us_setfacetsize(el_curwindowpart);
-						us_setgridsize(el_curwindowpart);
-						if (el_curwindowpart->redisphandler != 0)
-							(*el_curwindowpart->redisphandler)(el_curwindowpart);
+						us_squarescreen(curw, NOWINDOWPART, 0, &lx, &hx, &ly, &hy);
+						curw->screenlx = lx;
+						curw->screenhx = hx;
+						curw->screenly = ly;
+						curw->screenhy = hy;
+						computewindowscale(curw);
+						us_setfacetname(curw);
+						us_setfacetsize(curw);
+						us_setgridsize(curw);
+						if (curw->redisphandler != 0)
+							(*curw->redisphandler)(curw);
 					}
 				}
+
+				/* redisplay the explorer window */
+				us_redoexplorerwindow();
 			}
 		} else if (mergelib == NOLIBRARY)
 		{
@@ -597,10 +608,10 @@ void us_library(INTSML count, char *par[])
 			{
 				/* this is not the current library: safe to delete */
 				lastlib = NOLIBRARY;
-				for(ol = el_curlib; ol != NOLIBRARY; ol = ol->nextlibrary)
+				for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
 				{
-					if (ol == lib) break;
-					lastlib = ol;
+					if (olib == lib) break;
+					lastlib = olib;
 				}
 				if (lastlib != NOLIBRARY) lastlib->nextlibrary = lib->nextlibrary;
 			} else
@@ -715,8 +726,8 @@ void us_library(INTSML count, char *par[])
 		/* rewind session logging if it is on */
 		if (us_logrecord != NULL)
 		{
-			us_logfinishrecord();
-			us_logstartrecord();
+			logfinishrecord();
+			logstartrecord();
 		}
 		return;
 	}
@@ -737,18 +748,20 @@ void us_library(INTSML count, char *par[])
 			if (l >= 1 && namesamen(pp, "binary", l) == 0) style = "binary"; else
 			if (l >= 1 && namesamen(pp, "cif", l) == 0) style = "cif"; else
 			if (l >= 1 && namesamen(pp, "dxf", l) == 0) style = "dxf"; else
-			if (l >= 1 && namesamen(pp, "edif", l) == 0) style = "edif"; else
+			if (l >= 2 && namesamen(pp, "eagle", l) == 0) style = "eagle"; else
+			if (l >= 2 && namesamen(pp, "edif", l) == 0) style = "edif"; else
 			if (l >= 1 && namesamen(pp, "gds", l) == 0) style = "gds"; else
 			if (l >= 1 && namesamen(pp, "hpgl", l) == 0) style = "hpgl"; else
 			if (l >= 2 && namesamen(pp, "lef", l) == 0) style = "lef"; else
 			if (l >= 1 && namesamen(pp, "l", l) == 0) style = "l"; else
+			if (l >= 2 && namesamen(pp, "pads", l) == 0) style = "pads"; else
 			if (l >= 2 && namesamen(pp, "postscript", l) == 0) style = "postscript"; else
 			if (l >= 2 && namesamen(pp, "printed-postscript", l) == 0) style = "printed-postscript"; else
 			if (l >= 1 && namesamen(pp, "quickdraw", l) == 0) style = "quickdraw"; else
 			if (l >= 1 && namesamen(pp, "skill", l) == 0) style = "skill"; else
 			if (l >= 1 && namesamen(pp, "text", l) == 0) style = "text"; else
 			{
-				us_abortcommand("File formats: binary|cif|dxf|edif|gds|hpgl|l|lef|postscript|printed-postscript|quickdraw|skill|text");
+				us_abortcommand("File formats: binary|cif|dxf|eagle|edif|gds|hpgl|l|lef|pads|postscript|printed-postscript|quickdraw|skill|text");
 				return;
 			}
 		}
@@ -772,8 +785,40 @@ void us_library(INTSML count, char *par[])
 		/* rewind session recording if writing in permanent format */
 		if ((*style == 'b' || *style == 't') && us_logrecord != NULL)
 		{
-			us_logfinishrecord();
-			us_logstartrecord();
+			logfinishrecord();
+			logstartrecord();
+		}
+
+		/* offer to save any referenced libraries */
+		if (namesame(style, "binary") == 0)
+		{
+			for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+				for(np = olib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+					np->temp2 = 0;
+			for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+				for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
+					if (ni->proto->primindex == 0)
+						ni->proto->temp2 = 1;
+			for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
+			{
+				if (olib == lib) continue;
+				for(np = olib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+					if (np->temp2 != 0) break;
+				if (np == NONODEPROTO) continue;
+				if ((olib->userbits&(LIBCHANGEDMAJOR | LIBCHANGEDMINOR)) == 0) continue;
+
+				(void)initinfstr();
+				(void)addstringtoinfstr("Also save referenced library ");
+				(void)addstringtoinfstr(olib->libname);
+				(void)addstringtoinfstr(" (which has changed ");
+				if ((olib->userbits&LIBCHANGEDMAJOR) == 0) (void)addstringtoinfstr("in");
+				(void)addstringtoinfstr("significantly)?");
+				i = ttygetparam(returninfstr(), &us_yesnop, 3, newpar);
+				if (i == 1 && newpar[0][0] == 'n') continue;
+				makeoptionstemporary(olib);
+				(void)askaid(io_aid, "write", (INTBIG)olib, (INTBIG)style);
+				restoreoptionstate();
+			}
 		}
 		return;
 	}
@@ -1333,12 +1378,12 @@ void us_menu(INTSML count, char *par[])
 				us_menubarsize = MENUBARHEIGHT;
 
 				/* load the information */
-				us_settextsize(el_curwindowpart, TXTMENU);
+				screensettextsize(el_curwindowpart, TXTMENU);
 				x = 0;
 				for(i=1; i<count; i++)
 				{
 					pm = us_pulldowns[i-1];
-					us_textsize(el_curwindowpart, us_stripampersand(pm->header), &wid, &hei);
+					screengettextsize(el_curwindowpart, us_stripampersand(pm->header), &wid, &hei);
 					x += wid + 12;
 					us_pulldownmenupos[i-1] = (INTSML)x;
 				}
@@ -1712,7 +1757,8 @@ void us_move(INTSML count, char *par[])
 	REGISTER ARCINST *ai;
 	REGISTER INTBIG amt, i, wid, len, itemHit;
 	REGISTER INTSML l, total;
-	INTBIG dx, dy, xcur, ycur, lx, hx, ly, hy, bestlx, bestly, p1x, p1y, p2x, p2y, snapx, snapy;
+	INTBIG dx, dy, xcur, ycur, lx, hx, ly, hy, bestlx, bestly, p1x, p1y, p2x, p2y,
+		snapx, snapy;
 	HIGHLIGHT high;
 	REGISTER GEOM **list;
 	HIGHLIGHT thishigh, otherhigh;
@@ -1727,9 +1773,6 @@ void us_move(INTSML count, char *par[])
 	np = us_needfacet();
 	if (np == NONODEPROTO) return;
 
-	/* disallow moving if lock is on */
-	if (us_canedit(np, NONODEPROTO, 1) != 0) return;
-
 	/* get the objects to be moved */
 	list = us_gethighlighted(OBJNODEINST | OBJARCINST);
 
@@ -1756,7 +1799,7 @@ void us_move(INTSML count, char *par[])
 	}
 
 	/* make sure they are all in the same facet */
-	for(i=1; list[i] != NOGEOM; i++) if (np != geomparent(list[i]))
+	for(i=0; list[i] != NOGEOM; i++) if (np != geomparent(list[i]))
 	{
 		us_abortcommand("All moved objects must be in the same facet");
 		return;
@@ -1996,19 +2039,43 @@ void us_move(INTSML count, char *par[])
 	dx = dy = 0;
 	if (namesamen(pp, "up", l) == 0 && l >= 1)
 	{
-		if (count < 2) amt = el_curtech->deflambda; else amt = atola(par[1]);
+		if (count < 2) amt = el_curtech->deflambda; else
+		{
+			l = el_units;
+			el_units = (el_units & ~DISPLAYUNITS) | DISPUNITLAMBDA;
+			amt = atola(par[1]);
+			el_units = l;
+		}
 		dy += amt;
 	} else if (namesamen(pp, "down", l) == 0 && l >= 1)
 	{
-		if (count < 2) amt = el_curtech->deflambda; else amt = atola(par[1]);
+		if (count < 2) amt = el_curtech->deflambda; else
+		{
+			l = el_units;
+			el_units = (el_units & ~DISPLAYUNITS) | DISPUNITLAMBDA;
+			amt = atola(par[1]);
+			el_units = l;
+		}
 		dy -= amt;
 	} else if (namesamen(pp, "left", l) == 0 && l >= 1)
 	{
-		if (count < 2) amt = el_curtech->deflambda; else amt = atola(par[1]);
+		if (count < 2) amt = el_curtech->deflambda; else
+		{
+			l = el_units;
+			el_units = (el_units & ~DISPLAYUNITS) | DISPUNITLAMBDA;
+			amt = atola(par[1]);
+			el_units = l;
+		}
 		dx -= amt;
 	} else if (namesamen(pp, "right", l) == 0 && l >= 1)
 	{
-		if (count < 2) amt = el_curtech->deflambda; else amt = atola(par[1]);
+		if (count < 2) amt = el_curtech->deflambda; else
+		{
+			l = el_units;
+			el_units = (el_units & ~DISPLAYUNITS) | DISPUNITLAMBDA;
+			amt = atola(par[1]);
+			el_units = l;
+		}
 		dx += amt;
 	} else
 	{
@@ -2043,7 +2110,6 @@ void us_node(INTSML count, char *par[])
 	/* disallow node modification if lock is on */
 	np = us_needfacet();
 	if (np == NONODEPROTO) return;
-	if (us_canedit(np, NONODEPROTO, 1) != 0) return;
 
 	if (count == 0)
 	{
@@ -2155,6 +2221,9 @@ void us_node(INTSML count, char *par[])
 		return;
 	}
 
+	/* everything after this point is structural and must check for locks */
+	if (us_canedit(np, NONODEPROTO, 1) != 0) return;
+
 	if (namesamen(pt, "cover-implant", l) == 0 && l >= 1)
 	{
 		us_coverimplant();
