@@ -350,12 +350,12 @@ void us_package(INTSML count, char *par[])
 
 void us_port(INTSML count, char *par[])
 {
-	INTBIG lx, hx, ly, hy, mlx, mly, mhx, mhy, bits, mask;
+	INTBIG lx, hx, ly, hy, mlx, mly, mhx, mhy, bits, mask, newbits, x, y, deleted;
 	REGISTER NODEINST *ni, *oni;
 	REGISTER NODEPROTO *np;
 	REGISTER PORTPROTO *pp, *ppt;
 	REGISTER PORTARCINST *pi;
-	REGISTER PORTEXPINST *pe;
+	REGISTER PORTEXPINST *pe, *nextpe;
 	REGISTER LIBRARY *olib;
 	GEOM *fromgeom, *togeom;
 	PORTPROTO *fromport, *toport;
@@ -647,7 +647,48 @@ void us_port(INTSML count, char *par[])
 			us_abortcommand("Usage: port not export [OPTIONS]");
 			return;
 		}
+		if (count == 3 && namesame(par[2], "geometry") == 0)
+		{
+			/* delete ports in selected area */
+			np = us_getareabounds(&lx, &hx, &ly, &hy);
+			if (np == NONODEPROTO)
+			{
+				us_abortcommand("Outline an area first");
+				return;
+			}
 
+			/* get the necessary polygon */
+			if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
+
+			/* remove highlighting */
+			us_pushhighlight();
+			us_clearhighlightcount();
+
+			deleted = 0;
+			for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
+			{
+				i = 0;
+				for(pe = ni->firstportexpinst; pe != NOPORTEXPINST; pe = nextpe)
+				{
+					nextpe = pe->nextportexpinst;
+					shapeportpoly(ni, pe->proto, poly, 0);
+					getcenter(poly, &x, &y);
+					if (x < lx || x > hx || y < ly || y > hy) continue;
+					ttyputmsg("Deleting port %s", pe->exportproto->protoname);
+					if (i == 0) startobjectchange((INTBIG)ni, VNODEINST);
+					if (killportproto(np, pe->exportproto))
+						ttyputerr("killportproto error");
+					i = 1;
+					deleted++;
+				}
+				if (i != 0) endobjectchange((INTBIG)ni, VNODEINST);
+			}
+
+			/* restore highlighting */
+			(void)us_pophighlight(0);
+			if (deleted == 0) ttyputmsg("No exported ports are in the selected area");
+			return;
+		}
 		if (count == 3 && namesame(par[2], "all") == 0)
 		{
 			g = us_gethighlighted(OBJNODEINST);
@@ -662,6 +703,7 @@ void us_port(INTSML count, char *par[])
 			us_clearhighlightcount();
 
 			/* delete ports from all selected node */
+			deleted = 0;
 			for(i=0; g[i] != NOGEOM; i++)
 			{
 				ni = g[i]->entryaddr.ni;
@@ -669,13 +711,20 @@ void us_port(INTSML count, char *par[])
 
 				/* remove the port(s) */
 				startobjectchange((INTBIG)ni, VNODEINST);
-				us_undoportproto(ni, NOPORTPROTO);
+				for(pe = ni->firstportexpinst; pe != NOPORTEXPINST; pe = nextpe)
+				{
+					nextpe = pe->nextportexpinst;
+					ttyputmsg("Deleting port %s", pe->exportproto->protoname);
+					if (killportproto(ni->parent, pe->exportproto))
+						ttyputerr("killportproto error");
+					deleted++;
+				}
 				endobjectchange((INTBIG)ni, VNODEINST);
 			}
 
 			/* restore highlighting */
 			(void)us_pophighlight(0);
-			ttyputverbose("All ports deleted on the selected nodes");
+			if (deleted == 0) ttyputmsg("No exported ports are on the selected nodes");
 			return;
 		}
 
@@ -806,12 +855,14 @@ void us_port(INTSML count, char *par[])
 		if (pp == NOPORTPROTO) return;
 
 		/* change the port characteristics */
+		newbits = pp->userbits;
 		if ((mask&STATEBITS) != 0)
-			pp->userbits = (pp->userbits & ~STATEBITS) | (bits & STATEBITS);
+			newbits = (newbits & ~STATEBITS) | (bits & STATEBITS);
 		if ((mask&PORTDRAWN) != 0)
-			pp->userbits = (pp->userbits & ~PORTDRAWN) | (bits & PORTDRAWN);
+			newbits = (newbits & ~PORTDRAWN) | (bits & PORTDRAWN);
 		if ((mask&BODYONLY) != 0)
-			pp->userbits = (pp->userbits & ~BODYONLY) | (bits & BODYONLY);
+			newbits = (newbits & ~BODYONLY) | (bits & BODYONLY);
+		setval((INTBIG)pp, VPORTPROTO, "userbits", newbits, VINTEGER);
 
 		/* propagate the change information up the hierarchy */
 		changeallports(pp);
