@@ -268,7 +268,7 @@ void us_terminal(INTSML count, char *par[])
 		us_technologycnsp, us_technologyclgp, us_technologyclsp, us_showdp,
 		us_colorhighp, us_bindgkeyp, us_gridp, us_technologytp, us_nodetcaip,
 		us_librarykp, us_findobjap, us_window3dp, us_nodetptlp, us_renamecp,
-		us_findexportp, us_findnnamep, us_showup;
+		us_findexportp, us_findnnamep, us_showup, us_varop, us_copyfacetdp;
 
 	REGISTER COMCOMP *comcomp;
 	float newfloat;
@@ -441,6 +441,7 @@ void us_terminal(INTSML count, char *par[])
 			if (namesame(par[3], "area") == 0)          comcomp = &us_technologycnsp; else
 			if (namesame(par[3], "array") == 0)         comcomp = &us_arrayxp; else
 			if (namesame(par[3], "artlook") == 0)       comcomp = &us_artlookp; else
+			if (namesame(par[3], "attributes") == 0)    comcomp = &us_varop; else
 			if (namesame(par[3], "capacitance") == 0)   comcomp = &us_technologyclscp; else
 			if (namesame(par[3], "change") == 0)        comcomp = &us_replacep; else
 			if (namesame(par[3], "chglibrary") == 0)    comcomp = &us_librarykp; else
@@ -463,6 +464,7 @@ void us_terminal(INTSML count, char *par[])
 			if (namesame(par[3], "grid") == 0)          comcomp = &us_gridp; else
 			if (namesame(par[3], "help") == 0)          comcomp = &us_helpp; else
 			if (namesame(par[3], "highlayer") == 0)     comcomp = &us_colorhighp; else
+			if (namesame(par[3], "iconopt") == 0)       comcomp = &us_copyfacetdp; else
 			if (namesame(par[3], "inductance") == 0)    comcomp = &us_technologyclsecp; else
 			if (namesame(par[3], "labels") == 0)        comcomp = &us_portlp; else
 			if (namesame(par[3], "lambda") == 0)        comcomp = &us_lambdachp; else
@@ -537,7 +539,7 @@ void us_terminal(INTSML count, char *par[])
 
 	if (namesamen(pp, "session", l) == 0)
 	{
-		if (us_graphicshas(CANLOGINPUT) == 0)
+		if (graphicshas(CANLOGINPUT) == 0)
 		{
 			us_abortcommand("Sorry, this display driver does not log sessions");
 			return;
@@ -554,7 +556,7 @@ void us_terminal(INTSML count, char *par[])
 				us_abortcommand("Usage: terminal session playback FILE");
 				return;
 			}
-			if (us_logplayback(par[2]) != 0)
+			if (logplayback(par[2]) != 0)
 				us_abortcommand("Cannot read playback file %s", par[2]);
 			return;
 		}
@@ -566,7 +568,7 @@ void us_terminal(INTSML count, char *par[])
 				us_abortcommand("Session is already being recorded");
 				return;
 			}
-			us_logstartrecord();
+			logstartrecord();
 			return;
 		}
 
@@ -579,14 +581,14 @@ void us_terminal(INTSML count, char *par[])
 
 		if (namesamen(pp, "end-record", l) == 0)
 		{
-			us_logfinishrecord();
+			logfinishrecord();
 			return;
 		}
 
 		if (namesamen(pp, "rewind-record", l) == 0)
 		{
-			us_logfinishrecord();
-			us_logstartrecord();
+			logfinishrecord();
+			logstartrecord();
 			return;
 		}
 
@@ -611,7 +613,7 @@ void us_terminal(INTSML count, char *par[])
 
 void us_text(INTSML count, char *par[])
 {
-	REGISTER INTBIG descript, grabpoint, olddescript, len, font;
+	REGISTER INTBIG descript, grabpoint, newgrabpoint, olddescript, len, font;
 	REGISTER INTSML i, l;
 	INTBIG xw, yw, xcur, ycur, adjxc, adjyc;
 	INTSML tsx, tsy, style;
@@ -840,6 +842,32 @@ void us_text(INTSML count, char *par[])
 		return;
 	}
 
+	/* handle default interior style */
+	if (namesamen(pp, "default-interior-only", l) == 0 && l >= 9)
+	{
+		/* get current value */
+		var = getval((INTBIG)us_aid, VAID, VINTEGER, "USER_default_text_style");
+		if (var == NOVARIABLE) grabpoint = VTPOSCENT; else grabpoint = var->addr;
+
+		/* set the default interior style */
+		setval((INTBIG)us_aid, VAID, "USER_default_text_style",
+			grabpoint|VTINTERIOR, VINTEGER|VDONTSAVE);
+		ttyputverbose("Default text is only visible inside facet");
+		return;
+	}
+	if (namesamen(pp, "default-exterior", l) == 0 && l >= 9)
+	{
+		/* get current value */
+		var = getval((INTBIG)us_aid, VAID, VINTEGER, "USER_default_text_style");
+		if (var == NOVARIABLE) grabpoint = VTPOSCENT; else grabpoint = var->addr;
+
+		/* set the default interior style */
+		setval((INTBIG)us_aid, VAID, "USER_default_text_style",
+			grabpoint & ~VTINTERIOR, VINTEGER|VDONTSAVE);
+		ttyputverbose("Default text is visible outside of facet");
+		return;
+	}
+
 	/* handle default style */
 	if (namesamen(pp, "default-style", l) == 0 && l >= 9)
 	{
@@ -851,15 +879,20 @@ void us_text(INTSML count, char *par[])
 		if (count < 2)
 		{
 			ttyputmsg("Default text style is %s", us_describestyle(grabpoint));
+			if ((grabpoint&VTINTERIOR) != 0)
+				ttyputmsg("Default text is only visible inside facet"); else
+					ttyputmsg("Default text is visible outside of facet");
 			return;
 		}
 
 		/* get new value */
-		grabpoint = us_gettextposition(par[1]);
-		if (grabpoint < 0) return;
+		newgrabpoint = us_gettextposition(par[1]);
+		if (newgrabpoint < 0) return;
+		grabpoint = (grabpoint & ~VTPOSITION) | newgrabpoint;
 
 		/* set the default text style */
-		setval((INTBIG)us_aid, VAID, "USER_default_text_style", grabpoint, VINTEGER|VDONTSAVE);
+		setval((INTBIG)us_aid, VAID, "USER_default_text_style",
+			grabpoint, VINTEGER|VDONTSAVE);
 		ttyputverbose("Default text style is now %s", us_describestyle(grabpoint));
 		return;
 	}
@@ -949,20 +982,8 @@ void us_text(INTSML count, char *par[])
 		us_gethightextcenter(high, &adjxc, &adjyc, &style);
 
 		/* get the text and current descriptor */
-		if (high->fromvar != NOVARIABLE)
-		{
-			olddescript = high->fromvar->textdescript;
-			str = describevariable(high->fromvar, -1, -1);
-		} else if (high->fromport != NOPORTPROTO)
-		{
-			olddescript = high->fromport->textdescript;
-			str = high->fromport->protoname;
-		} else if (high->fromgeom->entrytype == OBJNODEINST)
-		{
-			ni = high->fromgeom->entryaddr.ni;
-			olddescript = ni->textdescript;
-			str = describenodeproto(ni->proto);
-		}
+		str = us_gethighstring(high);
+		olddescript = us_gethighdescript(high);
 
 		/* get size of text */
 		len = 1;
@@ -977,9 +998,9 @@ void us_text(INTSML count, char *par[])
 			if (high->fromgeom->entrytype == OBJARCINST)
 				tech = high->fromgeom->entryaddr.ai->proto->tech; else
 					tech = whattech(high->fromgeom->entryaddr.ni->proto);
-			us_settextsize(el_curwindowpart, truefontsize((INTSML)((olddescript&VTSIZE)>>VTSIZESH),
+			screensettextsize(el_curwindowpart, truefontsize((INTSML)((olddescript&VTSIZE)>>VTSIZESH),
 				el_curwindowpart, tech));
-			us_textsize(el_curwindowpart, str, &tsx, &tsy);
+			screengettextsize(el_curwindowpart, str, &tsx, &tsy);
 			xw = muldiv(tsx, el_curwindowpart->screenhx-el_curwindowpart->screenlx,
 				el_curwindowpart->usehx-el_curwindowpart->uselx);
 			yw = muldiv(tsy, el_curwindowpart->screenhy-el_curwindowpart->screenly,
@@ -1084,6 +1105,45 @@ void us_text(INTSML count, char *par[])
 		return;
 	}
 
+	if (namesamen(pp, "interior-only", l) == 0)
+	{
+		if (high->fromvar == NOVARIABLE)
+		{
+			us_abortcommand("Interior style only applies to text variables");
+			return;
+		}
+		if (high->fromgeom->entrytype != OBJNODEINST)
+		{
+			us_abortcommand("Interior style only applies to nodes");
+			return;
+		}
+
+		ni = high->fromgeom->entryaddr.ni;
+		startobjectchange((INTBIG)ni, VNODEINST);
+		us_modifytextdescript(high, high->fromvar->textdescript | VTINTERIOR);
+		endobjectchange((INTBIG)ni, VNODEINST);
+		return;
+	}
+
+	if (namesamen(pp, "exterior", l) == 0 && l >= 2)
+	{
+		if (high->fromvar == NOVARIABLE)
+		{
+			us_abortcommand("Interior style only applies to text variables");
+			return;
+		}
+		if (high->fromgeom->entrytype != OBJNODEINST)
+		{
+			us_abortcommand("Interior style only applies to nodes");
+			return;
+		}
+		ni = high->fromgeom->entryaddr.ni;
+		startobjectchange((INTBIG)ni, VNODEINST);
+		us_modifytextdescript(high, high->fromvar->textdescript & ~VTINTERIOR);
+		endobjectchange((INTBIG)ni, VNODEINST);
+		return;
+	}
+
 	us_abortcommand("Invalid TEXT option: %s", pp);
 }
 
@@ -1093,6 +1153,7 @@ void us_undo(INTSML count, char *par[])
 	REGISTER char *direction;
 	AIDENTRY *aid;
 	REGISTER NODEPROTO *np;
+	REGISTER LIBRARY *lib;
 	INTSML *totals;
 
 	if (count > 0 && namesamen(par[0], "save", strlen(par[0])) == 0)
@@ -1200,8 +1261,9 @@ void us_undo(INTSML count, char *par[])
 	if (us_curnodeproto != NONODEPROTO && us_curnodeproto->primindex == 0 &&
 		(us_state&NONPERSISTENTCURNODE) == 0)
 	{
-		for(np = el_curlib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
-			if (us_curnodeproto == np) break;
+		for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+			for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+				if (us_curnodeproto == np) break;
 		if (np == NONODEPROTO)
 		{
 			np = el_curlib->firstnodeproto;
@@ -1266,6 +1328,8 @@ void us_var(INTSML count, char *par[])
 	INTSML aindex, dummy, comvar, but, x, y;
 	REGISTER VARIABLE *var, *tvar, *res, *var1, *var2;
 	static VARIABLE fvar;
+	REGISTER GEOM **list, *geom;
+	REGISTER NODEINST *ni;
 	VARIABLE *evar, fvar1, fvar2;
 	REGISTER NODEPROTO *np;
 	REGISTER ARCPROTO *ap;
@@ -1331,6 +1395,25 @@ void us_var(INTSML count, char *par[])
 		return;
 	}
 
+	if (namesamen(pp, "reinherit", l) == 0 && l >= 1)
+	{
+		list = us_gethighlighted(OBJNODEINST);
+		if (list[0] == NOGEOM)
+		{
+			us_abortcommand("Must select nodes for reinheriting attributes");
+			return;
+		}
+		for(i=0; list[i] != NOGEOM; i++)
+		{
+			geom = list[i];
+			if (geom->entrytype != OBJNODEINST) continue;
+			ni = geom->entryaddr.ni;
+			us_inheritattributes(ni);
+		}
+		ttyputmsg("%d nodes have had their attributes reinherited", i);
+		return;
+	}
+
 	if (namesamen(pp, "examine", l) == 0 && l >= 1)
 	{
 		if (count <= 1)
@@ -1469,7 +1552,7 @@ void us_var(INTSML count, char *par[])
 			if (negated == 0)
 			{
 				var->type |= VDISPLAY;
-				modifydescript(var, newdescript);
+				modifydescript(objaddr, objtype, var, newdescript);
 			} else var->type &= ~VDISPLAY;
 
 			/* signal end of change to nodes and arcs */
@@ -1709,7 +1792,7 @@ void us_var(INTSML count, char *par[])
 			{
 				if (ttydataready() != 0 && wpop->charhandler != 0)
 				{
-					i = us_getnxtchar();
+					i = getnxtchar();
 					if ((*wpop->charhandler)(wpop, (INTSML)i) != 0) break;
 				} else
 				{
@@ -2733,7 +2816,6 @@ void us_view(INTSML count, char *par[])
 	REGISTER VIEW *nv;
 	REGISTER WINDOWPART *w;
 	REGISTER EDITOR *ed;
-	static INTBIG facet_schematic_page_size = 0;
 
 	if (count == 0)
 	{
@@ -2963,18 +3045,14 @@ void us_view(INTSML count, char *par[])
 		/* get facet */
 		np = us_needfacet();
 		if (np == NONODEPROTO) return;
-		if (facet_schematic_page_size == 0)
-			facet_schematic_page_size = makekey("FACET_schematic_page_size");
 
 		if (count == 1)
 		{
 			/* report the frame in use here */
-			var = getvalkey((INTBIG)np, VNODEPROTO, VSTRING, facet_schematic_page_size);
+			var = getvalkey((INTBIG)np, VNODEPROTO, VSTRING, el_schematic_page_size);
 			if (var == NOVARIABLE)
 			{
-				if ((np->cellview->viewstate&MULTIPAGEVIEW) != 0)
-					ttyputmsg("This facet has a default frame of size D"); else
-						ttyputmsg("This facet has no schematic frame");
+				ttyputmsg("This facet has no schematic frame");
 				return;
 			}
 			pt = (char *)var->addr;
@@ -2982,7 +3060,10 @@ void us_view(INTSML count, char *par[])
 			switch (pt[0])
 			{
 				case 'a': case 'A': (void)addtoinfstr('A');   break;
+				case 'b': case 'B': (void)addtoinfstr('B');   break;
+				case 'c': case 'C': (void)addtoinfstr('C');   break;
 				case 'd': case 'D': (void)addtoinfstr('D');   break;
+				case 'e': case 'E': (void)addtoinfstr('E');   break;
 			}
 			if (pt[1] == 'v' || pt[1] == 'V') (void)addstringtoinfstr(", vertical");
 			ttyputmsg("This facet has a frame of size %s", returninfstr());
@@ -2995,20 +3076,41 @@ void us_view(INTSML count, char *par[])
 		{
 			pt = "a";
 			if (count > 2 && namesamen(par[2], "vertical", strlen(par[2])) == 0) pt = "av";
-			(void)setvalkey((INTBIG)np, VNODEPROTO, facet_schematic_page_size, (INTBIG)pt, VSTRING);
+			(void)setvalkey((INTBIG)np, VNODEPROTO, el_schematic_page_size, (INTBIG)pt, VSTRING);
+			return;
+		}
+		if (namesamen(pp, "B", l) == 0 && l >= 1)
+		{
+			pt = "b";
+			if (count > 2 && namesamen(par[2], "vertical", strlen(par[2])) == 0) pt = "bv";
+			(void)setvalkey((INTBIG)np, VNODEPROTO, el_schematic_page_size, (INTBIG)pt, VSTRING);
+			return;
+		}
+		if (namesamen(pp, "C", l) == 0 && l >= 1)
+		{
+			pt = "c";
+			if (count > 2 && namesamen(par[2], "vertical", strlen(par[2])) == 0) pt = "cv";
+			(void)setvalkey((INTBIG)np, VNODEPROTO, el_schematic_page_size, (INTBIG)pt, VSTRING);
 			return;
 		}
 		if (namesamen(pp, "D", l) == 0 && l >= 1)
 		{
 			pt = "d";
 			if (count > 2 && namesamen(par[2], "vertical", strlen(par[2])) == 0) pt = "dv";
-			(void)setvalkey((INTBIG)np, VNODEPROTO, facet_schematic_page_size, (INTBIG)pt, VSTRING);
+			(void)setvalkey((INTBIG)np, VNODEPROTO, el_schematic_page_size, (INTBIG)pt, VSTRING);
+			return;
+		}
+		if (namesamen(pp, "E", l) == 0 && l >= 1)
+		{
+			pt = "e";
+			if (count > 2 && namesamen(par[2], "vertical", strlen(par[2])) == 0) pt = "ev";
+			(void)setvalkey((INTBIG)np, VNODEPROTO, el_schematic_page_size, (INTBIG)pt, VSTRING);
 			return;
 		}
 		if (namesamen(pp, "none", l) == 0 && l >= 1)
 		{
-			if (getvalkey((INTBIG)np, VNODEPROTO, VSTRING, facet_schematic_page_size) != NOVARIABLE)
-				(void)delvalkey((INTBIG)np, VNODEPROTO, facet_schematic_page_size);
+			if (getvalkey((INTBIG)np, VNODEPROTO, VSTRING, el_schematic_page_size) != NOVARIABLE)
+				(void)delvalkey((INTBIG)np, VNODEPROTO, el_schematic_page_size);
 			return;
 		}
 		us_abortcommand("Unknown VIEW FRAME option: %s", pp);
