@@ -268,7 +268,7 @@ void us_window(INTSML count, char *par[])
 		if (closefrontmostmessages() != 0) return;
 
 		/* delete split if there are no multiple window frames */
-		if (us_graphicshas(CANUSEFRAMES) == 0)
+		if (graphicshas(CANUSEFRAMES) == 0)
 		{
 			if (us_needwindow()) return;
 			us_killcurrentwindow(1);
@@ -276,7 +276,7 @@ void us_window(INTSML count, char *par[])
 		}
 
 		/* disallow if this is the last window and it must remain */
-		if (us_graphicshas(CANHAVENOWINDOWS) == 0)
+		if (graphicshas(CANHAVENOWINDOWS) == 0)
 		{
 			/* disallow deletion if this is the last window */
 			i = 0;
@@ -290,7 +290,7 @@ void us_window(INTSML count, char *par[])
 		}
 
 		/* get the current frame */
-		wf = us_getwindowframe(0);
+		wf = getwindowframe(0);
 		if (wf == NOWINDOWFRAME)
 		{
 			us_abortcommand("No current window to delete");
@@ -319,7 +319,8 @@ void us_window(INTSML count, char *par[])
 		}
 		endobjectchange((INTBIG)us_aid, VAID);
 
-		if (neww == NOWINDOWPART) el_curwindowpart = NOWINDOWPART;
+		/* if (neww == NOWINDOWPART) */
+		el_curwindowpart = NOWINDOWPART;
 		(void)setvalkey((INTBIG)us_aid, VAID, us_current_window, (INTBIG)neww, VWINDOWPART|VDONTSAVE);
 		if (neww != NOWINDOWPART) np = neww->curnodeproto; else np = NONODEPROTO;
 		(void)setval((INTBIG)el_curlib, VLIBRARY, "curnodeproto", (INTBIG)np, VNODEPROTO);
@@ -388,6 +389,64 @@ void us_window(INTSML count, char *par[])
 		return;
 	}
 
+	if (namesamen(pp, "explore", l) == 0)
+	{
+		extern INTBIG us_explorefirstline;
+
+		/* see if the explorer window is already there */
+		for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
+			if ((w->state&WINDOWTYPE) == EXPLORERWINDOW) break;
+		if (w != NOWINDOWPART)
+		{
+			/* if the explorer shares a window, delete it */
+			el_curwindowpart = w;
+			us_killcurrentwindow(1);
+			return;
+		}
+
+		/* make an explorer window */
+		w = el_curwindowpart;
+		if (w == NOWINDOWPART)
+		{
+			w = us_wantnewwindow(0);
+			if (w == NOWINDOWPART)
+			{
+				us_abortcommand("Cannot create new window frame");
+				return;
+			}
+		}
+		if (strcmp(w->location, "entire") == 0)
+		{
+			w = us_splitcurrentwindow(2, 0);
+			if (w == NOWINDOWPART) return;
+		}
+
+		/* find the other window */
+		for(oldw = el_topwindowpart; oldw != NOWINDOWPART; oldw = oldw->nextwindowpart)
+			if (oldw != w && oldw->frame == w->frame) break;
+
+		us_explorefirstline = 0;
+		us_createexplorerstruct();
+		startobjectchange((INTBIG)w, VWINDOWPART);
+		(void)setval((INTBIG)w, VWINDOWPART, "buttonhandler", (INTBIG)us_explorebuttonhandler,
+			VADDRESS);
+		(void)setval((INTBIG)w, VWINDOWPART, "charhandler", (INTBIG)us_explorecharhandler,
+			VADDRESS);
+		(void)setval((INTBIG)w, VWINDOWPART, "redisphandler", (INTBIG)us_exploreredisphandler,
+			VADDRESS);
+		(void)setval((INTBIG)w, VWINDOWPART, "state",
+			(w->state & ~(WINDOWTYPE|WINDOWSIMULATING)) | EXPLORERWINDOW, VINTEGER);
+		endobjectchange((INTBIG)w, VWINDOWPART);
+
+		if (oldw != NOWINDOWPART)
+		{
+			(void)setvalkey((INTBIG)us_aid, VAID, us_current_window, (INTBIG)oldw,
+				VWINDOWPART|VDONTSAVE);
+			us_setfacetname(oldw);
+		}
+		return;
+	}
+
 	if (namesamen(pp, "highlight-displayed", l) == 0 && l >= 1)
 	{
 		if (us_needwindow()) return;
@@ -802,9 +861,9 @@ void us_window(INTSML count, char *par[])
 			return;
 		}
 		l = strlen(pp = par[1]);
-		if (namesamen(pp, "standard", l) == 0) us_setnormalcursor(NORMALCURSOR); else
-			if (namesamen(pp, "pen", l) == 0) us_setnormalcursor(PENCURSOR); else
-				if (namesamen(pp, "tee", l) == 0) us_setnormalcursor(TECHCURSOR); else
+		if (namesamen(pp, "standard", l) == 0) setnormalcursor(NORMALCURSOR); else
+			if (namesamen(pp, "pen", l) == 0) setnormalcursor(PENCURSOR); else
+				if (namesamen(pp, "tee", l) == 0) setnormalcursor(TECHCURSOR); else
 					us_abortcommand("Bad window cursor option: %s", pp);
 		return;
 	}
@@ -1450,6 +1509,7 @@ void us_window(INTSML count, char *par[])
 		for(w = el_topwindowpart; w != NOWINDOWPART; w = nextw)
 		{
 			nextw = w->nextwindowpart;
+			if (w->frame != neww->frame) continue;
 			if (w != neww) killwindowpart(w);
 		}
 
