@@ -38,6 +38,7 @@
 #include "usr.h"
 #include "drc.h"
 #include "usredtec.h"		/* for technology documentation */
+#include <math.h>
 
 #define TRAN_PIN   -1
 #define TRAN_FACET -2
@@ -74,6 +75,7 @@ NODEPROTO *us_convertfacet(NODEPROTO *oldfacet, TECHNOLOGY *newtech)
 	NODEPROTO *(*localconversion)(NODEPROTO*, TECHNOLOGY*);
 	REGISTER TECHNOLOGY *oldtech;
 	REGISTER ARCINST *ai;
+	REGISTER LIBRARY *lib;
 	VARIABLE *var;
 
 	/* cannot convert text-only views */
@@ -107,8 +109,9 @@ NODEPROTO *us_convertfacet(NODEPROTO *oldfacet, TECHNOLOGY *newtech)
 			us_tran_logmakearcs(oldfacet, newfacet);
 
 			/* now make adjustments for manhattan-ness */
-			for(np = newfacet->cell->lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
-				np->temp1 = 0;
+			for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+				for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+					np->temp1 = 0;
 			us_tran_logadjustnodes(newfacet);
 
 			/* set "FIXANG" if reasonable */
@@ -1164,7 +1167,7 @@ void us_printtechnology(TECHNOLOGY *tech)
 				poly->layer = arclay->lay;
 			}
 			(void)allocstring(&layers[tot], layername(tech, poly->layer), el_tempcluster);
-			area = abs(areapoly(poly)) / 4000;
+			area = (INTBIG)(fabs(areapoly(poly)) / 4000.0);
 			saveunit = el_units & DISPLAYUNITS;
 			el_units = (el_units & ~DISPLAYUNITS) | DISPUNITMIC;
 			(void)allocstring(&layersizes[tot], latoa(area), el_tempcluster);
@@ -1209,7 +1212,7 @@ void us_printtechnology(TECHNOLOGY *tech)
 	/* allocate space for all node fields */
 	ni = dummynode();
 	tot = 1;
-	for(np=tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+	for(np = tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 	{
 		ni->proto = np;
 		if (tech->nodepolys != 0) j = (*(tech->nodepolys))(ni); else
@@ -1247,7 +1250,7 @@ void us_printtechnology(TECHNOLOGY *tech)
 	(void)allocstring(&connections[0], "Connections", el_tempcluster);
 
 	tot = 1;
-	for(np=tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+	for(np = tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 	{
 		base = tot;
 		(void)allocstring(&names[tot], np->primname, el_tempcluster);
