@@ -94,6 +94,8 @@ INTSML us_drawarcinstpeek(ARCINST*, XARRAY, INTSML);
 INTSML us_drawnodeinstpeek(NODEINST*, XARRAY, INTSML);
 void us_peeksetlayer(INTSML, TECHNOLOGY*);
 void us_peekwritepolygon(INTBIG*, INTBIG*, INTSML, float);
+void us_drawnodeprotovariables(NODEPROTO *np, XARRAY trans, WINDOWPART *w);
+void us_drawportprotovariables(PORTPROTO *pp, INTSML on, XARRAY trans, WINDOWPART *w);
 
 /******************** WINDOW CONTROL ********************/
 
@@ -104,7 +106,7 @@ void us_drawwindow(WINDOWPART *w, INTSML color)
 {
 	WINDOWPART ww;
 	static POLYGON *poly = NOPOLYGON;
-	REGISTER INTBIG lx, hx, ly, hy;
+	INTBIG lx, hx, ly, hy;
 
 	/* get polygon */
 	if (poly == NOPOLYGON) poly = allocstaticpolygon(5, us_aid->cluster);
@@ -115,18 +117,9 @@ void us_drawwindow(WINDOWPART *w, INTSML color)
 	/* don't draw window border around popup text editors */
 	if ((w->state & WINDOWTYPE) == POPTEXTWINDOW) return;
 
-	lx = w->uselx - 1;   hx = w->usehx + 1;
-	ly = w->usely - 1;   hy = w->usehy + 1;
-	if ((w->state&WINDOWSIMULATING) != 0)
-	{
-		lx -= SIMULATINGBORDERSIZE;   hx += SIMULATINGBORDERSIZE;
-		ly -= SIMULATINGBORDERSIZE;   hy += SIMULATINGBORDERSIZE;
-	}
-	if ((w->state&WINDOWTYPE) == DISPWINDOW)
-	{
-		hx += DISPLAYSLIDERSIZE;
-		ly -= DISPLAYSLIDERSIZE;
-	}
+	us_gettruewindowbounds(w, &lx, &hx, &ly, &hy);
+	lx--;   hx++;
+	ly--;   hy++;
 
 	ww.screenlx = ww.uselx = (INTSML)lx;
 	ww.screenhx = ww.usehx = (INTSML)hx;
@@ -142,6 +135,31 @@ void us_drawwindow(WINDOWPART *w, INTSML color)
 	(void)us_showpoly(poly, &ww);
 }
 
+/*
+ * Routine to get the actual drawing area in window "w".  This excludes borders and
+ * sliders.  The bounds are returned in (lx/hx/ly/hy).
+ */
+void us_gettruewindowbounds(WINDOWPART *w, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
+{
+	*lx = w->uselx;   *hx = w->usehx;
+	*ly = w->usely;   *hy = w->usehy;
+	if ((w->state&WINDOWSIMULATING) != 0)
+	{
+		*lx -= SIMULATINGBORDERSIZE;   *hx += SIMULATINGBORDERSIZE;
+		*ly -= SIMULATINGBORDERSIZE;   *hy += SIMULATINGBORDERSIZE;
+	}
+	if ((w->state&WINDOWTYPE) == DISPWINDOW)
+	{
+		*hx += DISPLAYSLIDERSIZE;
+		*ly -= DISPLAYSLIDERSIZE;
+	}
+	if ((w->state&WINDOWTYPE) == WAVEFORMWINDOW)
+	{
+		*lx -= DISPLAYSLIDERSIZE;
+		*ly -= DISPLAYSLIDERSIZE;
+	}
+}
+
 /*
  * routine to adjust screen boundaries to fill the view
  */
@@ -300,6 +318,16 @@ void us_fullview(NODEPROTO *np, INTBIG *screenlx, INTBIG *screenhx, INTBIG *scre
 				}
 			}
 		}
+
+		/* set default size if nothing is there */
+		if (first != 0)
+		{
+			*screenlx = np->lowx;
+			*screenhx = np->highx;
+			*screenly = np->lowy;
+			*screenhy = np->highy;
+		}
+
 		if (*screenlx >= *screenhx && *screenly >= *screenhy)
 		{
 			*screenlx -= 20 * el_curtech->deflambda;
@@ -433,11 +461,13 @@ WINDOWPART *us_subwindow(INTBIG lx, INTBIG hx, INTBIG ly, INTBIG hy, WINDOWPART
 	ww.screenhy = muldiv(w->screenhy-w->screenly, ww.usehy-w->usely, w->usehy-w->usely) +
 		w->screenly;
 	ww.frame = w->frame;
+	ww.curnodeproto = w->curnodeproto;
 	ww.state = DISPWINDOW;
 	computewindowscale(&ww);
 	return(&ww);
 }
 
+
 /*
  * routine to erase window "w" and draw everything that should be in it.
  */
@@ -445,13 +475,7 @@ void us_redisplay(WINDOWPART *w)
 {
 	WINDOWPART ww;
 	static POLYGON *poly = NOPOLYGON;
-	REGISTER INTSML lx, hx, ly, hy, i;
-#  define ARROWPOINTS 7
-	static INTSML xfrompoint[] = {1, DISPLAYSLIDERSIZE/2, DISPLAYSLIDERSIZE/2, DISPLAYSLIDERSIZE-1,
-		DISPLAYSLIDERSIZE-1, DISPLAYSLIDERSIZE/2, DISPLAYSLIDERSIZE/2};
-	static INTSML yfrompoint[] = {1+DISPLAYSLIDERSIZE/2, DISPLAYSLIDERSIZE,
-		1+DISPLAYSLIDERSIZE/3*2, 1+DISPLAYSLIDERSIZE/3*2, 2+DISPLAYSLIDERSIZE/3,
-		2+DISPLAYSLIDERSIZE/3, 2};
+	INTBIG lx, hx, ly, hy;
 
 	/* get polygon */
 	if (poly == NOPOLYGON) poly = allocstaticpolygon(7, us_aid->cluster);
@@ -459,19 +483,11 @@ void us_redisplay(WINDOWPART *w)
 	/* draw fat red border if simulating */
 	if ((w->state&WINDOWSIMULATING) != 0)
 	{
-		lx = w->uselx;   hx = w->usehx;
-		ly = w->usely;   hy = w->usehy;
-		if ((w->state&WINDOWTYPE) == DISPWINDOW)
-		{
-			hx += DISPLAYSLIDERSIZE;
-			ly -= DISPLAYSLIDERSIZE;
-		}
-		lx -= SIMULATINGBORDERSIZE;   hx += SIMULATINGBORDERSIZE;
-		ly -= SIMULATINGBORDERSIZE;   hy += SIMULATINGBORDERSIZE;
-		ww.screenlx = ww.uselx = lx;
-		ww.screenhx = ww.usehx = hx;
-		ww.screenly = ww.usely = ly;
-		ww.screenhy = ww.usehy = hy;
+		us_gettruewindowbounds(w, &lx, &hx, &ly, &hy);
+		ww.screenlx = ww.uselx = (INTSML)lx;
+		ww.screenhx = ww.usehx = (INTSML)hx;
+		ww.screenly = ww.usely = (INTSML)ly;
+		ww.screenhy = ww.usehy = (INTSML)hy;
 		ww.frame = w->frame;
 		ww.state = DISPWINDOW;
 		computewindowscale(&ww);
@@ -494,86 +510,17 @@ void us_redisplay(WINDOWPART *w)
 	{
 		lx = w->uselx;   hx = w->usehx;
 		ly = w->usely;   hy = w->usehy;
-		ww.screenlx = ww.uselx = lx;
-		ww.screenhx = ww.usehx = hx+DISPLAYSLIDERSIZE;
-		ww.screenly = ww.usely = ly-DISPLAYSLIDERSIZE;
-		ww.screenhy = ww.usehy = hy;
-		ww.frame = w->frame;
-		ww.state = DISPWINDOW;
-		computewindowscale(&ww);
-		poly->desc = &us_box;
 
-		/* the vertical slider on the right */
-		us_box.col = WHITE;
-		poly->style = FILLEDRECT;
-		maketruerectpoly(hx, hx+DISPLAYSLIDERSIZE, ly-1, hy, poly);
-		(void)us_showpoly(poly, &ww);
-
-		/* the horizontal slider on the bottom */
-		maketruerectpoly(lx, hx+1, ly-DISPLAYSLIDERSIZE, ly, poly);
-		(void)us_showpoly(poly, &ww);
+		/* the slider on the bottom and right */
+		w->usehx += DISPLAYSLIDERSIZE;
+		w->usely -= DISPLAYSLIDERSIZE;
+		us_drawverticalslider(w, (INTSML)hx, (INTSML)ly, (INTSML)hy, 0);
+		us_drawhorizontalslider(w, (INTSML)ly, (INTSML)lx, (INTSML)hx);
+		w->usehx -= DISPLAYSLIDERSIZE;
+		w->usely += DISPLAYSLIDERSIZE;
 
 		/* the corner */
-		us_box.col = DGRAY;
-		poly->style = FILLEDRECT;
-		maketruerectpoly(hx+1, hx+DISPLAYSLIDERSIZE, ly-DISPLAYSLIDERSIZE, ly-1, poly);
-		(void)us_showpoly(poly, &ww);
-
-		/* draw arrows */
-		poly->count = ARROWPOINTS;
-		us_box.col = BLACK;
-		poly->style = FILLED;
-		for(i=0; i<ARROWPOINTS; i++)
-		{
-			poly->xv[i] = lx + xfrompoint[i];
-			poly->yv[i] = ly - yfrompoint[i];
-		}
-		(void)us_showpoly(poly, &ww);
-		for(i=0; i<ARROWPOINTS; i++)
-		{
-			poly->xv[i] = hx - xfrompoint[i];
-			poly->yv[i] = ly - yfrompoint[i];
-		}
-		(void)us_showpoly(poly, &ww);
-		for(i=0; i<ARROWPOINTS; i++)
-		{
-			poly->xv[i] = hx + yfrompoint[i];
-			poly->yv[i] = ly + xfrompoint[i];
-		}
-		(void)us_showpoly(poly, &ww);
-		for(i=0; i<ARROWPOINTS; i++)
-		{
-			poly->xv[i] = hx + yfrompoint[i];
-			poly->yv[i] = hy - xfrompoint[i];
-		}
-		(void)us_showpoly(poly, &ww);
-
-		/* draw border lines */
-		poly->count = 2;
-		us_box.col = BLACK;
-		poly->style = OPENED;
-		poly->xv[0] = lx;                     poly->yv[0] = ly-1;
-		poly->xv[1] = hx+DISPLAYSLIDERSIZE;   poly->yv[1] = ly-1;
-		(void)us_showpoly(poly, &ww);
-		poly->xv[0] = hx+1;   poly->yv[0] = ly-DISPLAYSLIDERSIZE;
-		poly->xv[1] = hx+1;   poly->yv[1] = hy;
-		(void)us_showpoly(poly, &ww);
-
-		/* draw borders enclosing horizontal arrows */
-		poly->xv[0] = lx+DISPLAYSLIDERSIZE+1;   poly->yv[0] = ly-DISPLAYSLIDERSIZE;
-		poly->xv[1] = lx+DISPLAYSLIDERSIZE+1;   poly->yv[1] = ly;
-		(void)us_showpoly(poly, &ww);
-		poly->xv[0] = hx-DISPLAYSLIDERSIZE-1;   poly->yv[0] = ly-DISPLAYSLIDERSIZE;
-		poly->xv[1] = hx-DISPLAYSLIDERSIZE-1;   poly->yv[1] = ly;
-		(void)us_showpoly(poly, &ww);
-
-		/* draw borders enclosing vertical arrows */
-		poly->xv[0] = hx;                     poly->yv[0] = hy-DISPLAYSLIDERSIZE-1;
-		poly->xv[1] = hx+DISPLAYSLIDERSIZE;   poly->yv[1] = hy-DISPLAYSLIDERSIZE-1;
-		(void)us_showpoly(poly, &ww);
-		poly->xv[0] = hx;                     poly->yv[0] = ly+DISPLAYSLIDERSIZE+1;
-		poly->xv[1] = hx+DISPLAYSLIDERSIZE;   poly->yv[1] = ly+DISPLAYSLIDERSIZE+1;
-		(void)us_showpoly(poly, &ww);
+		us_drawslidercorner(w, hx+1, hx+DISPLAYSLIDERSIZE, ly-DISPLAYSLIDERSIZE, ly-1);
 
 		/* fill in the current slider positions */
 		us_drawdispwindowsliders(w);
@@ -583,29 +530,276 @@ void us_redisplay(WINDOWPART *w)
 		us_redisplaynow(w, 0);
 }
 
-void us_drawdispwindowsliders(WINDOWPART *w)
+/*
+ * Routine to draw a grey box in the corner where two sliders meet.
+ */
+void us_drawslidercorner(WINDOWPART *win, INTBIG lx, INTBIG hx, INTBIG ly, INTBIG hy)
 {
-	WINDOWPART ww;
 	static POLYGON *poly = NOPOLYGON;
-	REGISTER INTSML lx, hx, ly, hy;
-	REGISTER NODEPROTO *np;
-	REGISTER INTBIG facetsizex, facetsizey, screensizex, screensizey,
-		thumbsizex, thumbsizey, thumbareax, thumbareay, thumbposx, thumbposy;
-
-	if ((w->state&WINDOWTYPE) != DISPWINDOW) return;
+	WINDOWPART ww;
 
 	/* get polygon */
 	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
 
+	ww.screenlx = ww.uselx = (INTSML)lx;
+	ww.screenhx = ww.usehx = (INTSML)hx;
+	ww.screenly = ww.usely = (INTSML)ly;
+	ww.screenhy = ww.usehy = (INTSML)hy;
+	ww.frame = win->frame;
+	ww.state = DISPWINDOW;
+	computewindowscale(&ww);
+	poly->desc = &us_box;
+	us_box.col = DGRAY;
+	poly->style = FILLEDRECT;
+	maketruerectpoly(lx, hx, ly, hy, poly);
+	(void)us_showpoly(poly, &ww);
+}
+
+/*
+ * Routine to determine the location of the thumb in the sliders of window "w"
+ * and to draw those thumbs.
+ */
+void us_drawdispwindowsliders(WINDOWPART *w)
+{
+	REGISTER INTSML lx, hx, ly, hy;
+	REGISTER NODEPROTO *np;
+	INTBIG facetsizex, facetsizey;
+	REGISTER INTBIG screensizex, screensizey,
+		thumbsizex, thumbsizey, thumbareax, thumbareay, thumbposx, thumbposy;
+
+	if ((w->state&WINDOWTYPE) != DISPWINDOW) return;
+
+	/* default: no thumb drawn in sliders */
+	w->thumblx = 0;  w->thumbhx = -1;
+	w->thumbly = 0;  w->thumbhy = -1;
+
+	np = w->curnodeproto;
 	lx = w->uselx;   hx = w->usehx;
 	ly = w->usely;   hy = w->usehy;
-	ww.screenlx = ww.uselx = lx;
-	ww.screenhx = ww.usehx = hx+DISPLAYSLIDERSIZE;
-	ww.screenly = ww.usely = ly-DISPLAYSLIDERSIZE;
-	ww.screenhy = ww.usehy = hy;
+	if (np != NONODEPROTO)
+	{
+		/* determine amount of the facet that is shown */
+		facetsizex = np->highx - np->lowx;
+		facetsizey = np->highy - np->lowy;
+		if (facetsizex > 0 && facetsizey >= 0)
+		{
+			screensizex = w->screenhx - w->screenlx;
+			screensizey = w->screenhy - w->screenly;
+			thumbareax = (hx - lx - DISPLAYSLIDERSIZE*2-4) / 2;
+			thumbareay = (hy - ly - DISPLAYSLIDERSIZE*2-4) / 2;
+			if (facetsizex <= screensizex)
+			{
+				thumbsizex = thumbareax;
+				thumbposx = muldiv(w->screenhx - (np->highx + np->lowx)/2, thumbareax,
+					w->screenhx-w->screenlx) + (hx - lx - thumbareax) / 2 + lx;
+			} else
+			{
+				thumbsizex = thumbareax * screensizex / facetsizex;
+				if (thumbsizex < 20) thumbsizex = 20;
+				thumbposx = muldiv((w->screenhx + w->screenlx)/2 - np->lowx, thumbareax,
+					np->highx-np->lowx) + (hx - lx - thumbareax) / 2 + lx;
+			}
+			if (facetsizey <= screensizey)
+			{
+				thumbsizey = thumbareay;
+				thumbposy = muldiv(w->screenhy - (np->highy + np->lowy)/2, thumbareay,
+					w->screenhy-w->screenly) + (hy - ly - thumbareay) / 2 + ly;
+			} else
+			{
+				thumbsizey = thumbareay * screensizey / facetsizey;
+				if (thumbsizey < 20) thumbsizey = 20;
+				thumbposy = muldiv((w->screenhy + w->screenly)/2 - np->lowy, thumbareay,
+					np->highy-np->lowy) + (hy - ly - thumbareay) / 2 + ly;
+			}
+			w->thumblx = thumbposx-thumbsizex/2;  w->thumbhx = thumbposx+thumbsizex/2;
+			w->thumbly = thumbposy-thumbsizey/2;  w->thumbhy = thumbposy+thumbsizey/2;
+			if (w->thumblx < lx + DISPLAYSLIDERSIZE + 2) w->thumblx = lx + DISPLAYSLIDERSIZE + 2;
+			if (w->thumbhx < lx + DISPLAYSLIDERSIZE + 20) w->thumbhx = lx + DISPLAYSLIDERSIZE + 20;
+			if (w->thumbhx > hx - DISPLAYSLIDERSIZE - 2) w->thumbhx = hx - DISPLAYSLIDERSIZE - 2;
+			if (w->thumblx > hx - DISPLAYSLIDERSIZE - 20) w->thumblx = hx - DISPLAYSLIDERSIZE - 20;
+
+			if (w->thumbly < ly + DISPLAYSLIDERSIZE + 2) w->thumbly = ly + DISPLAYSLIDERSIZE + 2;
+			if (w->thumbhy < ly + DISPLAYSLIDERSIZE + 20) w->thumbhy = ly + DISPLAYSLIDERSIZE + 20;
+			if (w->thumbhy > hy - DISPLAYSLIDERSIZE - 2) w->thumbhy = hy - DISPLAYSLIDERSIZE - 2;
+			if (w->thumbly > hy - DISPLAYSLIDERSIZE - 20) w->thumbly = hy - DISPLAYSLIDERSIZE - 20;
+		}
+	}
+
+	/* prepare a window in which to draw the thumbs */
+	lx = w->uselx;   hx = w->usehx;
+	ly = w->usely;   hy = w->usehy;
+
+	/* now draw the thumbs */
+	w->usehx += DISPLAYSLIDERSIZE;
+	w->usely -= DISPLAYSLIDERSIZE;
+	us_drawverticalsliderthumb(w, hx, ly, hy, w->thumbly, w->thumbhy);
+	us_drawhorizontalsliderthumb(w, ly, lx, hx, w->thumblx, w->thumbhx);
+	w->usehx -= DISPLAYSLIDERSIZE;
+	w->usely += DISPLAYSLIDERSIZE;
+}
+
+#define ARROWPOINTS 7
+static INTSML us_xarrowpoint[] = {1, DISPLAYSLIDERSIZE/2, DISPLAYSLIDERSIZE/2, DISPLAYSLIDERSIZE-1,
+	DISPLAYSLIDERSIZE-1, DISPLAYSLIDERSIZE/2, DISPLAYSLIDERSIZE/2};
+static INTSML us_yarrowpoint[] = {1+DISPLAYSLIDERSIZE/2, DISPLAYSLIDERSIZE,
+	1+DISPLAYSLIDERSIZE/3*2, 1+DISPLAYSLIDERSIZE/3*2, 2+DISPLAYSLIDERSIZE/3,
+	2+DISPLAYSLIDERSIZE/3, 2};
+
+/*
+ * Routine to draw a vertical slider in window "w" whose left edge is "lx" and which
+ * runs vertically from "ly" to "hy".  The slider has arrows and borders, but no thumb.
+ * If "onleft" is nonzero, this slider is on the left.
+ */
+void us_drawverticalslider(WINDOWPART *w, INTSML lx, INTSML ly, INTSML hy, INTSML onleft)
+{
+	static POLYGON *poly = NOPOLYGON;
+	REGISTER INTBIG i;
+	WINDOWPART ww;
+
+	/* get polygon */
+	if (poly == NOPOLYGON) poly = allocstaticpolygon(7, us_aid->cluster);
+
+	ww.screenlx = ww.uselx = w->uselx;
+	ww.screenhx = ww.usehx = w->usehx;
+	ww.screenly = ww.usely = w->usely;
+	ww.screenhy = ww.usehy = w->usehy;
 	ww.frame = w->frame;
 	ww.state = DISPWINDOW;
 	computewindowscale(&ww);
+
+	/* erase the area */
+	poly->desc = &us_box;
+	us_box.col = WHITE;
+	poly->style = FILLEDRECT;
+	maketruerectpoly(lx+1, lx+DISPLAYSLIDERSIZE, ly-1, hy, poly);
+	(void)us_showpoly(poly, &ww);
+
+	/* draw arrows */
+	poly->count = ARROWPOINTS;
+	us_box.col = BLACK;
+	poly->style = FILLED;
+	for(i=0; i<ARROWPOINTS; i++)
+	{
+		poly->xv[i] = lx + us_yarrowpoint[i];
+		poly->yv[i] = ly + us_xarrowpoint[i];
+	}
+	(void)us_showpoly(poly, &ww);
+	for(i=0; i<ARROWPOINTS; i++)
+	{
+		poly->xv[i] = lx + us_yarrowpoint[i];
+		poly->yv[i] = hy - us_xarrowpoint[i];
+	}
+	(void)us_showpoly(poly, &ww);
+
+	/* draw border lines */
+	poly->count = 2;
+	us_box.col = BLACK;
+	poly->style = OPENED;
+	if (onleft == 0)
+	{
+		poly->xv[0] = lx+1;   poly->yv[0] = ly-DISPLAYSLIDERSIZE;
+		poly->xv[1] = lx+1;   poly->yv[1] = hy;
+		(void)us_showpoly(poly, &ww);
+	} else
+	{
+		poly->xv[0] = lx+DISPLAYSLIDERSIZE+1;   poly->yv[0] = ly-DISPLAYSLIDERSIZE;
+		poly->xv[1] = lx+DISPLAYSLIDERSIZE+1;   poly->yv[1] = hy;
+		(void)us_showpoly(poly, &ww);
+	}
+
+	/* draw borders enclosing vertical arrows */
+	poly->xv[0] = lx+1;                   poly->yv[0] = hy-DISPLAYSLIDERSIZE-1;
+	poly->xv[1] = lx+DISPLAYSLIDERSIZE;   poly->yv[1] = hy-DISPLAYSLIDERSIZE-1;
+	(void)us_showpoly(poly, &ww);
+	poly->xv[0] = lx+1;                   poly->yv[0] = ly+DISPLAYSLIDERSIZE+1;
+	poly->xv[1] = lx+DISPLAYSLIDERSIZE;   poly->yv[1] = ly+DISPLAYSLIDERSIZE+1;
+	(void)us_showpoly(poly, &ww);
+}
+
+/*
+ * Routine to draw a horizontal slider in window "w" whose top edge is "hy" and which
+ * runs horizontally from "lx" to "hx".  The slider has arrows and borders, but no thumb.
+ */
+void us_drawhorizontalslider(WINDOWPART *w, INTSML hy, INTSML lx, INTSML hx)
+{
+	static POLYGON *poly = NOPOLYGON;
+	REGISTER INTBIG i;
+	WINDOWPART ww;
+
+	/* get polygon */
+	if (poly == NOPOLYGON) poly = allocstaticpolygon(7, us_aid->cluster);
+
+	ww.screenlx = ww.uselx = w->uselx;
+	ww.screenhx = ww.usehx = w->usehx;
+	ww.screenly = ww.usely = w->usely;
+	ww.screenhy = ww.usehy = w->usehy;
+	ww.frame = w->frame;
+	ww.state = DISPWINDOW;
+	computewindowscale(&ww);
+
+	/* erase the area */
+	poly->desc = &us_box;
+	us_box.col = WHITE;
+	poly->style = FILLEDRECT;
+	maketruerectpoly(lx, hx+1, hy-DISPLAYSLIDERSIZE, hy-1, poly);
+	(void)us_showpoly(poly, &ww);
+
+	/* draw arrows */
+	poly->count = ARROWPOINTS;
+	us_box.col = BLACK;
+	poly->style = FILLED;
+	for(i=0; i<ARROWPOINTS; i++)
+	{
+		poly->xv[i] = lx + us_xarrowpoint[i];
+		poly->yv[i] = hy - us_yarrowpoint[i];
+	}
+	(void)us_showpoly(poly, &ww);
+	for(i=0; i<ARROWPOINTS; i++)
+	{
+		poly->xv[i] = hx - us_xarrowpoint[i];
+		poly->yv[i] = hy - us_yarrowpoint[i];
+	}
+	(void)us_showpoly(poly, &ww);
+
+	/* draw border lines */
+	poly->count = 2;
+	us_box.col = BLACK;
+	poly->style = OPENED;
+	poly->xv[0] = lx;                     poly->yv[0] = hy-1;
+	poly->xv[1] = hx+DISPLAYSLIDERSIZE;   poly->yv[1] = hy-1;
+	(void)us_showpoly(poly, &ww);
+
+	/* draw borders enclosing vertical arrows */
+	poly->xv[0] = lx+DISPLAYSLIDERSIZE+1;   poly->yv[0] = hy-DISPLAYSLIDERSIZE;
+	poly->xv[1] = lx+DISPLAYSLIDERSIZE+1;   poly->yv[1] = hy-1;
+	(void)us_showpoly(poly, &ww);
+	poly->xv[0] = hx-DISPLAYSLIDERSIZE-1;   poly->yv[0] = hy-DISPLAYSLIDERSIZE;
+	poly->xv[1] = hx-DISPLAYSLIDERSIZE-1;   poly->yv[1] = hy-1;
+	(void)us_showpoly(poly, &ww);
+}
+
+/*
+ * Routine to draw the thumb in the vertical slider of window "w".  The slider has
+ * its left edge at "hx" and runs from "ly" to "hy".  The thumb runs from
+ * "lt" to "ht" (if "lt" is greater than "ht", don't draw a thumb).
+ */
+void us_drawverticalsliderthumb(WINDOWPART *w, INTBIG hx, INTBIG ly, INTBIG hy,
+	INTBIG lt, INTBIG ht)
+{
+	static POLYGON *poly = NOPOLYGON;
+	WINDOWPART ww;
+
+	/* get polygon */
+	if (poly == NOPOLYGON) poly = allocstaticpolygon(7, us_aid->cluster);
+
+	ww.screenlx = ww.uselx = w->uselx;
+	ww.screenhx = ww.usehx = w->usehx;
+	ww.screenly = ww.usely = w->usely;
+	ww.screenhy = ww.usehy = w->usehy;
+	ww.frame = w->frame;
+	ww.state = DISPWINDOW;
+	computewindowscale(&ww);
+
 	poly->desc = &us_box;
 
 	/* erase the vertical slider on the right */
@@ -614,81 +808,70 @@ void us_drawdispwindowsliders(WINDOWPART *w)
 	maketruerectpoly(hx+2, hx+DISPLAYSLIDERSIZE, ly+DISPLAYSLIDERSIZE+2, hy-DISPLAYSLIDERSIZE-2, poly);
 	(void)us_showpoly(poly, &ww);
 
-	/* erase the horizontal slider on the bottom */
+	/* stop now if no thumb requested */
+	if (lt > ht) return;
+
+	/* draw the vertical thumb */
+	us_box.col = BLACK;
+	maketruerectpoly(hx+2, hx+DISPLAYSLIDERSIZE, lt, ht, poly);
+	(void)us_showpoly(poly, &ww);
+
+	/* draw lines around the vertical thumb */
+	poly->count = 4;
+	poly->style = CLOSED;
+	us_box.col = WHITE;
+	poly->xv[0] = hx+3;                     poly->yv[0] = lt+1;
+	poly->xv[1] = hx+3;                     poly->yv[1] = ht-1;
+	poly->xv[2] = hx+DISPLAYSLIDERSIZE-1;   poly->yv[2] = ht-1;
+	poly->xv[3] = hx+DISPLAYSLIDERSIZE-1;   poly->yv[3] = lt+1;
+	(void)us_showpoly(poly, &ww);
+}
+
+/*
+ * Routine to draw the thumb in the horizontal slider of window "w".  The slider has
+ * its top edge at "ly" and runs from "lx" to "hx".  The thumb runs from
+ * "lt" to "ht" (if "lt" is greater than "ht", don't draw a thumb).
+ */
+void us_drawhorizontalsliderthumb(WINDOWPART *w, INTBIG ly, INTBIG lx, INTBIG hx, INTBIG lt, INTBIG ht)
+{
+	static POLYGON *poly = NOPOLYGON;
+	WINDOWPART ww;
+
+	/* get polygon */
+	if (poly == NOPOLYGON) poly = allocstaticpolygon(7, us_aid->cluster);
+
+	ww.screenlx = ww.uselx = w->uselx;
+	ww.screenhx = ww.usehx = w->usehx;
+	ww.screenly = ww.usely = w->usely;
+	ww.screenhy = ww.usehy = w->usehy;
+	ww.frame = w->frame;
+	ww.state = DISPWINDOW;
+	computewindowscale(&ww);
+
+	poly->desc = &us_box;
+
+	/* erase the vertical slider on the right */
+	us_box.col = LGRAY;
+	poly->style = FILLEDRECT;
 	maketruerectpoly(lx+DISPLAYSLIDERSIZE+2, hx-DISPLAYSLIDERSIZE-2, ly-DISPLAYSLIDERSIZE, ly-2, poly);
 	(void)us_showpoly(poly, &ww);
 
-	np = w->curnodeproto;
-	if (np == NONODEPROTO) return;
-
-	/* determine amount of the facet that is shown */
-	facetsizex = np->highx - np->lowx;
-	facetsizey = np->highy - np->lowy;
-	if (facetsizex <= 0 || facetsizey <= 0) return;
-	screensizex = w->screenhx - w->screenlx;
-	screensizey = w->screenhy - w->screenly;
-	thumbareax = (hx - lx - DISPLAYSLIDERSIZE*2-4) / 2;
-	thumbareay = (hy - ly - DISPLAYSLIDERSIZE*2-4) / 2;
-	if (facetsizex <= screensizex)
-	{
-		thumbsizex = thumbareax;
-		thumbposx = (w->screenhx - (np->highx + np->lowx)/2) * thumbareax / (w->screenhx-w->screenlx) +
-			(hx - lx - thumbareax) / 2 + lx;
-	} else
-	{
-		thumbsizex = thumbareax * screensizex / facetsizex;
-		if (thumbsizex < 20) thumbsizex = 20;
-		thumbposx = ((w->screenhx + w->screenlx)/2 - np->lowx) * thumbareax / (np->highx-np->lowx) +
-			(hx - lx - thumbareax) / 2 + lx;
-	}
-	if (facetsizey <= screensizey)
-	{
-		thumbsizey = thumbareay;
-		thumbposy = (w->screenhy - (np->highy + np->lowy)/2) * thumbareay / (w->screenhy-w->screenly) +
-			(hy - ly - thumbareay) / 2 + ly;
-	} else
-	{
-		thumbsizey = thumbareay * screensizey / facetsizey;
-		if (thumbsizey < 20) thumbsizey = 20;
-		thumbposy = ((w->screenhy + w->screenly)/2 - np->lowy) * thumbareay / (np->highy-np->lowy) +
-			(hy - ly - thumbareay) / 2 + ly;
-	}
-	w->thumblx = thumbposx-thumbsizex/2;  w->thumbhx = thumbposx+thumbsizex/2;
-	w->thumbly = thumbposy-thumbsizey/2;  w->thumbhy = thumbposy+thumbsizey/2;
-	if (w->thumblx < lx + DISPLAYSLIDERSIZE + 2) w->thumblx = lx + DISPLAYSLIDERSIZE + 2;
-	if (w->thumbhx < lx + DISPLAYSLIDERSIZE + 20) w->thumbhx = lx + DISPLAYSLIDERSIZE + 20;
-	if (w->thumbhx > hx - DISPLAYSLIDERSIZE - 2) w->thumbhx = hx - DISPLAYSLIDERSIZE - 2;
-	if (w->thumblx > hx - DISPLAYSLIDERSIZE - 20) w->thumblx = hx - DISPLAYSLIDERSIZE - 20;
-
-	if (w->thumbly < ly + DISPLAYSLIDERSIZE + 2) w->thumbly = ly + DISPLAYSLIDERSIZE + 2;
-	if (w->thumbhy < ly + DISPLAYSLIDERSIZE + 20) w->thumbhy = ly + DISPLAYSLIDERSIZE + 20;
-	if (w->thumbhy > hy - DISPLAYSLIDERSIZE - 2) w->thumbhy = hy - DISPLAYSLIDERSIZE - 2;
-	if (w->thumbly > hy - DISPLAYSLIDERSIZE - 20) w->thumbly = hy - DISPLAYSLIDERSIZE - 20;
+	/* stop now if no thumb requested */
+	if (lt > ht) return;
 
 	/* draw the horizontal thumb */
 	us_box.col = BLACK;
-	maketruerectpoly(w->thumblx, w->thumbhx, ly-DISPLAYSLIDERSIZE, ly-2, poly);
-	(void)us_showpoly(poly, &ww);
-
-	/* draw the vertical thumb */
-	maketruerectpoly(hx+2, hx+DISPLAYSLIDERSIZE, w->thumbly, w->thumbhy, poly);
+	maketruerectpoly(lt, ht, ly-DISPLAYSLIDERSIZE, ly-2, poly);
 	(void)us_showpoly(poly, &ww);
 
 	/* draw lines around the horizontal thumb */
 	poly->count = 4;
 	poly->style = CLOSED;
 	us_box.col = WHITE;
-	poly->xv[0] = w->thumblx+1;   poly->yv[0] = ly-DISPLAYSLIDERSIZE+1;
-	poly->xv[1] = w->thumblx+1;   poly->yv[1] = ly-3;
-	poly->xv[2] = w->thumbhx-1;   poly->yv[2] = ly-3;
-	poly->xv[3] = w->thumbhx-1;   poly->yv[3] = ly-DISPLAYSLIDERSIZE+1;
-	(void)us_showpoly(poly, &ww);
-
-	/* draw lines around the vertical thumb */
-	poly->xv[0] = hx+3;                     poly->yv[0] = w->thumbly+1;
-	poly->xv[1] = hx+3;                     poly->yv[1] = w->thumbhy-1;
-	poly->xv[2] = hx+DISPLAYSLIDERSIZE-1;   poly->yv[2] = w->thumbhy-1;
-	poly->xv[3] = hx+DISPLAYSLIDERSIZE-1;   poly->yv[3] = w->thumbly+1;
+	poly->xv[0] = lt+1;   poly->yv[0] = ly-DISPLAYSLIDERSIZE+1;
+	poly->xv[1] = lt+1;   poly->yv[1] = ly-3;
+	poly->xv[2] = ht-1;   poly->yv[2] = ly-3;
+	poly->xv[3] = ht-1;   poly->yv[3] = ly-DISPLAYSLIDERSIZE+1;
 	(void)us_showpoly(poly, &ww);
 }
 
@@ -764,6 +947,9 @@ void us_redisplaynow(WINDOWPART *w, INTSML now)
 		}
 	}
 
+	/* show this facet and exported port variables */
+	us_drawnodeprotovariables(facet, el_matid, w);
+
 	/* re-draw grid if on */
 	if ((w->state&(GRIDON|GRIDTOOSMALL)) == GRIDON &&
 		(w->state&WINDOWTYPE) != DISP3DWINDOW)
@@ -822,7 +1008,7 @@ void us_erasescreen(WINDOWFRAME *mw)
 	for(frame = el_firstwindowframe; frame != NOWINDOWFRAME; frame = frame->nextwindowframe)
 	{
 		if (mw != NOWINDOWFRAME && frame != mw) continue;
-		us_getwindowsize(frame, &swid, &shei);
+		getwindowframesize(frame, &swid, &shei);
 		ww.screenlx = ww.uselx = 0;   ww.screenhx = ww.usehx = swid-1;
 		ww.screenly = ww.usely = 0;   ww.screenhy = ww.usehy = shei-1;
 		ww.frame = frame;
@@ -887,7 +1073,6 @@ void us_gridset(WINDOWPART *w, INTBIG state)
 	y0 = applyyscale(w, 0-w->screenly) + w->usely;
 	y1 = applyyscale(w, w->gridy-w->screenly) + w->usely;
 
-/*	if (abs(x1-x0) < MINGRID || abs(y1-y0) < MINGRID) */
 	if (x1 == x0 || y1 == y0)
 	{
 		if ((w->state&GRIDTOOSMALL) == 0)
@@ -1140,7 +1325,7 @@ INTSML us_drawfacet(NODEINST *ni, INTSML on, XARRAY prevtrans, INTSML layers, WI
  */
 INTSML us_drawnodeinst(NODEINST *ni, INTSML on, XARRAY prevtrans, INTSML layers, WINDOWPART *w)
 {
-	REGISTER INTSML i, j, res, displaytotal;
+	REGISTER INTSML i, j, res, displaytotal, dontdraw;
 	INTSML color, bits, moretodo, low, high;
 	XARRAY localtran, subrot, trans;
 	INTBIG bx, by, ux, uy, portcliplx, portcliphx, portcliply, portcliphy;
@@ -1188,54 +1373,69 @@ INTSML us_drawnodeinst(NODEINST *ni, INTSML on, XARRAY prevtrans, INTSML layers,
 	/* primitive nodeinst: ask the technology how to draw it */
 	if (np->primindex != 0)
 	{
-		downhierarchy(ni);
-		i = nodepolys(ni);
-		if ((us_state&NONOVERLAPPABLEDISPLAY) != 0)
+		/* do not draw displayable text that is in a subfacet and only appears in its facet */
+		dontdraw = 0;
+		if (np == gen_invispinprim && ni->parent != w->curnodeproto)
 		{
-			/* no overlappable display: just draw everything */
-			low = 0;   high = i;
-		} else
-		{
-			if ((np->userbits&NHASOPA) == 0) j = i; else
+			for(i=0; i<ni->numvar; i++)
 			{
-				j = (INTSML)((np->userbits&NFIRSTOPA) >> NFIRSTOPASH);
-				if (i < j) j = i;
+				var = &ni->firstvar[i];
+				if ((var->type&VDISPLAY) != 0 &&
+					(var->textdescript&VTINTERIOR) != 0) break;
 			}
-			if ((layers&1) != 0) low = 0; else low = j;
-			if ((layers&2) != 0) high = i; else high = j;
-			if (low > 0) moretodo |= 1;
-			if (high < i) moretodo |= 2;
+			if (i < ni->numvar) dontdraw = 1;
 		}
-
-		/* don't draw invisible pins to alternative output */
-		if (us_displayroutine != us_showpoly)
-			if (np == gen_invispinprim && low == 0) low++;
-
-		for(j=low; j<high; j++)
+		if (dontdraw == 0)
 		{
-			/* get description of this layer */
-			shapenodepoly(ni, j, poly);
+			downhierarchy(ni);
+			i = nodepolys(ni);
+			if ((us_state&NONOVERLAPPABLEDISPLAY) != 0)
+			{
+				/* no overlappable display: just draw everything */
+				low = 0;   high = i;
+			} else
+			{
+				if ((np->userbits&NHASOPA) == 0) j = i; else
+				{
+					j = (INTSML)((np->userbits&NFIRSTOPA) >> NFIRSTOPASH);
+					if (i < j) j = i;
+				}
+				if ((layers&1) != 0) low = 0; else low = j;
+				if ((layers&2) != 0) high = i; else high = j;
+				if (low > 0) moretodo |= 1;
+				if (high < i) moretodo |= 2;
+			}
 
-			/* ignore if this layer is not being displayed */
-			gra = poly->desc;
-			if ((gra->colstyle&INVISIBLE) != 0) continue;
+			/* don't draw invisible pins to alternative output */
+			if (us_displayroutine != us_showpoly)
+				if (np == gen_invispinprim && low == 0) low++;
 
-			/* ignore if no bits are to be drawn */
-			if ((on&gra->col) == 0 && on != 0) continue;
+			for(j=low; j<high; j++)
+			{
+				/* get description of this layer */
+				shapenodepoly(ni, j, poly);
 
-			/* draw the nodeinst */
-			xformpoly(poly, prevtrans);
+				/* ignore if this layer is not being displayed */
+				gra = poly->desc;
+				if ((gra->colstyle&INVISIBLE) != 0) continue;
 
-			/* save the color information and update it */
-			i = gra->col;
-			gra->col &= on;
+				/* ignore if no bits are to be drawn */
+				if ((on&gra->col) == 0 && on != 0) continue;
 
-			/* draw the nodeinst and restore the color */
-			if ((w->state&WINDOWTYPE) == DISP3DWINDOW) us_3dshowpoly(poly, w); else
-				(*us_displayroutine)(poly, w);
-			gra->col = i;
+				/* draw the nodeinst */
+				xformpoly(poly, prevtrans);
+
+				/* save the color information and update it */
+				i = gra->col;
+				gra->col &= on;
+
+				/* draw the nodeinst and restore the color */
+				if ((w->state&WINDOWTYPE) == DISP3DWINDOW) us_3dshowpoly(poly, w); else
+					(*us_displayroutine)(poly, w);
+				gra->col = i;
+			}
+			uphierarchy();
 		}
-		uphierarchy();
 	} else
 	{
 		/* draw a facet */
@@ -1299,7 +1499,7 @@ INTSML us_drawnodeinst(NODEINST *ni, INTSML on, XARRAY prevtrans, INTSML layers,
 					poly->xv[0] = (ni->lowx + ni->highx) / 2;
 					poly->yv[0] = (ni->lowy + ni->highy) / 2;
 				}
-				adjustdisoffset(ni->geom, poly, ni->textdescript);
+				adjustdisoffset((INTBIG)ni, VNODEINST, whattech(ni->proto), poly, ni->textdescript);
 				xformpoly(poly, prevtrans);
 				poly->desc = &us_facetgra;   us_facetgra.col = el_colfacettxt;
 				poly->string = describenodeproto(ni->proto);
@@ -1404,7 +1604,7 @@ INTSML us_drawnodeinst(NODEINST *ni, INTSML on, XARRAY prevtrans, INTSML layers,
 					}
 				}
 
-				/* see if there are displayable variables on the facet */
+				/* see if there are displayable variables on the facet instance */
 				displaytotal = tech_displayablenvars(ni);
 				for(i = 0; i < displaytotal; i++)
 				{
@@ -1413,6 +1613,11 @@ INTSML us_drawnodeinst(NODEINST *ni, INTSML on, XARRAY prevtrans, INTSML layers,
 					if ((w->state&WINDOWTYPE) == DISP3DWINDOW) us_3dshowpoly(poly, w); else
 						(*us_displayroutine)(poly, w);
 				}
+
+				/* 
+				 * draw facet variables on expanded facets
+				 * us_drawnodeprotovariables(ni->proto, prevtrans, w);
+				 */
 			}
 
 			/* search through facet */
@@ -1438,13 +1643,60 @@ INTSML us_drawnodeinst(NODEINST *ni, INTSML on, XARRAY prevtrans, INTSML layers,
 		if ((layers&2) == 0) moretodo |= 2; else
 		{
 			for(pe = ni->firstportexpinst; pe != NOPORTEXPINST; pe = pe->nextportexpinst)
+			{
 				us_writeprotoname(pe->exportproto, on, prevtrans, LAYERO,
 					(INTSML)(el_colfacettxt&on), w, 0, 0, 0, 0);
+				us_drawportprotovariables(pe->exportproto, on, prevtrans, w);
+			}
 		}
 	}
 	return(moretodo);
 }
 
+void us_drawnodeprotovariables(NODEPROTO *np, XARRAY trans, WINDOWPART *w)
+{
+	REGISTER INTBIG i, displaytotal;
+	static POLYGON *poly = NOPOLYGON;
+
+	/* get polygon */
+	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
+
+	displaytotal = tech_displayablefacetvars(np);
+	for(i = 0; i < displaytotal; i++)
+	{
+		(void)tech_filldisplayablefacetvar(np, poly);
+		xformpoly(poly, trans);
+		if ((w->state&WINDOWTYPE) == DISP3DWINDOW) us_3dshowpoly(poly, w); else
+			(*us_displayroutine)(poly, w);
+	}
+}
+
+void us_drawportprotovariables(PORTPROTO *pp, INTSML on, XARRAY trans, WINDOWPART *w)
+{
+	REGISTER INTBIG i, displaytotal;
+/*	INTBIG xc, yc; */
+	static POLYGON *poly = NOPOLYGON;
+
+	/* get polygon */
+	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
+
+	displaytotal = tech_displayableportvars(pp);
+/*	portposition(pp->subnodeinst, pp->subportproto, &xc, &yc); */
+	for(i = 0; i < displaytotal; i++)
+	{
+		(void)tech_filldisplayableportvar(pp, poly);
+		xformpoly(poly, trans);
+		if (on == 0)
+		{
+			us_arbit = *poly->desc;
+			us_arbit.col = 0;
+			poly->desc = &us_arbit;
+		}
+		if ((w->state&WINDOWTYPE) == DISP3DWINDOW) us_3dshowpoly(poly, w); else
+			(*us_displayroutine)(poly, w);
+	}
+}
+
 /*
  * routine to determine the color of port "pp" by combining the colors
  * of all arcs that can connect to that port into the integers
@@ -1657,8 +1909,12 @@ void us_writeprotoname(PORTPROTO *pp, INTSML on, XARRAY prevtrans, INTSML bits,
 
 	/* get the center position of the port */
 	shapetransportpoly(ni, rpp, poly, *t1);
-	getcenter(poly, &px, &py);
-	poly->xv[0] = px;   poly->yv[0] = py;   poly->count = 1;
+	if ((us_useroptions&PORTLABELS) == PORTSCROSS ||
+		(pp->textdescript&VTPOSITION) != VTPOSBOXED)
+	{
+		getcenter(poly, &px, &py);
+		poly->xv[0] = px;   poly->yv[0] = py;   poly->count = 1;
+	}
 
 	/* simple description if only drawing ports as crosses */
 	if ((us_useroptions&PORTLABELS) == PORTSCROSS) poly->style = CROSS; else
@@ -1696,9 +1952,8 @@ void us_writeprotoname(PORTPROTO *pp, INTSML on, XARRAY prevtrans, INTSML bits,
 			}
 		}
 
-		/* see if node rotation affects port position */
-		if (poly->style != TEXTCENT)
-			poly->style = rotatelabel(poly->style, prevtrans);
+		/* acount for node rotation, which port position */
+		poly->style = rotatelabel(poly->style, prevtrans);
 
 		/* clip the port text to the facet bounds, if requested */
 		if (portcliplx != portcliphx)
@@ -1707,8 +1962,8 @@ void us_writeprotoname(PORTPROTO *pp, INTSML on, XARRAY prevtrans, INTSML bits,
 			sy = applyyscale(w, py-w->screenly) + w->usely;
 			if (sx < portcliplx || sx > portcliphx || sy < portcliply || sy > portcliphy)
 				return;
-			us_settextsize(w, poly->font);
-			us_textsize(w, poly->string, &wid, &hei);   wid++;   hei++;
+			screensettextsize(w, poly->font);
+			screengettextsize(w, poly->string, &wid, &hei);   wid++;   hei++;
 			switch (poly->style)
 			{
 				case TEXTCENT:
@@ -1943,6 +2198,7 @@ INTSML us_drawnodeinstpeek(NODEINST *ni, XARRAY prevtrans, INTSML layers)
 {
 	REGISTER INTSML i, j;
 	INTSML moretodo, low, high;
+	REGISTER VARIABLE *var;
 	static POLYGON *poly = NOPOLYGON;
 	REGISTER GRAPHICS *gra;
 	REGISTER NODEPROTO *np;
@@ -1950,8 +2206,18 @@ INTSML us_drawnodeinstpeek(NODEINST *ni, XARRAY prevtrans, INTSML layers)
 	/* get polygon */
 	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
 
-	/* determine which polygons to draw */
 	np = ni->proto;
+	if (np == gen_invispinprim && ni->parent != us_peekwindow->curnodeproto)
+	{
+		for(i=0; i<ni->numvar; i++)
+		{
+			var = &ni->firstvar[i];
+			if ((var->type&VDISPLAY) != 0 &&
+				(var->textdescript&VTINTERIOR) != 0) return(0);
+		}
+	}
+
+	/* determine which polygons to draw */
 	i = nodepolys(ni);
 	moretodo = 0;
 	if ((us_state&NONOVERLAPPABLEDISPLAY) != 0)
