@@ -158,7 +158,7 @@ WINDOWPART *us_editemacsmakeeditor(WINDOWPART *oriwin, char *header, INTSML *cha
 	win = oriwin;
 	if (win == NOWINDOWPART)
 	{
-		us_getpaletteparameters(&swid, &shei, &pwid);
+		getpaletteparameters(&swid, &shei, &pwid);
 		if (el_curwindowpart == NOWINDOWPART)
 		{
 			x = swid / 2;
@@ -206,8 +206,8 @@ WINDOWPART *us_editemacsmakeeditor(WINDOWPART *oriwin, char *header, INTSML *cha
 	var = getval((INTBIG)us_aid, VAID, VINTEGER, "USER_textedit_font");
 	if (var != NOVARIABLE && var->addr >= TXT4P && var->addr <= TXT20P)
 		us_editemacsfont = (INTSML)var->addr;
-	us_settextsize(win, us_editemacsfont);
-	us_textsize(win, "X", &us_twid, &us_thei);
+	screensettextsize(win, us_editemacsfont);
+	screengettextsize(win, "X", &us_twid, &us_thei);
 
 	/* create a new editor object */
 	e = us_alloceditor();
@@ -269,7 +269,7 @@ WINDOWPART *us_editemacsmakeeditor(WINDOWPART *oriwin, char *header, INTSML *cha
 	} else
 	{
 		/* finish initializing window if it is a new popup one */
-		e->savedbox = us_savebox(win, (INTSML)maxi(e->offx-2,0), (INTSML)(e->offx+e->swid+2),
+		e->savedbox = screensavebox(win, (INTSML)maxi(e->offx-2,0), (INTSML)(e->offx+e->swid+2),
 			(INTSML)maxi(e->revy-e->shei-2,0), (INTSML)(e->revy+us_thei*HEADERLINES+1));
 		endobjectchange((INTBIG)us_aid, VAID);
 	}
@@ -515,7 +515,7 @@ void us_editemacseditorterm(WINDOWPART *win)
 
 	if ((win->state&WINDOWTYPE) == POPTEXTWINDOW)
 	{
-		(void)us_restorebox(e->savedbox, 0);
+		(void)screenrestorebox(e->savedbox, 0);
 	}
 }
 
@@ -1589,7 +1589,7 @@ void us_editemacssetheader(WINDOWPART *win, char *header)
 
 	e = win->editor;
 	if (e == NOEDITOR) return;
-	us_drawbox(win, (INTSML)(e->offx-1), (INTSML)(e->offx+e->swid), (INTSML)(e->revy+1),
+	screendrawbox(win, (INTSML)(e->offx-1), (INTSML)(e->offx+e->swid), (INTSML)(e->revy+1),
 		(INTSML)(e->revy+us_thei*HEADERLINES), &us_ebox);
 	if ((INTBIG)strlen(header) > e->screenchars)
 	{
@@ -1602,13 +1602,13 @@ void us_editemacssetheader(WINDOWPART *win, char *header)
 			us_editemacstext(win, "Type ^Xd when done editing", 0, -1);
 
 	if (save != 0) header[e->screenchars] = (char)save;
-	us_invertbox(win, (INTSML)(e->offx-1), (INTSML)(e->offx+e->swid), (INTSML)(e->revy+1), (INTSML)(e->revy+us_thei*HEADERLINES));
+	screeninvertbox(win, (INTSML)(e->offx-1), (INTSML)(e->offx+e->swid), (INTSML)(e->revy+1), (INTSML)(e->revy+us_thei*HEADERLINES));
 	us_menufigs.col = el_colmengly;
-	us_drawline(win, (INTSML)(e->offx-1), e->revy, (INTSML)(e->offx-1), (INTSML)(e->revy-e->shei-1),
+	screendrawline(win, (INTSML)(e->offx-1), e->revy, (INTSML)(e->offx-1), (INTSML)(e->revy-e->shei-1),
 		&us_menufigs, 0);
-	us_drawline(win, (INTSML)(e->offx+e->swid+1), e->revy, (INTSML)(e->offx+e->swid+1),
+	screendrawline(win, (INTSML)(e->offx+e->swid+1), e->revy, (INTSML)(e->offx+e->swid+1),
 		(INTSML)(e->revy-e->shei-1), &us_menufigs, 0);
-	us_drawline(win, (INTSML)(e->offx-1), (INTSML)(e->revy-e->shei-1), (INTSML)(e->offx+e->swid+1),
+	screendrawline(win, (INTSML)(e->offx-1), (INTSML)(e->revy-e->shei-1), (INTSML)(e->offx+e->swid+1),
 		(INTSML)(e->revy-e->shei-1), &us_menufigs, 0);
 }
 
@@ -1789,7 +1789,7 @@ void us_editemacsmovebox(WINDOWPART *win, INTSML dx, INTSML dy, INTSML wid, INTS
 
 	e = win->editor;
 	if (e == NOEDITOR) return;
-	us_movebox(win, (INTSML)(sx*us_twid+e->offx), (INTSML)(e->revy-(sy*us_thei+hei-1)), wid, hei,
+	screenmovebox(win, (INTSML)(sx*us_twid+e->offx), (INTSML)(e->revy-(sy*us_thei+hei-1)), wid, hei,
 		(INTSML)(dx*us_twid+e->offx), (INTSML)(e->revy-(dy*us_thei+hei-1)));
 }
 
@@ -1805,7 +1805,7 @@ void us_editemacsclearbox(WINDOWPART *win, INTSML dx, INTSML dy, INTSML wid, INT
 
 	dx = (dx * us_twid) + e->offx;
 	dy *= us_thei;
-	us_drawbox(win, dx, (INTSML)(dx+wid-1), (INTSML)(e->revy-(dy+hei-1)), (INTSML)(e->revy-dy),
+	screendrawbox(win, dx, (INTSML)(dx+wid-1), (INTSML)(e->revy-(dy+hei-1)), (INTSML)(e->revy-dy),
 		&us_ebox);
 }
 
@@ -1821,7 +1821,7 @@ void us_editemacsinvertbox(WINDOWPART *win, INTSML dx, INTSML dy, INTSML wid, IN
 
 	dx = (dx * us_twid) + e->offx;
 	dy *= us_thei;
-	us_invertbox(win, dx, (INTSML)(dx+wid-1), (INTSML)(e->revy-(dy+hei-1)), (INTSML)(e->revy-dy));
+	screeninvertbox(win, dx, (INTSML)(dx+wid-1), (INTSML)(e->revy-(dy+hei-1)), (INTSML)(e->revy-dy));
 }
 
 /*
@@ -1834,7 +1834,7 @@ void us_editemacstext(WINDOWPART *win, char *str, INTSML x, INTSML y)
 	e = win->editor;
 	if (e == NOEDITOR) return;
 
-	us_settextsize(win, us_editemacsfont);
+	screensettextsize(win, us_editemacsfont);
 	us_menutext.col = el_colmentxt;
-	us_puttext(win, (INTSML)(x*us_twid + e->offx), (INTSML)(e->revy-(y+1)*us_thei), str, &us_menutext);
+	screendrawtext(win, (INTSML)(x*us_twid + e->offx), (INTSML)(e->revy-(y+1)*us_thei), str, &us_menutext);
 }
