@@ -38,21 +38,9 @@
 #define	DELETECHAR    010				/* the Delete character */
 #define	BSCHAR       0177				/* the Backspace character */
 #define	ESCCHAR       033				/* the Escape character */
-#define	LEFTARROWKEY  034				/* the Left Arrow key */
-#define	RIGHTARROWKEY 035				/* the Right Arrow key */
-#define	UPARROWKEY    036				/* the Up Arrow key */
-#define	DOWNARROWKEY  037				/* the Down Arrow key */
-
-/* the four scroller arrows */
-#define	UPARROW      0
-#define	DOWNARROW    1
-#define	LEFTARROW    2
-#define	RIGHTARROW   3
 
 #define	THUMBSIZE   16					/* width of the thumb area in scroll slider */
-#define	THUMBWIDTH  30					/* width of thumb-dragging corridor */
 #define	HEADERLINES  2					/* number of lines of text at top of editor */
-#define	SCROLLWIDTH 15					/* width of scroll bars */
 #define	LEFTINDENT   4					/* space between left edge and text */
 #define	DEFAULTWID   1					/* default number of characters on a line */
 
@@ -88,19 +76,16 @@ INTSML us_editpacdownarrow(INTBIG, INTBIG);
 INTSML us_editpacuparrow(INTBIG, INTBIG);
 INTSML us_editpacdownpage(INTBIG, INTBIG);
 INTSML us_editpacuppage(INTBIG, INTBIG);
-INTSML us_editpacvscroll(INTBIG, INTBIG);
 INTSML us_editpacrightarrow(INTBIG, INTBIG);
 INTSML us_editpacleftarrow(INTBIG, INTBIG);
 INTSML us_editpacrightpage(INTBIG, INTBIG);
 INTSML us_editpacleftpage(INTBIG, INTBIG);
-INTSML us_editpachscroll(INTBIG, INTBIG);
 INTSML us_editpacimplementchar(WINDOWPART*, INTSML, INTSML);
 void us_editpacremoveselection(WINDOWPART*);
 void us_editpacredrawscreen(WINDOWPART*);
 void us_drawsliders(WINDOWPART*);
 void us_editpacdrawhscroll(WINDOWPART*);
 void us_editpacdrawvscroll(WINDOWPART*);
-void us_drawarrow(WINDOWPART*, INTSML, INTSML);
 void us_editpaccleanupline(WINDOWPART*, INTBIG);
 void us_editpacredrawlines(WINDOWPART*);
 void us_editpacsetheader(WINDOWPART*, char*);
@@ -151,7 +136,7 @@ WINDOWPART *us_editpacmakeeditor(WINDOWPART *oriwin, char *header, INTSML *chars
 	win = oriwin;
 	if (win == NOWINDOWPART)
 	{
-		us_getpaletteparameters(&swid, &shei, &pwid);
+		getpaletteparameters(&swid, &shei, &pwid);
 		if (el_curwindowpart == NOWINDOWPART)
 		{
 			x = swid / 2;
@@ -166,9 +151,9 @@ WINDOWPART *us_editpacmakeeditor(WINDOWPART *oriwin, char *header, INTSML *chars
 		/* create a window that covers the popup */
 		startobjectchange((INTBIG)us_aid, VAID);
 		win = newwindowpart("popup", el_curwindowpart);
-		us_settextsize(win, us_editpacfont);
-		us_textsize(win, "X", &us_twid, &us_thei);
-		xs = 400;   ys = us_thei*(HEADERLINES+1)+SCROLLWIDTH+4;
+		screensettextsize(win, us_editpacfont);
+		screengettextsize(win, "X", &us_twid, &us_thei);
+		xs = 400;   ys = us_thei*(HEADERLINES+1)+DISPLAYSLIDERSIZE+4;
 		win->uselx = (INTSML)maxi(x-xs/2, 0);
 		win->usehx = (INTSML)mini(win->uselx+xs, swid-1);
 		win->usely = (INTSML)maxi(y-ys, 0);
@@ -177,7 +162,7 @@ WINDOWPART *us_editpacmakeeditor(WINDOWPART *oriwin, char *header, INTSML *chars
 	} else
 	{
 		/* make sure there is room for at least one line */
-		if (win->usehy - win->usely - us_thei*HEADERLINES - 1 - SCROLLWIDTH < us_thei)
+		if (win->usehy - win->usely - us_thei*HEADERLINES - 1 - DISPLAYSLIDERSIZE < us_thei)
 			return(NOWINDOWPART);
 
 		if ((win->state&WINDOWTYPE) == DISPWINDOW)
@@ -185,9 +170,14 @@ WINDOWPART *us_editpacmakeeditor(WINDOWPART *oriwin, char *header, INTSML *chars
 			win->usehx += DISPLAYSLIDERSIZE;
 			win->usely -= DISPLAYSLIDERSIZE;
 		}
+		if ((win->state&WINDOWTYPE) == WAVEFORMWINDOW)
+		{
+			win->uselx -= DISPLAYSLIDERSIZE;
+			win->usely -= DISPLAYSLIDERSIZE;
+		}
 		win->state = (win->state & ~(WINDOWTYPE|WINDOWSIMULATING)) | TEXTWINDOW;
-		us_settextsize(win, us_editpacfont);
-		us_textsize(win, "X", &us_twid, &us_thei);
+		screensettextsize(win, us_editpacfont);
+		screengettextsize(win, "X", &us_twid, &us_thei);
 	}
 
 	win->curnodeproto = NONODEPROTO;
@@ -255,7 +245,7 @@ WINDOWPART *us_editpacmakeeditor(WINDOWPART *oriwin, char *header, INTSML *chars
 	} else
 	{
 		/* finish initializing popup text window */
-		e->savedbox = us_savebox(win, win->uselx, win->usehx, win->usely, win->usehy);
+		e->savedbox = screensavebox(win, win->uselx, win->usehx, win->usely, win->usehy);
 		endobjectchange((INTBIG)us_aid, VAID);
 	}
 	return(win);
@@ -318,7 +308,7 @@ void us_editpacaddline(WINDOWPART *win, INTBIG lindex, char *str)
 
 	e = win->editor;
 	if (e == NOEDITOR) return;
-	if ((e->state&EGRAPHICSOFF) == 0) us_settextsize(win, us_editpacfont);
+	if ((e->state&EGRAPHICSOFF) == 0) screensettextsize(win, us_editpacfont);
 
 	/* send out any pending changes */
 	us_editpacshipchanges(win);
@@ -355,7 +345,7 @@ void us_editpacreplaceline(WINDOWPART *win, INTBIG lindex, char *str)
 
 	e = win->editor;
 	if (e == NOEDITOR) return;
-	if ((e->state&EGRAPHICSOFF) == 0) us_settextsize(win, us_editpacfont);
+	if ((e->state&EGRAPHICSOFF) == 0) screensettextsize(win, us_editpacfont);
 
 	/* send out any pending changes */
 	us_editpacshipchanges(win);
@@ -391,7 +381,7 @@ void us_editpacdeleteline(WINDOWPART *win, INTBIG lindex)
 
 	e = win->editor;
 	if (e == NOEDITOR) return;
-	if ((e->state&EGRAPHICSOFF) == 0) us_settextsize(win, us_editpacfont);
+	if ((e->state&EGRAPHICSOFF) == 0) screensettextsize(win, us_editpacfont);
 
 	/* send out any pending changes */
 	us_editpacshipchanges(win);
@@ -426,7 +416,7 @@ void us_editpachighlightline(WINDOWPART *win, INTBIG lindex)
 
 	e = win->editor;
 	if (e == NOEDITOR) return;
-	us_settextsize(win, us_editpacfont);
+	screensettextsize(win, us_editpacfont);
 	us_editpacinvertselection(win);
 	e->curline = e->endline = lindex;
 	e->curchar = 0;   e->endchar = e->maxchars[lindex];
@@ -465,7 +455,7 @@ void us_editpacresumegraphics(WINDOWPART *win)
 	e = win->editor;
 	if (e == NOEDITOR) return;
 	e->state &= ~EGRAPHICSOFF;
-	us_settextsize(win, us_editpacfont);
+	screensettextsize(win, us_editpacfont);
 	us_editpacredrawscreen(win);
 }
 
@@ -590,7 +580,7 @@ void us_editpacreadtextfile(WINDOWPART *win, char *file)
 
 	/* restore the display */
 	e->state &= ~EGRAPHICSOFF;
-	us_settextsize(win, us_editpacfont);
+	screensettextsize(win, us_editpacfont);
 	e->curline = e->curchar = 0;
 	e->endline = e->endchar = 0;
 	us_editpacbeginwork(win);
@@ -612,7 +602,7 @@ void us_editpaceditorterm(WINDOWPART *win)
 	/* restore window state */
 	if ((win->state&WINDOWTYPE) == POPTEXTWINDOW)
 	{
-		(void)us_restorebox(e->savedbox, 0);
+		(void)screenrestorebox(e->savedbox, 0);
 	}
 }
 
@@ -647,7 +637,7 @@ INTSML us_editpacgotchar(WINDOWPART *win, INTSML i)
 
 	e = win->editor;
 	if (e == NOEDITOR) return(1);
-	us_settextsize(win, us_editpacfont);
+	screensettextsize(win, us_editpacfont);
 	us_editpacinvertselection(win);
 	if (us_editpacimplementchar(win, i, 1) != 0) return(1);
 	us_editpacinvertselection(win);
@@ -666,7 +656,7 @@ void us_editpaccut(WINDOWPART *win)
 	us_editpaccopy(win);
 
 	/* remove the selection from the edit buffer */
-	us_settextsize(win, us_editpacfont);
+	screensettextsize(win, us_editpacfont);
 	us_editpacinvertselection(win);
 	(void)us_editpacimplementchar(win, DELETECHAR, 1);
 	us_editpacinvertselection(win);
@@ -725,7 +715,7 @@ void us_editpacpaste(WINDOWPART *win)
 
 	e = win->editor;
 	if (e == NOEDITOR) return;
-	us_settextsize(win, us_editpacfont);
+	screensettextsize(win, us_editpacfont);
 
 	us_editpacinvertselection(win);
 	for(i=0; us_editpacbuffer[i] != 0; i++)
@@ -752,7 +742,7 @@ void us_editpacundo(WINDOWPART *win)
 	(void)allocstring(&savebuffer, us_editpacundobuffer, el_tempcluster);
 
 	/* replace what was typed */
-	us_settextsize(win, us_editpacfont);
+	screensettextsize(win, us_editpacfont);
 	us_editpacinvertselection(win);
 	e->curline = us_editpacundonewcurline;    e->curchar = us_editpacundonewcurchar;
 	e->endline = us_editpacundonewendline;    e->endchar = us_editpacundonewendchar;
@@ -825,7 +815,7 @@ void us_editpacsearch(WINDOWPART *win, char *str, INTSML fromtop)
 	}
 
 	/* string found */
-	us_settextsize(win, us_editpacfont);
+	screensettextsize(win, us_editpacfont);
 	us_editpacdonetyping(e);
 	us_editpacinvertselection(win);
 	e->curline = lne;   e->curchar = chr;
@@ -862,7 +852,7 @@ void us_editpacredraweditor(WINDOWPART *win)
 
 	e = win->editor;
 	if (e == NOEDITOR) return;
-	us_settextsize(win, us_editpacfont);
+	screensettextsize(win, us_editpacfont);
 
 	/* compute window extents */
 	us_editpaccomputebounds(win, e);
@@ -880,39 +870,35 @@ void us_editpacgotbutton(WINDOWPART *win, INTSML but, INTSML x, INTSML y)
 
 	e = win->editor;
 	if (e == NOEDITOR) return;
-	us_settextsize(win, us_editpacfont);
+	screensettextsize(win, us_editpacfont);
 	us_editpaccurrentwin = win;
 
 	if ((win->state&WINDOWTYPE) != POPTEXTWINDOW && x > e->offx + e->swid)
 	{
 		/* cursor in vertical slider */
 		if (e->linecount <= e->screenlines) return;
-		if (y < e->revy - e->shei + SCROLLWIDTH)
+		if (y < e->revy - e->shei + DISPLAYSLIDERSIZE)
 		{
 			/* the down arrow */
-			us_drawarrow(win, DOWNARROW, 1);
 			trackcursor(0, us_nullup, us_nullvoid, us_editpacdownarrow,
 				us_nullchar, us_nullvoid, TRACKNORMAL);
-			us_drawarrow(win, DOWNARROW, 0);
 			return;
 		}
-		if (y > e->revy - SCROLLWIDTH)
+		if (y > e->revy - DISPLAYSLIDERSIZE)
 		{
 			/* the up arrow */
-			us_drawarrow(win, UPARROW, 1);
 			trackcursor(0, us_nullup, us_nullvoid, us_editpacuparrow,
 				us_nullchar, us_nullvoid, TRACKNORMAL);
-			us_drawarrow(win, UPARROW, 0);
 			return;
 		}
-		if (y < e->vthumbpos-THUMBSIZE/2 && y >= e->revy - e->shei + SCROLLWIDTH)
+		if (y < e->vthumbpos-THUMBSIZE/2 && y >= e->revy - e->shei + DISPLAYSLIDERSIZE)
 		{
 			/* scroll down one page */
 			trackcursor(0, us_nullup, us_nullvoid, us_editpacdownpage,
 				us_nullchar, us_nullvoid, TRACKNORMAL);
 			return;
 		}
-		if (y > e->vthumbpos+THUMBSIZE/2 && y <= e->revy - SCROLLWIDTH)
+		if (y > e->vthumbpos+THUMBSIZE/2 && y <= e->revy - DISPLAYSLIDERSIZE)
 		{
 			/* scroll up one page */
 			trackcursor(0, us_nullup, us_nullvoid, us_editpacuppage,
@@ -922,18 +908,11 @@ void us_editpacgotbutton(WINDOWPART *win, INTSML but, INTSML x, INTSML y)
 		if (y >= e->vthumbpos-THUMBSIZE/2 && y <= e->vthumbpos+THUMBSIZE/2)
 		{
 			/* drag slider appropriately */
-			us_editpacinvertrectframe(win, (INTSML)(e->offx+e->swid+1), (INTSML)(e->vthumbpos-THUMBSIZE/2));
-			us_editpacoffset = e->vthumbpos - y;
-			trackcursor(0, us_nullup, us_nullvoid, us_editpacvscroll,
-				us_nullchar, us_nullvoid, TRACKNORMAL);
-			us_editpacinvertrectframe(win, (INTSML)(e->offx+e->swid+1), (INTSML)(e->vthumbpos-THUMBSIZE/2));
-			i = e->shei - SCROLLWIDTH*2 - THUMBSIZE;
-			i = ((e->revy - SCROLLWIDTH - e->vthumbpos - THUMBSIZE/2) *
-				(e->linecount - e->screenlines) + i/2) / i;
-			if (i < 0) i = 0;
-			if (e->linecount-i <= e->screenlines) i = e->linecount - e->screenlines;
-			if (i == e->firstline) return;
-			e->firstline = i;
+			us_vthumbbegin(y, win, win->usehx-DISPLAYSLIDERSIZE, win->usely+DISPLAYSLIDERSIZE,
+				win->usehy-us_thei*HEADERLINES, e->linecount, 0);
+			trackcursor(0, us_nullup, us_nullvoid, us_vthumbdown, us_nullchar,
+				us_evthumbdone, TRACKNORMAL);
+			e->firstline = (INTSML)us_evthumbend();
 			us_editpacredrawlines(win);
 			us_editpacdrawvscroll(win);
 			return;
@@ -943,32 +922,28 @@ void us_editpacgotbutton(WINDOWPART *win, INTSML but, INTSML x, INTSML y)
 	if (y <= e->revy - e->shei)
 	{
 		/* cursor in horizontal slider */
-		if (x > e->offx + e->swid - SCROLLWIDTH)
+		if (x > e->offx + e->swid - DISPLAYSLIDERSIZE)
 		{
 			/* the right arrow */
-			us_drawarrow(win, RIGHTARROW, 1);
 			trackcursor(0, us_nullup, us_nullvoid, us_editpacrightarrow,
 				us_nullchar, us_nullvoid, TRACKNORMAL);
-			us_drawarrow(win, RIGHTARROW, 0);
 			return;
 		}
-		if (x < e->offx + SCROLLWIDTH - LEFTINDENT)
+		if (x < e->offx + DISPLAYSLIDERSIZE - LEFTINDENT)
 		{
 			/* the left arrow */
-			us_drawarrow(win, LEFTARROW, 1);
 			trackcursor(0, us_nullup, us_nullvoid, us_editpacleftarrow,
 				us_nullchar, us_nullvoid, TRACKNORMAL);
-			us_drawarrow(win, LEFTARROW, 0);
 			return;
 		}
-		if (x > e->hthumbpos+THUMBSIZE/2 && x <= e->offx + e->swid - SCROLLWIDTH)
+		if (x > e->hthumbpos+THUMBSIZE/2 && x <= e->offx + e->swid - DISPLAYSLIDERSIZE)
 		{
 			/* scroll right one page */
 			trackcursor(0, us_nullup, us_nullvoid, us_editpacrightpage,
 				us_nullchar, us_nullvoid, TRACKNORMAL);
 			return;
 		}
-		if (x < e->hthumbpos-THUMBSIZE/2 && x >= e->offx + SCROLLWIDTH - LEFTINDENT)
+		if (x < e->hthumbpos-THUMBSIZE/2 && x >= e->offx + DISPLAYSLIDERSIZE - LEFTINDENT)
 		{
 			/* scroll left one page */
 			trackcursor(0, us_nullup, us_nullvoid, us_editpacleftpage,
@@ -978,15 +953,12 @@ void us_editpacgotbutton(WINDOWPART *win, INTSML but, INTSML x, INTSML y)
 		if (x >= e->hthumbpos-THUMBSIZE/2 && x <= e->hthumbpos+THUMBSIZE/2)
 		{
 			/* drag slider appropriately */
-			us_editpacinvertrectframe(win, (INTSML)(e->hthumbpos-THUMBSIZE/2),
-				(INTSML)(e->revy - e->shei - SCROLLWIDTH - 2));
-			us_editpacoffset = e->hthumbpos - x;
-			trackcursor(0, us_nullup, us_nullvoid, us_editpachscroll,
-				us_nullchar, us_nullvoid, TRACKNORMAL);
-			us_editpacinvertrectframe(win, (INTSML)(e->hthumbpos-THUMBSIZE/2),
-				(INTSML)(e->revy - e->shei - SCROLLWIDTH - 2));
-			i = e->swid - SCROLLWIDTH*2 + LEFTINDENT - THUMBSIZE;
-			i = ((e->hthumbpos - (e->offx+SCROLLWIDTH-LEFTINDENT+THUMBSIZE/2)) *
+			us_hthumbbegin(x, win, win->usely+DISPLAYSLIDERSIZE, win->uselx,
+				win->usehx-DISPLAYSLIDERSIZE);
+			trackcursor(0, us_nullup, us_nullvoid, us_hthumbdown, us_nullchar,
+				us_ehthumbdone, TRACKNORMAL);
+			i = e->swid - DISPLAYSLIDERSIZE*2 + LEFTINDENT - THUMBSIZE;
+			i = ((us_ehthumbend() - (e->offx+DISPLAYSLIDERSIZE-LEFTINDENT+THUMBSIZE/2)) *
 				e->screenchars*10 + i/2) / i;
 			if (i < 0) i = 0;
 			if (i == e->horizfactor) return;
@@ -1165,20 +1137,20 @@ INTSML us_editpacdownarrow(INTBIG x, INTBIG y)
 	REGISTER INTBIG i;
 
 	e = us_editpaccurrentwin->editor;
-	if (x <= e->offx + e->swid || x > e->offx + e->swid + SCROLLWIDTH) return(0);
-	if (y >= e->revy - e->shei + SCROLLWIDTH || y < e->revy - e->shei) return(0);
+	if (x <= e->offx + e->swid || x > e->offx + e->swid + DISPLAYSLIDERSIZE) return(0);
+	if (y >= e->revy - e->shei + DISPLAYSLIDERSIZE || y < e->revy - e->shei) return(0);
 	if (e->linecount-e->firstline <= e->screenlines) return(1);
 
 	us_editpacinvertselection(us_editpaccurrentwin);
 	e->firstline++;
 
 	/* shift the window contents up by 1 line (us_thei) */
-	us_movebox(us_editpaccurrentwin, (INTSML)(e->offx-LEFTINDENT), (INTSML)(e->revy-e->shei),
+	screenmovebox(us_editpaccurrentwin, (INTSML)(e->offx-LEFTINDENT), (INTSML)(e->revy-e->shei),
 		(INTSML)(e->swid+LEFTINDENT), (INTSML)(e->shei-us_thei), (INTSML)(e->offx-LEFTINDENT),
 			(INTSML)(e->revy-e->shei+us_thei));
 
 	/* fill in a new last line */
-	us_drawbox(us_editpaccurrentwin, (INTSML)(e->offx-LEFTINDENT), (INTSML)(e->offx+e->swid),
+	screendrawbox(us_editpaccurrentwin, (INTSML)(e->offx-LEFTINDENT), (INTSML)(e->offx+e->swid),
 		(INTSML)(e->revy-e->shei), (INTSML)(e->revy-e->shei+us_thei), &us_ebox);
 	i = e->firstline + e->screenlines - 1;
 	if (i < e->linecount)
@@ -1193,19 +1165,19 @@ INTSML us_editpacuparrow(INTBIG x, INTBIG y)
 	REGISTER EDITOR *e;
 
 	e = us_editpaccurrentwin->editor;
-	if (x <= e->offx + e->swid || x > e->offx + e->swid + SCROLLWIDTH) return(0);
-	if (y <= e->revy - SCROLLWIDTH || y > e->revy) return(0);
+	if (x <= e->offx + e->swid || x > e->offx + e->swid + DISPLAYSLIDERSIZE) return(0);
+	if (y <= e->revy - DISPLAYSLIDERSIZE || y > e->revy) return(0);
 	if (e->firstline <= 0) return(1);
 
 	us_editpacinvertselection(us_editpaccurrentwin);
 	e->firstline--;
 
 	/* shift the window contents down by 1 line (us_thei) */
-	us_movebox(us_editpaccurrentwin, (INTSML)(e->offx-LEFTINDENT), (INTSML)(e->revy-e->shei+us_thei), (INTSML)(e->swid+LEFTINDENT),
+	screenmovebox(us_editpaccurrentwin, (INTSML)(e->offx-LEFTINDENT), (INTSML)(e->revy-e->shei+us_thei), (INTSML)(e->swid+LEFTINDENT),
 		(INTSML)(e->shei-us_thei), (INTSML)(e->offx-LEFTINDENT), (INTSML)(e->revy-e->shei));
 
 	/* fill in a new first line */
-	us_drawbox(us_editpaccurrentwin, (INTSML)(e->offx-LEFTINDENT), (INTSML)(e->offx+e->swid),
+	screendrawbox(us_editpaccurrentwin, (INTSML)(e->offx-LEFTINDENT), (INTSML)(e->offx+e->swid),
 		(INTSML)(e->revy-us_thei), e->revy, &us_ebox);
 	us_editpactext(us_editpaccurrentwin, e->textarray[e->firstline], 0, 0);
 	us_editpacinvertselection(us_editpaccurrentwin);
@@ -1218,8 +1190,8 @@ INTSML us_editpacdownpage(INTBIG x, INTBIG y)
 	REGISTER EDITOR *e;
 
 	e = us_editpaccurrentwin->editor;
-	if (x <= e->offx + e->swid || x > e->offx + e->swid + SCROLLWIDTH) return(0);
-	if (y >= e->vthumbpos-THUMBSIZE/2 || y < e->revy - e->shei + SCROLLWIDTH) return(0);
+	if (x <= e->offx + e->swid || x > e->offx + e->swid + DISPLAYSLIDERSIZE) return(0);
+	if (y >= e->vthumbpos-THUMBSIZE/2 || y < e->revy - e->shei + DISPLAYSLIDERSIZE) return(0);
 	if (e->linecount-e->firstline <= e->screenlines) return(1);
 	e->firstline += e->screenlines-1;
 	if (e->linecount-e->firstline <= e->screenlines)
@@ -1234,8 +1206,8 @@ INTSML us_editpacuppage(INTBIG x, INTBIG y)
 	REGISTER EDITOR *e;
 
 	e = us_editpaccurrentwin->editor;
-	if (x <= e->offx + e->swid || x > e->offx + e->swid + SCROLLWIDTH) return(0);
-	if (y <= e->vthumbpos+THUMBSIZE/2 || y > e->revy - SCROLLWIDTH) return(0);
+	if (x <= e->offx + e->swid || x > e->offx + e->swid + DISPLAYSLIDERSIZE) return(0);
+	if (y <= e->vthumbpos+THUMBSIZE/2 || y > e->revy - DISPLAYSLIDERSIZE) return(0);
 	if (e->firstline <= 0) return(1);
 	e->firstline -= e->screenlines-1;
 	if (e->firstline < 0) e->firstline = 0;
@@ -1244,35 +1216,13 @@ INTSML us_editpacuppage(INTBIG x, INTBIG y)
 	return(0);
 }
 
-INTSML us_editpacvscroll(INTBIG x, INTBIG y)
-{
-	REGISTER EDITOR *e;
-	REGISTER INTSML l;
-
-	e = us_editpaccurrentwin->editor;
-	if (x <= e->offx + e->swid - THUMBWIDTH || x > e->offx + e->swid + SCROLLWIDTH + THUMBWIDTH)
-		return(0);
-	l = e->vthumbpos;
-	e->vthumbpos = y + us_editpacoffset;
-	if (e->vthumbpos < e->revy-e->shei+SCROLLWIDTH+THUMBSIZE/2)
-		e->vthumbpos = e->revy-e->shei+SCROLLWIDTH+THUMBSIZE/2;
-	if (e->vthumbpos > e->revy-SCROLLWIDTH-THUMBSIZE/2)
-		e->vthumbpos = e->revy-SCROLLWIDTH-THUMBSIZE/2;
-	if (e->vthumbpos == l) return(0);
-	us_editpacinvertrectframe(us_editpaccurrentwin, (INTSML)(e->offx+e->swid+1),
-		(INTSML)(l-THUMBSIZE/2));
-	us_editpacinvertrectframe(us_editpaccurrentwin, (INTSML)(e->offx+e->swid+1),
-		(INTSML)(e->vthumbpos-THUMBSIZE/2));
-	return(0);
-}
-
 INTSML us_editpacrightarrow(INTBIG x, INTBIG y)
 {
 	REGISTER EDITOR *e;
 
 	e = us_editpaccurrentwin->editor;
-	if (y > e->revy - e->shei || y < e->revy - e->shei - SCROLLWIDTH) return(0);
-	if (x <= e->offx + e->swid - SCROLLWIDTH || x > e->offx + e->swid) return(0);
+	if (y > e->revy - e->shei || y < e->revy - e->shei - DISPLAYSLIDERSIZE) return(0);
+	if (x <= e->offx + e->swid - DISPLAYSLIDERSIZE || x > e->offx + e->swid) return(0);
 	if (e->horizfactor >= e->screenchars*10) return(1);
 	e->horizfactor++;
 	us_editpacredrawlines(us_editpaccurrentwin);
@@ -1285,8 +1235,8 @@ INTSML us_editpacleftarrow(INTBIG x, INTBIG y)
 	REGISTER EDITOR *e;
 
 	e = us_editpaccurrentwin->editor;
-	if (y > e->revy - e->shei || y < e->revy - e->shei - SCROLLWIDTH) return(0);
-	if (x >= e->offx + SCROLLWIDTH - LEFTINDENT || x < e->offx - LEFTINDENT) return(0);
+	if (y > e->revy - e->shei || y < e->revy - e->shei - DISPLAYSLIDERSIZE) return(0);
+	if (x >= e->offx + DISPLAYSLIDERSIZE - LEFTINDENT || x < e->offx - LEFTINDENT) return(0);
 	if (e->horizfactor <= 0) return(1);
 	e->horizfactor--;
 	us_editpacredrawlines(us_editpaccurrentwin);
@@ -1299,8 +1249,8 @@ INTSML us_editpacrightpage(INTBIG x, INTBIG y)
 	REGISTER EDITOR *e;
 
 	e = us_editpaccurrentwin->editor;
-	if (y > e->revy - e->shei || y < e->revy - e->shei - SCROLLWIDTH) return(0);
-	if (x <= e->hthumbpos+THUMBSIZE/2 || x > e->offx + e->swid - SCROLLWIDTH) return(0);
+	if (y > e->revy - e->shei || y < e->revy - e->shei - DISPLAYSLIDERSIZE) return(0);
+	if (x <= e->hthumbpos+THUMBSIZE/2 || x > e->offx + e->swid - DISPLAYSLIDERSIZE) return(0);
 	if (e->horizfactor >= e->screenchars*10) return(1);
 	e->horizfactor += 10;
 	if (e->horizfactor >= e->screenchars*10) e->horizfactor = e->screenchars*10;
@@ -1314,8 +1264,8 @@ INTSML us_editpacleftpage(INTBIG x, INTBIG y)
 	REGISTER EDITOR *e;
 
 	e = us_editpaccurrentwin->editor;
-	if (y > e->revy - e->shei || y < e->revy - e->shei - SCROLLWIDTH) return(0);
-	if (x >= e->hthumbpos-THUMBSIZE/2 || x < e->offx + SCROLLWIDTH - LEFTINDENT) return(0);
+	if (y > e->revy - e->shei || y < e->revy - e->shei - DISPLAYSLIDERSIZE) return(0);
+	if (x >= e->hthumbpos-THUMBSIZE/2 || x < e->offx + DISPLAYSLIDERSIZE - LEFTINDENT) return(0);
 	if (e->horizfactor <= 0) return(1);
 	e->horizfactor -= 10;
 	if (e->horizfactor <= 0) e->horizfactor = 0;
@@ -1324,28 +1274,6 @@ INTSML us_editpacleftpage(INTBIG x, INTBIG y)
 	return(0);
 }
 
-INTSML us_editpachscroll(INTBIG x, INTBIG y)
-{
-	REGISTER EDITOR *e;
-	REGISTER INTSML l;
-
-	e = us_editpaccurrentwin->editor;
-	if (y > e->revy - e->shei + THUMBWIDTH || y < e->revy - e->shei - SCROLLWIDTH - THUMBWIDTH)
-		return(0);
-	l = e->hthumbpos;
-	e->hthumbpos = x + us_editpacoffset;
-	if (e->hthumbpos < e->offx+SCROLLWIDTH-LEFTINDENT+THUMBSIZE/2)
-		e->hthumbpos = e->offx+SCROLLWIDTH-LEFTINDENT+THUMBSIZE/2;
-	if (e->hthumbpos > e->offx+e->swid-SCROLLWIDTH-THUMBSIZE/2)
-		e->hthumbpos = e->offx+e->swid-SCROLLWIDTH-THUMBSIZE/2;
-	if (e->hthumbpos == l) return(0);
-	us_editpacinvertrectframe(us_editpaccurrentwin, (INTSML)(l-THUMBSIZE/2),
-		(INTSML)(e->revy-e->shei-SCROLLWIDTH-2));
-	us_editpacinvertrectframe(us_editpaccurrentwin, (INTSML)(e->hthumbpos-THUMBSIZE/2),
-		(INTSML)(e->revy-e->shei-SCROLLWIDTH-2));
-	return(0);
-}
-
 /******************** CHANGES TO EDIT BUFFER ********************/
 
 /*
@@ -1693,43 +1621,27 @@ void us_editpacredrawscreen(WINDOWPART *win)
 
 void us_drawsliders(WINDOWPART *win)
 {
-	REGISTER INTSML left, right, top, bottom;
 	REGISTER EDITOR *e;
 
 	e = win->editor;
 	if (e == NOEDITOR) return;
 
-	top = e->revy - SCROLLWIDTH;
-	bottom = e->revy - e->shei - 1;
-	left = e->offx-LEFTINDENT;
-	right = e->offx + e->swid + 1;
-	us_menufigs.col = el_colmengly;
+	/* draw horizontal slider */
+	us_drawhorizontalslider(win, (INTSML)(win->usely+DISPLAYSLIDERSIZE), win->uselx,
+		(INTSML)(win->usehx-DISPLAYSLIDERSIZE));
+	us_editpacdrawhscroll(win);
 
 	/* draw vertical slider if appropriate */
 	if ((win->state&WINDOWTYPE) != POPTEXTWINDOW)
 	{
-		us_drawbox(win, (INTSML)(right+1), (INTSML)(right+SCROLLWIDTH),
-			(INTSML)(e->revy-e->shei-SCROLLWIDTH), e->revy, &us_ebox);
-		us_drawline(win, right, (INTSML)(top+1), (INTSML)(right+SCROLLWIDTH), (INTSML)(top+1),
-			&us_menufigs, 0);
-		us_drawarrow(win, UPARROW, 0);
-		us_drawline(win, right, (INTSML)(bottom+SCROLLWIDTH), (INTSML)(right+SCROLLWIDTH),
-			(INTSML)(bottom+SCROLLWIDTH), &us_menufigs, 0);
-		us_drawarrow(win, DOWNARROW, 0);
-		us_drawline(win, right, bottom, (INTSML)(right+SCROLLWIDTH), bottom, &us_menufigs, 0);
+		us_drawverticalslider(win, (INTSML)(win->usehx-DISPLAYSLIDERSIZE),
+			(INTSML)(win->usely+DISPLAYSLIDERSIZE), (INTSML)(win->usehy-us_thei*HEADERLINES), 0);
+
+		/* the corner */
+		us_drawslidercorner(win, win->usehx-DISPLAYSLIDERSIZE+1, win->usehx, win->usely,
+			win->usely+DISPLAYSLIDERSIZE-1);
 		us_editpacdrawvscroll(win);
 	}
-
-	/* draw horizontal slider */
-	us_drawbox(win, left, (INTSML)(right-1), (INTSML)(bottom-SCROLLWIDTH), (INTSML)(bottom-1),
-		&us_ebox);
-	us_drawline(win, (INTSML)(left+SCROLLWIDTH-1), (INTSML)(bottom-SCROLLWIDTH),
-		(INTSML)(left+SCROLLWIDTH-1), bottom, &us_menufigs, 0);
-	us_drawarrow(win, LEFTARROW, 0);
-	us_drawline(win, (INTSML)(right-SCROLLWIDTH), (INTSML)(bottom-SCROLLWIDTH),
-		(INTSML)(right-SCROLLWIDTH), bottom, &us_menufigs, 0);
-	us_drawarrow(win, RIGHTARROW, 0);
-	us_editpacdrawhscroll(win);
 }
 
 /*
@@ -1737,7 +1649,7 @@ void us_drawsliders(WINDOWPART *win)
  */
 void us_editpacdrawhscroll(WINDOWPART *win)
 {
-	INTSML left, top, bottom, right;
+	INTSML left, right;
 	INTBIG f;
 	REGISTER EDITOR *e;
 
@@ -1745,13 +1657,8 @@ void us_editpacdrawhscroll(WINDOWPART *win)
 	if (e == NOEDITOR) return;
 
 	/* get the area of the slider without arrows */
-	top = e->revy - e->shei - 2;
-	bottom = e->revy - e->shei - SCROLLWIDTH - 1;
-	left = e->offx+SCROLLWIDTH-LEFTINDENT;
-	right = e->offx + e->swid - SCROLLWIDTH;
-
-	/* gray the scroll area */
-	us_drawbox(win, left, right, bottom, top, &us_drawgray);
+	left = e->offx+DISPLAYSLIDERSIZE-LEFTINDENT;
+	right = e->offx + e->swid - DISPLAYSLIDERSIZE;
 
 	/* compute position of vertical thumb area */
 	f = e->horizfactor;   f *= (INTBIG)(right-left-THUMBSIZE);   f /= e->screenchars*10;
@@ -1759,11 +1666,9 @@ void us_editpacdrawhscroll(WINDOWPART *win)
 	if (e->hthumbpos > right-THUMBSIZE/2) e->hthumbpos = right-THUMBSIZE/2;
 
 	/* draw the thumb */
-	left = e->hthumbpos - THUMBSIZE/2;
-	right = e->hthumbpos + THUMBSIZE/2;
-	us_menufigs.col = el_colmengly;
-	us_drawbox(win, left, right, bottom, top, &us_menufigs);
-	us_drawbox(win, (INTSML)(left+1), (INTSML)(right-1), (INTSML)(bottom+1), (INTSML)(top-1), &us_ebox);
+	win->thumblx = e->hthumbpos - THUMBSIZE/2;   win->thumbhx = e->hthumbpos + THUMBSIZE/2;
+	us_drawhorizontalsliderthumb(win, win->usely+DISPLAYSLIDERSIZE, win->uselx,
+		win->usehx-DISPLAYSLIDERSIZE, win->thumblx, win->thumbhx);
 }
 
 /*
@@ -1771,7 +1676,7 @@ void us_editpacdrawhscroll(WINDOWPART *win)
  */
 void us_editpacdrawvscroll(WINDOWPART *win)
 {
-	INTSML left, top, bottom, right;
+	INTSML top, bottom;
 	INTBIG den;
 	REGISTER EDITOR *e;
 
@@ -1779,83 +1684,28 @@ void us_editpacdrawvscroll(WINDOWPART *win)
 	if (e == NOEDITOR) return;
 
 	/* get the area of the slider without arrows */
-	top = e->revy - SCROLLWIDTH;
-	bottom = e->revy - e->shei + SCROLLWIDTH;
-	left = e->offx + e->swid + 2;
-	right = e->offx + e->swid + SCROLLWIDTH + 1;
+	top = e->revy - DISPLAYSLIDERSIZE;
+	bottom = e->revy - e->shei + DISPLAYSLIDERSIZE;
 
 	/* if there is nothing to scroll, clear this area */
 	if (e->linecount <= e->screenlines)
 	{
-		us_drawbox(win, left, right, bottom, top, &us_ebox);
+		us_drawverticalsliderthumb(win, win->usehx-DISPLAYSLIDERSIZE, win->usely+DISPLAYSLIDERSIZE,
+			win->usehy-us_thei*HEADERLINES, 1, 0);
 		return;
 	}
 
-	/* gray the scroll area */
-	us_drawbox(win, left, right, bottom, top, &us_drawgray);
-
 	/* compute position of vertical thumb area */
 	den = e->linecount - e->screenlines;
-	e->vthumbpos = (INTSML)(e->revy - SCROLLWIDTH - THUMBSIZE/2 -
-		(e->firstline * (e->shei-SCROLLWIDTH*2-THUMBSIZE) + den/2) / den);
-	if (e->vthumbpos > top-THUMBSIZE/2) e->vthumbpos = top-THUMBSIZE/2;
-	if (e->vthumbpos < bottom+THUMBSIZE/2) e->vthumbpos = bottom+THUMBSIZE/2;
+	e->vthumbpos = (INTSML)(e->revy - DISPLAYSLIDERSIZE - THUMBSIZE/2 -
+		(e->firstline * (e->shei-DISPLAYSLIDERSIZE*2-THUMBSIZE) + den/2) / den);
+	if (e->vthumbpos >= top-THUMBSIZE/2) e->vthumbpos = top-THUMBSIZE/2-1;
+	if (e->vthumbpos <= bottom+THUMBSIZE/2) e->vthumbpos = bottom+THUMBSIZE/2+1;
 
-	/* draw the thumb */
-	bottom = e->vthumbpos - THUMBSIZE/2;
-	top = e->vthumbpos + THUMBSIZE/2;
-	us_menufigs.col = el_colmengly;
-	us_drawbox(win, left, right, bottom, top, &us_menufigs);
-	us_drawbox(win, (INTSML)(left+1), (INTSML)(right-1), (INTSML)(bottom+1), (INTSML)(top-1), &us_ebox);
-}
-
-void us_drawarrow(WINDOWPART *win, INTSML which, INTSML filled)
-{
-	INTSML i, l, left, top, bottom, right;
-	INTBIG xv[7], yv[7];
-	REGISTER EDITOR *e;
-	static INTSML dia_arrowx[7] = {4, 10, 10, 13, 7, 1, 4};
-	static INTSML dia_arrowy[7] = {12, 12, 8, 8, 2, 8, 8};
-
-	e = win->editor;
-	if (e == NOEDITOR) return;
-	top = e->revy;
-	bottom = e->revy - e->shei - 1;
-	left = e->offx - LEFTINDENT;
-	right = e->offx + e->swid + 1;
-	for(i=0; i<7; i++)
-	{
-		switch (which)
-		{
-			case UPARROW:
-				xv[i] = right+dia_arrowx[i]+1;
-				yv[i] = top-dia_arrowy[i]+1;
-				break;
-			case DOWNARROW:
-				xv[i] = right+dia_arrowx[i]+1;
-				yv[i] = bottom+dia_arrowy[i];
-				break;
-			case LEFTARROW:
-				xv[i] = left+dia_arrowy[i]-1;
-				yv[i] = bottom+dia_arrowx[i]-SCROLLWIDTH;
-				break;
-			case RIGHTARROW:
-				xv[i] = right-dia_arrowy[i];
-				yv[i] = bottom+dia_arrowx[i]-SCROLLWIDTH;
-				break;
-		}
-	}
-
-	us_menufigs.col = el_colmengly;
-	if (filled == 0)
-	{
-		us_drawpolygon(win, xv, yv, 7, &us_ebox);
-		for(i=0; i<7; i++)
-		{
-			if (i == 0) l = 7-1; else l = i-1;
-			us_drawline(win, (INTSML)xv[l], (INTSML)yv[l], (INTSML)xv[i], (INTSML)yv[i], &us_menufigs, 0);
-		}
-	} else us_drawpolygon(win, xv, yv, 7, &us_menufigs);
+	win->thumbly = e->vthumbpos - THUMBSIZE/2;
+	win->thumbhy = e->vthumbpos + THUMBSIZE/2;
+	us_drawverticalsliderthumb(win, win->usehx-DISPLAYSLIDERSIZE, win->usely+DISPLAYSLIDERSIZE,
+		win->usehy-us_thei*HEADERLINES, win->thumbly, win->thumbhy);
 }
 
 /*
@@ -1890,7 +1740,7 @@ void us_editpacredrawlines(WINDOWPART *win)
 
 	/* clear the screen */
 	bottom = e->revy - e->shei;
-	us_drawbox(win, (INTSML)(e->offx-LEFTINDENT), (INTSML)(e->offx+e->swid), bottom, e->revy,
+	screendrawbox(win, (INTSML)(e->offx-LEFTINDENT), (INTSML)(e->offx+e->swid), bottom, e->revy,
 		&us_ebox);
 
 	/* rewrite each line */
@@ -1909,7 +1759,7 @@ void us_editpacsetheader(WINDOWPART *win, char *header)
 
 	e = win->editor;
 	if (e == NOEDITOR) return;
-	us_drawbox(win, win->uselx, win->usehx, (INTSML)(e->revy+1),
+	screendrawbox(win, win->uselx, win->usehx, (INTSML)(e->revy+1),
 		(INTSML)(e->revy+us_thei*HEADERLINES), &us_ebox);
 	if ((INTBIG)strlen(header) > e->screenchars)
 	{
@@ -1925,7 +1775,7 @@ void us_editpacsetheader(WINDOWPART *win, char *header)
 		if (strcmp(win->location, "entire") != 0)
 			us_editpactext(win, "Type ESC when done", 0, -1); else
 		{
-			if (us_graphicshas(CANUSEFRAMES) != 0)
+			if (graphicshas(CANUSEFRAMES) != 0)
 				us_editpactext(win, "Close the window when done", 0, -1); else
 					us_editpactext(win, "This facet is purely textual", 0, -1);
 		}
@@ -1934,18 +1784,15 @@ void us_editpacsetheader(WINDOWPART *win, char *header)
 
 	if (save != 0) header[e->screenchars] = (char)save;
 	bottom = e->revy - e->shei - 1;
-	us_invertbox(win, win->uselx, win->usehx, (INTSML)(e->revy+1),
+	screeninvertbox(win, win->uselx, win->usehx, (INTSML)(e->revy+1),
 		(INTSML)(e->revy+us_thei*HEADERLINES));
 	us_menufigs.col = el_colmengly;
-	us_drawline(win, win->uselx, e->revy, win->uselx, bottom, &us_menufigs, 0);
-	us_drawline(win, (INTSML)(e->offx+e->swid+1), e->revy, (INTSML)(e->offx+e->swid+1),
-		(INTSML)(bottom-SCROLLWIDTH), &us_menufigs, 0);
-	us_drawline(win, win->uselx, bottom, win->usehx, bottom, &us_menufigs, 0);
+	screendrawline(win, win->uselx, e->revy, win->uselx, bottom, &us_menufigs, 0);
 	if ((win->state&WINDOWTYPE) == POPTEXTWINDOW)
 	{
-		us_drawline(win, win->uselx, win->usely, win->uselx, win->usehy, &us_menufigs, 0);
-		us_drawline(win, win->uselx, win->usely, win->usehx, win->usely, &us_menufigs, 0);
-		us_drawline(win, win->usehx, win->usely, win->usehx, win->usehy, &us_menufigs, 0);
+		screendrawline(win, win->uselx, win->usely, win->uselx, win->usehy, &us_menufigs, 0);
+		screendrawline(win, win->uselx, win->usely, win->usehx, win->usely, &us_menufigs, 0);
+		screendrawline(win, win->usehx, win->usely, win->usehx, win->usehy, &us_menufigs, 0);
 	}
 }
 
@@ -1990,7 +1837,7 @@ void us_editpacinvertselection(WINDOWPART *win)
 		if (ly < e->revy - e->shei) ly = e->revy - e->shei;
 		if (hy > e->revy) hy = e->revy;
 
-		us_invertbox(win, (INTSML)lx, (INTSML)hx, (INTSML)ly, (INTSML)hy);
+		screeninvertbox(win, (INTSML)lx, (INTSML)hx, (INTSML)ly, (INTSML)hy);
 	}
 }
 
@@ -2145,10 +1992,10 @@ void us_editpaccomputebounds(WINDOWPART *win, EDITOR *e)
 	e->offx = win->uselx + 1 + LEFTINDENT;
 	e->swid = win->usehx - e->offx - 1;
 	offset = 1;
-	if ((win->state&WINDOWTYPE) != POPTEXTWINDOW) e->swid -= SCROLLWIDTH;
+	if ((win->state&WINDOWTYPE) != POPTEXTWINDOW) e->swid -= DISPLAYSLIDERSIZE;
 		else offset++;
-	e->shei = win->usehy - win->usely - us_thei*HEADERLINES - offset - SCROLLWIDTH;
-	e->revy = e->shei + win->usely + offset + SCROLLWIDTH;
+	e->shei = win->usehy - win->usely - us_thei*HEADERLINES - offset - DISPLAYSLIDERSIZE;
+	e->revy = e->shei + win->usely + offset + DISPLAYSLIDERSIZE;
 	e->screenlines = e->shei / us_thei;
 	e->screenchars = e->swid / us_twid;
 }
@@ -2254,7 +2101,7 @@ void us_editpacmovebox(WINDOWPART *win, INTSML dx, INTSML dy, INTSML wid, INTSML
 
 	e = win->editor;
 	if (e == NOEDITOR) return;
-	us_movebox(win, (INTSML)(sx*us_twid+e->offx), (INTSML)(e->revy-(sy*us_thei+hei-1)), wid, hei,
+	screenmovebox(win, (INTSML)(sx*us_twid+e->offx), (INTSML)(e->revy-(sy*us_thei+hei-1)), wid, hei,
 		(INTSML)(dx*us_twid+e->offx), (INTSML)(e->revy-(dy*us_thei+hei-1)));
 }
 
@@ -2270,7 +2117,7 @@ void us_editpacclearbox(WINDOWPART *win, INTSML dx, INTSML dy, INTSML wid, INTSM
 
 	dx = (dx * us_twid) + e->offx;
 	dy *= us_thei;
-	us_drawbox(win, dx, (INTSML)(dx+wid-1), (INTSML)(e->revy-(dy+hei-1)), (INTSML)(e->revy-dy),
+	screendrawbox(win, dx, (INTSML)(dx+wid-1), (INTSML)(e->revy-(dy+hei-1)), (INTSML)(e->revy-dy),
 		&us_ebox);
 }
 
@@ -2279,11 +2126,11 @@ void us_editpacclearbox(WINDOWPART *win, INTSML dx, INTSML dy, INTSML wid, INTSM
  */
 void us_editpacinvertrectframe(WINDOWPART *win, INTSML x, INTSML y)
 {
-	us_invertbox(win, x, (INTSML)(x+THUMBSIZE), y, y);
-	us_invertbox(win, x, (INTSML)(x+THUMBSIZE), (INTSML)(y+THUMBSIZE), (INTSML)(y+THUMBSIZE));
-	us_invertbox(win, (INTSML)(x+THUMBSIZE), (INTSML)(x+THUMBSIZE), (INTSML)(y+1),
+	screeninvertbox(win, x, (INTSML)(x+THUMBSIZE), y, y);
+	screeninvertbox(win, x, (INTSML)(x+THUMBSIZE), (INTSML)(y+THUMBSIZE), (INTSML)(y+THUMBSIZE));
+	screeninvertbox(win, (INTSML)(x+THUMBSIZE), (INTSML)(x+THUMBSIZE), (INTSML)(y+1),
 		(INTSML)(y+THUMBSIZE-1));
-	us_invertbox(win, x, x, (INTSML)(y+1), (INTSML)(y+THUMBSIZE-1));
+	screeninvertbox(win, x, x, (INTSML)(y+1), (INTSML)(y+THUMBSIZE-1));
 }
 
 /*
@@ -2320,6 +2167,6 @@ void us_editpactext(WINDOWPART *win, char *str, INTSML x, INTSML y)
 		save = str[e->screenchars + e->horizfactor - x];
 		str[e->screenchars + e->horizfactor - x] = 0;
 	} else save = 0;
-	us_puttext(win, (INTSML)(xpos + e->offx), (INTSML)(e->revy-(y+1)*us_thei), str, &us_menutext);
+	screendrawtext(win, (INTSML)(xpos + e->offx), (INTSML)(e->revy-(y+1)*us_thei), str, &us_menutext);
 	if (save != 0) str[e->screenchars + e->horizfactor - x] = (char)save;
 }
