@@ -1932,7 +1932,7 @@ void us_tecedsetnode(NODEINST *ni, char *chr)
 	var = getvalkey((INTBIG)ni, VNODEINST, VSTRING, art_messagekey);
 	if (var == NOVARIABLE) descript = 0; else descript = var->textdescript;
 	var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)chr, VSTRING|VDISPLAY);
-	if (var != NOVARIABLE) modifydescript(var, descript);
+	if (var != NOVARIABLE) modifydescript((INTBIG)ni, VNODEINST, var, descript);
 	endobjectchange((INTBIG)ni, VNODEINST);
 }
 
