@@ -45,6 +45,7 @@ void us_graphfacets(void)
 	REGISTER NODEPROTO *np, *sub, *graphnp, *truenp, *truesubnp;
 	REGISTER NODEINST *ni, *nibot, *toppin;
 	REGISTER ARCINST *ai;
+	REGISTER LIBRARY *lib;
 	REGISTER INTSML more, maxdepth, color;
 	REGISTER INTBIG *xval, i, x, y, xe, ye, clock, maxwidth, xsc;
 	REGISTER PORTPROTO *pinpp;
@@ -59,7 +60,9 @@ void us_graphfacets(void)
 			ttyputverbose("Creating facet: FacetStructure");
 
 	/* clear flags on all of the facets */
-	for(np = el_curlib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto) np->temp1 = -1;
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+			np->temp1 = -1;
 
 	/* find all top-level facets */
 	for(np = el_curlib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
@@ -76,7 +79,8 @@ void us_graphfacets(void)
 	while (more != 0)
 	{
 		more = 0;
-		for(np = el_curlib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+		for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+			for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 		{
 			if (np->temp1 == -1) continue;
 			for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
@@ -115,9 +119,11 @@ void us_graphfacets(void)
 	xval = emalloc((SIZEOFINTBIG * maxdepth), el_tempcluster);
 	if (xval == 0) return;
 	for(i=0; i<maxdepth; i++) xval[i] = 0;
-	for(np = el_curlib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 	{
 		/* ignore icon facets from the graph (merge with contents) */
+		if (np->temp1 == -1) continue;
 		truenp = contentsview(np);
 		if (truenp != NONODEPROTO) continue;
 
@@ -138,9 +144,11 @@ void us_graphfacets(void)
 		VSTRING|VDISPLAY);
 
 	/* place the components */
-	for(np = el_curlib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 	{
 		if (np == graphnp) continue;
+		if (np->temp1 == -1) continue;
 
 		/* always use the contents facet, not the icon */
 		truenp = contentsview(np);
@@ -160,15 +168,19 @@ void us_graphfacets(void)
 	}
 
 	/* build wires between the hierarchical levels */
-	for(np = el_curlib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto) np->temp2 = 0;
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+			np->temp2 = 0;
 	clock = 0;
-	for(np = el_curlib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 	{
 		if (np == graphnp) continue;
 
 		/* always use the contents facet, not the icon */
 		truenp = contentsview(np);
 		if (truenp == NONODEPROTO) truenp = np;
+		if (truenp->temp1 == -1) continue;
 
 		toppin = NONODEINST;
 		clock++;
@@ -403,6 +415,8 @@ COMILL *us_illustratecommand(char *name, INTSML depth, COMILL *thisci)
 		if (cc->toplist == topofcells) arg = "CELL"; else
 		if (cc->toplist == topofviews) arg = "VIEW"; else
 		if (cc->toplist == topofnets) arg = "NET"; else
+		if (cc->toplist == topofarcs) arg = "ARC"; else
+		if (cc->toplist == topoffacets) arg = "FACET"; else
 		if (cc->toplist == us_topofcommands) arg = "COM"; else
 		if (cc->toplist == us_topofmacros) arg = "MACRO"; else
 		if (cc->toplist == us_topofpopupmenu) arg = "POPUP"; else
@@ -414,10 +428,8 @@ COMILL *us_illustratecommand(char *name, INTSML depth, COMILL *thisci)
 		if (cc->toplist == us_topofhighlight) arg = "HIGH"; else
 		if (cc->toplist == us_topofarcnodes) arg = "ARC/NODE"; else
 		if (cc->toplist == us_topofnodes) arg = "NODE"; else
-		if (cc->toplist == us_topoffacets) arg = "FACET"; else
 		if (cc->toplist == us_topofcellfacets) arg = "FACET/CELL"; else
 		if (cc->toplist == us_topofprims) arg = "PRIM"; else
-		if (cc->toplist == us_topofarcs) arg = "ARC"; else
 		if (cc->toplist == us_topofconstraints) arg = "CONSTR"; else
 		if (cc->toplist == us_topofmbuttons) arg = "BUTTON"; else
 		if (cc->toplist == us_topofedteclay) arg = "LAYER"; else
