@@ -31,6 +31,7 @@
 #include "global.h"
 #include "egraphics.h"
 #include "usr.h"
+#include "tech.h"
 #include "tecgen.h"
 
 #define DOTRADIUS 4				/* size of dots in nodes when highlighting networks */
@@ -78,11 +79,11 @@ void us_highlightmenu(INTBIG x, INTBIG y, INTSML color)
 	/* set drawing bounds to include menu */
 	if (us_menuframe == NOWINDOWFRAME)
 	{
-		us_getpaletteparameters(&swid, &shei, &pwid);
-		ww.frame = us_getwindowframe(1);
+		getpaletteparameters(&swid, &shei, &pwid);
+		ww.frame = getwindowframe(1);
 	} else
 	{
-		us_getwindowsize(us_menuframe, &swid, &shei);
+		getwindowframesize(us_menuframe, &swid, &shei);
 		ww.frame = us_menuframe;
 	}
 	ww.screenlx = ww.screenly = ww.uselx = ww.usely = 0;
@@ -328,7 +329,6 @@ void us_setfind(HIGHLIGHT *newhigh, INTSML findpoint, INTSML extrainfo, INTSML f
 	}
 
 	/* load the highlight structure */
-	newhigh->facet = geomparent(newhigh->fromgeom);
 	newhigh->status |= extrainfo;
 	newhigh->frompoint = (INTSML)bestpoint;
 	if (nobox != 0) newhigh->status |= HIGHNOBOX;
@@ -489,9 +489,11 @@ GEOM **us_gethighlighted(INTSML type)
 		/* special case: accept ports and invisible node if it holds highlighted text */
 		if ((high.status&HIGHTYPE) == HIGHTEXT)
 		{
-			if (high.fromvar != NOVARIABLE && high.fromgeom->entrytype == OBJNODEINST &&
-				high.fromgeom->entryaddr.ni->proto == gen_invispinprim && (type&OBJNODEINST) != 0)
-					total++;
+			if (high.fromvar != NOVARIABLE && high.fromgeom != NOGEOM &&
+				high.fromgeom->entrytype == OBJNODEINST &&
+					high.fromgeom->entryaddr.ni->proto == gen_invispinprim &&
+						(type&OBJNODEINST) != 0)
+							total++;
 		}
 	}
 
@@ -576,9 +578,11 @@ GEOM **us_gethighlighted(INTSML type)
 		}
 		if ((high.status&HIGHTYPE) == HIGHTEXT)
 		{
-			if (high.fromvar != NOVARIABLE && high.fromgeom->entrytype == OBJNODEINST &&
-				high.fromgeom->entryaddr.ni->proto == gen_invispinprim && (type&OBJNODEINST) != 0)
-					us_gethighlist[total++] = high.fromgeom;
+			if (high.fromvar != NOVARIABLE && high.fromgeom != NOGEOM &&
+				high.fromgeom->entrytype == OBJNODEINST &&
+					high.fromgeom->entryaddr.ni->proto == gen_invispinprim &&
+						(type&OBJNODEINST) != 0)
+							us_gethighlist[total++] = high.fromgeom;
 		}
 	}
 	us_gethighlist[total] = NOGEOM;
@@ -757,6 +761,7 @@ NODEPROTO *us_getareabounds(INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 	XARRAY trans;
 	static POLYGON *poly = NOPOLYGON;
 	REGISTER NODEPROTO *thisfacet;
+	REGISTER TECHNOLOGY *tech;
 	REGISTER VARIABLE *var;
 
 	/* get polygon */
@@ -781,7 +786,9 @@ NODEPROTO *us_getareabounds(INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 			if (high.fromvar != NOVARIABLE)
 			{
 				pointlen = 1;
-				if (high.fromgeom->entrytype != OBJNODEINST) ni = NONODEINST; else
+				ni = NONODEINST;
+				if (high.fromport != NOPORTPROTO) ni = high.fromport->subnodeinst; else
+					if (high.fromgeom != NOGEOM && high.fromgeom->entrytype == OBJNODEINST)
 				{
 					ni = high.fromgeom->entryaddr.ni;
 					if ((high.fromvar->type&VISARRAY) != 0)
@@ -790,10 +797,18 @@ NODEPROTO *us_getareabounds(INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 				if (pointlen > 1)
 				{
 					makedisparrayvarpoly(high.fromgeom, el_curwindowpart, high.fromvar, poly);
-				} else us_maketextpoly(describevariable(high.fromvar, -1, -1), el_curwindowpart,
-					(high.fromgeom->lowx + high.fromgeom->highx) / 2,
-						(high.fromgeom->lowy + high.fromgeom->highy) / 2,
-							high.fromgeom, high.fromvar->textdescript, poly);
+				} else
+				{
+					if (ni != NONODEINST) tech = whattech(ni->proto); else
+					{
+						if (high.fromgeom != NOGEOM) tech = high.fromgeom->entryaddr.ai->proto->tech; else
+							tech = whattech(high.facet);
+					}
+					us_maketextpoly(describevariable(high.fromvar, -1, -1), el_curwindowpart,
+						(high.fromgeom->lowx + high.fromgeom->highx) / 2,
+							(high.fromgeom->lowy + high.fromgeom->highy) / 2,
+								ni, tech, high.fromvar->textdescript, poly);
+				}
 				if (ni != NONODEINST)
 				{
 					makerot(ni, trans);
@@ -803,13 +818,14 @@ NODEPROTO *us_getareabounds(INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 			{
 				portposition(high.fromport->subnodeinst, high.fromport->subportproto, &xc, &yc);
 				us_maketextpoly(high.fromport->protoname, el_curwindowpart, xc, yc,
-					high.fromport->subnodeinst->geom, high.fromport->textdescript, poly);
+					high.fromport->subnodeinst, whattech(high.fromport->subnodeinst->proto),
+						high.fromport->textdescript, poly);
 			} else if (high.fromgeom->entrytype == OBJNODEINST)
 			{
 				ni = high.fromgeom->entryaddr.ni;
 				us_maketextpoly(describenodeproto(ni->proto), el_curwindowpart,
 					(ni->lowx + ni->highx) / 2, (ni->lowy + ni->highy) / 2,
-						ni->geom, ni->textdescript, poly);
+						ni, whattech(ni->proto), ni->textdescript, poly);
 			}
 			getbbox(poly, &tlx, &thx, &tly, &thy);
 		}
@@ -875,9 +891,8 @@ void us_sethighlight(HIGHLIGHT *high, INTSML on)
 {
 	REGISTER INTSML len;
 	REGISTER INTBIG xw, yw, descript;
-	INTBIG xc, yc, lx, hx, ly, hy;
+	INTBIG xc, yc, lx, hx, ly, hy, addr, type;
 	INTSML tsx, tsy, style;
-	REGISTER NODEINST *ni;
 	REGISTER WINDOWPART *w;
 	REGISTER TECHNOLOGY *tech;
 	static POLYGON *poly = NOPOLYGON;
@@ -889,7 +904,7 @@ void us_sethighlight(HIGHLIGHT *high, INTSML on)
 	/* handle drawing bounding boxes */
 	if ((high->status&HIGHTYPE) == HIGHBBOX)
 	{
-		maketruerectpoly(high->stalx,high->stahx, high->staly,high->stahy, poly);
+		maketruerectpoly(high->stalx, high->stahx, high->staly, high->stahy, poly);
 		poly->desc = &us_hbox;
 		poly->style = CLOSEDRECT;
 		us_hbox.col = on;
@@ -923,26 +938,8 @@ void us_sethighlight(HIGHLIGHT *high, INTSML on)
 	/* handle text outlines */
 	if ((high->status&HIGHTYPE) == HIGHTEXT)
 	{
-		if (high->fromvar != NOVARIABLE)
-		{
-			str = describevariable(high->fromvar, -1, -1);
-			if ((high->fromvar->type&VISARRAY) == 0)
-			{
-				len = (INTSML)((high->fromvar->type&VLENGTH) >> VLENGTHSH);
-				if (len >= (INTBIG)strlen(str)) return;
-				str += len;
-			}
-			descript = high->fromvar->textdescript;
-		} else if (high->fromport != NOPORTPROTO)
-		{
-			str = high->fromport->protoname;
-			descript = high->fromport->textdescript;
-		} else if (high->fromgeom->entrytype == OBJNODEINST)
-		{
-			ni = high->fromgeom->entryaddr.ni;
-			str = describenodeproto(ni->proto);
-			descript = ni->textdescript;
-		} else return;
+		descript = us_gethighdescript(high);
+		str = us_gethighstring(high);
 
 		/* determine center of text */
 		us_gethightextcenter(high, &xc, &yc, &style);
@@ -974,17 +971,17 @@ void us_sethighlight(HIGHLIGHT *high, INTSML on)
 			} else
 			{
 				/* determine size from text */
-				if (high->fromgeom->entrytype == OBJARCINST)
-					tech = high->fromgeom->entryaddr.ai->proto->tech; else
-						tech = whattech(high->fromgeom->entryaddr.ni->proto);
-				us_settextsize(w, truefontsize((INTSML)((descript&VTSIZE)>>VTSIZESH), w, tech));
-				us_textsize(w, str, &tsx, &tsy);
+				tech = us_hightech(high);
+				screensettextsize(w, truefontsize((INTSML)((descript&VTSIZE)>>VTSIZESH), w, tech));
+				screengettextsize(w, str, &tsx, &tsy);
 				xw = muldiv(tsx, w->screenhx-w->screenlx, w->usehx-w->uselx);
 				yw = muldiv(tsy, w->screenhy-w->screenly, w->usehy-w->usely);
 			}
 
 			/* construct polygon */
-			us_buildtexthighpoly(high->fromgeom, xc, yc, xw, yw, style, poly);
+			us_gethighaddrtype(high, &addr, &type);
+			(void)us_getobjectinfo(addr, type, &lx, &hx, &ly, &hy);
+			us_buildtexthighpoly(lx, hx, ly, hy, xc, yc, xw, yw, style, poly);
 			(void)us_showpoly(poly, w);
 		}
 		return;
@@ -1020,8 +1017,8 @@ void us_sethighlight(HIGHLIGHT *high, INTSML on)
  * style is in "style" (either TEXTCENT, TEXTBOT, ...) and whose attached
  * object is "geom".
  */
-void us_buildtexthighpoly(GEOM *geom, INTBIG xc, INTBIG yc, INTBIG xw, INTBIG yw,
-	INTSML style, POLYGON *poly)
+void us_buildtexthighpoly(INTBIG lx, INTBIG hx, INTBIG ly, INTBIG hy,
+	INTBIG xc, INTBIG yc, INTBIG xw, INTBIG yw, INTSML style, POLYGON *poly)
 {
 	REGISTER INTBIG xsh, ysh;
 
@@ -1037,16 +1034,16 @@ void us_buildtexthighpoly(GEOM *geom, INTBIG xc, INTBIG yc, INTBIG xw, INTBIG yw
 			poly->style = VECTORS;     poly->count = 4;
 			if (style == TEXTCENT) break;
 			if (poly->limit < 12) (void)extendpolygon(poly, 12);
-			xsh = (geom->highx - geom->lowx) / 6;
-			ysh = (geom->highy - geom->lowy) / 6;
-			poly->xv[4] = geom->lowx + xsh;   poly->yv[4] = geom->lowy;
-			poly->xv[5] = geom->highx - xsh;  poly->yv[5] = geom->lowy;
-			poly->xv[6] = geom->lowx + xsh;   poly->yv[6] = geom->highy;
-			poly->xv[7] = geom->highx - xsh;  poly->yv[7] = geom->highy;
-			poly->xv[8] = geom->lowx;         poly->yv[8] = geom->lowy + ysh;
-			poly->xv[9] = geom->lowx;         poly->yv[9] = geom->highy - ysh;
-			poly->xv[10] = geom->highx;       poly->yv[10] = geom->lowy + ysh;
-			poly->xv[11] = geom->highx;       poly->yv[11] = geom->highy - ysh;
+			xsh = (hx - lx) / 6;
+			ysh = (hy - ly) / 6;
+			poly->xv[4] = lx + xsh;   poly->yv[4] = ly;
+			poly->xv[5] = hx - xsh;  poly->yv[5] = ly;
+			poly->xv[6] = lx + xsh;   poly->yv[6] = hy;
+			poly->xv[7] = hx - xsh;  poly->yv[7] = hy;
+			poly->xv[8] = lx;         poly->yv[8] = ly + ysh;
+			poly->xv[9] = lx;         poly->yv[9] = hy - ysh;
+			poly->xv[10] = hx;       poly->yv[10] = ly + ysh;
+			poly->xv[11] = hx;       poly->yv[11] = hy - ysh;
 			poly->count = 12;
 			break;
 		case TEXTBOT:
@@ -1593,6 +1590,11 @@ void us_makecurrentobject(void)
 		return;
 	}
 	from = high.fromgeom;
+	if (from == NOGEOM)
+	{
+		us_setnodeproto(NONODEPROTO);
+		return;
+	}
 
 	/* if an arc is selected, show it */
 	if (from->entrytype == OBJARCINST)
@@ -1693,12 +1695,21 @@ INTSML us_makehighlight(char *str, HIGHLIGHT *high)
 		if (keyword == NOSTRING) return(1);
 		if (*keyword == '-') high->fromvar = NOVARIABLE; else
 		{
-			if (high->fromgeom->entrytype == OBJNODEINST)
+			if (high->fromport != NOPORTPROTO)
 			{
-				high->fromvar = getval((INTBIG)high->fromgeom->entryaddr.ni, VNODEINST, -1, keyword);
+				high->fromvar = getval((INTBIG)high->fromport, VPORTPROTO, -1, keyword);
+			} else if (high->fromgeom != NOGEOM)
+			{
+				if (high->fromgeom->entrytype == OBJNODEINST)
+				{
+					high->fromvar = getval((INTBIG)high->fromgeom->entryaddr.ni, VNODEINST, -1, keyword);
+				} else
+				{
+					high->fromvar = getval((INTBIG)high->fromgeom->entryaddr.ai, VARCINST, -1, keyword);
+				}
 			} else
 			{
-				high->fromvar = getval((INTBIG)high->fromgeom->entryaddr.ai, VARCINST, -1, keyword);
+				high->fromvar = getval((INTBIG)high->facet, VNODEPROTO, -1, keyword);
 			}
 		}
 	} else return(1);
@@ -1823,20 +1834,37 @@ void us_gethightextcenter(HIGHLIGHT *high, INTBIG *xc, INTBIG *yc, INTSML *style
 	if (high->fromvar != NOVARIABLE)
 	{
 		descript = high->fromvar->textdescript;
-		if (high->fromgeom->entrytype == OBJNODEINST)
+		if (high->fromport != NOPORTPROTO)
 		{
-			ni = high->fromgeom->entryaddr.ni;
-			poly->xv[0] = (ni->lowx + ni->highx) / 2;
-			poly->yv[0] = (ni->lowy + ni->highy) / 2;
-			adjustdisoffset(ni->geom, poly, descript);
-			makerot(ni, trans);
+			portposition(high->fromport->subnodeinst, high->fromport->subportproto,
+				&poly->xv[0], &poly->yv[0]);
+			adjustdisoffset((INTBIG)high->fromport, VPORTPROTO, whattech(high->facet),
+				poly, descript);
+			makerot(high->fromport->subnodeinst, trans);
 			xformpoly(poly, trans);
+		} else if (high->fromgeom != NOGEOM)
+		{
+			if (high->fromgeom->entrytype == OBJNODEINST)
+			{
+				ni = high->fromgeom->entryaddr.ni;
+				poly->xv[0] = (ni->lowx + ni->highx) / 2;
+				poly->yv[0] = (ni->lowy + ni->highy) / 2;
+				adjustdisoffset((INTBIG)ni, VNODEINST, whattech(ni->proto), poly, descript);
+				makerot(ni, trans);
+				xformpoly(poly, trans);
+			} else
+			{
+				ai = high->fromgeom->entryaddr.ai;
+				poly->xv[0] = (ai->end[0].xpos + ai->end[1].xpos) / 2;
+				poly->yv[0] = (ai->end[0].ypos + ai->end[1].ypos) / 2;
+				adjustdisoffset((INTBIG)ai, VARCINST, ai->proto->tech, poly, descript);
+			}
 		} else
 		{
-			ai = high->fromgeom->entryaddr.ai;
-			poly->xv[0] = (ai->end[0].xpos + ai->end[1].xpos) / 2;
-			poly->yv[0] = (ai->end[0].ypos + ai->end[1].ypos) / 2;
-			adjustdisoffset(ai->geom, poly, descript);
+			poly->xv[0] = (high->facet->lowx + high->facet->highx) / 2;
+			poly->yv[0] = (high->facet->lowy + high->facet->highy) / 2;
+			adjustdisoffset((INTBIG)high->facet, VNODEPROTO, whattech(high->facet),
+				poly, descript);
 		}
 		*style = poly->style;
 	} else if (high->fromport != NOPORTPROTO)
@@ -1875,10 +1903,254 @@ void us_gethightextcenter(HIGHLIGHT *high, INTBIG *xc, INTBIG *yc, INTSML *style
 		descript = ni->textdescript;
 		poly->xv[0] = (ni->lowx + ni->highx) / 2;
 		poly->yv[0] = (ni->lowy + ni->highy) / 2;
-		adjustdisoffset(ni->geom, poly, descript);
+		adjustdisoffset((INTBIG)ni, VNODEINST, whattech(ni->proto), poly, descript);
 		makerot(ni, trans);
 		xformpoly(poly, trans);
 		*style = poly->style;
 	}
 	getcenter(poly, xc, yc);
 }
+
+/*
+ * Routine to determine whether highlight "high" resides on a node, and return
+ * that node if so (NONODEINST if not).
+ */
+NODEINST *us_gethighnodeinst(HIGHLIGHT *high)
+{
+	if (high->fromport != NOPORTPROTO) return(high->fromport->subnodeinst);
+	if (high->fromgeom != NOGEOM && high->fromgeom->entrytype == OBJNODEINST)
+		return(high->fromgeom->entryaddr.ni);
+	return(NONODEINST);
+}
+
+/*
+ * Routine to return the text descriptor on highlight "high".
+ */
+INTBIG us_gethighdescript(HIGHLIGHT *high)
+{
+	if (high->fromvar != NOVARIABLE)
+		return(high->fromvar->textdescript);
+	if (high->fromport != NOPORTPROTO)
+		return(high->fromport->textdescript);
+	if (high->fromgeom->entrytype == OBJNODEINST)
+		return(high->fromgeom->entryaddr.ni->textdescript);
+	return(0);
+}
+
+/*
+ * Routine to return the text on highlight "high".
+ */
+char *us_gethighstring(HIGHLIGHT *high)
+{
+	REGISTER char *str;
+	REGISTER INTBIG len, slen;
+
+	if (high->fromvar != NOVARIABLE)
+	{
+		str = describevariable(high->fromvar, -1, -1);
+		if ((high->fromvar->type&VISARRAY) == 0 &&
+			(high->fromvar->textdescript&VTDISPLAYPART) != VTDISPLAYNAMEVALUE)
+		{
+			len = (high->fromvar->type&VLENGTH) >> VLENGTHSH;
+			slen = strlen(str);
+			if (len > slen) len = slen;
+			str += len;
+		}
+		return(str);
+	}
+	if (high->fromport != NOPORTPROTO)
+		return(high->fromport->protoname);
+	if (high->fromgeom->entrytype == OBJNODEINST)
+		return(describenodeproto(high->fromgeom->entryaddr.ni->proto));
+	return("");
+}
+
+/*
+ * Routine to examine highlight "high" and return the address and type
+ * of the object to which it refers.
+ */
+void us_gethighaddrtype(HIGHLIGHT *high, INTBIG *addr, INTBIG *type)
+{
+	if ((high->status&HIGHTYPE) == HIGHTEXT)
+	{
+		if (high->fromvar != NOVARIABLE)
+		{
+			if (high->fromport != NOPORTPROTO)
+			{
+				*addr = (INTBIG)high->fromport;
+				*type = VPORTPROTO;
+				return;
+			}
+			if (high->fromgeom == NOGEOM)
+			{
+				*addr = (INTBIG)high->facet;
+				*type = VNODEPROTO;
+				return;
+			}
+
+			*addr = (INTBIG)high->fromgeom->entryaddr.blind;
+			if (high->fromgeom->entrytype == OBJNODEINST)
+			{
+				*type = VNODEINST;
+			} else
+			{
+				*type = VARCINST;
+			}
+			return;
+		}
+		if (high->fromport != NOPORTPROTO)
+		{
+			*addr = (INTBIG)high->fromport;
+			*type = VPORTPROTO;
+			return;
+		}
+		*addr = (INTBIG)high->fromgeom->entryaddr.blind;
+		if (high->fromgeom->entrytype == OBJNODEINST) *type = VNODEINST; else
+			*type = VARCINST;
+		return;
+	}
+}
+
+/*
+ * Routine to figure out the technology to use for highlight "high".
+ */
+TECHNOLOGY *us_hightech(HIGHLIGHT *high)
+{
+	if (high->fromvar != NOVARIABLE)
+	{
+		if (high->fromport != NOPORTPROTO)
+			return(whattech(high->fromport->subnodeinst->proto));
+		if (high->fromgeom == NOGEOM) return(whattech(high->facet));
+		if (high->fromgeom->entrytype == OBJNODEINST)
+			return(whattech(high->fromgeom->entryaddr.ni->proto));
+		return(high->fromgeom->entryaddr.ai->proto->tech);
+	}
+	if (high->fromport != NOPORTPROTO)
+		return(whattech(high->fromgeom->entryaddr.ni->proto));
+	if (high->fromgeom->entrytype == OBJNODEINST)
+		return(whattech(high->fromgeom->entryaddr.ni->proto));
+	if (high->fromgeom->entrytype == OBJARCINST)
+		return(high->fromgeom->entryaddr.ai->proto->tech);
+	return(NOTECHNOLOGY);
+}
+
+TECHNOLOGY *us_getobjectinfo(INTBIG addr, INTBIG type, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
+{
+	REGISTER NODEINST *ni;
+	REGISTER ARCINST *ai;
+	REGISTER PORTPROTO *pp;
+	REGISTER NODEPROTO *np;
+
+	switch (type)
+	{
+		case VNODEINST:
+			ni = (NODEINST *)addr;
+			*lx = ni->geom->lowx;   *hx = ni->geom->highx;
+			*ly = ni->geom->lowy;   *hy = ni->geom->highy;
+			return(whattech(ni->proto));
+		case VARCINST:
+			ai = (ARCINST *)addr;
+			*lx = ai->geom->lowx;   *hx = ai->geom->highx;
+			*ly = ai->geom->lowy;   *hy = ai->geom->highy;
+			return(ai->proto->tech);
+		case VPORTPROTO:
+			pp = (PORTPROTO *)addr;
+			ni = pp->subnodeinst;
+			*lx = ni->geom->lowx;   *hx = ni->geom->highx;
+			*ly = ni->geom->lowy;   *hy = ni->geom->highy;
+			return(whattech(pp->parent));
+		case VNODEPROTO:
+			np = (NODEPROTO *)addr;
+			*lx = np->lowx;   *hx = np->highx;
+			*ly = np->lowy;   *hy = np->highy;
+			return(whattech(np));
+	}
+	return(NOTECHNOLOGY);
+}
+
+void us_drawfacetvariable(VARIABLE *var, NODEPROTO *np)
+{
+	REGISTER INTBIG i, displaytotal;
+	REGISTER VARIABLE *thisvar;
+	REGISTER WINDOWPART *w;
+	static POLYGON *poly = NOPOLYGON;
+
+	/* get polygon */
+	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
+
+	displaytotal = tech_displayablefacetvars(np);
+	for(i = 0; i < displaytotal; i++)
+	{
+		thisvar = tech_filldisplayablefacetvar(np, poly);
+		for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
+		{
+			if (w->curnodeproto != np) continue;
+			(*us_displayroutine)(poly, w);
+		}
+	}
+}
+
+void us_undrawfacetvariable(VARIABLE *var, NODEPROTO *np)
+{
+	INTBIG x, y;
+	INTSML tsx, tsy;
+	REGISTER INTSML tf, font;
+	REGISTER INTBIG objwid, objhei, sea, slx, shx, sly, shy, nudge;
+	INTBIG lx, hx, ly, hy;
+	REGISTER TECHNOLOGY *tech;
+	REGISTER NODEINST *ni;
+	REGISTER ARCINST *ai;
+	REGISTER GEOM *geom;
+	REGISTER char *str;
+	REGISTER WINDOWPART *w;
+	extern GRAPHICS us_ebox;
+
+	if ((var->type&VDISPLAY) == 0) return;
+	tech = whattech(np);
+	font = (INTSML)((var->textdescript & VTSIZE) >> VTSIZESH);
+	str = describevariable(var, -1, -1);
+	for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
+	{
+		if ((w->state&WINDOWTYPE) != DISPWINDOW) continue;
+		if (w->curnodeproto != np) continue;
+		tf = truefontsize(font, w, tech);
+		screensettextsize(w, tf);
+
+		getdisparrayvarlinepos((INTBIG)np, VNODEPROTO, tech,
+			w, var, 0, &x, &y, 1);
+		screengettextsize(w, str, &tsx, &tsy);
+		objwid = roundfloat((float)tsx / w->scalex);
+		objhei = roundfloat((float)tsy / w->scaley);
+		slx = x;   shx = slx + objwid;
+		sly = y;   shy = sly + objhei;
+
+		/* erase that area */
+		lx = slx;   hx = shx;
+		ly = sly;   hy = shy;
+		(void)us_makescreen(&lx, &ly, &hx, &hy, w);
+		screendrawbox(w, (INTSML)lx, (INTSML)hx, (INTSML)ly, (INTSML)hy, &us_ebox);
+
+		/* now redraw everything that touches this box */
+		nudge = (INTBIG)(1.0f/w->scalex);
+		slx -= nudge;
+		shx += nudge;
+		nudge = (INTBIG)(1.0f/w->scaley);
+		sly -= nudge;
+		shy += nudge;
+		sea = initsearch(slx, shx, sly, shy, np);
+		for(;;)
+		{
+			geom = nextobject(sea);
+			if (geom == NOGEOM) break;
+			if (geom->entrytype == OBJNODEINST)
+			{
+				ni = geom->entryaddr.ni;
+				(void)us_drawfacet(ni, LAYERA, el_matid, 3, w);
+			} else
+			{
+				ai = geom->entryaddr.ai;
+				(void)us_drawarcinst(ai, LAYERA, el_matid, 3, w);
+			}
+		}
+	}
+}
