@@ -97,7 +97,7 @@ void us_drawmenu(INTSML scaletofit, WINDOWFRAME *mw)
 		if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
 
 		/* get size of current window */
-		us_getwindowsize(el_curwindowpart->frame, &swid, &shei);
+		getwindowframesize(el_curwindowpart->frame, &swid, &shei);
 
 		/* set drawing bounds to address the world */
 		ww.screenlx = ww.screenly = ww.uselx = ww.usely = 0;
@@ -196,7 +196,7 @@ void us_drawmenuentry(INTBIG x, INTBIG y, char *str)
 	/* set drawing bounds to address the world */
 	if (us_menuframe != NOWINDOWFRAME) menuframe = us_menuframe; else
 		menuframe = el_curwindowpart->frame;
-	us_getwindowsize(menuframe, &swid, &shei);
+	getwindowframesize(menuframe, &swid, &shei);
 	ww.screenlx = ww.screenly = ww.uselx = ww.usely = 0;
 	ww.screenhx = ww.usehx = swid-1;   ww.screenhy = ww.usehy = shei-1;
 	ww.frame = menuframe;
@@ -754,7 +754,7 @@ POPUPMENUITEM *us_popupmenu(POPUPMENU **cmenu, INTSML *waitfordown, INTSML heade
 	if (i >= menu->total)
 	{
 		/* no type-in entries: simple popup menu */
-		if (us_graphicshas(CANSHOWPOPUP) != 0)
+		if (graphicshas(CANSHOWPOPUP) != 0)
 		{
 			i = nativepopupmenu(menu, header, left, top);
 			if (i < 0) return(NOPOPUPMENUITEM);
@@ -777,8 +777,8 @@ POPUPMENUITEM *us_popupmenu(POPUPMENU **cmenu, INTSML *waitfordown, INTSML heade
 	us_pmcur.yshift = 0;
 
 	/* get size of current window */
-	frame = us_getwindowframe(0);
-	us_getwindowsize(frame, &swid, &shei);
+	frame = getwindowframe(0);
+	getwindowframesize(frame, &swid, &shei);
 	us_pmcur.popupwindow.screenlx = us_pmcur.popupwindow.screenly = 0;
 	us_pmcur.popupwindow.uselx = us_pmcur.popupwindow.usely = 0;
 	us_pmcur.popupwindow.screenhx = us_pmcur.popupwindow.usehx = swid-1;
@@ -796,19 +796,19 @@ POPUPMENUITEM *us_popupmenu(POPUPMENU **cmenu, INTSML *waitfordown, INTSML heade
 	for(textscale = startfont; textscale >= TXT4P; textscale--)
 	{
 		us_pmcur.attlen = us_pmcur.vallen = 0;
-		if (textscale == TXT14P) us_settextsize(&us_pmcur.popupwindow, TXTMENU); else
-			us_settextsize(&us_pmcur.popupwindow, textscale);
-		us_textsize(&us_pmcur.popupwindow, "0", &chwid, &y);
+		if (textscale == TXT14P) screensettextsize(&us_pmcur.popupwindow, TXTMENU); else
+			screensettextsize(&us_pmcur.popupwindow, textscale);
+		screengettextsize(&us_pmcur.popupwindow, "0", &chwid, &y);
 		us_pmcur.txthei = y+2;
 		for(i=0; i<us_pmcur.current->total; i++)
 		{
 			mi = &us_pmcur.current->list[i];
 			mi->changed = 0;
 			pt = us_stripampersand(mi->attribute);
-			us_textsize(&us_pmcur.popupwindow, pt, &x, &y);
+			screengettextsize(&us_pmcur.popupwindow, pt, &x, &y);
 			if (x > us_pmcur.attlen) us_pmcur.attlen = x;
 			if (mi->value == 0) continue;
-			us_textsize(&us_pmcur.popupwindow, mi->value, &x, &y);
+			screengettextsize(&us_pmcur.popupwindow, mi->value, &x, &y);
 			if (x > us_pmcur.vallen) us_pmcur.vallen = x;
 			if (mi->maxlen*chwid > us_pmcur.vallen) us_pmcur.vallen = mi->maxlen*chwid;
 		}
@@ -817,7 +817,7 @@ POPUPMENUITEM *us_popupmenu(POPUPMENU **cmenu, INTSML *waitfordown, INTSML heade
 		if (header != 0)
 		{
 			pt = us_stripampersand(us_pmcur.current->header);
-			us_textsize(&us_pmcur.popupwindow, pt, &x, &y);
+			screengettextsize(&us_pmcur.popupwindow, pt, &x, &y);
 			if (us_pmcur.vallen == 0)
 			{
 				if (x > us_pmcur.attlen) us_pmcur.attlen = x;
@@ -870,35 +870,35 @@ POPUPMENUITEM *us_popupmenu(POPUPMENU **cmenu, INTSML *waitfordown, INTSML heade
 		us_pmcur.posy = shei - us_menubarsize - us_pmcur.sizey;
 
 	/* save the display under the menu */
-	save_pix = us_savebox(&us_pmcur.popupwindow, us_pmcur.posx,
+	save_pix = screensavebox(&us_pmcur.popupwindow, us_pmcur.posx,
 		(INTSML)(us_pmcur.posx+us_pmcur.sizex+MENUSHADOW),
 		(INTSML)(us_pmcur.posy-MENUSHADOW-1), (INTSML)(us_pmcur.posy+us_pmcur.sizey-1));
 
 	/* write the menu on the screen */
 	us_menufigs.col = el_colmengly;
 	us_menutext.col = el_colmentxt;
-	us_drawbox(&us_pmcur.popupwindow, us_pmcur.posx,
+	screendrawbox(&us_pmcur.popupwindow, us_pmcur.posx,
 		(INTSML)(us_pmcur.posx+us_pmcur.sizex-1),
 		us_pmcur.posy, (INTSML)(us_pmcur.posy+us_pmcur.sizey-1), &us_ebox);
-	us_drawline(&us_pmcur.popupwindow, us_pmcur.posx, us_pmcur.posy,
+	screendrawline(&us_pmcur.popupwindow, us_pmcur.posx, us_pmcur.posy,
 		(INTSML)(us_pmcur.posx+us_pmcur.sizex-1), us_pmcur.posy, &us_menufigs, 0);
-	us_drawline(&us_pmcur.popupwindow, (INTSML)(us_pmcur.posx+us_pmcur.sizex-1),
+	screendrawline(&us_pmcur.popupwindow, (INTSML)(us_pmcur.posx+us_pmcur.sizex-1),
 		us_pmcur.posy, (INTSML)(us_pmcur.posx+us_pmcur.sizex-1),
 		(INTSML)(us_pmcur.posy+us_pmcur.sizey-1), &us_menufigs, 0);
-	us_drawline(&us_pmcur.popupwindow, (INTSML)(us_pmcur.posx+us_pmcur.sizex-1),
+	screendrawline(&us_pmcur.popupwindow, (INTSML)(us_pmcur.posx+us_pmcur.sizex-1),
 		(INTSML)(us_pmcur.posy+us_pmcur.sizey-1), us_pmcur.posx,
 			(INTSML)(us_pmcur.posy+us_pmcur.sizey-1), &us_menufigs, 0);
-	us_drawline(&us_pmcur.popupwindow, us_pmcur.posx,
+	screendrawline(&us_pmcur.popupwindow, us_pmcur.posx,
 		(INTSML)(us_pmcur.posy+us_pmcur.sizey-1),
 		us_pmcur.posx, us_pmcur.posy, &us_menufigs, 0);
 
 	/* now draw a shadow */
 	us_shadowdesc.col = el_colmengly;
-	us_drawbox(&us_pmcur.popupwindow, (INTSML)(us_pmcur.posx+MENUSHADOW),
+	screendrawbox(&us_pmcur.popupwindow, (INTSML)(us_pmcur.posx+MENUSHADOW),
 		(INTSML)(us_pmcur.posx+us_pmcur.sizex), (INTSML)(us_pmcur.posy-MENUSHADOW-1),
 		(INTSML)(us_pmcur.posy-1), &us_shadowdesc);
 	if (quitline != 2)
-		us_drawbox(&us_pmcur.popupwindow, (INTSML)(us_pmcur.posx+us_pmcur.sizex),
+		screendrawbox(&us_pmcur.popupwindow, (INTSML)(us_pmcur.posx+us_pmcur.sizex),
 			(INTSML)(us_pmcur.posx+us_pmcur.sizex+MENUSHADOW),
 			(INTSML)(us_pmcur.posy-MENUSHADOW-1),
 			(INTSML)(us_pmcur.posy+us_pmcur.sizey-MENUSHADOW), &us_shadowdesc);
@@ -906,16 +906,16 @@ POPUPMENUITEM *us_popupmenu(POPUPMENU **cmenu, INTSML *waitfordown, INTSML heade
 	/* write the header */
 	if (header != 0)
 	{
-		us_drawline(&us_pmcur.popupwindow, us_pmcur.posx,
+		screendrawline(&us_pmcur.popupwindow, us_pmcur.posx,
 			(INTSML)(us_pmcur.posy+us_pmcur.sizey-us_pmcur.txthei-2),
 			(INTSML)(us_pmcur.posx+us_pmcur.sizex-1),
 			(INTSML)(us_pmcur.posy+us_pmcur.sizey-us_pmcur.txthei-2),
 			&us_menufigs, 0);
 		pt = us_stripampersand(us_pmcur.current->header);
-		us_puttext(&us_pmcur.popupwindow, (INTSML)(us_pmcur.posx+2),
+		screendrawtext(&us_pmcur.popupwindow, (INTSML)(us_pmcur.posx+2),
 			(INTSML)(us_pmcur.posy+us_pmcur.sizey-us_pmcur.txthei-1),
 			pt, &us_menutext);
-		us_invertbox(&us_pmcur.popupwindow, (INTSML)(us_pmcur.posx+1),
+		screeninvertbox(&us_pmcur.popupwindow, (INTSML)(us_pmcur.posx+1),
 			(INTSML)(us_pmcur.posx+us_pmcur.sizex-2),
 			(INTSML)(us_pmcur.posy+us_pmcur.sizey-us_pmcur.txthei-1),
 			(INTSML)(us_pmcur.posy+us_pmcur.sizey-2));
@@ -929,7 +929,7 @@ POPUPMENUITEM *us_popupmenu(POPUPMENU **cmenu, INTSML *waitfordown, INTSML heade
 		if (*mi->attribute == 0 && mi->value == 0)
 		{
 			/* no entry: draw a dotted line */
-			us_drawline(&us_pmcur.popupwindow, us_pmcur.posx, (INTSML)(yp+us_pmcur.txthei/2),
+			screendrawline(&us_pmcur.popupwindow, us_pmcur.posx, (INTSML)(yp+us_pmcur.txthei/2),
 				(INTSML)(us_pmcur.posx+us_pmcur.sizex-1), (INTSML)(yp+us_pmcur.txthei/2),
 				&us_menufigs, 1);
 		} else
@@ -939,12 +939,12 @@ POPUPMENUITEM *us_popupmenu(POPUPMENU **cmenu, INTSML *waitfordown, INTSML heade
 			if (mi->attribute[0] != '>' && *savepos == '<') *savepos = 0; else
 				savepos = 0;
 			pt = us_stripampersand(mi->attribute);
-			us_puttext(&us_pmcur.popupwindow, (INTSML)(us_pmcur.posx+2), yp, pt,
+			screendrawtext(&us_pmcur.popupwindow, (INTSML)(us_pmcur.posx+2), yp, pt,
 				&us_menutext);
 			if (savepos != 0) *savepos = '<';
 			if (mi->value != 0)
 			{
-				us_puttext(&us_pmcur.popupwindow,
+				screendrawtext(&us_pmcur.popupwindow,
 					(INTSML)(us_pmcur.posx+us_pmcur.attlen+ATTVALGAP+2),
 					yp, mi->value, &us_menutext);
 			}
@@ -968,7 +968,7 @@ POPUPMENUITEM *us_popupmenu(POPUPMENU **cmenu, INTSML *waitfordown, INTSML heade
 		us_pmdone, (INTSML)((quitline == 0 ? TRACKSELECTING : TRACKHSELECTING)));
 
 	/* restore what was under the menu */
-	(void)us_restorebox(save_pix, 1);
+	(void)screenrestorebox(save_pix, 1);
 
 	/* report the menu entry selected */
 	if (us_pmcur.inarea < 0) *waitfordown = 1;
@@ -1002,9 +1002,9 @@ INTSML us_pmeachchar(INTBIG x, INTBIG y, INTSML chr)
 		if (us_pmcur.curpos <= 0) return(0);
 		us_pmcur.curpos--;
 		mi->value[us_pmcur.curpos] = 0;
-		us_textsize(&us_pmcur.popupwindow, mi->value, &xs, &ys);
+		screengettextsize(&us_pmcur.popupwindow, mi->value, &xs, &ys);
 		us_pminvertline();
-		us_drawbox(&us_pmcur.popupwindow,
+		screendrawbox(&us_pmcur.popupwindow,
 			(INTSML)(us_pmcur.posx+2+us_pmcur.attlen+ATTVALGAP+xs),
 			(INTSML)(us_pmcur.posx+us_pmcur.sizex-1),
 			(INTSML)(us_pmcur.posy+us_pmcur.sizey-us_pmcur.txthei*(us_pmcur.hline+2)-2),
@@ -1021,7 +1021,7 @@ INTSML us_pmeachchar(INTBIG x, INTBIG y, INTSML chr)
 		us_pmcur.curpos = 0;
 		mi->value[us_pmcur.curpos] = 0;
 		us_pminvertline();
-		us_drawbox(&us_pmcur.popupwindow,
+		screendrawbox(&us_pmcur.popupwindow,
 			(INTSML)(us_pmcur.posx+2+us_pmcur.attlen+ATTVALGAP),
 			(INTSML)(us_pmcur.posx+us_pmcur.sizex-1),
 			(INTSML)(us_pmcur.posy+us_pmcur.sizey-us_pmcur.txthei*(us_pmcur.hline+2)-2),
@@ -1043,7 +1043,7 @@ INTSML us_pmeachchar(INTBIG x, INTBIG y, INTSML chr)
 		us_pmcur.curpos = 0;
 		mi->changed++;
 		us_pminvertline();
-		us_drawbox(&us_pmcur.popupwindow,
+		screendrawbox(&us_pmcur.popupwindow,
 			(INTSML)(us_pmcur.posx+2+us_pmcur.attlen+ATTVALGAP),
 			(INTSML)(us_pmcur.posx+us_pmcur.sizex-1),
 			(INTSML)(us_pmcur.posy+us_pmcur.sizey-us_pmcur.txthei*(us_pmcur.hline+2)-2),
@@ -1054,13 +1054,13 @@ INTSML us_pmeachchar(INTBIG x, INTBIG y, INTSML chr)
 	}
 
 	/* add a letter to a value field */
-	us_textsize(&us_pmcur.popupwindow, mi->value, &xs, &ys);
+	screengettextsize(&us_pmcur.popupwindow, mi->value, &xs, &ys);
 	i = us_pmcur.posx+2+us_pmcur.attlen+ATTVALGAP+xs;
 	mi->value[us_pmcur.curpos] = (char)chr;
 	us_pmcur.curpos++;
 	mi->value[us_pmcur.curpos] = 0;
 	us_pminvertline();
-	us_puttext(&us_pmcur.popupwindow, i,
+	screendrawtext(&us_pmcur.popupwindow, i,
 		(INTSML)(us_pmcur.posy+us_pmcur.sizey-us_pmcur.txthei*(us_pmcur.hline+2)-2),
 			&mi->value[us_pmcur.curpos-1], &us_menutext);
 	us_pminvertline();
@@ -1083,7 +1083,7 @@ INTSML us_pmeachdown(INTBIG x, INTBIG y)
 	maxy = us_pmcur.posy+us_pmcur.sizey-2;
 	if (us_pmcur.header != 0) maxy -= us_pmcur.txthei;
 
-	us_getpaletteparameters(&swid, &shei, &pwid);
+	getpaletteparameters(&swid, &shei, &pwid);
 
 	for(;;)
 	{
@@ -1193,7 +1193,7 @@ void us_pminvertline(void)
 
 	yp = us_pmcur.posy+us_pmcur.sizey-us_pmcur.txthei*(us_pmcur.hline+1)-2;
 	if (us_pmcur.header != 0) yp -= us_pmcur.txthei;
-	us_invertbox(&us_pmcur.popupwindow, (INTSML)(us_pmcur.posx+1),
+	screeninvertbox(&us_pmcur.popupwindow, (INTSML)(us_pmcur.posx+1),
 		(INTSML)(us_pmcur.posx+us_pmcur.sizex-2), yp, (INTSML)(yp+us_pmcur.txthei-1));
 }
 
