@@ -1532,7 +1532,8 @@ INTSML us_specialparse(char *prompt, COMCOMP *parameter, INTSML keycount, char *
 		us_technologycnsp, us_technologyclgp, us_technologyclsp, us_showdp, us_createxp,
 		us_nodetcaip, us_bindgkeyp, us_sizep, us_sizewp, us_defnodep, us_showup,
 		us_librarykp, us_findobjap, us_window3dp, us_gridp, us_technologytp,
-		us_renamecp, us_findnnamep, us_findexportp;
+		us_renamecp, us_findnnamep, us_findexportp, us_noyesalwaysp, us_varop,
+		us_copyfacetdp;
 
 	for(dh = us_firstdialoghook; dh != NODIALOGHOOK; dh = dh->nextdialoghook)
 		if (parameter == dh->getlinecomp)
@@ -1547,6 +1548,7 @@ INTSML us_specialparse(char *prompt, COMCOMP *parameter, INTSML keycount, char *
 	if (parameter == &us_technologycnsp)      return(us_areadlog(prompt, paramstart, 0));
 	if (parameter == &us_arrayxp)             return(us_arraydlog(paramstart));
 	if (parameter == &us_artlookp)            return(us_artlookdlog(paramstart));
+	if (parameter == &us_varop)               return(us_attributesdlog(paramstart));
 	if (parameter == &us_technologyclscp)     return(us_capacitancedlog(paramstart, 0));
 	if (parameter == &us_copyfacetp)          return(us_copyfacetdlog(prompt, paramstart));
 	if (parameter == &us_createxp)            return(us_createtodlog(paramstart));
@@ -1563,6 +1565,7 @@ INTSML us_specialparse(char *prompt, COMCOMP *parameter, INTSML keycount, char *
 	if (parameter == &us_gridp)               return(us_griddlog(paramstart));
 	if (parameter == &us_helpp)               return(us_helpdlog(prompt, paramstart));
 	if (parameter == &us_colorhighp)          return(us_highlayerlog(paramstart));
+	if (parameter == &us_copyfacetdp)         return(us_iconstyledlog(paramstart));
 	if (parameter == &us_technologyclsecp)    return(us_inductancedlog(paramstart, 0));
 	if (parameter == &us_lambdachp)           return(us_lambdadlog(paramstart));
 	if (parameter == &us_librarydp)           return(us_librarypathdlog(paramstart));
@@ -1570,6 +1573,7 @@ INTSML us_specialparse(char *prompt, COMCOMP *parameter, INTSML keycount, char *
 	if (parameter == &us_viewn1p)             return(us_newviewdlog(paramstart));
 	if (parameter == &us_sizep)               return(us_nodesizedlog(paramstart));
 	if (parameter == &us_noyesp)              return(us_noyesdlog(prompt, paramstart));
+	if (parameter == &us_noyesalwaysp)        return(us_noyesalwaysdlog(prompt, paramstart));
 	if (parameter == &us_librarykp)           return(us_oldlibrarydlog(paramstart));
 	if (parameter == &us_librarywp)           return(us_optionsavingdlog(paramstart));
 	if (parameter == &us_colorpvaluep)        return(us_patterndlog(paramstart));
