@@ -31,6 +31,7 @@
 #include "global.h"
 #include "egraphics.h"
 #include "usr.h"
+#include "drc.h"
 #include "network.h"
 #include "efunction.h"
 #include "tecgen.h"
@@ -355,7 +356,10 @@ void us_setfacetname(WINDOWPART *w)
 	REGISTER char *npname;
 	REGISTER INTSML len;
 	REGISTER EDITOR *ed;
-	REGISTER LIBRARY *savelib;
+
+	/* do not show name of explorer window when it is split */
+	if ((w->state&WINDOWTYPE) == EXPLORERWINDOW &&
+		strcmp(w->location, "entire") != 0) return;
 
 	/* write the facet name */
 	(void)initinfstr();
@@ -374,23 +378,20 @@ void us_setfacetname(WINDOWPART *w)
 		case DISP3DWINDOW:
 			if (w->curnodeproto == NONODEPROTO) len = 0; else
 			{
-				savelib = el_curlib;
-				el_curlib = NOLIBRARY;
 				npname = describenodeproto(w->curnodeproto);
-				el_curlib = savelib;
 				(void)addstringtoinfstr(npname);
 				len = strlen(npname);
 			}
 			break;
+		case EXPLORERWINDOW:
+			(void)addstringtoinfstr("Facet Explorer");
+			break;
 		case WAVEFORMWINDOW:
 			(void)addstringtoinfstr("Waveform");
 			len = 8;
 			if (w->curnodeproto != NONODEPROTO)
 			{
-				savelib = el_curlib;
-				el_curlib = NOLIBRARY;
 				npname = describenodeproto(w->curnodeproto);
-				el_curlib = savelib;
 				(void)addstringtoinfstr(" of ");
 				(void)addstringtoinfstr(npname);
 				len += strlen(npname) + 4;
@@ -462,7 +463,7 @@ void us_redostatus(WINDOWFRAME *frame)
 	for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
 	{
 		us_setfacetname(w);
-		if (us_graphicshas(CANSTATUSPERFRAME) != 0 || w->frame == frame)
+		if (graphicshas(CANSTATUSPERFRAME) != 0 || w->frame == frame)
 			us_setfacetsize(w);
 		us_setgridsize(w);
 	}
@@ -682,13 +683,17 @@ void us_describecontents(NODEINST *ni)
 {
 	REGISTER INTSML i;
 	REGISTER NODEPROTO *np;
+	REGISTER LIBRARY *lib;
 	REGISTER TECHNOLOGY *curtech, *printtech, *tech;
 	REGISTER NODEPROTO **sorted;
 
 	/* first zero the count of each nodeproto */
 	for(tech = el_technologies; tech != NOTECHNOLOGY; tech = tech->nexttechnology)
-		for(np = tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto) np->temp1 = 0;
-	for(np = el_curlib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto) np->temp1 = 0;
+		for(np = tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+			np->temp1 = 0;
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+			np->temp1 = 0;
 
 	/* now look at every object recursively in this facet */
 	us_addobjects(ni->proto);
@@ -793,7 +798,7 @@ void us_addobjects(NODEPROTO *np)
  * routine to sort the facets in library "lib" by facet name and return an
  * array of NODEPROTO pointers (terminating with NONODEPROTO).  If "matchspec"
  * is nonzero, it is a string that forces wildcard matching of facet names.
- * If "shortlist" is nonzero, only list one facet of every facet.  Remember
+ * If "shortlist" is nonzero, only list one facet of every cell.  Remember
  * to free the returned array when done.  Routine returns zero upon error.
  */
 NODEPROTO **us_sortlib(LIBRARY *lib, char *matchspec, INTSML shortlist)
@@ -1171,6 +1176,12 @@ char *us_keyname(INTSML key)
 	pt = line;
 	if (key >= 0200)
 	{
+		if (key >= FUNCTIONF1 && key <= FUNCTIONF12)
+		{
+			sprintf(pt, "F%ld", key - FUNCTIONF1 + 1);
+			return(line);
+		}
+
 		*pt++ = 'M';
 		*pt++ = '-';
 		key -= 0200;
@@ -1241,15 +1252,18 @@ void us_printcolorvalue(INTSML ind)
  * routine to make a line of text that describes facet "np".  The text
  * includes the facet name, version, creation, and revision dates.  The
  * amount of space to leave for the facet name is "maxlen" characters
+ * (which is negative if dumping to a file).
  */
 char *us_makefacetline(NODEPROTO *np, INTSML maxlen)
 {
-	char line[20];
-	REGISTER INTSML l, i;
+	char line[40];
+	REGISTER INTSML l, i, gooddrc;
+	INTSML year, month, mday, hour, minute, second;
 	REGISTER INTBIG len, total;
 	REGISTER char *pt;
 	REGISTER VARIABLE *var;
 	REGISTER NODEINST *ni;
+	REGISTER UINTBIG lastgooddate, lastchangedate;
 
 	(void)initinfstr();
 	pt = nldescribenodeproto(np);
@@ -1273,16 +1287,24 @@ char *us_makefacetline(NODEPROTO *np, INTSML maxlen)
 			(void)addstringtoinfstr("     UNRECORDED     ");
 	} else
 	{
-		pt = timetostring(np->creationdate);
-		if (pt == NULL)
+		if (maxlen < 0)
 		{
-			if (maxlen < 0) (void)addstringtoinfstr("UNAVAILABLE"); else
-				(void)addstringtoinfstr("     UNAVAILABLE     ");
+			parsetime(np->creationdate, &year, &month, &mday, &hour, &minute, &second);
+			sprintf(line, "%4d-%02d-%02d %02d:%02d:%02d", year+1900, month, mday, hour,
+				minute, second);
+			(void)addstringtoinfstr(line);
 		} else
 		{
-		    for(i=4; i<=9; i++) (void)addtoinfstr(pt[i]);
-		    for(i=19; i<=23; i++) (void)addtoinfstr(pt[i]);
-		    for(i=10; i<=18; i++) (void)addtoinfstr(pt[i]);
+			pt = timetostring(np->creationdate);
+			if (pt == NULL)
+			{
+				(void)addstringtoinfstr("     UNAVAILABLE     ");
+			} else
+			{
+				for(i=4; i<=9; i++) (void)addtoinfstr(pt[i]);
+				for(i=19; i<=23; i++) (void)addtoinfstr(pt[i]);
+				for(i=10; i<=18; i++) (void)addtoinfstr(pt[i]);
+			}
 		}
 	}
 
@@ -1295,16 +1317,24 @@ char *us_makefacetline(NODEPROTO *np, INTSML maxlen)
 			(void)addstringtoinfstr("     UNRECORDED     ");
 	} else
 	{
-		pt = timetostring(np->revisiondate);
-		if (pt == NULL)
+		if (maxlen < 0)
 		{
-			if (maxlen < 0) (void)addstringtoinfstr("UNAVAILABLE"); else
-			    (void)addstringtoinfstr("     UNAVAILABLE     ");
+			parsetime(np->revisiondate, &year, &month, &mday, &hour, &minute, &second);
+			sprintf(line, "%4d-%02d-%02d %02d:%02d:%02d", year+1900, month, mday, hour,
+				minute, second);
+			(void)addstringtoinfstr(line);
 		} else
 		{
-		    for(i=4; i<=9; i++) (void)addtoinfstr(pt[i]);
-		    for(i=19; i<=23; i++) (void)addtoinfstr(pt[i]);
-		    for(i=10; i<=18; i++) (void)addtoinfstr(pt[i]);
+			pt = timetostring(np->revisiondate);
+			if (pt == NULL)
+			{
+				(void)addstringtoinfstr("     UNAVAILABLE     ");
+			} else
+			{
+				for(i=4; i<=9; i++) (void)addtoinfstr(pt[i]);
+				for(i=19; i<=23; i++) (void)addtoinfstr(pt[i]);
+				for(i=10; i<=18; i++) (void)addtoinfstr(pt[i]);
+			}
 		}
 	}
 
@@ -1317,7 +1347,7 @@ char *us_makefacetline(NODEPROTO *np, INTSML maxlen)
 		if (var == NOVARIABLE) len = 0; else
 			len = getlength(var);
 		if (maxlen < 0) sprintf(line, "%ld lines", len); else
-			sprintf(line, "%8ld lines", len);
+			sprintf(line, "%8ld lines   ", len);
 		(void)addstringtoinfstr(line);
 	} else
 	{
@@ -1357,6 +1387,18 @@ char *us_makefacetline(NODEPROTO *np, INTSML maxlen)
 		(void)addstringtoinfstr(" ");
 	if ((np->userbits&INCELLLIBRARY) != 0) (void)addstringtoinfstr("C"); else
 		(void)addstringtoinfstr(" ");
+	if (maxlen < 0) (void)addtoinfstr('\t'); else
+		(void)addstringtoinfstr(" ");
+	gooddrc = 0;
+	var = getvalkey((INTBIG)np, VNODEPROTO, VINTEGER, dr_lastgooddrckey);
+	if (var != NOVARIABLE)
+	{
+		lastgooddate = (UINTBIG)var->addr;
+		lastchangedate = np->revisiondate;
+		if (lastchangedate <= lastgooddate) gooddrc = 1;
+	}
+	if (gooddrc != 0) (void)addstringtoinfstr("D"); else
+		(void)addstringtoinfstr(" ");
 	return(returninfstr());
 }
 
@@ -1457,16 +1499,11 @@ void us_saveoptions(void)
 {
 	REGISTER INTBIG retval;
 	REGISTER LIBRARY *l;
-	REGISTER char *libname, *libfile, *hd;
+	REGISTER char *libname, *libfile;
 	extern AIDENTRY *io_aid;
 
 	libname = us_tempoptionslibraryname();
-	(void)initinfstr();
-	hd = hashomedir();
-	if (hd == 0) (void)addstringtoinfstr(el_libdir); else
-		(void)addstringtoinfstr(hd);
-	(void)addstringtoinfstr(OPTIONSFILE);
-	libfile = returninfstr();
+	libfile = optionsfilepath();
 	l = newlibrary(libname, libfile);
 	if (l == NOLIBRARY)
 	{
@@ -1599,9 +1636,11 @@ char *us_uniqueportname(char *name, NODEPROTO *facet)
 		baselen--;
 		trailingpart = &name[baselen];
 		while (baselen > 0 && isdigit(name[baselen-1]) != 0) baselen--;
-		if (name[baselen] == ']' || baselen == 0 || name[baselen-1] != '[')
+		for(i=baselen-1; i>0; i--)
+			if (name[i] == '[' || name[i] == ':' || name[i] == ',') break;
+		if (name[i] != '[')
 		{
-			/* did not find a valid number inside square brackets */
+			/* did not find a single entry inside square brackets */
 			while (baselen > 0 && name[baselen] != '[') baselen--;
 			if (baselen == 0) baselen = strlen(name); else
 			{
