@@ -39,6 +39,7 @@
 #include "efunction.h"
 #include "tecart.h"
 #include "tecgen.h"
+#include "sim.h"
 
 #define	MAXTRACE 400
 #define MINSLIDEDELAY 60			/* ticks between shifts of window when dragging to edge */
@@ -52,12 +53,15 @@ static INTBIG us_dragx, us_dragy, us_dragox, us_dragoy, us_dragoffx,
 	us_dragnodetotal, us_dragdescript, us_cantdrag, us_draglowval, us_draghighval;
 static INTSML us_dragshown, us_dragangle, us_dragstate, us_dragfport, us_dragextra,
 	us_dragnobox, us_dragstayonhigh, us_dragstill, us_dragcorner, us_dragspecial;
+static INTBIG      us_arrowamount;				/* for slider tracking */
+static INTBIG      us_arrowlx, us_arrowhx, us_arrowly, us_arrowhy;
 static WINDOWPART *us_dragwindow;
 static POLYGON    *us_dragpoly = NOPOLYGON;
 static NODEPROTO  *us_dragnodeproto;
 static NODEINST   *us_dragnodeinst, **us_dragnodelist;
 static ARCINST    *us_dragarcinst;
 static GEOM       *us_dragobject, **us_dragobjectlist;
+static INTBIG      us_dragaddr, us_dragtype;
 static HIGHLIGHT   us_draghigh;
 static char        us_dragmessage[100];
 static UINTBIG     us_beforepantime = 0;
@@ -86,7 +90,7 @@ INTSML us_tracedown(INTBIG x, INTBIG y)
 {
 	if (us_tracelist >= MAXTRACE) return(1);
 	if (us_tracelist != 0)
-		us_drawline(el_curwindowpart, (INTSML)us_tracedata[us_tracelist-2], (INTSML)us_tracedata[us_tracelist-1],
+		screendrawline(el_curwindowpart, (INTSML)us_tracedata[us_tracelist-2], (INTSML)us_tracedata[us_tracelist-1],
 			(INTSML)x, (INTSML)y, &us_highl, 0);
 	us_tracedata[us_tracelist++] = x;
 	us_tracedata[us_tracelist++] = y;
@@ -108,7 +112,7 @@ void us_traceup(void)
 		(INTBIG)(VINTEGER|VISARRAY|(us_tracelist<<VLENGTHSH)|VDONTSAVE));
 	us_highl.col = 0;
 	for(i=2; i<us_tracelist; i += 2)
-		us_drawline(el_curwindowpart, (INTSML)us_tracedata[i-2], (INTSML)us_tracedata[i-1],
+		screendrawline(el_curwindowpart, (INTSML)us_tracedata[i-2], (INTSML)us_tracedata[i-1],
 			(INTSML)us_tracedata[i], (INTSML)us_tracedata[i+1], &us_highl, 0);
 }
 
@@ -152,6 +156,7 @@ void us_findpointbegin(void)
 	if (highvar == NOVARIABLE) return;
 	(void)us_makehighlight(((char **)highvar->addr)[0], &newhigh);
 	newhigh.fromport = NOPORTPROTO;
+	newhigh.facet = getcurfacet();
 	us_setfind(&newhigh, 1, 0, 0, 0);
 
 	/* get the point to be moved */
@@ -376,8 +381,9 @@ INTSML us_textgrabdown(INTBIG x, INTBIG y)
 	}
 
 	/* convert the text descriptor into a POLYGON */
-	us_buildtexthighpoly(us_dragobject, us_dragx, us_dragy, us_dragox, us_dragoy,
-		style, us_dragpoly);
+	us_buildtexthighpoly(us_dragobject->lowx, us_dragobject->highx,
+		us_dragobject->lowy, us_dragobject->highy, us_dragx, us_dragy, us_dragox, us_dragoy,
+			style, us_dragpoly);
 
 	/* draw the new box */
 	us_highl.col = HIGHLIT;
@@ -396,7 +402,9 @@ void us_textmoveinit(INTBIG xc, INTBIG yc, INTBIG xw, INTBIG yw, INTBIG descript
 	us_dragoffx = xc;   us_dragoffy = yc;
 	us_dragox = xw;     us_dragoy = yw;
 	us_dragdescript = descript;
-	us_dragobject = geom;
+	us_dragaddr = (INTBIG)geom->entryaddr.blind;
+	if (geom->entrytype == OBJNODEINST) us_dragtype = VNODEINST; else
+		us_dragtype = VARCINST;
 }
 
 /* initialization routine when positioning text */
@@ -414,6 +422,8 @@ void us_textmovebegin(void)
 INTSML us_textmovedown(INTBIG x, INTBIG y)
 {
 	REGISTER INTBIG xcur, ycur, lambda;
+	INTBIG lx, hx, ly, hy;
+	REGISTER TECHNOLOGY *tech;
 
 	/* get the cursor value */
 	if (us_setxy((INTSML)x, (INTSML)y) != 0 || us_dragwindow != el_curwindowpart)
@@ -436,10 +446,12 @@ INTSML us_textmovedown(INTBIG x, INTBIG y)
 	/* convert the text descriptor into a GRAPHICS style */
 	us_dragpoly->count = 1;
 	us_dragpoly->xv[0] = us_dragpoly->yv[0] = 0;
-	adjustdisoffset(us_dragobject, us_dragpoly, us_dragdescript);
+
+	tech = us_getobjectinfo(us_dragaddr, us_dragtype, &lx, &hx, &ly, &hy);
+	adjustdisoffset(us_dragaddr, us_dragtype, tech, us_dragpoly, us_dragdescript);
 
 	/* clip range of text descriptor */
-	lambda = figurelambda(us_dragobject);
+	lambda = tech->deflambda;
 	xcur = (us_lastcurx - us_dragx) * 4 / lambda;
 	us_lastcurx = us_lastcurx - us_dragx + us_dragoffx;
 	if (xcur > 1023)
@@ -454,7 +466,7 @@ INTSML us_textmovedown(INTBIG x, INTBIG y)
 				us_lastcury = -1023 * lambda / 4 + us_dragoffy;
 
 	/* draw the descriptor */
-	us_buildtexthighpoly(us_dragobject, us_lastcurx, us_lastcury,
+	us_buildtexthighpoly(lx, hx, ly, hy, us_lastcurx, us_lastcury,
 		us_dragox, us_dragoy, us_dragpoly->style, us_dragpoly);
 
 	/* draw the new box */
@@ -952,7 +964,7 @@ void us_finddoibegin(INTSML compl)
 		}
 		us_findobject(xcur, ycur, el_curwindowpart, &newhigh, 0, another, us_dragfport, 1, us_dragspecial);
 	}
-	if (newhigh.fromgeom == NOGEOM)
+	if (newhigh.status == 0)
 	{
 		/* nothing under cursor: select an area */
 		if (compl != 0) us_dragstate = 2; else us_dragstate = 1;
@@ -983,6 +995,7 @@ void us_finddoibegin(INTSML compl)
 	if (compl == 0)
 	{
 		/* normal find/move */
+		newhigh.facet = getcurfacet();
 		if (i >= len || us_dragpoint >= 0)
 		{
 			/* object is not highlighted: select only it and move */
@@ -991,7 +1004,6 @@ void us_finddoibegin(INTSML compl)
 		} else
 		{
 			/* object is already highlighted: rehighlight and move */
-			newhigh.facet = geomparent(newhigh.fromgeom);
 			(void)us_delhighlight(&newhigh);
 			(void)us_addhighlight(&newhigh);
 		}
@@ -1033,29 +1045,11 @@ void us_finddoibegin(INTSML compl)
 		if ((us_draghigh.status&HIGHTYPE) == HIGHTEXT)
 		{
 			/* get descriptor and text string */
-			us_dragnodeinst = NONODEINST;
-			if (us_draghigh.fromvar != NOVARIABLE)
-			{
-				if (us_draghigh.fromgeom->entrytype == OBJNODEINST)
-					us_dragnodeinst = us_draghigh.fromgeom->entryaddr.ni;
-				us_dragdescript = us_draghigh.fromvar->textdescript;
-				str = describevariable(us_draghigh.fromvar, -1, -1);
-				if ((us_draghigh.fromvar->type&VISARRAY) == 0)
-				{
-					j = (INTSML)((us_draghigh.fromvar->type&VLENGTH) >> VLENGTHSH);
-					if (j < (INTBIG)strlen(str)) str += j;
-				}
-			} else if (us_draghigh.fromport != NOPORTPROTO)
-			{
-				us_dragnodeinst = us_draghigh.fromgeom->entryaddr.ni;
-				us_dragdescript = us_draghigh.fromport->textdescript;
-				str = us_draghigh.fromport->protoname;
-			} else if (us_draghigh.fromgeom->entrytype == OBJNODEINST)
-			{
-				us_dragnodeinst = us_draghigh.fromgeom->entryaddr.ni;
-				us_dragdescript = us_dragnodeinst->textdescript;
-				str = describenodeproto(us_dragnodeinst->proto);
-			}
+			us_gethighaddrtype(&us_draghigh, &us_dragaddr, &us_dragtype);
+			us_dragnodeinst = us_gethighnodeinst(&us_draghigh);
+			tech = us_hightech(&us_draghigh);
+			us_dragdescript = us_gethighdescript(&us_draghigh);
+			str = us_gethighstring(&us_draghigh);
 
 			/* determine number of lines of text and text size */
 			len = 1;
@@ -1069,12 +1063,9 @@ void us_finddoibegin(INTSML compl)
 				yw = hy - ly;
 			} else
 			{
-				if (us_draghigh.fromgeom->entrytype == OBJARCINST)
-					tech = us_draghigh.fromgeom->entryaddr.ai->proto->tech; else
-						tech = whattech(us_draghigh.fromgeom->entryaddr.ni->proto);
-				us_settextsize(el_curwindowpart, truefontsize((INTSML)((us_dragdescript&VTSIZE)>>VTSIZESH),
+				screensettextsize(el_curwindowpart, truefontsize((INTSML)((us_dragdescript&VTSIZE)>>VTSIZESH),
 					el_curwindowpart, tech));
-				us_textsize(el_curwindowpart, str, &tsx, &tsy);
+				screengettextsize(el_curwindowpart, str, &tsx, &tsy);
 				xw = muldiv(tsx, el_curwindowpart->screenhx-el_curwindowpart->screenlx,
 					el_curwindowpart->usehx-el_curwindowpart->uselx);
 				yw = muldiv(tsy, el_curwindowpart->screenhy-el_curwindowpart->screenly,
@@ -1082,7 +1073,6 @@ void us_finddoibegin(INTSML compl)
 			}
 
 			/* get offset */
-			us_dragobject = us_draghigh.fromgeom;
 			us_gethightextcenter(&us_draghigh, &xc, &yc, &j);
 			us_dragoffx = xc;  us_dragoffy = yc;
 			us_dragox = xw;    us_dragoy = yw;
@@ -1538,7 +1528,8 @@ void us_findiup(void)
 
 		/* move entire node if it is an "invisible pin" */
 		if (us_dragnodeinst != NONODEINST &&
-			us_dragnodeinst->proto == gen_invispinprim)
+			us_dragnodeinst->proto == gen_invispinprim &&
+			us_dragtype == VNODEINST)
 		{
 			startobjectchange((INTBIG)us_dragnodeinst, VNODEINST);
 			modifynodeinst(us_dragnodeinst, dx, dy, dx, dy, 0, 0);
@@ -1548,8 +1539,16 @@ void us_findiup(void)
 		}
 
 		/* undraw the text */
-		startobjectchange((INTBIG)us_draghigh.fromgeom->entryaddr.blind,
-			us_draghigh.fromgeom->entrytype == OBJNODEINST ? VNODEINST : VARCINST);
+		if (us_dragtype == VPORTPROTO)
+		{
+			startobjectchange((INTBIG)(((PORTPROTO *)us_dragaddr)->subnodeinst),
+				VNODEINST);
+		} else
+		{
+			startobjectchange(us_dragaddr, us_dragtype);
+		}
+		if (us_dragtype == VNODEPROTO && us_draghigh.fromvar != NOVARIABLE)
+			us_undrawfacetvariable(us_draghigh.fromvar, (NODEPROTO *)us_dragaddr);
 
 		/* set the new descriptor on the text */
 		if (us_dragnodeinst != NONODEINST)
@@ -1559,7 +1558,7 @@ void us_findiup(void)
 					makeangle((INTSML)((3600-us_dragnodeinst->rotation)%3600), 0, trans);
 			xform(dx, dy, &dx, &dy, trans);
 		}
-		lambda = figurelambda(us_draghigh.fromgeom);
+		lambda = us_hightech(&us_draghigh)->deflambda;
 		dx = dx*4/lambda;   dy = dy*4/lambda;
 		if ((us_dragdescript&VTXOFFNEG) != 0) dx -= (us_dragdescript&VTXOFF) >> VTXOFFSH; else
 			dx += (us_dragdescript&VTXOFF) >> VTXOFFSH;
@@ -1569,8 +1568,16 @@ void us_findiup(void)
 		us_modifytextdescript(&us_draghigh, us_dragdescript);
 
 		/* redisplay the text */
-		endobjectchange((INTBIG)us_draghigh.fromgeom->entryaddr.blind,
-			us_draghigh.fromgeom->entrytype == OBJNODEINST ? VNODEINST : VARCINST);
+		if (us_dragtype == VNODEPROTO && us_draghigh.fromvar != NOVARIABLE)
+			us_drawfacetvariable(us_draghigh.fromvar, (NODEPROTO *)us_dragaddr);
+		if (us_dragtype == VPORTPROTO)
+		{
+			endobjectchange((INTBIG)(((PORTPROTO *)us_dragaddr)->subnodeinst),
+				VNODEINST);
+		} else
+		{
+			endobjectchange(us_dragaddr, us_dragtype);
+		}
 		(void)us_addhighlight(&us_draghigh);
 
 		/* modify all higher-level nodes if port moved */
@@ -1664,7 +1671,7 @@ void us_createbegin(void)
 	us_dragpoly->desc = &us_highl;
 
 	(void)getxy(&xcur, &ycur);
-	curframe = us_getwindowframe(1);
+	curframe = getwindowframe(1);
 	if ((us_aid->aidstate&MENUON) != 0)
 	{
 		if (curframe == us_menuframe ||
@@ -1871,6 +1878,7 @@ void us_multidraginit(INTBIG xc, INTBIG yc, GEOM **geomlist, NODEINST **nodelist
 	us_dragnodetotal = total;
 	us_dragangle = angle;
 	us_dragextra = extra;
+	us_cantdrag = 0;
 }
 
 /* initialization routine when duplicating objects */
@@ -2012,9 +2020,9 @@ void us_multidragdraw(INTBIG col, INTBIG dx, INTBIG dy)
 					yw = ni->geom->highy - ni->geom->lowy;
 				} else
 				{
-					us_settextsize(el_curwindowpart, truefontsize((INTSML)((var->textdescript&VTSIZE)>>VTSIZESH),
+					screensettextsize(el_curwindowpart, truefontsize((INTSML)((var->textdescript&VTSIZE)>>VTSIZESH),
 						el_curwindowpart, ni->proto->tech));
-					us_textsize(el_curwindowpart, str, &tsx, &tsy);
+					screengettextsize(el_curwindowpart, str, &tsx, &tsy);
 					xw = muldiv(tsx, el_curwindowpart->screenhx-el_curwindowpart->screenlx,
 						el_curwindowpart->usehx-el_curwindowpart->uselx);
 					yw = muldiv(tsy, el_curwindowpart->screenhy-el_curwindowpart->screenly,
@@ -2026,7 +2034,7 @@ void us_multidragdraw(INTBIG col, INTBIG dx, INTBIG dy)
 				us_dragpoly->style = FILLED;
 				us_dragpoly->xv[0] = (ni->lowx + ni->highx) / 2;
 				us_dragpoly->yv[0] = (ni->lowy + ni->highy) / 2;
-				adjustdisoffset(ni->geom, us_dragpoly, var->textdescript);
+				adjustdisoffset((INTBIG)ni, VNODEINST, whattech(ni->proto), us_dragpoly, var->textdescript);
 				makerot(ni, trans);
 				xformpoly(us_dragpoly, trans);
 				getcenter(us_dragpoly, &xc, &yc);
@@ -2043,7 +2051,9 @@ void us_multidragdraw(INTBIG col, INTBIG dx, INTBIG dy)
 					if (ycur < -1023) yc = -1023 * lambda / 4;
 
 				/* draw the descriptor */
-				us_buildtexthighpoly(ni->geom, xc, yc, xw, yw, us_dragpoly->style, us_dragpoly);
+				us_buildtexthighpoly(ni->geom->lowx, ni->geom->highx,
+					ni->geom->lowy, ni->geom->highy, xc, yc, xw, yw,
+						us_dragpoly->style, us_dragpoly);
 
 				/* draw the text highlight */
 				(void)us_showpoly(us_dragpoly, us_dragwindow);
@@ -2317,7 +2327,12 @@ void us_distanceup(void)
 
 /************************* SLIDER THUMB TRACKING *************************/
 
-void us_hthumbbegin(INTBIG x, WINDOWPART *w)
+/*
+ * initialization method for tracking the horizontal thumb.  The cursor X coordinate
+ * is "x" and the window is "w".  The horizontal slider's top side is "hy" and it
+ * runs horizontally from "lx" to "hx" (including arrows).
+ */
+void us_hthumbbegin(INTBIG x, WINDOWPART *w, INTBIG hy, INTBIG lx, INTBIG hx)
 {
 	/* initialize polygon */
 	if (us_dragpoly == NOPOLYGON) us_dragpoly = allocpolygon(4, us_aid->cluster);
@@ -2327,15 +2342,16 @@ void us_hthumbbegin(INTBIG x, WINDOWPART *w)
 	us_dragwindow = w;
 	us_initthumbcoord = x;
 
-	us_trackww.screenlx = us_trackww.uselx = w->uselx;
-	us_trackww.screenhx = us_trackww.usehx = w->usehx+DISPLAYSLIDERSIZE;
-	us_trackww.screenly = us_trackww.usely = w->usely-DISPLAYSLIDERSIZE;
-	us_trackww.screenhy = us_trackww.usehy = w->usehy;
+	us_trackww.screenlx = us_trackww.uselx = (INTSML)lx;
+	us_trackww.screenhx = us_trackww.usehx = (INTSML)hx;
+	us_trackww.screenly = us_trackww.usely = (INTSML)(hy-DISPLAYSLIDERSIZE);
+	us_trackww.screenhy = us_trackww.usehy = (INTSML)hy;
 	us_trackww.frame = w->frame;
 	us_trackww.state = DISPWINDOW;
 	computewindowscale(&us_trackww);
 }
 
+/* tracking method for the horizontal thumb */
 INTSML us_hthumbdown(INTBIG x, INTBIG y)
 {
 	if (us_dragshown != 0)
@@ -2346,9 +2362,9 @@ INTSML us_hthumbdown(INTBIG x, INTBIG y)
 		/* undraw the outline */
 		us_highl.col = 0;
 		us_dragpoly->xv[0] = us_dragwindow->thumblx + (us_lastcurx-us_initthumbcoord)+1;
-		us_dragpoly->yv[0] = us_dragwindow->usely-3;
+		us_dragpoly->yv[0] = us_trackww.usehy-3;
 		us_dragpoly->xv[1] = us_dragwindow->thumbhx + (us_lastcurx-us_initthumbcoord)-1;
-		us_dragpoly->yv[1] = us_dragwindow->usely-DISPLAYSLIDERSIZE+1;
+		us_dragpoly->yv[1] = us_trackww.usehy-DISPLAYSLIDERSIZE+1;
 		us_dragpoly->count = 2;
 		us_dragpoly->style = FILLEDRECT;
 		(void)us_showpoly(us_dragpoly, &us_trackww);
@@ -2356,14 +2372,14 @@ INTSML us_hthumbdown(INTBIG x, INTBIG y)
 
 	/* draw the outline */
 	us_highl.col = HIGHLIT;
-	if (us_dragwindow->thumblx + (x-us_initthumbcoord) <= us_dragwindow->uselx+DISPLAYSLIDERSIZE)
-		 x = us_dragwindow->uselx+DISPLAYSLIDERSIZE+1 - us_dragwindow->thumblx + us_initthumbcoord;
-	if (us_dragwindow->thumbhx + (x-us_initthumbcoord) >= us_dragwindow->usehx-DISPLAYSLIDERSIZE)
-		 x = us_dragwindow->usehx-DISPLAYSLIDERSIZE-1 - us_dragwindow->thumbhx + us_initthumbcoord;
+	if (us_dragwindow->thumblx + (x-us_initthumbcoord) <= us_trackww.uselx+DISPLAYSLIDERSIZE)
+		 x = us_trackww.uselx+DISPLAYSLIDERSIZE+1 - us_dragwindow->thumblx + us_initthumbcoord;
+	if (us_dragwindow->thumbhx + (x-us_initthumbcoord) >= us_trackww.usehx-DISPLAYSLIDERSIZE)
+		 x = us_trackww.usehx-DISPLAYSLIDERSIZE-1 - us_dragwindow->thumbhx + us_initthumbcoord;
 	us_dragpoly->xv[0] = us_dragwindow->thumblx + (x-us_initthumbcoord)+1;
-	us_dragpoly->yv[0] = us_dragwindow->usely-3;
+	us_dragpoly->yv[0] = us_trackww.usehy-3;
 	us_dragpoly->xv[1] = us_dragwindow->thumbhx + (x-us_initthumbcoord)-1;
-	us_dragpoly->yv[1] = us_dragwindow->usely-DISPLAYSLIDERSIZE+1;
+	us_dragpoly->yv[1] = us_trackww.usehy-DISPLAYSLIDERSIZE+1;
 	us_dragpoly->count = 2;
 	us_dragpoly->style = FILLEDRECT;
 	(void)us_showpoly(us_dragpoly, &us_trackww);
@@ -2373,6 +2389,7 @@ INTSML us_hthumbdown(INTBIG x, INTBIG y)
 	return(0);
 }
 
+/* completion method for tracking the horozintal thumb in a regular edit window */
 void us_hthumbdone(void)
 {
 	REGISTER NODEPROTO *np;
@@ -2383,9 +2400,9 @@ void us_hthumbdone(void)
 		/* undraw the outline */
 		us_highl.col = 0;
 		us_dragpoly->xv[0] = us_dragwindow->thumblx + (us_lastcurx-us_initthumbcoord)+1;
-		us_dragpoly->yv[0] = us_dragwindow->usely-3;
+		us_dragpoly->yv[0] = us_trackww.usehy-3;
 		us_dragpoly->xv[1] = us_dragwindow->thumbhx + (us_lastcurx-us_initthumbcoord)-1;
-		us_dragpoly->yv[1] = us_dragwindow->usely-DISPLAYSLIDERSIZE+1;
+		us_dragpoly->yv[1] = us_trackww.usehy-DISPLAYSLIDERSIZE+1;
 		us_dragpoly->count = 2;
 		us_dragpoly->style = FILLEDRECT;
 		(void)us_showpoly(us_dragpoly, &us_trackww);
@@ -2408,7 +2425,41 @@ void us_hthumbdone(void)
 	}
 }
 
-void us_vthumbbegin(INTBIG y, WINDOWPART *w)
+/* completion method for tracking the horizontal thumb in a text-edit window */
+void us_ehthumbdone(void)
+{
+	if (us_dragshown != 0)
+	{
+		/* undraw the outline */
+		us_highl.col = 0;
+		us_dragpoly->xv[0] = us_dragwindow->thumblx + (us_lastcurx-us_initthumbcoord)+1;
+		us_dragpoly->yv[0] = us_trackww.usehy-3;
+		us_dragpoly->xv[1] = us_dragwindow->thumbhx + (us_lastcurx-us_initthumbcoord)-1;
+		us_dragpoly->yv[1] = us_trackww.usehy-DISPLAYSLIDERSIZE+1;
+		us_dragpoly->count = 2;
+		us_dragpoly->style = FILLEDRECT;
+		(void)us_showpoly(us_dragpoly, &us_trackww);
+
+		us_dragpoint = (us_dragwindow->thumblx + us_dragwindow->thumbhx) / 2 +
+			(us_lastcurx-us_initthumbcoord);
+	}
+}
+
+/* return method for reporting the new horizontal thumb in a text-edit window */
+INTBIG us_ehthumbend(void)
+{
+	return(us_dragpoint);
+}
+
+/*
+ * initialization method for tracking the vertical thumb.  The cursor Y coordinate
+ * is "y" and the window is "w".  The vertical slider's left side is "hx" and it
+ * runs vertically from "ly" to "hy" (including arrows).  The total number of
+ * lines of text is "totallines" and the slider is on the left side if "onleft"
+ * is nonzero.
+ */
+void us_vthumbbegin(INTBIG y, WINDOWPART *w, INTBIG hx, INTBIG ly, INTBIG hy,
+	INTBIG totallines, INTSML onleft)
 {
 	/* initialize polygon */
 	if (us_dragpoly == NOPOLYGON) us_dragpoly = allocpolygon(4, us_aid->cluster);
@@ -2417,16 +2468,32 @@ void us_vthumbbegin(INTBIG y, WINDOWPART *w)
 	us_dragshown = 0;
 	us_dragwindow = w;
 	us_initthumbcoord = y;
+	us_dragnodetotal = totallines;
+	us_dragpoint = -1;
 
-	us_trackww.screenlx = us_trackww.uselx = w->uselx;
-	us_trackww.screenhx = us_trackww.usehx = w->usehx+DISPLAYSLIDERSIZE;
-	us_trackww.screenly = us_trackww.usely = w->usely-DISPLAYSLIDERSIZE;
-	us_trackww.screenhy = us_trackww.usehy = w->usehy;
+	if (onleft != 0)
+	{
+		/* slider is on the left (simulation window) */
+		us_trackww.screenlx = us_trackww.uselx = (INTSML)hx;
+		us_trackww.screenhx = us_trackww.usehx = w->usehx;
+		us_dragoffx = hx-2;
+	} else
+	{
+		/* slider is on the right (edit, text edit, and explorer window) */
+		us_trackww.screenlx = us_trackww.uselx = w->uselx;
+		us_trackww.screenhx = us_trackww.usehx = hx+DISPLAYSLIDERSIZE;
+		us_dragoffx = hx;
+	}
+	us_trackww.screenly = ly;
+	us_trackww.usely = (INTSML)ly;
+	us_trackww.screenhy = hy;
+	us_trackww.usehy = (INTSML)hy;
 	us_trackww.frame = w->frame;
 	us_trackww.state = DISPWINDOW;
 	computewindowscale(&us_trackww);
 }
 
+/* tracking method for the vertical thumb */
 INTSML us_vthumbdown(INTBIG x, INTBIG y)
 {
 	if (us_dragshown != 0)
@@ -2436,10 +2503,10 @@ INTSML us_vthumbdown(INTBIG x, INTBIG y)
 
 		/* undraw the outline */
 		us_highl.col = 0;
-		us_dragpoly->xv[0] = us_dragwindow->usehx+3;
+		us_dragpoly->xv[0] = us_dragoffx+3;
 		us_dragpoly->yv[0] = us_dragwindow->thumbly + (us_lastcury-us_initthumbcoord)+1;
 		us_dragpoly->yv[1] = us_dragwindow->thumbhy + (us_lastcury-us_initthumbcoord)-1;
-		us_dragpoly->xv[1] = us_dragwindow->usehx+DISPLAYSLIDERSIZE-1;
+		us_dragpoly->xv[1] = us_dragoffx+DISPLAYSLIDERSIZE-1;
 		us_dragpoly->count = 2;
 		us_dragpoly->style = FILLEDRECT;
 		(void)us_showpoly(us_dragpoly, &us_trackww);
@@ -2447,14 +2514,14 @@ INTSML us_vthumbdown(INTBIG x, INTBIG y)
 
 	/* draw the outline */
 	us_highl.col = HIGHLIT;
-	if (us_dragwindow->thumbly + (y-us_initthumbcoord) <= us_dragwindow->usely+DISPLAYSLIDERSIZE)
-		 y = us_dragwindow->usely+DISPLAYSLIDERSIZE+1 - us_dragwindow->thumbly + us_initthumbcoord;
-	if (us_dragwindow->thumbhy + (y-us_initthumbcoord) >= us_dragwindow->usehy-DISPLAYSLIDERSIZE)
-		 y = us_dragwindow->usehy-DISPLAYSLIDERSIZE-1 - us_dragwindow->thumbhy + us_initthumbcoord;
-	us_dragpoly->xv[0] = us_dragwindow->usehx+3;
+	if (us_dragwindow->thumbly + (y-us_initthumbcoord) <= us_trackww.usely+DISPLAYSLIDERSIZE)
+		 y = us_trackww.usely+DISPLAYSLIDERSIZE+1 - us_dragwindow->thumbly + us_initthumbcoord;
+	if (us_dragwindow->thumbhy + (y-us_initthumbcoord) >= us_trackww.usehy-DISPLAYSLIDERSIZE)
+		 y = us_trackww.usehy-DISPLAYSLIDERSIZE-1 - us_dragwindow->thumbhy + us_initthumbcoord;
+	us_dragpoly->xv[0] = us_dragoffx+3;
 	us_dragpoly->yv[0] = us_dragwindow->thumbly + (y-us_initthumbcoord)+1;
 	us_dragpoly->yv[1] = us_dragwindow->thumbhy + (y-us_initthumbcoord)-1;
-	us_dragpoly->xv[1] = us_dragwindow->usehx+DISPLAYSLIDERSIZE-1;
+	us_dragpoly->xv[1] = us_dragoffx+DISPLAYSLIDERSIZE-1;
 	us_dragpoly->count = 2;
 	us_dragpoly->style = FILLEDRECT;
 	(void)us_showpoly(us_dragpoly, &us_trackww);
@@ -2464,6 +2531,7 @@ INTSML us_vthumbdown(INTBIG x, INTBIG y)
 	return(0);
 }
 
+/* completion method for tracking the vertical thumb in regular edit window */
 void us_vthumbdone(void)
 {
 	REGISTER NODEPROTO *np;
@@ -2473,10 +2541,10 @@ void us_vthumbdone(void)
 	{
 		/* undraw the outline */
 		us_highl.col = 0;
-		us_dragpoly->xv[0] = us_dragwindow->usehx+3;
+		us_dragpoly->xv[0] = us_dragoffx+3;
 		us_dragpoly->yv[0] = us_dragwindow->thumbly + (us_lastcury-us_initthumbcoord)+1;
 		us_dragpoly->yv[1] = us_dragwindow->thumbhy + (us_lastcury-us_initthumbcoord)-1;
-		us_dragpoly->xv[1] = us_dragwindow->usehx+DISPLAYSLIDERSIZE-1;
+		us_dragpoly->xv[1] = us_dragoffx+DISPLAYSLIDERSIZE-1;
 		us_dragpoly->count = 2;
 		us_dragpoly->style = FILLEDRECT;
 		(void)us_showpoly(us_dragpoly, &us_trackww);
@@ -2499,6 +2567,134 @@ void us_vthumbdone(void)
 	}
 }
 
+/* initialization method for tracking the arrows on sliders */
+void us_arrowclickbegin(WINDOWPART *w, INTBIG lx, INTBIG hx, INTBIG ly, INTBIG hy, INTBIG amount)
+{
+	us_dragwindow = w;
+	us_arrowlx = lx;
+	us_arrowhx = hx;
+	us_arrowly = ly;
+	us_arrowhy = hy;
+	us_arrowamount = amount;
+}
+
+INTSML us_varrowdown(INTBIG x, INTBIG y)
+{
+	if (x < us_arrowlx || x > us_arrowhx) return(0);
+	if (y < us_arrowly || y > us_arrowhy) return(0);
+	if (y >= us_dragwindow->thumbly && y <= us_dragwindow->thumbhy) return(0);
+	us_slideup(us_arrowamount);
+	us_endbatch();
+	return(0);
+}
+
+INTSML us_harrowdown(INTBIG x, INTBIG y)
+{
+	if (x < us_arrowlx || x > us_arrowhx) return(0);
+	if (y < us_arrowly || y > us_arrowhy) return(0);
+	if (x >= us_dragwindow->thumblx && x <= us_dragwindow->thumbhx) return(0);
+	us_slideleft(us_arrowamount);
+	us_endbatch();
+	return(0);
+}
+
+/* completion method for tracking the vertical thumb in an explorer or text-edit window */
+void us_evthumbdone(void)
+{
+	REGISTER INTBIG thumbarea;
+
+	if (us_dragshown != 0)
+	{
+		/* undraw the outline */
+		us_highl.col = 0;
+		us_dragpoly->xv[0] = us_dragoffx+3;
+		us_dragpoly->yv[0] = us_dragwindow->thumbly + (us_lastcury-us_initthumbcoord)+1;
+		us_dragpoly->yv[1] = us_dragwindow->thumbhy + (us_lastcury-us_initthumbcoord)-1;
+		us_dragpoly->xv[1] = us_dragoffx+DISPLAYSLIDERSIZE-1;
+		us_dragpoly->count = 2;
+		us_dragpoly->style = FILLEDRECT;
+		(void)us_showpoly(us_dragpoly, &us_trackww);
+
+		thumbarea = us_trackww.usehy - us_trackww.usely - DISPLAYSLIDERSIZE*2;
+		us_dragpoint = (us_trackww.usehy - DISPLAYSLIDERSIZE - 1 -
+			(us_dragwindow->thumbhy + (us_lastcury-us_initthumbcoord))) *
+				us_dragnodetotal / thumbarea;
+		if (us_dragpoint < 0) us_dragpoint = 0;
+	}
+}
+
+/* return method for reporting the vertical thumb in an explorer window */
+INTBIG us_evthumbend(void)
+{
+	return(us_dragpoint);
+}
+
+/* completion method for tracking the vertical thumb in a simulation waveform window */
+void us_svthumbdone(void)
+{
+	REGISTER INTBIG thumbarea;
+
+	if (us_dragshown != 0)
+	{
+		/* undraw the outline */
+		us_highl.col = 0;
+		us_dragpoly->xv[0] = us_dragoffx+3;
+		us_dragpoly->yv[0] = us_dragwindow->thumbly + (us_lastcury-us_initthumbcoord)+1;
+		us_dragpoly->yv[1] = us_dragwindow->thumbhy + (us_lastcury-us_initthumbcoord)-1;
+		us_dragpoly->xv[1] = us_dragoffx+DISPLAYSLIDERSIZE-1;
+		us_dragpoly->count = 2;
+		us_dragpoly->style = FILLEDRECT;
+		(void)us_showpoly(us_dragpoly, &us_trackww);
+
+		thumbarea = us_dragwindow->usehy - us_dragwindow->usely - DISPLAYSLIDERSIZE*2;
+		us_dragpoint = (us_dragwindow->usehy - DISPLAYSLIDERSIZE -
+			(us_dragwindow->thumbhy + (us_lastcury-us_initthumbcoord))) *
+				us_dragnodetotal / thumbarea;
+		if (us_dragpoint < 0) us_dragpoint = 0;
+	}
+}
+
+/* return method for reporting the vertical thumb in a simulation waveform window */
+INTBIG us_svthumbend(void)
+{
+	return(us_dragpoint);
+}
+
+float us_dragmintime;
+
+/* completion method for tracking the horizontal thumb in a simulation waveform window */
+void us_shthumbdone(void)
+{
+	REGISTER INTBIG thumbarea;
+	float mintime, maxtime, range;
+
+	if (us_dragshown != 0)
+	{
+		/* undraw the outline */
+		us_highl.col = 0;
+		us_dragpoly->xv[0] = us_dragwindow->thumblx + (us_lastcurx-us_initthumbcoord)+1;
+		us_dragpoly->yv[0] = us_trackww.usehy-3;
+		us_dragpoly->xv[1] = us_dragwindow->thumbhx + (us_lastcurx-us_initthumbcoord)-1;
+		us_dragpoly->yv[1] = us_trackww.usehy-DISPLAYSLIDERSIZE+1;
+		us_dragpoly->count = 2;
+		us_dragpoly->style = FILLEDRECT;
+		(void)us_showpoly(us_dragpoly, &us_trackww);
+
+		thumbarea = us_dragwindow->usehx - us_dragwindow->uselx - DISPLAYSLIDERSIZE*2;
+		sim_window_gettimerange(&mintime, &maxtime);
+		range = maxtime * 2.0f;
+		us_dragmintime = (us_dragwindow->thumblx + (us_lastcurx-us_initthumbcoord) -
+			(us_dragwindow->uselx+DISPLAYSLIDERSIZE+2)) * range / thumbarea;
+		if (us_dragmintime < 0.0) us_dragmintime = 0.0;
+	}
+}
+
+/* return method for reporting the horizontal thumb in a simulation waveform window */
+float us_shthumbend(void)
+{
+	return(us_dragmintime);
+}
+
 /************************* WINDOW PARTITION DIVIDER TRACKING *************************/
 
 void us_vpartdividerbegin(INTBIG x, INTBIG ly, INTBIG hy, WINDOWFRAME *wf)
@@ -2539,24 +2735,27 @@ INTSML us_vpartdividerdown(INTBIG x, INTBIG y)
 		(void)us_showpoly(us_dragpoly, &us_trackww);
 	}
 
+	us_lastcurx = x;
+	if (us_lastcurx >= us_trackww.usehx) us_lastcurx = us_trackww.usehx-1;
+	if (us_lastcurx < us_trackww.uselx) us_lastcurx = us_trackww.uselx;
+
 	/* draw the outline */
 	us_highl.col = HIGHLIT;
-	us_dragpoly->xv[0] = x;
+	us_dragpoly->xv[0] = us_lastcurx;
 	us_dragpoly->yv[0] = us_draglowval;
-	us_dragpoly->xv[1] = x;
+	us_dragpoly->xv[1] = us_lastcurx;
 	us_dragpoly->yv[1] = us_draghighval;
 	us_dragpoly->count = 2;
 	us_dragpoly->style = VECTORS;
 	(void)us_showpoly(us_dragpoly, &us_trackww);
 
 	us_dragshown = 1;
-	us_lastcurx = x;
 	return(0);
 }
 
 void us_vpartdividerdone(void)
 {
-	REGISTER INTBIG lx, hx, ly, hy;
+	INTBIG lx, hx, ly, hy;
 	REGISTER WINDOWPART *w;
 
 	if (us_dragshown != 0)
@@ -2577,29 +2776,40 @@ void us_vpartdividerdone(void)
 			if (w->frame != us_trackww.frame) continue;
 			if (strcmp(w->location, "entire") == 0) continue;
 
-			lx = w->uselx-1;   hx = w->usehx+1;
-			ly = w->usely-1;   hy = w->usehy+1;
-			if ((w->state&WINDOWSIMULATING) != 0)
-			{
-				lx -= SIMULATINGBORDERSIZE;   hx += SIMULATINGBORDERSIZE;
-				ly -= SIMULATINGBORDERSIZE;   hy += SIMULATINGBORDERSIZE;
-			}
-			if ((w->state&WINDOWTYPE) == DISPWINDOW)
-			{
-				hx += DISPLAYSLIDERSIZE;
-				ly -= DISPLAYSLIDERSIZE;
-			}
+			us_gettruewindowbounds(w, &lx, &hx, &ly, &hy);
+			lx--;   hx++;
+			ly--;   hy++;
 			if (us_initthumbcoord >= lx-2 && us_initthumbcoord <= lx+2)
 			{
 				w->hratio = (INTSML)(((hx - us_lastcurx) * w->hratio + (hx - lx)/2) / (hx - lx));
-				if (w->hratio > 95) w->hratio = 95;
-				if (w->hratio < 5) w->hratio = 5;
+				if (w->hratio > 95)
+				{
+					el_curwindowpart = w;
+					us_killcurrentwindow(0);
+					return;
+				}
+				if (w->hratio < 5)
+				{
+					el_curwindowpart = w;
+					us_killcurrentwindow(1);
+					return;
+				}
 			}
 			if (us_initthumbcoord >= hx-2 && us_initthumbcoord <= hx+2)
 			{
 				w->hratio = (INTSML)(((us_lastcurx - lx) * w->hratio + (hx - lx)/2) / (hx - lx));
-				if (w->hratio > 95) w->hratio = 95;
-				if (w->hratio < 5) w->hratio = 5;
+				if (w->hratio > 95)
+				{
+					el_curwindowpart = w;
+					us_killcurrentwindow(0);
+					return;
+				}
+				if (w->hratio < 5)
+				{
+					el_curwindowpart = w;
+					us_killcurrentwindow(1);
+					return;
+				}
 			}
 		}
 		us_beginchanges();
@@ -2648,24 +2858,27 @@ INTSML us_hpartdividerdown(INTBIG x, INTBIG y)
 		(void)us_showpoly(us_dragpoly, &us_trackww);
 	}
 
+	us_lastcury = y;
+	if (us_lastcury >= us_trackww.usehy) us_lastcury = us_trackww.usehy-1;
+	if (us_lastcury < us_trackww.usely) us_lastcury = us_trackww.usely;
+
 	/* draw the outline */
 	us_highl.col = HIGHLIT;
 	us_dragpoly->xv[0] = us_draglowval;
-	us_dragpoly->yv[0] = y;
+	us_dragpoly->yv[0] = us_lastcury;
 	us_dragpoly->xv[1] = us_draghighval;
-	us_dragpoly->yv[1] = y;
+	us_dragpoly->yv[1] = us_lastcury;
 	us_dragpoly->count = 2;
 	us_dragpoly->style = VECTORS;
 	(void)us_showpoly(us_dragpoly, &us_trackww);
 
 	us_dragshown = 1;
-	us_lastcury = y;
 	return(0);
 }
 
 void us_hpartdividerdone(void)
 {
-	REGISTER INTBIG lx, hx, ly, hy;
+	INTBIG lx, hx, ly, hy;
 	REGISTER WINDOWPART *w;
 
 	if (us_dragshown != 0)
@@ -2686,29 +2899,40 @@ void us_hpartdividerdone(void)
 			if (w->frame != us_trackww.frame) continue;
 			if (strcmp(w->location, "entire") == 0) continue;
 
-			lx = w->uselx-1;   hx = w->usehx+1;
-			ly = w->usely-1;   hy = w->usehy+1;
-			if ((w->state&WINDOWSIMULATING) != 0)
-			{
-				lx -= SIMULATINGBORDERSIZE;   hx += SIMULATINGBORDERSIZE;
-				ly -= SIMULATINGBORDERSIZE;   hy += SIMULATINGBORDERSIZE;
-			}
-			if ((w->state&WINDOWTYPE) == DISPWINDOW)
-			{
-				hx += DISPLAYSLIDERSIZE;
-				ly -= DISPLAYSLIDERSIZE;
-			}
+			us_gettruewindowbounds(w, &lx, &hx, &ly, &hy);
+			lx--;   hx++;
+			ly--;   hy++;
 			if (us_initthumbcoord >= ly-2 && us_initthumbcoord <= ly+2)
 			{
 				w->vratio = (INTSML)(((hy - us_lastcury) * w->vratio + (hy - ly)/2) / (hy - ly));
-				if (w->vratio > 95) w->vratio = 95;
-				if (w->vratio < 5) w->vratio = 5;
+				if (w->vratio > 95)
+				{
+					el_curwindowpart = w;
+					us_killcurrentwindow(0);
+					return;
+				}
+				if (w->vratio < 5)
+				{
+					el_curwindowpart = w;
+					us_killcurrentwindow(1);
+					return;
+				}
 			}
 			if (us_initthumbcoord >= hy-2 && us_initthumbcoord <= hy+2)
 			{
 				w->vratio = (INTSML)(((us_lastcury - ly) * w->vratio + (hy - ly)/2) / (hy - ly));
-				if (w->vratio > 95) w->vratio = 95;
-				if (w->vratio < 5) w->vratio = 5;
+				if (w->vratio > 95)
+				{
+					el_curwindowpart = w;
+					us_killcurrentwindow(0);
+					return;
+				}
+				if (w->vratio < 5)
+				{
+					el_curwindowpart = w;
+					us_killcurrentwindow(1);
+					return;
+				}
 			}
 		}
 		us_beginchanges();
@@ -2770,7 +2994,7 @@ void us_wanttoinvert(INTBIG fx, INTBIG fy, INTBIG tx, INTBIG ty, WINDOWPART *w)
 	tx = applyxscale(w, tx-w->screenlx) + w->uselx;
 	ty = applyyscale(w, ty-w->screenly) + w->usely;
 	if (clipline(&fx, &fy, &tx, &ty, w->uselx, w->usehx, w->usely, w->usehy) != 0) return;
-	us_invertline(w, (INTSML)fx, (INTSML)fy, (INTSML)tx, (INTSML)ty);
+	screeninvertline(w, (INTSML)fx, (INTSML)fy, (INTSML)tx, (INTSML)ty);
 }
 
 /* cursor advance routine when creating or moving an object */
