@@ -79,9 +79,6 @@ INTBIG      us_3dpolycount;
 INTBIG      us_3dpolytotal = 0;
 POLY3D    **us_3dpolylist;
 WINDOWPART *us_3dwindowpart;
-TECHNOLOGY *us_3dcurtech;
-VARIABLE   *us_3dcurheightvar;
-VARIABLE   *us_3dcurthicknessvar;
 
 /* interaction information */
 INTSML      us_3dinteraction = ROTATEVIEW;
@@ -213,6 +210,12 @@ void us_switchtofacet(NODEPROTO *cur, INTBIG lx, INTBIG hx, INTBIG ly, INTBIG hy
 		location = "Entire";
 	} else
 	{
+		if ((oldw->state&WINDOWTYPE) == EXPLORERWINDOW)
+		{
+			for(neww = el_topwindowpart; neww != NOWINDOWPART; neww = neww->nextwindowpart)
+				if (neww != oldw && neww->frame == oldw->frame) break;
+			if (neww != NOWINDOWPART) oldw = neww;
+		}
 		oldstate = oldw->state;
 		location = oldw->location;
 	}
@@ -307,6 +310,20 @@ void us_switchtofacet(NODEPROTO *cur, INTBIG lx, INTBIG hx, INTBIG ly, INTBIG hy
 			neww->usely -= DISPLAYSLIDERSIZE;
 		}
 
+		/* adjust window if it changed from waveform window to something else */
+		if ((neww->state&WINDOWTYPE) == WAVEFORMWINDOW && (oldstate&WINDOWTYPE) != WAVEFORMWINDOW)
+		{
+			/* became a waveform window: shrink the bottom and right edge */
+			neww->uselx += DISPLAYSLIDERSIZE;
+			neww->usely += DISPLAYSLIDERSIZE;
+		}
+		if ((neww->state&WINDOWTYPE) != WAVEFORMWINDOW && (oldstate&WINDOWTYPE) == WAVEFORMWINDOW)
+		{
+			/* no longer a waveform window: shrink the bottom and right edge */
+			neww->uselx -= DISPLAYSLIDERSIZE;
+			neww->usely -= DISPLAYSLIDERSIZE;
+		}
+
 		if (resethandlers != 0)
 		{
 			neww->buttonhandler = DEFAULTBUTTONHANDLER;
@@ -348,7 +365,6 @@ void us_switchtofacet(NODEPROTO *cur, INTBIG lx, INTBIG hx, INTBIG ly, INTBIG hy
  */
 void us_beginchanges(void)
 {
-	us_startbatching();
 }
 
 /*
@@ -502,7 +518,7 @@ void us_endchanges(WINDOWPART *which)
 	}
 	us_usedredraw = NOREDRAW;
 
-	us_endbatching();
+	flushscreen();
 }
 
 /*
@@ -864,7 +880,7 @@ INTSML us_showpoly(POLYGON *obj, WINDOWPART *w)
 	/* special case for grid display */
 	if (obj->style == GRIDDOTS)
 	{
-		us_drawgrid(w, obj);
+		screendrawgrid(w, obj);
 		return(0);
 	}
 
@@ -878,15 +894,15 @@ INTSML us_showpoly(POLYGON *obj, WINDOWPART *w)
 			{
 				/* simple rectangular box: transform, clip, and draw */
 				if (us_makescreen(&lx, &ly, &ux, &uy, w)) return(1);
-				us_drawbox(w, (INTSML)lx, (INTSML)ux, (INTSML)ly, (INTSML)uy, gra);
+				screendrawbox(w, (INTSML)lx, (INTSML)ux, (INTSML)ly, (INTSML)uy, gra);
 
 				/* for patterned and outlined rectangles, draw the box too */
 				if ((gra->colstyle&(NATURE|OUTLINEPAT)) == (PATTERNED|OUTLINEPAT))
 				{
-					us_drawline(w, (INTSML)lx, (INTSML)ly, (INTSML)lx, (INTSML)uy, gra, 0);
-					us_drawline(w, (INTSML)lx, (INTSML)uy, (INTSML)ux, (INTSML)uy, gra, 0);
-					us_drawline(w, (INTSML)ux, (INTSML)uy, (INTSML)ux, (INTSML)ly, gra, 0);
-					us_drawline(w, (INTSML)ux, (INTSML)ly, (INTSML)lx, (INTSML)ly, gra, 0);
+					screendrawline(w, (INTSML)lx, (INTSML)ly, (INTSML)lx, (INTSML)uy, gra, 0);
+					screendrawline(w, (INTSML)lx, (INTSML)uy, (INTSML)ux, (INTSML)uy, gra, 0);
+					screendrawline(w, (INTSML)ux, (INTSML)uy, (INTSML)ux, (INTSML)ly, gra, 0);
+					screendrawline(w, (INTSML)ux, (INTSML)ly, (INTSML)lx, (INTSML)ly, gra, 0);
 				}
 				return(0);
 			}
@@ -909,8 +925,8 @@ INTSML us_showpoly(POLYGON *obj, WINDOWPART *w)
 			if (objc->count > 2)
 			{
 				/* always clockwise */
-				if (areapoly(objc) < 0) us_reverse(objc);
-				us_drawpolygon(w, objc->xv, objc->yv, objc->count, objc->desc);
+				if (areapoly(objc) < 0.0) us_reverse(objc);
+				screendrawpolygon(w, objc->xv, objc->yv, objc->count, objc->desc);
 
 				/* for patterned and outlined polygons, draw the outline too */
 				if ((objc->desc->colstyle&(NATURE|OUTLINEPAT)) ==
@@ -919,11 +935,11 @@ INTSML us_showpoly(POLYGON *obj, WINDOWPART *w)
 					for(i=0; i<objc->count; i++)
 					{
 						if (i == 0) pre = objc->count-1; else pre = i-1;
-						us_drawline(w, (INTSML)objc->xv[pre], (INTSML)objc->yv[pre], (INTSML)objc->xv[i], (INTSML)objc->yv[i],
+						screendrawline(w, (INTSML)objc->xv[pre], (INTSML)objc->yv[pre], (INTSML)objc->xv[i], (INTSML)objc->yv[i],
 							objc->desc, 0);
 					}
 				}
-			} else us_drawline(w, (INTSML)objc->xv[0], (INTSML)objc->yv[0], (INTSML)objc->xv[1], (INTSML)objc->yv[1],
+			} else screendrawline(w, (INTSML)objc->xv[0], (INTSML)objc->yv[0], (INTSML)objc->xv[1], (INTSML)objc->yv[1],
 				gra, 0);
 			return(0);
 
@@ -1003,8 +1019,8 @@ INTSML us_showpoly(POLYGON *obj, WINDOWPART *w)
 			ux = applyxscale(w, ux-w->screenlx) + w->uselx;
 			uy = applyyscale(w, uy-w->screenly) + w->usely;
 			i = truefontsize(obj->font, w, obj->tech);
-			us_settextsize(w, (INTSML)i);
-			us_textsize(w, obj->string, &tsx, &tsy);
+			screensettextsize(w, (INTSML)i);
+			screengettextsize(w, obj->string, &tsx, &tsy);
 			switch (obj->style)
 			{
 				case TEXTCENT:
@@ -1046,7 +1062,7 @@ INTSML us_showpoly(POLYGON *obj, WINDOWPART *w)
 			}
 			if (tx > w->usehx || tx+tsx < w->uselx ||
 				ty > w->usehy || ty+tsy < w->usely) return(0);
-			us_puttext(w, tx, ty, obj->string, gra);
+			screendrawtext(w, tx, ty, obj->string, gra);
 			return(0);
 
 		case TEXTBOX:		/* text centered and contained in box */
@@ -1088,7 +1104,7 @@ INTSML us_showpoly(POLYGON *obj, WINDOWPART *w)
 			{
 				if (objc->count != 2) return(1);
 				i = computedistance(objc->xv[0], objc->yv[0], objc->xv[1], objc->yv[1]);
-				us_drawcircle(w, ZDN(objc->xv[0]), ZDN(objc->yv[0]), ZDN(i), gra);
+				screendrawcircle(w, ZDN(objc->xv[0]), ZDN(objc->yv[0]), ZDN(i), gra);
 				return(0);
 			}
 
@@ -1098,7 +1114,7 @@ INTSML us_showpoly(POLYGON *obj, WINDOWPART *w)
 				if (objc->count != 2) return(1);
 				i = ZDN(computedistance(objc->xv[0], objc->yv[0], objc->xv[1], objc->yv[1]));
 				if (i <= MINDISCSIZE) i = MINDISCSIZE;
-				us_drawdisc(w, ZDN(objc->xv[0]), ZDN(objc->yv[0]), i, gra);
+				screendrawdisc(w, ZDN(objc->xv[0]), ZDN(objc->yv[0]), i, gra);
 				return(0);
 			}
 
@@ -1106,7 +1122,7 @@ INTSML us_showpoly(POLYGON *obj, WINDOWPART *w)
 			if (objc->count == 0) return(1);
 			if ((objc->count%3) != 0) return(1);
 			for (i=0; i<objc->count; i += 3)
-				us_drawcirclearc(w, ZDN(objc->xv[i]), ZDN(objc->yv[i]), ZDN(objc->xv[i+1]),
+				screendrawcirclearc(w, ZDN(objc->xv[i]), ZDN(objc->yv[i]), ZDN(objc->xv[i+1]),
 					ZDN(objc->yv[1+i]), ZDN(objc->xv[i+2]), ZDN(objc->yv[i+2]), gra);
 			return(0);
 	}
@@ -1127,7 +1143,7 @@ void us_wanttodraw(INTBIG fx, INTBIG fy, INTBIG tx, INTBIG ty, WINDOWPART *w, GR
 	tx = applyxscale(w, tx-w->screenlx) + w->uselx;
 	ty = applyyscale(w, ty-w->screenly) + w->usely;
 	if (clipline(&fx, &fy, &tx, &ty, w->uselx, w->usehx, w->usely, w->usehy) != 0) return;
-	us_drawline(w, (INTSML)fx, (INTSML)fy, (INTSML)tx, (INTSML)ty, desc, texture);
+	screendrawline(w, (INTSML)fx, (INTSML)fy, (INTSML)tx, (INTSML)ty, desc, texture);
 }
 
 /*
@@ -1142,7 +1158,7 @@ void us_wanttodrawo(INTBIG fx, INTSML fxo, INTBIG fy, INTSML fyo, INTBIG tx, INT
 	tx = applyxscale(w, tx-w->screenlx) + w->uselx + txo;
 	ty = applyyscale(w, ty-w->screenly) + w->usely + tyo;
 	if (clipline(&fx, &fy, &tx, &ty, w->uselx, w->usehx, w->usely, w->usehy) != 0) return;
-	us_drawline(w, (INTSML)fx, (INTSML)fy, (INTSML)tx, (INTSML)ty, desc, texture);
+	screendrawline(w, (INTSML)fx, (INTSML)fy, (INTSML)tx, (INTSML)ty, desc, texture);
 }
 
 /*
@@ -1253,8 +1269,8 @@ INTSML us_writetext(INTBIG lx, INTBIG ux, INTBIG ly, INTBIG uy, GRAPHICS *desc,
 	/* scan for a font that fits */
 	for(font=initialfont; font>=TXT4P; font--)
 	{
-		us_settextsize(win, font);
-		us_textsize(win, txt, &six, &siy);
+		screensettextsize(win, font);
+		screengettextsize(win, txt, &six, &siy);
 		if (six <= ux-lx && siy <= uy-ly) break;
 	}
 
@@ -1268,11 +1284,11 @@ INTSML us_writetext(INTBIG lx, INTBIG ux, INTBIG ly, INTBIG uy, GRAPHICS *desc,
 		stop /= six;
 		if (stop == 0) return(1);
 		save = txt[stop];   txt[stop] = 0;
-		us_textsize(win, txt, &six, &siy);
+		screengettextsize(win, txt, &six, &siy);
 	} else stop = -1;
 
 	/* draw the text */
-	us_puttext(win, (INTSML)(lx+(ux-lx-six)/2), (INTSML)(ly+(uy-ly-siy)/2), txt, desc);
+	screendrawtext(win, (INTSML)(lx+(ux-lx-six)/2), (INTSML)(ly+(uy-ly-siy)/2), txt, desc);
 	if (stop >= 0) txt[stop] = (char)save;
 	return(0);
 }
@@ -1389,31 +1405,6 @@ void us_reverse(POLYGON *poly)
 
 /******************** 3D DISPLAY ********************/
 
-/*
- * Routine to establish default layer height and thickness for
- * technology "tech".
- */
-void us_3ddefaultlayerheight(TECHNOLOGY *tech)
-{
-	REGISTER INTBIG *layerheight, *layerthickness, i, funct;
-
-	layerheight = (INTBIG *)emalloc(tech->layercount * SIZEOFINTBIG, el_tempcluster);
-	layerthickness = (INTBIG *)emalloc(tech->layercount * SIZEOFINTBIG, el_tempcluster);
-	for(i=0; i<tech->layercount; i++)
-	{
-		funct = layerfunction(tech, (INTSML)i) & LFTYPE;
-		layerheight[i] = layerfunctionheight(funct);
-		layerthickness[i] = 0;
-		if (layeriscontact(funct) != 0) layerthickness[i] = 2;
-	}
-	setvalkey((INTBIG)tech, VTECHNOLOGY, us_techlayer3dheightkey, (INTBIG)layerheight,
-		VINTEGER|VISARRAY|(tech->layercount<<VLENGTHSH)|VDONTSAVE);
-	setvalkey((INTBIG)tech, VTECHNOLOGY, us_techlayer3dthicknesskey, (INTBIG)layerthickness,
-		VINTEGER|VISARRAY|(tech->layercount<<VLENGTHSH)|VDONTSAVE);
-	efree((char *)layerheight);
-	efree((char *)layerthickness);
-}
-
 /*
  * Routine to setup the transformation matrix for 3D viewing.
  * Called once when the display is initially converted to 3D.
@@ -1421,36 +1412,19 @@ void us_3ddefaultlayerheight(TECHNOLOGY *tech)
 void us_3dsetupviewing(WINDOWPART *w)
 {
 	REGISTER TECHNOLOGY *tech;
-	REGISTER VARIABLE *varh, *vart;
-	REGISTER INTBIG i, len, height, lowheight, highheight;
-	static INTSML heightsetup = 0;
+	REGISTER INTBIG i, lowheight, highheight;
+	INTBIG thickness, height;
 	float scale, cx, cy, cz, sx, sy, sz;
 	REGISTER NODEPROTO *np;
 	REGISTER XFORM3D *xf3;
 
-	/* make sure that every technology has layer height information */
-	if (heightsetup == 0)
-	{
-		heightsetup = 1;
-		for(tech = el_technologies; tech != NOTECHNOLOGY; tech = tech->nexttechnology)
-		{
-			varh = getvalkey((INTBIG)tech, VTECHNOLOGY, VINTEGER|VISARRAY, us_techlayer3dheightkey);
-			vart = getvalkey((INTBIG)tech, VTECHNOLOGY, VINTEGER|VISARRAY, us_techlayer3dthicknesskey);
-			if (varh == NOVARIABLE || vart == NOVARIABLE)
-				us_3ddefaultlayerheight(tech);
-		}
-	}
-
 	/* determine height range */
 	lowheight = 0;   highheight = -1;
 	for(tech = el_technologies; tech != NOTECHNOLOGY; tech = tech->nexttechnology)
 	{
-		varh = getvalkey((INTBIG)tech, VTECHNOLOGY, VINTEGER|VISARRAY, us_techlayer3dheightkey);
-		if (varh == NOVARIABLE) continue;
-		len = getlength(varh);
-		for(i=0; i<len; i++)
+		for(i=0; i<tech->layercount; i++)
 		{
-			height = ((INTBIG *)varh->addr)[i];
+			if (get3dfactors(tech, i, &height, &thickness) != 0) continue;
 			if (highheight < lowheight)
 			{
 				highheight = lowheight = height;
@@ -1610,7 +1584,6 @@ void us_3dbuildtransform(XFORM3D *xf3)
  */
 void us_3dstartdrawing(WINDOWPART *win)
 {
-	us_3dcurtech = NOTECHNOLOGY;
 	us_3dgatheringpolys = 1;
 	us_3dpolycount = 0;
 	us_3dwindowpart = win;
@@ -1907,9 +1880,9 @@ INTSML us_3deachdown(INTBIG x, INTBIG y)
  */
 INTSML us_3dshowpoly(POLYGON *poly, WINDOWPART *w)
 {
-	REGISTER INTBIG i, previ, thickness, depth, topdepth, botdepth, centerdepth;
+	REGISTER INTBIG i, previ, topdepth, botdepth, centerdepth;
 	float topheight, botheight;
-	INTBIG lx, hx, ly, hy;
+	INTBIG lx, hx, ly, hy, thickness, depth;
 	REGISTER POLY3D *poly3d;
 
 	/* ignore polygons with no color */
@@ -1918,7 +1891,8 @@ INTSML us_3dshowpoly(POLYGON *poly, WINDOWPART *w)
 	/* special case when drawing instance boundaries */
 	if (poly->desc->col == el_colfacet)
 	{
-		topdepth = ((INTBIG *)us_3dcurheightvar->addr)[0] * el_curtech->deflambda;
+		(void)get3dfactors(el_curtech, 0, &depth, &thickness);
+		topdepth = depth;
 		topheight = (float)topdepth;
 		if (isbox(poly, &lx, &hx, &ly, &hy) != 0)
 		{
@@ -1955,24 +1929,9 @@ INTSML us_3dshowpoly(POLYGON *poly, WINDOWPART *w)
 
 	/* make sure there is a technology and a layer for this polygon */
 	if (poly->tech == NOTECHNOLOGY) return(1);
-	if (poly->tech != us_3dcurtech)
-	{
-		us_3dcurtech = poly->tech;
-		us_3dcurheightvar = getvalkey((INTBIG)us_3dcurtech, VTECHNOLOGY, VINTEGER|VISARRAY,
-			us_techlayer3dheightkey);
-		us_3dcurthicknessvar = getvalkey((INTBIG)us_3dcurtech, VTECHNOLOGY, VINTEGER|VISARRAY,
-			us_techlayer3dthicknesskey);
-		if (us_3dcurheightvar == NOVARIABLE || us_3dcurthicknessvar == NOVARIABLE)
-		{
-			us_3dcurtech = NOTECHNOLOGY;
-			return(1);
-		}
-	}
-	if (poly->layer < 0 || poly->layer >= poly->tech->layercount) return(1);
+	if (get3dfactors(poly->tech, poly->layer, &depth, &thickness) != 0) return(1);
 
 	/* setup a 3D polygon */
-	thickness = ((INTBIG *)us_3dcurthicknessvar->addr)[poly->layer];
-	depth = ((INTBIG *)us_3dcurheightvar->addr)[poly->layer];
 	topdepth = (depth + thickness/2) * el_curtech->deflambda;
 	topheight = (float)topdepth;
 	if (thickness != 0)
