@@ -922,6 +922,7 @@ UINTBIG vhdl_getfacetdate(NODEPROTO *np)
 void vhdl_convertfacet(NODEPROTO *np, INTSML force)
 {
 	REGISTER NODEPROTO *npvhdl, *onp;
+	REGISTER LIBRARY *lib;
 	REGISTER INTSML i, backannotate, vhdlondisk, len;
 	REGISTER UINTBIG dateoflayout;
 	REGISTER VARIABLE *var;
@@ -990,8 +991,9 @@ void vhdl_convertfacet(NODEPROTO *np, INTSML force)
 		intended);
 
 	/* recursively generate the VHDL */
-	for(onp = np->cell->lib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
-		onp->temp1 = 0;
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(onp = lib->firstnodeproto; onp != NONODEPROTO; onp = onp->nextnodeproto)
+			onp->temp1 = 0;
 	backannotate = vhdl_generatevhdl(np);
 	if (backannotate != 0)
 		ttyputmsg("Back-annotation information has been added (library must be saved)");
@@ -1107,6 +1109,9 @@ INTSML vhdl_generatevhdl(NODEPROTO *np)
 			continue;
 		}
 
+		/* ignore component with no ports */
+		if (ni->proto->firstportproto == NOPORTPROTO) continue;
+
 		/* see if this component is already written to the header */
 		for(i=0; i<componentcount; i++)
 			if (namesame(pt, componentlist[i]) == 0) break;
@@ -1254,6 +1259,9 @@ INTSML vhdl_generatevhdl(NODEPROTO *np)
 	vhdl_print("begin");
 	for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 	{
+		/* ignore component with no ports */
+		if (ni->proto->firstportproto == NOPORTPROTO) continue;
+
 		pt = vhdl_primname(ni, &special);
 		if (pt[0] == 0) continue;
 
@@ -1332,6 +1340,9 @@ INTSML vhdl_generatevhdl(NODEPROTO *np)
 		if (cnp == NONODEPROTO) cnp = ni->proto;
 		if (cnp->temp1 != 0) continue;
 
+		/* ignore component with no ports */
+		if (cnp->firstportproto == NOPORTPROTO) continue;
+
 		/* if ALS, see if this facet is in the reference library */
 		if (vhdl_lib != NOLIBRARY && vhdl_target == TARGET_ALS)
 		{
