@@ -2166,7 +2166,7 @@ void vhdl_freeports(FPORTLIST *ports)
 {
 	FPORTLIST *nextports;
 
-	for( ; ports != NULL; ports = nextports)
+	for( ; ports != NULL && ports != ((FPORTLIST *)-1); ports = nextports)
 	{
 		nextports = ports->next;
 		vhdl_freeidentlist(ports->names);
